	CREATE TABLE objects (
	  objectid INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	  title varchar(200) NOT NULL,
	  description varchar(1000) default NULL,
	  url varchar(2048) default NULL,
	  saved INTEGER default NULL,
	  trackedpermission INTEGER default 0,
	  trackable INTEGER default 1,
	  pluginid INTEGER default -1,
	  objecttype varchar(20) default '',
	  format varchar(20) default NULL,
	  method varchar(20) default NULL,
	  concept CLOB(65K) default '',
	  implementation CLOB(65K) default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  lifetime INTEGER default 0,
      uuid char(36) NOT NULL,
	  PRIMARY KEY  (objectid)
	);
	
	CREATE TABLE associations (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid0 INTEGER NOT NULL,
	  objectid1 INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid0) REFERENCES objects(objectid) ON DELETE CASCADE,
	  FOREIGN KEY (objectid1) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE categories (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  categoryid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE categorynames (
	  categoryid INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  categoryname varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (categoryid)
	);

	CREATE TABLE comments (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER default NULL,
	  commenttitle varchar(200) default NULL,
	  commenttext varchar(1000) default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE acls (
	  id INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	  objectid INTEGER default 0,
	  username varchar(100) default NULL,
	  groupname varchar(100) default NULL,
	  r INTEGER default 0,
	  e INTEGER default 0,
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE objectdata (
	  objectid INTEGER NOT NULL,
	  totalvotes INTEGER default 0,
	  totalvoters INTEGER default 0,
	  rating FLOAT default 0,
	  hits INTEGER default 0,
	  numcomments INTEGER default 0,
	  PRIMARY KEY  (objectid),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE sessions (
	  id CHAR(32) NOT NULL,
	  expiration INTEGER NOT NULL,
	  context CLOB(65K) NOT NULL default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE plugins (
	  pluginid INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  name varchar(32) NOT NULL,
	  dir varchar(100) NOT NULL,
	  editor varchar(32) NOT NULL,
	  generator varchar(32) NOT NULL,
	  collection varchar(32) default NULL,
	  collectionname varchar(32) default NULL,
	  objecttype varchar(20) default NULL,
	  description varchar(255) NOT NULL,
	  icon16path varchar(255) NOT NULL,
	  icon32path varchar(255) NOT NULL,
	  icon64path varchar(255) NOT NULL,
	  creatorid varchar(100) default NULL,
	  category varchar(32) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (pluginid)
	);

	CREATE TABLE plugindata (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  pluginid INTEGER NOT NULL,
	  name varchar(100) NOT NULL,
	  value CLOB(10M) NOT NULL default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE,
	  FOREIGN KEY (pluginid) REFERENCES plugins(pluginid) ON DELETE CASCADE
	);

	CREATE TABLE favorites (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE params (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  parametername varchar(100) NOT NULL,
	  parametertype varchar(20) default NULL,
	  defaultvalue CLOB(65K) default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  helptext varchar(256) default '',
	  prompt varchar(256) default NULL,
	  parameterorder INTEGER default NULL,
	  optional char,
	  displayname varchar(256) default '',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE purexmldata (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  feedid INTEGER NOT NULL,
	  db2datasource varchar(100) default NULL,
	  db2table varchar(255) default NULL,
	  db2column varchar(255) default NULL,
	  db2row varchar(255) default NULL,
	  db2exists INTEGER default NULL,
	  db2title varchar(255) default NULL,
	  db2summary varchar(255) default NULL,
	  db2creator varchar(255) default NULL,
	  db2rows varchar(255) default NULL,
	   PRIMARY KEY  (id)
	);

	CREATE TABLE ratings (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  ratingvalue INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE sqldata (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  feedid INTEGER default NULL,
	  sqlstatement varchar(500) default NULL,
	  sqldb varchar(100) default NULL,
	  PRIMARY KEY (id)
	);

	CREATE TABLE tags (
	  id INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	  tagname varchar(50) NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE users (
	  username varchar(60) NOT NULL,
	  fullname varchar(100) NOT NULL,
	  password varchar(60) NOT NULL,
	  email varchar(128) default NULL,
	  permissions INTEGER NOT NULL,
	  reportemail INTEGER default 0,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (username)
	);

	CREATE TABLE groups (
	  id INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	  groupname varchar(50) default NULL,
	  query varchar(1024) default NULL,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE keywords (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  keywords CLOB(65K) NOT NULL default '',
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE urlinfo (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  url varchar(2048) NOT NULL,
	  hourlycount INTEGER default 1,
	  lasttime INTEGER default 0,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE trackingconfig (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  type INTEGER NOT NULL default 0,
	  host varchar(255) default NULL,
	  vname varchar(255) default NULL,
	  vpk  varchar(255) default NULL,
	  creatorid varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE trackedexternal (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL default 0,
	  pluginid INTEGER NOT NULL default 0,
	  serviceid varchar(255) default NULL,
	  parameters varchar(255) default NULL,
	  endpoint  varchar(255) default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE trackedlocal (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL default 0,
	  pluginid INTEGER NOT NULL default 0,
	  ip varchar(255) default NULL,
	  username varchar(255) default NULL,
	  executiontime INTEGER default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE resources (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  content blob(1G) NOT NULL,
	  mimetype varchar(30) NOT NULL,
	  filename varchar(60) NOT NULL,
      timestamp INTEGER default 0,
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE events (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  action varchar(60) default NULL,
	  description VARCHAR(2048) NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE log (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  level varchar(2) NOT NULL,
	  component varchar(30) NOT NULL,
	  error CLOB(65K) NOT NULL default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE db_conns (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  creatorid varchar(100) NOT NULL,
	  name varchar(1024) NOT NULL,
	  dbtype varchar(20) default NULL,
	  dbname varchar(1024) default NULL,
	  dbhost varchar(100) default NULL,
	  dbport varchar(100) default NULL,
	  dbusr varchar(100) default NULL,
	  dbpwd varchar(100) default NULL,
	  dbsrv varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE connections (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  creatorid varchar(100) default '',
	  cname varchar(100) NOT NULL,
	  username varchar(1024) NOT NULL,
	  password varchar(1024) NOT NULL,
	  data CLOB(65K) default '',
	  timestamp INTEGER default 0,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpobjects (
	  objectid INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  creatorid varchar(60) NOT NULL,
	  startedit INTEGER default 0,
	  PRIMARY KEY  (objectid)
	);

	CREATE TABLE tmpplugindata (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  pluginid INTEGER NOT NULL,
	  name varchar(100) NOT NULL,
	  value CLOB(10M) NOT NULL default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpparams (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  parametername varchar(100) NOT NULL,
	  parametertype varchar(20) default NULL,
	  defaultvalue CLOB(65K) default '',
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  helptext varchar(256) default '',
	  prompt varchar(256) default NULL,
	  parameterorder INTEGER default NULL,
	  optional char,
	  displayname varchar(256) default '',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpresources (
	  id INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
	  objectid INTEGER NOT NULL,
	  content blob(1G) NOT NULL,
	  mimetype varchar(30) NOT NULL,
	  filename varchar(60) NOT NULL,
      timestamp INTEGER default 0,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE db_conns_use (
	  dbconnid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE connections_use (
	  connectionid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmpdb_conns_use (
	  dbconnid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmpconnections_use (
	  connectionid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmprc (
	  id INTEGER NOT NULL
	);

CREATE INDEX TMPOBJ_IDX_CID ON TMPOBJECTS(CREATORID);
CREATE INDEX TMPPLGDATA_IDX_OBJID ON TMPPLUGINDATA(OBJECTID);
CREATE INDEX TMPPARAM_IDX_OBJID ON TMPPARAMS(OBJECTID);
CREATE INDEX TMPRES_IDX_OBJID ON TMPRESOURCES(OBJECTID);
CREATE INDEX CONNUSE_IDX_OBJID ON CONNECTIONS_USE(OBJECTID);
CREATE INDEX DBCONNUSE_IDX_OBJID ON DB_CONNS_USE(OBJECTID);
CREATE INDEX TMPCONNUSE_IDX_OBJID ON TMPCONNECTIONS_USE(OBJECTID);
CREATE INDEX TMP_DBCONNUSE_IDX_OBJID ON TMPDB_CONNS_USE(OBJECTID);
CREATE INDEX TMPRC_IDX_ID ON TMPRC(ID);