	CREATE TABLE objects (
	  objectid SERIAL NOT NULL,
	  title varchar(200) NOT NULL,
	  description lvarchar(1000) default NULL,
	  url lvarchar(2048) default NULL,
	  saved INTEGER default NULL,
	  trackedpermission INTEGER default 0,
	  trackable INTEGER default 1,
	  pluginid INTEGER default -1,
	  objecttype varchar(20) default '',
	  format varchar(20) default NULL,
	  method varchar(20) default NULL,
	  concept CLOB,
	  implementation CLOB,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  lifetime INTEGER default 0,
      uuid char(36) NOT NULL,
	  PRIMARY KEY  (objectid)
  	);
  	
	CREATE TABLE connections (
	  id SERIAL NOT NULL,
	  creatorid varchar(100) default '',
	  cname varchar(100) NOT NULL,
	  username lvarchar(1024) NOT NULL,
	  password lvarchar(1024) NOT NULL,
	  data CLOB,
	  timestamp INTEGER default 0,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE associations (
	  id SERIAL NOT NULL,
	  objectid0 INTEGER NOT NULL,
	  objectid1 INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid0) REFERENCES objects(objectid) ON DELETE CASCADE,
	  FOREIGN KEY (objectid1) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE categories (
	  id SERIAL NOT NULL,
	  categoryid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE categorynames (
	  categoryid SERIAL NOT NULL,
	  categoryname varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (categoryid)
	);

	CREATE TABLE comments (
	  id SERIAL NOT NULL,
	  objectid INTEGER default NULL,
	  commenttitle varchar(200) default NULL,
	  commenttext lvarchar(1000) default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);
	
	CREATE TABLE acls (
	  id SERIAL NOT NULL,
	  objectid INTEGER default 0,
	  username varchar(100) default NULL,
	  groupname varchar(100) default NULL,
	  r INTEGER default 0,
	  e INTEGER default 0,
	  PRIMARY KEY (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE objectdata (
	  objectid INTEGER NOT NULL,
	  totalvotes INTEGER default 0,
	  totalvoters INTEGER default 0,
	  rating FLOAT default 0,
	  hits INTEGER default 0,
	  numcomments INTEGER default 0,
	  PRIMARY KEY  (objectid),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE sessions (
	  id CHAR(32) NOT NULL,
	  expiration INTEGER NOT NULL,
	  context CLOB NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE plugins (
	  pluginid SERIAL NOT NULL,
	  name varchar(32) NOT NULL,
	  dir varchar(100) NOT NULL,
	  editor varchar(32) NOT NULL,
	  generator varchar(32) NOT NULL,
	  collection varchar(32) default NULL,
	  collectionname varchar(32) default NULL,
      objecttype varchar(20) default NULL,
	  description varchar(255) NOT NULL,
	  icon16path varchar(255) NOT NULL,
	  icon32path varchar(255) NOT NULL,
	  icon64path varchar(255) NOT NULL,
	  creatorid varchar(100) default NULL,
	  category varchar(32) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (pluginid)
	);

	CREATE TABLE plugindata (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  pluginid INTEGER NOT NULL,
	  name varchar(100) NOT NULL,
	  value CLOB NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE,
	  FOREIGN KEY (pluginid) REFERENCES plugins(pluginid) ON DELETE CASCADE
	);

	CREATE TABLE favorites (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE params (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  parametername varchar(100) NOT NULL,
	  parametertype varchar(20) default NULL,
	  defaultvalue CLOB,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  helptext lvarchar(256) default '',
	  prompt lvarchar(256) default NULL,
	  parameterorder INTEGER default NULL,
	  optional char,
	  displayname lvarchar(256) default '',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE purexmldata (
	  id SERIAL NOT NULL,
	  feedid INTEGER NOT NULL,
	  db2datasource varchar(100) default NULL,
	  db2table varchar(255) default NULL,
	  db2column varchar(255) default NULL,
	  db2row varchar(255) default NULL,
	  db2exists INTEGER default NULL,
	  db2title varchar(255) default NULL,
	  db2summary varchar(255) default NULL,
	  db2creator varchar(255) default NULL,
	  db2rows varchar(255) default NULL,
	   PRIMARY KEY  (id)
	);

	CREATE TABLE ratings (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  ratingvalue INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE sqldata (
	  id SERIAL NOT NULL,
	  feedid INTEGER default NULL,
	  sqlstatement lvarchar(500) default NULL,
	  sqldb varchar(100) default NULL,
	  PRIMARY KEY (id)
	);

	CREATE TABLE tags (
	  id SERIAL NOT NULL,
	  tagname varchar(50) NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE users (
	  username varchar(60) NOT NULL,
	  fullname varchar(100) NOT NULL,
	  password varchar(60) NOT NULL,
	  email varchar(128) default NULL,
	  permissions INTEGER NOT NULL,
	  reportemail INTEGER default 0,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (username)
	);

	CREATE TABLE groups (
	  id SERIAL NOT NULL,
	  groupname varchar(50) default NULL,
	  query lvarchar(1024) default NULL,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE keywords (
	  id SERIAL NOT NULL,
	  keywords CLOB NOT NULL,
	  objectid INTEGER NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE urlinfo (
	  id SERIAL NOT NULL,
	  url lvarchar(2048) NOT NULL,
	  hourlycount INTEGER default 1,
	  lasttime INTEGER default 0,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE trackingconfig (
	  id SERIAL NOT NULL,
	  type INTEGER default 0 NOT NULL,
	  host varchar(255) default NULL,
	  vname varchar(255) default NULL,
	  vpk  varchar(255) default NULL,
	  creatorid varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE trackedexternal (
	  id SERIAL NOT NULL,
	  objectid INTEGER default 0 NOT NULL,
	  pluginid INTEGER default 0 NOT NULL,
	  serviceid varchar(255) default NULL,
	  parameters varchar(255) default NULL,
	  endpoint  varchar(255) default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE trackedlocal (
	  id SERIAL NOT NULL,
	  objectid INTEGER default 0 NOT NULL,
	  pluginid INTEGER default 0 NOT NULL,
	  ip varchar(255) default NULL,
	  username varchar(255) default NULL,
	  executiontime INTEGER default NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE resources (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  content blob NOT NULL,
	  mimetype varchar(30) NOT NULL,
	  filename varchar(60) NOT NULL,
      timestamp INTEGER default 0,
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE events (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  action varchar(60) default NULL,
	  description lVARCHAR(2048) NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id),
	  FOREIGN KEY (objectid) REFERENCES objects(objectid) ON DELETE CASCADE
	);

	CREATE TABLE log (
	  id SERIAL NOT NULL,
	  level varchar(2) NOT NULL,
	  component varchar(30) NOT NULL,
	  error CLOB NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE db_conns (
	  id SERIAL NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  name lvarchar(1024) NOT NULL,
	  dbtype varchar(20) default NULL,
	  dbname lvarchar(1024) default NULL,
	  dbhost varchar(100) default NULL,
	  dbport varchar(100) default NULL,
	  dbusr varchar(100) default NULL,
	  dbpwd varchar(100) default NULL,
	  dbsrv varchar(100) default NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpobjects (
	  objectid SERIAL NOT NULL,
	  creatorid varchar(60) NOT NULL,
	  startedit INTEGER default 0,
	  PRIMARY KEY  (objectid)
  	);

	CREATE TABLE tmpplugindata (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  pluginid INTEGER NOT NULL,
	  name varchar(100) NOT NULL,
	  value CLOB NOT NULL,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpparams (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  parametername varchar(100) NOT NULL,
	  parametertype varchar(20) default NULL,
	  defaultvalue CLOB,
	  creatorid varchar(100) NOT NULL,
	  timestamp INTEGER default 0,
	  version varchar(20) default '1.0',
	  helptext lvarchar(256) default '',
	  prompt lvarchar(256) default NULL,
	  parameterorder INTEGER default NULL,
	  displayname lvarchar(256) default '',
	  optional char,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE tmpresources (
	  id SERIAL NOT NULL,
	  objectid INTEGER NOT NULL,
	  content blob NOT NULL,
	  mimetype varchar(30) NOT NULL,
	  filename varchar(60) NOT NULL,
      timestamp INTEGER default 0,
	  PRIMARY KEY  (id)
	);

	CREATE TABLE connections_use (
	  connectionid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE db_conns_use (
	  dbconnid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmpconnections_use (
	  connectionid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmpdb_conns_use (
	  dbconnid INTEGER NOT NULL,
	  objectid INTEGER NOT NULL,
	  timestamp INTEGER default 0
	);

	CREATE TABLE tmprc (
	  id INTEGER NOT NULL
	);

CREATE INDEX TMPOBJ_IDX_CID ON TMPOBJECTS(CREATORID);
CREATE INDEX TMPPLGDATA_IDX_OBJID ON TMPPLUGINDATA(OBJECTID);
CREATE INDEX TMPPARAM_IDX_OBJID ON TMPPARAMS(OBJECTID);
CREATE INDEX TMPRES_IDX_OBJID ON TMPRESOURCES(OBJECTID);
CREATE INDEX CONNUSE_IDX_OBJID ON CONNECTIONS_USE(OBJECTID);
CREATE INDEX DBCONNUSE_IDX_OBJID ON DB_CONNS_USE(OBJECTID);
CREATE INDEX TMPCONNUSE_IDX_OBJID ON TMPCONNECTIONS_USE(OBJECTID);
CREATE INDEX TMP_DBCONNUSE_IDX_OBJID ON TMPDB_CONNS_USE(OBJECTID);
CREATE INDEX TMPRC_IDX_ID ON TMPRC(ID);
