<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
xmlns:mashup="http://ibm.com/mashuphub" 
xmlns="http://www.w3.org/TR/REC-html40"

>
 	<xsl:output indent="yes" method="xml" omit-xml-declaration = "yes" encoding="UTF-8" />

	<xsl:variable name="lcase">abcdefghijklmnopqrstuvwxyz</xsl:variable>
	<xsl:variable name="ucase">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
	<xsl:variable name="flowprefix">flow</xsl:variable>

		<xsl:template match="//mashup:flow">
		<xsl:variable name="operatorcount" select="count(mashup:*[name()='operator' or name()='source'])"/>
	<xsl:element  namespace=""  name="MASHUP">

	<xsl:element  namespace=""  name="METADATA">
		<xsl:element  namespace="" name="FLOW-SCHEMA-VERSION">development</xsl:element>
	</xsl:element>
	
	<xsl:if test="$operatorcount>0">
		<xsl:if test="count(mashup:namespaces/mashup:namespace)>0">
			<xsl:element  namespace="" name="NAMESPACES">
			<xsl:for-each select="mashup:namespaces/mashup:namespace" >
				<xsl:element  namespace="" name="NS">
					<xsl:attribute name="prefix"><xsl:value-of select="@prefix"/></xsl:attribute> <xsl:attribute name="uri"><xsl:value-of select="."/></xsl:attribute>
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
		</xsl:if>

	
		<xsl:variable name="flowaction" select="@action"/>
	
		<xsl:choose>
			<xsl:when test="$flowaction='Preview' and @operatorid!=''">
				<xsl:call-template name="dopreviewoperator"><xsl:with-param name="operatorid" select="@operatorid" /></xsl:call-template>				
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="mashup:*" >
					<xsl:call-template name="dooneoperator"><xsl:with-param name="flowaction" select="$flowaction" /></xsl:call-template>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>


		<xsl:element  namespace="" name="INPUT">
			<xsl:for-each select="//mashup:param[@type='dynamic']" >
				<xsl:variable name="paramid" select="@id"/>
				<xsl:variable name="vtype">
				<xsl:choose>
					<xsl:when test="//mashup:param[@type='variable' and text()=$paramid and ../../@type='foreach']">DERIVED</xsl:when>
					<xsl:otherwise>URL</xsl:otherwise>
				</xsl:choose>
				</xsl:variable>

				<xsl:variable name="vname">
				<xsl:choose>
					<xsl:when test="$vtype='DERIVED'"><xsl:value-of select="@id"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="@caption"/></xsl:otherwise>
				</xsl:choose>
				</xsl:variable>

				<xsl:if test="//mashup:param[@type='variable' and text()=$paramid]">
				<xsl:element  namespace="" name="PARAM">
					<xsl:attribute name="NAME"><xsl:value-of select="$vname"/></xsl:attribute>
					 <xsl:attribute name="TYPE"><xsl:value-of select="$vtype"/></xsl:attribute>
					  <xsl:attribute name="BINDVAR">$<xsl:value-of select="@id"/></xsl:attribute>

					<xsl:element  namespace="" name="VALUE">
						<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
					</xsl:element> 
					<xsl:if test="$vtype='URL'">
						<xsl:element  namespace="" name="PROMPT"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="//mashup:param[@id=$paramid]/@comments"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
						</xsl:element>
					</xsl:if>
				</xsl:element>
				</xsl:if>
			</xsl:for-each>

			<xsl:for-each select="mashup:operator[@type='foreach']/mashup:condition/mashup:param[@position='0' and @type!='']" >
				<xsl:variable name="paramid" select="text()"/>
				<xsl:if test="//mashup:param[@id=$paramid]/@type!='dynamic' " >
					<xsl:element  namespace="" name="PARAM">
						<xsl:attribute name="NAME"><xsl:value-of select="//mashup:param[@id=$paramid]/@caption"/></xsl:attribute> 
						<xsl:attribute name="TYPE">DERIVED</xsl:attribute>  
						<xsl:attribute name="BINDVAR">$<xsl:value-of select="text()"/><xsl:value-of select="@id"/></xsl:attribute>
						<xsl:element  namespace="" name="VALUE">			
							<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="//mashup:param[@id=$paramid]/text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
						</xsl:element>
					</xsl:element>
				</xsl:if>
			</xsl:for-each>
		</xsl:element>
	</xsl:if>

	</xsl:element>
	</xsl:template>


	<xsl:template name="dopreviewoperator"> 
		<xsl:param name="operatorid"/>					

		<xsl:for-each select="//mashup:*[@id=$operatorid]" >
			<xsl:call-template name="dooneoperator"></xsl:call-template>
			<xsl:for-each select="mashup:input" >
				<xsl:call-template name="dopreviewoperator"><xsl:with-param name="operatorid" select="@id" /></xsl:call-template>				
			</xsl:for-each>
		</xsl:for-each>

	</xsl:template>


	<xsl:template name="dooneoperator"> 
		<xsl:param name="flowaction"/>					
			<xsl:choose>
			<xsl:when test="local-name()='source'">
					<xsl:call-template name="source"/>
			</xsl:when>
			<xsl:when test="local-name()='publish' and ($flowaction!='Preview' and $flowaction!='StoreRaw') ">
				<xsl:call-template name="publish"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='filter'">
				<xsl:call-template name="filter"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='merge'">
				<xsl:call-template name="merge"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='combine' ">
				<xsl:call-template name="combine"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='sort'">
				<xsl:call-template name="sort"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='transform'">
				<xsl:call-template name="transform"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='groupby'">
				<xsl:call-template name="groupby"></xsl:call-template>
			</xsl:when>
			<xsl:when test="@type='foreach'">
				<xsl:call-template name="foreach"></xsl:call-template>
			</xsl:when>
			</xsl:choose>
	</xsl:template>


	<xsl:template name="doonesourceparam"> 
		<xsl:choose>	
<!-- temp workaround put in for QA. Should be removed in the release -->
			<xsl:when test="position()=1 and @caption='domain' and @operatorspecific='file'"></xsl:when>
			<xsl:when test="position()=1 and @caption='domain'"><xsl:value-of select="@operatorspecific"/>://</xsl:when>
			<xsl:when test="@operatorspecific='/'">/</xsl:when>
			<xsl:when test="@operatorspecific='?'">?</xsl:when>
			<xsl:when test="@operatorspecific='/?'">/?</xsl:when>
			<xsl:when test="@operatorspecific='/?='">/?<xsl:value-of select="@caption"/>=</xsl:when>
			<xsl:when test="@operatorspecific='?='">?<xsl:value-of select="@caption"/>=</xsl:when>
			<xsl:when test="@operatorspecific='='"><xsl:text  disable-output-escaping="yes">&amp;</xsl:text><xsl:value-of select="@caption"/>=</xsl:when>
			<xsl:otherwise>&amp;</xsl:otherwise>
		</xsl:choose>

		<xsl:variable name="paramid" select="@id"/>
		<xsl:choose>
			<xsl:when test="//mashup:param[text()=$paramid]">$<xsl:value-of select="@id"/><xsl:value-of select="//mashup:param[text()=$paramid]/@id"/></xsl:when>
			<xsl:when test="@type='variable'">$<xsl:value-of select="text()"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="text()"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="source"> 
		<xsl:variable name="notprimitive" select="count(//mashup:flow[@primitive='true'])=0"/>
		<xsl:choose>
			<xsl:when test="@type!='' and $notprimitive">
				<xsl:element  namespace="" name="IMPORT-FEED">
					<xsl:attribute name="ALIAS">$<xsl:value-of select="@id"/></xsl:attribute>
					
					<xsl:variable name="settype">
						<xsl:choose>
							<xsl:when test="@type='ATOM'"><xsl:value-of select="@type"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="@type"/>				
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<xsl:element  namespace="" name="ITEM">
						 <xsl:attribute name="OUTVAR">$<xsl:value-of select="@id"/></xsl:attribute>
						<xsl:attribute name="FTYPE"><xsl:value-of select="$settype"/></xsl:attribute>
						<xsl:attribute name="TYPE">URL</xsl:attribute>
						<xsl:attribute name="FRESHNESS"><xsl:value-of select=".//mashup:advanced/@interval"/></xsl:attribute>

						<xsl:element  namespace="" name="SOURCE">
							<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>
								<xsl:for-each select="mashup:param" >
									<xsl:call-template name="doonesourceparam"></xsl:call-template>
								</xsl:for-each>
							<xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
						</xsl:element>
						<xsl:element  namespace="" name="CONTAINER"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select=".//mashup:advanced/@repeatelement"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element  namespace="" name="IMPORT">
					<xsl:attribute name="ALIAS">$<xsl:value-of select="@id"/></xsl:attribute>
					<xsl:element  namespace="" name="ITEM">
						 	<xsl:attribute name="OUTVAR">$<xsl:value-of select="@id"/></xsl:attribute>
						  	<xsl:attribute name="TYPE">URL</xsl:attribute>
							<xsl:attribute name="FRESHNESS"><xsl:value-of select=".//mashup:advanced/@interval"/></xsl:attribute>
							<xsl:element  namespace="" name="PATH">
								<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:for-each select="mashup:param" >
									<xsl:call-template name="doonesourceparam"></xsl:call-template>
								</xsl:for-each>
							<xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
						</xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template name="getvalidid">
		<xsl:param name="xpath"/>
	 	<xsl:choose>
	 		<xsl:when test="$xpath">$<xsl:value-of select="$xpath"/></xsl:when>
	 		<xsl:otherwise>
	 		</xsl:otherwise>
	 	</xsl:choose>
	</xsl:template>

	<xsl:template name="filter"> 
		<xsl:element  namespace="" name="FILTER-FEED">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INOP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:call-template name="doconditions"></xsl:call-template>
		</xsl:element>
	</xsl:template>

	

	<xsl:template name="combine"> 
		<xsl:element  namespace="" name="UNION-MULTIOP">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:for-each select="mashup:input" >
				<xsl:element  namespace="" name="ITEM">
					<xsl:attribute name="INVAR">$<xsl:value-of select="@id"/></xsl:attribute>
					<xsl:attribute name="INOP">$<xsl:value-of select="@id"/></xsl:attribute>
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>


	<xsl:template name="merge"> 
		<xsl:element  namespace="" name="MERGE-FEEDS">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTEROP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='0']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTERVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='0']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INNEROP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='1']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INNERVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='1']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:call-template name="doconditions"></xsl:call-template>
		</xsl:element>
	</xsl:template>


	<xsl:template name="sort"> 
		<xsl:element  namespace="" name="SORT-FEED">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INOP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
	
				<xsl:for-each select="mashup:condition" >
					<xsl:call-template name="dobindexp"></xsl:call-template>
				</xsl:for-each>
				<xsl:if test="count(mashup:datatype)>0">

					<xsl:for-each select="mashup:condition/mashup:param[@position='0']" >
						<xsl:variable name="path" select="."/>
						<xsl:if test="../../mashup:datatype[text()=$path]">
							<xsl:element  namespace="" name="EXPR">
							<xsl:element  namespace="" name="OUTPUT"><xsl:element  namespace="" name="LITERAL">
								<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="../@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
							</xsl:element>
							<xsl:call-template name="doxpath"></xsl:call-template>
							</xsl:element>
							</xsl:element>
						</xsl:if>
					</xsl:for-each>
				</xsl:if>
				<xsl:for-each select="mashup:condition[mashup:param[@position='0']/text()!='']" >
					<xsl:sort data-type="text" order="ascending" select="@position"/>
					<xsl:element  namespace="" name="KEY">
						<xsl:attribute name="ORDER">
							<xsl:choose><xsl:when test="mashup:param[@position='1']/text()=0">A</xsl:when><xsl:otherwise>D</xsl:otherwise></xsl:choose>
						</xsl:attribute> 
						<xsl:variable name="path" select="mashup:param[@position='0']/text()"/>
						<xsl:choose>
							<xsl:when test="../mashup:datatype[text()=$path]">
								<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="mashup:param[@position='0']/@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:element>
				</xsl:for-each>
			</xsl:element>
	</xsl:template>


	<xsl:template name="publish"> 
		<xsl:if test="count(mashup:input)>0">
			<xsl:variable name="ptype">
				<xsl:choose>
					<xsl:when test="mashup:param[@position='0']/text()='ATOM'  ">application/atom+xml</xsl:when>
					<xsl:when test="mashup:param[@position='0']/text()='RSS'  ">application/rss+xml</xsl:when>
					<xsl:otherwise>application/xml</xsl:otherwise>
				</xsl:choose>
			</xsl:variable> 

			<xsl:element  namespace="" name="PUBLISH-FEED">
			<xsl:attribute name="ALIAS">$<xsl:value-of select="@id"/></xsl:attribute>
			<xsl:attribute name="OUTVAR">$<xsl:value-of select="@id"/></xsl:attribute>
			<xsl:attribute name="TYPE"><xsl:value-of select="$ptype"/></xsl:attribute>
			<xsl:attribute name="INOP">$<xsl:value-of select="mashup:input/@id"/></xsl:attribute>
			<xsl:attribute name="INVAR">$<xsl:value-of select="mashup:input/@id"/></xsl:attribute>

			<xsl:variable name="p1">
				<xsl:choose>
					<xsl:when test="mashup:param[@position='1']/text()!=''"><xsl:value-of select="mashup:param[@position='1']/text()"/></xsl:when>
					<xsl:when test="mashup:param[@position='0']/text()='XML'  ">item</xsl:when>
					<xsl:otherwise>Title</xsl:otherwise>
				</xsl:choose>
			</xsl:variable> 
			<xsl:variable name="p2">
				<xsl:choose>
					<xsl:when test="mashup:param[@position='2']/text()!=''"><xsl:value-of select="mashup:param[@position='2']/text()"/></xsl:when>
					<xsl:when test="mashup:param[@position='0']/text()='XML'  ">root</xsl:when>
					<xsl:otherwise>http://ibm.com/mashuphub</xsl:otherwise>
				</xsl:choose>
			</xsl:variable> 
			<xsl:variable name="p3">
				<xsl:choose>
					<xsl:when test="mashup:param[@position='3']/text()!=''"><xsl:value-of select="mashup:param[@position='3']/text()"/></xsl:when>
					<xsl:when test="mashup:param[@position='0']/text()='ATOM'  ">http://www.ibm.com</xsl:when>
					<xsl:otherwise>description</xsl:otherwise>
				</xsl:choose>
			</xsl:variable> 


			<xsl:choose>
				<xsl:when test="mashup:param[@position='0']/text()='RSS'">
					<xsl:attribute name="CONTAINER">item</xsl:attribute>
					<xsl:element  namespace="" name="HEADER"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>rss version="2.0"<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>channel<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>title<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="$p1"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/title<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>link<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="$p2"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/link<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>description<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="$p3"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/description<xsl:text  disable-output-escaping="yes">&gt;</xsl:text>{$<xsl:value-of select="mashup:input/@id"/>}<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/channel<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/rss<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:when>
				<xsl:when test="mashup:param[@position='0']/text()='ATOM'">
					<xsl:attribute name="CONTAINER">entry</xsl:attribute>
					<xsl:element  namespace="" name="HEADER"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>feed xmlns="http://www.w3.org/2005/Atom"<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>id<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="$p2"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/id<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>title type="text"<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="$p1"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/title<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>link rel="self" type="application/atom+xml" href="<xsl:value-of select="$p3"/>"/<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>updated<xsl:text  disable-output-escaping="yes">&gt;</xsl:text>{$__NOW}<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/updated<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>subtitle type="text"<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/subtitle<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>generator<xsl:text  disable-output-escaping="yes">&gt;</xsl:text>Mashup Hub Mashup Generator<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/generator<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>author<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>name<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="mashup:param[@position='4']/text()"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/name<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/author<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>rights<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:value-of select="mashup:param[@position='5']/text()"/><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/rights<xsl:text  disable-output-escaping="yes">&gt;</xsl:text>{$<xsl:value-of select="mashup:input/@id"/>}<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/feed<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:when>
				<xsl:when test="mashup:param[@position='0']/text()='XML' and count(mashup:param)>1">
					<xsl:attribute name="CONTAINER"><xsl:value-of select="$p1"/></xsl:attribute>
					<xsl:element  namespace="" name="HEADER"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="$p2"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text>{$<xsl:value-of select="mashup:input/@id"/>}<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/<xsl:value-of select="$p2"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="CONTAINER">item</xsl:attribute>
					<xsl:element  namespace="" name="HEADER"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:text  disable-output-escaping="yes">&lt;</xsl:text>item<xsl:text  disable-output-escaping="yes">&gt;</xsl:text>{$<xsl:value-of select="mashup:input/@id"/>}<xsl:text  disable-output-escaping="yes">&lt;</xsl:text>/item<xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:otherwise>
			</xsl:choose>
			</xsl:element>
		</xsl:if>
	</xsl:template>

	<xsl:template name="groupby"> 
		<xsl:element  namespace="" name="GROUP-FEED">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INOP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
				<xsl:call-template name="dobindexp"></xsl:call-template>
				<xsl:for-each select="mashup:param[@operatorspecific='grouped' and text()!='' ]" >
					<xsl:sort data-type="text" order="ascending" select="@position"/>
					<xsl:element  namespace="" name="GROUPEXP"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:for-each>
				<xsl:for-each select="mashup:param[@operatorspecific='nested' and text()!='' ]" >
					<xsl:sort data-type="text" order="ascending" select="@position"/>
					<xsl:element  namespace="" name="NESTEXP"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:for-each>

		</xsl:element>
	</xsl:template>


	<xsl:template name="foreach"> 
		<xsl:element  namespace="" name="AUGMENT-FEEDS">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTEROP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='0']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTERVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='0']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INNEROP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='1']/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INNERVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input[@position='1']/@id" /></xsl:call-template></xsl:attribute>
				<xsl:for-each select="mashup:condition[mashup:param/@type!='']" >
					<xsl:variable name="paramid" select="mashup:param[@position='0']/text()"/>
					<xsl:variable name="outvar">
						<xsl:choose>
							<xsl:when test="//mashup:param[@id=$paramid]/@type='dynamic' ">
							<xsl:value-of select="$paramid"/>
							</xsl:when>
							<xsl:otherwise>
							<xsl:value-of select="$paramid"/><xsl:value-of select="mashup:param[@position='0']/@id"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<xsl:element  namespace="" name="BINDEXP">
						<xsl:attribute name="TYPE">$EXTRACT</xsl:attribute>
						<xsl:attribute name="OUTVAR">$<xsl:value-of select="$outvar"/></xsl:attribute>
						<xsl:attribute name="INVAR">$<xsl:value-of select="mashup:param[@position='1']/@inputid"/></xsl:attribute>
						<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="mashup:param[@position='1']/text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
					</xsl:element>
				</xsl:for-each>
		</xsl:element>
	</xsl:template>


	<xsl:template name="transform"> 
		<xsl:element  namespace="" name="TRANSFORM-FEED">
			<xsl:attribute name="ALIAS"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="OUTVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INOP"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:attribute name="INVAR"><xsl:call-template name="getvalidid"><xsl:with-param name="xpath" select="mashup:input/@id" /></xsl:call-template></xsl:attribute>
			<xsl:for-each select=".//mashup:param[@type='xpath'] " >
					<xsl:element  namespace="" name="BINDEXP">
						<xsl:attribute name="OUTVAR">$<xsl:value-of select="@id"/></xsl:attribute>
						<xsl:attribute name="TYPE">$EXTRACT</xsl:attribute>
						<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
					</xsl:element>
			</xsl:for-each>


			<xsl:if test="count(.//mashup:param[@type='function'] )>0 " >
			<xsl:element  namespace="" name="EXPR">
				<xsl:for-each select=".//mashup:param[@type='function'] " >
					<xsl:element  namespace="" name="OUTPUT"><xsl:element  namespace="" name="LITERAL">
						<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text> $<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
					</xsl:element>
					<xsl:call-template name="dofunction"></xsl:call-template>
				</xsl:element>
			</xsl:for-each>
			</xsl:element>
			</xsl:if>

			<xsl:element  namespace="" name="TEMPLATE">
			<xsl:for-each select=".//mashup:template/*" >	 
				<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:call-template name = "tostring"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
			</xsl:for-each>
			</xsl:element>

			<xsl:for-each select="./mashup:param" >
				<xsl:element  namespace="" name="REPLACE">
					<xsl:attribute name="INVAR">$<xsl:value-of select="@id"/></xsl:attribute>
					<xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="@operatorspecific"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<xsl:template  name="tostring" >
		<xsl:param name="ns"/>					
<xsl:choose><xsl:when test="self::text()">
<xsl:value-of disable-output-escaping="no" select="."/>
</xsl:when>
<xsl:otherwise>
	<xsl:variable name="elemns" select="namespace-uri()" />
	<xsl:variable name="elemprefix" select="//mashup:namespaces/mashup:namespace[.=$elemns]/@prefix" />
<xsl:choose>
<xsl:when test="string-length(namespace-uri())>0 and count(//mashup:namespaces/mashup:namespace[.=$elemns]) ">
	<xsl:text  disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="$elemprefix"/>:<xsl:value-of  select="local-name()"/> xmlns:<xsl:value-of select="$elemprefix"/>="<xsl:value-of select="namespace-uri()" />"</xsl:when>
	<xsl:otherwise><xsl:text  disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of  select="name()"/></xsl:otherwise></xsl:choose >	
<xsl:for-each select="@*" ><xsl:text> </xsl:text> <xsl:value-of  select="name()"/>="<xsl:value-of  select="."/>"</xsl:for-each><xsl:text  disable-output-escaping="yes">&gt;</xsl:text><xsl:for-each select="node()" ><xsl:call-template name = "tostring"></xsl:call-template></xsl:for-each>
<xsl:choose>
<xsl:when test="string-length(namespace-uri())>0 and count(//mashup:namespaces/mashup:namespace[.=$elemns]) ">
<xsl:text  disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of select="$elemprefix"/>:<xsl:value-of  select="local-name()"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text></xsl:when><xsl:otherwise>
<xsl:text  disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of  select="name()"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text></xsl:otherwise></xsl:choose>
</xsl:otherwise></xsl:choose >	
	</xsl:template>	



	<xsl:template name="doparams"> 
		<xsl:for-each select="mashup:param">
			<xsl:choose>
				<xsl:when test="@type='xpath'">
					<xsl:call-template name="doxpath"></xsl:call-template>
 				</xsl:when>
				<xsl:when test="@type='function'">
					<xsl:call-template name="dofunction"></xsl:call-template>
 				</xsl:when>
				<xsl:when test="@type='dynamic'">
					<xsl:element  namespace="" name="VARIABLE"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
 				</xsl:when>
				<xsl:when test="@type='variable'">
					<xsl:element  namespace="" name="VARIABLE"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
 				</xsl:when>
				<xsl:otherwise>
					<xsl:element  namespace="" name="LITERAL"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>		
	</xsl:template>



	<xsl:template name="dobindexp"> 
		<xsl:for-each select=".//mashup:param[text()!='']">
			<xsl:choose>
				<xsl:when test="@type='xpath' and @inputid!='' ">
					<xsl:element  namespace="" name="BINDEXP">
						<xsl:attribute name="TYPE">$EXTRACT</xsl:attribute>
						<xsl:attribute name="OUTVAR">$<xsl:value-of select="@id"/></xsl:attribute>
						<xsl:attribute name="INVAR">$<xsl:value-of select="@inputid"/></xsl:attribute>
						<xsl:text disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="text()"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text>
					</xsl:element>
 				</xsl:when>
			</xsl:choose>
		</xsl:for-each>		
	</xsl:template>

	<xsl:template name="doconditions"> 
		<xsl:if test="count(mashup:condition)>0 and count(mashup:condition/mashup:param[text()!=''])>0 ">
			<xsl:for-each select="mashup:condition" >
				<xsl:call-template name="dobindexp"></xsl:call-template>
			</xsl:for-each>
				<xsl:element  namespace="" name="EXPR">
			<xsl:variable name="ccount" select="count(mashup:condition[.//mashup:param[text()!='']])"/>
			<xsl:for-each select="mashup:condition[.//mashup:param[text()!='']]" >
				<xsl:if test="position()=1">
						<xsl:text disable-output-escaping="yes">&lt;OUTPUT></xsl:text><xsl:text disable-output-escaping="yes">&lt;LITERAL></xsl:text><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text><xsl:text disable-output-escaping="yes">&lt;/LITERAL></xsl:text>
						<xsl:if test="$ccount>1">
							<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="translate(@conjunction,$lcase,$ucase)"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text>
						</xsl:if>
				</xsl:if>
					<xsl:call-template name="docondition"></xsl:call-template>

				<xsl:if test="position()=$ccount">
					<xsl:if test="$ccount>1">
						<xsl:text disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of select="translate(@conjunction,$lcase,$ucase)"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text>
					</xsl:if>
					<xsl:text disable-output-escaping="yes">&lt;/OUTPUT></xsl:text>
				</xsl:if>
			</xsl:for-each>
			</xsl:element>
		</xsl:if>
	</xsl:template>


	<xsl:template name="docondition"> 
		<xsl:choose>
			<xsl:when test="@operation='='">
				<xsl:text disable-output-escaping="yes">&lt;EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='!='">
				<xsl:text disable-output-escaping="yes">&lt;NOT-EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='&lt;' ">
				<xsl:text disable-output-escaping="yes">&lt;LESS-THAN&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='>' ">
				<xsl:text disable-output-escaping="yes">&lt;GREATER-THAN&gt;</xsl:text>
			</xsl:when>
			<xsl:when test="@operation='&lt;=' ">
				<xsl:text disable-output-escaping="yes">&lt;LESS-THAN-OR-EQUAL&gt;</xsl:text>
			</xsl:when>
			<xsl:when test="@operation='>=' ">
				<xsl:text disable-output-escaping="yes">&lt;GREATER-THAN-OR-EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='notcontains' ">
				<xsl:text disable-output-escaping="yes">&lt;NOT&gt;</xsl:text> 
				<xsl:text disable-output-escaping="yes">&lt;CONTAINS&gt;</xsl:text> 
			</xsl:when>

			<xsl:otherwise>
				<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="translate(@operation,$lcase,$ucase)"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:call-template name="doparams"></xsl:call-template>

		<xsl:choose>
			<xsl:when test="@operation='='">
				<xsl:text disable-output-escaping="yes">&lt;/EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='!='">
				<xsl:text disable-output-escaping="yes">&lt;/NOT-EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='&lt;' ">
				<xsl:text disable-output-escaping="yes">&lt;/LESS-THAN&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='>' ">
				<xsl:text disable-output-escaping="yes">&lt;/GREATER-THAN&gt;</xsl:text>
			</xsl:when>
			<xsl:when test="@operation='&lt;=' ">
				<xsl:text disable-output-escaping="yes">&lt;/LESS-THAN-OR-EQUAL&gt;</xsl:text>
			</xsl:when>
			<xsl:when test="@operation='>=' ">
				<xsl:text disable-output-escaping="yes">&lt;/GREATER-THAN-OR-EQUAL&gt;</xsl:text> 
			</xsl:when>
			<xsl:when test="@operation='notcontains' ">
				<xsl:text disable-output-escaping="yes">&lt;/CONTAINS&gt;</xsl:text> 
				<xsl:text disable-output-escaping="yes">&lt;/NOT&gt;</xsl:text> 
			</xsl:when>
			<xsl:otherwise>
				<xsl:text disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of select="translate(@operation,$lcase,$ucase)"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text> 
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>


	<xsl:template name="doxpath"> 
		<xsl:variable name="path" select="."/>
		<xsl:choose>
			<xsl:when test="../../mashup:datatype[text()=$path]">
					<xsl:element  namespace="" name="CAST">
					<xsl:element  namespace="" name="LITERAL"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text><xsl:value-of select="translate(../../mashup:datatype[text()=$path]/@type,$lcase,$ucase)"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
						<xsl:element  namespace="" name="SEQUENCE"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text> </xsl:element>
					</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element  namespace="" name="SEQUENCE"><xsl:text  disable-output-escaping="yes">&lt;![CDATA[</xsl:text>$<xsl:value-of select="@id"/><xsl:text  disable-output-escaping="yes">]]&gt;</xsl:text></xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="dofunction"> 
		<xsl:for-each select="*">
		<xsl:choose>
			<xsl:when test="@name='xpath' ">
			</xsl:when>
			<xsl:when test="@name='cast' ">
				<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="translate(@name,$lcase,$ucase)"/> type="<xsl:value-of select="@type"/>">
					<xsl:call-template name="doparams"></xsl:call-template>
				<xsl:text disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of select="translate(@name,$lcase,$ucase)"/><xsl:text  disable-output-escaping="yes">&gt;</xsl:text>

			</xsl:when>
			<xsl:otherwise>
				<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="translate(@name,$lcase,$ucase)"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>
					<xsl:call-template name="doparams"></xsl:call-template>
				<xsl:text disable-output-escaping="yes">&lt;/</xsl:text><xsl:value-of select="translate(@name,$lcase,$ucase)"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		</xsl:for-each>
	</xsl:template>


</xsl:stylesheet>





















