/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.Model");

dojo.declare("flow.Model", null, {
  flowNS: "http://ibm.com/mashuphub",
  mhubNS: "http://w3.ibm.com/mhub",
  atomNS: "http://www.w3.org/2005/Atom",

  xpathTypes: {firstType: 1,
               stringType: 2,
               anyType: 3},

  // the xml document
  xmlModel: null,

  // model shortcuts
  flowNode: null,
  dataNode: null,
  prologNode: null,
  publishNode: null,

  // indicator for when we load in mashups
  isLoading: false,

  // array of loaded BaseOperators
  operators: [],

  // reference to the canvas
  canvas: null,

  constructor: function(canvas) {
    this.xmlModel = flow.xml.loadDocument("<mashup xmlns='"+this.flowNS+"' xmlns:flow='"+this.flowNS+"' version='1.0' id='' url=''><flow action='' operatorid=''><prolog/><namespaces><namespace prefix='flow'><![CDATA["+this.flowNS+"]]></namespace><namespace prefix='mhub'><![CDATA["+this.mhubNS+"]]></namespace><namespace prefix='atom'><![CDATA["+this.atomNS+"]]></namespace></namespaces><variables caption='Global Variables'/><publish type='publish' caption='' id='publish1' x='-1' y='-1'><param caption='' operatorspecific='' inputid='' id='' position='0' type='' comments=''>XML</param><param position='1' caption='repeating' operatorspecific='' inputid='' id='repeating' type='text' comments=''>item</param><param caption='header' operatorspecific='' inputid='' id='header' position='2' type='text' comments=''>root</param></publish></flow><data/></mashup>");
    this.flowNode = this.getNode("/flow:mashup/flow:flow");
    this.dataNode = this.getNode("/flow:mashup/flow:data");
    this.prologNode = this.getNode("/flow:mashup/flow:flow/flow:prolog");
    this.publishNode = this.getNode("//flow:publish", this.flowNode);

    this.loadFunctions();
    this.canvas = canvas;
  },

  /* 
   * Loads in a mashup by passing in the XML root node or the serialized string
   *
   */
  load: function(flownode) {
    this.clear();

    if (typeof(flownode) == "string") {
      flownode = this.parseXML(flownode);
    } else {
      flownode = flow.xml._importNode(this.xmlModel, flownode, true);
    }

    this.publishNode = null;
    this.flowNode.parentNode.replaceChild(flownode, this.flowNode);
    this.flowNode = flownode;
    // since we replaced the flow node, reget the publish node
    this.publishNode = this.getNode("//flow:publish", this.flowNode);

    // now parse out all operators - runXPath can return a nodelist for IE
    var operators = [];

    var sources = this.runXPath("flow:source", this.flowNode);
    for (var i = 0; i < sources.length; i++) {
      operators.push(sources[i]);
    }

    var flowoperators = this.runXPath("flow:operator", this.flowNode);
    for (var i = 0; i < flowoperators.length; i++) {
      operators.push(flowoperators[i]);
    }

    for (var i = 0; i < operators.length; i++) {
      var op = operators[i];

      var type;
      if (op.nodeName == "source") {
        type = "source";
      } else {
        type = this.getValue(op, "@type");
      }

      this.addItem(type, op);
      this.canvas.createOperator(type, op.getAttribute("id"), op.getAttribute("x"), op.getAttribute("y"));
    }

    // handle publish operator
    if (operators.length > 0 && !this.canvas.hasEndpointOperator) {
      this.canvas.hasEndpointOperator = true;
      this.canvas.createPublishOperator();
    }

    operators.push(this.publishNode);

    // draw connections - gotta make sure all operators are loaded
    for (var i = 0; i < operators.length; i++) {
      var op = operators[i];
      var opid = this.getValue(op, "@id");

      var inputs = this.getInputs(opid);

      for (var j = 0; j < inputs.length; j++) {
        var input = inputs[j];
        var sourceid = this.getValue(input, "@id");
        this.canvas.createConnection(sourceid, opid);
      }
    }

    this.setProtected(false);
  },

  getOperatorCount: function() {
    var count = 0;
    for (var x in this.operators) {
      count++;
    }

    return count;
  },

  clear: function() {
    for (var id in this.operators) {
      if (this.operators[id].type != "publish") {
        this.removeItem(id);
      }
    }

    this.operators = [];
    this.removeNodes("flow:*[name()='source' or name()='operator']", this.flowNode)
    this.removeNodes("./flow:feed", this.dataNode)
    this.removeNodes("./flow:variables/flow:param", this.flowNode);

    this.setProtected(true);
  },

  createItem: function(type) {
    var count = 0;
    for (var id in this.operators) {
      count++;
    }

    var offset = count * 10;

    var id = this.addItem(type);
    this.canvas.createOperator(type, id, offset, offset, true);
    this.canvas.showPropertyEditorForNewOperator(id);

    this.setProtected(true);
  },

  addItem: function(type, node) {
    dojo.require("flow.operator.BaseOperator");

    // Upper case first letter
    type = type.substr(0, 1).toUpperCase() + type.substr(1);

    var op = new flow.operator[type+"Operator"]();

    if (node) {
      op.init(this, node.getAttribute("id"));
      op.node = node;
    } else {
      op.init(this);
      op.create();
    }

    this.operators[op.id] = op;

    this.setProtected(true);

    // if source and publish operator, add the endpoint publish operator
    if (op.type == "source" && !this.canvas.hasEndpointOperator) {
      this.canvas.createPublishOperator();
      this.canvas.hasEndpointOperator = true;
    }

    return op.id;
  },

  createPublishOperator: function() {
    dojo.require("flow.operator.BaseOperator");

    var node = this.getPublishNode();

    var op = new flow.operator.PublishOperator();
    op.init(this, node.getAttribute("id"));
    op.node = node;

    this.operators[op.id] = op;
  },

  removeItem: function(id) {
    var elm = this.getById(id);

    this.operators[id].destroy();
    delete this.operators[id];

    // return a list of all inputs this operator had
    var ids = [];
    var inputs = this.runXPath("flow:input", elm);
    for (var i = 0; i < inputs.length; i++) {
      ids.push(inputs[i].getAttribute("id"));
    }

    if (elm) {
      elm.parentNode.removeChild(elm);
    }

    this.setProtected(true);

    return ids;
  },

  connectOperators: function(srcId, targetId) {
    var pos = this.runXPath("flow:operator[@id='"+targetId+"']/flow:input", this.flowNode).length;
    var template = '<input xmlns="'+this.flowNS+'" timestamp="" caption="" id="'+srcId+'" position="'+pos+'"/>';

    var node;
    if (targetId == "publish1") {
      // special case publish
      node = this.getPublishNode();

      // mark as dirty
      this.setValue("@dirty", node, "1");
    } else {
      node = this.getNode("flow:operator[@id='"+targetId+"']", this.flowNode);
    }

    node.appendChild(this.parseXML(template));

    this.setProtected(true);
  },

  disconnectOperator: function(id) {
    // removes any inputs that use the operator being removed.  Returns an
    // array of ids of the operators who had an input
    var ids = [];

    var nodes = this.runXPath("//flow:input[@id='"+id+"']", this.flowNode);
    for (var i = 0; i < nodes.length; i++) {
      ids.push(nodes[i].getAttribute("id"));
      nodes[i].parentNode.removeChild(nodes[i]);
    }

    this.setProtected(true);

    return ids;
  },

  // removes the input and any associated params
  removeInput: function(node, srcId) {
    this.removeNode("flow:input[@id='"+srcId+"']", node);
    this.removeNode("//flow:param[@inputid='"+srcId+"']", node);
  },

  disconnectOperators: function(srcId, targetId) {
    var sourceOperator = this.operators[srcId];
    var targetOperator = this.operators[targetId];

    var handled = targetOperator.inputRemoved(sourceOperator);

    if (!handled) {
      var inputposition = this.getNode("flow:input[@id='"+srcId+"']", targetOperator.node).getAttribute("position");
      this.removeInput(targetOperator.node, srcId);

      if (targetOperator.maximumInputs == 1) {
        // only one input, so remove all conditions/params
        this.removeNodes("flow:param", targetOperator.node);
        this.removeNodes("flow:condition", targetOperator.node);
      }

      if (targetOperator.maximumInputs > 1 && this.getInputCount(targetId) == 0) {
        // all inputs gone, remove all children
        this.removeNodes("flow:param", targetOperator.node);
        this.removeNodes("flow:condition", targetOperator.node);
      }

      // reset positions of other inputs
      var inputs = this.runXPath("flow:input", targetOperator.node);
      for (var i = 0; i < inputs.length; i++) {
        var input = inputs[i];
        var position = this.getValue(input, "@position");

        if (position > inputposition) {
          this.setValue("@position", input, position-1);
        }
      }
    }

    this.setProtected(true);
  },

  clearOperatorData: function(operatorId) {
    var op = this.getOperatorById(operatorId);

    this.removeNodes("flow:param", op.node);
    this.removeNodes("flow:condition", op.node);
  },

  getOperatorById: function(id) {
    return this.operators[id];
  },

  getById: function(id) {
    return this.runXPath("//*[@id='"+id+"']", null, this.xpathTypes.firstType);
  },

  generateId: function(type) {
    return type + Math.ceil(Math.random()*10000)
  },

  runXPath: function(xpath, parent, type) {
    if (!parent) {
      parent = this.xmlModel.documentElement;
    }

    if (typeof(type) == "undefined" || !type) {
      type = this.xpathTypes.anyType;
    }

    var result;
    var nsProps = {flow:this.flowNS,mhub:this.mhubNS};

    if (type == this.xpathTypes.anyType) {
      result = flow.xml.findNodes(parent, xpath, nsProps);
    } else if (type == this.xpathTypes.firstType) {
      result = flow.xml.findNode(parent, xpath, nsProps);
    } else if (type == this.xpathTypes.stringType) {
      var res = flow.xml.findNode(parent, xpath, nsProps);
      //console.log(xpath)
      if (res.firstChild) {
        result = res.firstChild.nodeValue;
      } else {
        result = "";
      }
    }

    return result;
  },

  getCount: function(node, tagName) {
    return this.runXPath(tagName, node).length;
  },

  getInputCount: function(operatorId) {
    return this.getInputs(operatorId).length;
  },

  getInputs: function(operatorId) {
    var op = this.getOperatorById(operatorId);
    return this.runXPath("flow:input", op.node);
  },

  areAllInputsSatisfied: function(operatorId) {
    // make sure all inputs produce data

    var inputs = this.getInputs(operatorId);

    var res = {satisfied: true};

    for (var i = 0; i < inputs.length; i++) {
      var id = this.getValue(inputs[0], "@id");
      var op = this.getOperatorById(id);

      if (op.getTree() == null) {
        // can't find tree
        res.satisfied = false;
        res.cause = op;
        break;
      }
    }

    return res;
  },

  createElementNS: function(namespace, tagname) {
    return flow.xml._createElementNS(this.model, namespace, tagname);
  },

  getNode: function(xpath, startnode) {
    return this.runXPath(xpath, startnode, this.xpathTypes.firstType);
  },

  getValue: function(node, xpath) {
    var value = null;

    try {
      var node = this.getNode(xpath, node);

      if (node == null && xpath != null && xpath.indexOf("text()")!=-1) {
        if (xpath.substring(xpath.length-6) == "text()") {
          var newxpath = xpath.substring(0, xpath.length-6);
          if (newxpath.charAt(newxpath.length-1) == '/'){
            newxpath = newxpath.substring(0, newxpath.length-1);
          }

          node = this.getNode(newxpath, node);
        }
      }

      if (node) {
        var type = node.nodeType;

        if (type == Node.ATTRIBUTE_NODE) {
          value = node.value;
        } else if (type == Node.TEXT_NODE) {
          value = node.nodeValue;
        } else if (type == Node.CDATA_SECTION_NODE) {
          value = node.nodeValue;
        }
      }
    } catch(e) {
    }

    return value;
  },

  setValue: function(xpath, startnode, value) {
    var node = this.getNode(xpath, startnode);

    if (!node) {
      if (xpath != null && xpath.indexOf("text()") != -1) {
        var trailing = xpath.substring(xpath.length-6);
        if (trailing == "text()") {
          var newxpath= xpath.substring(0, xpath.length-6);
          if (newxpath.charAt(newxpath.length-1)=='/') {
            newxpath = newxpath.substring(0, newxpath.length-1);
          }

          if (newxpath == "" && startnode != null) {
            node = startnode;
          } else {
            node = this.getNode(newxpath, startnode);
          }

          if (node) {
            node.appendChild(this.xmlModel.createTextNode(value));
          }
        }
      } else {
        var lastindex = xpath.lastIndexOf("/");
        if (xpath.substr(lastindex+1, 1) == "@") {
          var newxpath = xpath.substr(0, lastindex);
          var attributename = xpath.substring(lastindex+2);

          if (!newxpath) {
            node = startnode;
          } else {
            node = this.getNode(newxpath, startnode);
          }

          if (node) {
            node.setAttribute(attributename, value);
          }
        }
      }
    } else {
      var type = node.nodeType;

      if (type == Node.ATTRIBUTE_NODE) {
        node.value = value;
      } else if (node.nodeType == Node.TEXT_NODE) {
        node.nodeValue = value;
      } else if (node.nodeType == Node.CDATA_SECTION_NODE) {
        node.nodeValue = value;
      }
    }

    this.setProtected(true);
  },

  removeNode: function(string, startnode, removeall) {
    if (removeall) {
      var nodes = this.runXPath(string, startnode);

      for (var i = nodes.length-1; i >= 0; i--) {
        nodes[i].parentNode.removeChild(nodes[i]);
      }
    } else {
      var node = this.getNode(string, startnode);

      if (node && node.parentNode) {
        node.parentNode.removeChild(node);
      }
    }

    this.setProtected(true);
  },

  removeNodes: function(string, startnode) {
    this.removeNode(string, startnode, true);
  },

  removeAllChildren: function(string, startnode) {
    var node = this.getNode(string,startnode);
    for (var i = node.childNodes.length-1; i >= 0; i--) {
      node.removeChild(node.childNodes[i]);
    }

    this.setProtected(true);
  },

  parseXML: function(string) {
    var node = null;

    try {
      var doc = flow.xml.loadDocument(string);
      node = flow.xml._importNode(this.xmlModel, doc.documentElement, true);
    } catch(e) {
    }

    return node;
  },

  removeDuplicateNodes: function(node) {
    try {
      node = (node==null) ? this.xmlRoot : node;

      var ns = node.namespaceURI;
      var nn = node.nodeName;
      var parent = node.parentNode;

      //remove duplicate siblings but copy their attributes  && children first
      for (var i=parent.childNodes.length-1; parent != null && i >= 0; i--)  {
        var tNode=parent.childNodes[i];
        if (tNode!=node && tNode.nodeName ==nn && tNode.namespaceURI==ns) {
          flow.xml.copyAttributes(node, tNode);
          for (var j=0;j<tNode.childNodes.length; j++) {
            if (tNode.childNodes[j].nodeType !=window.Node.TEXT_NODE) {
              node.appendChild(tNode.childNodes[j]);
            }
          }
          parent.removeChild(tNode);
        }
      }
      for (var i = 0; i < node.childNodes.length; i++) {
        this.removeDuplicateNodes(node.childNodes[i]);
      }

      this.setProtected(true);
    } catch(ex) {
      alert("removeDuplicateNodes:"+ex.name+"   "+ex.message);
    }
  },

  loadFunctions: function(node) {
    var myThis = this;
    dojo.xhrGet(
      {url: dojo._getModulePrefix("flow")+"/data/functions.xml",
       handleAs: "xml",
       timeout: 5000,
       load: function(response, ioArgs) {
         myThis.functionXML = response;
         return response;
       },

       error: function(response, ioArgs) {
         return response;
       }
      });
  },

  getDataType: function(xpath, operatorid) {
    var opnode = this.getOperatorById(operatorid).node;
    var datatype = this.getNode("flow:datatype[text()='./"+xpath+"']", opnode);

    var type = null;
    if (datatype) {
      type = this.getValue(datatype, "@type");
    }

    return type;
  },

  updateCoordinates: function(id, left, top) {
    var op = this.getOperatorById(id);
    this.setValue("@x", op.node, left);
    this.setValue("@y", op.node, top);

    this.setProtected(true);
  },

  getPublishNode: function() {
    return this.getNode("//flow:publish", this.flowNode);
  },

  getPublishType: function() {
    var param = this.getNode("//flow:publish/flow:param[@position=0]", this.flowNode);
    return this.getValue(param, "text()");
  },

  getPublishParam: function(type) {
    var param = this.getNode("//flow:publish/flow:param[@caption='"+type+"']", this.flowNode);
    return this.getValue(param, "text()");
  },

  getRequiredPublishParams: function(type) {
    if (type == "RSS") {
      return ["title", "link", "desc"];
    } else if (type == "ATOM") {
      return ["title", "id", "link", "author"];
    } else if (type == "XML") {
      return ["repeating", "header"];
    }
  },

  getPublishInputs: function() {
    return this.getInputs(this.getValue(this.publishNode, "@id"));
  },

  isPublishValid: function() {
    var isValid = true;
    var type = this.getPublishType();

    var requirements = this.getRequiredPublishParams(type);
    for (var i = 0; i < requirements.length; i++) {
      if (!this.getPublishParam(requirements[i])) {
        isValid = false;
        break;
      }
    }

    return isValid;
  },

  setProtected: function(p) {
    if (FLOW && FLOW.plugin) {
      FLOW.plugin.tab.setProtected(p);
    }
  },

  setPublishInput: function(ptype) {
    try {
      if(ptype!=null) this.setValue("flow:publish/flow:param/text()", this.flowNode, ptype);
      var operators = this.runXPath("flow:operator/@id", this.flowNode);
      if (operators.length > 0) {
        for (var i = 0; i < operators.length; i++) {
          var operatorid = operators[i].value;
          if (this.getValue(this.flowNode, "flow:operator/flow:input[@id='"+operatorid+"']/@id") == null) {
            this.setValue("flow:publish/flow:input/@id", this.flowNode, operatorid);
            break;
          }
        }
      } else {
        var operatorid=this.getValue(this.flowNode, "flow:source/@id");
        this.setValue("flow:publish/flow:input/@id", this.flowNode, operatorid);
      }
    }catch(ex){}
  },

  stripSctipt:function(str)
  {
  	return str.replace(/<script/g,"");
  },
  destroy: function(id) {
    for (var index in this.operators) {
      delete this.operators[index];
    }

    this.setProtected(true);
  }
});
