/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.data.XMLStore");

dojo.require("dojo.data.util.simpleFetch");

dojo.declare("flow.data.XMLStore", null, {
  path: null,
  removeDuplicates: false,

  constructor: function(params){
    this.rootElement = params.rootElement;
    this.removeDuplicates = params.removeDuplicates;

    this.setNamespace("http://ibm.com/mashuphub");

    this.nodeHash = [];
    this.typeHash = [];
  },

  nodeHash: null,

  operator : null,

  setNamespace: function(namespace) {
  	if (this.operator) {
  		return this.operator.setNamespace(namespace);
    }

    return "";
  },

  getNode: function(item) {
    return this.nodeHash[item.id];
  },

  getNamespace: function(item) {
    return this.getNode(item).namespaceURI;
  },

  getId: function(node) {
    //return node.xmlstoreid;

    // really really really slow
    for (var id in this.nodeHash) {
      if (this.nodeHash[id] == node) {
        return id;
      }
    }

    return null;
  },

  setId: function(node, id) {
    //node.xmlstoreid = id;
    this.nodeHash[id] = node;
  },

  _fetchItems: function(request, fetchHandler, errorHandler) {
    var query = request.query;

    var items = [];
    items.push(this.getItem(this.rootElement))

    fetchHandler(items, request);
  },

  hasAttribute: function(item, attribute) {
    return this.getValue(item, attribute);
  },

  getValue: function(item, attribute, defaultvalue) {
    var node = this.getNode(item);

    if (attribute == "children") {
      if (node.nodeType == 1) {
        return (node.childNodes.length > 0 || node.attributes.length > 0)
      }

      return false;
    } else if (attribute == "canHaveChildren") {
      return (node.nodeType == 1);
    } else if (attribute == "flowtype") {
      var xpath = this.buildXPath(node, true);
      return this.operator.model.getDataType(xpath, this.operator.id);
    }
  },

  getValues: function(item, attribute, defaultvalue) {
    var elm = this.getNode(item);

    var values = [];

    if (attribute == "children") {

      for (var i = 0; i < elm.attributes.length; i++){
        var item = this.getItem(elm.attributes[i]);
        if (item) {
          values.push(item);
        }
      }

      var dupeName;
      var dupeNS;

      for (var i = 0; i < elm.childNodes.length; i++) {
        var node = elm.childNodes[i];

        // if text node and is empty, don't load
        // TODO: we used to check for nextSibling to keep last text node around
        if ((node.nodeType == 3 || node.nodeType == 4) && !dojo.trim(node.nodeValue)) {
          continue;
        }

        if (this.removeDuplicates) {
          var name = flow.xml.getLocalName(node);
          var ns = node.namespaceURI;

          if (dupeName) {
            if (name == dupeName && ns == dupeNS) {
              continue;
            }
          }

          dupeName = name;
          dupeNS = ns;
        }

        var item = this.getItem(node);
        if (item) {
          values.push(item);
        }
      }
    }

    return values;
  },

  getAttributes: function(item) {
    var attributes = [];

    if (this.getNode(item).nodeType == 1) {
      // element
      attributes.push("children");
    }

    return attributes;
  },

  getLabel: function(item) {
    var node = this.getNode(item);

    if (node.nodeType == 1) {
      return flow.xml.getLocalName(node);
    } else if (node.nodeType == 2) {
      return node.nodeName + " = " + node.nodeValue;
    } else {
      var str = node.nodeValue;

      if (str.length > 30) {
        // check for image pattern
        var imgPattern = new RegExp("<img[^>]+>");

        if (!imgPattern.test(str)) {
          str = str.substr(0, 30) + "...";
        }
      }

      return str;
    }
  },

  getTitle: function(item) {
    var node = this.getNode(item);
    var title = null;

    if (node.nodeType == 3 || node.nodeType == 4 ) {
      var str = node.nodeValue;
      if (str.length > 30) {
        title = str;
      }
    } else {
      title = node.namespaceURI;
    }

    return title;
  },

  getIdentity: function(item) {
    // return unique id
    return item.id;
  },

  getFeatures: function() {
    var features = {
      "dojo.data.api.Read": true,
      "dojo.data.api.Identity": true,
      "dojo.data.api.Write": false,
      "dojo.data.api.Notification": true
    };

    return features;
  },

  getItem: function(node) {
    if (!node) {
      return;
    }

    // We have an issue here - each node needs a unique id, but the id cannot
    // change as the tree widget caches ids.  We could use xpath, except we have
    // editable trees, so the xpath could change but the node is the same (change
    // node name, add a same named element as a sibling, etc).  So we use random
    // ids and have a lookup hash of our own.

    var id = this.getId(node);

    if (!id) {
      id = Math.floor(Math.random()*1000000);
      this.setId(node, id);
    }

    return {id: id, store: this};
  },

  isItemLoaded: function(item) {
    return true;
  },

  buildXPath: function(node, usepath, ignoreduplicatenodes, skipParent, parentOverride) {
    var xpath = "";

    while (node) {
      if (node.nodeType == 2) {
        if (ignoreduplicatenodes) {
          xpath = "/@" + node.nodeName + xpath;
        } else {
          xpath = "/@" + node.nodeName + '=\"' + node.nodeValue + '"' + xpath;
        }

        node = node.ownerElement;

        // IE doesn't support owner element
        if (!node && parentOverride) {
          node = parentOverride;
        }
      } else if (node.nodeType == 1) {
        var ns = "";
        // ns is "" in IE
        if (node.namespaceURI != null && node.namespaceURI != "") {
        	ns = this.setNamespace(node.namespaceURI)+":";
        }

        var dupecount = 1;

        // check for any previous siblings with the same name
        var parent = node.parentNode;
        var localname = flow.xml.getLocalName(node);

        // skip if parent is dataNode
        if (parent && parent != this.operator.model.dataNode) {
          for (var i = 0; i < parent.childNodes.length; i++) {
            var child = parent.childNodes[i];
            if (child == node) {
              break;
            }

            if (flow.xml.getLocalName(child) == localname && child.namespaceURI == node.namespaceURI) {
              dupecount++;
            }
          }
        }

        var position = "";
        if (dupecount > 1 && !ignoreduplicatenodes) {
          position = "["+dupecount+"]"
        }

        xpath = "/" + ns + flow.xml.getLocalName(node) + position + xpath;
        node = node.parentNode;
      }	else if (node.nodeType == 3 || node.nodeType == 4 ) {
        var dupecount = 1;
        var parent = node.parentNode;
        for (var i = 0; i < parent.childNodes.length; i++) {
          var child = parent.childNodes[i];
          if (child == node) break;
          if (child.nodeType == 3 || child.nodeType == 4)  dupecount++;
        }

        xpath = "/text()["+dupecount+"]" +xpath;
        node = node.parentNode;
      } else {
        //xpath = "/" + node.nodeValue + xpath;
        node = node.parentNode;
      }

      if (skipParent) {
        break;
      }
    }

    if (!xpath) {
      xpath = "/";
    }

    if (usepath && this.path) {
      // TODO: this code looks really wrong.
      var startpos = xpath.indexOf(this.path) + this.path.length;
      if (startpos > 0) {
        xpath = xpath.substring(startpos);
      }
    }

    return xpath;
  },

  addNewNode: function(node, parentnode) {
    var importednode = flow.xml._importNode(this.rootElement.ownerDocument, node, true);

    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);

    parentnode.appendChild(importednode);
    var item = this.getItem(importednode);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  addNewAttribute: function(parentnode) {
    return this.addAttribute(parentnode, null, null);
  },

  addAttribute: function(parentnode, attrname, attrvalue) {
    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);

    attrname = (attrname != null) ? attrname : "newattr";
    attrvalue = (attrvalue != null) ? attrvalue: "";

    var count = 0;
    // IE has no hasAttribute
    while (typeof(this.getDOMAttribute(parentnode, attrname)) != "undefined") {
      count++;
      attrname = "newattr"+count;
    }

    parentnode.setAttribute(attrname, attrvalue);

    var newattr = this.getDOMAttribute(parentnode, attrname);

    var item = this.getItem(newattr);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  getDOMAttribute: function(node, name) {
    var attr;

    // IE doesn't allow foo.attributes["key"] - so walk!
    for (var i = 0; i < node.attributes.length; i++) {
      if (node.attributes[i].name == name) {
        attr = node.attributes[i];
        break;
      }
    }

    return attr;
  },

  addNewTextNode: function (parentnode) {
    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);
    var textnode = this.rootElement.ownerDocument.createTextNode("some text");
    parentnode.appendChild(textnode);

    var item = this.getItem(textnode);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  deleteNode: function(item,parentnode) {
    var node = this.getNode(item);

    if (node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4 ) {
      // element or textnode
      if (!node.parentNode) {
        return;
      }

      node.parentNode.removeChild(node);
    } else if (node.nodeType == 2) {
      // attribute
      if (dojo.isIE && parentnode ) parentnode.removeAttribute(node.name);
      else node.ownerElement.removeAttribute(node.name);
    }

    if (this.typeHash[item.id]) {
      delete this.typeHash[item.id];
    }

    this.onDelete(item);
  },

  cloneNode: function(node) {
    var newnode;

    if (node.namespaceURI) {
      newnode = this.rootElement.ownerDocument.createElementNS(node.namespaceURI, node.nodeName);
    } else {
      newnode = this.rootElement.ownerDocument.createElement(node.nodeName);
    }

    this.setId(newnode, this.getId(node));

    // copy over attributes
    for (var i = 0; i < node.attributes.length; i++) {
      var attr = node.attributes[i];
      newnode.setAttribute(attr.name, attr.value);

      // update hash as node has changed
      this.setId(this.getDOMAttribute(newnode, attr.name), this.getId(attr));
    }

    // copy over children
    for (var i = 0; i < node.childNodes.length; i++) {
      newnode.appendChild(this.cloneNode(node.childNodes[i]));
    }

    return newnode;
  },

  setValue: function(item, attribute, value, parentItem) {
    try   {
    var doc = this.rootElement.ownerDocument;
    var node = this.getNode(item);
    var clearChildren = false;

    if (attribute == "value") {
      if (node.nodeType == 1) {
        clearChildren = true;
        // build a new node
        var newnode;

        if (node.namespaceURI) {
          newnode = doc.createElementNS(node.namespaceURI, value);
        } else {
          newnode = doc.createElement(value);
        }

        // copy over attributes
        for (var i = 0; i < node.attributes.length; i++) {
          var attr = node.attributes[i];
          newnode.setAttribute(attr.name, attr.value);

          // update hash as node has changed
          this.setId(this.getDOMAttribute(newnode, attr.name), this.getId(attr));
        }

        // copy over children
        for (var i = 0; i < node.childNodes.length; i++) {
          newnode.appendChild(this.cloneNode(node.childNodes[i]));
        }

        // update id hash
        this.setId(newnode, item.id);

        // finally replace the old with the new
        if (this.rootElement == node) {
          this.rootElement = newnode;
        } else {
          var parent = node.parentNode;
          parent.replaceChild(newnode, node);
        }

        /*this.onDelete(item);
        item.element = newnode;
        var parentitem = null;
        if (parent) {
          parentitem = this.getItem(newnode.parentNode);
        }

        this.onNew(item, parentitem ? {item: parentitem, attribute: this.getAttributes(parentitem)} : null);*/
      } else if (node.nodeType == 2) {
        var owner = node.ownerElement;
        if (!owner) {
          owner = this.getNode(parentItem);
        }

        owner.removeAttribute(node.name)
        owner.setAttribute(value.name, value.value);

        // update the hash
        this.setId(this.getDOMAttribute(owner, value.name), item.id);

        /*var parentitem = this.getItem(owner);
        this.onDelete(item);
        item.element = owner.attributes[value.name];
        this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});*/
      } else if (node.nodeType == 3 || node.nodeType == 4 ) {
        node.nodeValue = value;
        //this.onSet(item);
      }
    }

    this.onSet(item);

    //console.log(flow.xml.innerXML(this.rootElement));
    }catch(ex){}
  },

  moveNode: function(item, direction) {
    var node = this.getNode(item);
    var parent = node.parentNode;

    var targetNode = (direction == "up") ? node.previousSibling : node.nextSibling;

    var clone = node.cloneNode(true);
    parent.removeChild(node);
    this.onDelete(item);

    if (direction == "up") {
      clone = parent.insertBefore(clone, targetNode);
    } else {
      if (targetNode.nextSibling) {
        // there is another node
        clone = parent.insertBefore(clone, targetNode.nextSibling);
      } else {
        // add to end
        clone = parent.appendChild(clone);
      }
    }

    // get position
    var pos = parent.attributes.length;

    for (var i = 0; i < parent.childNodes.length; i++) {
      if (parent.childNodes[i] == clone) {
        break;
      } else {
        pos++;
      }
    }

    var parentitem = this.getItem(parent);
    item = this.getItem(clone);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)}, pos);
    //console.log(flow.xml.innerXML(this.rootElement));
  },

  // dojo.data.api.Write
  newItem: function(newitem, parentInfo){
    var item = newitem.item;
    var node = item.store.getNode(item);

    this.addNewNode(node, parentInfo ? this.getNode(parentInfo.parent) : null);
  },

  /* dojo.data.api.Notification */
  /* Dummy functions that the store user will use */
  onSet: function(item, attribute, oldValue, newValue){
	},

  // addition - pass through the index
	onNew: function(newItem, parentInfo, index){
	},

	onDelete: function(deletedItem){
	}
});

//Mix in the simple fetch implementation to this class.
dojo.extend(flow.data.XMLStore, dojo.data.util.simpleFetch);
