/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Canvas");

dojo.require("flow.hale.EditArea");
dojo.require("flow.Model");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "labels");

dojo.declare(
  "flow.dijit.Canvas",
  flow.hale.EditArea,
  {
    // data
    model: null,
    contextMenu: null,
    dialogContainer: null,
    operators: null,

    // internal constants	-- for autoArrange()
    LAYOUTBEINGCOMPUTED:  -1,
    LAYOUTTOBECOMPUTED:   -2,
    LAYOUTSKIPLOOP:       -3,

    operatorDropped: false,
    hasEndpointOperator: false,

    dialogs: [],

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      this.inherited("postMixInProperties", arguments);
    },

    postCreate: function() {
      // init the model
      this.model = new flow.Model(this);
      this.operators = [];
      this.inherited("postCreate", arguments);

      // add class ourselves
      dojo.addClass(this.domNode, "flowcanvas");

      // listen for events
      dojo.connect(this.domNode, "onkeydown", this, "onKeyDown");
    },

    loadMashup: function(mashup) {
      this.model.isLoading = true;
      this.model.load(mashup);
      this.model.isLoading = false;
    },

    createOperator: function(type, id, x, y, setFocus) {
      if (!x) {
        x = 0;
      }

      if (!y) {
        y = 0;
      }

      // called when we load a model in
      var props = {__style:{left: parseInt(x, 10)+"px", top: parseInt(y, 10)+"px", position: "absolute"}};

      var widget = this.createComponent("flow.dijit.CanvasOperator", props);
      widget.operatorType = type;
      widget.operatorId = id;

      this.operators[id] = widget;

      // associative array -- has to explicitly increment length
      // used in autoArrange()
      this.operators.length++;

      dojo.require("btp.widget.command.CompoundCommand");
      dojo.require("btp.widget.command.EditCommand");

      var command = new btp.widget.command.CompoundCommand();
      command.append(new btp.widget.command.EditCommand.AppendChildCommand(this, widget));
      this.getEditDomain().executeCommand(command);

      if (setFocus) {
        var editor = btp.getEditors()[0];
        editor.setFocus(widget);
      }

      // init preview
      // TODO: may cause errors, should we try/catch?
      this.model.getOperatorById(id).doPreview();
    },

    createConnection: function(srcId, targetId) {
      var srcWidget = this.operators[srcId];
      var targetWidget = this.operators[targetId];

      var props = {
        sourceAnchor: srcWidget.id+":input",
        targetAnchor: targetWidget.id+":output"
      };

      var connection = this.createComponent("gdc.widget.CurvylineConnection", props);
      var offsetParent = gdl.getOffsetParent(srcWidget, targetWidget);

      if (!offsetParent) {
        offsetParent = this;
      }

      dojo.require("gdl.widget.command.ConnectionCommand");
      var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);

      this.getEditDomain().executeCommand(command);
      connection.setMouseOverHandler();
    },

    addOperator: function(widget) {
      // called when a user drag and drops an operator
      var id = this.model.addItem(widget.operatorType);
      this.model.updateCoordinates(id, widget.__style.left, widget.__style.top);
      widget.operatorId = id;
      this.operators[id] = widget;

      // associative array -- has to explicitly increment length
      // used in autoArrange()
      this.operators.length++;

      // show the property editor
      this.showPropertyEditorForNewOperator(id)
 
      return id;
    },

    createPublishOperator: function() {
      var node = this.model.getPublishNode();

      // create model operator
      this.model.createPublishOperator();

      var x = parseInt(node.getAttribute("x"), 10);
      var y = parseInt(node.getAttribute("y"), 10);

      if (!!x && !!y) {
        // calculate position - width/height of op is 140/51
        var mb = dojo.marginBox(this.domNode);
        x = mb.w - 140 - 20; // 20 to make it not be at the complete right hand side
        y = (mb.h / 2) - (51/2);
      }

      // create visual operator
      this.createOperator("publish", node.getAttribute("id"), x, y);
    },

    showPropertyEditorForNewOperator: function(id) {
      if (!this.operatorDropped) {
        this.showPropertyEditor(id);
        this.operatorDropped = true;
      } else {
        if (this.dialogs["property"]) {
          var widget = this.dialogs["property"].widget;
          // update visible property editor
          if (widget.isVisible()) {
            this.showPropertyEditor(id, true);
          }
        }
      }
    },

    connectWidgets: function(sourceWidget, targetWidget) {
      // TODO: checks

      this.model.connectOperators(sourceWidget.operatorId, targetWidget.operatorId);
      this.updateDialogs(targetWidget.operatorId);

      this.model.getOperatorById(sourceWidget.operatorId).connectedToTarget(true);
    },

    disconnectWidgets: function(sourceWidget, targetWidget) {
      this.disconnectWidgetsById(sourceWidget.operatorId, targetWidget.operatorId);
    },

    disconnectWidgetsById: function(srcid,targetid) {
      // TODO: checks
      // TODO: ask user if he is sure, given there are params/conditions

      // call model
      this.model.disconnectOperators(srcid, targetid);
      this.updateDialogs(targetid);
    },

    showPropertyEditor: function(operatorId, forceReload) {
      var operator = this.model.getOperatorById(operatorId);
      if (!operator) {
        // TODO: throw?
        return;
      }

      dojo.requireLocalization("flow", "propertyeditor");
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");

      var caption = operator.getName();
      var title = dojo.string.substitute(locale.PROPERTYEDITOR_TITLE, [operator.getLabel(), caption]);

      if (this.dialogs["property"]) {
        // check if same operator! and the property dialog is open
        if (!forceReload && this.dialogs["property"].propertyEditor && this.dialogs["property"].propertyEditor.operatorId == operator.id && this.dialogs["property"].widget.isVisible()) {
          return;
        }

        if (this.dialogs["property"].propertyEditor) {
          this.dialogs["property"].propertyEditor.destroyRendering();
          this.dialogs["property"].propertyEditor = null;
        }

        if (this.dialogs["property"].widget) {
          // move domNode
          var node = dojo.body().removeChild(this.dialogs["property"].widget.domNode);
          dojo.body().appendChild(node);
          this.dialogs["property"].widget.domNode = node;
        }

        this.dialogs["property"].widget.setTitle(title);
        this.dialogs["property"].widget.show();
      } else {
        dojo.require("flow.dijit.Dialog");
        dojo.requireLocalization("flow", "propertyeditor");

        var div = document.createElement("div");
        this.dialogContainer.appendChild(div);

        var widget = new flow.dijit.Dialog({title: title, resizable: true, closeable: true}, div);
        this.dialogs["property"] = {widget: widget};

        // size the dialog - 65% of screen, or 500px at most.
        var box = dojo.contentBox(dojo.body());
        var height = Math.floor(box.h * .65);

        if (height > 500) {
          height = 500;
        }

        widget.resize({w: 550, h: height});
        widget.show();
      }

      dojo.require("flow.dijit.PropertyEditor");
      var div = document.createElement("div");
      this.dialogs["property"].widget.containerNode.appendChild(div);

      this.dialogs["property"].propertyEditor = new flow.dijit.PropertyEditor({operator: this.model.operators[operator.id], canvas: this}, div);
    },

    closePropertyEditor: function(operatorId) {
      if (this.dialogs["property"]) {
        if (this.dialogs["property"].propertyEditor) {
          this.dialogs["property"].propertyEditor.destroyRendering();
          this.dialogs["property"].propertyEditor = null;
        }

        if (this.dialogs["property"].widget) {
          this.dialogs["property"].widget.hide();
        }
      }
    },

    getPropertyEditor: function(operator) {
      return this.dialogs["property"] ? this.dialogs["property"].propertyEditor : null;
    },

    showOperatorHelp: function(operator) {
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");
      var title = dojo.string.substitute(locale.HELP_TITLE, [operator.getLabel()]);
      var url = FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_"+operator.type+".htm";
      this.showHelp(url, title);
    },

    showHelp: function(url, title, chain) {
      if (!this.dialogs["help"]) {
        dojo.require("flow.dijit.Dialog");
        dojo.requireLocalization("flow", "propertyeditor");

        var div = document.createElement("div");
        this.dialogContainer.appendChild(div);
        var dialog = new flow.dijit.Dialog({title: title, resizable: true, closeable: true, chain: chain ? chain : null}, div);

        this.dialogs["help"] = {widget: dialog};

        dialog.resize({w: 550, h: 300});
        dialog.show();
      } else {
        if (this.dialogs["help"].iframe) {
          this.dialogs["help"].iframe.parentNode.removeChild(this.dialogs["help"].iframe);
          this.dialogs["help"].iframe = null;
        }

        if (chain) {
          this.dialogs["help"].widget.chain = chain;
          this.dialogs["help"].widget.updateZIndex();
        }

        // move domNode
        var node = dojo.body().removeChild(this.dialogs["help"].widget.domNode);
        dojo.body().appendChild(node);
        this.dialogs["help"].widget.domNode = node;

        this.dialogs["help"].widget.setTitle(title);
        this.dialogs["help"].widget.show();
      }

      if (this.dialogs["property"]) {
        // move help dialog if it has the same x coordinate as the property editor
        var helpx = dojo.style(this.dialogs["help"].widget.domNode, "left");

        var propx = dojo.style(this.dialogs["property"].widget.domNode, "left");

        if (propx == helpx) {
          var helpy = dojo.style(this.dialogs["help"].widget.domNode, "top");
          dojo.style(this.dialogs["help"].widget.domNode, "left", helpx + 55 + "px");
          dojo.style(this.dialogs["help"].widget.domNode, "top", helpy + 35 + "px");
        }
      }

      dojo.style(this.dialogs["help"].widget.containerNode, "overflow", "hidden");
      var iframe = document.createElement("iframe");
      iframe.style.width = "100%";
      iframe.style.height = "100%";
      iframe.style.borderWidth = "0";
      iframe.setAttribute("src", url);
      this.dialogs["help"].iframe = this.dialogs["help"].widget.containerNode.appendChild(iframe);
    },

    closeHelp: function() {
      if (this.dialogs["help"]) {
        this.dialogs["help"].iframe = null;
        this.dialogs["help"].widget.destroyRendering();
        this.dialogs["help"].widget = null;
        this.dialogs["help"] = null;
      }
    },

    updateDialogs: function(operatorId) {
      // don't update if no dialog or it is hidden
      if (!this.dialogs["property"] || (this.dialogs["property"].widget && !this.dialogs["property"].widget.isVisible())) {
        return;
      }

      // called when an operator is changed in a way that might require an
      // update of the visible dialogs
      var prop = this.dialogs["property"].propertyEditor;

      if (prop.operatorId == operatorId) {
        // the operator has a property editor

        // lets check if the property editor is in an error state
        if (prop.hasError) {
          // lets check if the error has been fixed
          if (!prop.checkForError()) {
            // error is gone, re-render the property editor
            this.showPropertyEditor(operatorId, true);
          }
        } else {
          // check if an error has occured, and if so, re-render the property editor
          if (prop.checkForError()) {
            this.showPropertyEditor(operatorId, true);
          }
        }
      }
    },

    destroyDialogsForOperator: function(operatorId) {
      if (this.dialogs["property"]) {
        if (this.dialogs["property"].propertyEditor && this.dialogs["property"].propertyEditor.operatorId == operatorId) {
        try
        {
          this.dialogs["property"].propertyEditor.destroyRecursive();
          this.dialogs["property"].widget.hide();
          this.dialogs["property"].widget.destroyRecursive();
          delete this.dialogs["property"];
         }
         catch(ex){}
        }
      }
    },

    removeOperator: function(operator) {
      if (operator.operatorType == "publish") {
        return;
      }

      if (!confirm(this.i18n.labels.OPERATOR_DELETE_CONFIRM)) {
        return;
      }

      // update hale
      this.deleteFocusWidget();

      // update model
      this.model.disconnectOperator(operator.operatorId)

      var ids = this.model.removeItem(operator.operatorId);
      for (var i = 0; i < ids.length; i++) {
        // show the out connector, as the target was removed
        this.operators[ids[i]].showOutConnector(true);
      }

      delete this.operators[operator.operatorId];

      // associative array -- has to explicitly decrement length
      // used in autoArrange()
      this.operators.length--;

      // destroy any dialogs associated with operator
      this.destroyDialogsForOperator(operator.operatorId);

      // unhook from context menu
      this.removeOperatorFromContextMenu(operator);

      // finally destroy the visual component
      operator.destroyRendering();
    },

    deleteAllOperators: function() {
      this.clear();
      this.closeHelp();
      this.model.clear();

      // associative array -- has to set length to 0
      // used in autoArrange()
      for (var operatorId in this.operators) {
        this.destroyDialogsForOperator(operatorId);
        delete this.operators[operatorId];
      }

      this.operators.length = 0;
      this.hasEndpointOperator = false;
    },

    addOperatorToContextMenu: function(operator) {
      if (!this.contextMenu) {
        this.contextMenu = new dijit.Menu();

        dojo.connect(this.contextMenu, "_openMyself", this, this.buildMenuItems);
      }

      this.contextMenu.bindDomNode(operator.shape.rawNode);
    },

    buildMenuItems: function(event) {
      // make sure we focus the target
      if (event.target.widget) {
        this.setFocus(event.target.widget);
      }

      if (!this.focusWidgets[0]) {
        return;
      }

      var contextmenu = this.contextMenu;
      dojo.forEach(contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      var type = this.model.getOperatorById(this.focusWidgets[0].operatorId).type;

      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.CONTEXTMENU_PROPERTIES,
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "propertyEditor")}));
      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.HELP, 
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "help")}));
      this.contextMenu.addChild(new dijit.MenuSeparator());
      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.CONTEXTMENU_DELETE,
                                                    disabled: (type == "publish"),
                                                    iconClass:"dijitEditorIcon dijitEditorIconDelete",
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "delete")}));

      var thisOperator = this.model.getOperatorById(this.focusWidgets[0].operatorId);
      var type = thisOperator.type;

  	  if (type != "publish") {
        this.contextMenu.addChild(new dijit.MenuSeparator());
        var xmlmodel = thisOperator.model;

        if (xmlmodel.runXPath("//flow:input[@id='"+thisOperator.id+"']").length == 0) {
          // create the submenu
          var pSubMenu = new dijit.Menu({parentMenu: this.contextMenu});

          var outputs = xmlmodel.runXPath("/flow:mashup/flow:flow/flow:*[(name()='operator' or name()='publish') and @id!='"+thisOperator.id+"']");
          for (var i = 0; i < outputs.length; i++) {
            var id = xmlmodel.getValue(outputs[i], "@id");
            var op = this.model.getOperatorById(id);

            var caption = op.getName();

            var maxinputs = op.maximumInputs;
            var actualinputs = xmlmodel.runXPath("flow:input", outputs[i]).length;

            var cyclicConnection = xmlmodel.runXPath("flow:input[@id='"+id+"']", thisOperator.node).length;

            if ((actualinputs < maxinputs || maxinputs == -1) && cyclicConnection == 0) {
              pSubMenu.addChild(new dijit.MenuItem({label: caption, onClick: dojo.hitch(this, "connectionMenuItemClick", id)}));
            }
          }

          // attach submenu to the main context menu
          this.contextMenu.addChild(new dijit.PopupMenuItem({label: this.i18n.labels.OPERATOR_MENU_CONNECTTO, popup: pSubMenu}));
        } else {
          var targetid = xmlmodel.getValue(xmlmodel.flowNode,"//flow:input[@id='"+this.focusWidgets[0].operatorId+"']/../@id");
          this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.OPERATOR_MENU_DISCONNECT,
                                    onClick: dojo.hitch(this, "disConnectionMenuItemClick", targetid)}));
        }
      }

      this.contextMenu.startup();
    },

    removeOperatorFromContextMenu: function(operator) {
      if (this.contextMenu) {
        this.contextMenu.unBindDomNode(operator.shape.rawNode);
      }
    },

	disConnectionMenuItemClick: function(action){
		this.disconnectWidgetsById(this.focusWidgets[0].operatorId,action);
		this.operators[this.focusWidgets[0].operatorId].showOutConnector(true);
		this.model.getOperatorById(this.focusWidgets[0].operatorId).connectedToTarget(false);

		// remove connections
		var compoundCommand = new btp.widget.command.CompoundCommand();
		var focusWidgets = this.focusWidgets;
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			for(var i = 0; i < 2; i++){
				var anchors = (i == 0) ? focusWidget.sourceConnectionAnchors : focusWidget.targetConnectionAnchors;
				if(i==1) continue; //we dont want to disconnect the inputs
				if(!anchors){ return; }
				for(var anchorName in anchors){
					var listeners = anchors[anchorName].listeners;
					if(!listeners){ continue; }
					for(var j = 0, len = listeners.length; j < len; j++){
						var connectionWidget = listeners[j];
						compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(connectionWidget));
					}
				}
			}
		}      
		this.getEditDomain().executeCommand(compoundCommand);
	},
	connectionMenuItemClick: function(action){
      this.createConnection(this.focusWidgets[0].operatorId, action);
	
	},
    contextMenuItemClick: function(action) {
      var selected = this.focusWidgets[0];
      if (action == "propertyEditor") {
        this.showPropertyEditor(selected.operatorId);
      } else if (action == "delete") {
        this.removeOperator(selected);
      } else if (action == "help") {
        // need the BaseOperator class
        this.showOperatorHelp(this.model.getOperatorById(selected.operatorId));
      }
    },

    onOperatorCaptionChanged: function(operator, caption) {
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");
      var title = dojo.string.substitute(locale.PROPERTYEDITOR_TITLE, [operator.getLabel(), caption]);

      this.dialogs["property"].widget.setTitle(title);
      this.operators[operator.id].setLabel(operator.getDisplayName());
    },

    onKeyDown: function(event) {
      if (event && event.keyCode == dojo.keys.DELETE && this && this.focusWidgets[0]) {
        this.removeOperator(this.focusWidgets[0]);
      }
    },

    refreshDirtyInputs: function(operatorId, callback) {
      var count = 0;
      var inputs = this.model.getInputs(operatorId);
      var operator;

      var myThis = this;

      function previewCallback(result) {
        operator.previewCallback(result);
        count++;

        if (count < inputs.length) {
          doPreview();
        } else {
          callback();
        }
      }

      function doPreview() {
        var id = inputs[count].getAttribute("id");
        operator = myThis.model.getOperatorById(id);

        myThis.model.setValue("@dirty", operator.node, "0");

        operator.doPreview(false, previewCallback);
      }

      doPreview();
    },

    autoArrange: function() {

    	if ( this.operators.length == 0 ) return;   // nothing to do

      // step 0 - cleanup or dynamically introduce working data members (properties) into each widget
      for (var operatorId in this.operators) {
		    var operator = this.operators[operatorId];
		    operator.dpCDPxlevel = this.LAYOUTTOBECOMPUTED;
		    operator.dpCDPylevel = this.LAYOUTTOBECOMPUTED;
      }

      // step 1. determine horizontal and vertical level of each widget
      //         Leaf operators (imports) are considered to be of horizontal level 0
      var  maxXLevel = 0;
      for (var operatorId in this.operators) {
		    var operator = this.operators[operatorId];
		    this.computeXLevel( operator );
		    if ( operator.dpCDPxlevel > maxXLevel ) maxXLevel = operator.dpCDPxlevel;
      }

    	// step 2. create an array to keep track of the next Y position for a given column i.e. X Level
      var  nextYLevel = new Array();
      for ( var i = 0 ;  i <= maxXLevel  ;  i++ )  nextYLevel[i] = 0;

      // setp 3. find a root (because of single out, we always have a tree) i.e. widget
      // with maximum X level, among widgets with not yet computed Y level
      while ( true ) {
		    var rootOperator = this.findXRoot( );
		    if ( rootOperator != null ) {
				rootOperator.dpCDPylevel = nextYLevel[ rootOperator.dpCDPxlevel ]++;
			    this.computeYLevel( nextYLevel, rootOperator );
        } else {
          break;
        }
      }

      // step 4. calculate and update top and left coordinates of each widget
      this.actualLayout( maxXLevel );
      this.adjustCanvasSize();

      // step 5. refresh bird view
      //this.firePropertyChanged("@ForceRefresh");
      var outlineView = this.getContentOutlinePage();
      if (outlineView != null) {
        outlineView.refreshOutline();
      }
 
    },

    computeXLevel: function( operator ) {
		if ( operator.dpCDPxlevel == this.LAYOUTTOBECOMPUTED ) {

		    operator.dpCDPxlevel = this.LAYOUTBEINGCOMPUTED;  // guard to stop "loops"

		    // recursively computeXLevel of upstream operators and take the max
		    var  maxChildLevel = -1;   // we always add 1 ====> xlevel >= 0

		    var  inputsXML   = this.model.getInputs(operator.operatorId);
		    var  upstreamOps = [];
		   	for (var i = 0  ;  i < inputsXML.length  ;  i++ )
		   	{
		   		upstreamOps[i] = this.operators[this.model.getValue(inputsXML[i], "@id")];
		   	}
		   	
		    for ( var i = 0  ;  i < upstreamOps.length  ;  i++ ) {
		        var  n = this.computeXLevel(  upstreamOps[i] );
		        if ( n == this.LAYOUTSKIPLOOP ) continue;  // skip and break the cycle
		        if ( n > maxChildLevel ) maxChildLevel = n;
		    }
		    operator.dpCDPxlevel = maxChildLevel + 1;
		    return operator.dpCDPxlevel; 

		} else if ( operator.dpCDPxlevel == this.LAYOUTBEINGCOMPUTED ) {
		    return this.LAYOUTSKIPLOOP;  // throw exception instead ????
		} else
		    return operator.dpCDPxlevel;
	},
	
	findXRoot: function(  ) {
		var maxLevel   = -1;   // level starts from 0, initialize to value less than 0
		var rootOperator = null;
		for ( var operatorId in this.operators ) {
			var	operator = this.operators[operatorId];

			if ( operator.dpCDPylevel != this.LAYOUTTOBECOMPUTED )  continue;  // skip computed entries
			
			if ( operator.dpCDPxlevel > maxLevel ) {
				maxLevel = operator.dpCDPxlevel;
			    rootOperator = operator;
			}
		}

		return rootOperator;
	},
	
	// set upstream widget's YLevel - for XLevel we set out own level based on upstream ops XLevel
	computeYLevel: function( nextYPosition, operator ) {

		var  inputsXML   = this.model.getInputs(operator.operatorId);
		var  upstreamOps = [];
		for (var i = 0  ;  i < inputsXML.length  ;  i++ )
	   	{
	   		upstreamOps[i] = this.operators[this.model.getValue(inputsXML[i], "@id")];
		}

		for ( var i = 0  ;  i < upstreamOps.length  ;  i++ ) {
		    if ( upstreamOps[i].dpCDPylevel != this.LAYOUTTOBECOMPUTED ) continue; // skip already assigned opers.  Happens because ofloop
		    // fix-up x level making all upstream operators 1 level away
		    upstreamOps[i].dpCDPxlevel = operator.dpCDPxlevel - 1;
		    upstreamOps[i].dpCDPylevel = nextYPosition[ operator.dpCDPxlevel - 1 ]++;
		    this.computeYLevel(  nextYPosition, upstreamOps[i] );
		}
	},

    actualLayout: function( maxXLevel ) {
      // layout one "column" (same x level) at a time from left to right
      var xMargin   = 135;
      var yMargin   = 51;

      var xPosition = 10;
      for ( var n = 0  ;  n <= maxXLevel   ;  n++ ) {

          for (var operatorId in this.operators) {
              var   operator = this.operators[operatorId];
              if ( operator.dpCDPxlevel != n || (operator.operatorType == "publish" && this.model.getPublishInputs().length == 0)) continue; // skip

              var yPosition = (2 * operator.dpCDPylevel + 1)  * yMargin;

              var props = {};
              props.left = xPosition;
              props.top = yPosition;
              this.model.updateCoordinates(operatorId, xPosition, yPosition);    		
            operator.update(props);
          }
          xPosition += xMargin;
      }
    },

    uninitialize: function() {
      if (this.contextMenu) {
        this.contextMenu.destroyRendering();
        this.contextMenu = null;
      }
    }
  }
);
