/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.CanvasOperator");

dojo.require("gdc.widget.AbstractNode");
dojo.require("gdc.widget.support.connectable");
dojo.require("gdc._base.shape");
dojo.require("dijit.Menu");

dojo.declare(
	"flow.dijit.CanvasOperator",
	gdc.widget.AbstractNode,
  {
    // hale stuff
    width: 140,
    height: 51,
    selected: false,
    anchor: "gdc.widget.anchor.FixedAnchor",

    // attach points
    bodyNode: null,
    outConnectorNode: null,
    outConnectorParentNode: null,
    outConnectorDeleteNode: null,
    inConnectorNode: null,

    // data
    type: null,
    operatorType: "",
    operatorId: null,

    // impl

    images: {
             left: {url: "op-left-open.png", selectedUrl: "op-left-open-selected.png", w: 8, h: 40},
             leftNoInput: {url: "op-left-closed.png", selectedUrl: "op-left-closed-selected.png", w: 8, h: 40},
             body: {url: "op-body.png", selectedUrl: "op-body-selected.png", w: 97, h: 40},
             outputLine: {url: "op-output.gif", w: 8, h: 2},
             right: {url: "op-plug.png", w: 20, h: 23},
             reflection: {url: "op-reflection.png", selectedUrl: "op-reflection-selected.png", w: 105, h: 13}
            },

    // can have inputs
    canHaveInputs: true,

    constructor: function(){
      dojo.extend(flow.dijit.CanvasOperator, gdc.widget.support.connectable);
		},

    postCreate: function() {
      this.inherited("postCreate", arguments);
      dojo.subscribe("btp:SelectionChanged", this, "selectionChanged");
    },

    getCanvas: function() {
      return this.getParent();
    },

    getOperator: function() {
      return this.getCanvas().model.operators[this.operatorId];
    },

    buildImageUrl: function(image, selected) {
      var url = "";

      if (selected && this.images[image].selectedUrl) {
        url = this.images[image].selectedUrl;
      } else {
        url = this.images[image].url;
      }

      return this.getImagePath() + "/" + url;
    },

    getImageData: function(name, selected) {
      var image = this.images[name];
      var data = {width: image.w, height: image.h, src: this.buildImageUrl(name, selected)};
      return data;
    },

    createTargetShape: function(parent) {
      var operator = dijit.byId("flowcanvas").model.getOperatorById(this.operatorId);
      this.canHaveInputs = (operator.minimumInputs > 0);
      // how many inputs
      this.maxTargetConnections = operator.maximumInputs;
      this.maxSourceConnections = 1;

      var shape = this.shape = parent.createGroup();

      var leftImage = this.inConnectorNode = shape.createGdcImage(this.getImageData(this.canHaveInputs ? "left" : "leftNoInput"));
      leftImage.setTransform({dx: 0});
      leftImage.rawNode.widget = this;

      var mainImage = this.bodyNode = shape.createGdcImage(this.getImageData("body"));
      mainImage.setTransform({dx:8});
      dojo.connect(this.bodyNode.rawNode, "onclick", this, "titleBarClick");
      this.bodyNode.rawNode.style.cursor = "move";

      // output line
      if (operator.hasOutput) {
        var output = shape.createGdcImage(this.getImageData("outputLine"));
        output.setTransform({dx:105, dy: 20});
      }

      this.reflectionNode = shape.createGdcImage(this.getImageData("reflection"));
      this.reflectionNode.setTransform({dx:0, dy: 40});

      if (operator.hasOutput) {
        var plugImage = this.outConnectorNode = shape.createGdcImage(this.getImageData("right"));
        plugImage.setTransform({dx:112, dy: 9});
        plugImage.rawNode.widget = this;

        plugImage.rawNode.style.cursor = "url('"+FLOW.baseResUrl+"/js/flow/dijit/templates/images/rightHandGrab.cur'), move";

        plugImage.rawNode.getDragTracker = function() {
          return "flow.hale.ConnectionDragCreationTool";
        }

        plugImage.rawNode.getConnectionClass = function(){
          return "gdc.widget.CurvylineConnection";
        };

        // set tooltip
        var labels = dojo.i18n.getLocalization("flow", "labels");
        plugImage.rawNode.setAttribute("title", labels.OPERATOR_PLUG_TOOLTIP);

        dojo.connect(this.outConnectorNode.rawNode, "onmouseover", this, "onConnectorNodeMouseOver");
        dojo.connect(this.outConnectorNode.rawNode, "onmouseout", this, "onConnectorNodeMouseOut");
      }

      dojo.require("gdc._base.shape");
      var labelText = this.labelText = shape.createGdcText({width: 59, height:17, text: operator.getDisplayName()});
      labelText.setFill(new dojo.Color("#000000"));
      labelText.setTransform({dx: 38, dy: 26});
 
      var fontName = "sans-serif"; // fr, de, it, es, pt, ca, da, nl, fi, el, no, sv, tr
      var locale = btpConfig.locale || dojo.locale;
      if(dojo.isFF && (dojo.isFF <= 2)) {
	      if(locale.indexOf("zh") != -1) {
	      	fontName = "SimHei";
	      } else if (locale.indexOf("jp") != -1) {
	      	fontName = "MS UI Gothic";
	      } else if (locale.indexOf("ko") != -1) {
	      	fontName = "Batang";
	      } else if (locale.indexOf("th") != -1) {
	      	fontName = "Tahoma";
	      } else if (locale.indexOf("ar") != -1) {
	      	fontName = "Arial";
	      } else if (locale.indexOf("he") != -1 || locale.indexOf("iw") != -1 ) {
	      	fontName = "Arial";
	      }
      }
      labelText.setFont({size: "10pt", family: fontName});

      labelText.rawNode.style.cursor = "move";
      dojo.connect(this.labelText.rawNode, "onclick", this, "titleBarClick");

      this.icon = shape.createGdcImage({width: 18, height: 19, src: this.getImagePath()+"/op-icon-"+this.operatorType+".png"});
      this.icon.setTransform({dx:13, dy: 11});
      dojo.connect(this.icon.rawNode, "onclick", this, "titleBarClick");

      this.createConnectionAnchors();

      // set label
      shape.rawNode.setAttribute("title", operator.getDisplayName());

      return shape;
    },

    onConnectorNodeMouseOver: function(e) {
      this.outConnectorNode.setShape({src: this.getImagePath() + "/op-plug-hover.png"});
    },

    onConnectorNodeMouseOut: function(e) {
      this.outConnectorNode.setShape({src: this.getImageData("right").src});
    },

    getShapeInfo: function(){
      return {
        type: 'group',
        width: this.getWidth(),
        height: this.getHeight()
      };
    },

    createConnectionAnchors: function() {
      dojo.require(this.anchor);
      var anchorImpl = dojo.getObject(this.anchor);

      // input anchor
      var anchor1 = new anchorImpl(this);
      anchor1.endPointShape = this.bodyNode;
      anchor1.centerOrigin = false;
      anchor1.offsetH = dojo.isIE ? 103 : 104;
      anchor1.offsetV = dojo.isIE ? 19 : 21;
      this.registerSourceConnectionAnchor(this.getShape().rawNode, "input", anchor1);

      // output anchor
      var anchor2 = new anchorImpl(this);
      anchor2.endPointShape = this.inConnectorNode;
      anchor2.centerOrigin = false;
      anchor2.offsetH = 10;
      anchor2.offsetV = 20;
      this.registerTargetConnectionAnchor(this.getShape().rawNode, "output", anchor2);
    },

    // for now, no focus frame
    hlp_setFocusFrame: function(){
    },

    hlp_removeFocusFrame: function(){
    },

    hlp_refreshFocusFrame: function(){
    },

    selectionChanged: function(message) {
      if (message.selection == this) {
        this.selected = true;
        this.inConnectorNode.setShape({src: this.buildImageUrl(this.canHaveInputs ? "left" : "leftNoInput", true)});
        this.bodyNode.setShape({src: this.buildImageUrl("body", true)});
        this.reflectionNode.setShape({src: this.buildImageUrl("reflection", true)});
      } else {
        if(this.selected) {
          this.selected = false;
          this.inConnectorNode.setShape({src: this.buildImageUrl(this.canHaveInputs ? "left" : "leftNoInput", false)});
          this.bodyNode.setShape({src: this.buildImageUrl("body", false)});
          this.reflectionNode.setShape({src: this.buildImageUrl("reflection", false)});
        }
      }
    },

    onSourceConnectionAdded: function(connection) {
      this.showOutConnector(false);

      if (this.getCanvas().model.isLoading) {
        // we are loading, so skip model notification
        return;
      }

      // notify model
      var targetWidget = connection.getTarget();

      this.getCanvas().connectWidgets(this, targetWidget);

      this.getOperator().connectedToTarget(true);
    },

    onRemovingSourceConnection: function(connection) {
      // notify model
      var targetWidget = connection.getTarget();
      this.getCanvas().disconnectWidgets(this, targetWidget);
      this.showOutConnector(true);
      this.getOperator().connectedToTarget(false);
    },

    titleBarMouseDown: function(event) {
      // focus node
      this.domNode.setAttribute("tabIndex", "-1");
      this.domNode.focus();
    },

    titleBarClick: function(event) {
      // if we were not moving the operator, show the property editor
      if (!this.moving) {
        this.getCanvas().showPropertyEditor(this.operatorId);
      } else {
        // since we were moving and now are after the move (click), clear it out
        this.moving = false;
      }
    },

    showOutConnector: function(flag) {
      if (flag) {
        this.outConnectorNode.rawNode.style.display = "";
      } else {
        this.outConnectorNode.rawNode.style.display = "none";
      }
    },

    setLabel: function(label) {
      this.labelText.setShape({text: label});

      // set label
      this.shape.rawNode.setAttribute("title", label);
    },

    hlp_onWidgetDragMove: function(event) {
      this.moving = true;
    },

    hlp_onWidgetDragEnd: function(event) {
      // finished moving - update the model with new coordinates
      this.getCanvas().model.updateCoordinates(this.operatorId, btp.getStyleValue(this, "left"), btp.getStyleValue(this, "top"));
    },

    // called when enter is pressed
    openContextMenu: function() {
      this.titleBarClick();
    },

    isTargetConnectionAcceptable: function(sourceNode) {
      // make sure you can't connect an operator to itself
      if (sourceNode == this){
        return false;
      } else {
        return this.inherited("isTargetConnectionAcceptable", arguments);
      }
    },

    getImagePath: function() {
      return dojo.moduleUrl("flow.dijit", "templates/images")
    },

    uninitialize: function() {
    }
  }
);
