/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Dialog");

dojo.require("dijit.Dialog");
dojo.require("dojo.dnd.move");
dojo.require("dojox.layout.ResizeHandle");

dojo.declare(
	"flow.dijit.Dialog",
	[dijit.Dialog, dijit._Container],
  {
    templateString: null,
    templatePath: dojo.moduleUrl("flow.dijit", "templates/Dialog.html"),
    actionNode: null,
    buttonNode: null,
    resizeHandleNode: null,
    operatorId: null,

    showUnderlay: false,
    hideTitleBar: false,
    hideActionBar: false,
    buttons: null,

    hasBeenResized: false,

    postCreate: function() {
      if (this.hideTitleBar) {
        this.titleBar.style.display = "none";
      }

      if (this.hideActionBar) {
        this.actionNode.style.display = "none";
      }

      //this.inherited("postCreate",arguments);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

      if (this.buttons) {
        for (var i = 0; i < this.buttons.length; i++) {
          var button = document.createElement("button")
          button.appendChild(document.createTextNode(this.buttons[i].label));

          if (this.buttons[i].onclick) {
            button.onclick = this.buttons[i].onclick;
          }

          this.buttons[i].node = this.buttonNode.appendChild(button);
        }
      } else {
        // since IE doesn't do min-height
        this.actionNode.style.height = "13px";
      }
    },

    _position: function() {
      if (this.domNode.parentNode == dojo.body()) {
        // default behavior
        if (!dojo.getComputedStyle(this.domNode)) {
          // means the document isn't visible, and dojo will barf
          return;
        }

        this.inherited("_position", arguments);
      } else {
        // Our dialogs sometimes need to live within tabs - so handle specially
        // - viewport is the FlowContainer
        var viewport = dojo.marginBox(this.domNode.parentNode.parentNode);
        var mb = dojo.marginBox(this.domNode);

        var style = this.domNode.style;
        style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
        style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
      }
		},

    show: function() {
      var doLayout = false;

      if (!this._alreadyInitialized) {
        this._setup();
        this._alreadyInitialized = true;
        doLayout = true;
      }

      this.domNode.style.display = "block";

      dojo.addClass(this.domNode, "flowdijitDialog");

      if (this.showUnderlay) {
        this._underlay.show();
      } else {
      }

      if (doLayout) {
        this.layout();
      }

      this.open = true;
      this._savedFocus = dijit.getFocus(this);
      dijit.focus(this.titleBar);

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    hide: function() {
      if (!this._alreadyInitialized) {
        return;
      }

      this.domNode.style.display = "none";

      if (this.showUnderlay) {
        this._underlay.hide();
      }

      dijit.focus(this._savedFocus);
      this.open = false;
    },

    isVisible: function() {
      return this.domNode.style.display == "block";
    },

    layout: function() {
      if (this.domNode.style.display == "block") {
        this._position();

        if (this._underlay) {
          this._underlay.layout();
          if (this.chain) {
            var zindex = parseInt(dojo.style(this._underlay.domNode, "zIndex"), 10)+this.chain;
            this._underlay.domNode.style.zIndex = zindex;
            this.domNode.style.zIndex = zindex+1;
          }
        } else if (this.chain) {
          this.updateZIndex();
        }
      }
    },

    resize: function(data) {
      var w = data.w;
      var h = data.h;

      // check if width/height puts us outside of window area
      if (this.hasBeenResized) {
        var coords = dojo.coords(this.domNode);
        var windowcoords = dojo.coords(dojo.body());

        //console.log("1: "+coords.x + " "+w+" > "+windowcoords.w);
        //console.log("2: "+coords.y + " "+h+" > "+windowcoords.h);

        if (coords.x + w >= windowcoords.w || coords.y + h >= windowcoords.h) {
          return;
        }
      } else {
        this.hasBeenResized = true;
      }

      dojo.style(this.domNode, "width", w+"px");
      dojo.style(this.domNode, "height", h+"px");

      // now resize the dialog contents
      var titlebarBox = dojo.marginBox(this.titleBar);

      // TODO: 4 is fixed padding (both sides, so 8), needs to be calculated
      // IE works differently, so no padding is needed (else we get a visual drawing bug)
      h -= titlebarBox.h + dojo.marginBox(this.actionNode).h + (dojo.isIE ? 1 : 9);
      dojo.style(this.containerNode, "height", h+"px");

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    resizeChildren: function(data) {
      var children = this.getChildren();
      for (var i = 0; i < children.length; i++) {
        if (children[i].resize) {
          children[i].resize();
        }
      }

      dojo.query(".flowResizable", this.domNode).forEach(function(node){
         dijit.byNode(node).resize();
      });
    },

    setTitle: function(title) {
      this.titleNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(title);
    },

    _setup: function() {
			if (this.titleBar) {
				this._moveable = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {handle: this.titleBar, area: "margin", within: true});
        dojo.connect(this._moveable, "onFirstMove", function(){this.handle.focus()});
			}

      if (this.showUnderlay) {
        this._underlay = new dijit.DialogUnderlay();
      }

      dojo.require("dojox.layout.ResizeHandle");

      var tmp = new dojox.layout.ResizeHandle({ 
        targetId: this.id,
        targetWidget: this,
        resizeAxis: "xy"
      }, this.resizeHandleNode);
    },

    updateZIndex: function() {
      var zindex = parseInt(dojo.style(this.domNode, "zIndex"), 10) + this.chain;
      dojo.style(this.domNode, "zIndex", zindex);
    }
  }
);
