/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.FlowComboBox");

dojo.require("dijit.form.ComboBox");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");
dojo.requireLocalization("flow", "labels");

dojo.declare(
  "flow.dijit.FlowComboBox",
  dijit.form.ComboBox,
  {
    // don't autocomplete
    autoComplete: false,
    dialog: null,
    callback: null,
    options: null,

    // parent chain count
    chain: null,
    chainParent: null,
    operator: null,
    selectType: "text",
    data: null,
    previousData: null,

    // for editing the value
    editButton: null,
    clearButton: null,
    handles: [],

    // icon stuff
    currentValueType: null,

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      // build our data store for the comobox dropdown
      this.operator = this;
      while (this.operator.chainParent) {
        this.operator = this.operator.chainParent;
      }

      var items = [];

      for (var i = 0; i < this.options.length; i++) {
        var caption = eval("this.operator.i18n.DROPDOWNCOMBO_LABEL_"+this.options[i].toUpperCase());

        // for variables check if any exist
        /*
        //Since this is checked when the property view is created. If the user creates a variable after the
        //property window is opened the change will not be reflected in the grid
        //moved the check to before opening the variable dialog
        if (this.options[i] == "variable") {
          if (!this.operator.getVariableCount()) {
            continue;
          }
        }
		*/
        items.push({name: caption, label: caption, type: this.options[i]});
      }

      this.store = new dojo.data.ItemFileReadStore({data: {identifier: "name", items: items}});

      // lets load in the text box code
      dijit.form.ValidationTextBox.prototype.postMixInProperties.call(this);
    },

    postCreate: function() {
      this.inherited("postCreate", arguments);

      // set text value
      if (this.data) {
        var type = this.data.type;
        this.showValue(type, this.data);

        if (this.data.tooltip) {
          this.textbox.setAttribute("title", this.data.tooltip);
        }

        if (type && type != "text") {
          this.createButtons();

          if (type != "xpath") {
            this.textbox.setAttribute("disabled", "true");
          }
          //this.hasDownArrow=false;
        }
      }
    },

    _startSearchFromInput: function() {
      // do nothing
    },

    getChainParent: function() {
      var parent = this;

      while (parent.chainParent) {
        parent = parent.chainParent;
        if (parent.model) {
          break;
        }
      }

      return parent;
    },

    showValue: function(type, data) {
      var newvalue;

      var icon = "";
      this.iconNode.style.visibility = "hidden";

      if (this.currentIconClass) {
        dojo.removeClass(this.iconNode, "flowtypeCombobox"+this.currentIconClass)
      }

      if (type == "variable") {
        var chainParent = this.getChainParent();
        var parentcaption;

        if (chainParent) {
          parentcaption = chainParent.model.getValue(chainParent.model.xmlModel, ".//flow:param[@id='"+data.value+"']/../@caption");
        }

        if (parentcaption == null) {
          parentcaption = chainParent.i18n.VARIABLE_GLOBAL_CAPTION;
        }

        newvalue = chainParent.createVariableLabel(parentcaption, chainParent.model.getValue(chainParent.model.xmlModel, ".//flow:param[@id='"+data.value+"']/@caption"));
        icon = "GetFromVariable";
      } else if (type == "dynamic") {
        newvalue = data.value ? data.value : "";
        icon = "DefineAsVariable";
      } else if (type == "function") {
        newvalue = data.name + "()";
        icon = "Function";
      } else if (type == "xpath") {
        newvalue = data.value;
        icon = "Tree";
      } else if (type == "text") {
        newvalue = data.value ? data.value : "";
        icon = "Text";
      }

      if (icon) {
        this.currentIconClass = icon
        dojo.addClass(this.iconNode, "flowtypeCombobox")
        dojo.addClass(this.iconNode, "flowtypeCombobox"+icon)
        this.iconNode.style.visibility = "visible";
      } else {
        this.currentIconClass = "";
      }

      this.setValue(newvalue);
    },

    _showResultList: function() {
      this.inherited("_showResultList", arguments);

      // TODO: big hack
      var zindex = parseInt(dojo.style(this._popupWidget.domNode.parentNode, "zIndex"), 10)+this.chain;
      this._popupWidget.domNode.parentNode.style.zIndex = zindex;
    },

    _doSelect: function(tgt) {
      var type = this.store.getValue(tgt.item, "type");

      if (type == "text" && this.item == tgt.item) {
        // if text and already text, do nothing
        return;
      }

      this.item = tgt.item;

      var type = this.store.getValue(tgt.item, "type");

      this.showInputSelection(type, this.data);
    },

    showInputSelection: function(type, data) {
      this.selectType = type;

      if (type == "text" || type == "xpath") {
        this.textbox.removeAttribute("disabled");
      } else {
        this.textbox.setAttribute("disabled", "true");
      }

      if (type == "text") {
        this.textbox.focus();
        this.showValue(type, "");
      } else {
				var div = document.createElement("div");
        dojo.body().appendChild(div);

        if (this.dialog) {
          this.dialog.hide();
          this.dialog.destroyRecursive();
          this.dialog = null;
        }

        dojo.require("flow.dijit.Dialog");
        this.dialog = new flow.dijit.Dialog({chain: this.chain, showUnderlay: true,
                                             buttons:[{label: this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)},
                                                      {label: this.i18n.labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);

        div = document.createElement("div");
        this.dialog.containerNode.appendChild(div);

        var width = 450, height = 350;

        if (type == "xpath") {
          dojo.require("flow.dijit.XMLInputSelection");
          this.selectWidget = new flow.dijit.XMLInputSelection({data: data, operator: this.operator, chain: this.chain}, div);
          this.dialog.setTitle(this.i18n.propertyeditor.GET_TREE_TITLE);
        } else if (type == "dynamic") {
          dojo.require("flow.dijit.RuntimeParameterInputSelection");
          this.selectWidget = new flow.dijit.RuntimeParameterInputSelection({data: data, operator: this.operator}, div);
          width = 200;
          height = 200;
          this.dialog.setTitle(this.i18n.propertyeditor.DECLARE_VARIABLE_TITLE);
        } else if (type == "function") {
          dojo.require("flow.dijit.FunctionInputSelection");
          this.selectWidget = new flow.dijit.FunctionInputSelection({data: data, chain: this.chain, chainParent: this.chainParent, operator: this.operator}, div);
          width = 450;
          height = 300;
          this.dialog.setTitle(this.i18n.propertyeditor.USE_FUNCTION_TITLE);
        } else if (type == "variable") {
          dojo.require("flow.dijit.Variables");
          this.selectWidget = new flow.dijit.Variables({model: this.getChainParent().model, chain: this.chain, canvas: this.operator.getCanvas()}, div);
          width = 600; height = 250;
          this.dialog.setTitle(this.i18n.labels.VARIABLES_DIALOG_TITLE);

        }

        this.dialog.resize({w: width, h: height});
        this.dialog.show();
      }
    },

    _onBlur: function() {
      this.inherited("_onBlur", arguments);

      // if type is text
      if (this.selectType == "text" || this.selectType == "xpath") {
        var data = flow.operator.Data.create(this.selectType);
        data.value = this.getDisplayedValue();
        this.callback(data);
      }
    },

    _acceptDialog: function(event) {
    	try {
	      var data = this.selectWidget.getData();

	      if (data) {
	        this.showValue(data.type, data);

	        this.callback(data);

          if (!this.previousData) {
            this.previousData = this.data;
          }

	        this.data = data;
	      }

	      if (this.selectType != "text") {
	        this.createButtons();
	      } else {
	        this.clearButtons();
	      }

	      this._cancelDialog();
      } catch(ex) {
        //debugger keyword chokes Dojo custom build
        //debugger;
        alert(ex)
      }
    },

    getDataType: function() {
      var type = (this.data) ? this.data.type : null;
      return type;
    },

    editButtonClick: function() {
      // edit the value
      this.showInputSelection(this.getDataType(), this.data);
    },

    clearButtonClick: function() {
      // create empty data
      var data;
      if (this.previousData) {
        data = this.previousData;
      } else {
        data = flow.operator.Data.create("text");
      }

      this.showValue("text", data);
      this.callback(data);

      this.selectType = null;
      // make sure it is editable
      this.textbox.removeAttribute("disabled");
      this.clearButtons();
    },

    createButtons: function() {
      if (!this.editButton) {
        // show buttons only if they don't exist
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.DROPDOWNCOMBO_BUTTON_EDIT));
        this.editButton = this.domNode.parentNode.appendChild(button);
        dojo.connect(this.editButton, "onclick", this, "editButtonClick");
      }

      if (!this.clearButton) {
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.DROPDOWNCOMBO_BUTTON_CLEAR));
        this.clearButton = this.domNode.parentNode.appendChild(button);
        dojo.connect(this.clearButton, "onclick", this, "clearButtonClick");
      }
    },

    clearButtons: function() {
      dojo.forEach(this.handles, dojo.disconnect);

      if (this.editButton) {
        this.editButton.parentNode.removeChild(this.editButton);
        this.editButton = null;
      }

      if (this.clearButton) {
        this.clearButton.parentNode.removeChild(this.clearButton);
        this.clearButton = null;
      }
    },

    _cancelDialog: function(event) {
      try {
        this.selectWidget.destroy();
        this.selectWidget = null;
      } catch(ex){}

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    },

    onkeypress: function(event) {
      // TODO: hack to allow ( and " - remove for Dojo 1.2
      if (dojo.isIE && (event.shiftKey && (event.keyCode == 40 || event.keyCode == 34))) {
        // do nothing
      } else {
        return this.inherited("onkeypress", arguments);
      }
    }
  }
);
