/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.FlowContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "labels");

dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.SplitContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.AccordionContainer");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");

dojo.require("flow.dijit.Canvas");
dojo.require("flow.dijit.PaletteItem");
dojo.require("flow.dijit.Palette");
dojo.require("flow.operator.BaseOperator");

//dojo.require("dijit.form.Button");

dojo.require("btp._base");
dojo.require("btp.widget.Model");
dojo.require("btp.widget.Palette")
dojo.require("gdl.widget.OutlineView");

dojo.declare(
  "flow.dijit.FlowContainer",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    "class": "flowContainer",

    // the id for the plugin
    flowid: null,

    // base ajax url
    ajaxUrl: null,

    // editor actions
    actions: null,

    hasBeenSaved: false,

    // attach points
    canvas: null,
    layoutcontainer: null,
    dialogContainer: null,
    outlineView: null,
    saveButtonNode: null,
    runAsMenuNode: null,

    i18n: null,

    // entry meta information
    title: "",
    creatorId: "",
    description: "",
    version: "",
    tags: "",
    isNew: false,

    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("flow.dijit", "templates/FlowContainer.html"),
    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      this.canvasmodel = new btp.widget.Model({src: dojo.moduleUrl("flow", "data/HaleOperators.xml").path});

      /*var doc = this.canvasmodel.getInstanceDocument();
      var elm = doc.createElement("component");
      var opts = [["name", "flow.dijit.CanvasOperator"],["description", "Combine"],["type", "combine"],["clickHandler", "btp.widget.tool.ResizeCreationTool"],["dropTargets", "connection"]];
      for (var i = 0; i < opts.length; i++) {
        var e = doc.createElement(opts[i][0]);
        e.appendChild(doc.createTextNode(opts[i][1]));
        elm.appendChild(e);
      }
      this.canvasmodel.getInstanceDocument().documentElement.firstChild.appendChild(elm);*/

      this.actions = {};
      this.updateActions( this.ajaxUrl );
    },

    postCreate: function() {
      dojo.connect(this.layoutcontainer, "layout", this, "layout");
      //this.canvas.reLayout();

      // we are in an iframe now
      this.canvas.dialogContainer = dojo.body(); //this.dialogContainer;

      // create menu items
//      var menu = this.runAsMenuNode
//      menu.addChild(new dijit.MenuItem({label: this.i18n.labels.RUN_RSS, onClick: dojo.hitch(this, this.onRunAsRSS)}));
//      menu.addChild(new dijit.MenuItem({label: this.i18n.labels.RUN_ATOM, onClick: dojo.hitch(this, this.onRunAsAtom)}));

      // setup outline
      var div = this.outlineView.appendChild(document.createElement("div"));
      //div.style.height = "100%";
      //div.style.width = "100%";
      var outlineview = new gdl.widget.OutlineView({target: "flow.dijit.Canvas", editArea: this.canvas}, div);

      // event catching for key bindings
      dojo.connect(dojo.body(), "onkeypress", this, "onKeyPress");
    },

    onKeyPress: function(event) {
      if (event.ctrlKey) {
        if (event.charCode == 109) {
          // ctrl-m - show model
          this.showModel();
        }
      }
    },

    layout: function() {
      // make sure dialogs are in view
      if (this.canvas.dialogs["property"]) {
        this.fitDialogIntoView(this.canvas.dialogs["property"].widget);
      }

      if (this.canvas.dialogs["help"]) {
        this.fitDialogIntoView(this.canvas.dialogs["help"].widget);
      }

      if (this.helpDialog) {
        this.fitDialogIntoView(this.helpDialog);
      }
    },

    fitDialogIntoView: function(dialog) {
      var top = parseInt(dojo.style(dialog.domNode, "top"), 10);
      if (top < 0) {
        dojo.style(dialog.domNode, "top", "0px");
      }

      // make sure the height is not taller than visible area
      var dialogbox = dojo.marginBox(dialog.domNode);
      var box = dojo.contentBox(dojo.body());

      // TODO: width resizing?
      var height = dialogbox.h
      var width = dialogbox.w
      var needToResize = false;

      if ((dialogbox.t + dialogbox.h) > box.h) {
        height = box.h - dialogbox.t - 10;
        needToResize = true;
      }

      if ((dialogbox.l + dialogbox.w) > box.w) {
        width = box.w - dialogbox.l - 10;
        needToResize = true;
      }

      if (needToResize) {
        dialog.resize({h: height, w: width});
      }
    },

    modelDialog: null,

    // updates the base ajax url and its editor actions
    updateActions: function ( baseAjaxUrl ) {
      if (baseAjaxUrl) {
        var actionList = [ "render", "store", "load", "preview", "execute", "cloneObject", "getObjectList", "close" ];
        this.ajaxUrl = baseAjaxUrl;

        for (var i = 0; i < actionList.length; i++) {
          this.actions[actionList[i]] = baseAjaxUrl + "/method/" + actionList[i];
        }
      }
    },

    getEditorActionUrl: function ( actionName ) {
      return this.actions[actionName];
    },

    onArrange: function() {
      var div = document.createElement("div");
      dojo.body().appendChild(div);
      dojo.require("flow.dijit.Dialog");

      var splashScreen = new flow.dijit.Dialog({title: "Arranging", hideTitleBar: true, hideActionBar: true, resizable: false}, div);
      var ml = "<center><span class='${0}' style='display: ${1}'><img src='${2}' /> ${3}</span></center>";
      var img = FLOW.baseUrl + "/images/spinner.gif";
      var str = dojo.string.substitute(ml, ["", "block", img, this.i18n.labels.ARRANGE_SPLASH_MESSAGE]);
      splashScreen.containerNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(str);
      splashScreen.resize({w: 300, h: 30});
      splashScreen.show();

      this.canvas.autoArrange();

      splashScreen.hide();
      splashScreen.destroyRecursive();
      splashScreen = null;
    },

    onClear: function() {
      if (confirm(this.i18n.labels.CLEAR_CONFIRM)) {
        // lets remove all operators
        this.canvas.deleteAllOperators();
      }
    },

    onLoad: function(id) {
      var actionUrl = this.getEditorActionUrl("load");
      var content = { id: id };
      this.callServer(actionUrl, content, dojo.hitch(this, this.loadCallback));
    },

    onSave: function() {
      // check if any operators exist.
      if (this.canvas.model.getOperatorCount() == 0) {
        return;
      }

      // check if publish is connected
      var publishinputs = this.canvas.model.getPublishInputs();
      if (!publishinputs || publishinputs.length == 0) {
        //alert(this.i18n.labels.ERROR_SAVE_PUBLISH_NO_INPUT);
        //return;
      }

      // check if the properties are set
      var isValid = this.canvas.model.isPublishValid();
      if (!isValid) {
        alert(this.i18n.labels.ERROR_SAVE_PUBLISH_DATA_INVALID);
      } else {
        this.doSave();
      }
    },

    doSave: function() {
      // save our plugin specific data
      if (this._hasBeenSaved()) {
      	this.showSaveProgress();
      }

      //this.setPublishInput(null);
      var actionUrl = this.getEditorActionUrl("store");
      var content = {storemode: 'normal', xml: flow.xml.innerXML (this.canvas.model.flowNode)};
      this.callServer( actionUrl, content, dojo.hitch(this, this.saveCallback) );
    },

    showDialog: function(title, buttons, width, height, hideUnderlay) {
      dojo.require("flow.dijit.Dialog");

      var div = dojo.body().appendChild(document.createElement("div"));
      var dialog = new flow.dijit.Dialog({title: title, buttons: buttons, showUnderlay: (hideUnderlay ? false : true), resizable: true, closeable: true}, div);

      if (!width) {
        width = 530;
      }

      if (!height) {
        height = 400;
      }

      dialog.resize({w: width, h: height});
      dialog.show();

      return dialog;
    },

    onShowHelp: function() {
      if (this.helpDialog) {
        this.helpDialog.hide();
        this.helpDialog.destroyRendering();
        this.helpDialog = null;
      }

      this.helpDialog = this.showDialog(this.i18n.labels.BUILDER_HELP_TITLE, null, null, null, true);

      dojo.require("dijit.layout.ContentPane");
      var url = FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_overview.htm";

      dojo.style(this.helpDialog.containerNode, "overflow", "hidden");
      var iframe = document.createElement("iframe");
      iframe.style.width = "100%";
      iframe.style.height = "100%";
      iframe.style.borderWidth = "0";
      iframe.setAttribute("src", url);
      this.helpDialog.containerNode.appendChild(iframe);
    },

    showSaveProgress: function() {
      dojo.require("flow.dijit.Dialog");
      var div = document.createElement("div");
      dojo.body().appendChild(div);
      this.saveProgressDialog = new flow.dijit.Dialog({resizable: false, closeable: false, hideTitleBar: true, hideActionBar: true}, div);
      this.saveProgressDialog.containerNode.innerHTML = "<center><img src='"+FLOW.baseUrl+"/images/spinner.gif'> "+this.i18n.labels.SAVE_PROGRESS_MSG+"</center>";
      this.saveProgressDialog.domNode.style.width = "300px";
      this.saveProgressDialog.domNode.style.zIndex = "1001";
      this.saveProgressDialog.show();
    },

    finishSave: function() {
    	if (this.saveProgressDialog) {
        this.saveProgressDialog.hide();
        this.saveProgressDialog.destroyRecursive();
        this.saveProgressDialog = null;
    	}
    },

    showSaveStatus: function(status) {
      var button = [{label:this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, _acceptDialog)}];
      var dialog = this.showDialog(this.i18n.labels.SAVE, button, 252, 100);

      function _acceptDialog() {
        dialog.hide();
        dialog.destroyRecursive();
        properties = null;
        dialog = null;
      }

      //dojo.require("dijit.layout.ContentPane");
      var div = document.createElement("div");
      div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(status);
      dialog.containerNode.appendChild(div);
    },

    showModel: function() {
      if (this.modelDialog) {
        this.modelDialog.hide();
        this.modelDialog.destroyRecursive();
        this.modelDialog = null;
      }

      this.onShowModel();
    },

    onShowModel: function() {
      this.modelDialog = this.showDialog(this.i18n.labels.MODEL_DIALOG_TITLE);

      dojo.require("dijit.layout.TabContainer");

      var div = document.createElement("div");
      this.modelDialog.containerNode.appendChild(div);
      dojo.require("flow.dijit.ShowModel");
      new flow.dijit.ShowModel({model: this.canvas.model}, div);
    },

    setPublishInput: function(ptype) {
      this.canvas.model.setPublishInput(ptype);
    },

    onRun: function(type) {
      var funcnode = this.canvas.model.getNode("flow:*[name()='operator' or name()='source']", this.canvas.model.flowNode);

      if (!funcnode) {
        var property = dojo.i18n.getLocalization("flow", "propertyeditor");
	  		alert(property.SERVER_EMPTY_MASHUP);
	  		return;
      }

      this.setPublishInput(null);
      document.flowContainer_runForm1.action = this.getEditorActionUrl("execute");
      document.flowContainer_runForm1.xml.value = flow.xml.innerXML(this.canvas.model.flowNode);

      // NOTE:  second arg cannot have space in between.  Will not work on IE.
      window.open("", "flowResult1", "");

      window.setTimeout("document.flowContainer_runForm1.submit();", 500); 
    },

    onRunAsRSS: function() {
      this.onRun("RSS");
    },

    onRunAsAtom: function() {
      this.onRun("ATOM");
    },

    getObjectMockupXml: function(mashupXml) {
       var xml = "";

       xml += '<OBJECTS><OBJECT>';
       xml += '<OBJECTID>35</OBJECTID>';
       xml += '<TITLE><![CDATA[CD Warehouse Mashup]]></TITLE>'
       xml += '<CREATORID>kfbombar</CREATORID>';
       xml += '<VERSION>1.0</VERSION>';
       xml += '<TIMESTAMP>1197935041</TIMESTAMP>';
       xml += '<DESCRIPTION><![CDATA[Example mashup for kfbombar]]></DESCRIPTION>';
       xml += '<STATUS>NEW</STATUS>';
       xml += '<URL><![CDATA[]]></URL>';
       xml += '<AJAXURL><![CDATA[http://localhost:8080/mashuphub/client/plugin/ajax/pluginid/6/entryid/35]]></AJAXURL>';
       xml += '<VISIBILITY>1</VISIBILITY>';
       xml += '<PUB>1</PUB>';
       xml += '<PARAMETERS/>';
       xml += '<TAGS/>';
       xml += '<FLOW>';
       xml += '<OBJECTID>35</OBJECTID>';
       xml += '<GRAPH><![CDATA[Data for the graph needed]]></GRAPH>';
       xml += '<RUNTIMEMODEL><MASHUP>' + mashupXml +'</MASHUP></RUNTIMEMODEL>';
       xml += '</FLOW>';
       xml += '</OBJECT></OBJECTS>';

       return xml;
    },

    _set_tab_title: function(title) {
       if (FLOW.plugin) {
          FLOW.plugin.tab.setTitle( title );
       }
    },

    _hasBeenSaved: function() {
       return ( FLOW.plugin.workflow.isReedit() || FLOW.plugin.workflow.isSaved() );
    },

    _isNewEntry: function(objectXml) {
       var status = objectXml.childNodes[6].firstChild.nodeValue;

       return (status == "NEW")?true:false;
    },

    loadCallback: function(result) {
      // TODO: use xpath
      var obj = result.documentElement.childNodes[0];
      if (obj.nodeName != "OBJECT") {
        // error ?
        this.finishLoad();
        return;
      }

      // Note:  new entry may contain content (e.g.: clone entry)
      if (this._isNewEntry(obj)) {
        this.isNew = true;
      } else {
        // notify mhub workflow that this is a saved entry.  Do not perform
        // a delete when the tab is closed.
        var title = obj.childNodes[1].firstChild.nodeValue;
        this._set_tab_title(title);
        this.isNew = false;
      }

      // get the flow model
      var graphNode = obj.getElementsByTagName('GRAPH')[0];
      var flowModelNode = "";

      if (graphNode) {
        flowModelNode = graphNode.firstChild;
      }

      // get the generator url
      var mashupurlNode = obj.childNodes[7].firstChild;
      var mashupurl = (mashupurlNode != null)?mashupurlNode.nodeValue:"";

      // get the ajax url
      var ajaxurl = obj.childNodes[8].firstChild.nodeValue;

      this.updateActions(ajaxurl);

      if (flowModelNode) {
         this.canvas.loadMashup(flowModelNode);
      }

	  if(this.isNew)
	  {
			var mashupid= obj.childNodes[0].firstChild.nodeValue;
			this.canvas.model.setValue("/flow:mashup/@id", null, mashupid);
			this.canvas.model.setValue("/flow:mashup/@url", null, mashupurl);
	  }

      this.finishLoad();
    },

    finishLoad: function() {
      // get rid of the splash screen
      if (gSplashDialog) {
      	//console.debug(gSplashDialog);
        gSplashDialog.hide();
        gSplashDialog.destroyRecursive();
        gSplashDialog = null;
      }
    },

    saveCallback: function(result) {
      var obj = result.documentElement.childNodes[0];

      // save entry metadata
      if (FLOW.plugin) {
        // clear protected since our plugin specific data has been saved.
        FLOW.plugin.tab.setProtected(false); 

        if (!this._hasBeenSaved()) {
          FLOW.plugin.done( { action: "save", text: "" } );
        } else {
          // silent save
          this.finishSave();
          //this.showSaveStatus(this.i18n.labels.SAVE_SUCC_MSG);
        }
      }

      this.hasBeenSaved = true;

      //alert( flow.xml.innerXML(result) );
    },

    saveAsCallback: function(result) {
      var obj = result.documentElement.childNodes[0];

      this.hasBeenSaved = true;

      this._set_tab_title( this.title );    // update tab title

      // show save button
      this.saveButtonNode.style.display = "inline";

      // clear protected
      if (FLOW.plugin) {
         FLOW.plugin.tab.setProtected(false); 
      }
    },

    runCallback: function(result) {
      alert( flow.xml.innerXML(result) );
    },

    callServer: function(actionUrl, postContent, callback) {
       dojo.xhrPost( {
           url: actionUrl,
           handleAs: "xml",
           content: postContent,
           contentType: "application/x-www-form-urlencoded; charset=utf-8",            
           load: function(response, ioArgs) {
              callback(response);
              return response;
           },
           error: function(response, ioArgs) {
              callback(null);
              return response;
           }
       });
    }
  }
);
