/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.FunctionInputSelection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.requireLocalization("flow", "functions");

dojo.declare(
	"flow.dijit.FunctionInputSelection",
	[dijit._Widget, dijit._Templated],
  {
    // attach points
    labelNode: null,
    selectNode: null,
    gridNode: null,
    commentLableNode: null,
    commentInputNode:null,

    // arguments
    chain: null,
    data: null,
    operator: null,
    chainParent: null,

    // impl
    grid: null,

    // means it can open new dialogs
    isChainable: true,

    // holds the generated tmp model data
    data: null,

    imgPath: dojo.moduleUrl("flow.dijit", "templates/images"),
    templatePath: dojo.moduleUrl("flow.dijit", "templates/FunctionInputSelection.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.functions = dojo.i18n.getLocalization("flow", "functions");
    },

    postCreate: function() {
      var defaultValue = null;

      if (this.data && this.data.name) {
        defaultValue = this.data.name;
      }

      var selectedFunction = null;
      var model = this.operator.model;

      var groupings = model.runXPath("//grouping", model.functionXML);
      for (var i = 0; i < groupings.length; i++) {
        var grouping = groupings[i];
        var optgroup = document.createElement("optgroup");

        var id = model.getValue(grouping, "@id");
        var label = this.getFunctionsString(id.toUpperCase()+"_GROUP_LABEL");

        optgroup.setAttribute("label", label);

        var functions = model.runXPath("function", grouping);

        for (var j = 0; j < functions.length; j++) {
          var func = functions[j];
          var id = model.getValue(func, "@id");
          var caption = this.getFunctionsString(id.toUpperCase()+"_LABEL");

          var option = document.createElement("option");
          option.setAttribute("value", id);
          option.setAttribute("title", this.getFunctionsString(id.toUpperCase()+"_TOOLTIP"));
          option.appendChild(document.createTextNode(caption));

          if (defaultValue && id == defaultValue) {
            option.setAttribute("selected", "true");
            selectedFunction = functions[j];
          }

          optgroup.appendChild(option);
        }

        this.selectNode.appendChild(optgroup);
      }

      if (this.data && selectedFunction) {
        // we have data, populate!
        this.commentInputNode.value = this.data.comments ? this.data.comments : "";
        var allowdynamic = (model.getValue(selectedFunction, "@fixedargs") != "true") ? true : false;

        // if we have data, we need to annote fixed from the available inputs
        if (selectedFunction && selectedFunction.inputs) {
          var args = model.runXPath("arg", selectedFunction);
          for (var i = 0; i < args.length; i++) {
            var arg = args[i];
            var id = model.getValue(arg, "@id");

            for (var j = 0; j < this.data.parameters.length; j++) {
              if (this.data.parameters[i].caption = id) {
                this.data.parameters[i].fixed = true;
                break;
              }
            }

          }
        }

        this.buildGrid(allowdynamic);
      } else {
        this.selectNode.selectedIndex = 0;
        this.data = flow.operator.Data.create("function");
      }
    },

    prepareLocalizationString: function(str) {
      // can't contain "-"
      str = str.replace(/-/g, "_");
      return str.toUpperCase();
    },

    getFunctionsString: function(name) {
      return this.i18n.functions[this.prepareLocalizationString(name)];
    },

    onCommentsChanged: function(e){
      this.data.comments = this.commentInputNode.value;
    },

    onFunctionSelect: function(e) {
      var value = this.selectNode.value;

      if (this.grid) {
        this.grid.destroy();
        this.grid = null;
      }

      // clear out data
      this.data.parameters = [];

      if (value == "-1") {
        // no function selected
        this.data.name = "";
        return;
      }

      var model = this.operator.model;
      this.data.name = value;

      var dynamicInputs = (model.getValue(model.functionXML, "//function[@id='"+value+"']/@fixedargs") != "true") ? true : false;

      var args = model.runXPath("//function[@id='"+value+"']/arg", model.functionXML);
      for (var i = 0; i < args.length; i++) {
        var arg = args[i];

        var argid = model.getValue(arg, "@id");

        var type = model.getValue(arg, "@type");
        // TODO: hack here!
        type = "text";

        var defaultvalue = model.getValue(arg, "@default");
        if (defaultvalue == null) {
          defaultvalue = "";
        }

        var param = flow.operator.Data.create(type, this.operator.model.generateId("param"));
        param.caption = argid;
        param.value = defaultvalue;
        param.fixed = true;

        // check if has values
        var options = model.runXPath(".//option", arg);
        if (options && options.length > 0) {
          param.values = [];
          for (var j = 0; j < options.length; j++) {
            var value = model.getValue(options[j], "@value");

            // if no default value, the value becomes the first option
            if (j == 0 && !defaultvalue) {
              param.value = value;
            }

            param.values.push({value: value, label: this.getFunctionsString(this.data.name.toUpperCase()+ "_ARGUMENTVALUE_" + value.toUpperCase())});
          }
        }

        this.data.parameters.push(param);
      }

      // build the grid - only if it has args
      if (args.length > 0) {
        this.buildGrid(dynamicInputs);
      }
    },

    buildGrid: function(allowDynamicRows) {
      dojo.require("flow.dijit.Grid");

      var grid = {id: this.id, layout: [{name: "name", label: this.i18n.propertyeditor.USE_FUNCTIONS_GRID_ARGUMENT_NAME, type: "text", width: "33%"},
                                        {name: "value", label: this.i18n.propertyeditor.USE_FUNCTIONS_GRID_ARGUMENT_VALUE, options: ["text", "xpath", "function", "variable"], type: "param",  width: "67%"}],
                  data: [], config: {}};
      grid.config.canAddRows = allowDynamicRows ? true : false;
      grid.config.canMoveRows = false;
      grid.config.allowGridEditing = true;

      if (this.data) {
        // load in parameters - and translate

        for (var i = 0; i < this.data.parameters.length; i++) {
          var param = this.data.parameters[i];
          var caption = this.getFunctionsString(this.data.name.toUpperCase()+ "_ARGUMENT_" + param.caption.toUpperCase());
          var item = {id: param.id, name: caption, fixed: param.fixed, value: {}};

          // TODO: value needs to exist to tell it to keep working.  ugly!
          item.value = param;

          grid.data.push(item);
        }
      }

      var div = document.createElement("div");
      this.gridNode.appendChild(div);
      this.grid = new flow.dijit.Grid({grid: grid, operator: this, chain: this.chain, chainParent: this}, div);
    },

    addChild: function() {
      var param = flow.operator.Data.create("text", this.operator.model.generateId("param"));
      param.name = "Custom";
      param.caption = "Custom";
      param.value = "";

      this.data.parameters.push(param);
      return param;
    },

    getChildIndexById: function(id) {
      var index = null;
      for (var i = 0; i < this.data.parameters.length; i++) {
        if (this.data.parameters[i].id == id) {
          index = i;
          break;
        }
      }

      return i;
    },

    getChildById: function(id) {
      return this.data.parameters[this.getChildIndexById(id)];
    },

    setChildValue: function(id, key, value) {
      var param = this.getChildById(id);

      if (value == null) {
        // clearing value
        param.value = "";
        param.type = "text";
      } else {
        // can either contain value/type or data/dtype
        if (value.type == "function") {
          param.value = value
        } else {
          if (value.type == "xpath" && value.inputid!=null) {
            param.inputid = value.inputid;
          }

          if (value.value != null) {
            param.value = value.value;
          } else {
            param.value = value.data;
          }
        }

        param.type = value.type;
      }
    },

    modifyChild: function(id, key, value) {
      // empty, as data will be updated during dialog acceptance - not dynamic!
    },

    removeChild: function(id) {
      // TODO: use splice?
      var index = this.getChildIndexById(id);

      this.data.parameters[index] = null;
      delete this.data.parameters[index];
    },

    getData: function() {
      return this.data;
    },

    showHelp: function() {
    	var title = dojo.string.substitute(this.i18n.propertyeditor.HELP_TITLE, ["Transform Functions"]);
    	var selectedFun = this.selectNode.value;
    	var url;
    	//console.debug(this.selectNode.value);

    	switch (selectedFun) {
        case "concat":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_concatenate.htm";
          break;
        case "contains":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_contains.htm";
          break;
        case "regexp":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_regexp.htm";
          break;
        case "trim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_trim.htm";
          break;
        case "ltrim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_lefttrim.htm";
          break;
        case "rtrim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_righttrim.htm";
          break;
        case "starts-with":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_startswith.htm";
          break;
        case "ends-with":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_endswith.htm";
          break;
        case "substring":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_substring.htm";
          break;
        case "not":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_not.htm";
          break;
        case "numeric-unary-plus":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_unaryplus.htm";
          break;
        case "numeric-unary-minus":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_unaryminus.htm";
          break;
        case "numeric-add":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_add.htm";
          break; 
        case "numeric-subtract":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_subtract.htm";
          break;
        case "numeric-multiply":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_multiply.htm";
          break;
        case "numeric-divide":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_divide.htm";
          break;
        case "max":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_maximum.htm";
          break;
        case "min":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_minimum.htm";
          break; 
        case "avg":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_average.htm";
          break;
        case "sum":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_sum.htm";
          break;
        case "count":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_count.htm";
          break;
        case "variance":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_variance.htm";
          break;
        case "current-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_currentdatetime.htm";
          break;
        case "format-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_formatdatetime.htm";
          break;
        case "day-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_dayfromdatetime.htm";
          break;
        case "month-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_monthfromdatetime.htm";
          break;
       	case "year-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_yearfromdatetime.htm";
          break;
        case "hours-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_hoursfromdatetime.htm";
          break;
        case "minutes-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_minutesfromdatetime.htm";
          break;
        case "seconds-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_secondsfromdatetime.htm";
          break;
        case "coalesce":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_coalesce.htm";
          break;
        case "urlencode":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_urlencode.htm";
          break;
        case "uuid":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_uuid.htm";
          break; 
        default:
          url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_overview.htm";
      }

    	this.operator.getCanvas().showHelp(url, title, this.chain);
    }
  }
);
