/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Grid");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.form.Button");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");

dojo.declare(
	"flow.dijit.Grid",
	[dijit._Widget, dijit._Templated, dijit._Container],
  {
    labelNode: null,
    theadNode: null,
    tbodyNode: null,
    footerNode: null,
    addButtonNode: null,
    upButtonNode: null,
    downButtonNode: null,
    removeButtonNode: null,
    contentNode: null,
    actionsNode: null,
    collapseNode: null,
    collapseNodeButton: null,

    grid: null,
    operatorId: null,
    operator: null,
               
    collapsable: false,
    collapseLabel: null,
    expandLabel: null,

    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    // parent chain
    chain: null,

    isEmpty: true,

    selectedRow: null,

    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("flow.dijit", "templates/GridTemplate.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      // handle collapsing
      if (this.collapsable) {
        // default to collapsed
        this.isCollapsed = true;
        this.collapseNodeButton.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.expandLabel);
        dojo.style(this.collapseNode, "display", "block");
      } else {
        dojo.style(this.contentNode, "display", "block");
      }

      // let's build the table - first the label
      if (this.grid.label) {
        this.labelNode.appendChild(document.createTextNode(this.grid.label));
      }

      // set button tooltips dynamically
      if (this.operator.type) {
        var type = this.operator.type.toUpperCase();

        if (this.grid.labelPrefix) {
          type = this.grid.labelPrefix;
        }

        // set add/remove row labels
        if (this.grid.config.canAddRows) {
          this.addButtonNode.setLabel(this.i18n.propertyeditor[type+"_ADD_ROW"]);
          this.removeButtonNode.setLabel(this.i18n.propertyeditor[type+"_REMOVE_ROW"]);
        }
      }

      // headers
      this.buildHeaders();

      // config the action buttons
      if (!this.grid.config.canAddRows) {
        dojo.addClass(this.addButtonNode.domNode, "dijitDisplayNone");
      }

      // config the action buttons
      if (!this.grid.config.canMoveRows) {
        this.upButtonNode.domNode.style.display = "none";
        this.downButtonNode.domNode.style.display = "none";
      }

      if (!this.grid.config.allowGridEditing) {
        this.actionsNode.style.display = "none";
      }

      // now add data rows
      if (this.grid.data) {
        if (this.grid.data.length > 0) {
          this.isEmpty = false;
        }

        for (var i = 0; i < this.grid.data.length; i++) {
          if (this.grid.data[i]) {
            var itemid = this.grid.data[i].id;
            var item = this.grid.data[i];

            this.addRow(itemid, item, i);
          }
        }
      }

      // if no rows, add some helpfull text
      if (this.isEmpty) {
        if (this.grid.config.canAddRows) {
          // if can add rows, add one for the user
          this.addNewRow();
        } else {
          this.addEmptyRow();
        }
      }
    },

    buildHeaders: function() {
      while (this.theadNode.hasChildNodes()) {
        this.theadNode.removeChild(this.theadNode.firstChild);
      }

      var tr = document.createElement("tr");

      for (var i = 0; i < this.grid.layout.length; i++) {
        var layout = this.grid.layout[i];

        if (layout.hidden) {
          continue;
        }

        var th = document.createElement("th");
        dojo.addClass(th, "flowgridcell");
        dojo.addClass(th, "flowgridheadercell");
        th.setAttribute("noWrap", "true");

        if (layout.width) {
          th.setAttribute("width", layout.width);
        }

        if (this.operator.type && this.operator.type.toUpperCase() == "FOREACH") {
          var src1id = this.operator.model.getValue(this.operator.node, "flow:input[@position='0']/@id");
          var src2id = this.operator.model.getValue(this.operator.node, "flow:input[@position='1']/@id");

          var src1cap = this.operator.model.getValue(this.operator.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
          var src2cap = this.operator.model.getValue(this.operator.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

          var i1 = layout.label.indexOf(src1cap) + layout.label.indexOf(src2cap)+ 1;
          var i2 = layout.label.indexOf(" ", i1);

          var font = document.createElement("font");
          font.style.color = "blue";

          th.appendChild(document.createTextNode(layout.label.substring(0,i1)));

          var forEachText;
          if (i2 == -1) {
            forEachText = document.createTextNode(layout.label.substring(i1));
            font.appendChild(forEachText);
            th.appendChild(font);
          } else {
            forEachText = document.createTextNode(layout.label.substring(i1,i2));
            font.appendChild(forEachText);
            th.appendChild(font);
            th.appendChild(document.createTextNode(layout.label.substring(i2)));
          }
        } else {
          th.appendChild(document.createTextNode(layout.label));
        }

        tr.appendChild(th);
      }

      this.theadNode.appendChild(tr);
    },

    addEmptyRow: function() {
      var tr = document.createElement("tr");
      var td = document.createElement("td");
      dojo.addClass(td, "flowgridcell");
      td.setAttribute("align", "center");
      td.appendChild(document.createTextNode(this.i18n.propertyeditor.GRID_NO_DATA_ENTERED));

      // only add link if one can add rows
      if (this.grid.config.canAddRows) {
        td.appendChild(document.createTextNode(" "));
        var a = document.createElement("a");
        a.setAttribute("href", "javascript:void()");
        var myThis = this;
        a.onclick = function() {
          myThis.addRowClick();
          return false;
        }
        a.appendChild(document.createTextNode(this.i18n.propertyeditor.GRID_CLICK_ADD_ROW));
        td.appendChild(a);
      }
      tr.appendChild(td);
      this.tbodyNode.appendChild(tr);

      td.setAttribute("colSpan", this.grid.layout.length);
    },

    addRow: function(itemid, item) {
      var tr = document.createElement("tr");

      tr.setAttribute("rowid", itemid);
      if (item.fixed) {
        tr.setAttribute("fixed", "true");
      }

      var grid = this.grid;
      var gridlayout = grid.layout;

      for (var j = 0; j < gridlayout.length; j++) {
        var info = gridlayout[j];
        var key = info.name;

        var td = document.createElement("td");
        td.setAttribute("valign", "top");
        dojo.addClass(td, "flowgridcell");

        var itemvalue;

        if (info.passAllData) {
          itemvalue = item;
        } else {
          itemvalue = item[key];
        }

        var itemdom;

        if (info.hidden) {
          continue;
        }

        var myThis = this;
        function generateCallback() {
          var storedkey = key;

          return function(value) {
            myThis.operator.setChildValue(itemid, storedkey, value);
          }
        }

        if (info.editable || (info.type && info.type == "textarea")) {
          if (info.editable) {
            itemdom = document.createElement("input");
            itemdom.setAttribute("value", itemvalue);
            itemdom.style.width = "90%";
            td.setAttribute("align", "center");
          } else {
            itemdom = document.createElement("textarea");
            itemdom.appendChild(document.createTextNode(itemvalue));
          }

          if (info.tooltip) {
            itemdom.setAttribute("title", info.tooltip);
          }

          // update the model
          itemdom.onblur = function() {
            var callback = generateCallback();
            var input = itemdom;
            return function() {
              callback(input.value);
            };
          }();
        } else if (info.values || itemvalue.values) {
          var values = info.values ? info.values : itemvalue.values;

          // we need a select
          itemvalue = typeof(itemvalue)=="object" ? itemvalue.value : itemvalue;
          itemdom = document.createElement("select");
          for (var k = 0; k < values.length; k++) {
            var label, value;

            if (typeof(values[k]) == "object") {
              label = values[k].label;
              value = values[k].value;
            } else {
              label = value = values[k];
            }

            var option = document.createElement("option");
            option.setAttribute("value", value);
            option.appendChild(document.createTextNode(label));

            // select the right option
            if (value == itemvalue) {
              option.setAttribute("selected", "true")
            }
            itemdom.appendChild(option);
          }

          // update the model
          itemdom.onchange = function() {
            var callback = generateCallback();
            var select = itemdom;
            return function() {
              callback({value: select.value, type: "text"});
            };
          }();
        } else if (info.type && info.type == "text") {
          itemdom = document.createElement("div");
          itemdom.appendChild(document.createTextNode(itemvalue));
        } else if (info.type && info.type == "tree") {
          // we need a tree dropdown
          itemdom = document.createElement("div");

          dojo.require("flow.dijit.XMLViewTextBox");

          var treediv = document.createElement("div");
          itemdom.appendChild(treediv);

          var templ = myThis.operator.getTree(info.pos);

          if (templ == null) {
            var src1id = myThis.operator.model.getValue(myThis.operator.node, "flow:input[@position='"+info.pos+"']/@id");
            var src1cap = myThis.operator.model.getValue(myThis.operator.model.flowNode, ".//flow:*[@id='"+src1id+"']/@caption");
            alert(dojo.string.substitute(myThis.operator.i18n.LABEL_INPUT_NOT_VAlID, [src1cap]));
            return;
          }

          myThis.operator.model.setValue("flow:input[@position='"+info.pos+"']/@timestamp",myThis.operator.node,templ.timestamp);
          var treewidget = new flow.dijit.XMLViewTextBox({xmlRootNode: templ.root, operator: myThis.operator, inputid: templ.id, path: templ.path, data: itemvalue, callback: generateCallback()}, treediv);
        } else if (info.type && info.type == "param") {
          dojo.require("flow.dijit.FlowComboBox");

          itemdom = document.createElement("div");
          var combodiv = document.createElement("select");
          itemdom.appendChild(combodiv);
          var widget = new flow.dijit.FlowComboBox({data: itemvalue, operator: myThis.operator, options: info.options, callback: generateCallback(), chainParent: this.operator, chain: this.chain+1}, combodiv);
        } else if (info.type && info.type == "custom") {
          itemdom = document.createElement("div");
          info.callback(itemdom, item);
        } else {
          itemdom = document.createTextNode(itemvalue);
        }

        td.appendChild(itemdom);
        tr.appendChild(td);
      }

      this.tbodyNode.appendChild(tr);

      if (this.grid.config.maxRows && this.tbodyNode.rows.length == this.grid.config.maxRows) {
        this.addButtonNode.setDisabled(true);
      }
    },

    selectRow: function(event) {
      if (this.isEmpty || !this.grid.config.allowGridEditing) {
        return;
      }

      var target = event.target;

      // make sure we only select tr's with a rowid
      while (target.parentNode && (target.tagName.toLowerCase() != "tr" || typeof(target.getAttribute("rowid")) != "string")) {
        target = target.parentNode;
      }

      // not in a row?
      if (target.tagName.toLowerCase() != "tr") {
        return;
      }

      if (this.selectedRow == target) {
        // already selected
        return;
      }

      if (this.selectedRow) {
        this.selectedRow.className = "";
      }

      target.className = "selectedrow";
      this.selectedRow = target;

      this.updateActions();
    },

    updateActions: function() {
      // update the actions based on the selection
      if (!this.grid.config.allowGridEditing) {
        return;
      }

      if (!this.selectedRow) {
        // nothing selected
        this.removeButtonNode.setDisabled(true);
        this.upButtonNode.setDisabled(true);
        this.downButtonNode.setDisabled(true);
      } else {
        var isFixed = false;
        if (this.selectedRow.getAttribute("fixed") == "true") {
          isFixed = true;
        }

        // we have something selected, so we can remove it
        if (!isFixed) {
          this.removeButtonNode.setDisabled(false);
        } else {
          this.removeButtonNode.setDisabled(true);
        }

        var length = this.tbodyNode.rows.length;

        // handle up button
        if (this.tbodyNode.rows[0] == this.selectedRow) {
          // if the currently selected row is the first row, disable
          this.upButtonNode.setDisabled(true);
        } else if (length == 1 || isFixed) {
          // only one row, can't move up
          this.upButtonNode.setDisabled(true);
        } else {
          this.upButtonNode.setDisabled(false);
        }

        // handle down button
        if (this.tbodyNode.rows[this.tbodyNode.rows.length-1] == this.selectedRow) {
          // if the currently selected row is the last row, disable
          this.downButtonNode.setDisabled(true);
        } else if (length == 1 || isFixed) {
          // only one row, can't move down
          this.downButtonNode.setDisabled(true);
        } else {
          this.downButtonNode.setDisabled(false);
        }
      }
    },

    addRowClick: function(event) {
      this.addNewRow();
    },

    addNewRow: function() {
      if (this.isEmpty) {
        this.isEmpty = false;

        if (this.tbodyNode.firstChild) {
          this.tbodyNode.removeChild(this.tbodyNode.firstChild);
        }
      }

      // tell model we added a row
      var newitem = this.operator.addChild(this.grid.id);
      this.grid.data.push(newitem);
      this.addRow(newitem.id, newitem);

      // finally, update the actions
      this.updateActions();
    },

    moveRowUpClick: function(event) {
      // probably useless check to make sure we are not the first row
      if (this.tbodyNode.rows[0] == this.selectedRow) {
        return;
      }

      // tell model we are changing the position!
      var pos = this.getPositionForRow(this.selectedRow);

      // lets swap out the rows - move them in the DOM
      var target = this.selectedRow.previousSibling;
      var source = this.tbodyNode.removeChild(this.selectedRow);

      var sourceid = source.getAttribute("rowid");
      var targetid = target.getAttribute("rowid");
      this.operator.modifyChild(sourceid, "position", pos-1);
      this.operator.modifyChild(targetid, "position", pos);

     this.selectedRow = this.tbodyNode.insertBefore(source, target);

      // finally, update the actions
      this.updateActions();
    },

    moveRowDownClick: function(event) {
      // probably useless check to make sure we are not the last row
      if (this.tbodyNode.rows[this.tbodyNode.rows.length-1] == this.selectedRow) {
        return;
      }

      // tell model we are changing the position!
      var pos = this.getPositionForRow(this.selectedRow);

      // lets swap out the rows - move them in the DOM
      var target = this.selectedRow.nextSibling;
      var source = this.tbodyNode.removeChild(this.selectedRow);

      var sourceid = source.getAttribute("rowid");
      var targetid = target.getAttribute("rowid");
      this.operator.modifyChild(sourceid, "position", pos+1);
      this.operator.modifyChild(targetid, "position", pos);

      if (target.nextSibling) {
        this.selectedRow = this.tbodyNode.insertBefore(source, target.nextSibling);
      } else {
        this.selectedRow = this.tbodyNode.appendChild(source);
      }

      // finally, update the actions
      this.updateActions();
    },

    clearGrid: function(onlygrid) {
      for (var i = this.tbodyNode.rows.length-1; i >= 0; i--) {
        this.tbodyNode.removeChild(this.tbodyNode.rows[i]);
      }
    },

    removeRowClick: function(event) {
      var id = this.selectedRow.getAttribute("rowid");
      var pos = this.getPositionForRow(this.selectedRow);

      this.operator.removeChild(id);

      // update all next sibling's position
      var elm = this.selectedRow;
      var count = 0;
      while (elm.nextSibling) {
        elm = elm.nextSibling;
        this.operator.modifyChild(elm.getAttribute("rowid"), "position", pos+count);
        count++;
      }

      this.tbodyNode.removeChild(this.selectedRow);
      this.selectedRow = null;

      // finally, update the actions
      this.updateActions();

      // if no rows, add some helpfull text
      if (this.tbodyNode.childNodes.length == 0) {
        this.isEmpty = true;
        this.addEmptyRow();
      }

      if (this.grid.config.maxRows && this.tbodyNode.rows.length < this.grid.config.maxRows) {
        this.addButtonNode.setDisabled(false);
      }
    },

    getPositionForRow: function(row) {
      // tell model we are changing the position!
      var pos = 0;
      for (var i = 0; i < this.tbodyNode.rows.length; i++) {
        if (this.tbodyNode.rows[i] != row) {
          pos++;
        } else {
          break;
        }
      }

      return pos;
    },

    toggleCollapse: function() {
      if (this.isCollapsed) {
        this.collapseNodeButton.innerHTML = this.collapseLabel;
        dojo.style(this.contentNode, "display", "block");
      } else {
        this.collapseNodeButton.innerHTML = this.expandLabel;
        dojo.style(this.contentNode, "display", "none");
      }

      this.isCollapsed = !this.isCollapsed;
    }
  }
);
