/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Properties");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "labels");

dojo.declare(
  "flow.dijit.Properties",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    selectNode: null,
    helpNode: null,

    rssDetailsNode: null,
    atomDetailsNode: null,
    xmlDetailsNode: null,

    rssDetailsForm: null,
    atomDetailsForm: null,
    xmlDetailsForm: null,

    // impl
    mode: null,
    model: null,

    templatePath: dojo.moduleUrl("flow.dijit", "templates/Properties.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      //var img = document.createElement("img");
      //img.setAttribute("src", dojo.moduleUrl("flow.dijit", "templates/images/dialoghelp.png"));
      //img.setAttribute("title", this.i18n.labels.TOOLTIP_HELP);
      //this.helpNode.appendChild(img);

      // load in model detaults
      var model = this.model;

      var type = model.getPublishType();
      this.selectNode.value = type.toLowerCase();
      this.updateMode();

      var form = this[this.mode+"DetailsForm"];
      for (var i = 0; i < form.elements.length; i++) {
        var name = form.elements[i].name;
        var val = model.getPublishParam(name);
        form.elements[i].value = val ? val : "";
      }

      this.validateInputs();

      dojo.connect(this.domNode, "onkeyup", this, "onKeyUp");
    },

    getMode: function() {
      return this.selectNode.value;
    },

    updateMode: function() {
      var mode = this.getMode();

      if (mode == this.mode) {
        return;
      }

      if (this.mode) {
        this[this.mode+"DetailsNode"].style.display = "none";
      }

      this[mode+"DetailsNode"].style.display = "block";
      this.mode = mode;

      this.validateInputs();
    },

    onChange: function(event) {
      this.updateMode();
      this.updateModel();
    },

    validateInputs: function() {
      var isValid = true;
      var mode = this.getMode().toUpperCase();

      var requirements = this.model.getRequiredPublishParams(mode);

      var form = this[this.mode+"DetailsForm"];
      for (var i = 0; i < requirements.length; i++) {
        var formelement;
        for (var j = 0; j < form.elements.length; j++) {
          if (form.elements[j].name == requirements[i]) {
            formelement = form.elements[j];
            break;
          }
        }

        if (!dojo.trim(formelement.value)) {
        	if((mode=="ATOM" ||mode=="RSS") && (requirements[i]=="id" || requirements[i]=="link"))
        	{
		        formelement.value =this.model.getValue(null,"/flow:mashup/@url"); 
				formelement.parentNode.parentNode.cells[0].style.color = "";
        	}
        	else
        	{
		          // mark as invalid
				formelement.parentNode.parentNode.cells[0].style.color = "red";
				isValid = false;
          	}
        } else {
          formelement.parentNode.parentNode.cells[0].style.color = "";
        }
      }

      if (this.onValidate) {
        this.onValidate(isValid);
      }
    },

    onKeyUp: function(e) {
      var target = e.target;
      if (target.tagName == "INPUT" || target.tagName == "TEXTAREA") {
        var mode = this.getMode().toUpperCase();
        var requirements = this.model.getRequiredPublishParams(mode);

        if (dojo.indexOf(requirements, target.name) >= 0) {
          if (!dojo.trim(target.value)) {
            // mark as invalid
            target.parentNode.parentNode.cells[0].style.color = "red";
          } else {
            target.parentNode.parentNode.cells[0].style.color = "";
          }
        }

        if (this.onValidate) {
          this.validateInputs()
        }

        // update the model
        var node = this.model.getNode(".//flow:param[@caption = '"+target.name+"']", this.model.publishNode);
        node.firstChild.nodeValue =  target.value;
      }
    },

    updateModel: function() {
      var mode = this.getMode();

      this.model.setPublishInput(mode.toUpperCase());
      var publishnode = this.model.getNode("flow:publish", this.model.flowNode);
      this.model.removeNodes(".//flow:param[@position !='0']", publishnode);

      //RSS- Title, Link, Description
      //Atom Title, ID, Link, Author, Rights, Link
      var form = this[mode+"DetailsForm"];
      // copy over elements from form into our data for the server to handle
      // as it sees fit
      for (var i = 0; i < form.elements.length; i++) {
        //data[form.elements[i].name] = form.elements[i].value;
        var value = form.elements[i].value;

        var template='<param xmlns="'+this.model.flowNS+'" position="'+(i+1)+'"  caption="'+form.elements[i].name+'" operatorspecific="" inputid="" id="'+form.elements[i].name+'" type="text" comments=""><![CDATA['+value+']]></param>';
        publishnode.appendChild(this.model.parseXML(template));
      }
    }
  }
);
