/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.PropertyEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.layout.TabContainer");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");
dojo.requireLocalization("flow", "labels");

dojo.require("flow.dijit.Grid");

dojo.declare(
  "flow.dijit.PropertyEditor",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    // arguments
    operator: null,
    canvas: null,

    // attach points
    containerNode: null,
    tabcontainerNode: null,
    previewContainerNode: null,
    advancedTabNode: null,
    commentsNode: null,
    previewTabNode: null,
    captionNode: null,

    // localization
    i18n: null,

    widgetsInTemplate: true,

    // impl
    operatorType: null,
    operatorId: null,
    operatorData: null,
    topSectionNode: null,
    refreshErrorMessageNode: null,
    messageNode: null,
    captioninput: null,
    previewTreeWidget: null,

    hasError: false,

    templatePath: dojo.moduleUrl("flow.dijit", "templates/PropertyEditor.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      var tab = this.tabcontainerNode;
      tab.startup();
      tab.resize();

      // add help icon
      var tablist = this.tabcontainerNode.tablist;
      var div = document.createElement("div");
      div.className = "flowDialogHelp"
      var img = document.createElement("img");
      img.setAttribute("src", dojo.moduleUrl("flow.dijit", "templates/images/dialoghelp.png"));
      img.setAttribute("title", this.i18n.propertyeditor.TOOLTIP_HELP);

      div.appendChild(img);
      dojo.connect(img, "onclick", this, "showHelp");

      tablist.domNode.appendChild(div);

      this.operatorType = this.operator.type;
      this.operatorId = this.operator.id;

      if (this.operator.canRename) {
        this.captioninput.value = this.operator.getName();
      } else {
        this.captionNode.style.display = "none";
      }

      var foo = function(widget){
        if (widget == this.previewTabNode) {this.refreshPreviewTree()};
      }
      dojo.connect(tablist, "onSelectChild", this, foo);

      // comments
      this.setComments(this.operator.getComment());

      // advanced tab
      if (!this.operator.hasAdvancedOptions) {
        this.tabcontainerNode.closeChild(this.advancedTabNode);
      }

      this.resizeComments();

      // operator data
      this.operatorData = this.operator.setupPropertyEditor();

      // check if requirements are met
      var requirements = this.operator.meetsRequirements();
      if (!requirements.pass) {
        this.containerNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.operator.createErrorMarkup(requirements.error));
        this.hasError = true;
        return;
      }

      var result = this.operator.getDirtyInputs(this.operator.id);
      if (result.length > 0) {
        this.messageNode.innerHTML = this.i18n.propertyeditor.LABEL_INPUT_DATA_DIRTY;
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.LABEL_INPUT_DATA_REFRESH));
        this.messageNode.appendChild(button);
        dojo.connect(button, "onclick", this, "refreshInputs");
        this.messageNode.style.display = "block";
        dojo.addClass(this.messageNode, "warningMsg");
      }

      this.hasError = false;

      if (!this.operatorData) {
        // no UI
        return;
      }

      var layout = this.operatorData.layout;

      for (var i = 0; i < layout.length; i++) {
        var item = layout[i];
        if (item.type == "grid") {
          this.generateGrid(i, item);
        } else if (item.type == "url") {
          var div = document.createElement("div");
          this.containerNode.appendChild(div);
          var x = new flow.dijit.PropertyEditor.URL({item: item, operatorId: this.operator.id, propEditor: this}, div);
        } else if (item.type == "inputselect") {
          var select = document.createElement("select");
          select.setAttribute("id", this.operatorId + "+_inputselect");

          for (var j = 0; j < item.data.length; j++) {
            var option = document.createElement("option");
            option.setAttribute("value", item.data[j]);
            option.appendChild(document.createTextNode(item.captions[j]));
            select.appendChild(option);
          }

          var myThis = this;
          select.onchange = function() {
            var index = (select.selectedIndex == 0) ? 1 : 0;
            if (myThis.operator.model.runXPath("flow:condition//flow:param[text()!='']", myThis.operator.node).length>0) {
              if (myThis.operator.model.getValue(myThis.operator.node, "flow:input[ @id='"+select.options[select.selectedIndex].value+"']/@position")!="0") {
                alert(myThis.operator.i18n.PROPERTY_TAB_INPUTCHANGE_ALERT);
                select.selectedIndex = index;
                return false;
              }
            }

            document.getElementById(myThis.operatorId + "+_inputselect_secondinput").setAttribute("value", select.options[index].text);

            myThis.operator.model.setValue("flow:input[ @id='"+select.options[select.selectedIndex].value+"']/@position", myThis.operator.node, 0);
            myThis.operator.model.setValue("flow:input[ @id='"+select.options[index].value+"']/@position", myThis.operator.node, 1);

            try {
              if (myThis.operator.firstInputSelected) {
                myThis.operator.firstInputSelected();
              }
            } catch(ex) {}
          }

          this.containerNode.appendChild(document.createTextNode(item.labels[0]));
          this.containerNode.appendChild(select);
          this.containerNode.appendChild(document.createTextNode(" "+item.labels[1]));

          // 2nd input
          var input = document.createElement("input");
          input.setAttribute("id", this.operatorId + "+_inputselect_secondinput");
          input.setAttribute("disabled", "true");
          input.setAttribute("value", item.captions[1]);
          this.containerNode.appendChild(input);

          // add new line
          this.containerNode.appendChild(document.createElement("br"));
        } else if (item.type == "custom") {
          this.operator.buildCustomValue(this.containerNode, item.id);
        }
      }

      // other tabs
      dojo.connect(this.advancedTabNode, "onShow", this, "loadAdvancedTab");
      dojo.connect(this.advancedTabNode, "onHide", this, "hideAdvancedTab");

    },

    loadAdvancedTab: function() {
      // always render when we show it - make sure to clear out first
      this.advancedTabNode.destroyDescendants();
      this.operator.renderAdvancedOptions(this.advancedTabNode.domNode)
      this.operator.isAdvancedOptionsVisible = true;
    },

    hideAdvancedTab: function() {
      this.operator.isAdvancedOptionsVisible = false;
    },

    onCaptionChanged: function() {
	  var val=dijit.byId("flowcanvas").model.stripSctipt(this.captioninput.value);
      this.operator.model.setValue("@caption", this.operator.node, val);
      this.operator.model.canvas.onOperatorCaptionChanged(this.operator, val);
    },

    doPreview: function() {
      this.operator.previewLoadComplete();
      this.operator.doPreview(this.refreshErrorMessageNode);
    },

    refreshPreviewTree: function() {
      this.refreshErrorMessageNode.innerHTML = "";
      this.refreshErrorMessageNode.style.display = "none";

      // remove the tree
      if (this.previewTreeWidget) {
        this.previewTreeWidget.destroyRecursive();
        this.previewTreeWidget = null;
      }

      if (this.operator.model.getValue(this.operator.node, "@dirty") == 1 &&
      	this.operator.loadingPreviewResult != true ) {

        // show loading message
        this.previewContainerNode.innerHTML = "<img align='top' src='"+FLOW.baseUrl+"/images/progress17x17.gif'> "+this.i18n.propertyeditor.PREVIEW_LOADING;

        this.operator.doPreview(this.refreshErrorMessageNode);
      }

      if (this.operator.loadingPreviewResult) {
        this.previewContainerNode.innerHTML = "";
        this.refreshErrorMessageNode.style.display = "block";
        this.refreshErrorMessageNode.innerHTML = this.operator.i18n.SOURCE_ADVANCED_ERROR_WAITING;
      }

      var node = this.operator.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.operator.id+"']/*");
      if (node) {
        dojo.require("flow.dijit.XMLViewTree");
        this.previewContainerNode.innerHTML = "";

        var div = document.createElement("div");
        this.previewContainerNode.appendChild(div);
        this.previewTreeWidget = new flow.dijit.XMLViewTree({xmlRootNode: node, operatorId: this.operator.id, operator: this.operator}, div);
      }
    },

    generateGrid: function(index, layout) {
      var gridContainer = document.createElement("div");
      this.containerNode.appendChild(gridContainer);

      var grid = this.operatorData.layout[index];

      dojo.require("flow.dijit.Grid");

      var widget = new flow.dijit.Grid({grid: grid, operatorId: this.operatorId,
                                        operator: this.operator, chain: 0,
                                        collapsable: layout.collapsable,
                                        collapseLabel: layout.collapseLabel,
                                        expandLabel: layout.expandLabel
                                       }, gridContainer);
      this.operator.grid = widget;

      if (layout.hidden) {
        widget.domNode.style.display = "none";
      }

    },

    checkForError: function() {
      var count = this.operator.model.getInputCount(this.operatorId);

      return (count < this.operator.minimumInputs);
    },

    saveComments: function() {
      var value = this.commentsNode.value;
      this.operator.saveComment(encodeURI(value));
    },

    setComments: function(value) {
      if (value) {
        this.commentsNode.value = decodeURI(value);
      }
    },

    showHelp: function() {
      this.canvas.showOperatorHelp(this.operator);
    },

    refreshInputs: function() {
      function callback() {
        this.messageNode.style.display = "none";
      }

      this.canvas.refreshDirtyInputs(this.operator.id, dojo.hitch(this, callback));
    },

    resize: function(args) {
      // resize tab container
      this.tabcontainerNode.resize(args);

      this.resizeComments();
    },

    resizeComments: function() {
      // IE doesn't support 100% height on textarea
      if (dojo.isIE) {
        var box = dojo.contentBox(this.tabcontainerNode.containerNode);
        this.commentsNode.style.height = (box.h - 20) + "px";
      }
    }
  }
);

dojo.declare(
  "flow.dijit.PropertyEditor.URL",
  [dijit._Widget, dijit._Templated],
  {
    // attach points
    urlRadioNode: null,
    urlNode: null,
    catalogRadioNode: null,
    catalogNode: null,
    urlLoadButtonNode: null,
    catalogBrowseButtonNode: null,
    messageNode: null,
    tableNode: null,

    item: null,
    operatorId: null,
    propEditor: null,

    templatePath: dojo.moduleUrl("flow.dijit", "templates/PropertyEditorUrl.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      if (this.item.data) {
        if (this.item.data.value) {
          // show loaded msg when data is present to make clear the data is there
          this.messageNode.innerHTML = this.i18n.propertyeditor.SOURCE_LOAD_FINISH;
        }

        if (this.item.data.type == "catalog") {
          var name = this.item.data.name;
          if (!name) {
            name = this.item.data.value
          }
          this.catalogNode.setAttribute("value", name);

          this.catalogRadioNode.setAttribute("checked", "true");
        } else {
          this.resize();
          if (this.item.data.value) {
            this.urlNode.value = this.item.data.value;
          } else {
            // default url
            var myThis = this;
            //window.setTimeout(function(){myThis.urlNode.setAttribute("value", "http://www-01.ibm.com/software/swnews/swnews.nsf/swnewsrss?openview&RestrictToCategory=data");}, 300);
            this.urlNode.value = "http://www-01.ibm.com/software/swnews/swnews.nsf/swnewsrss?openview&RestrictToCategory=data";
          }
        }
      }

      this.onRadioChange(null, true);
      dojo.addClass(this.domNode, "flowResizable");
    },

    onRadioChange: function(e, skipUpdate) {
      var isUrlMode = this.urlRadioNode.checked;

      function _setState(elm, enabled) {
        if (enabled) {
          elm.removeAttribute("disabled");
        } else {
          elm.setAttribute("disabled", true);
        }
      }

      _setState(this.urlNode, isUrlMode ? true : false);
      _setState(this.urlLoadButtonNode, isUrlMode ? true : false);
      _setState(this.catalogBrowseButtonNode, isUrlMode ? false : true);

      if (!skipUpdate) {
        // tell callback the radio changed
        if (isUrlMode) {
          this.item.callback(null);
        } else {
          this.item.callback(null, true);
        }
      }
    },

    onKeyPress: function(e) {
      if (!this.item.callback) {
        return;
      }

      if (e.target == this.urlNode) {
        if (e.which == 13) {
          this.item.callback(this.urlNode.value);
        }
      }
    },

    onBlur: function(e) {
      this.loadButtonClick();
    },

    loadButtonClick: function(e) {
      if (!this.item.callback) {
        return;
      }

      if (this.urlRadioNode.checked) {
        this.callCallback(this.urlNode.value);
      }
    },

    callCallback: function(url, catalogid, name) {
      this.messageNode.innerHTML = "<img align='top' src='"+FLOW.baseUrl+"/images/progress17x17.gif'> " + this.i18n.propertyeditor.SOURCE_LOADING;

      this.topics = {};
      this.topics["load"] = dojo.subscribe("flow:"+this.operatorId+":loadURL", this, this.processLoadTopic);

      this.item.callback(url, catalogid, name);
    },

    processLoadTopic: function(data) {
      if (data.success) {
         this.messageNode.innerHTML = this.i18n.propertyeditor.SOURCE_LOAD_FINISH;
      } else if (data.error) {
        if (data.message) {
          this.messageNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt("<div style='color: red'>"+data.message+"</div>");
        } else {
          this.messageNode.innerHTML = "<div style='color: red'>"+this.i18n.propertyeditor.SOURCE_LOAD_ERROR+"</div>";
        }
      }
    },

    loadCatalogClick: function(e) {
      var div = document.createElement("div");
      dojo.body().appendChild(div);

      dojo.require("flow.dijit.Dialog");

      this.dialog = new flow.dijit.Dialog({title: this.i18n.propertyeditor.CATALOG_TITLE, chain: 2, showUnderlay: true, buttons:[{label: this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)}, {label: this.i18n.labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);

      // show and resize before we add contents
      this.dialog.resize({w: 400, h: 430});
      this.dialog.show();

      div = document.createElement("div");
      this.dialog.containerNode.appendChild(div);

      dojo.require("flow.dijit.Catalog");
      this.catalog = new flow.dijit.Catalog({}, div);
      this.dialog.show();
    },

    resize: function() {
      // IE table fix - the url input field will grow to the size of its value
      // due to some bizarre IE bug.  This "fixes" it by setting width on the
      // input field.
      if (!dojo.isIE) {
        return;
      }

      var width = this.propEditor.captionNode.offsetWidth;
      width -= this.tableNode.rows[1].cells[0].offsetWidth;
      width -= this.tableNode.rows[1].cells[2].offsetWidth;
      dojo.style(this.urlNode, "width", width-58);
    },

    _acceptDialog: function() {
      var id = this.catalog.getSelectedId();
      var name = this.catalog.getSelectedName();
      var link = this.catalog.getSelectedUrl();

      if (id && link) {
        this.callCallback(link, id, name);
        this.catalogNode.value = name;
      }

      this._cancelDialog();
    },

    _cancelDialog: function() {
      this.catalog.destroyRecursive();
      this.catalog = null;

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    }
  }
);

