/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.SourceAdvanced");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("flow.dijit.XMLViewTextBox");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");

dojo.declare(
  "flow.dijit.SourceAdvanced",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    operator: null,

    // attach points
    typeNode: null,
    inputNode: null,
    durationNode: null,

    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("flow.dijit", "templates/SourceAdvanced.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    buildRendering: function() {
      this.buildingRendering = true;

      this.inherited("buildRendering", arguments);
      var url = this.operator.getUrlValue();

      // check if operator has a url
      if (url) {
        var settype = this.operator.model.getValue(this.operator.node,"@type");
        var type = this.operator.model.getValue(this.operator.node,"@originaltype");
        var option = document.createElement("option");

        if (type == "RSS" || type == "ATOM") {
          option.setAttribute("value", (type == "RSS") ? "RSS" : "ATOM");
          option.appendChild(document.createTextNode((type == "RSS") ? "RSS" : "ATOM"));
          this.typeNode.appendChild(option);
        }

        option = document.createElement("option");
        option.setAttribute("value", "XML");
        option.appendChild(document.createTextNode("XML"));

        if (settype == "XML") {
          option.setAttribute("selected","true");
        }

        this.typeNode.appendChild(option);
        this.typeNode.removeAttribute("disabled");
        this.inputNode.setDisabled(false);
      } else {
        this.typeNode.setAttribute("disabled", true);
        this.inputNode.setDisabled(true);
      }

      this.durationNode.value = this.operator.model.getValue(this.operator.node,".//flow:advanced/@interval");
      this.inputNode.callback = dojo.hitch(this, this.onTreeSelect);

      var tree = this.operator.getTree(0,true);

      if (tree && tree.root) {
        this.inputNode.textbox.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
	  	  this.inputNode.xmlRootNode = tree.root;
        this.inputNode.path = tree.path;
        this.inputNode.operator = this.operator;
        if (settype == "XML") this.inputNode.setValue(this.operator.model.getValue(this.operator.node, ".//flow:advanced/@repeatelement"));
      } else {
        // no tree data found
        this.inputNode.setDisabled(true);
      }

      this.inputNode.callback = dojo.hitch(this, this.onTreeSelect);
      this.inputNode.removeDuplicates = true;

      if (url && tree && tree.root) {
        this.onModeChanged();
      }

      this.durationNode.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@interval");
/*
      if (this.operator.getUrlType() == "external") {
        // load interval from model
        this.durationNode.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@interval");
      } else {
        // catalog feeds have their own duration setting
        this.durationNode.setAttribute("disabled", true);
      }
*/

      this.buildingRendering = false;
    },

    onModeChanged: function() {
      var mode = this.typeNode.value;

      var settype = this.operator.model.getValue(this.operator.node,"@type");

      this.operator.model.setValue("@type", this.operator.node, mode);

      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      if (mode == "XML") {
        dojo.removeClass(this.inputNode.downArrowNode, "dijitDisplayNone");

        if (settype != "XML") {
          value =this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
          this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
          this.inputNode.textbox.value=value;
        }

        var tree = this.operator.getTree();
        if (tree && tree.root) {
          this.inputNode.setDisabled(false);
        } else {
          this.inputNode.setDisabled(true);
        }
      } else if (mode == "ATOM") {
        dojo.addClass(this.inputNode.downArrowNode, "dijitDisplayNone");
        var value=this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
        this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
        //this.inputNode.setValue("//atom:entry");
        this.inputNode.textbox.value = value;
        this.inputNode.setDisabled(true);
      } else {
        dojo.addClass(this.inputNode.downArrowNode, "dijitDisplayNone");
        var value=this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
        this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
        this.inputNode.textbox.value = value;
        this.inputNode.setDisabled(true);
      }
    },

    onIntervalChange: function(e) {
      var value = e.target.value;

      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      this.operator.model.setValue(".//flow:advanced/@interval", this.operator.node, value);
    },

    onTreeSelect: function(data) {
      var value = data.value;

      if (value.substring(0,2)=="./") {
        value = value.substring(2);
      }

      var org = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement")
      org = (org.indexOf("/",1)>0) ? org.substring(0,org.indexOf("/",1)):org;

      if (value == "" || value == "/*") {
      	value = org;
      } else {
      	if(value.substring(0,1) != "/") {
          value = org + "/"+value;
        }
      }

      this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value)
      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      this.inputNode.textbox.value = value;
    }
  }
);

