/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Transform");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.SplitContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require("flow.dijit.XMLEditTree");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");
dojo.requireLocalization("flow", "labels");

dojo.declare(
  "flow.dijit.Transform",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    "class": "flowResizable",

    // args
    operator: null,

    // attach points
    layoutNode: null,
    inputNode: null,
    outputNode: null,

    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("flow.dijit", "templates/Transform.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    buildRendering: function() {
      this.inherited("buildRendering", arguments);

      var xmlDoc = this.operator.getTree(0);
      if (xmlDoc == null) {
        return;
      }

      if (xmlDoc) {
        this.operator.model.setValue("flow:input[@position='0']/@timestamp", this.operator.node, xmlDoc.timestamp);
      }
      var idArray = new Array('copy_to_output');
      var captionArr = new Array(this.i18n.TRANSFORM_MENU_DRAGTOOUTPUT);

      var menuinfo = {callback: dojo.hitch(this, this.menuCallback),
                      items: [{id: "copy_to_output", label: this.i18n.TRANSFORM_MENU_DRAGTOOUTPUT}]};

      this.sourceTree = new flow.dijit.XMLEditTree({canDrag: true, menuinfo: menuinfo, xmlRootNode: xmlDoc.root, path: xmlDoc.path, operatorId: this.operator.id, operator: this.operator}, this.inputNode);

      var templ = this.operator.model.getNode("flow:template/*", this.operator.node);

      menuinfo = {getMenuItems: dojo.hitch(this, this.getMenuItems),
                  callback: dojo.hitch(this, this.menuCallback)};

      var myThis = this;

      function _buildType(type) {
        return "flowtype" + type.substr(0,1).toUpperCase() + type.substr(1);
      }

      function _getIconClass(item) {
        if (!item) {
          return;
        }

        var type = "flowtype ";

        var node = this.store.getNode(item);

        var typeitem;
        if (node.nodeType == 1 && node.childNodes.length == 0) {
          var xpath = item.store.buildXPath(item.store.getNode(item), true);
          var param = myThis.operator.model.getNode("flow:param[@operatorspecific='"+xpath+"']", this.operator.node);
          if (param) {
            typeitem = myThis.operator.model.getValue(param, "@type");
          }
        }

        if (typeitem) {
          type += _buildType(typeitem);
        } else {
          type = this.inherited("getIconClass", arguments);
        }

        return type;
      }

      this.targetTree = new flow.dijit.XMLEditTree({canDrop: true, menuinfo: menuinfo, xmlRootNode: templ, operatorId: this.operator.id, path: this.operator.TEMPLATEPATH, operator: this.operator, getIconClass: _getIconClass}, this.outputNode);

      this.resize();
      dojo.connect(dijit.byId("flowcanvas").dialogs["property"].widget, "resize", this, "resize");

      // make the tree cover the whole pane
      this.targetTree.domNode.style.height = "100%";
    },

    checkAncestorNodesForParam: function(treenode, direction) {
      var item = this.getParamForTreeNode(treenode);
      if (!item.param) {
        var parameters = this.operator.model.runXPath("flow:param", this.operator.node);
        for (var i = 0; i < parameters.length; i++) {
          var parameter = parameters[i];
          var operatorspecific = this.operator.model.getValue(parameter, "@operatorspecific");
          if (direction) {
            if ((item.operatorspecific.indexOf(operatorspecific) != -1) && operatorspecific.indexOf("@")==-1) {
              item.param = parameter;
              item.type = this.operator.model.getValue(item.param, "@type");
              break;
            }
          } else {
            if ((operatorspecific.indexOf(item.operatorspecific) != -1)  && item.operatorspecific.indexOf("@")==-1) {
              item.param = parameter;
              item.type = this.operator.model.getValue(item.param, "@type");
              break;
            }
          }
        }
      } else {
        item.exactmatch = true;
      }
      return item;
    },

    getMenuItems: function(treenode) {
      var node = treenode.tree.store.getNode(treenode.item);

      var isAttribute = (node.nodeType == 2) ? true : false;
      var canMoveDown = ((node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4) && node.nextSibling) ? true : false;
      var canMoveUp = ((node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4) && node.previousSibling) ? true : false;
      var canEdit=  true;
      var canHaveChild = node.nodeType == 1;
      var isRoot = treenode.getParent().isTree;
      var item = this.checkAncestorNodesForParam(treenode, true);

      if (item.param) {
        if (item.type == "xpath") {
          canEdit = false;
          canHaveChild = false;
          if(!item.exactmatch) {
            canMoveDown=false;
            canMoveUp=false;
            isRoot=true;
          }
        }

        if (item.type == "function") {
          canHaveChild = false;
        }
      }

      var item1 = this.checkAncestorNodesForParam(treenode, false);

      if (item1.param) {
        if ((item1.type == "xpath" || item1.type == "function") && !item1.exactmatch) {
          canEdit = false;
          isRoot = true;
        }
      }

      return [{id: "new_element", label: this.i18n.TRANSFORM_MENU_NEWELEMENT, disabled: !canHaveChild},
              {id: "new_attribute", label: this.i18n.TRANSFORM_MENU_NEWATTRIBUTE, disabled: !canHaveChild},
              {id: "new_text", label: this.i18n.TRANSFORM_MENU_NEWTEXT, disabled: !canHaveChild},
              {id: ""},
              {id: "functions", label: this.i18n.DROPDOWNCOMBO_LABEL_FUNCTION, disabled: !canHaveChild},
              {id: ""},
              {id: "edit", label: this.i18n.TRANSFORM_MENU_EDIT, disabled: !canEdit, hidden: isAttribute},
              {id: "edit_name", label: this.i18n.TRANSFORM_MENU_EDIT_ATTRIBUTE_NAME, disabled: !canEdit, hidden: !isAttribute},
              {id: "edit_value", label: this.i18n.TRANSFORM_MENU_EDIT_ATTRIBUTE_VALUE, disabled: !canEdit, hidden: !isAttribute},
              {id: "delete", label: this.i18n.TRANSFORM_MENU_DELETE, disabled: isRoot},
              {id: "move_up", label: this.i18n.TRANSFORM_MENU_MOVEUP, disabled: !canMoveUp},
              {id: "move_down", label: this.i18n.TRANSFORM_MENU_MOVEDOWN, disabled: !canMoveDown}
             ];
    },

    getParamForTreeNode: function(treenode) {
      if (!treenode) {
        return {};
      }

      var item = {};
      try {
        if (treenode.nodeType) {
          // treenode can be the domnode for iconclass stuff
          item.node = treenode;
        } else {
          item.node = treenode.tree.store.getNode(treenode.item);
        }
        
		var parentnode =  (item.node.nodeType==2)?this.targetTree.store.getNode(this.targetTree.getTreeNodeByItem(treenode.item).getParent().item):null;
        
        item.operatorspecific = this.targetTree.store.buildXPath(item.node,true,true,null,parentnode);

        item.param = this.operator.model.getNode("flow:param[@operatorspecific='"+item.operatorspecific+"']", this.operator.node);
        if(item.param) item.type= this.operator.model.getValue(item.param, "@type");
      } catch(ex){}

      return item;
    },

    menuCallback: function(menuitem, treenode) {
      var action = menuitem.action;
      this.menuNode = treenode;
  
      if (action == "copy_to_output")  {
        this.dragDrop(treenode, this.targetTree.getSelectedNode())
      } else if (action == "new_element") {
        this.targetTree.addNewElement(treenode);
      } else if (action == "new_attribute") {
        this.targetTree.addNewAttribute(treenode);
      } else if (action == "new_text") {
        this.targetTree.addNewTextNode(treenode);
      } else if (action == "edit") {
        var item = this.getParamForTreeNode(this.menuNode);
        if (item.param) {
          if (item.type == "function") {
            this.showFunctionDialog(this.operator.generateDataFromParameter(item.param));
          } else if (item.type == "xpath") {
          } else {
            this.targetTree.editNode(treenode);
          }
        } else {
          this.targetTree.editNode(treenode);
        }
      } else if (action == "edit_name" || action == "edit_value") {
        // for editing attributes
        var editValue = (action == "edit_value") ? true : false;
        this.targetTree.editNode(treenode, editValue);
      } else if (action == "delete") {
        var item= this.getParamForTreeNode(this.menuNode);
        if (item.param) {
          item.param.parentNode.removeChild(item.param);
          this.operator.model.setValue("@dirty", this.operator.node, "1");
        }
        this.targetTree.deleteNode(treenode);
      } else if (action == "move_up") {
        this.targetTree.moveNode(treenode, "up");
      } else if (action == "move_down") {
        this.targetTree.moveNode(treenode, "down");
      } else if (action == "valuechanged") {
      } else if (action == "functions")  {
        this.showFunctionDialog({});
      }
    },

    dragDrop: function(srctreenode, targettreenode) {
      if (targettreenode.item.store == this.sourceTree.store) {
        // dojo bug, target is in source tree
        return false;
      }

      if (this.getParamForTreeNode(targettreenode).param) {
        alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYONEXISTINGSOURCE);
        return false;
      }

      var titem = this.checkAncestorNodesForParam(targettreenode, true);

      if (titem.param) {
        if (titem.type == "xpath") {
          alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYONEXISTINGSOURCE);
          return false;
        }
      }

      var srcnode = this.sourceTree.store.getNode(srctreenode.item);

      // uh, IE doesn't allow you to go from attribute to parent
      var parentnode =  (srcnode.nodeType==2)?this.sourceTree.store.getNode(this.sourceTree.getTreeNodeByItem(srctreenode.item).getParent().item):null;

      var srcpath = this.sourceTree.store.buildXPath(srcnode, true, true, null, parentnode);

      //var srcpath = this.getOperatorSpecificPathForNode(this.FEEDPATH, srcnode);
      var item = this.getParamForTreeNode(targettreenode);
      var targetnode = item.node;

      if (srcnode.nodeType == 1 && (item.node.nodeType==3 || item.node.nodeType==2 || item.node.nodeType == 4)) {
        alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYELEMENTONATTRIBUTE);
        return false;
      }

      // new item is an element
      if (item.node.nodeType == 1) {
        if (srcnode.nodeType == 3 || srcnode.nodeType == 4) {
          var dupecount = 1;
          for (var i = 0; i < item.node.childNodes.length; i++) {
            if (item.node.childNodes[i].nodeType == 3 || item.node.childNodes[i].nodeType == 4)  dupecount++;
          }

          item.operatorspecific+="/text()["+dupecount+"]";
        } else if (srcnode.nodeType == 2) {
          item.operatorspecific+="/@"+srcnode.name;
        	if(this.operator.model.getNode("flow:param[@operatorspecific='"+item.operatorspecific+"']", this.operator.node)!=null)
        	{
        		alert(dojo.string.substitute(this.i18n.TRANSFORM_ERRORMSG_ATTRIBUTE_EXISTS, [srcnode.name]));
        		return;
        	}
        } else if (srcnode.nodeType == 1) {
          var preview = this.operator.setNamespace(srcnode.namespaceURI);
          preview= (preview != "") ? preview + ":" : "";
          item.operatorspecific += "/" + preview + flow.xml.getLocalName(srcnode);
        }
      }

      var value = {type: "xpath", caption: "", inputid: ""};
      value.operatorspecific = item.operatorspecific;

      if (item.param) {
        value.id = this.operator.model.getValue(item.param, "@id");
        value.data = srcpath;
        this.operator.modifyChild(value.id, "value", value);
      } else {
        value.id = this.operator.model.generateId("param");
        value.value = srcpath;
        this.operator.createParameter(value, this.operator.node);
      }

      var targetparentnode =  (item.node.nodeType==2)?this.targetTree.getTreeNodeByItem(targettreenode.item).getParent():null;
      if ((srcnode.nodeType == 2 || srcnode.nodeType == 3|| srcnode.nodeType == 4) && (item.node.nodeType == 2 || item.node.nodeType == 3 || item.node.nodeType == 4)) {
        // dnd a text/attr to a text/attr
        var newvalue;

        if (item.node.nodeType == 3 ||item.node.nodeType == 4) {
          newvalue = srcnode.nodeValue;
        } else if (item.node.nodeType == 2) {
          newvalue = {name: item.node.name, value: srcnode.nodeValue};
        }

        this.targetTree.setStoreValue(targettreenode, newvalue,targetparentnode);
      } else if (item.param || item.node.nodeType == 2) {
        var newvalue={};

        if (item.node.nodeType == 1) {
          newvalue = srcnode.value;
        } else if (item.node.nodeType == 3||item.node.nodeType == 4) {
          newvalue = srcnode.nodeValue;
        } else if (item.node.nodeType == 2) {
          newvalue = {name: item.node.name, value: (srcnode.value!=null?srcnode.value:srcnode.nodeValue)};
        }

        this.targetTree.setStoreValue(targettreenode, newvalue,targetparentnode);
      } else {
        if (srcnode.nodeType == 2) {
          this.targetTree.addAttribute(targettreenode, srcnode.name, srcnode.value);
        } else {
          this.targetTree.importTreeNode(srctreenode, targettreenode);
        }
      }

      return true;
    },

    showFunctionDialog: function(value) {
      dojo.require("flow.dijit.Dialog");

      var div = document.createElement("div");
      dojo.body().appendChild(div);

      if (this.dialog) {
        this.dialog.hide();
        this.dialog.destroyRecursive();
        this.dialog = null;
      }

      var labels = dojo.i18n.getLocalization("flow", "labels");

      this.dialog = new flow.dijit.Dialog({title: this.i18n.USE_FUNCTION_TITLE, chain: 0, showUnderlay: true, buttons:[{label: labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)}, {label:labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);
      this.dialog.resize({w:400, h: 400});
      this.dialog.show();

      div = document.createElement("div");
      this.dialog.containerNode.appendChild(div);
      dojo.require("flow.dijit.FunctionInputSelection");
      this.selectWidget = new flow.dijit.FunctionInputSelection({data: value, chain: 0, chainParent: this.operator, operator: this.operator}, div);
    },

    _cancelDialog: function(event) {
      this.selectWidget.destroy();
      this.selectWidget = null;

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    },

    _acceptDialog: function(event) {
      var value = this.selectWidget.getData();

      if (value) {
        var item = this.getParamForTreeNode(this.menuNode);
        value.operatorspecific = item.operatorspecific;
        var count = this.operator.model.runXPath(".//flow:param[@type='function']/flow:function[@name='"+value.name+"']",this.operator.node).length;

        if (item.param) {
          // this is an edit
          var orgname=item.node.nodeName;
		  var expectedname=this.operator.model.getValue(item.param,"flow:function/@name")          
		  if(orgname!=expectedname)	value.caption = value.name+(count>0?"-"+count:"");

          value.type = "function";
          value.id = this.operator.model.getValue(item.param, "@id");
          value.operatorspecific=value.operatorspecific.substring(0,value.operatorspecific.lastIndexOf("/"))
          value.operatorspecific += "/" + value.caption;
          this.operator.model.setValue(".//flow:param[@id='"+value.id+"']/@operatorspecific", this.operator.node, value.operatorspecific);
          this.operator.model.setValue(".//flow:param[@id='"+value.id+"']/@caption", this.operator.node, value.caption);
          this.operator.modifyChild(value.id, "value", value);
		  if(orgname!=expectedname)	this.targetTree.setStoreValue(this.menuNode, value.caption);
        } else {
		  value.caption = value.name+(count>0?"-"+count:"");
          value.operatorspecific += "/" + value.caption;
          value.id = this.operator.model.generateId("param");
          this.operator.createParameter(value, this.operator.node);
          var newitem = this.targetTree.addElement(this.menuNode, value.caption);
        }
      }

      this._cancelDialog();
    },

    getOperatorSpecificPathForNode: function(basepath, node) {
    //not used should be removed
      var tree= basepath==this.TEMPLATEPATH?this.targetTree:this.sourceTree;
      var path=tree.store.buildXPath(node,true,true);
      /*
      if(node.nodeType == 2) {
        return path.substring(basepath.length,path.lastIndexOf("/"))+"/@"+node.name;
      } else {
        return  path.substring(basepath.length);
      }
      */
    },

    resize: function(args) {
      if (!this.propertyEditor) {
        this.propertyEditor = dijit.byNode(this.domNode.parentNode.parentNode.parentNode.parentNode.parentNode);
      }

      var height = this.propertyEditor.tabcontainerNode.containerNode.offsetHeight;
      height -= this.propertyEditor.captioninput.parentNode.offsetHeight + 22;

      this.domNode.style.height = height + "px";

      this.layoutNode.resize();
    }
  }
);

