/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Variables");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "labels");

dojo.require("flow.dijit.Grid");

dojo.declare(
  "flow.dijit.Variables",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    // attach points
    gridNode: null,
    grid: null,

    // impl
    model: null,
    canvas: null,
    chain: null,

    imgPath: dojo.moduleUrl("flow.dijit", "templates/images"),

    templatePath: dojo.moduleUrl("flow.dijit", "templates/Variables.html"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      this.variables = [];

      // load variables
      var variables = this.model.runXPath("flow:variables/flow:param", this.model.flowNode);
      var data = [];

      for (var i = 0; i < variables.length; i++) {
        this.variables[variables[i].getAttribute("id")] = {id: variables[i].getAttribute("id"),
                                                           name: variables[i].getAttribute("caption"),
                                                           defaultvalue: this.model.getValue(variables[i], "text()"),
                                                           description: variables[i].getAttribute("comments")
                                                          };

        data.push(this.variables[variables[i].getAttribute("id")]);
      }

      var griddata = {layout: null, data: data, config: {allowGridEditing: true, canMoveRows: false, canAddRows: true}}
      griddata.layout = [
                         {name: "name", label: this.i18n.labels.VARIABLES_GRID_TITLE_NAME, tooltip: this.i18n.labels.VARIABLES_GRID_NAME_TOOLTIP, editable: true, width: "30%"},
                         {name: "defaultvalue", label: this.i18n.labels.VARIABLES_GRID_TITLE_DEFAULT_VALUE, tooltip: this.i18n.labels.VARIABLES_GRID_DEFAULT_VALUE_TOOLTIP, editable: true, width: "30%"},
                         {name: "description", label: this.i18n.labels.VARIABLES_GRID_TITLE_DESCRIPTION, tooltip: this.i18n.labels.VARIABLES_GRID_DESCRIPTION_TOOLTIP, editable: true, width: "40%"}
                        ];

      this.grid = new flow.dijit.Grid({grid: griddata, operator: this}, this.gridNode);
    },

    addChild: function() {
      var id = this.model.generateId("variable");

      this.variables[id] = {id: id, name: "", defaultvalue: "", description: ""};
      return this.variables[id];
    },

    setChildValue: function(id, key, value) {
      this.modifyChild(id, key, value);
    },

    modifyChild: function(id, key, value) {
      if (key == "name" || key == "defaultvalue" || key == "description") {
        this.variables[id][key] = value;
      }
    },

    removeChild: function(id) {
      delete this.variables[id];
    },

    updateModel: function() {
      // update/delete/create variables
      var variablesNode = this.model.getNode("flow:variables", this.model.flowNode)

      // make list of existing variables
      var variables = this.model.runXPath("flow:param", variablesNode);
      var variablelist = {};
      for (var i = 0; i < variables.length; i++) {
        variablelist[variables[i].getAttribute("id")] = true;
      }

      // update existing variables
      for (var id in this.variables) {
        if (!this.variables[id].name) {
          // no name, delete it
          delete variablelist[id];
          continue;
        }

        if (variablelist[id]) {
          // exists already
          var variable = this.model.getNode("flow:param[@id='"+id+"']", variablesNode);

          // remove from the list, already handled
          delete variablelist[id];
        } else {
          // create

          var template='<param xmlns="'+this.model.flowNS +
                      '" caption="" operatorspecific="" inputid="" id="'+id +
                      '" position="" type="dynamic" comments=""></param>';

//          var template = '<variable xmlns="'+this.model.flowNS+'" name="" default="" caption="" id="'+id+'"/>';

          var variable = this.model.parseXML(template);
          variable = variablesNode.appendChild(variable);
        }

        this.model.setValue("@caption", variable, this.variables[id].name);
        this.model.setValue("text()", variable, this.variables[id].defaultvalue);
        this.model.setValue("@comments", variable, this.variables[id].description);
      }

      // remove any variables still in the list
      for (var id in variablelist) {
        this.model.removeNode("flow:param[@id='"+id+"']", variablesNode);
      }
    },

    getData: function() {
      this.updateModel();	
      if (this.grid.selectedRow) {
        var data = {};
        data.type = "variable";
        var id = this.grid.selectedRow.getAttribute("rowid");
        var name = this.variables[id].name;
        data.value = id;
        return data;
      }
      return null;
    },

    showHelp: function() {
      this.canvas.showHelp(FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_variables.htm", this.i18n.labels.VARIABLES_HELP_TITLE, this.chain);
    }
  }
);
