/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.XMLEditTree");

dojo.require("flow.dijit.XMLViewTree");
dojo.require("dijit.Menu");

dojo.declare(
  "flow.dijit.XMLEditTree",
  flow.dijit.XMLViewTree,
  {
    menus: null,
    store: null,
    path: null,
    canDrag: false,
    canDrop: false,
    operator: null,

    menuinfo: null,
    contextmenu: null,

    hoverTargetTreeNode: null,

    postCreate: function() {
      // drag and drop
      if (this.canDrag || this.canDrop) {
        dojo.require("dijit._tree.dndSource");
        this.dndController = this.canDrag ? "dijit._tree.dndSource" : "dijit._tree.dndTarget";
      }

      this.inherited("postCreate", arguments);
      this.store.operator = this.operator;

      if (this.menuinfo) {
        this.contextmenu = new dijit.Menu({targetNodeIds: [this.domNode.getAttribute("id")]});

        if (this.menuinfo.items) {
          this.buildMenuItems(this.menuinfo.items);
        }

        dojo.connect(this.contextmenu, "onItemClick", this, this.onItemClick);
        dojo.connect(this.contextmenu, "_openMyself", this, this.openContextMenu);
        dojo.connect(this.contextmenu, "onClose", this, this.closeContextMenu);
      }
    },

    buildMenuItems: function(items) {
      var contextmenu = this.contextmenu;
      dojo.forEach(this.contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      for (var i = 0; i < items.length; i++) {
        var item = items[i];

        if (item.hidden) {
          continue;
        }

        if (item.label) {
          var menuitem = new dijit.MenuItem({label: item.label, action: item.id, disabled: item.disabled});
          this.contextmenu.addChild(menuitem);
        } else {
          this.contextmenu.addChild(new dijit.MenuSeparator());
        }
      }
    },

    openContextMenu: function(event) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.labelNode, "selected");
      }

      this.selectedTreeNode = dijit.getEnclosingWidget(event.target);

      if (this.selectedTreeNode.isTree) {
        // is tree
        this.selectedTreeNode = this.selectedTreeNode.getChildren()[0];
      }

      dojo.addClass(this.selectedTreeNode.labelNode, "selected");

      if (this.menuinfo.getMenuItems) {
        var items = this.menuinfo.getMenuItems(this.selectedTreeNode);
        this.buildMenuItems(items);
      }
    },

    closeContextMenu: function(event) {
    },

    onItemClick: function(menuitem) {
      if (menuitem.disabled) {
        return false;
      }

      this.menuinfo.callback(menuitem, this.selectedTreeNode);
    },

    makeNodeEditable: function(treenode, item, editValue) {
      var myThis = this;
      var node = this.store.getNode(item);

      function setNodeValue() {
        var legalNamePattern;

        if (node.nodeType == 2 && editValue) {
          // attribute values can have numbers anywhere
          legalNamePattern = /^[a-zA-Z0-9\u0100-\uFFFF_\-\.]*$/;
        } else {
          legalNamePattern = /^[a-zA-Z\u0100-\uFFFF_][a-zA-Z0-9\u0100-\uFFFF_\-\.]*$/;
        }

        if (node.nodeType == 3 || (input && input.value.match(legalNamePattern))) {
          var newvalue;
          var parentitem;

          if (node.nodeType == 1 || node.nodeType == 3) {
            newvalue = input.value;
          } else if (node.nodeType == 2) {
	          var name = node.nodeName, value = node.nodeValue;
	          if (editValue) {
	            value = input.value;
	          } else {
	            name = input.value;
	          }

            // attributes are bad in IE, so pass parent along
            parentitem = treenode.getParent()

	          newvalue = {name: name, value: value};
	        }
          myThis.setStoreValue(treenode, newvalue, parentitem);
          resetNodeValue();
	        //treenode.labelNode.focus();
	        //treenode.labelNode.setAttribute("tabIndex", "0");
	        //dojo.addClass(treenode.labelNode, "dijitTreeLabelFocused");
	        //myThis.lastFocused = treenode;
	        var menuitem = {};
	        menuitem.action = "valuechanged";
	        myThis.menuinfo.callback(menuitem, treenode);
        } else {
          resetNodeValue();
        }
      }

      function keyPressed(e) {
        try {
        	var evtobj = window.event? event : e;
          var key = evtobj.charCode? evtobj.charCode : evtobj.keyCode;
        	//var key = window.event ? e.keyCode : e.which ? e.which : e.charCode;;

          if (key == dojo.keys.ESCAPE) {
            resetNodeValue();
          } else if (key == dojo.keys.ENTER) {
            setNodeValue();
	        } else if (node.nodeType != 3) {
            // 37 - left arrow, 39 - right arrow
            if (!evtobj.charCode && (key == 37 || key == 39)) {
              return true;
            } else if (node.nodeType == 2 && editValue && (key >= 48 && key <= 57)) {
              // editing an attribute and editing value - allow numbers anywhere
              return true;
            } else if (key < 8 || (key >= 9 && key < 27) || (key >= 30 && key <= 44) || key == 47 || (key >= 58 && key <= 64)
		        		       || (key >= 91 && key <= 94) || key == 96 || (key >= 123 && key <= 126)
                       || (((key >= 48 && key <= 57) || key == 45 || key == 46 )&& (input && input.value == ""))
		        		      ) {
		        	return false;
            } else {
		        	return true;
		        }
          }
        } catch(ex){}
      }

      function onBlur(e) {
        setNodeValue();
      }

      function resetNodeValue() {
        var value = myThis.store.getLabel(item);
        treenode.labelNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(value);
        treenode.labelNode.focus();
        treenode.labelNode.setAttribute("tabIndex", "0");
        dojo.addClass(treenode.labelNode, "dijitTreeLabelFocused");
        myThis.lastFocused = treenode;
      }

      var value = treenode.labelNode.firstChild.nodeValue;
      var span = document.createElement("span");

      var input;

      if (node.nodeType == 1 || node.nodeType == 3) {
        // element
        input = document.createElement("input");
        input.setAttribute("value", value);
        span.appendChild(input);
      } else if (node.nodeType == 2) {
        // attribute - either edit name or value
        input = document.createElement("input");

        if (editValue) {
          span.appendChild(document.createTextNode(node.nodeName));
          span.appendChild(document.createTextNode(" = "));

          input.setAttribute("value", node.nodeValue);
          span.appendChild(input);
        } else {
          input.setAttribute("value", node.nodeName);
          span.appendChild(input);
          span.appendChild(document.createTextNode(" = "));
          span.appendChild(document.createTextNode(node.nodeValue));
        }
      }

      input.onkeypress = keyPressed;
      //dojo.connect(input, 'onkeypress', function(e) {keyPressed(e);});
      dojo.connect(input, 'onblur', function(e) {onBlur(e);});

      treenode.labelNode.replaceChild(span, treenode.labelNode.firstChild);

      // stop the treenode to steal focus from our controls
      dojo.connect(span, "onkeypress", function(e) {
                   e.stopPropagation();
      });

      dojo.connect(span, "onfocus", function(e) {
                   e.stopPropagation();
      });

      dojo.connect(span, "onclick", function(e) {
                   e.stopPropagation();
      });

      // unfocus existing and focus us
      this.blurNode();
      //span.setAttribute("tabIndex", "0");
      input.focus();
    },

    setStoreValue: function(treenode, value, parenttreenode) {
      this.store.setValue(treenode.item, "value", value, parenttreenode ? parenttreenode.item : null);
    },

    importTreeNode: function(treenode, parentitem) {
      var domnode = treenode.item.store.getNode(treenode.item);

      if (!domnode) {
        return;
      }

      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      this.store.addNewNode(domnode, parentnode);
    },

    addNewElement: function(parentitem) {
      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var element = this.xmlRootNode.ownerDocument.createElement("new");
      var newitem = this.store.addNewNode(element, parentnode);

      // expand parent item - else no treenode will be created
      if (parentitem) {
        this._expandNode(parentitem);
      } else {
        this._expandNode(this._itemNodeMap[this.store.getIdentity(this.store.getItem(this.store.rootElement))]);
      }

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout - so double it
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 300)
    },

    addElement: function(parentitem, name) {
      if (parentitem) {
        var parentnode = this.store.getNode(parentitem.item);
        var element = this.xmlRootNode.ownerDocument.createElement(name);
        var newitem = this.store.addNewNode(element, parentnode);
      }

      if (parentitem) {
        this._expandNode(parentitem);
      } else {
        this._expandNode(this._itemNodeMap[this.store.getIdentity(this.store.getItem(this.store.rootElement))]);
      }

      return newitem;
    },

    addAttribute: function(parentitem, name, value) {
      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
        var newitem = this.store.addAttribute(parentnode, name, value);
      }
    },

    addNewAttribute: function(parentitem) {
      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var newitem = this.store.addNewAttribute(parentnode);

      // expand parent item - else no treenode will be created
      this._expandNode(parentitem);

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 250)
    },

    _onNewItem: function(item, parentInfo, index) {
      var node = this.store.getNode(item);

      if (typeof(index) != "number" && node.nodeType != 2) {
        this.inherited("_onNewItem", arguments);
        return;
      }

      // attributes go at the beginning
      var parent = this._itemNodeMap[this.getItemParentIdentity(item, parentInfo)];

			if (!parent.isExpandable) {
				parent.makeExpandable();
			}

      var childParams = {
        item: item,
        isExpandable: this.mayHaveChildren(item)
      };

      var child = new dijit._TreeNode(
        dojo.mixin({
					tree: this.tree,
				  label: this.tree.getLabel(childParams.item)
			  }, childParams)
      );

      // calculate index - length of attributes
      if (typeof(index) == "undefined") {
        index = this.store.getNode(parent.item).attributes.length - 1;
      }

      parent.addChild(child, index);
      child._updateLayout();

      var childrenMap = [];
      childrenMap[this.tree.store.getIdentity(childParams.item)] = child;

			dojo.mixin(this._itemNodeMap, childrenMap);
    },

    addNewTextNode: function(parentitem) {
      var parentnode = null;

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var newitem = this.store.addNewTextNode(parentnode);

      // expand parent item - else no treenode will be created
      this._expandNode(parentitem);

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 250)
    },

    editNode: function(targetitem, editValue) {
      var treenode = this._itemNodeMap[this.store.getIdentity(targetitem.item)];
      this.makeNodeEditable(treenode, targetitem.item, editValue);
    },

    deleteNode: function(targetitem) {
		var node = this.store.getNode(targetitem.item);
		var parentnode=null;
    	if (dojo.isIE && node.nodeType == 2) {
        	parentnode =this.store.getNode(this.selectedTreeNode.getParent().item);
		}
      	this.store.deleteNode(targetitem.item,parentnode);

      // clear out selected node
      this.selectedTreeNode = null;
    },

    moveNode: function(targetitem, direction) {
      this.store.moveNode(targetitem.item, direction);

      // clear out selected node
      this.selectedTreeNode = null;
    },

    // drag and drop support
    checkAcceptance: function(source, nodes) {
      return this.tree.canDrop;
    },

    checkItemAcceptance: function(node, source) {
      // who do we want to allow to drop on:
      //   - elements
      //   - attribute if the source is a attribute or text node
      //   - text node if the source is a attribute or text node
      var widget = dijit.getEnclosingWidget(node);

      if (this.hoverTargetTreeNode) {
        dojo.removeClass(this.hoverTargetTreeNode.labelNode, "flowTreeNodeHover");
      }

      dojo.addClass(widget.labelNode, "flowTreeNodeHover");
      this.hoverTargetTreeNode = widget;

      var targetItem = widget.item;

      var allow = false;

      if (this.tree.store.hasAttribute(targetItem, "canHaveChildren")) {
        allow = true
      } else {
        // only one allowed
        var sourceitem = source.getSelectedItems()[0];

        var targetNode = this.tree.store.getNode(targetItem);
        var sourceNode = sourceitem.store.getNode(sourceitem);

        if ((targetNode.nodeType == 2 || targetNode.nodeType == 3) &&
            (sourceNode.nodeType == 2 || sourceNode.nodeType == 3)) {
          allow = true;
        }
      }

      return allow;
    },

    itemCreator: function(nodes) {
      var items = []

      for (var i=0; i < nodes.length; i++) {
        var widget = dijit.getEnclosingWidget(nodes[i]);
        if (!widget) {
          continue;
        }

        var item = widget.item;
        items.push({
          id: nodes[i].id,
          item: item
        });
      }
      return items;
    },

    onDndDrop: function(source, nodes, copy) {
      if (this.containerState == "Over") {
        var target = this.current;
        var items = this.itemCreator(nodes, target);

        var targetitem;
        if (!target || target == this.tree.domNode) {
          var tree = this.tree;
          var store = tree.store;
          targetitem = tree._itemNodeMap[store.getIdentity(store.getItem(store.rootElement))];
        } else {
          targetitem = dijit.getEnclosingWidget(target);
        }

        for (var i = 0; i < items.length; i++) {
          this.tree.operator.dragDrop(items[i], targetitem);
        }

        // if we drop on an expandable but closed tree item, expand it
        if (targetitem.isExpandable && !targetitem.isExpanded) {
          this.tree._expandNode(targetitem);
        }

      }

      this.onDndCancel();
    },

    onDndCancel: function() {
      this.inherited("onDndCancel", arguments);

      if (this.hoverTargetTreeNode) {
        dojo.removeClass(this.hoverTargetTreeNode.labelNode, "flowTreeNodeHover");
        this.hoverTargetTreeNode = null;
      }
    },

    getSelectedNode: function() {
      return this.selectedTreeNode ?  this.selectedTreeNode : this.getChildren()[0];
    }
  }
);
