/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.XMLTypeTree");

dojo.require("flow.dijit.XMLViewTree");
dojo.require("dijit.Menu");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "labels");

dojo.declare(
  "flow.dijit.XMLTypeTree",
  flow.dijit.XMLViewTree,
  {
    chain: null,
    contextmenu: null,
    isTypeTree: true,

    postCreate: function() {
      this.inherited("postCreate", arguments);

      if (dojo.byId(this.domNode.getAttribute("id"))) {
        // make sure the domNode is in the DOM for real - XMLViewTextBox is the
        // culprit.
        this.initContextMenu();
      }
    },

    initContextMenu: function() {
      if (this.contextmenu) {
        return;
      }

      this.contextmenu = new flow.dijit.Menu({hparent: this, targetNodeIds: [this.domNode.getAttribute("id")]});

      dojo.connect(this.contextmenu, "onItemClick", this, this.onItemClick);
      dojo.connect(this.contextmenu, "_openMyself", this, this.openContextMenu);
      dojo.connect(this.contextmenu, "onClose", this, this.closeContextMenu);
    },

    buildMenuItems: function() {
      var contextmenu = this.contextmenu;
      dojo.forEach(this.contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      var labels = dojo.i18n.getLocalization("flow", "labels");
      var items = [
                   {id: "date", label: labels.TYPE_DATE},
                   {id: "decimal", label: labels.TYPE_DECIMAL},
                   {id: "integer", label: labels.TYPE_INTEGER},
                   {id: "string", label: labels.TYPE_STRING},
                   {},
                   {id: "clear", label: labels.TYPE_CLEAR, disabled: this.selectedTreeNode.flowtype ? false : true}
                   ];

      for (var i = 0; i < items.length; i++) {
        var item = items[i];

        if (typeof(item.label) == "string") {
          var type = item.id.substr(0, 1).toUpperCase() + item.id.substr(1)
          var menuitem = new dijit.MenuItem({label: item.label, action: item.id, disabled: item.disabled, iconClass: "flowtype flowtype"+type});
          this.contextmenu.addChild(menuitem);
        } else {
          this.contextmenu.addChild(new dijit.MenuSeparator());
        }
      }
    },

    openContextMenu: function(event) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.domNode, "selected");
      }

      this.selectedTreeNode = dijit.getEnclosingWidget(event.target);
      dojo.addClass(this.selectedTreeNode.domNode, "selected");

      this.buildMenuItems();

      if (this.chain) {
        // reset zIndex of popup
        var zindex = parseInt(dojo.style(this.contextmenu.domNode.parentNode, "zIndex"), 10)+this.chain;
        dojo.style(this.contextmenu.domNode.parentNode, "zIndex", zindex);
      }
      dojo.stopEvent(event);
    },

    closeContextMenu: function(event) {
    },

    onItemClick: function(menuitem) {
      if (menuitem.disabled) {
        return false;
      }

      var action = menuitem.action;

      if (action == "clear") {
        this.clearType();
      } else {
        this.setType(action);
      }

      //dijit.popup.close(this.contextmenu);
      //dijit.focus(this.contextmenu.savedFocus);
      // can't close the context menu via popup as it will hide the dropdown
      dijit.focus(this.contextmenu.savedFocus);
      dojo.style(this.contextmenu.domNode, "display", "none");
    },

    setType: function(type, treenode) {
      var node = treenode ? treenode : this.selectedTreeNode;

      if (node.flowtype == type) {
        return;
      }

      // check if type exists.  If so, remove
      this.clearType(node);

      // remember the type
      node.flowtype = type;

	  var parentnode =  this.store.getNode(this.getTreeNodeByItem(node.item).getParent().item);
      var path = "./"+this.store.buildXPath(this.store.getNode(node.item),true,true,null,parentnode);
      var template = '<datatype xmlns="'+this.operator.flowNS+'" type="'+type+'" ><![CDATA['+path+']]></datatype>';
      this.operator.node.appendChild(this.operator.model.parseXML(template));

      node.iconNode.style.width = node.iconNode.offsetWidth + 16 + "px";
      var newnode = document.createElement("div");
      newnode.className = "dijitInline flowtype "+this.getActionClassName(type);
      //newnode.style.marginLeft = "16px";
      node.iconNode.appendChild(newnode);
    },

    clearType: function(treenode) {
      var node = treenode ? treenode : this.selectedTreeNode;

	  var parentnode =  this.store.getNode(this.getTreeNodeByItem(node.item).getParent().item);
      var path = "./"+this.store.buildXPath(this.store.getNode(node.item),true,true,null,parentnode);
      this.operator.model.removeNodes(".//flow:datatype[text()='"+path+"']", this.operator.node);

      if (!node.flowtype) {
        return;
      }

      var nodes = dojo.query("div."+this.getActionClassName(node.flowtype), node.iconNode);

      if (nodes[0]) {
        nodes[0].parentNode.removeChild(nodes[0]);
        node.iconNode.style.width = "";
        node.flowtype = null;
      }
    },

    getActionClassName: function(action) {
      var classname = "flowtype"+action.charAt(0).toUpperCase() + action.substr(1);
      return classname;
    },

    getIconClass: function(item) {
      if (item) {

      }

      this.inherited("getIconClass", arguments);
    }
  }
);

// menu subclass to work around a dijit.popup bug.  We need to have the popup the
// menu creates be parented to our dropdown, or else showing the context menu
// will close the dropdown.

dojo.declare(
	"flow.dijit.Menu",
	dijit.Menu,
{
  _openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
    this.savedFocus = savedFocus;
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
      parent: this.hparent, // needed to make context menu work on a tree in a dropdown
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	}
});
