/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.XMLViewTextBox");

dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.TextBox");

dojo.require("flow.dijit.XMLTypeTree");

dojo.declare(
  "flow.dijit.XMLViewTextBox",
  [dijit.form.TextBox, dijit.form.ComboBoxMixin],
  {
    // callback to call with set value
    callback: null,
    data: null,
    operator: null,
    removeDuplicates: false,

    selectedTreeItem: null,
    xmlRootNode: null,
    inputid: null,
    path: null,

    isLoaded: false,

    baseClass: "flowdijitXMLViewTextBox dijitComboBox",

    postMixInProperties: function(){
      dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
      dijit.form.TextBox.prototype.postMixInProperties.apply(this, arguments);
    },

    postCreate: function() {
      dijit.form.TextBox.prototype.postCreate.apply(this, arguments);

      if (!this.operator) {
        this.setDisabled(true);
      }

      // don't disable the textbox for now
      //this.textbox.setAttribute("disabled", "true");

      if (this.data && this.data.value) {
        this.selectedTreeItem = this.data.value;
        this.setValue(this.data.value);
      } else {
        this.setValue("");
      }

      dojo.addClass(this.iconNode, "flowtypeCombobox")
      dojo.addClass(this.iconNode, "flowtypeComboboxTree")
      this.iconNode.style.visibility = "visible";

      this.isLoaded = true;
    },

    _onArrowMouseDown: function(e) {
      dojo.stopEvent(e);
      if (this._isShowingNow) {
        this.closeDropdown();
        this._isShowingNow = false;
      } else {
        this._picker = new flow.dijit.XMLTypeTree({xmlRootNode: this.xmlRootNode,
                                                   operator: this.operator,
                                                   path: this.path,
                                                   selectedXPath: this.selectedTreeItem,
                                                   onClick: dojo.hitch(this, this.onTreeItemClick),
                                                   getLabelClass: dojo.hitch(this, this.onTreeGetLabelClass),
                                                   removeDuplicates: this.removeDuplicates
                                                  });
        this._picker.domNode.style.height = "200px";

        this.showDropdown();
        this._isShowingNow = true;
      }
    },

		_hideResultList: function(){
			if(this._isShowingNow){
				dijit.popup.close(this._picker);
				this._arrowIdle();
				this._isShowingNow=false;
			}
		},

    showDropdown: function() {
      dijit.popup.open({
          parent: this,
          popup: this._picker,
          around: this.domNode,
          onCancel: dojo.hitch(this, this.closeDropdown),
          onClose: dojo.hitch(this, this.closeDropdown)
      });

      dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth, h: 200});
      this._picker.domNode.style.backgroundColor = "white";
      this._picker.domNode.style.overflow = "auto";
      this._picker.domNode.style.border = "1px solid black";

      // focus the node so that blur happens
      this._picker.domNode.focus();
      this._picker.initContextMenu();
    },

    closeDropdown: function() {
      this._isShowingNow = false;
      dijit.popup.close(this._picker);
    },

    onTreeItemClick: function(item, node) {
      var value = "./"+this._picker.getXPathForTreeItem(item);

      this.selectedTreeItem = value;

      this.setValue(value);

      this.closeDropdown();
    },

    setValue: function(value, priorityChange) {
      this.inherited("setValue", arguments);

      this.callCallback(value);
    },

    callCallback: function(value) {
      // don't callback if not loaded yet
      if (this.callback && this.isLoaded) {
        var data = flow.operator.Data.create("xpath");
        data.value = value;
        data.inputid = this.inputid;
        this.callback(data);
      }
    },

    onTreeGetLabelClass: function(item) {
      if (item && this.selectedTreeItem && "./"+item.store.buildXPath(item.store.getNode(item), true) == this.selectedTreeItem) {
        return "xmlviewtreeselected";
      }

      return "";
    }
  }
);
