/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.XMLViewTree");

dojo.require("dijit.Tree");
dojo.require("flow.data.XMLStore");
dojo.require("dojox.data.dom");

dojo.declare(
  "flow.dijit.XMLViewTree",
  dijit.Tree,
  {
    xmlRootNode: null,
    store: null,
    path: null,
    operator: null,
    removeDuplicates: false,
    selectedXPath: null,

    selectedItem: null,
    selectedTreeNode: null,

    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    postMixInProperties: function() {
      // setup the store
      if (!this.xmlRootNode) {
        var x = document.createElement("root");
        x.setAttribute("id", " ");

        this.store = new flow.data.XMLStore({rootElement: x});
      } else {
        this.store = new flow.data.XMLStore({rootElement: this.xmlRootNode, removeDuplicates: this.removeDuplicates});
      }

      if (this.operator) {
        this.store.operator = this.operator;
      }

      if (this.path) {
        this.store.path = this.path;
      }

      flow.dijit.XMLViewTree.superclass.postMixInProperties.call(this);
    },

    postCreate: function() {
      this.inherited("postCreate", arguments);

      // make sure root is expanded
      var children = this.getChildren();
      if (!children[0].isExpanded && children[0].isExpandable) {
        this._expandNode(children[0])
      }

      dojo.addClass(this.domNode, "flowXMLTree");

      // if selected xpath exists, open tree
      if (this.selectedXPath) {
        var expandedXPath = this.selectedXPath.split("/");
        var count = 1;
        var node = children[0];

        while (count < expandedXPath.length) {
          if (!node.isExpandable) {
            // should never happen
            break;
          }

          var children = node.getChildren();
          for (var i = 0; i < children.length; i++) {
            var xpath = this.store.buildXPath(this.store.getNode(children[i].item), false, null, true);

            if (xpath == ("/" + expandedXPath[count])) {
              node = children[i];

              if (count < expandedXPath.length-1) {
                this._expandNode(node)
              }
              break;
            }
          }

          count++;
        }

      }
    },

    onClick: function(item, node) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.labelNode, "selected");
      }

      this.selectedItem = item;
      this.selectedTreeNode = node;

      dojo.addClass(this.selectedTreeNode.labelNode, "selected");
    },

    getIconClass: function(item) {
      if (!item) {
        return;
      }

      var type = "flowtype ";

      // called to get the icon for a tree
      var node = this.store.getNode(item);
      if (node.nodeType == 1) {
        type += "flowtypeElement";
      } else if (node.nodeType == 2) {
        type += "flowtypeAttribute";
      } else if (node.nodeType == 3 || node.nodeType == 4) {
        type += "flowtypeText";
      }

      return type;
    },

    getTreeNodeByItem: function(item) {
      return this._itemNodeMap[this.store.getIdentity(item)];
    },

    getXPathForTreeItem: function(item) {
      var node = this.store.getNode(item);

      var parentnode;

      if (dojo.isIE && node.nodeType == 2) {
        // IE doesn't support getting parent of an attribute node
        parentnode = this.store.getNode(this.getTreeNodeByItem(item).getParent().item);
      }

      return this.store.buildXPath(node, true, true, null, parentnode);
    }
  }
);

// update tree node for our needs (type stuff)
var treenodepostcreate = dijit._TreeNode.prototype.postCreate;
dojo.extend(dijit._TreeNode, {
  postCreate: function() {
    treenodepostcreate.apply(this, arguments);
    if (this.tree.isTypeTree && !this.isTree) {
      var datatype = this.tree.store.getValue(this.item, "flowtype");
      this.tree.setType(datatype, this);
    }

    // image check
    var content = dojox.data.dom.textContent(this.labelNode);
    var imgPattern = new RegExp("<img[^>]+>");
    if (imgPattern.test(content)) {
      this.labelNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(content);
    }

    if (this.item) {
      var title = this.item.store.getTitle(this.item);
      if (title) {
        this.labelNode.setAttribute("title", title);
      }
    }
  }
});


