/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("ged.widget.EditArea");

dojo.require("gdl.widget.EditArea");

dojo.declare(
	"ged.widget.EditArea",
	gdl.widget.EditArea,
{
	constructor: function _ged_widget_EditArea_constructor(){
		this.defaultHelperClasses = [
			"btp.widget.helper.baseHelper",
			"btp.widget.helper.focusHelper",
			"btp.widget.helper.styleHelper",
			"btp.widget.helper.dropHelper",
			"btp.widget.propsrc.componentPropertySource"
		];
		this.actionClasses = [
			"btp.widget.action.Actions"
		];
		this.loaderClass = "btp.widget.data.htmlLoader";
		this.generatorClass = "btp.widget.data.htmlGenerator";
		this.contentProviderClass = "btp.widget.provider.WidgetTreeContentProvider";

		// contentOutlinePageClass: Specifies a content outline page class which creates an
		// outline of the EditArea to be displayed on the Outline view.
		// TreeContentOutlinePage creates a tree view by traversing a widget tree on the EditArea.
		// CloneContentOutlinePage creates a pane that displays a copy of the EditArea.
		// CloneContentOutlinePage is just an experiment, not for practical use.
		//
		//this.contentOutlinePageClass = "btp.widget.outline.CloneContentOutlinePage";
		this.contentOutlinePageClass = "btp.widget.outline.TreeContentOutlinePage";

		// dragMoveTrackerClass: Specifies a DragTracker class that moves a component on the EditArea.
		// DragMoveTracker moves an actual component while dragging.
		// DragGhostMoveTracker moves a dashed bounding box of the component just like eclipse GEF.
		//
		this.dragMoveTrackerClass = "btp.widget.tool.DragGhostMoveTracker";
		//this.dragMoveTrackerClass = "btp.widget.tool.DragMoveTracker";
	},

	postCreate: function _ged_widget_EditArea_postCreate(){
		this.inherited("postCreate", arguments);
		if(window.location.href.indexOf("http") == 0){
			this.ioClass = "btp.widget.io.RestIO";
		}else {
			this.ioClass = "btp.widget.io.FileIO";
		}
		this.resizeConn = dojo.connect(this, "onEaMouseOverHandler", this, "loadResizeExt");
		this.setOutlineMode("outlineMode");
	},

	loadResizeExt: function _ged_widget_EditArea_loadResizeExt(){
		this.disconnect(this.resizeConn);


		// selectionHandles displays selection handles for a resizable component when it is focused.
		// Events for the handles are handled by a DragTracker (ResizeTracker).
		btp.loadExtension(this, "btp.widget.ext.selectionHandles");

		// selectionKnobType: Specifies a selection knob type from one of the followings.
		// "blackSquare" - A small black square, which is made of a <div> element. This is the default.
		// "whiteCircle" - A small circle just like a PowerPoint's selection handle. Made of an <img> element.
		// path - A path of an image file relative to the btp folder.
		// element - A DOM element.
		//
		// Some examples:
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "blackSquare";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "whiteCircle";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "widget/templates/EditArea/handle.gif";
		//var img = dojo.doc.createElement("IMG");
		//img.src = dojo.moduleUrl("btp", "widget/templates/EditArea/handle.gif")+""
		//img.style.zIndex = "1100";
		//img.style.position = "absolute";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = img;


		// connectionEndpoints places handles at the two ends of a gfx line widget.
		// Events for the handles are handled by a DragTracker (ReconnectTracker).
		btp.loadExtension(this, "gdl.widget.ext.connectionEndpoints");

		// endpointKnobType: Specifies an endpoint knob type.
		// Possible types are the same as those of selectionKnobType.
		//this.extensions["gdl.widget.ext.connectionEndpoints"].endpointKnobType = "blackSquare";
	},

	// override
	getContentOutlinePageClass: function(){
		var mode = this.getOutlineMode();
		if(mode == "outlineMode"){
			this.contentOutlinePageClass = "btp.widget.outline.TreeContentOutlinePage";
			this.contentProviderClass = "btp.widget.provider.WidgetTreeContentProvider";
		}else if(mode == "overviewMode"){
			this.contentOutlinePageClass = "gdl.widget.outline.CloneContentOutlinePage";
//			this.contentProviderClass = "btp.widget.provider.WidgetTreeContentProvider";
			this.contentProviderClass = "btp.widget.provider.GfxNodeTreeContentProvider";
		}else{
			alert("EditArea: unknown mode: "+mode);
		}
		return this.contentOutlinePageClass;
	},

	getOutlineToolbarConf: function(){
		var json = {
		    toolBar: {
		      item: [{
		        "@command": "changeOutlineMode",
		        "@arg": "outlineMode",
		        "@tooltip": "Outline Mode",
		        "@radio": "true",
		        "@name": "outline1",
		        "@selected": "true",
		        "@iconPath": "widget/templates/OutlineView/outline.gif"
		      },
		      {
		        "@command": "changeOutlineMode",
		        "@arg": "overviewMode",
		        "@tooltip": "Overview Mode",
		        "@radio": "true",
		        "@name": "outline1",
		        "@iconPath": "widget/templates/OutlineView/overview.gif"
		      }]
		    }
		};
		return json;
	}
});
