/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp._base.style");

// summary:
//		Utility functions for style manipulatoin.

dojo.mixin(btp,
{
	hasAccessor: function(widget, property){
		var info = widget.hlp_getComponentInfo();
		if(!info){ return false; }
		var cap = property.charAt(0).toUpperCase() + property.substring(1);
		if(info.accessors && (info.accessors["int"+cap+"Accessor"] || info.accessors[cap+"Accessor"])){
			return true;
		}else{
			return false;
		}
	},

	isResizableHorizontally: function(widget){
		var info = widget.hlp_getComponentInfo();
		if(!info || !info.resizable){
			return false;
		}
		if(!info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("EAST") != -1
			|| info.resizable.directions.indexOf("WEST") != -1;
	},

	isResizableVertically: function(widget){
		var info = widget.hlp_getComponentInfo();
		if(!info || !info.resizable){
			return false;
		}
		if(!info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("NORTH") != -1
			|| info.resizable.directions.indexOf("SOUTH") != -1;
	},

	// summary: get style value from an accessor if available or from domNode.
	// widget: target widget
	// property: "top", "left", "width", or "height"
	// return: integer value
	getStyleValue: function(widget, property){
		var info = widget.hlp_getComponentInfo();
		if(info && info.accessors){
			var cap = property.charAt(0).toUpperCase() + property.substring(1);
			if(info.accessors["int"+cap+"Accessor"]){
				var accessor = info.accessors["int"+cap+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["get"+cap]){
					return widget["get"+cap]();
				}else{
					return widget[accessor];
				}
			}else if(info.accessors[property+"Accessor"]){
				var accessor = info.accessors[property+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["get"+cap]){
					return parseInt(widget["get"+cap]());
				}else{
					return parseInt(widget[accessor]);
				}
			}else if(info.accessors[property]){
				return parseInt(info.accessors[property]);
			}
			// through
		}
		// To get the value of top and left, you need to use dojo.style.
		// On FF, widget.domNode.offsetTop/Left always returns zero when the EditArea is hidden.
		if(property == "top"){
			return parseInt(dojo.style(widget.domNode, "top"));
		}else if(property == "left"){
			return parseInt(dojo.style(widget.domNode, "left"));
		}else if(property == "width"){
			return dojo.marginBox(widget.domNode).w;
		}else if(property == "height"){
			return dojo.marginBox(widget.domNode).h;
		}
		return null;
	},

	// summary: set style value through an accessor if available or directly to domNode.
	// widget: target widget
	// property: "top", "left", etc.
	// value: string value with "px". (Ex. "100px")
	setStyleValue: function(widget, property, value){
		var info = widget.hlp_getComponentInfo();
		if(info && info.accessors){
			var cap = property.charAt(0).toUpperCase() + property.substring(1);
			if(info.accessors["int"+cap+"Accessor"]){
				var accessor = info.accessors["int"+cap+"Accessor"];
				cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["set"+cap]){
					widget["set"+cap](parseInt(value));
				}else{
					widget[accessor] = parseInt(value);
				}
				return;
			}else if(info.accessors[property+"Accessor"]){
				var accessor = info.accessors[property+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["set"+cap]){
					widget["set"+cap](value);
				}else{
					widget[accessor] = value;
				}
				return;
			}
			// through
		}
		widget.domNode.style[property] = value;
	},

/*
	XXgetStyleDimensions: function(widget, parent){
		var widgetDim = btp.getDimensions(widget.domNode);
		var parentDim = btp.getDimensions(parent.domNode);
		var obj = dojo._getBorderExtents(widget.domNode);
		var dim = {
			top: widgetDim.top - parentDim.top, // dy
			left: widgetDim.left - parentDim.left, // dx
			width: btp.getStyleValue(widget, "width")
				 + obj.w,
			height: btp.getStyleValue(widget, "height")
				 + obj.h
		};
		return dim;
	}
*/

	getStyleDimensions: function(widget, parent){
		var dim = btp.getDimensions(widget.domNode, parent.domNode);
		dim.width = btp.getStyleValue(widget, "width");
		dim.height = btp.getStyleValue(widget, "height");
		return dim;
	}
});
