/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp._base.utils");

// summary:
//		Utility functions.

dojo.mixin(btp,
{
	weakMixin: function(/*Object*/obj, /*Object*/props){
		// summary:
		//		Copies non-existing properties and methods of props to obj.
		var o = {};
		for(var x in props){
			if((o[x] == undefined || o[x] != props[x])
				&& obj[x] == undefined){
				obj[x] = props[x];
			}
		}
		return obj;
	},

	matchFilter: function(message, filter){
		if(!filter){ return false; }
		if(filter == "*"){ return true; }
		var arr = filter.split(",");
		var id = message.src.domNode.getAttribute("widgetId")
		var type = message.src.declaredClass;
		for(var i = 0, len = arr.length; i < len; i++){
			if(arr[i] == id || arr[i] == type){
				return true;
			}
		}
		return false;
	},

	getWidgetByElement: function(elem){
		while(elem){
			if(elem.widget){ // gfx raw node has the widget property
				return elem.widget;
			}
			var widget = dijit.byNode(elem);
			if(widget){
				return widget;
			}
			elem = elem.parentNode;
			if(!elem || elem.tagName == "BODY"){
				return null;
			}
		}
		return null;
	},

	isOrphan: function(elem){
		for(var n = elem; n; n = n.parentNode){
			if(n.tagName == "BODY")
				return false;
		}
		return true;
	},

	getEditors: function(/*Widget?*/top){
		var editors = [];
		if(top){
			dojo.query("[widgetId]", top.domNode).forEach(function(elem){
				var widget = dijit.byNode(elem);
				if(widget.isEditor){ editors.push(widget); }
			});
		}else{
			dijit.registry.forEach(function(widget){
				if(widget.isEditor){ editors.push(widget); }
			});
		}
		return editors;
	},

	isShowing: function(widget){
		for(var w = widget; w; w = w.getParent()){
			if(dojo.style(w.domNode, "display") == "none"){
				return false;
			}
		}
		return true;
	},

	isGfxWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.GfxWidget
				&& widget instanceof gdc.widget.GfxWidget);
	},

	isGfxLineWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.AbstractLine
				&& widget instanceof gdc.widget.AbstractLine);
	},

	isGfxNodeWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.AbstractNode
				&& widget instanceof gdc.widget.AbstractNode);
	},
	
	getLayoutParent: function(domNode) {
		var node = domNode;
		var layoutParent = null;
		while(node != dojo.body()){
			node = node.parentNode;
			var widget = dijit.byNode(node);
			if(widget && widget.layout) return widget;
		}
		return null;
	}
});
