/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.dnd.DropTarget");
dojo.require("btp.dnd.DragManager");

dojo.declare("btp.dnd.DropTarget", null, {
	disabled: false, // disable this drop target
	offsetParent: null,
	showingFeedback: false,
	acceptedTypes: null,
	nodeUnderMouse: null,
	caret: null,
	vertical: false,
	data: null, // any user data associated with this dt. (widget, etc.)

	constructor: function(node, types, data){
		this.data = data;
		var node = this.domNode = dojo.byId(node);
		node.dropTarget = this; // [custom property]
		if(!types){
			this.acceptedTypes = [];
		}else if(typeof(types) == "string"){
			this.acceptedTypes = [types];
		}else{
			this.acceptedTypes = types;
		}
		btp.dnd.dragManager.registerDropTarget(this);
	},

	// The onmousedown event has fired, but it doesn't mean D&D has started.
	onDragDown: function(e){
	},

	// D&D operation has been started
	onDragStart: function(e){
	},

	onDragOver: function(e){
		if(!this.accepts(e.dragSource)){ return false; }
		this.showFeedback(this);
		this.showingFeedback = true;
		return true;
	},

	onDragMove: function(e){
		if(this.shouldShowCaret(e)){
			this.showCaret(e);
		}
	},

	// D&D operation has been ended
	onDragEnd: function(e){
	},

	onDropStart: function(e){
	},

	onDrop: function(e){
	},

	onDropEnd: function(){
	},

	onDragOut: function(e){
		if(this.showingFeedback){
			this.hideFeedback(this);
			this.showingFeedback = false;
		}
		this.hideCaret();
	},

	unregister: function(){
		btp.dnd.dragManager.unregisterDropTarget(this);
		this.domNode.dropTarget = null; // [custom property]
	},

	reregister: function(){
		btp.dnd.dragManager.registerDropTarget(this);
		this.domNode.dropTarget = this; // [custom property]
	},

	// You may want to override this
	isAcceptable: function(ds, dt){
		return true;
	},

	accepts: function(ds){
		if(!this.isAcceptable(ds, this)){ return false; }
		for(var i = 0, len = this.acceptedTypes.length; i < len; i++){
			if(this.acceptedTypes[i] == ds.type){ return true; }
		}
		return false;
	},

	showFeedback: function(dt){
	},

	hideFeedback: function(dt){
	},

	getNodeUnderMouse: function(){
		return this.nodeUnderMouse;
	},

	shouldShowCaret: function(e){
		// check the node under the mouse cursor
		var node = this.nodeUnderMouse;
		if(!node){ return false; }
		if(dojo.style(node, "position") != "static"){ return false; }

		// check the drag source
		var ds = e.dragSource;
		if(!ds.targetShouldShowCaret || ds.positioning != "static"){
			return false;
		}

		return true;
	},

	showCaret: function(e){
	return;
		if(!this.nodeUnderMouse){ return; }
		this.createCaret();
		var caret = this.caret;
		var node = this.nodeUnderMouse;
		var gravity = dojo.html.gravity(node, e);
		var display = dojo.style(node, "display");
		if(display == "block"){
			caret.style.width = node.offsetWidth + 3 + "px";
			caret.style.height = "1px";
			caret.style.borderWidth = "0px 0px 2px 0px";
			if(gravity & dojo.html.gravity.NORTH){
				caret.style.top = node.offsetTop + "px";
			}else{
				caret.style.top = node.offsetTop + node.offsetHeight + "px";
			}
			caret.style.left = node.offsetLeft + "px";
			this.vertical = false;
		}else{
			caret.style.width = "1px";
			caret.style.height = node.offsetHeight + 3 + "px";
			caret.style.borderWidth = "0px 2px 0px 0px";
			caret.style.top = node.offsetTop + "px";
			if(gravity & dojo.html.gravity.WEST){
				caret.style.left = node.offsetLeft + "px";
			}else{
				caret.style.left = node.offsetLeft + node.offsetWidth + "px";
			}
			this.vertical = true;
		}
		caret.style.display = "";
	},

	hideCaret: function(){
		if(this.caret){
			this.caret.style.display = "none";
		}
	},

	createCaret: function(){
	alert("createCaret");
		if(!this.caret){
			var caret = this.caret = dojo.doc.createElement("DIV");
			caret.className = "btpCaret";
			caret.style.zIndex = 1000;
			caret.style.position = "absolute";
			caret.style.fontSize = "1px";
			caret.style.borderStyle = "solid";
			caret.style.borderColor = "black";
			caret.style.display = "none";
			this.offsetParent.appendChild(this.caret);
		}
	}
});
