/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.EditArea");

dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("btp.widget.tool.EditDomain");
dojo.require("btp.widget.data.dataStub");
dojo.require("btp.widget._editArea.componentInfo");
dojo.require("btp.widget._editArea.focus");
dojo.require("btp.widget._editArea.canvas");
dojo.require("btp.widget._editArea.event");
dojo.require("btp.widget._editArea.snap");
dojo.require("btp.widget._editArea.outline");
dojo.require("btp.widget._editArea.provider");

dojo.declare(
	"btp.widget.EditArea",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	isEditor: true, // indicates this is an editor

	src: '',
	label: '',
	componentInfoId: '',
	commands: '',
	target: '',
	actionInvokedFilter: '',
	paletteSelectionChangedFilter: '*',
	propertyChangedFilter: '*',

	freeLayoutMode: true,

	// subclass may want to override these to configure the EditArea
	defaultHelperClasses: null, // array of class names
	actionClasses: null, // array of class names
	ioClass: null, // class name
	loaderClass: null, // class name
	generatorClass: null, // class name
	resizeTrackerClass: "btp.widget.tool.ResizeTracker",
	dragMoveTrackerClass: "btp.widget.tool.DragGhostMoveTracker",

    __editArea: null,
	_editDomain: null,
	_moveTarget: null,
	_connectors: null,
	_events: [
		"MouseDown", "MouseUp", "MouseMove", "MouseOver", "MouseOut",
		"Click", "DblClick", "KeyDown", "KeyUp",
		"DragStart", "DragOver", "DragMove", "DragEnd", "DropStart",
		"Drop", "DropEnd", "DragOut"
	],
	
	_designTimeMethods: null,

	constructor: function(){
		this.defaultHelperClasses = [];
		this.actionClasses = [];
		this._contentOutlinePages = {};
		this._connectors = [];
	},

	destroy: function(){
		//TODO
		this.inherited("destroy", arguments);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){if(child.startup){child.startup();}});
		this._started = true; // By doing this, startup() of dynamically added child will be called.
	},

	postCreate: function(){
		this.containerNode = this.domNode;
		this.domNode.className = "btpEditArea";
		this.domNode.setAttribute("tabIndex", "0");
		this.__editArea = this;
		this.hlp_dropTargets = ["component"];
//		this._editDomain = new btp.widget.tool.EditDomain("btp.widget.tool.PortalLayoutTool");
//		this._editDomain = new btp.widget.tool.EditDomain("btp.widget.tool.DefaultSelectionTool");
		if(this.commands) //TODO: Do not omit curly braces, '{' and '}'.
			this.actionInvokedFilter = this.commands;

		this.initCanvas();
		for(var i = 0, len = this._events.length; i < len; i++){
			var eventName = this._events[i];
			var handlerName = "onEa"+eventName+"Handler";
			var lowerName = eventName.charAt(0).toLowerCase() + eventName.substring(1);
			this[handlerName] = this._createHandler(lowerName);
			if(eventName.charAt(0) != "D"){
				this._connectors.push(dojo.connect(this.domNode, "on"+eventName.toLowerCase(), this, handlerName));
			}
		}

		dojo.subscribe("btp:ActionInvoked", this, "dispatchCommand");
		dojo.subscribe("btp:PaletteSelectionChanged", this, "paletteSelectionChanged");
		dojo.subscribe("btp:PreferenceChanged", this, "preferenceChanged");
		dojo.subscribe("btp:PropertyChanged", this, "propertyChanged");

		// load contents
		this.loadContents();
	},

	_createHandler: function(lowerName){
		return function(e){
			this.getEditDomain()[lowerName](e, this);
		};
	},

	initCanvas: function(){
		if(btpConfig.useDropTarget){
			var _this = this;
			dojo.addOnLoad(function(){ // lazy initializing to improve startup performance.
				dojo["require"]("btp.widget.helper.dropHelper");
				btp.weakMixin(_this, btp.widget.helper.dropHelper);
				_this.hlp_setupDropTarget();
			});
		}
	},

	getEditDomain: function(){
		return this._editDomain;
	},

	setEditDomain: function(domain){
		this._editDomain = domain;
	},

	dispatchCommand: function(message){
		if(!btp.matchFilter(message, this.actionInvokedFilter)){
			return; // filter out the message
		}
		if(!btp.isShowing(this)){ return; }

		for(var i = 0; i < this.actionClasses.length; i++){
			btp.mixinExtension(this, this.actionClasses[i]);
		}
		this.actionClasses = [];

		if(this.ioClass){
			btp.mixinExtension(this, this.ioClass);
			this.ioClass = null;
		}

		if(!this[message.command]){ return; }
		this[message.command](message);
	},

	// subclass may want to override this
	loadContents: function(){
		var url = this.src;
    	if(!url){ return false; }
		if(url.indexOf("://") == -1){
    		url = dojo.moduleUrl("btp", url).toString();
    	}
		var contents = dojo._getText(url, true);
		if(this.load(contents)){
			this.setTabTitle(url);
		}
	},

	getDragTracker: function(){
		return this.dragMoveTrackerClass;
	},

	setTabTitle: function(url){
		if(!url){ return; }
		var parent = this.getParent();
		if(!parent)
			return; // this EditArea was instantiated dynamically
		var tabPanel;
		var pane;
		if(parent.declaredClass == "btp.widget.ToolTabPanel"){
			tabPanel = parent;
			pane = this;
		}else if(parent.declaredClass == "btp.widget.EditorFrame" && parent.getParent().declaredClass == "btp.widget.ToolTabPanel"){
			tabPanel = parent.getParent();
			pane = parent;
		}else
			return;
		pane.label = btp.createLabelStringFromUrl(url);
		pane.tooltip = url;
		if(pane.tab)
			tabPanel.setTitle(pane.tab, pane.label, url);
	},

	activate: function(force){
/*
		if(!force && btp.isShowing(this)) //TODO: Do not omit curly braces, '{' and '}'.
			return;
		var page = this;
		var tabPanel = this.parent;
		if(tabPanel.widgetType == "BottomTabPanel"){
			tabPanel.selectTab(page.tab);
			page = tabPanel;
			tabPanel = tabPanel.parent;
		}
		if(tabPanel.widgetType == "TabPanel") //TODO: Do not omit curly braces, '{' and '}'.
			tabPanel.selectTab(page.tab);
//		this.domNode.focus(); // can't do this. this removes focus from a clicked component.
*/
	},

	loadComponent: function(widgetName){
		var info = this.getComponentInfo(widgetName); // Ex. widgetName: "btp.widget.Button"
		dojo["require"](widgetName);
		var impl = dojo.getObject(widgetName);
		if(!impl.__loadComponentOk){
			impl.__loadComponentOk = true;
			if(info.helper){ // add helpers to the widget prototype
				for(var i = 0, len = info.helper.length; i < len; i++){
					dojo["require"](info.helper[i]);
					var obj = dojo.getObject(info.helper[i]);
					dojo.extend(impl, obj);
				}
			}
		}
		return impl;
	},

	loadComponents: function(rootNode){
		var list = dojo.query('[dojoType]', rootNode);
		for(var i = 0, len = list.length; i < len; i++){
			this.loadComponent(list[i].getAttribute("dojoType"));
		}
	},

	createComponent: function(/*String*/name, /*Object?*/props){
		var info = this.getComponentInfo(name); // Ex. name: "btp.widget.Button"

		if(!this.hlp_getEditArea){
			dojo["require"]("btp.widget.helper.baseHelper");
			btp.weakMixin(this, btp.widget.helper.baseHelper);
		}

		// remove component index. (btp.widget.Button:1 -> btp.widget.Button)
		var widgetName = name.replace(/:.*/, "");

		var impl = this.loadComponent(widgetName);

		// set default attribute values if any
		if(!props){
			props = {};
		}
		var attrs = info.attribute;
		for(var i = 0, len = attrs ? attrs.length : 0; i < len; i++){
			var attr = attrs[i];
			if(props[attr.name]){
				continue; // respect the value of props
			}
			if(typeof(attr.value) != "undefined"){
				var type = typeof(impl.prototype[attr.name]);
				if(type == "number"){
					props[attr.name] = attr.value - 0;
				}else if(type == "boolean"){
					props[attr.name] = (attr.value == "true");
				}else if(!attr.value){
					props[attr.name] = "";
				}else{
					// get localized string
					dojo["require"]("btp.i18n");
					var s = btp.i18n.gets(attr.value, "component_"+name.replace(/\./g, "_")+"_"); // Ex. "component_com_ibm_button_Push Button"
					if(s == attr.value){
						s = btp.i18n.gets(attr.value, "component_"); // Ex. "component_Push Button"
					}
					props[attr.name] = s;
				}
			}
		}

		var widget = new impl(props);
		widget.__component_info = info;
		widget.btpComponentName = name; // name with component index.

		this.setupWidgetForEdit(widget);
		if(props.__style){
			for(var property in props.__style){
				btp.setStyleValue(widget, property, props.__style[property]);
			}
		}

		return widget;
	},

	setupWidgetsForEdit: function(widget){
		this.setupWidgetForEdit(widget);
		var children = widget.getChildren();
		for(var i = 0, len = children.length; i < len; i++){
			this.setupWidgetForEdit(children[i]);
		}
	},

	setupWidgetForEdit: function(widget){
		btp.weakMixin(widget, dijit._Contained.prototype);
		this.mixinDesignTimeObjs(widget);
		this.getComponentInfo(widget); // to assign info to each widget
		widget.hlp_setEditArea(this);
		widget.hlp_initLiveWidget(this);
	},

	getDesignTimeObjs: function(widget){
		if(!this._designTimeMethods){
			this._designTimeMethods = {};
			for(var i = 0; i < this.defaultHelperClasses.length; i++){
				var helper = this.defaultHelperClasses[i];
				dojo["require"](helper);
				var props = dojo.getObject(helper);
				dojo.mixin(this._designTimeMethods, props);
			}
		}
		return this._designTimeMethods;
	},

	mixinDesignTimeObjs: function(widget){
		var cls = dojo.getObject(widget.declaredClass);
		var mp = cls.prototype; // my prototype
		var sp = cls.superclass; // superclass's prototype
		var objs = this.getDesignTimeObjs(widget);
		for(var x in objs){
			if(x == "destroy"){
				// if the helper object has the destroy method,
				// it should be called before the widget.destroy() is called.
				if(!widget.__destructors){
					widget.__destructors = [];
				}
				widget.__destructors.push(objs.destroy);
			}else if(!widget[x] || (mp[x] && mp[x] == sp[x])){
				// x doesn't exist or x is an inherited property.
				// If the widget has x, do not mix-in the LiveWidget's x.
				// But if x is an inherited property such as addChild, override it with LiveWidget's addChild.
				// Of course, if the widget has its own addChild, it should not be overridden.
				widget[x] = objs[x];
			}
		}
		if(widget.__destructors){
			if(widget.destroy){
				widget.__destructors.push(widget.destroy);
			}
			widget.destroy = function widget_destroy(){
				alert("destroy");
				for(var i = 0, len = widget.__destructors.length; i < len; i++){
					widget.__destructors[i]();
				}
			}
		}
	},

	getFreeLayoutMode: function(){
		return this.freeLayoutMode;
	},

	getMoveTarget: function(){
		if(!this._moveTarget){
			var moveTarget = this._moveTarget = dojo.doc.createElement("DIV");
			// On IE, rubber band doesn't work smoothly if you set width and height to 0px
			// or set style.display to "none".
			moveTarget.className = "moveTarget";
			moveTarget.style.fontSize = "1px";
			moveTarget.style.width = "1px";
			moveTarget.style.height = "1px";
			moveTarget.style.position = "absolute";
		}
		return this._moveTarget;
	},

	createKnob: function(type){
		if(typeof(type) == "object" && type.tagName){
			return type;
		}else if(type == "blackSquare"){
			var div = dojo.doc.createElement("DIV");
			div.className = "resizeKnob";
			div.style.width = "4px";
			div.style.height = "4px";
			div.style.fontSize = "1px";
			div.style.backgroundColor = "black";
			div.style.position = "absolute";
			div.style.border = "1px solid white";
			div.style.zIndex = "1100";
			return div;
		}else if(type == "whiteCircle"){
			var img = dojo.doc.createElement("IMG");
			img.className = "resizeKnob";
			img.alt = "";
			img.src = dojo.moduleUrl("btp", "widget/templates/EditArea/white-circle-knob.gif")+""
			img.style.zIndex = "1100";
			img.style.position = "absolute";
			return img;
		}else if(type.match(/\.(gif|jpg|jpeg|png)$/i)){
			var img = dojo.doc.createElement("IMG");
			img.className = "resizeKnob";
			img.alt = "";
			img.src = dojo.moduleUrl("btp", type)+""
			img.style.zIndex = "1100";
			img.style.position = "absolute";
			return img;
		}
		return null; // never reached
	}
});
dojo.extend(btp.widget.EditArea, btp.widget.data.dataStub);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.componentInfo);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.focus);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.canvas);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.event);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.snap);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.outline);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.provider);
