/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.Model");

dojo.require("dijit._Widget");

dojo.declare(
	"btp.widget.Model",
	dijit._Widget,
{
	src: '',
	format: '', // "json" or "xml"(default)

	ml: '',
	dom: null,
	doc: null,
	json: null,
	parsedSrc: null,

	loadXMLString: function(value){
		var xml = null;
		if((typeof dojo.global["DOMParser"]) !== "undefined"){
			var parser = new DOMParser();
			xml = parser.parseFromString(value, "text/xml");
		}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
			xml = new ActiveXObject("Microsoft.XMLDOM");
			xml.async = false;
			xml.validateOnParse = false;
			xml.resolveExternals = false;
			if(!xml.loadXML(value)){
				var args = new Array;
				args[0] = xml.parseError.line;
				args[1] = xml.parseError.linepos;
				args[2] = xml.parseError.srcText;
				args[3] = xml.parseError.reason;
				if(args[0] == 0 && args[1] == 0){
//					return null; // maybe the file is empty
				}
				var result = "Xml error at Line: " + args[0] + 
					" Pos: " + args[1] + 
					" \nSource: " + args[2] +
					" \nReason: " + args[3];
				alert(result);
				throw new Error(result);
			}else{
				xml.setProperty("SelectionLanguage", "XPath");
			}
		}
		return xml;
	},

	loadXML: function(url){
		if(!url){ return null; }
		var xml = null;
		if((typeof dojo.global["DOMParser"]) !== "undefined"){
			var req = new XMLHttpRequest();
			req.open("GET",url,false);
			req.overrideMimeType("text/xml");
			req.send(null);
			xml = req.responseXML;
		}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
			xml = new ActiveXObject("Microsoft.XMLDOM");
			xml.async = false;
			xml.validateOnParse = false;
			xml.resolveExternals = false;
			if(!xml.load(url)){
				var args = new Array;
				args[0] = xml.parseError.line;
				args[1] = xml.parseError.linepos;
				args[2] = xml.parseError.srcText;
				args[3] = xml.parseError.reason;
				if(args[0] == 0 && args[1] == 0){
//					return null; // maybe the file is empty
				}
				var result = "Xml error at Line: " + args[0] + 
					" Pos: " + args[1] + 
					" \nSource: " + args[2] +
					" \nReason: " + args[3] +
					" \nUrl: " + url;
				alert(result);
				throw new Error(result);
			}else{
				xml.setProperty("SelectionLanguage", "XPath");
			}	
		}
		return xml;
	},

	getInstanceDocument: function(){
		if(!this.src){ return null; }
		var prev = this.parsedSrc;
		this.parsedSrc = this.parseSrc();
		if(this.parsedSrc != prev){
			this.dom = null;
			this.json = null;
		}
		if(this.json){
			return this.json;
		}else if(this.format == "json" || this.parsedSrc.match(/\.json$/)){
			this.doc = this.json = this._getJsonObject();
			return null;
		}

		if(this.dom){
			return this.dom;
		}else{
			this.dom = this.loadXML(dojo.moduleUrl("btp", this.parsedSrc).uri);
			if(this.dom != null){
				this.doc = this.dom.documentElement;
			}
			if(navigator.appName == "Netscape"){
				this._removeWhiteSpaceNodes(this.dom);
			}
			return this.dom;
		}
	},

	_removeWhiteSpaceNodes: function(xml){
		for(var i = 0; i < xml.childNodes.length; i++){
			var child = xml.childNodes[i];
			if(child.nodeType == 1){
				this._removeWhiteSpaceNodes(child);
			}
			if(child.nodeType == 3 && child.nodeValue.match(/^\s*$/)){
				xml.removeChild(child);
				i--;
			}
		}
	},

	getDocumentElement: function(){
		if(!this.src){ return null; }
		this.getInstanceDocument();
		return this.doc;
	},

	selectSingleNode: function(ref){
		var root = this.getDocumentElement();
		if(root == null){ return null; }
		return this.evaluateXPath(root, ref)[0];
	},

	selectSingleNodeFromJson: function(ref){
		var cache = window.haleModelCache ? window.haleModelCache[this.src] : null;
		if(cache){
			this.json = cache;
		}else {
			var root = this.getDocumentElement();
			if(root == null){ return null; }
			if(!this.json){
				this._convertToJson();
			}
		}
		if(!ref || ref == "/"){
			return this.json;
		}else if(ref == "."){
			for(var i in this.json){
				return this.json[i];
			}
			return null;
		}else if(ref.charAt(0) == "/"){ // ex. "/root/users"
			var arr = ref.split("/");
			var node = this.json;
			for(var i = 1, len = arr.length; i < len; i++){
				node = node[arr[i]];
				if(!node){ return null; }
			}
			return node;
		}
		return null;
	},

	selectNodes: function(nodeset){
		var root = this.getDocumentElement();
		if(root == null){ return []; }
		return this.evaluateXPath(root, nodeset);
	},

	selectNodesFromJson: function(nodeset){
		var cache = window.haleModelCache ? window.haleModelCache[this.src] : null;
		if(cache){
			this.json = cache;
		}else {
			var root = this.getDocumentElement();
			if(root == null){ return null; }
			if(!this.json){
				this._convertToJson();
			}
		}
		if(!nodeset || nodeset == "/"){
			return [this.json];
		}else if(nodeset == "."){
			for(var i in this.json){
				return [this.json[i]];
			}
			return null;
		}else if(nodeset.charAt(0) == "/"){ // ex. "/root/users"
			var arr = nodeset.split("/");
			var node = this.json;
			for(var i = 1, len = arr.length; i < len; i++){
				node = node[arr[i]];
				if(!node){ return null; }
			}
			return dojo.isArray(node) ? node : [node];
		}
		return null;
	},

	evaluateXPath: function(node, expression){
		var arr = new Array();	
		if(navigator.appName == "Netscape"){
			var xpath = new XPathEvaluator();
			var nsnode = node.ownerDocument.documentElement;
			var resolver = xpath.createNSResolver(nsnode);
			var result = xpath.evaluate(expression, node, resolver, 0, null);
			while (res = result.iterateNext()){
				arr.push(res);
			}
		}else{
			var result = node.selectNodes(expression); 
			for(var i=0; i < result.length; ++i){
				arr.push(result[i]); 
			}	
		}
		return arr;
	},

	_getJsonObject: function(){
		var text = dojo._getText(dojo.moduleUrl("btp", this.parsedSrc).uri);
		var obj = eval('(' + text + ')');
		return obj;
	},

	_convertToJson: function(){
		if(this.json || this.format == "json" || this.parsedSrc.match(/\.json$/)){
			return;
		}
		var root = this.getDocumentElement();
		if(root == null){ return; }
		var json = {};
		json[root.nodeName] = this._convertToJson_1(root);
		this.json = json;
	},

	_convertToJson_1: function(node){
		var json = null;

		for(var i = 0, len = node.attributes.length; i < len; i++){
			if(!json) {
				json = {};
			}
			var attr = node.attributes[i];
			json["@"+attr.nodeName] = attr.nodeValue;
		}

		var len = node.childNodes.length;
		if(len == 1 && (node.firstChild.nodeType == 3 || node.firstChild.nodeType == 4)){
			// eg. <name>foo</name>
			if(!json){
				json = node.firstChild.nodeValue;
			}else{
				json["#text"] = node.firstChild.nodeValue;
			}
		}
		else{
			for(var i = 0, len; i < len; i++){
				var child = node.childNodes[i];
				if(child.nodeType != 1){
					continue;
				}
				if(!json){
					json = {};
				}
				var ret = this._convertToJson_1(child);
				if(!json[child.nodeName]){
					json[child.nodeName] = ret;
				}else{
					var tmp = json[child.nodeName];
					if(typeof(tmp) != "string" && typeof(tmp.length) != "undefined"){ // tmp is an array
						tmp.push(ret);
					}else{
						json[child.nodeName] = [tmp,ret];
					}
				}
			}
		}
		return json;
	},

	clear: function(){
		this.src = null;
		this.parsedSrc = null;
		this.dom = null;
		this.json = null;
	},

	setSrc: function(src){
		this.clear();
		this.src = src;
	},

	parseSrc: function(){
		if(!this.src || this.src.indexOf('$') == -1){
			return this.src;
		}
		var src = this.src;
		var val = "";
		while(true){
			var idx = src.indexOf("${");
			if(idx == -1){
				val += src;
				break;
			}
			var head = src.substring(0, idx);
			var tail = src.substring(idx + 2);
			val += head;
			idx = tail.indexOf("}");
			if(idx == -1){ break; }
			var ref = tail.substring(0, idx);
			var tail = tail.substring(idx + 1);
			var widget = dijit.byId(ref);
			if(widget){
				val += widget.value;
			}
			src = tail;
		}
		return val;
	},

	fireModelChanged: function(triggerWidget){
		var message = {
			type: "ModelChangedNotification",
			src: this,
			triggerWidget: triggerWidget
		};
		dojo.event.topic.publish("btp:ModelChanged", message);
	}
});
