/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.OutlineView");

dojo.require("dijit._Container");

dojo.declare(
	"btp.widget.OutlineView",
	[dijit._Widget, dijit._Contained, dijit._Container],
{
	target: '',
	propertyChangedFilter: '',
	selectionChangedFilter: '',

	editArea: null,
	
	postCreate: function(){
		this.domNode.className = "btpOutlineView btpSimplePane";
		if(this.target){
			this.propertyChangedFilter = this.target;
			this.selectionChangedFilter = this.target;
		}

		this._topics = [
			dojo.subscribe("btp:PropertyChanged", this, "propertyChanged"),
			dojo.subscribe("btp:SelectionChanged", this, "selectionChanged"),
			dojo.subscribe("btp:ActiveEditorChanged", this, "editorActivated"),
			dojo.subscribe("btp:InputChanged", this, "editorActivated")
		];
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	loadContentOutlinePage: function(){
		var contentOutlinePage = this.editArea.getContentOutlinePage()
			|| this.editArea.createContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.setOutlineView(this);
			contentOutlinePage.createOutlinePage();
			contentOutlinePage.refreshOutline();
		}
	},

	// ======== Topic Subscribers ====================================
	editorActivated: function(message){
		var oldPage = this.editArea && this.editArea.getContentOutlinePage();
		if(message.type == "ActiveEditorChangedNotification"){
			if(this.editArea == message.editor){ return; } // already activated
			this.editArea = message.editor; // tab has been changed
			if(oldPage){ oldPage.detachOutline(); }
			var newPage = this.editArea.getContentOutlinePage();
			if(newPage){
				if(newPage.attachOutline()){
					return; // cache exists
				}
			}
		}else if(message.type == "InputChangedNotification"){
			this.editArea = message.src; // document has been loaded
			if(oldPage){ oldPage.detachOutline(); }
		}

		if(this.editArea.getChildren().length == 0){
			// If the editor starts up from scratch, we do not load ContentOutlinePage
			// until propertyChanged or selectionChanged is called.
		}else{
			this.loadContentOutlinePage();
		}
	},

	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}

		var contentOutlinePage = this.editArea.getContentOutlinePage();
		if(!contentOutlinePage){
			this.loadContentOutlinePage();
		}
		contentOutlinePage = this.editArea.getContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.updateOutline(message);
		}
	},

	selectionChanged: function(message){
		if(!btp.matchFilter(message, this.selectionChangedFilter)){
			return; // filter out the message
		}

		var contentOutlinePage = this.editArea.getContentOutlinePage();
		if(!contentOutlinePage && message.selection){
			this.loadContentOutlinePage();
		}
		contentOutlinePage = this.editArea.getContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.selectOutline(message.selection);
		}
	}
});
