/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.ScrollWindow");

dojo.require("dijit._Widget");
dojo.require("btp.dnd.DragMoveSource");
dojo.require("dojox.gfx._base");

dojo.declare(
	"btp.widget.ScrollWindow",
	dijit._Widget,
{
	target: "",
	
	dragSource: null,
	
	parentDiv: null,
	
	_conn: null,
	
	_dragging: false,
	
	postCreate: function(){
		this.domNode.className = "btpScrollWindow";
		this.dragSource = new btp.dnd.DragMoveSource(this.domNode);
		this.dragSource.constrainTo();
		dojo.connect(this.dragSource, "onDragMove", this, "_onWindowScroll");
		dojo.connect(this.dragSource, "onDragEnd", this, "_onWindowScrollEnd");
	},
	
	startup: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget) return;
		var layoutParent = btp.getLayoutParent(this.domNode);
		if(layoutParent) {
			dojo.connect(layoutParent, "layout", this, "_resizeWindow");
		}
		if(targetWidget.adjustCanvasSize){
			dojo.connect(targetWidget, "adjustCanvasWidth", this, "_resizeWindow");
			dojo.connect(targetWidget, "adjustCanvasHeight", this, "_resizeWindow");
		}
		this._resizeWindow();
		
		dojo.connect(targetWidget.domNode, "onscroll", this, "_onTargetScroll");
		
		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		this.dragSource.setOffsetX(borderLeftWidth)
		this.dragSource.setOffsetY(borderTopWidth);
	},
	
	constrainTo: function(div){
		this.parentDiv = div;
		this.dragSource.constrainTo(div);
	},
	
	_resizeWindow: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget) return;		
		
		var div = this.parentDiv ? this.parentDiv : this.domNode.parentNode;
		var scrollArea = targetWidget.domNode;
		
		var propX = 1.0 * scrollArea.clientWidth / scrollArea.scrollWidth;
		var propY = 1.0 * scrollArea.clientHeight / scrollArea.scrollHeight;
		
		var width = Math.floor(div.clientWidth * propX);
		var height = Math.floor(div.clientHeight * propY);
		
		this.domNode.style.width = width + "px";
		this.domNode.style.height = height + "px";
		
		this._adjustWindowPos();
	},
	
	_onWindowScroll: function(e) {
		this._dragging = true;
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return; }
		
		var div = e.target;
		if(div != this.domNode){ return;}
		var parentDiv = this.parentDiv ? this.parentDiv: this.domNode.parentNode;

		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		
		var x = div.offsetLeft - parentDiv.offsetLeft + borderLeftWidth;
		var y = div.offsetTop - parentDiv.offsetTop + borderTopWidth;
		
		var scrollArea = targetWidget.domNode;
		var propX = 1.0 * scrollArea.scrollWidth / parentDiv.offsetWidth;
		var propY = 1.0 * scrollArea.scrollHeight / parentDiv.offsetHeight;

		scrollArea.scrollLeft = Math.floor( x * propX );		
		scrollArea.scrollTop = Math.floor( y * propY );
	},
	
	_onWindowScrollEnd: function(e) {
		this._dragging = false;
	},
	
	_adjustWindowPos: function(){
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return; }
		
		var parentDiv = this.parentDiv ? this.parentDiv : this.domNode.parentNode;
		var target = targetWidget.domNode;
		var x = target.scrollLeft;
		var y = target.scrollTop;
		x *= parentDiv.offsetWidth / target.scrollWidth;
		y *= parentDiv.offsetHeight / target.scrollHeight;
		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		this.domNode.style.left = Math.floor(x) + parentDiv.offsetLeft - borderLeftWidth + "px";
		this.domNode.style.top = Math.floor(y) + parentDiv.offsetTop - borderTopWidth + "px";
	},
	
	_onTargetScroll: function(e) {
		if(this._dragging) return;
		this._adjustWindowPos();
	}
});
