/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.SimpleTabPanel");

dojo.require("btp.widget._TabPanelBase");

dojo.declare(
	"btp.widget.SimpleTabPanel",
	btp.widget._TabPanelBase,
{
	tailTab: null,

	lightGray: "#DEDFDE",
	darkGray: "#ACA899",

	topicNamePrefix: "SimpleTab",

	postCreate: function(){
		this.domNode.className = "tabPanel simpleTabPanel";
		var header = this.tabHeaderNode;
		this.tabs = [];

		this.childWidgets = [
			{domNode: this.tabHeaderNode, layoutAlign: "top"},
			{domNode: this.containerNode, layoutAlign: "client"}
		];

		// to keep height even when there are no tabs
		var dummy = dojo.doc.createElement("DIV");
		dummy.className = "tabPanelDummyTab";
		header.appendChild(dummy);

		// to draw an lower line on the right side
		var tab = this.tailTab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.style.width = "20px";
		tab.style.borderTopColor = "white";
		tab.style.borderLeftColor = "white";
		tab.style.borderRightColor = "white";
		header.appendChild(tab);

		btp.widget.SimpleTabPanel.superclass.postCreate.apply(this, arguments);
	},

	setupChild: function(/*ContentPane*/ page){
		var header = this.tabHeaderNode;
		var tab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.page = page; // [custom property]
		page.tab = tab; // [custom property]
		this.tabs.push(tab);

		page.domNode.style.position = "relative";
		page.domNode.style.display = "block";

		// Title Node
		var titleNode = dojo.doc.createElement("DIV");
		titleNode.className = "tabPanelTabTitle";
		titleNode.tabIndex = 0;
		var label = page.label; //TODO: hale.i18n.gets(page.label, this.i18nPrefix);
		titleNode.innerHTML = label;
		if(page.tooltip){
			tab.title = page.tooltip;
		}
		tab.appendChild(titleNode);
		tab.titleNode = titleNode; // [custom property]

		header.insertBefore(tab, this.tailTab);

		tab._connectors = [];
		tab._connectors.push(dojo.connect(tab, "onclick", this, "onTabClick"));
		tab._connectors.push(dojo.connect(tab, "ondblclick", this, "onTabDblClick"));
		tab._connectors.push(dojo.connect(tab, "onkeydown", this, "onTabKeyDown"));

		page.domNode.style.display = "none";
	},

	// override dijit._Container.removeChild
	removeChild: function(/*Widget*/widget){
		var page = widget;
		var tab = widget.tab;
		for(var i = 0, len = this.tabs.length; i < len; i++){
			if(this.tabs[i] == tab){
				this.tabs.splice(i, 1); // remove tab from tabs
				break;
			}
		}
		if(this.tabs.length == 0){
			this.unhilightTab(tab);
		}
		this.tabHeaderNode.removeChild(tab);
		if(this.selectedPage == page){
			this.selectedPage = null;
		}
		dojo.forEach(tab._connectors, dojo.disconnect);
		page.destroy();
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "none";
			this.edgeImages[0].style.display = "none";
			this.edgeImages[1].style.display = "none";
			this.containerNode.style.backgroundColor = "#EFEBDE";
			this.fireTabChanged(null);
		}else{
			this.selectTab(this.tabs[0]);
		}
	},

	hilightTab: function(tab, focus){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/simple-tab-bg.gif")+")";
		tab.style.borderTopColor = this.darkGray;
		tab.style.borderBottomColor = "white";
		tab.style.borderLeftColor = this.darkGray;
		tab.style.borderRightColor = this.darkGray;
		this.isActive = true;

		// focus (necessary for keyboard navigation)
		if(focus){
			tab.titleNode.focus();
		}

		// set tabIndex only for the selected tab
		for(var i = 0, len = this.tabs.length; i < len; i++){
			this.tabs[i].titleNode.tabIndex = (this.tabs[i] == tab) ? 0 : -1;
		}
	},

	unhilightTab: function(tab){
		this.hilightTab(tab);
	},

	grayoutTab: function(tab){
		tab.style.backgroundImage = "";
		tab.style.borderColor = this.lightGray;
	}
});
