/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.SimpleTree");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");

dojo.declare(
	"btp.widget.SimpleTree",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	selectedNode: null,

	postCreate: function(){
		this.domNode.className = "btpSimpleTree";
		this.connect(this.domNode, "onkeydown", "onKeyDown");
		this.connect(this.domNode, "onmousedown", "onMouseDown");
	},

	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		widget.tree = this;
		btp.widget.SimpleTree.superclass.addChild.apply(this, arguments);
		this.fireTreeNodeAdded(widget);
	},

	onKeyDown: function(e){
		var elem = e.target; // clicked element
		if(elem.tagName != "SPAN"){ return; }
		var node = dijit.byNode(elem.parentNode);
		if(e.keyCode == dojo.keys.UP_ARROW){
			var prev;
			if(node.getPreviousSibling()){
				var prev = node.getPreviousSibling();
				if(prev.containerNode.style.display != "none" && prev.hasChildren()){
					var children = prev.getChildren();
					prev = children[children.length - 1];
				}
			}else if(node.getParent()){
				prev = node.getParent();
			}
			if(prev && prev.declaredClass == "btp.widget.SimpleTreeNode"){
				this.select(prev);
				prev.setFocus();
			}
		}else if(e.keyCode == dojo.keys.DOWN_ARROW){
			var next;
			if(node.containerNode.style.display != "none" && node.hasChildren()){
				next = node.getChildren()[0];
			}else if(node.getNextSibling()){
				next = node.getNextSibling();
			}else if(node.getParent()){
				next = node.getParent().getNextSibling();
			}
			if(next){
				this.select(next);
				next.setFocus();
			}
		}else if(e.keyCode == dojo.keys.LEFT_ARROW){
			node.collapse();
		}else if(e.keyCode == dojo.keys.RIGHT_ARROW){
			node.expand();
		}
	},

	onMouseDown: function(e){
		var elem = e.target; // clicked element
		var node;
		if(elem.className == "btpTreeLabel" || elem.className == "btpTreeFolderIcon"){
			node = dijit.byNode(elem.parentNode.parentNode);
		}
		if(!node){ return; }
		this.select(node);
		node.setFocus();
	},

	select: function(/*SimpleTreeNode*/node){
		if(node == this.selectedNode){ return; }
		if(this.selectedNode){
			this.selectedNode.deselect();
		}
		if(!node){ return; }
		for(var n = node.getParent(); n.declaredClass == "btp.widget.SimpleTreeNode"; n = n.getParent()){
			n.expand();
		}
		node.select();
		this.selectedNode = node;
		this.fireTreeNodeSelected(node);
	},

	deselectAll: function(){
		if(this.getSelectedNode()){
			this.getSelectedNode().deselect();
		}
	},

	getSelectedNode: function(){
		return this.selectedNode;
	},

	fireTreeNodeSelected: function(node){
		var message = {
			type: "TreeNodeSelectedNotification",
			src: this,
			selection: node
		};
		dojo.publish("btp:TreeNodeSelected", [message]);
	},

	fireTreeNodeAdded: function(node){
		var message = {
			type: "TreeNodeAddedNotification",
			src: this,
			node: node
		};
		dojo.publish("btp:TreeNodeAdded", [message]);
	},

	onBeforeExpand: function(/*SimpleTreeNode*/node){
		// Called before a node is expanded. You may want to override this or connect to this.
	}
});

dojo.declare(
	"btp.widget.SimpleTreeNode",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	label: "",
	iconImage: "",
	isExpandable: "",
	tooltip: "",

	tree: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/SimpleTree/"),
	plusIcons: ["plus0.gif", "plus2.gif", "plus12.gif", "plus23.gif", "plus123.gif"],
	minusIcons: ["minus0.gif", "minus2.gif", "minus12.gif", "minus23.gif", "minus123.gif"],
	emptyIcons: ["empty0.gif", "empty2.gif", "empty12.gif", "empty23.gif", "empty123.gif", "empty13.gif"],
	folderIcon: "folderXP.gif",
	fileIcon: "file.gif",

	constructor: function(params){
		dojo.mixin(this, params);
	},

	postCreate: function(){
		var node = this.domNode;
		node.className = "btpSimpleTreeNode";

		var sectionImg = this.sectionImg = dojo.doc.createElement("IMG");
		sectionImg.className = "btpTreeSectionIcon";
		if(this.isExpandable){
			sectionImg.src = this.imgDir + this.plusIcons[0];
		}else{
			sectionImg.src = this.imgDir + this.emptyIcons[0];
		}
		sectionImg.align = "middle";
		node.appendChild(sectionImg);
		this.connect(sectionImg, "onclick", "changeTreeState");
	
		var contentNode = this.contentNode = dojo.doc.createElement("SPAN");
		contentNode.className = "btpTreeContentNode";
		node.appendChild(contentNode);

		var folderImg = this.folderImg = dojo.doc.createElement("IMG");
		folderImg.className = "btpTreeFolderIcon";
		if(this.iconImage){
			folderImg.src = (this.iconImage.indexOf("/") != -1) ? this.iconImage : (this.imgDir + this.iconImage);
		}else{
			folderImg.src = this.imgDir + this.folderIcon;
		}
		folderImg.align = "middle";
		contentNode.appendChild(folderImg);
	
		var labelNode = this.labelNode = dojo.doc.createElement("SPAN");
		labelNode.className = "btpTreeLabel";
		labelNode.tabIndex = "0";
		labelNode.title = this.tooltip;
		labelNode.innerHTML = this.label;
		contentNode.appendChild(labelNode);

		var containerNode = this.containerNode = dojo.doc.createElement("DIV");
		containerNode.style.display = "none";
		node.appendChild(containerNode);
	},

	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		if(!this.isExpandable){
			this.isExpandable = true;
			this.sectionImg.src = this.imgDir + this.plusIcons[0];
		}
		widget.tree = this.tree;
		btp.widget.SimpleTreeNode.superclass.addChild.apply(this, arguments);
		this.tree.fireTreeNodeAdded(widget);
	},

	changeTreeState: function(e){
		if(this.containerNode.style.display == "none"){
			this.expand();
		}else{
			this.collapse();
		}
	},

	expand: function(){
		if(this.containerNode.style.display != "none"){ return; }
		if(this.tree){ this.tree.onBeforeExpand(this); }
		if(!this.hasChildren()){
			this.sectionImg.src = this.imgDir + this.emptyIcons[0];
			return;
		}
		this.sectionImg.src = this.imgDir + this.minusIcons[0];
		this.containerNode.style.display = "";
	},

	collapse: function(){
		if(this.containerNode.style.display == "none" || !this.hasChildren()){ return; }
		this.sectionImg.src = this.imgDir + this.plusIcons[0];
		this.containerNode.style.display = "none";
	},

	setFocus: function(){
		this.labelNode.focus();
	},

	select: function(){
		dojo.addClass(this.labelNode, "btpSimpleTreeNodeSelected");
	},

	deselect: function(){
		dojo.removeClass(this.labelNode, "btpSimpleTreeNodeSelected");
	}
});
