/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.ToolTabPanel");

dojo.require("btp.widget._TabPanelBase");

dojo.declare(
	"btp.widget.ToolTabPanel",
	btp.widget._TabPanelBase,
{
	unhilightBorderColor: "#ECE9D8",

	edgeImages: null,
	upperLineNode: null,
	lowerLineNode: null,
	lowerRightLineNode: null,
	localToolbarArea: null,

	gray: "#ECE9D8",
	darkGray: "#ACA899",
	lightBlue: "#99BAF3",

	topicNamePrefix: "ToolTab",

	postCreate: function(){
		this.domNode.className = "tabPanel toolTabPanel";
		var header = this.tabHeaderNode;
		this.tabs = [];

		this.childWidgets = [
			{domNode: this.tabHeaderNode, layoutAlign: "top"},
			{domNode: this.containerNode, layoutAlign: "client"}
		];

		// Edge
		this.edgeImages = [];
		var img;
		img = this.edgeImages[0] = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/edge-hi.gif");
		img.style.position = "absolute";
		img.style.display = "none";
		header.appendChild(img);

		img = this.edgeImages[1] = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/edge-lo.gif");
		img.style.position = "absolute";
		img.style.display = "none";
		header.appendChild(img);

		// Delete Button
		if(this.closable){
			this.crossImgPath = dojo.moduleUrl("btp", "widget/themes/images/cross.gif");
			this.redCrossImgPath = dojo.moduleUrl("btp", "widget/themes/images/cross-red.gif");
			img = this.deleteButton = dojo.doc.createElement("IMG");
			img.alt = "";
			img.src = this.crossImgPath;
			img.style.position = "absolute";
			img.style.display = "none";
			this._connectors.push(dojo.connect(img, "onmouseover", this, "deleteButtonMouseOverHandler"));
			this._connectors.push(dojo.connect(img, "onmouseout", this, "deleteButtonMouseOutHandler"));
			this._connectors.push(dojo.connect(img, "onclick", this, "deleteButtonClickHandler"));
			header.appendChild(img);
		}

		// Upper Line
		var line = this.upperLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineUpper";
		line.style.top = "0px";
		line.style.left = "0px";
		header.appendChild(line);

		// Lower Line
		line = this.lowerLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineLower";
		line.style.left = "0px";
		header.appendChild(line);

		// Lower Right Line
		line = this.lowerRightLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineLower";
		header.appendChild(line);

		// to keep height even when there are no tabs
		var dummy = dojo.doc.createElement("DIV");
		dummy.className = "tabPanelDummyTab";
		header.appendChild(dummy);

		var area = this.localToolbarArea = dojo.doc.createElement("DIV");
		area.className = "localToolbarArea";
		header.appendChild(area);

		btp.widget.ToolTabPanel.superclass.postCreate.apply(this, arguments);
	},

	setupChild: function(/*ContentPane*/ page){
		var header = this.tabHeaderNode;
		var tab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.page = page; // [custom property]
		page.tab = tab; // [custom property]
		this.tabs.push(tab);

		page.domNode.style.position = "relative";
		page.domNode.style.display = "block";

		if(page.icon){
			var icon = dojo.doc.createElement("IMG");
			icon.alt = "";
			icon.width = "16";
			icon.height = "16";
			icon.className = "tabPanelTabIcon";
			icon.src = dojo.moduleUrl("btp", page.icon);
			tab.appendChild(icon);
			tab.iconNode = icon; // [custom property]
			if(!this.showInactiveIcons){
				icon.style.display = "none";
			}
		}

		// Title Node
		var titleNode = dojo.doc.createElement("DIV");
		titleNode.className = "tabPanelTabTitle";
		titleNode.tabIndex = 0;
		var label = page.label; //TODO: hale.i18n.gets(page.label, this.i18nPrefix);
		titleNode.innerHTML = label;
		if(page.tooltip){
			tab.title = page.tooltip;
		}
		tab.appendChild(titleNode);
		tab.titleNode = titleNode; // [custom property]

		// Tail Image (hilight)
		var img = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/tail-hi.gif");
		img.className = "tabPanelTailImg";
		img.style.width = "0px";
		header.appendChild(img);
		tab.appendChild(img);
		tab.hilightNode = img; // [custom property]

		// Tail Image (unhilight)
		img = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/tail-lo.gif");
		img.className = "tabPanelTailImg";
		img.style.width = "0px";
		tab.appendChild(img);
		tab.unhilightNode = img; // [custom property]

		header.insertBefore(tab, this.localToolbarArea);

		// local toolbar
/*
		if(page.localToolbarWidget){
			page.localToolbarWidget.domNode.style.styleFloat = "right"; // IE
			page.localToolbarWidget.domNode.style.cssFloat = "right";   // Mozilla
			this.localToolbarArea.appendChild(page.localToolbarWidget.domNode);
		}
*/

		tab._connectors = [];
		tab._connectors.push(dojo.connect(tab, "onclick", this, "onTabClick"));
		tab._connectors.push(dojo.connect(tab, "ondblclick", this, "onTabDblClick"));
		tab._connectors.push(dojo.connect(tab, "onkeydown", this, "onTabKeyDown"));

		page.domNode.style.display = "none";
	},

	// override dijit._Container.removeChild
	removeChild: function(/*Widget*/widget){
		var page = widget;
		var tab = widget.tab;
		for(var i = 0, len = this.tabs.length; i < len; i++){
			if(this.tabs[i] == tab){
				this.tabs.splice(i, 1); // remove tab from tabs
				break;
			}
		}
		if(this.tabs.length == 0){
			this.unhilightTab(tab);
		}
		this.tabHeaderNode.removeChild(tab);
		dojo.forEach(tab._connectors, dojo.disconnect);
		page.destroy();
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "none";
			this.edgeImages[0].style.display = "none";
			this.edgeImages[1].style.display = "none";
			this.containerNode.style.backgroundColor = "#EFEBDE";
			this.fireTabChanged(null);
		}else{
			this.selectTab(this.tabs[0]);
			this.reposition(this.tabs[0]);
		}
	},

	hilightTab: function(tab, focus){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/hilight-bg.gif")+")";
		tab.style.borderRightStyle = "none";

		this.isActive = true;

		// icon
		if(!this.showInactiveIcons && tab.iconNode){
			tab.iconNode.style.display = "";
		}

		// tail
		tab.hilightNode.style.width = "";
		tab.unhilightNode.style.width = "0px";

		// edge
		this.edgeImages[0].style.display = "";
		this.edgeImages[1].style.display = "none";
		this.reposition(tab);

		// border of the content
		this.containerNode.style.border = "solid 2px "+this.lightBlue;

		// focus (necessary for keyboard navigation)
		if(focus){
			tab.titleNode.focus();
		}

		// set tabIndex only for the selected tab
		for(var i = 0, len = this.tabs.length; i < len; i++){
			this.tabs[i].titleNode.tabIndex = (this.tabs[i] == tab) ? 0 : -1;
		}
	},

	unhilightTab: function(tab){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/unhilight-bg.gif")+")";
		tab.style.borderRightStyle = "none";

		this.isActive = false;

		// tail
		tab.hilightNode.style.width = "0px";
		tab.unhilightNode.style.width = "";

		// edge
		this.edgeImages[0].style.display = "none";
		this.edgeImages[1].style.display = "";
		this.reposition(tab);

		// border of the content
		this.containerNode.style.border = "solid 2px "+this.unhilightBorderColor;
	},

	grayoutTab: function(tab){
		tab.style.backgroundImage = "";
		tab.style.borderRightStyle = "solid";
		tab.style.borderBottom = "solid 1px "+this.darkGray;

		// icon
		if(!this.showInactiveIcons && tab.iconNode){
			tab.iconNode.style.display = "none";
		}

		// tail
		tab.hilightNode.style.width = "0px";
		tab.unhilightNode.style.width = "0px";
	},

	reposition: function(tab){
		// Edge
		var img, idx;
		if(this.isActive){
			img = tab.hilightNode;
			idx = 0;
		}else{
			img = tab.unhilightNode;
			idx = 1;
		}
		var dim = btp.getDimensions(img, this.tabHeaderNode);
		this.edgeImages[idx].style.top = dim.top + "px";
		this.edgeImages[idx].style.left = dim.left + img.offsetWidth + "px";

		// Delete Button
		if(this.closable){
			this.deleteButton.style.top = dim.top + "px";
			this.deleteButton.style.left = dim.left - 5 + "px";
		}

		// Lower Line
		this.lowerLineNode.style.top = this.lowerRightLineNode.style.top = dim.top + dim.height + "px";
		if(this.isActive){
			this.lowerLineNode.style.backgroundColor = this.lightBlue;
		}else{
			this.lowerLineNode.style.backgroundColor = this.darkGray;
		}

		// Lower Right Line
		if(this.selectedPage){
			var lastTabDim = btp.getDimensions(this.selectedPage.tab, this.tabHeaderNode);
			var pos = lastTabDim.left + lastTabDim.width + 15;
			this.lowerRightLineNode.style.left = pos + "px";
		}
	}
});
