/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.Toolbar");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("btp.widget.Model");
dojo.require("btp.i18n");

dojo.declare(
	"btp.widget.Toolbar",
	[dijit._Widget, dijit._Templated],
{
	modelId: '',
	ref: '.',
	target: '',

	conf: null,
	items: null,
	iconPalette: null,
	iconBase: null,

	templateString: '<div class="btpToolbar" dojoAttachPoint="toolbarNode"><table cellspacing="0" cellpadding="0"><tbody><tr></tr></tbody></table></div>',
//	templateCssPath: dojo.moduleUrl("btp", "widget/templates/Toolbar/Toolbar.css"),
	i18nPrefix: "Toolbar_",

	postCreate: function(){
		if(this.target){
			this.editorActivatedFilter = this.target;
			this._topics = [
				dojo.subscribe("btp:ActiveEditorChanged", this, "editorActivated")
			];
		}
		this.updateToolbar();
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	updateToolbar: function(json){
		var selectedNode;
		if(json){
			selectedNode = json;
		}else if(this.modelId){
			var model = dijit.byId(this.modelId);
			selectedNode = model.selectSingleNodeFromJson(this.ref); // <application>
		}else{
			return;
		}
		this.conf = selectedNode;
		this.iconBase = selectedNode.toolBar["@iconBase"];
		var items = btp.getChildNodes(selectedNode.toolBar, "item");
		this.items = [];
		for(var i = 0, len = items.length; i < len; i++){
			var item = items[i]; // <item>
			var elem;
			if(item["@separator"] == "true"){
				elem = this.createToolbarSeparator(this, item);
			}else if(item["@type"] == "select"){
				elem = this.createToolbarDropdown(this, item);
			}else{
				elem = this.createToolbarIcon(this, item);
			}
			this.items.push(elem);
			this.items[elem.command] = elem;
		}
	},

	createToolbarSeparator: function(parent, item){
		var img = dojo.doc.createElement("IMG");
		img.className = "btpToolbarSeparator";
		img.src = dojo.moduleUrl("btp", "widget/templates/Toolbar/icon-separator.gif");

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.appendChild(img);

		return img;
	},

	/*
	    <div class="btpToolbarIcon">
	      <a href="javascript:void(0)">
	        <img src="open.gif">
	        label text
	      </a>
	    </div>
	*/
	createToolbarIcon: function(parent, item){
		var command = item["@command"];
		var tooltip = item["@tooltip"];
		var arg = item["@arg"];
		var iconPath = item["@iconPath"];
		var iconFile = item["@iconFile"];
		var toggle = !!item["@toggle"];
		var radio = !!item["@radio"];
		var name = !!item["@name"];
		var selected = !!item["@selected"];
		var text = item["@text"];
		if(!tooltip){
			tooltip = command;
		}
		if(this.iconBase && !iconPath && iconFile){
			iconPath = this.iconBase + iconFile;
		}

		text = btp.i18n.gets(text, this.i18nPrefix);
		tooltip = btp.i18n.gets(tooltip, this.i18nPrefix);

		var div = dojo.doc.createElement("DIV");
		div.className = "btpToolbarIcon";

		div.title = tooltip;
		div.command = command; // [custom property]
		div.arg = arg; // [custom property]
		div.toggle = toggle; // [custom property]
		div.radio = radio; // [custom property]
		div._name = name; // [custom property]
		div._selected = selected; // [custom property]

		var a = dojo.doc.createElement("A");
		a.href = "javascript:void(0)";
		a.onmouseover = "window.status=''";
		a.onclick = this.nop; // to avoid firing the onbeforeunload event (dojo.connect doesn't work for this purpose)
		div.appendChild(a);

		var img = dojo.doc.createElement("IMG");
		img.alt = tooltip;
		img.src = dojo.moduleUrl("btp", iconPath);
		img.border = "0";
		div.iconImg = img; // [custom property]
		div.enabledSrc = img.src; // [custom property]
		div.disabledSrc = img.src.replace(/\.([^\.]+)$/, "_d.$1"); // [custom property]
		a.appendChild(img);
		
		if(text){
			a.appendChild(dojo.doc.createTextNode(text));
			div.style.paddingRight = "6px";
		}

		if(toggle || radio){
			if(selected == "true"){
				div.toggleOn = true;
				this.sunken(div);
			}
			else
				div.toggleOn = false;
		}

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.appendChild(div);

		dojo.connect(div, "onclick", this, "commandHandler");
		dojo.connect(div, "onmouseover", this, "onMouseOverHandler");
		dojo.connect(div, "onmouseout", this, "onMouseOutHandler");
		dojo.connect(div, "onmousedown", this, "onMouseDownHandler");
		dojo.connect(div, "onkeydown", this, "onKeyDownHandler");
		
		return div;
	},

	createToolbarDropdown: function(parent, item){
		var command = item["@command"];
		var tooltip = item["@tooltip"];
		var arg = item["@arg"];
		var sel = dojo.doc.createElement("SELECT");
		sel.className = "btpToolbarDropdown";
		sel.command = command; // [custom property]
		sel.arg = arg; // [custom property]
		sel.title = tooltip;

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.noWrap = true;
		cell.appendChild(sel);

		// for accessibility
		var label = dojo.doc.createElement("LABEL");
		label.innerHTML = tooltip;
		label.className = "hidden-label";
		cell.appendChild(label);

		dojo.connect(sel, "onchange", this, "commandHandler");
		
		return sel;
	},

	getItemElement: function(command){
		return this.items[command];
	},

	changeToggleButtonState: function(item, /*Boolean or undifined*/flag){
		if(!item){ return; }
		var elem;
		if(typeof(item) == "number"){ // index
			elem = this.items[i];
		}else if(typeof(item) == "string"){ // command name
			elem = this.getItemElement(item);
		}else{
			elem = item;
		}
		if(!elem.toggle){ return; }
		if(flag == true){
			if(elem.toggleOn != true){
				elem.toggleOn = true; // [custom property]
				this.sunken(elem);
			}
		}else if(flag == false){
			if(elem.toggleOn != false){
				elem.toggleOn = false; // [custom property]
				this.flat(elem);
			}
		}else{
			elem.toggleOn = !elem.toggleOn; // [custom property]
			if(elem.toggleOn){
				this.sunken(elem);
			}
			else{
				this.flat(elem);
			}
		}
	},

	selectRadioButton: function(item){
		if(!item){ return; }
		var elem;
		if(typeof(item) == "number"){ // index
			elem = this.items[i];
		}else if(typeof(item) == "string"){ // command name
			elem = this.getItemElement(item);
		}else{
			elem = item;
		}
		if(!elem.radio || elem.toggleOn){ return; }
		for(var i = 0, len = this.items.length; i < len; i++){
			var item = this.items[i];
			if(item == elem){
				elem.toggleOn = true; // [custom property]
				this.sunken(elem);
			}else if(item.radio && item._name == elem._name){
				item.toggleOn = false;
				this.flat(item);
			}
		}
	},

	refreshState: function(editor){
		var items = btp.getChildNodes(this.conf.toolBar, "item");
		for(var i = 0, len = items.length; i < len; i++){
			var item = items[i]; // <item>
			var prop = item["@prop"];
			if(prop){
				var div = this.items[i];
				if(div.toggle || div.radio){
					var cap = prop.charAt(0).toUpperCase() + prop.substring(1);
					var flag;
					if(editor["get"+cap]){
						flag = editor["get"+cap]();
					}else if (editor[prop]){
						flag = editor[prop];
					}else{
						break;
					}
					div.toggleOn = flag; // [custom property]
					if(flag){
						this.sunken(div);
					}else{
						this.flat(div);
					}
				}
			}
		}
	},

// ======================= Event handlers =================================
	_isMatch: function(editor, filter){ // same logic as btp.matchFilter()
		if(!filter){ return false; }
		if(filter == "*"){ return true; }
		var arr = filter.split(",");
		var id = editor.domNode.getAttribute("widgetId");
		var type = editor.declaredClass;
		for(var i = 0, len = arr.length; i < len; i++){
			if(arr[i] == id || arr[i] == type){
				return true;
			}
		}
		return false;
	},

	editorActivated: function(message){
		if(!this._isMatch(message.editor, this.target)){ return; }
		this.refreshState(message.editor);
	},

	commandHandler: function(e){
		var elem = e.currentTarget; // <div> or <select>
		if(!this.isEnabled(elem)){ return; }
		if(elem.toggle || elem.radio){
			elem.toggleOn = !elem.toggleOn; // [custom property]
			if(elem.toggleOn){
				this.sunken(elem);
			}
			else{
				this.flat(elem);
			}
			if(elem.radio){
				for(var i = 0, len = this.items.length; i < len; i++){
					var item = this.items[i];
					if(item != elem && item.radio && item._name == elem._name){
						item.toggleOn = false;
						this.flat(item);
					}
				}
			}
		}
		else if(elem.tagName == "DIV"){
			var _this = this;
			setTimeout(function(){_this.flat(elem);}, 100);
		}

		var message = {
			type: "ActionInvokedNotification",
			src: this,
			command: e.currentTarget.command,
			arg: e.currentTarget.arg,
			item: e.currentTarget
		};
		dojo.publish("btp:ActionInvoked", [message]);
	},

	onMouseOverHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		if(div.toggleOn){
			return;
		}else{
			this.raised(div);
		}
	},

	onMouseOutHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		if(div.toggleOn){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	onMouseDownHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		this.sunken(div);
	},

	onKeyDownHandler: function(e){
		if(e.keyCode == 13){ // Enter
			this.onMouseDownHandler(e);
		}
	},

	nop: function(){
		return false;
	},


	enable: function(div){
		div.iconImg.src = div.enabledSrc;
	},

	disable: function(div){
		div.iconImg.src = div.disabledSrc;
	},

	isEnabled: function(elem){
		if(elem.tagName == "DIV"){
			return !elem.iconImg.src.match(/_d\.([^\.]+)$/);
		}else if(elem.tagName == "SELECT"){
			return true;
		}
	},


	flat: function(div){
		div.style.borderColor = "#ECE9D8";
		div.iconImg.style.padding = "1px 2px 1px 1px";
	},

	raised: function(div){
		div.style.borderColor = "white #ACA899 #ACA899 white";
		div.iconImg.style.padding = "1px 2px 1px 1px";
	},

	sunken: function(div){
		div.style.borderColor = "#ACA899 white white #ACA899";
		div.iconImg.style.padding = "2px 1px 0px 2px";
	}

});
