/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.Workbench");

dojo.declare(
	"btp.widget.Workbench",
	dijit._Widget,
{
	commands: '',
	actionInvokedFilter: '',
	tabChangedFilter: '*',
	tabDoubleClickedFilter: '*',

	activeView: null,
	activeEditor: null,

	postCreate: function(){
		this._topics = [
			dojo.subscribe("btp:ToolTabChanged", this, "onTabChanged"),
			dojo.subscribe("btp:ToolTabDoubleClicked", this, "onTabDblClick"),
			dojo.subscribe("btp:ActionInvoked", this, "dispatchCommand")
		];

		var _this = this;
		dojo.addOnLoad(function(){
			var editors = btp.getEditors();
			var editArea = editors.length > 0 ? editors[0] : null;
			if(editArea){
				_this.fireActiveEditorChanged(editArea);
			}
		});
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	dispatchCommand: function(message){
	},

	onTabChanged: function(message){
		if(!btp.matchFilter(message, this.tabChangedFilter)){
			return; // filter out the message
		}

		this.activeView = message.tabPage;
		var message = {
			type: "ActiveViewChangedNotification",
			src: this,
			view: this.activeView
		};
		dojo.publish("btp:ActiveViewChanged", [message]);

		if(!this.activeView){
			var editors = btp.getEditors();
			if(editors.length == 0){
				this.activeEditor = null;
			}
			return;
		}

		var editors = btp.getEditors(this.activeView);
		var editArea = editors.length > 0 ? editors[0] : null;
		if(editArea){
			if(editArea == this.activeEditor){ return; }
			this.fireActiveEditorChanged(editArea);
			editArea.fireSelectionChanged(editArea.focusWidgets && editArea.focusWidgets[0]); // to update PropertiesView, StyleView and OutlineView
		}
	},

	onTabDblClick: function(message){
	},

	fireActiveEditorChanged: function(/*Widget?*/editArea){
		var message = {
			type: "ActiveEditorChangedNotification",
			src: this,
			editor: editArea
		};
		dojo.publish("btp:ActiveEditorChanged", [message]);
		this.activeEditor = editArea;
	}
});
