/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.XmlTree");

dojo.require("btp.widget.SimpleTree");
dojo.require("btp.widget.Model");

dojo.declare(
	"btp.widget.XmlTree",
	btp.widget.SimpleTree,
{
	modelId: '',
	nodeset: '',

	selectedNodes: null,

	postCreate: function(){
		btp.widget.XmlTree.superclass.postCreate.apply(this, arguments);
		this.updateTree();
	},

	updateTree: function(json){
		this.selectedNode = null;
		if(json)
			this.selectedNode = json;
		else if(this.modelId){
			var model = dijit.byId(this.modelId);
			if(!model)
				return;
			this.selectedNodes = model.selectNodesFromJson(this.nodeset);
			if(!this.selectedNodes){
				alert("XmlTree: "+model.src+" not found.");
				return;
			}
		}
		else
			return;
		for(var i = 0, len = this.selectedNodes.length; i < len; i++){
			var node = this.createNode(this.selectedNodes[i]);
			this.addChild(node);
		}
	},

	createNode: function(obj){
		var label = obj["@label"];
		var tooltip = obj["@tooltip"];
		var iconPath = obj["@iconPath"];
		var isFolder = obj["@isFolder"] == "true";
		var iconImage = iconPath ? dojo.moduleUrl("btp", iconPath)+"" : null;
		if(!isFolder){
			for(var key in obj){
				if(key.charAt(0) != "@"){
					isFolder = true;
					break;
				}
			}
		}

		var props = {
			label: label,
			tooltip: tooltip,
			iconImage: iconImage,
			isExpandable: isFolder
		};
		var node = new btp.widget.SimpleTreeNode(props);
		node.jsonObj = obj;
		return node;
	},

	// override
	onBeforeExpand: function(/*SimpleTreeNode*/node){
		if(!node.isExpandable || node.getChildren().length > 0 || !node.jsonObj){ return; }
		for(var key in node.jsonObj){
			if(key.charAt(0) == "@"){ continue; }
			var arr = node.jsonObj[key];
			if(!dojo.isArray(arr)){
				arr = [arr];
			}
			for(var i = 0, len = arr.length; i < len; i++){
				var child = this.createNode(arr[i]);
				node.addChild(child);
			}
		}
	}
});
