/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._PaletteBase");
dojo.provide("btp.widget._PaletteFolderBase");
dojo.provide("btp.widget._PaletteItemBase");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("btp.dnd.DragManager");
dojo.require("btp.dnd.DragSource");
dojo.require("btp.i18n");

dojo.declare(
	"btp.widget._PaletteBase",
	[dijit._Widget, dijit._Container],
{
	modelId: '',
	ref: '.',
	label: '',
	lazy: '',

	defaultClickHandler: "btp.widget.tool.DefaultCreationTool",
	defaultDropHandler: "btp.widget.tool.DefaultCreationTool",

	// The default class names for PaletteFolder and PaletteItem.
	// Subclass may want to override these.
	paletteFolderClass: "btp.widget.PaletteFolder",
	paletteItemClass: "btp.widget.PaletteItem",

	i18nPrefix: "Palette_",
	folderTemplate: null,
	itemTemplate: null,
	dragSources: null,
	selectedItem: null,
	toolMap: {},
	_connectors: null,

	constructor: function(){
		this._connectors = [];
		this.dragSources = [];
	},

	postCreate: function(){
		this.domNode.className = "btpPalette";
		this.domNode.style.overflow = "auto"; // to suppress the unnecessary horizontal scrollbar
		if(this.lazy != "true"){
			this.postProcess();
		}
	},

	postProcess: function(){
		if(this._initialized){ return; }
//@TODO		if(!hale.widget.util.common.isShowing(this)){
//@TODO			this._pending_initialization = true;
//@TODO			return;
//@TODO		}
		this._initialized = true;

		this.makeFolderTemplate();
		this.makeItemTemplate();

		if(this.modelId){
			var arr = this.modelId.split(",");
			for(var i = 0, len = arr.length; i < len; i++){
				this.createPalette(arr[i]);
			}
		}
	},

	// override
	show: function(){
		if(!this._initialized && this._pending_initialization)
			this.postProcess(); // lazy initialization
	},

	makeFolderTemplate: function(){
		// subclass must implement
	},

	makeItemTemplate: function(){
		// subclass must implement
	},

	// subclass may want to override this to handle your own XML
	createPalette: function(modelId){
		var model = dijit.byId(modelId);
		var selectedNode = model.selectSingleNodeFromJson("."); // <components>
		var names = {};

		var categoryNodes = btp.getChildNodes(selectedNode, "category");
		for(var i = 0, len = categoryNodes.length; i < len; i++){
			var folderNode = categoryNodes[i]; // <category>
			if(folderNode["display"] == "false"){
				continue;
			}
			var folderIcon = folderNode["icon"];
			var folderDesc = folderNode["description"];
			var folderInitialState = folderNode["initialState"];
			var folder = this.createFolder(folderIcon, folderDesc);

			var componentNodes = btp.getChildNodes(folderNode, "component");
			for(var j = 0, len2 = componentNodes.length; j < len2; j++){
				var componentNode = componentNodes[j]; // </component>
				if(componentNode["display"] == "false"){
					continue;
				}
				var name = componentNode["name"];
				var description = componentNode["description"];
				var icon = componentNode["icon"];
				var clickHandler = componentNode["clickHandler"];
				var dropHandler = componentNode["dropHandler"];

				if(!icon){
					icon = "../hale/src/hale/widget/templates/Palette/noop.gif";
				}
				if(!clickHandler){
					clickHandler = this.defaultClickHandler;
				}
				if(!dropHandler){
					dropHandler = this.defaultDropHandler;
				}

				if(names[name]){
					var k;
					for(k = 2; names[name+":"+k]; k++)
						;
					name = name+":"+k;
				}
				names[name] = true;

				var opt = {
					icon: icon,
					description: description,
					name: name,
					paletteId: this.id,
					clickHandler: clickHandler,
					dropHandler: dropHandler
				};

				var item = this.createItem(opt);
			}
			if(folderInitialState == "closed"){
				folder.folderClickHandler();
			}
		}
	},

	createFolder: function(imagePath, text){
		var opt = {
			icon: imagePath,
			description: text,
			paletteId: this.id
		};

		var impl = dojo.getObject(this.paletteFolderClass);
		var node = new impl(opt);
		this.addChild(node);
		return node;
	},

	createItem: function(opt){
		var impl = dojo.getObject(this.paletteItemClass);
		var node = new impl(opt);
		this.addChild(node);

		var ds = new btp.dnd.DragSource(node.domNode, "component", node);
		ds.targetShouldShowCaret = true;
		dojo.connect(ds, "onDragStart", this, "onDragStart"); // move start
		dojo.connect(ds, "onDragEnd", this, "onDragEnd"); // move end
		ds.offsetParent = dojo.doc.body;
		this.dragSources.push(ds);
		return node;
	},
	
	onDragStart: function(e){
		this._connector = dojo.connect(btp.dnd.dragManager, "onMouseMove", this, "onDragMove"); // moving
		var item = dijit.byNode(e.dragSource.domNode);
		if(item.dropHandler)
			item.changeTool(item.dropHandler);
		else
			item.changeTool(this.defaultDropHandler);
					
	},

	onDragEnd: function(e){
		dojo.disconnect(this._connector); // moving
		this.pushedItem = null;
	},
	
	onDragMove: function(e){
		// someone may want to connect to this...
	},

	nop: function(){
		return false;
	},

	clearPaletteItems: function(){
		for(var i = 0, len = this.dragSources.length; i < len; i++){
			var ds = this.dragSources[i];
			ds.unregister();
			this.dragSources[i] = null;
		}
		this.dragSources = [];
		this.destroyDescendants();
	},
	
	deselect: function(){
		if(this.selectedItem){
			this.selectedItem.flat(this.selectedItem.domNode);
			this.selectedItem = null;
		}
	},


	itemSelected: function(itemWidget, tool){
		var message = {
			type: "PaletteSelectionChangedNotification",
			src: this,
			selection: itemWidget,
			tool: tool
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);
	},

	itemDeselected: function(itemWidget){
		var message = {
			type: "PaletteSelectionChangedNotification",
			src: this,
			selection: null
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);
	}
});


dojo.declare(
	"btp.widget._PaletteFolderBase",
	dijit._Widget,
{
	icon: "",
	description: "",
	paletteId: "",

	palette: null,

	buildRendering: function(){
		this.palette = dijit.byId(this.paletteId);
		var div = this.domNode = this.palette.folderTemplate.cloneNode(true);
		var a = div.firstChild;
		a.onclick = this.palette.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)
		var img = a.firstChild;

		img.src = dojo.moduleUrl("btp", this.icon);
		var label = btp.i18n.gets(this.description, this.palette.i18nPrefix);
		a.appendChild(dojo.doc.createTextNode(label));
	},

	postCreate: function(){
		this._connects = [];
		this._connects.push(dojo.connect(this.domNode, "onmouseover", this, "folderMouseOverHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmouseout", this, "folderMouseOutHandler"));
		this._connects.push(dojo.connect(this.domNode, "onclick", this, "folderClickHandler"));
	},

	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		this.inherited("destroy", arguments);
	},

	folderClickHandler: function(evt){
		var children = this.palette.getChildren();
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			if(child != this){
				continue;
			}
			for(var j = i + 1; j < len; j++){
				child = children[j];
				if(!(child instanceof btp.widget._PaletteItemBase)){
					break;
				}
				if(child.domNode.style.display == "none"){
					child.domNode.style.display = "";
				}else{
					child.domNode.style.display = "none";
				}
			}
			break;
		}
		return false;
	},

	folderMouseOverHandler: function(evt){
		dojo.removeClass(this.domNode, "btpPaletteFolderLow");
		dojo.addClass(this.domNode, "btpPaletteFolderHi");
	},

	folderMouseOutHandler: function(evt){
		dojo.removeClass(this.domNode, "btpPaletteFolderHi");
		dojo.addClass(this.domNode, "btpPaletteFolderLow");
	}
});



dojo.declare(
	"btp.widget._PaletteItemBase",
	dijit._Widget,
{
	icon: "",
	description: "",
	name: "",
	paletteId: "",
	clickHandler: null,
	dropHandler: null,

	palette: null,

	buildRendering: function(){
		this.palette = dijit.byId(this.paletteId);
		var div = this.domNode = this.palette.itemTemplate.cloneNode(true);
		var a = div.firstChild;
		a.onclick = this.palette.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)
		var img = a.firstChild;

		var offsetX = 0;
		var offsetY = 0;
		if(this.icon.indexOf(",") != -1){
			var arr = this.icon.split(",");
			this.icon = arr[0];
			offsetX = arr[1] - 0;
			offsetY = arr[2] - 0;
		}

		img.src = dojo.moduleUrl("btp", this.icon);
		var label = btp.i18n.gets(this.description, this.palette.i18nPrefix);
		a.appendChild(dojo.doc.createTextNode(label));

		this.domNode.description = this.description;
		this.domNode.componentClassName = this.name; // ex. "btp.widget.Hello"
	},

	postCreate: function(){
		this.inherited("postCreate", arguments);
		this._connects = [];
		this._connects.push(dojo.connect(this.domNode, "onmouseover", this, "itemMouseOverHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmouseout", this, "itemMouseOutHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmousedown", this, "itemMouseDownHander"));
		this._connects.push(dojo.connect(this.domNode, "onmouseup", this, "itemMouseUpHander"));
	},

	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		this.inherited("destroy", arguments);
	},

	deselect: function(){
		this.flat(this.domNode);
		this.palette.selectedItem = null;
	},

	itemMouseOverHandler: function(e){
		var div = this.domNode;
		if(this.palette.selectedItem == this){
			;
		}else{
			this.raised(div);
		}
	},

	itemMouseOutHandler: function(e){
		var div = this.domNode;
		if(this.palette.selectedItem == this){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	itemMouseDownHander: function(e){
		var div = this.domNode;
		this.sunken(div);
		if(this.palette.selectedItem && this.palette.selectedItem != this){
			this.flat(this.palette.selectedItem.domNode);
			this.palette.selectedItem = null;
		}
		this.palette.pushedItem = this;
	},

	itemMouseUpHander: function(e){
		if(this.palette.pushedItem != this){
			this.palette.pushedItem = null;
			var div = this.domNode;
			this.raised(div);
			return;
		}
		if(this.palette.selectedItem == this){
			var div = this.domNode;
			this.raised(div);
			this.palette.selectedItem = null;
			this.palette.itemDeselected(this);
			return;
		}
		this.palette.selectedItem = this;
		this.palette.pushedItem = null;
		this.changeTool();
	},

	changeTool: function(tool){
		if(!tool){
			tool = this.clickHandler;
		}
		// Change the current tool to DefaultCreationTool or
		// a tool specified in the component definition file.
		var myTool = this.palette.toolMap[tool];
		if(!myTool) {
			dojo["require"](tool);
			var obj = dojo.getObject(tool);
			myTool = this.palette.toolMap[tool] = new obj();
		}
		this.palette.itemSelected(this, myTool);
	},

	flat: function(div){
		div.style.borderColor = "#ECE9D8";
		div.style.backgroundColor = "#ECE9D8";
		div.style.margin = "0px 1px 1px 0px";
	},

	raised: function(div){
		div.style.borderColor = "white #ACA899 #ACA899 white";
		div.style.backgroundColor = "#ECE9D8";
		div.style.margin = "0px 1px 1px 0px";
	},

	sunken: function(div){
		div.style.borderColor = "#ACA899 white white #ACA899";
		div.style.backgroundColor = "white";
		div.style.margin = "1px 0px 0px 1px";
	}
});
