/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._TabPanelBase");

dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.declare(
	"btp.widget._TabPanelBase",
//	[dijit.layout.LayoutContainer, dijit._Templated],
	[dijit.layout._LayoutWidget, dijit._Templated],
//	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	closable: false,
	isContainer: true,
	isActive: false,
	showInactiveIcons: false,
	type: "simple", // "eclipse"(default) or "simple"

	templateString: '<div class="tabPanel" dojoAttachPoint="tabPanelNode">'+
						'<div class="tabPanelHeader" dojoAttachPoint="tabHeaderNode"></div>'+
						'<div class="tabPanelPane" dojoAttachPoint="containerNode"></div>'+
					'</div>',
	i18nPrefix: "TabPanel_",

	deleteButton: null,
	selectedPage: null,
	tabs: null,
	crossImgPath: null,
	redCrossImgPath: null,

	_connectors: null,
	_topics: null,

	constructor: function(){
		this._connectors = [];
		this._topics = [];
	},

	postCreate: function(){
		btp.widget._TabPanelBase.superclass.postCreate.apply(this, arguments);
		this._connectors.push(dojo.connect(this.containerNode, "onmousedown", this, "onTabClick"));
		this._topics.push(dojo.subscribe("btp:"+this.topicNamePrefix+"Changed", this, "onTabChanged"));
	},

	startup: function(){
		this.setupChildren();
		btp.widget._TabPanelBase.superclass.startup.apply(this, arguments);
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setupChildren: function(){
		var children = this.getChildren();
		dojo.forEach(children, this.setupChild, this);
		if(this.tabs.length > 0){
			if(this.deleteButton){
				this.deleteButton.style.display = "";
			}
			this.selectTab(this.tabs[0], true);
			var node = this.tabs[0].page.domNode;
			if(this.tabs[0].page.parseOnLoad == false){
				setTimeout(function(){ dojo.parser.parse(node); }, 0);
			}
		}
	},

	setupChild: function(/*ContentPane*/ page){
		alert("subclass must implement this");
	},

	selectTab: function(tab, init, focus){
		if(typeof(tab) == "number"){
			tab = this.tabs[tab];
		}
		var page = tab.page;

		if(this.selectedPage == tab.page){ // already showing
			if(this.isActive){
				return; // already active
			}
		}else{
			if(this.selectedPage){
				var selectedTab = this.selectedPage.tab;
				this.grayoutTab(selectedTab);
				this.selectedPage.domNode.style.display = "none";
				if(this.selectedPage.hide){
					this.selectedPage.hide();
				}
			}
			page.domNode.style.display = "";
			if(page.show){
				page.show();
			}
			if(page.parseOnLoad == false){
				dojo.parser.parse(page.domNode);
			}
			if(!init && page.resize){
				page.resize();
			}
		}
		this.selectedPage = page;

		if(init){
			this.unhilightTab(tab);
			if(!this.showInactiveIcons && tab.iconNode){
				tab.iconNode.style.display = "";
			}
		}else{
			this.hilightTab(tab, focus);
			this.fireTabChanged(page);
		}
	},

	// override dijit._Container.addChild
	addChild: function(page){
		btp.widget._TabPanelBase.superclass.addChild.apply(this, arguments);
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "";
		}
		this.setupChild(page);
		this.selectTab(page.tab);
//		this.onResized();
	},

	hilightTab: function(tab, focus){
		// subclass may want to override this
	},

	unhilightTab: function(tab){
		// subclass may want to override this
	},

	grayoutTab: function(tab){
		// subclass may want to override this
	},

	reposition: function(tab){
		// subclass may want to override this
	},

	setDirty: function(page, flag){
		var tab = page.tab;
		var titleNode = tab.titleNode;
		var title = titleNode.innerHTML;
		if(flag && title.charAt(0) != "*"){
			title = "*" + title;
			titleNode.innerHTML = title;
			this.reposition(tab);
		}else if(!flag && title.charAt(0) == "*"){
			title = title.substring(1);
			titleNode.innerHTML = title;
			this.reposition(tab);
		}
	},

	setTitle: function(tab, title, tooltip){
		if(typeof(tab) == "number"){
			tab = this.tabs[tab];
		}
		var page = tab.page;
		var titleNode = tab.titleNode;
		titleNode.innerHTML = title;
		if(tooltip){
			tab.title = tooltip;
			page.tooltip = tooltip;
		}
		page.label = title;
		this.onResized();
	},

	// ======== Topic Publisher ====================================
	fireTabChanged: function(page){
		var message = {
			type: this.topicNamePrefix+"ChangedNotification",
			src: this,
			tabPage: page
		};
		dojo.publish("btp:"+this.topicNamePrefix+"Changed", [message]);
	},

	fireTabDoubleClicked: function(page){
		var message = {
			type: this.topicNamePrefix+"DoubleClickedNotification",
			src: this,
			tabPage: page
		};
		dojo.publish("btp:"+this.topicNamePrefix+"DoubleClicked", [message]);
	},


	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.childWidgets);

		if(!this.selectedPage){ return; }
		if(this.selectedPage.resize){
			var content = dojo.contentBox(this.containerNode);
			this.selectedPage.resize(content.width, content.height);
		}

		if(!this.selectedPage){ return; }
		var tab = this.selectedPage.tab;
		this.reposition(tab);
	},

	onTabClick: function(e){
		if(this.tabs.length == 0){
			return;
		}
		var target = e.currentTarget;
		var tab;
		if(target.className == "tabPanelTab"){
			tab = target;
			this.selectTab(tab);
		}else if(target.className.indexOf("tabPanelPane") != -1){
			tab = this.selectedPage.tab;
			this.selectTab(tab);
		}
	},

	onTabDblClick: function(e){
		var tab = e.currentTarget;
		this.fireTabDoubleClicked(tab.page);
	},

	onTabKeyDown: function(e){
		var tab = e.currentTarget;
		var key = e.keyCode;
		if(key == 37 || key == 38){ // 37: left arrow, 38: up arrow
			var prev = tab.previousSibling;
			if(!prev || !prev.page){
				return;
			}
			this.selectTab(prev, false, true);
		}else if(key == 39 || key == 40){ // 39: right arrow, 40: down arrow
			var next = tab.nextSibling;
			if(!next || !next.page){
				return;
			}
			this.selectTab(next, false, true);
		}
	},

	// ======== Topic Subscriber ====================================
	onTabChanged: function(message){
		if(message.src == this){
			return; // I am interested in the other tabPanel only
		}
		if(!message.tabPage){
			return; // all tabs have been deleted
		}
		var tab = message.tabPage.tab;
		if(tab && this.selectedPage){
			this.unhilightTab(this.selectedPage.tab);
		}
	},


	deleteButtonMouseOverHandler: function(e){
		var img = e.currentTarget;
		img.src = this.redCrossImgPath;
	},

	deleteButtonMouseOutHandler: function(e){
		var img = e.currentTarget;
		img.src = this.crossImgPath;
	},

	deleteButtonClickHandler: function(e){
		this.removeChild(this.selectedPage);
	}
});

dojo.extend(dijit._Widget, {
	label: "",
	icon: "",
	selected: false
});
