/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._editArea.event");

// summary:
//		Publishers and Subscribers.
//		'this' points to the EditArea.

btp.widget._editArea.event = {
	// ======== Topic Publishers ====================================
	firePropertyChanged: function(propName, oldValue, newValue, /*Object?*/hint){
		var message = {
			type: "PropertyChangedNotification",
			src: this,
			propName: propName,
			oldValue: oldValue,
			newValue: newValue,
			hint: hint
		};
		dojo.publish("btp:PropertyChanged", [message]);
	},

	fireSelectionChanged: function(selection){ // cf. StructuredViewer
		var message = {
			type: "SelectionChangedNotification",
			src: this,
			selection: selection
		};
		dojo.publish("btp:SelectionChanged", [message]);
	},

	fireMessagePosted: function(text){
		var message = {
			type: "MessagePostedNotification",
			src: this,
			text: text
		};
		dojo.publish("btp:MessagePosted", [message]);
	},

	fireInputChanged: function(){
		var message = {
			type: "InputChangedNotification",
			src: this
		};
		dojo.publish("btp:InputChanged", [message]);
	},

	// ======== Topic Subscribers ====================================
	paletteSelectionChanged: function(message){
		if(!btp.matchFilter(message, this.paletteSelectionChangedFilter)){
			return; // filter out the message
		}

		if(!btp.isShowing(this)){ return; }
		this.selectedPaletteItem = message.selection;
		this.activeToolChanged(message);
	},

	activeToolChanged: function(message){
		// [Temporary Implementation]
		// EditDomain connects to this.
	},

	preferenceChanged: function(evt){
		var workbench = evt.src;
		var showGrid = workbench.getCookieValue("General/Appearance:showGrid");
		var gridSize = workbench.getCookieValue("General/Appearance:gridSize");
		var snapTo = workbench.getCookieValue("General/Appearance:snapTo");
		if(gridSize){
			this.gridWidth = gridSize;
		}
		this.snapObjectsToGrid(snapTo)
		this.displayGrid(showGrid)
	},

	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}
		if(!btp.isShowing(this)){
			return; // EditArea is inactive
		}

		if(this.focusWidgets && this.focusWidgets[0]){
			this.refreshFocusFrame(this.focusWidgets[0]);
		}
	}
};
