/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._editArea.focus");

// summary:
//		Handles focus for components on EditArea.
//		'this' points to the EditArea.

btp.widget._editArea.focus = {
	setFocus: function(widget, notFocusOnEditArea){
		if(!this.focusWidgets){
			this.focusWidgets = [];
		}
		if(this.focusWidgets[0] == widget){ return; }
		if(widget == this){
			widget = null;
		}
		this.activate(true);
		if(!widget){
			this.removeFocus(null, notFocusOnEditArea);
			return;
		}
		if(this.focusWidgets[0]){
			// Hopefully, this removes focus from the currently focused widget without firing
			// the selectionChanged event even if the clicked widget has no focusable elements.
			this.removeFocus(this.focusWidgets);
		}

		if(dojo.isMozilla){
			// Put focus on the EditArea.
			// FF needs this to respond to keys such as DEL, Ctrl+Z, etc.
			// However, IE scrolls up the canvas if you do this. IE doesn't need this to respond to keys.
			this.domNode.focus();
		}

		this.setFocusFrame(widget);

		if(widget.focus){
			widget.focus();
		}else{
			try {
//				widget.domNode.focus();
			} catch(e){}
		}

		this.focusWidgets = [widget];
		this.fireSelectionChanged(widget);
	},

	setFocusFrame: function(widget){
		if(widget.hlp_setFocusFrame){
			widget.hlp_setFocusFrame();
		}
	},

	// If from is given, remove focus from it.
	// Otherwise, remove focus from the this.focusWidgets and fire the event.
	removeFocus: function(from, notFocusOnEditArea){
		if((!this.focusWidgets || this.focusWidgets.length == 0) && !from){ return; }
		var widget = from ? from[0] : this.focusWidgets[0];

		this.removeFocusFrame(widget);

		if(widget.blur){
			widget.blur();
		}else{
			if(!notFocusOnEditArea){
				/* hack: When navigating widgets with shift+tab (reverse direction),
				 * domNode.blur() causes unnecessary focus on EditArea */
				widget.domNode.blur();
			}
		}
		if(!from){
			this.focusWidgets = [];
			this.fireSelectionChanged(null);
		}
	},

	removeFocusFrame: function(widget){
		if(widget.hlp_removeFocusFrame){
			widget.hlp_removeFocusFrame();
		}
	},

	refreshFocusFrame: function(widget){
		this.removeFocusFrame(widget);
		this.setFocusFrame(widget);
	},

	deleteFocusWidget: function(){
		if(!this.focusWidgets || this.focusWidgets.length == 0){ return; }

		var focusWidgets = this.focusWidgets;
		this.removeFocus();

		// TODO: What if two widgets have parent-child relationship?
		var compoundCommand = new btp.widget.command.CompoundCommand();
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			var parent = focusWidget.getParent();
			if(parent.update){
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
			}else{
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
			}
		}
		this.getEditDomain().executeCommand(compoundCommand);
	}
};
