/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._propertiesView.event");

// summary:
//		Handles events on the PropertiesView.
//		'this' points to the PropertiesView.

btp.widget._propertiesView.event = {
	selection: null,
	editArea: null,

	// ======== Topic Subscribers ====================================
	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}
		
		if(message.propName == "_RemoveChild_"){
			if(message.target == this.selection){
				return; // focused widget was removed. selectionChanged(null) will be fired soon.
			}
			// ex. DataGridColumn was removed, but the focus is DataGrid. selectionChanged won't be fired.
			if(!this.selection){
				return; // maybe an undo operation removed something while no focus exists.
			}
		}else if(message.propName == "_AddChild_"){
			if(!this.selection){
				return; // selectionChanged will be fired soon.
			}
		}

		// Call the PropPage's updateView if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.updateView){
			this.currentPageObj.updateView(this.selection);
		}else{
			this.updateView(this.selection);
		}
	},
	
	selectionChanged: function(message){
		if(!btp.matchFilter(message, this.selectionChangedFilter)){
			return; // filter out the message
		}

		var sel = this.selection = message.selection;
		var src = message.src;
		if(sel && !sel.ps_getPropertyDescriptors){
			return; // doesn't look PropertySource is implemented
		}
		this.editArea = (message.src instanceof btp.widget.EditArea) ? message.src : null;

		// Hide the previous PropPage
		if(this.currentPageNode){
			this.currentPageNode.style.display = "none";
		}
		if(!sel){ return; }

		var name = sel.declaredClass;
		if(sel.widgetTypeSuffix){
			name += ":" + sel.widgetTypeSuffix;
		}
		var obj = this.propPageMap[name];
		if(!obj && this.editArea){
			var info = sel.hlp_getComponentInfo();
			if(info){
				obj = info.proppage;
			}
		}

		// Create a PropPage instance unless cache exists
		if(obj && typeof(obj) == "string"){
			if(this.isWidgetName(obj)){ // PropPage Widget
				if(!this.createPropPageFromWidget){
					dojo["require"]("btp.widget._propertiesView.fromWidget");
					dojo.mixin(this, btp.widget._propertiesView.fromWidget);
				}
				obj = this.createPropPageFromWidget(obj, name, sel);
			}else{ // HTML Fragment
				if(!this.createPropPageFromHtml){
					dojo["require"]("btp.widget._propertiesView.fromHtml");
					dojo.mixin(this, btp.widget._propertiesView.fromHtml);
				}
				obj = this.createPropPageFromHtml(obj, name, sel);
			}
			this.propPageMap[name] = obj; // cache
		}else if(!obj){ // Default PropPage
			if(!this.createPropPageFromXml){
				dojo["require"]("btp.widget._propertiesView.fromXml");
				dojo.mixin(this, btp.widget._propertiesView.fromXml);
			}
			obj = this.createPropPageFromXml(name, sel);
			this.propPageMap[name] = obj; // cache
		}

		// Save the current status
		if(obj instanceof dijit._Widget){ // PropPage Widget
			this.currentPageObj = obj;
			this.currentPageNode = obj.domNode;
		}else /* if(obj.tagName && obj.name == name) */{ // HTML Fragment or Default PropPage
			this.currentPageObj = obj;
			this.currentPageNode = obj;
		}

		// Initialize the PropPage
		this.currentPageNode.style.display = "";

		// Call the PropPage's updateView if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.updateView){
			this.currentPageObj.updateView(sel);
		}else{
			this.updateView(sel);
		}
	},

	setOnChangeHandler: function(root){
		var fields = root.getElementsByTagName("input");
		var field;
		for(var i=0, len=fields.length; i<len; i++){
			field = fields[i];
			dojo.connect(field, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
			if(dojo.isIE){
				dojo.connect(field, "onkeypress", this, "onKeyPressHandler"); //TODO: you may want to put connect() into this._connects.push()
			}
		}
		fields = null;
		fields = root.getElementsByTagName("select");
		for(var j=0, len2=fields.length; j<len2; j++){
			field = fields[j];
			dojo.connect(field, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},


	// ======== Handle User Input ================================
	onChangeHandler: function(e){
		// Call the PropPage's onChangeHandler if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.onChangeHandler){
			this.currentPageObj.onChangeHandler(e);
			return;
		}
		
		var field = e.target;
		var attrName = field.name;
		var val = field.value;
		if(!attrName){ return; }
		if(field.type != "checkbox"){
			this.setAttribute(this.selection, attrName, val);
		}else{
			this.setAttribute(this.selection, attrName, field.checked);
		}
	},
	
	onKeyPressHandler: function(e){
		if(e.keyCode == 13){ // Enter key
			this.onChangeHandler(e);
		}
	}
};
