/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._propertiesView.fromXml");

// summary:
//		Creates a property page from XML
//		'this' points to the PropertiesView.

btp.widget._propertiesView.fromXml = {
	createPropPageFromXml: function(name, sel){
		var root;
		var table = root = dojo.doc.createElement("TABLE");
		table.className = "btpPropertyViewTable";
		table.cellSpacing = "2px";
		table.cellPadding = "0px";
		table.style.width = "90%";
		table.style.marginTop = "3px";
		table.style.marginLeft = "3px";
		var attributes = sel.ps_getPropertyDescriptors();
		for(var i = 0, len = attributes.length; i < len; i++){
			var attr = attributes[i];
			var attrName = attr.name;
			var type = attr.type;
			var fieldType = attr.fieldType;

			if(attr.display == "false"){
				continue;
			}

			var row = table.insertRow(-1);

			if(fieldType == "select"){
				this.createDefaultCombobox(name, row, attr);
			}else if(fieldType == "checkbox"){
				this.createDefaultCheckbox(name, row, attr);
			}else if(fieldType == "radio"){
				this.createDefaultRadioButtons(name, row, attr);
			}else /* if(fieldType == "input") */ {
				this.createDefaultTextField(name, row, attr);
				attr.fieldType = "text";
			}
		}
		this.domNode.appendChild(root);

		this.createElementsMap(root); // elements map
		root.name = name; // key to cache

		return root;
	},
	
	createDefaultTextField: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_"); // Ex. component_live:pushbutton_value
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_"); // Ex. component_value
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPorpInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "btpPorpInputField";
		cell.width = "100%";
		cell.attribute = attr;
		cell.innerHTML = "<INPUT id='"+id+"' name='"+attrName+"' class='btpPropInput' size='50'>";
		var input = cell.firstChild;
		if(attr.readonly == "true"){
			input.readOnly = true;
			input.style.color = "#ACA899";
		}

		dojo.connect(input, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		if(dojo.isIE){
			dojo.connect(input, "onkeypress", this, "onKeyPressHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},
	
	createDefaultCheckbox: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPropInputLabel";
		cell.noWrap = true;
		cell.colSpan = "2";
		cell.innerHTML = "<INPUT id='"+id+"' name='"+attrName+"' class='btpPropCheckBox' type='checkbox'>"
						+"<LABEL for='"+id+"'>"+label+"</LABEL>";
		var checkbox = cell.firstChild;
		if(attr.value && attr.value == "true"){
			checkbox.checked = "checked";
		}
		dojo.connect(checkbox, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
	},
	
	createDefaultRadioButtons: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPropInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "btpPropInputField";
		cell.width = "100%";
		cell.attribute = attr;
		var option;
		if(attr.option){
			option = attr.option;
		}else{
			option = ["true", "false"];
		}

		for(var i=0, len=option.length; i<len; i++){
			cell.innerHTML = "<INPUT id='"+id+"_"+option[i]+"' name='"+attrName+"' class='btpPropRadio' type='radio'>"
							+"<LABEL for='"+id+"_"+option[i]+"'>"+option[i]+"</LABEL>";
			var radioButton = cell.firstChild;
			radioButton.value = option[i];
			if(i == 0 || attr.value && attr.value == option[i]){
				radioButton.checked = "checked";
			}
			dojo.connect(radioButton, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},
	
	createDefaultCombobox: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "porpInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "propInputField";
		cell.width = "100%";
		cell.attribute = attr;
		cell.innerHTML = "<SELECT id='"+id+"' name='"+attrName+"' class='btpPropComboBox'>";
		var comboBox = cell.firstChild;
		var option;
		if(attr.option){
			option = attr.option;
		}else{
			option = ["true", "false"];
		}
		
		var selectedIndex = 0;
		for(var i = 0, len = option.length; i < len; i++){
			var opt = new Option(option[i], option[i]);
			comboBox.options[i] = opt;
			if(attr.value && attr.value == option[i]){
				selectedIndex = i;
			}
		}
		comboBox.options[selectedIndex].selected = true;
		comboBox.options[selectedIndex].defaultSelected = true;
		dojo.connect(comboBox, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
	}
};
