/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget._propertiesView.update");

// summary:
//		Updates the PropertiesView.
//		'this' points to the PropertiesView.

btp.widget._propertiesView.update = {
	currentPageObj: null,  // DOM node or widget
	currentPageNode: null, // DOM node

	// ======== Fill-in the view with the attributes of the widget =======
	// Your PropPage Widget may want to override this
	updateView: function(widget){
		if(!widget || !widget.ps_getPropertyDescriptors){
			if(this.currentPageNode){
				this.currentPageNode.style.display = "none";
			}
			return;
		}

		var attributes = widget.ps_getPropertyDescriptors();
		var root = this.currentPageNode;
		
		for(var i = 0, len = attributes.length; i < len; i++){
			var attr = attributes[i];
			var attrName = attr.name;
			var type = attr.type;
			var fieldType = attr.fieldType;
			var input = root.elements[widget.widgetType+":"+attrName] ? root.elements[widget.declaredClass+":"+attrName] : root.elements[attrName];
			if(!input){
				continue;
			}
			
			var val = widget.ps_getPropertyValue(attrName);
			switch(fieldType){
				case "radio":
					if(val && val == input.value){
						input.checked = "checked";
					}
					break;
				case "select":
					var option = input.options;
					for(var j=0, len2=option.length; j<len2; j++){
						if(option[j].value == val){
							input.selectedIndex = j;
						}
					}
					break;
				case "checkbox":
					if(val){
						input.checked = "checked";
					}else{
						input.checked = "";
					}
					break;
				case "text":
				default:
					input.value = val? val : "";
					break;
			}
		}
	},

	
	setAttribute: function(widget, attribute, val){
		if(typeof(attribute) == "string"){
			var idx = attribute.indexOf(":");
			if(idx != -1){ // remove widgetType prefix (ex. DataGrid:ref -> ref) //TODO: Do not omit curly braces, '{' and '}'. //TODO: widgetType -> declaredClass
				attribute = attribute.substring(idx + 1);
			}
			attribute = this.editArea.getAttributeInfo(widget, attribute);
		}
		if(attribute.type == "integer"){
			val = val - 0;
		}
		var command = new btp.widget.command.EditCommand.SetPropertyValueCommand(widget, attribute, val);
		this.executeCommand(command);
	},

	
	// ======== Utilities ========================================
	executeCommand: function(command){
		this.editArea.getEditDomain().executeCommand(command);
	},
	
	setFocus: function(widget){
		this.editArea.setFocus(widget);
	},
	
	isWidgetName: function(str){
		var exp = /^[A-z]+:+[A-z0-9]+$/;
		var result = str.match(exp);
		return !!result;
	},

	createElementsMap: function(node){
		node.elements = []; // [custom property]
		var elems = node.getElementsByTagName("INPUT");
		for(var i = 0, len = elems.length; i < len; i++){
			var input = elems[i];
			if(input.name){
				node.elements.push(input); // array
				node.elements[input.name] = input; // map
			}
		}
		elems = node.getElementsByTagName("SELECT");
		for(var i = 0, len = elems.length; i < len; i++){
			var input = elems[i];
			if(input.name){
				node.elements.push(input); // array
				node.elements[input.name] = input; // map
			}
		}
	}
};
