/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.helper.dropHelper");

// summary:
//		Make a widget "droppable".
//		If used for EditArea, methods here are called from btp.sub.dropTargetStub.
//		If used for a component, methods here are mixed into it.
//		'this' points to the EditArea or a container type widget.

btp.widget.helper.dropHelper = {
	hlp_dropTargets: null, // each component's helper may want to override this

	hlp_setupDropTarget: function(){
		if(!this.hlp_dropTargets || this.hlp_dropTargets.length == 0){ return; }
		dojo["require"]("btp.dnd.DropTarget");
		var dt;
		if(btp.isGfxWidget(this)){
			var shape = this.getTargetShape();
			if(!shape){ return; }
			dt = this.__dt = new btp.dnd.DropTarget(shape.rawNode, this.hlp_dropTargets, this);
			this.domNode.dropTarget = dt; // [custom property]

			// It looks shape's rowNode does not have parent-child relationship even when
			// two shapes has such relationship. DragManager#findDropTarget fails to find
			// a drop target because of this. Fortunately, parent of the shape's rawNode,
			// which is always the <group> element, has parent-child relationship.
			this.getShape().rawNode.dropTarget = dt;
//			shape.rawNode.parentNode.dropTarget = dt;
		}else{
			dt = this.__dt = new btp.dnd.DropTarget(this.domNode, this.hlp_dropTargets, this);
		}
		dt.offsetParent = this.__editArea.domNode;
		dt.isAcceptable = function dt_isAcceptable(ds, dt){
			// If the drag source is an ancestor of the drop target, do not accept.
			var dsWidget = ds.data;
			var dtWidget = dt.data;
			if(!dsWidget && !dtWidget){ return true; }
			for(var w = dtWidget; w; w = w.parent){
				if(w == dsWidget){
					return false;
				}
			}
			return true;
		};
		this.__dtConns = [];
		this.__dtConns.push(dojo.connect(dt, "onDragStart", this.__editArea, "onEaDragStartHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragOver", this.__editArea, "onEaDragOverHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragMove", this.__editArea, "onEaDragMoveHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragEnd", this.__editArea, "onEaDragEndHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDropStart", this.__editArea, "onEaDropStartHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDrop", this.__editArea, "onEaDropHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDropEnd", this.__editArea, "onEaDropEndHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragOut", this.__editArea, "onEaDragOutHandler"));
		if(this.hlp_showAllowDropFeedback){
			this.__dtConns.push(dojo.connect(dt, "showFeedback", this, "hlp_showAllowDropFeedback"));
		}
		if(this.hlp_hideAllowDropFeedback){
			this.__dtConns.push(dojo.connect(dt, "hideFeedback", this, "hlp_hideAllowDropFeedback"));
		}
		if(this.hlp_dropTargetDragStart){
			this.__dtConns.push(dojo.connect(dt, "onDragStart", this, "hlp_dropTargetDragStart"));
		}
	},

	hlp_cleanupDropTarget: function(){
		var dt = this.__dt;
		if(dt){
			dojo.forEach(this.__dtConns, dojo.disconnect);
			dt.unregister();
			this.__dt = null;
		}
	},

	hlp_getDroppedDomNode: function(e){
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			var ds = e.dragSource;
			srcElem = ds.dragObject; // dropped DOM node
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			srcElem = this.__editArea.selectedPaletteItem.domNode;
		}
		return srcElem;
	}
};
