/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.io.RestIO");

btp.widget.io.RestIO = {
	getResourceUrl: function(path){
		// summary:
		//		Get a url for the raw resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		var url = path;
		if(!url){
			var model = dijit.byId(this.modelId);
			url = model.src;
		}
		if(!url){
			url = window.location.href;
		}
		if(url.indexOf("://") != -1){ // full URL
			var frag = btp.splitUrl(url);
			if(!frag["resourceId"]){
				return null;
			}
			url = frag["url"];
		}else{
			var frag = btp.splitUrl(window.location.href);
			url = frag["list"]+"/"+path;
		}
		return url;
	},

	getViewUrl: function(path){
		// summary:
		//		Get a url for an html representation of resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1.html)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		var url = this.getResourceUrl(path);
		if(!url){ return null; }
		return url+".html";
	},

	getEditUrl: function(path){
		// summary:
		//		Get a url for an editing form of resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		return this.getResourceUrl(path)+";edit";
	},
	
	getListUrl: function(){
		// summary:
		//		Get a url for a listing of resources.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1)
		var url = window.location.href;
		var frag = btp.splitUrl(url);
		if(frag["spaceId"]){
			return frag["list"];
		}
		return null;
	},

	_sendRequest: function(props){
		var url = props.url;
		var method = props.method;
		var content = props.content;
		var postContent = props.postContent;

		var data = null;
		if(content){
			data = dojo.io.argsFromMap(content, "utf8");
		}else{
			data = postContent;
		}

		dojo.debug("sendRequest: "+method+" "+url);
		var http = dojo.hostenv.getXmlhttpObject();
		http.open(method, url, false);
		if(props.contentType){
			http.setRequestHeader("Content-Type", props.contentType);
		}else{
			http.setRequestHeader("Content-Type", "text/html; charset=UTF-8");
		}

		try {
			http.send(data);
		}
		catch (e){
			alert("HTTP "+method+" request Failed: "+url); // TODO: i18n
			return null;
		}

		if(method != "GET" && http.responseText.indexOf("Success") == -1){
			return false;
		}
		return true;
	},

	sendOpenFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "GET",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendNewFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "POST",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendNewFolderRequest: function(path, opts){
		var url = this.getResourceUrl(path)+"/";
		var req = {
			method: "PUT",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendRenameRequest: function(oldPath, newPath, opts){
		var url = this.getResourceUrl(oldPath);
		var req = {
			method: "POST",
			url: url,
			content: {newResourceId:newPath},
			contentType: "application/x-www-form-urlencoded"
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendDeleteFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "DELETE",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},

	sendSaveRequest: function(content, path, opts){
		// path can be null or undefined
		var url = this.getResourceUrl(path);
		var req = {
			method: "PUT",
			url: url,
			postContent: content
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	}
};
