/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.outline.CloneContentOutlinePage");

// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("btp.widget.outline.CloneContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	canvas: null,
	_connectors: null,
	_topics: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
	},

	createOutlinePage: function(){
		if(!this.canvas){
			if(window.gdl){
				dojo["require"]("gdc.widget.Surface");
				this.canvas = new gdc.widget.Surface({width:1000,height:1000}); //@knitta@ temporary fix
				this.canvas.domNode.style.backgroundColor = 'pink'; //@kamki@ delete this later
			}else{
				dojo["require"]("btp.widget.SimplePane");
				this.canvas = new btp.widget.SimplePane();
			}
			this.outlineView.addChild(this.canvas);
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.addChild(this.canvas);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.removeChild(this.canvas);
		}
	},

	refreshOutline: function(){
		this.canvas.destroyDescendants();
		var root = this.contentProvider.getElements(this.editArea);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshCloneView(null, root[i]);
		}
	},

	_refreshCloneView: function(parentObj, obj){
		var node = obj.outlineNode = this._cloneWidget(obj.targetNode); // Widget
		node.outlineObj = obj;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.canvas;
		parentNode.addChild(node);
		if(btp.isGfxWidget(node)){
			node.createShape();
			node.update();
		}

		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshCloneView(obj, child);
		}
	},

	updateOutline: function(message){
		if(message.propName == "@AddChild" || message.propName == "@RemoveChild"){
			var root = this.contentProvider.getElements(this.editArea);
			this._updateCloneView(root, this.canvas);
		}else if(message.propName == "@SetStyles"){
			var widget = message.hint.widget;
			var outlineNode = widget.outlineObj.outlineNode;
			for(var key in message.hint.props){
				outlineNode.domNode.style[key] = widget.domNode.style[key];
			}
		}else{
			var widget = message.hint.widget;
			var outlineNode = widget.outlineObj.outlineNode;
			var name = message.propName;
			var f = "set" + name.charAt(0).toUpperCase() + name.substring(1);
			if(outlineNode[f]){
				outlineNode[f](message.newValue);
			}else{
				outlineNode[name] = message.newValue;
			}
		}
	},

	_updateCloneView: function(objs, /*Widget*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.getChildren ? parentNode.getChildren().length : 0;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateCloneView(chlidObjs, parentNode.getChildren()[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshCloneView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.destroy();
					break;
				}
			}
		}
	},

	_cloneWidget: function(/*Widget*/widget){
		var impl = dojo.getObject(widget.declaredClass);
		var clone = new impl();
		var styles = ["position", "top", "left"];
		for(var i = 0, len = styles.length; i < len; i++){
			clone.domNode.style[styles[i]] = widget.domNode.style[styles[i]];
		}
		return clone;
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
	},

	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
	},

	deleteOutlineTarget: function(){
	}
});
