/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.AbstractDragTool");
dojo.require("btp.widget.tool.AbstractTool");
dojo.require("btp.dnd.DragManager");

dojo.declare(
	"btp.widget.tool.AbstractDragTool",
	btp.widget.tool.AbstractTool,
{
	_dropTargetWidget: null,
	_nodeUnderMouse: null,
	_conn: null,
	_type: "component",

	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		this.storeMouseDownInfo(e);
		this.findDropTarget(e);

		// In order to catch all the move events of the entire page, we do not use handleMouseMove.
		this._conn = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove");
	},
	
	onMouseMove: function(e){
		e.preventDefault(); // this disables selection
	},

	handleMouseUp: function(e, editPartViewer){
		// disconnect the mouseMove handler
		dojo.disconnect(this._conn);

		// deselect the palette
		if(editPartViewer.selectedPaletteItem){
			if(editPartViewer.selectedPaletteItem.deselect){
				editPartViewer.selectedPaletteItem.deselect();
			}
		}

		// reset the tool
		this._editDomain.loadDefaultTool();
	},

	storeMouseDownInfo: function(e){
		this.layerX = e.layerX;
		this.layerY = e.layerY;
		this.pageX = e.pageX;
		this.pageY = e.pageY;
		this.coords = btp.getEventCoords(e);
		var pos = dojo._abs(this._editPartViewer.domNode, false);
		this.offsetX = pos.x;
		this.offsetY = pos.y;
		this.startX = this.pageX - pos.x;
		this.startY = this.pageY - pos.y;
	},

	findDropTarget: function(e){
		var dt = btp.dnd.dragManager.findDropTarget(e.target, {type:this._type});
		if(dt){
			this._nodeUnderMouse = dt.nodeUnderMouse;
			this._dropTargetWidget = btp.getWidgetByElement(dt.domNode);
		}else{
			this._nodeUnderMouse = null;
			this._dropTargetWidget = null;
		}
		return dt;
	},

	// override (AbstractTool.js)
	findRefWidget: function(e){
		return this._nodeUnderMouse;
	},

	// override (AbstractTool.js)
	getDragSourceDomNode: function(e){
		return this._editPartViewer.selectedPaletteItem.domNode;
	},

	// override (AbstractTool.js)
	getDropTargetWidget: function(e){
		return this._dropTargetWidget;
	},

	// override (AbstractTool.js)
	getNodeUnderMouse: function(e){
		return this._dropTargetWidget.domNode;
	},

	// override (AbstractTool.js)
	getOrientation: function(e){
		var node = this._dropTargetWidget.domNode;
		var display = dojo.style(node, "display");
		if(display == "block"){
			return false;
		}else{
			return true;
		}
	}
});
