/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.AbstractTool");

dojo.require("btp.widget.command.CompoundCommand");
dojo.require("btp.widget.command.EditCommand");

dojo.declare("btp.widget.tool.AbstractTool", null, {
	_editDomain: null,
	_editPartViewer: null,
	
	activate: function(){
	},
	
	deactivate: function(){
	},
	
	getEditDomain: function(){
		return this._editDomain;
	},

	setEditDomain: function(domain){
		this._editDomain = domain;
	},
	
	setEditPartViewer: function(editPartViewer){
		this._editPartViewer = editPartViewer;
	},


	createInsertComponentCommand: function(e, refWidget, targetWidget, widget){
		var command = new btp.widget.command.CompoundCommand();
		if(!refWidget || refWidget == targetWidget){
			command.append(new btp.widget.command.EditCommand.AppendChildCommand(targetWidget, widget));
		}else{
			command.append(new btp.widget.command.EditCommand.InsertBeforeCommand(targetWidget, widget, refWidget));
		}
		return command;
	},

	insertComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.createDroppedComponent(e);
		var command = this.createInsertComponentCommand(e, refWidget, targetWidget, widget);
		this.getEditDomain().executeCommand(command);
		this._editPartViewer.setFocus(widget);
		return true;
	},

	createReparentComponentCommand: function(e, refWidget, targetWidget, widget){
		var ds = e.dragSource;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var fromParent = dojo._abs(widget.getParent().domNode, false);
		var toParent = dojo._abs(targetWidget.domNode, false);
		var pdx = fromParent.x - toParent.x;
		var pdy = fromParent.y - toParent.y;
		var toDim = {
			top: ds.dragStartCssPosition.y + dy + pdy + "px",
			left: ds.dragStartCssPosition.x + dx + pdx + "px"
		};

		var command = new btp.widget.command.CompoundCommand();
		if(dojo.style(widget.domNode, "position") != ds.positioning){
			command.append(new btp.widget.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning));
		}
		command.append(new btp.widget.command.EditCommand.RemoveChildCommand(widget));
		if(!refWidget || refWidget == targetWidget){
			command.append(new btp.widget.command.EditCommand.AppendChildCommand(targetWidget, widget));
		}else{
			command.append(new btp.widget.command.EditCommand.InsertBeforeCommand(targetWidget, widget, refWidget));
		}
		command.append(new btp.widget.command.EditCommand.MoveCommand(widget, toDim));
		return command;
	},

	reparentComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.getDragSourceWidget(e);
		if(refWidget == widget){ return; }
		var command = this.createReparentComponentCommand(e, refWidget, targetWidget, widget);
		this._editDomain.executeCommand(command);
		this._editPartViewer.setFocusFrame(widget);
		return true;
	},


	createDroppedComponent: function(e){
		var targetWidget = this.getDropTargetWidget(e);
		var targetDomNode = targetWidget.domNode; // domNode of EditArea or parent container
		var srcElem = this.getDragSourceDomNode(e);
		var dim = btp.getDimensions(targetDomNode);
		var props;
		if(srcElem.props){
			props = srcElem.props;
			srcElem.props = null; // clear
		}else{
			props = {};
		}
		var x, y;
		if(e.dragObject && e.dragObject.dragClone){
			var dragClone = e.dragObject.dragClone;
			y = dragClone.offsetTop - dim.top + targetDomNode.parentNode.scrollTop;
			x = dragClone.offsetLeft - dim.left + targetDomNode.parentNode.scrollLeft;
		}else{ // click and release
			var coords;
			if(props.coords){ // ex. ResizeCreationTool stores the drag start position in props.coords
				coords = props.coords;
			}else{
				coords = btp.getEventCoords(e);
			}
			y = coords.y - dim.top + targetDomNode.scrollTop;// - 4; // for some reason, both IE and FF need -4
			x = coords.x - dim.left + targetDomNode.scrollLeft;// - 4;
		}

		if(this._editPartViewer.snapObjectsToGridFlag){
			y = this._editPartViewer.snap(y);
			x = this._editPartViewer.snap(x);
		}
		if(x < 0) x=0;
		if(y < 0) y=0;
		if(this._editPartViewer.fireMessagePosted){
			this._editPartViewer.fireMessagePosted("("+x+","+y+")");
		}
		var name = srcElem.componentClassName;
		if(this._editPartViewer.freeLayoutMode == true){//TODO: should allow customization with config and widget property
			if(!props.__style){
				props.__style = {};
			}
			if(!props.__style.top){
				props.__style.top = y + "px";
			}
			if(!props.__style.left){
				props.__style.left = x + "px";
			}
			if(!props.__style.position){
				props.__style.position = "absolute";
			}
		}
		var widget = this._editPartViewer.createComponent(name, props);
		var info = this._editPartViewer.getComponentInfo(name);
		this.addChildWidgets(widget, info);
		
		return widget;
	},

	addChildWidgets: function(widget, info){
		if(!info || !info.children){ return; }
		var childNodes = btp.getChildNodes(info.children, "child");

		for(var i = 0, len = childNodes.length; i < len; i++){
			var childNode = childNodes[i]; // <child>
			if(childNode.attribute && typeof(childNode.attribute.length) == "undefined"){
				childNode.attribute = [childNode.attribute]; // make it an array
			}

			// set default attribute values if any
			var opt = {};
			for(var j = 0, len2 = childNode.attribute.length; j < len2; j++){
				var attr = childNode.attribute[j];
				if(attr.value){
					opt[attr.name] = attr.value;
				}
			}
			var childName = childNode.name;
			var childWidget = this._editPartViewer.createComponent(childName, opt);

			// repeat for 1-level deeper depth
			this.addChildWidgets(childWidget, childNode);

			widget.addChild(childWidget);
		}
	},

	findRefWidget: function(e){
		var targetWidget = this.getDropTargetWidget(e);
		var refWidget = null;
		if(e.type == "mouseup"){ // Dropped with D&D
			//TODO: what happens when the target is absolute positioning?
			var node = this.getNodeUnderMouse(e);
			var ref = null;
			if(node){
				refWidget = btp.getWidgetByElement(node);
				var isVertical = this.getOrientation(e);
				if(isVertical){
					if(dojo.html.gravity(node, e) & dojo.html.gravity.EAST){
						refWidget = refWidget.getNextSibling();
					}
				}else{
					if(dojo.html.gravity(node, e) & dojo.html.gravity.SOUTH){
						refWidget = refWidget.getNextSibling();
					}
				}
			}
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			// nop
		}
		return refWidget;
	},

	getDragSourceDomNode: function(e){
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			var ds = e.dragSource;
			srcElem = ds.dragObject; // dropped DOM node
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			srcElem = this._editPartViewer.selectedPaletteItem.domNode;
		}
		return srcElem;
	},

	getDragSourceWidget: function(e){
		var sourceWidget;
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			sourceWidget = e.dragSource.data;
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			sourceWidget = this._editPartViewer.selectedPaletteItem;
		}
		return sourceWidget;
	},

	getDropTargetWidget: function(e){
		var targetWidget; // EditArea or parent container
		if(e.type == "mouseup"){ // Dropped with D&D
			targetWidget = e.dropTarget && e.dropTarget.data;
		}else /*if(e.type == "click")*/ { // Dropped with click and release
			targetWidget = btp.getWidgetByElement(e.target);
		}
		return targetWidget;
	},

	getNodeUnderMouse: function(e){
		var dt = e.dropTarget;
		return dt.getNodeUnderMouse();
	},

	// true: orientation is vertical      (caret shape is like |)
	// false: orientation is horizontal   (caret shape is like -)
	getOrientation: function(e){
		var dt = e.dropTarget;
		return dt.vertical;
	},


	// -------------------------------------------------------------------
	// Default Event Handlers
	// -------------------------------------------------------------------

	// Called when the mouse button has been pressed.
	handleMouseDown: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse button has been released.
	handleMouseUp: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse is moved.
	handleMouseMove: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer is moved onto an element.
	handleMouseOver: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer is moved out of an element.
	handleMouseOut: function(e, editPartViewer){
		return false;
	},

	// Called when a mouse click occurs.
	handleClick: function(e, editPartViewer){
		return false;
	},

	// Called when a mouse double-click occurs.
	handleDblClick: function(e, editPartViewer){
		return false;
	},

	// Called when the key is pressed.
	handleKeyDown: function(e, editPartViewer){
		return false;
	},

	// Called when the key is released.
	handleKeyUp: function(e, editPartViewer){
		return false;
	},

	// Called only once during a drag when the drag threshold has been exceeded.
	handleDragStart: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse hovers over an element during dragging.
	handleDragOver: function(e, editPartViewer){
		return false;
	},

	// Called whenever the mouse is being dragged.
	handleDragMove: function(e, editPartViewer){
		return false;
	},

	// Called when the drag is finished. handleDropXX will be called soon.
	handleDragEnd: function(e, editPartViewer){
		return false;
	},

	// Called before checking whether the drop is allowed.
	handleDropStart: function(e, editPartViewer){
		return false;
	},

	// Called when a drop operation is performed.
	handleDrop: function(e, editPartViewer){
		return false;
	},

	// Called when the drop operation has been finished. e.dragStatus has the result.
	handleDropEnd: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer moves out of an element during dragging.
	handleDragOut: function(e, editPartViewer){
		return false;
	}
});
