/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.DefaultSelectionTool");

dojo.require("btp.widget.tool.AbstractTool");
dojo.require("btp.widget.command.CompoundCommand");
dojo.require("btp.widget.command.EditCommand");

dojo.declare(
	"btp.widget.tool.DefaultSelectionTool",
	btp.widget.tool.AbstractTool,
{
	_draggingWidget: null,

	// -------------------------------------------------------------------
	// Event Handlers
	// -------------------------------------------------------------------
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		var dragTracker;

		// Look for a DragTracker from the clicked DOM node
		if(e.target.getDragTracker){ // probably resize knob
			dragTracker = e.target.getDragTracker();
		}

		// Look for a DragTracker from the clicked widget
		if(!dragTracker){
			var elem = e.target;
			while(elem){
				dragTracker = elem.getDragTracker && elem.getDragTracker();
				if(dragTracker){ break; }
				var widget = elem.widget || dijit.byNode(elem); // gfx raw node may have the widget property
				if(widget){
					editPartViewer.setFocus((editPartViewer != widget) ? widget : null);
					dragTracker = widget.getDragTracker && widget.getDragTracker();
					break;
				}
				elem = elem.parentNode;
				if(!elem || elem.tagName == "BODY"){ break; }
			}
		}

		// Look for a DragTracker from the EditArea
		if(!dragTracker){
			dragTracker = editPartViewer.getDragTracker && editPartViewer.getDragTracker();
		}

		// Use the default DragTracker
		if(!dragTracker){
			dragTracker = "btp.widget.tool.DragGhostMoveTracker";
		}

		this.setDragTracker(dragTracker);
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseDown(e, editPartViewer);
		}
	},

	handleMouseUp: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseUp(e, editPartViewer);
		}
	},
	
	handleMouseMove: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseMove(e, editPartViewer);
		}
	},
	
	handleMouseOver: function(e, editPartViewer){
		editPartViewer.domNode.style.cursor = "auto";
	},
	
	handleClick: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleKeyDown(e, editPartViewer);
		}
		var widget = btp.getWidgetByElement(e.target);
		if(!widget){
			return;
		}else if(editPartViewer != widget){
			editPartViewer.setFocus(widget); // Google Maps needs this
		}
	},

	handleKeyDown: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			if(this.getDragTracker().handleKeyDown(e, editPartViewer) == true) return true; 
		}
		if(e.keyCode == 46){ // Del key
			this.del(editPartViewer);
		}
		else if(e.ctrlKey && e.keyCode == 123){ // Ctrl + F12
//			console.open();
		}
		else if(e.ctrlKey && e.keyCode == 90){ // Ctrl + Z
			this.undo();
		}
		else if(e.ctrlKey && e.keyCode == 89){ // Ctrl + Y
			this.redo();
		}
		else if(e.keyCode == 27){ // ESC
			this.setDragTracker(null);
		}
		if(window["btpConfig"] && btpConfig.keyboardNav){
			var diff = btpConfig.moveAmount ? btpConfig.moveAmount : 5;
			if(e.keyCode == dojo.keys.LEFT_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: -diff, y:0});
				if(command) {
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}
			}
			else if(e.keyCode == dojo.keys.RIGHT_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: diff, y:0});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.UP_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: 0, y:-diff});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.DOWN_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: 0, y:diff});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.TAB){
				if(!e.shiftKey){
					var ret = this._selectNextWidget(e, editPartViewer); 
					if(ret){
						dojo.stopEvent(e);
					}
				}else{
					var ret = this._selectPreviousWidget(e, editPartViewer);
					if(ret){
						dojo.stopEvent(e);
					}
				}
			}
			else if(e.keyCode == dojo.keys.ENTER){
				if(editPartViewer.focusWidgets && editPartViewer.focusWidgets[0]){
					var focused = editPartViewer.focusWidgets[0];
					if(focused && focused.openContextMenu){
						focused.openContextMenu();
					}
				}
			}	
		}
	},
	
	// utility function
	_nextNonLineWidget: function(widgets, index){
		var i = index + 1;
		for(; i<widgets.length; i++){
			if(widgets[i] instanceof gdc.widget.AbstractNode){ break;	};
		}
		return i;
	},
	
	_previousNonLineWidget: function(widgets, index){
		var i = index - 1;
		for(; i>= 0; i--){
			if(widgets[i] instanceof gdc.widget.AbstractNode){ break;	};
		}
		return i;
	},
	
	_selectNextWidget: function(e, editPartViewer){
		var widgets = editPartViewer.getChildren();
		var efs = editPartViewer.focusWidgets;
		var cur = -1;
		if(efs && efs[0]){
			for(; cur<widgets.length; cur++){
				if(widgets[cur] == efs[0]){ break; }
			}
		}
		var nextIndex = this._nextNonLineWidget(widgets, cur);
		if(nextIndex >= widgets.length){
			editPartViewer.setFocus(null);			
			return false;
		}
		else {
			editPartViewer.setFocus(widgets[nextIndex]);		
			return true;
		}				
	},
	
	_selectPreviousWidget: function(e, editPartViewer){
		var widgets = editPartViewer.getChildren();
		var efs = editPartViewer.focusWidgets;
		var cur = widgets.length;
		if(efs && efs[0]){
			for(; cur>= 0; cur--){
				if(widgets[cur] == efs[0]){ break; }
			}
		}
		var prevIndex = this._previousNonLineWidget(widgets, cur);
		if(prevIndex < 0 ){
			editPartViewer.setFocus(null, true);
			return false;
		} else {
			editPartViewer.setFocus(widgets[prevIndex]);		
			return true;
		}
	},
	
	// -------------------------------------------------------------------
	// Drop Target Handlers
	// -------------------------------------------------------------------
	handleDragStart: function(e, editPartViewer){
		if(e.dropTarget.data != editPartViewer){ return; } // dicard events for components on the editor
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDragStart(e, editPartViewer);
		}
	},

	handleDropStart: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDropStart(e, editPartViewer);
		}
	},

	handleDrop: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDrop(e, editPartViewer);
		}
		this.setEditPartViewer(editPartViewer);
		var targetWidget = this.getDropTargetWidget(e);
		var type = e.dragSource.type;
		var ret;
		if(targetWidget[type+"DropHandler"]){
			ret = targetWidget[type+"DropHandler"](e, editPartViewer);
		}else{
			ret = this[type+"DropHandler"](e, editPartViewer);
		}
		this.getEditDomain().loadDefaultTool();
		e.returnValue = ret; // [custom property]
	},

	handleDropEnd: function(e, editPartViewer){
		btp.dnd.dragManager.currentDropTarget = null;
		this.setDragTracker(null);
	},


	// -------------------------------------------------------------------
	// Internal methods
	// -------------------------------------------------------------------
	del: function(editPartViewer){
		// TODO: Can/Should we call an action here? We need to discuss it after the summer vacation...
		if(editPartViewer.del){
			editPartViewer.del();  // action
		}
/*
		if(editPartViewer.focusWidgets && editPartViewer.focusWidgets[0]){
			var command = new btp.widget.command.EditCommand.RemoveChildCommand(editPartViewer.focusWidgets[0]);
			this.getEditDomain().executeCommand(command);
		}
*/
	},
	
	undo: function(){
		this.getEditDomain().getCommandStack().undo_it();
	},
	
	redo: function(){
		this.getEditDomain().getCommandStack().execute_it();
	},


	createBindingDropCommand: function(e, editPartViewer){
		var ds = e.dragSource;
		var treeNodeWidget = ds.treeNode; // TreeNode widget
		var targetWidget = this.getDropTargetWidget(e);

		var model = null;
		var models = dojo.widget.manager.getWidgetsByType("Model"); // Hmm.. prefix doesn't seem to be allowed
		var ids = {};
		for(var i = 0, len = models.length; i < len; i++){
			var m = models[i];
			if(m.src == treeNodeWidget.data){
				model = m;
				break;
			}
			ids[m.id] = true;
		}

		var ref = treeNodeWidget.ref ? treeNodeWidget.ref : ".";
		if(model && targetWidget.modelId == model.id && targetWidget.ref == ref){
			return null; // nothing has been changed
		}

		var compoundCommand = new btp.widget.command.CompoundCommand();

		compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(targetWidget));
		if(targetWidget.setTitle){
			var titleInfo = editPartViewer.getAttributeInfo(targetWidget, "title");
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, titleInfo, treeNodeWidget.title));
		}

		var modelIdInfo = editPartViewer.getAttributeInfo(targetWidget, "modelId");
		var refInfo = editPartViewer.getAttributeInfo(targetWidget, "ref");
		if(model){
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, model.id));
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
		}else{
			// <btp:model id="model1" src="some.xml"></btp:model>
			var modelId;
			for(var i = 1; ; i++){
				if(!ids["model"+i]){
					modelId = "model"+i;
					break;
				}
			}
			var opt = {
				id: modelId,
				src: treeNodeWidget.data,
				format: treeNodeWidget.format
			};
			var widget = editPartViewer.createLiveWidget("btp:model", opt);

			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, modelId));
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
			compoundCommand.append(new btp.widget.command.EditCommand.InsertBeforeCommand(editPartViewer, widget, editPartViewer.getChildren()[0]));

		}
		compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(targetWidget));
		return compoundCommand;
	},

	bindingDropHandler: function(e, editPartViewer){
		var command = this.createBindingDropCommand(e, editPartViewer);
		if(command){
			this.getEditDomain().executeCommand(command);
		}
		return true;
	},

	getDragTracker: function(){
		return this.dragTracker;
	},

	setDragTracker: function(/*String|DragTracker*/newDragTracker){
		if(typeof(newDragTracker) == "string"){
			if(!this._dragTrackerCache){ this._dragTrackerCache = []; }
			if(this._dragTrackerCache[newDragTracker]){
				newDragTracker = this._dragTrackerCache[newDragTracker];
			}else{
				dojo["require"](newDragTracker);
				var impl = dojo.getObject(newDragTracker);
				newDragTracker = this._dragTrackerCache[newDragTracker] = new impl();
				newDragTracker.setEditDomain(this.getEditDomain());
			}
		}
		this.dragTracker = newDragTracker;
	},
	
	createMoveComponentCommand: function(e, editPartViewer, diff) {
		if(editPartViewer.focusWidgets && editPartViewer.focusWidgets.length > 0) {
			var widget = editPartViewer.focusWidgets[0];
			var x = widget.domNode.offsetLeft + diff.x;
			var y = widget.domNode.offsetTop + diff.y;
			if(this.snapObjectsToGridFlag){
				x = this.snap(x);
				y = this.snap(y);
			}
			if(x < 0){ x=0; }
			if(y < 0){ y=0; }
			var toDim = {
				top: y + "px",
				left: x + "px"
			};
			var fromDim = {
				top: widget.domNode.offsetTop + "px",
				left: widget.domNode.offsetLeft + "px"
			};
			var command = new btp.widget.command.EditCommand.MoveCommand(widget, toDim, fromDim);
			return command;			
		} else {
			return null;
		}
	}	
});
