/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.DragMoveTracker");

dojo.require("btp.widget.tool.AbstractTool");
dojo.require("btp.dnd.DragGhostMoveSource");

// summary:
//		A DragTracker that moves components on the editor with drag and drop operation.

dojo.declare(
	"btp.widget.tool.DragMoveTracker",
	btp.widget.tool.AbstractTool,
{
	_dragGhost: false, // true: drag a dashed frame, false: drag an actual component
	_moveConn: null,

	handleMouseDown: function(e, editPartViewer){
		// handle left buttun down only
		var leftButton = dojo.isIE ? 1 : 0;
		if(e.button != leftButton) return;
		
		this.setEditPartViewer(editPartViewer);
		var widget = btp.getWidgetByElement(e.target);
		if(!widget || editPartViewer == widget){ return; }
		this._draggingWidget = widget;
		this.enableDragMove(widget);
	},

	enableDragMove: function(widget){
		var ds = widget.__ds;
		if(!ds){
			var node = btp.isGfxWidget(widget) ? widget.getShape().rawNode : widget.domNode;
			if(this._dragGhost){
				ds = widget.__ds = new btp.dnd.DragGhostMoveSource(node, "component", widget);
			}else{
				ds = widget.__ds = new btp.dnd.DragMoveSource(node, "component", widget);
			}
			ds.setDragTarget(widget.domNode);
			ds.offsetParent = this._editPartViewer.domNode;
			if(btp.isGfxWidget(widget)){
				ds.positioning = "absolute";
				ds.useCurrentPositioning = false; // always absolute
				ds.refNode = widget.domNode; // FF cannot get dimension info from an SVG node
			}else{
				ds.targetShouldShowCaret = true;
			}
			dojo.connect(ds, "onDragStart", this, "onDragStart");
			dojo.connect(ds, "onDragEnd", this, "onDragEnd");
			ds.registered = true; // [custom property]
			return;
		}
		if(ds.registered){ return; }
		ds.reregister();
		ds.registered = true; // [custom property]
	},

	disableDragMove: function(widget){
		var ds = widget.__ds;
		if(!ds || !ds.registered){ return; }
		ds.unregister();
		ds.registered = false; // [custom property]
	},

	onDragStart: function(e){
		if(!this._dragGhost){
			this._editPartViewer.removeFocus();
		}
		this._moveConn = dojo.connect(btp.dnd.dragManager, "onMouseMove", this, "onDragMove");
		var widget = this._draggingWidget;
		if(widget && widget.hlp_onWidgetDragStart){
			widget.hlp_onWidgetDragStart(e);
		}
	},

	onDragMove: function(e){
		var ds = btp.dnd.dragManager.currentDragSource;
		var y = ds.dragClone.offsetTop;
		var x = ds.dragClone.offsetLeft;
		if(this.snapObjectsToGridFlag){
			x = this.snap(x);
			y = this.snap(y);
			ds.dragClone.style.top = y + "px";
			ds.dragClone.style.left = x + "px";
		}
		
		// EXPERIMENTAL CODE START
		// (must improve later)
		if(btpConfig && btpConfig.autoScroll){
			
			var threshold = 10;
			var scrollAmount = 5;
			
			var editArea = this._editPartViewer.domNode;
			
			var editorWidth = editArea.clientWidth;
			var editorHeight = editArea. clientHeight;
			var editorOffset = dojo.coords(editArea);
			
			var ds = btp.dnd.dragManager.currentDragSource;
			var dsLeft = ds.dragClone.offsetLeft; 
			var dsTop = ds.dragClone.offsetTop; 
			
			var pos = {x: e.clientX - editorOffset.x, y: e.clientY - editorOffset.y};
			var currentLeft = editArea.scrollLeft;
			var currentTop = editArea.scrollTop;
			if(pos.x < threshold){
				editArea.scrollLeft -= scrollAmount;
				x -= scrollAmount;
				ds.dragClone.style.left = x + "px";
				ds.dragStartCssPosition.x += editArea.scrollLeft - currentLeft;
			} else if ( pos.x > editorWidth - threshold){
				editArea.scrollLeft += scrollAmount;
				x += scrollAmount;
				ds.dragClone.style.left = x + "px";
				ds.dragStartCssPosition.x += editArea.scrollLeft - currentLeft;
			}
			if(pos.y < threshold){
				editArea.scrollTop -= scrollAmount;
				y -= scrollAmount;
				ds.dragClone.style.top = y + "px";
				ds.dragStartCssPosition.y += editArea.scrollTop - currentTop;
			} else if( pos.y > editorHeight - threshold){
				editArea.scrollTop += scrollAmount;
				y += scrollAmount;
				ds.dragClone.style.top = y + "px";
				ds.dragStartCssPosition.y += editArea.scrollTop - currentTop;
			}
		}
		// EXPERIMENTAL CODE END
				
		if(this._editPartViewer.fireMessagePosted){
			this._editPartViewer.fireMessagePosted("("+x+","+y+")");
		}
		var widget = this._draggingWidget;
		if(!this._dragGhost && btp.isGfxWidget(widget)){
			widget.update();
		}
		if(widget && widget.hlp_onWidgetDragMove){
			widget.hlp_onWidgetDragMove(e);
		}
	},

	onDragEnd: function(e){
		dojo.disconnect(this._moveConn);
		var widget = this._draggingWidget;
		if(!widget){ return; }
		var targetWidget = this.getDropTargetWidget(e);
		if(targetWidget && targetWidget.componentMoveHandler){
			targetWidget.componentMoveHandler(e, this._editPartViewer);
		}else{
			this.componentMoveHandler(e, this._editPartViewer);
		}
		this.disableDragMove(this._draggingWidget);
		this._draggingWidget = null;

		if(widget.hlp_onWidgetDragEnd){
			widget.hlp_onWidgetDragEnd(e);
		}
	},

	componentMoveHandler: function(e, editPartViewer){
		var sourceWidget = this._draggingWidget;
		var targetWidget = this.getDropTargetWidget(e) || this._editPartViewer;
		if(sourceWidget.getParent() != targetWidget // moved to the different level
			&& sourceWidget != targetWidget){ // sourceWidget was dropped on other than itself
			var refWidget = this.findRefWidget(e);
			return this.reparentComponent(e, refWidget);
		}else{
			return this.moveComponent(e);
		}
	},

	moveComponent: function(e){
		var widget = this._draggingWidget;
		var command = this.createMoveComponentCommand(e, widget);
		this._editDomain.executeCommand(command);
		this._editPartViewer.setFocusFrame(widget);
		return true;
	},

	createMoveComponentCommand: function(e, widget){
		var ds = e.dragSource;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var x = ds.dragStartCssPosition.x + dx;
		var y = ds.dragStartCssPosition.y + dy;
		if(this._editPartViewer.snapObjectsToGridFlag){
			x = this._editPartViewer.snap(x);
			y = this._editPartViewer.snap(y);
		}
		if(x < 0) x = 0;
		if(y < 0) y = 0;
		var toDim = {
			top: y + "px",
			left: x + "px"
		};
		var fromDim;
		if(!this._dragGhost){
			fromDim = {
				top: ds.dragStartCssPosition.y + "px",
				left: ds.dragStartCssPosition.x + "px"
			};
		}
		var command = new btp.widget.command.CompoundCommand();
		if(ds.positioning && dojo.style(widget.domNode, "position") != ds.positioning){
			command.append(new btp.widget.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning));
		}
		command.append(new btp.widget.command.EditCommand.MoveCommand(widget, toDim, fromDim));
		return command;
	}
});
