/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.EditDomain");

dojo.declare("btp.widget.tool.EditDomain", null, {
	_commandStackName: null,
	_commandStack: null,
	_defaultToolName: null,
	_defaultTool: null,
	_activeTool: null,
	_paletteViewer: null,
	
	_events: [
		"MouseDown", "MouseUp", "MouseMove", "MouseOver", "MouseOut",
		"Click", "DblClick", "KeyDown", "KeyUp",
		"DragStart", "DragOver", "DragMove", "DragEnd", "DropStart",
		"Drop", "DropEnd", "DragOut"
	],

	constructor: function(defaultTool, defaultCommandStack){
		if(!defaultTool){
			defaultTool = "btp.widget.tool.DefaultSelectionTool";
		}
		this._defaultToolName = defaultTool;

		if(!defaultCommandStack){
			defaultCommandStack = "btp.widget.command.CommandProcessor";
		}
		this._commandStackName = defaultCommandStack;

		for(var i = 0, len = this._events.length; i < len; i++){
			var eventName = this._events[i];
			var lowerName = eventName.charAt(0).toLowerCase() + eventName.substring(1);
			this[lowerName] = this._createHandler(eventName);
		}
	},

	_createHandler: function(eventName){
		return function(e, editPartViewer){
			if(this.getActiveTool()["handle"+eventName]){
				this.getActiveTool()["handle"+eventName](e, editPartViewer);
			}
		};
	},

	setActiveTool: function(tool){
		if(this._activeTool){
			this._activeTool.deactivate();
		}
		this._activeTool = tool;
		this._activeTool.setEditDomain(this);
		this._activeTool.activate();
	},
	
	getActiveTool: function(){
		if(!this._activeTool){
			this.setActiveTool(this.getDefaultTool());
		}
		return this._activeTool;
	},
	
	getDefaultTool: function(){
		if(!this._defaultTool){
			dojo["require"](this._defaultToolName);
			var cls = dojo.getObject(this._defaultToolName);
			this._defaultTool = new cls();
		}
		return this._defaultTool;
	},

	loadDefaultTool: function(){
		this.setActiveTool(this.getDefaultTool());
	},
	
	setPaletteViewer: function(paletteViewer){
		this._paletteViewer = paletteViewer;
		dojo.connect(paletteViewer, "activeToolChanged", this, "activeToolChanged");
	},

	getCommandStack: function(){
		if(!this._commandStack){
			dojo["require"](this._commandStackName);
			var cls = dojo.getObject(this._commandStackName);
			this._commandStack = new cls();
		}
		return this._commandStack;
	},
	
	setCommandStack: function(stack){
		this._commandStack = stack;
	},
	
	executeCommand: function(command){
		this.getCommandStack().push(command);
		this.getCommandStack().execute_it();
	},
	
	undo: function(){
		this.getCommandStack().undo_it();
	},
	
	redo: function(){
		this.getCommandStack().execute_it();
	},
	
	clear: function(){
		this.getCommandStack().clear();
	},
	
	activeToolChanged: function(msg){
		if(msg.tool){
			this.setActiveTool(msg.tool);
		}else{
			this.loadDefaultTool();
		}
	}
});
