/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.tool.ResizeTracker");

dojo.require("btp.widget.tool.AbstractTool");
dojo.require("btp.dnd.DragMoveSource");
dojo.require("btp.dnd.DragManager");

// summary:
//		A DragTracker for dragging a resize handle.

dojo.declare(
	"btp.widget.tool.ResizeTracker",
	btp.widget.tool.AbstractTool,
{
	_rubberBand: null,

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	stopResizing: function(){
		var rubberBand = this._editPartViewer._rubberBand;
		if(rubberBand.parentNode != this._editPartViewer.domNode){ return; } // already canceled

		this._editPartViewer.domNode.removeChild(rubberBand);
		this._editPartViewer.domNode.removeChild(this._editPartViewer.getMoveTarget());
		this._editPartViewer.domNode.style.cursor = "";
	},

	// ======== Event Handlers ====================================
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
	},

	handleDragStart: function(e, editPartViewer){
		var movingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		var widget = this._editPartViewer.focusWidgets[0];

		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand){
			rubberBand = this._editPartViewer._rubberBand = dojo.doc.createElement("DIV");
			rubberBand.style.fontSize = "1px";
			rubberBand.style.position = "absolute";
			rubberBand.style.border = "1px dashed black";
		}
		this._editPartViewer.domNode.appendChild(rubberBand);

		var dim = widget.__dim = btp.getStyleDimensions(widget, this._editPartViewer);
		rubberBand.style.top = dim.top + "px";
		rubberBand.style.left = dim.left + "px";
		rubberBand.style.width = dim.width + "px";
		rubberBand.style.height = dim.height + "px";

		var moveTarget = this._editPartViewer.getMoveTarget();
		moveTarget.style.top = movingKnob.offsetTop + "px";
		moveTarget.style.left = movingKnob.offsetLeft + "px";
		this._editPartViewer.domNode.appendChild(moveTarget);
		this._editPartViewer.domNode.style.cursor = "crosshair";
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e, editPartViewer){
		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this._editPartViewer.domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var movingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		var widget = this._editPartViewer.focusWidgets[0];
		var dim = widget.__dim;
		var rubberBand = this._editPartViewer._rubberBand;
		var knobPosition = movingKnob.knobPosition; // [custom property]
		var info = widget.hlp_getComponentInfo();

		var moveTarget = this._editPartViewer.getMoveTarget();
		var mouseX = moveTarget.offsetLeft;
		var mouseY = moveTarget.offsetTop;
		if(dojo.isMozilla){
			mouseX--;
			mouseY--;
		}

		var dirs = widget.__resizeDirections;
		if(!e.shiftKey && !e.ctrlKey){ // resize freely
			if(dirs & this._NORTH
				&& (knobPosition == 0 || knobPosition == 1 || knobPosition == 2)){ // NORTH
				var h = dim.top - mouseY + dim.height;
				if(h > 0){
					rubberBand.style.top = this._editPartViewer.snap(mouseY) + "px";
					rubberBand.style.height = this._editPartViewer.snap(h) + "px";
				}
			}else if(dirs & this._SOUTH
				&& (knobPosition == 4 || knobPosition == 5 || knobPosition == 6)){ // SOUTH
				var h = mouseY - dim.top;
				if(h > 0){
					rubberBand.style.height = this._editPartViewer.snap(h) + "px";
				}
			}
	
			if(dirs & this._EAST
				&& (knobPosition == 2 || knobPosition == 3 || knobPosition == 4)){ // EAST
				var w = mouseX - dim.left;
				if(w > 0){
					rubberBand.style.width = this._editPartViewer.snap(w) + "px";
				}
			}else if(dirs & this._WEST
				&& (knobPosition == 0 || knobPosition == 6 || knobPosition == 7)){ // WEST
				var w = dim.left - mouseX + dim.width;
				if(w > 0){
					rubberBand.style.left = this._editPartViewer.snap(mouseX) + "px";
					rubberBand.style.width = this._editPartViewer.snap(w) + "px";
				}
			}
		}else if(e.shiftKey && !e.ctrlKey){ // resize keeping the aspect ratio
			if((dirs & this._NORTH) && (dirs & this._EAST) && knobPosition == 2){ // top-right
				var w = mouseX - dim.left;
				var h = dim.top + dim.height - mouseY;
				if(w > 0 && h > 0){
					var ratioX = w / dim.width;
					var ratioY = h / dim.height;
				}
			}
		}else if(!e.shiftKey && e.ctrlKey){ // resize keeping the center
		}else if(e.shiftKey && e.ctrlKey){ // resize keeping the aspect ratio and the center
		}
	},

	handleMouseUp: function(e, editPartViewer){
		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this._editPartViewer.domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var widget = this._editPartViewer.focusWidgets[0];
		var parentDim = btp.getDimensions(widget.getParent().domNode);
		var editAreaDim = btp.getDimensions(this._editPartViewer.domNode);
		var toBox = dojo.contentBox(rubberBand);
		var toDim = {
			top: (rubberBand.offsetTop - parentDim.top + editAreaDim.top) + "px",
			left: (rubberBand.offsetLeft - parentDim.left + editAreaDim.left) + "px",
			width: toBox.w + "px",
			height: toBox.h + "px"
		};

		var command = new btp.widget.command.EditCommand.ResizeCommand(widget, toDim);
		this._editPartViewer.getEditDomain().executeCommand(command);

		this.stopResizing();
		this._editPartViewer.fireSelectionChanged(widget);
	},

	handleKeyDown: function(e, editPartViewer){
		if(e.keyCode == 27){ // ESC
			this.stopResizing();
		}
	}
});
