/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc._base.shape");

dojo.require("dojox.gfx");
dojo.require("dojox.gfx.shape");

gdc._base.shape.getClipPathId = function(width, height, dx, dy){
	var id = "w"+width+"h"+height;
	if(dx){
		id += "x"+dx+"y"+dy;
	}
	return id;
}

gdc._base.shape.getClipPath = function(width, height, dx, dy){
	var id = gdc._base.shape.getClipPathId(width, height, dx, dy);
	var editor = btp.getEditors()[0];	
	var defNode = editor.surface.defNode;
	var clipPath = document.getElementById(id);
	if(!clipPath){
		clipPath = dojo.doc.createElementNS(dojox.gfx.svg.xmlns.svg, "clipPath");
		clipPath.setAttributeNS(null, "id", id);
		defNode.appendChild(clipPath);
		var rect = dojo.doc.createElementNS(dojox.gfx.svg.xmlns.svg, "rect");
		height += 20;	// temporary hack
		rect.setAttributeNS(null, "width", width);
		rect.setAttributeNS(null, "height", height);
		if(dx){
			rect.setAttributeNS(null, "transform", "translate("+dx+","+(dy-20)+")");
		}
		clipPath.appendChild(rect);
	}
	return clipPath;
}

gdc._base.shape.Creator = {
	createGdcImage: function(image){ //for using transparent pngs in IE6 
		var ret = this.createImage(image);
		if(dojox.gfx.renderer == "vml"){
			ret.rawNode.style.filter = "";
			var img = ret.rawNode.firstChild;
			var imgSrc = img.src;
			if(imgSrc.match(/png$/i)) {
				var doDomReplace = function() {
					var imgStyle = "display: inline-block;" + img.style.cssText;
					var span = "<span style=\"" 
					 + "width:" + img.width + "px; height:" + img.height + "px;" 
					 + imgStyle + ";"
			         + "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader"
			         + "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>"
					img.outerHTML = span;
				};
				setTimeout(doDomReplace, 0);
			}
		}
		return ret;
	},
	
	createGdcText: function(text){ //for clipping with defined size
		var ret;
		if(dojox.gfx.renderer == "vml"){
			ret = this.createText(text);
			var clipText = "rect(0px "+text.width+"px "+text.height+"px 0px)";
			ret.rawNode.style.clip = clipText;
			return ret;
		} else {
			ret = this.createGroup();
			var clipPathId = gdc._base.shape.getClipPathId(text.width, text.height);
			var clipPath = gdc._base.shape.getClipPath(text.width, text.height);
			ret.rawNode.setAttributeNS(null, "clip-path", "url(#"+ clipPathId +")");
			var textElem = ret.createText(text);
			
			textElem.setTransform = function(transform){
				dojox.gfx.Shape.prototype.setTransform.apply(this, arguments);
				var newClipPathId = gdc._base.shape.getClipPathId(text.width, text.height, transform.dx, transform.dy);
				gdc._base.shape.getClipPath(text.width, text.height, transform.dx, transform.dy);
				ret.rawNode.setAttributeNS(null, "clip-path", "url(#"+ newClipPathId +")");
			}
			return textElem;
		}
		
	}
}
dojo.extend(dojox.gfx.Group, gdc._base.shape.Creator);
dojo.extend(dojox.gfx.Surface, gdc._base.shape.Creator);

// hack for IE performance problem
if(dojox.gfx.renderer == "vml"){
	dojox.gfx.Image.prototype.setShape = function(newShape){
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var firstChild = this.rawNode.firstChild;
		if(firstChild.tagName == "SPAN"){
			firstChild.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+shape.src+"\', sizingMethod='scale')";
		} else {
			firstChild.src = shape.src;
		}
        if(shape.width || shape.height){
			this.rawNode.style.width = firstChild.style.width  = shape.width;
			this.rawNode.style.height = firstChild.style.height = shape.height;
        }
		return this.setTransform(this.matrix);
	}
	
	dojox.gfx.Image.prototype._applyTransform = function(image){
		var matrix = this._getRealMatrix();
		if(!matrix) return this;
		matrix = dojox.gfx.matrix.multiply(matrix, {dx: this.shape.x, dy: this.shape.y});
		var imgNode = this.rawNode.firstChild;
		var shape = this.shape;
		if(shape.width || shape.height){
			imgNode.style.width = shape.width * matrix.xx;
			imgNode.style.height = shape.height * matrix.yy;
		}
		this.rawNode.style.left = matrix.dx;
		this.rawNode.style.top = matrix.dy;
		return this;			
	}		
}
