/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.AbstractLine");

dojo.require("gdc.widget.GfxWidget");
dojo.require("gdc.widget.GfxContainer");

dojo.declare(
	"gdc.widget.AbstractLine",
	[gdc.widget.GfxWidget, gdc.widget.GfxContainer],
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"

	// default line styles (see also dojox.gfx.defaultStroke)
	lineColor: "black",
	lineStyle: "solid",
	lineWidth: 2, // Integer
	lineCap: "butt",
	lineJoin: 4, // Integer

	sourceDecoration: "", // decoration class name
	sourceDecorationProps: null, // properties for the source decoration
	targetDecoration: "", // decoration class name
	targetDecorationProps: null, // properties for the target decoration

	sourceDecorationObj: null, // decoration object
	targetDecorationObj: null, // decoration object

	postMixInProperties: function(){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')');
		}
		if(this.sourceDecoration){
			var dec = this.sourceDecoration;
			this.sourceDecoration = null;
			this.setSourceDecoration(dec);
		}
		if(this.targetDecoration){
			var dec = this.targetDecoration;
			this.targetDecoration = null;
			this.setTargetDecoration(dec);
		}
	},

	postCreate: function(){
		this.domNode.style.display = "none";
		this.inherited("postCreate", arguments);
	},

	createTargetShape: function(parent){
		if(this.sourceDecorationObj){
			this.sourceDecorationObj.createTargetShape(parent);
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.createTargetShape(parent);
		}
	},

	restoreShape: function(){
		this.inherited("restoreShape", arguments);
		if(this.sourceDecorationObj){
			this.recoverLostInfo(this.sourceDecorationObj.shape);
		}
		if(this.targetDecorationObj){
			this.recoverLostInfo(this.targetDecorationObj.shape);
		}
	},

	update: function(props){
		this.inherited("update", arguments);
		var shape = this.shape;
		if(!shape){ return; }

		if((props && props.forceUpdate) || !this._cur
			|| this._cur.color != this.lineColor
			|| this._cur.style != this.lineStyle
			|| this._cur.width != this.lineWidth
			|| this._cur.cap != this.lineCap
			|| this._cur.join != this.lineJoin){
			shape.setStroke({
				color: dojox.gfx.normalizeColor(this.lineColor),
				style: this.lineStyle,
				width: this.lineWidth,
				cap: this.lineCap,
				join: this.lineJoin
			});
		}

		// save the current status
		this._cur = {
			color: this.lineColor,
			style: this.lineStyle,
			width: this.lineWidth,
			cap: this.lineCap,
			join: this.lineJoin,
			fillColor: this.fillColor
		};

		if(this.sourceDecorationObj){
			this.sourceDecorationObj.update();

			var pt = this.points[0];
			this.sourceDecorationObj.setLocation(pt);

			var angle = this.calculateAngle(this.points[1], pt);
			this.sourceDecorationObj.setRotation(angle);
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.update();

			var pt = this.points[this.points.length - 1];
			this.targetDecorationObj.setLocation(pt);

			var angle = this.calculateAngle(this.points[this.points.length - 2], pt);
			this.targetDecorationObj.setRotation(angle);
		}
	},
	
	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	},

	removeShape: function(){
		if(this.sourceDecorationObj){
			this.sourceDecorationObj.removeShape();
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.removeShape();
		}
		this.inherited("removeShape", arguments);
	},

	calculateAngle: function(pt0, pt1){
		return 0;
	},

	// Called when points are added or changed. You may want to connect to this.
	onPointsChanged: function(){
	},

	_setDecoration: function(/*String|Decoration*/dec, which){
		if(!dec){
			this[which+"Decoration"] = "";
			this[which+"DecorationObj"] = null;
			return;
		}
		if(this[which+"Decoration"] == dec || this[which+"DecorationObj"] == dec){ return; }
		var newInstance;
		if(typeof(dec) == "string"){
			this[which+"Decoration"] = dec;
			dojo["require"](dec);
			var decorationImpl = dojo.getObject(dec);
			var props = this[which+"DecorationProps"];
			if(props && typeof(props) == "string"){
				props = eval('('+props+')');
			}
			newInstance = new decorationImpl(this, props);
		}else{
			this[which+"Decoration"] = dec.declaredClass;
			newInstance = dec;
		}
		if(this[which+"DecorationObj"]){
			this[which+"DecorationObj"].destroy(); // destroy the old instance
			newInstance.createTargetShape(this.groupBox);
		}
		this[which+"DecorationObj"] = newInstance;
		this.update();
	},

	_setDecorationProps: function(/*String|Object*/props, which){
		if(props && typeof(props) == "string"){
			props = eval('('+props+')');
		}
		this[which+"DecorationProps"] = props;
		if(this[which+"DecorationObj"]){
			this[which+"DecorationObj"].setProps(props);
		}
		this.update();
	},

	setSourceDecoration: function(/*String|Decoration*/dec){
		this._setDecoration(dec, "source");
	},

	setSourceDecorationProps: function(/*String|Object*/props){
		this._setDecorationProps(props, "source");
	},

	setTargetDecoration: function(dec){
		this._setDecoration(dec, "target");
	},

	setTargetDecorationProps: function(/*String|Object*/props){
		this._setDecorationProps(props, "target");
	},

	// Adds the passed point to the Polyline.
	addPoint: function(/*{x:#,y:#}*/pt){
		this.points.push(pt);
	},

	// Inserts a given point at a specified index in the Polyline.
	insertPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points.splice(index, 0, pt);
	},

	// Erases the Polyline and removes all of its points.
	removeAllPoints: function(){
		this.points = [];
	},

	// Removes a point from the Polyline.
	removePoint: function(index){
		this.points.splice(index, 1);
	},

	// Sets the points at both extremes of the Polyline.
	setEndpoints: function(/*{x:#,y:#}*/start, /*{x:#,y:#}*/end){
		this.setStart(start);
		this.setEnd(end);
	},

	// Sets the point at index to the point pt.
	setPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points[index] = pt;
	},


// ======================= Setters/Getters =================================

	// Returns the first point in the Polyline.
	getStart: function(){
		if(this.points.length == 0){ return null; }
		return this.points[0];
	},

	// Sets the start point of the Polyline.
	setStart: function(/*{x:#,y:#}*/start){
		if (this.points.length == 0){
			this.addPoint(start);
		}else{
			this.setPoint(start, 0);
		}
	},


	// Returns the last point in the Polyline.
	getEnd: function(){
		if(this.points.length == 0){ return null; }
		return this.points[this.points.length - 1];
	},

	// Sets the end point of the Polyline.
	setEnd: function(/*{x:#,y:#}*/end){
		if (this.points.length < 2){
			this.addPoint(end);
		}else{
			this.setPoint(end, this.points.length - 1);
		}
	},


	// Returns the points in this Polyline.
	getPoints: function(){
		return this.points;
	},

	// Sets the list of points to be used by this polyline.
	// Removes any previously existing points.
	setPoints: function(/*Array|String*/points){
		if (!points){
			this.points = [];
		}else if (typeof(points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+points+')');
		}else{
			this.points = points;
		}
	},
	
	
	getLineColor: function(){
		return this.lineColor;
	},

	setLineColor: function(color){
		this.lineColor = dojox.gfx.normalizeColor(color);
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.color = color;
		this.shape.setStroke(strokeStyle);
	},


	getLineStyle: function(){
		return this.lineStyle;
	},

	setLineStyle: function(style){
		this.lineStyle = style;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.style = style;
		this.shape.setStroke(strokeStyle);
	},


	getLineWidth: function(){
		return this.lineWidth;
	},

	setLineWidth: function(width){
		this.lineWidth = width - 0;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.width = width - 0;
		this.shape.setStroke(strokeStyle);
	},


	getLineCap: function(){
		return this.lineCap;
	},

	setLineCap: function(cap){
		this.lineCap = cap;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.cap = cap;
		this.shape.setStroke(strokeStyle);
	},


	getLineJoin: function(){
		return this.lineJoin;
	},

	setLineJoin: function(join){
		this.lineJoin = join - 0;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.join = join - 0;
		this.shape.setStroke(strokeStyle);
	}
});
