/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.AbstractNode");

dojo.require("gdc.widget.GfxWidget");

dojo.declare(
	"gdc.widget.AbstractNode",
	gdc.widget.GfxWidget,
{
	left: 0, // Integer
	top: 0, // Integer
	width: 200, // Integer
	height: 150, // Integer
	sourceAnchor: "", // anchor class name (ex. "gdc.widget.anchor.ChopboxAnchor")
	targetAnchor: "", // anchor class name (ex. "gdc.widget.anchor.ChopboxAnchor")
	sourceAnchorName: "base", // anchor name
	targetAnchorName: "base", // anchor name

	sourceAnchorObj: null, // anchor object
	targetAnchorObj: null, // anchor object

	postMixInProperties: function(){
		if(this.sourceAnchor || this.targetAnchor){
			// mixin connectable.js to make this widget a connectable node
			dojo["require"]("gdc.widget.support.connectable");
			var props = dojo.getObject("gdc.widget.support.connectable");
			dojo.mixin(this, props);
		}
	},

	postCreate: function(){
		this.domNode.style.position = "absolute";
		this.domNode.style.zIndex = "-1"; // dummy domNode must be lower than shape nodes to receive events from shape nodes.
		this.domNode.style.top = this.top + "px";
		this.domNode.style.left = this.left + "px";
		this.domNode.style.width = this.width + "px";
		this.domNode.style.height = this.height + "px";
		this.inherited("postCreate", arguments);
	},
	
	createShape: function() {
		this.inherited("createShape", arguments);
		
		//@@must improve later@@ (should be implemented in gdl)
		if(this.domNode.dropTarget){
			this.domNode.dropTarget.isAcceptable = function(ds, dt) {
				var dsWidget = ds.data;
				var dtWidget = dt.data;
				
				if(ds.type != "connection") {
					if(!dsWidget && !dtWidget){ return true; }
					for(var w = dtWidget; w; w = w.parent){
						if(w == dsWidget){
							return false;
						}
					}
					return true;
				} else {
					if(dtWidget.isTargetConnectionAcceptable(dsWidget)) return true;
					else return false;
				}
			}
		}
	},	
	
	// subclass must implement
	getShapeInfo: function(){
		return null;
	},

	// subclass's createTargetShape() should call this after the target shape is created
	createConnectionAnchors: function(){
		if(this.sourceAnchor){
			dojo["require"](this.sourceAnchor);
			var anchorImpl = dojo.getObject(this.sourceAnchor);
			this.sourceAnchorObj = new anchorImpl(this);
			this.registerSourceConnectionAnchor(this.getShape().rawNode, this.sourceAnchorName, this.sourceAnchorObj);
		}
		if(this.targetAnchor){
			dojo["require"](this.targetAnchor);
			var anchorImpl = dojo.getObject(this.targetAnchor);
			this.targetAnchorObj = new anchorImpl(this);
			this.registerTargetConnectionAnchor(this.getShape().rawNode, this.targetAnchorName, this.targetAnchorObj);
		}
	},

	// subclass may override
	update: function(props){
		this.inherited("update", arguments);
		if(!this.shape){ return; }
// TODO: Is this really needed?
//		if(dojo.isMozilla){
//			this.shape.rawNode.offsetWidth = this.width;
//			this.shape.rawNode.offsetHeight = this.height;
//		}
		this.setPosition();
		this.setShape();
	},

	setPosition: function(){
		// Respect the DOM information. If this node is dropped on the canvas
		// from the palette, only the DOM information is updated.
		this.top = this.domNode.offsetTop;
		this.left = this.domNode.offsetLeft;
		this.width = this.domNode.offsetWidth;
		this.height = this.domNode.offsetHeight;

		var shape = this.getShape();
		if(!shape){ return; }
		var dx = this.left;
		var dy = this.top;
		shape.setTransform({ dx:dx, dy:dy });
		if(this.nodeMoved){
			this.nodeMoved(this);
		}
	},

	setShape: function(){
		var shapeInfo = this.getShapeInfo();
		if(!shapeInfo){ return; }
		var current = this.shape.getShape();
		if(current == null) return null;	// Group
		
		for(var prop in shapeInfo){
			if(shapeInfo[prop] != current[prop]){
				// something has been changed
				return this.shape.setShape(shapeInfo);
			}
		}
		return this.shape;
	},

	nodeMoved: function(/*Widget*/widget){
	},

// ======================= Setters/Getters =================================
	getTop: function(){
		return this.top;
	},

	setTop: function(/*Integer*/top){
		this.top = top;
		this.domNode.style.top = top + "px";
		this.setPosition();
	},


	getLeft: function(){
		return this.left;
	},

	setLeft: function(/*Integer*/left){
		this.left = left;
		this.domNode.style.left = left + "px";
		this.setPosition();
	},


	getWidth: function(){
		return this.width;
	},
	
	setWidth: function(/*Integer*/width){
		this.width = width;
		this.domNode.style.width = width + "px";
		this.update();
	},


	getHeight: function(){
		return this.height;
	},
	
	setHeight: function(/*Integer*/height){
		this.height = height;
		this.domNode.style.height = height + "px";
		this.update();
	},
	
	// subclass may override
	// Returns true if this node accepts connections of which this is the source.
	isTargetConnectionAcceptable: function(sourceNode/*Widget*/) {
		if(this.maxTargetConnections > -1) {
			if(this.getTargetConnections().length < this.maxTargetConnections) return true;
			else return false;
		} else return true;
	},
	
	// subclass rarely override
	// Returns true if user can start dragging a sourceAnchor to create connections
	canCreateSourceConnection: function() {
		if(this.maxSourceConnections > -1) {
			if(this.getSourceConnections().length < this.maxSourceConnections) return true;
			else return false;
		} else return true;
	},
	
	// === Events ===
	onSourceConnectionAdded: function(connection) {
	},
	onTargetConnectionAdded: function(connection) {
	},
	onRemovingSourceConnection: function(connection) {
	},
	onRemovingTargetConnection: function(connection) {
	}
});
