/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.decoration.PolylineDecoration");

// Summary:
//		A decorative Figure intended to be placed on a Polyline.
//		Note that this is not a widget.

dojo.require("gdc.widget.decoration.AbstractDecoration");

dojo.declare(
	"gdc.widget.decoration.PolylineDecoration",
	gdc.widget.decoration.AbstractDecoration,
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"

	// default line styles (see also dojox.gfx.defaultStroke)
	lineColor: "black",
	lineStyle: "solid",
	lineWidth: 1, // Integer
	lineCap: "butt",
	lineJoin: 4, // Integer

	fillColor: "",
	scale: 1, // Integer

	shape: null, // gfx Shape object
	groupBox: null, // gfx Group object that contains 'shape'.
	parentGroupBox: null, // dojox.gfx.Group to add this shape

	constructor: function(owner, props){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')');
		}
		this.processPreset();
	},

	destroy: function(){
		this.shape.removeShape();
	},

	// TODO: those pre-defined samples should be outside of this module
	processPreset: function(){
		if(!this.preset){ return; }
		if(this.preset == "white_arrow"){
			this.points = [{x: 0, y: 0}, {x: -10, y: -5}, {x: -10, y: 5}, {x: 0, y: 0}];
			this.fillColor = "white";
		}else if(this.preset == "black_arrow"){
			this.points = [{x: 0, y: 0}, {x: -10, y: -5}, {x: -10, y: 5}, {x: 0, y: 0}];
			this.fillColor = "black";
		}else if(this.preset == "arrow"){
			this.points = [{x: -10, y: -5}, {x: 0, y: 0}, {x: -10, y: 5}];
			this.fillColor = ""; // no fill
		}else if(this.preset == "white_diamond"){
			this.points = [{x: 0, y: 0}, {x: -8, y: 5}, {x: -16, y: 0}, {x: -8, y: -5}, {x: 0, y: 0}];
			this.fillColor = "white";
		}else if(this.preset == "black_diamond"){
			this.points = [{x: 0, y: 0}, {x: -8, y: 5}, {x: -16, y: 0}, {x: -8, y: -5}, {x: 0, y: 0}];
			this.fillColor = "black";
		}
	},

	setProps: function(props){
		var prev = this.preset;
		this.inherited("setProps", arguments);
		if(this.preset != prev){
			this.processPreset();
		}
	},

	createTargetShape: function(parent){
		this.parentGroupBox = parent;
		this.groupBox = parent.createGroup();
		var shape = this.shape = this.groupBox.createPolyline(this.points);

		// assign the owner of this decoration to the top level dom node
		// so that the owner widget can be found from dom node. (See btp._base.utils.getWidgetByElement)
		shape.rawNode.widget = this.owner; // [custom property]
		return shape;
	},

	update: function(props){
		var shape = this.shape;
		if(!shape){ return; }

		// setShape is not low cost especially on IE, so compare the points first to see if something has really changed.
		if(this._comparePoints(this.points, this.shape.shape.points) != 0){
			this.shape.setShape(this.points);
		}

		if((props && props.forceUpdate) || !this._cur
			|| this._cur.color != this.lineColor
			|| this._cur.style != this.lineStyle
			|| this._cur.width != this.lineWidth
			|| this._cur.cap != this.lineCap
			|| this._cur.join != this.lineJoin){
			shape.setStroke({
				color: this.lineColor,
				style: this.lineStyle,
				width: this.lineWidth,
				cap: this.lineCap,
				join: this.lineJoin
			});
		}

		if((props && props.forceUpdate) || !this._cur || this._cur.fillColor != this.fillColor){
			if(this.fillColor){
				shape.setFill(new dojo.Color(this.fillColor));
			}else{
				shape.setFill(null);
			}
		}

		// save the current status
		this._cur = {
			color: this.lineColor,
			style: this.lineStyle,
			width: this.lineWidth,
			cap: this.lineCap,
			join: this.lineJoin,
			fillColor: this.fillColor
		};
	},

	setLocation: function(/*{x:#,y:#}*/pt){
		this.shape.setTransform({dx:pt.x, dy:pt.y});
		if(!this._cur && this.scale != 1 || this._cur.scale != this.scale){
			var matrix = [dojox.gfx.matrix.scaleAt(this.scale, this.scale, pt.x, pt.y)];
			this.groupBox.setTransform(matrix);
		}
		this._cur.scale = this.scale;
	},

	setRotation: function(/*float*/angle){
		var rotate = dojox.gfx.matrix.rotategAt(angle, 0, 0);
		this.shape.applyTransform(rotate);
	},

	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	}
});
