/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.router.ManhattanConnectionRouter");

dojo.require("gdc.widget.router.AbstractConnectionRouter");

// summary:
//		Update points for a router.
//		'this' points to a gfx line widget.

dojo.declare(
	"gdc.widget.router.ManhattanConnectionRouter",
	gdc.widget.router.AbstractConnectionRouter,
{
	delta: 20,
		
	_getPoints: function(){
		var pt0 = this.owner.points[0];
		var pt = this.owner.points[this.owner.points.length-1];
		var x1 = pt0.x;
		var y1 = pt0.y;
		var startdir = pt0.dir;
		var x2 = pt.x;
		var y2 = pt.y;
		var enddir = pt.dir;		
		var delta = this.delta;

		// top-bottom combination
		var points = [];
		if((startdir=="top" && enddir=="bottom" && y1>=y2) ||
			(startdir=="bottom" && enddir=="top" && y1<y2)){
			points = [{x:x1,y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
			
		}else if((startdir=="top" && enddir=="bottom" && y1<y2)){
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if((startdir=="bottom" && enddir=="top" && y1>=y2)) {
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];

		// left-right combination
		}else if((startdir=="left" && enddir=="right" && x1<x2)){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if((startdir=="right" && enddir=="left" && x1>=x2)){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];

		// left-top|bottom, right-top|bottom combination
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1<y2) ||
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1>=y2) ||
			(startdir=="right" && enddir=="top" && x1<x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1>=y2)
			){
			points = [{x:x2,y:y1}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1<y2) ||
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1>=y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1>=y2) ||
			(startdir=="right" && enddir=="top" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1<y2)
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1<y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1>=y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1<y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];

		// top-left|right, bottom-left|right combination
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1>=y2) ||
			(startdir=="top" && enddir=="right" && x1>=x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1<y2) ||
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1<y2) ||
			(startdir=="top" && enddir=="right" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1<y2)
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2-delta),y:(y1-delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2+delta),y:(y1-delta)},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2-delta),y:(y1+delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2+delta),y:(y1+delta)},{x:(x2+delta),y:y2}];

		// same direction
		}else if(
			(startdir=="left" && enddir=="left" && x1>=x2)
			){
			points = [{x:(x2-delta),y:y1},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="left" && enddir=="left" && x1<x2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1>=x2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1<x2)
			){
			points = [{x:(x2+delta),y:y1},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="top" && y1>=y2)
			){
			points = [{x:x1,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="top" && enddir=="top" && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:x2,y:(y1-delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:x2,y:(y1+delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1<y2)
			){
			points = [{x:x1,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else{ // includes left-right, right-left
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:y2}];
		}		
		return points;
	},
	
	update: function(){
		var len = this.owner.points.length;
		if(len<2){ return; }
		this.owner.points.splice(1, len-2);
		var points = this._getPoints();
		var i = 1;
		dojo.forEach(points, dojo.hitch(this, function(pt){
			this.owner.points.splice(i, 0, pt);
			i++;
		}));
	}
});