/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.samples.PlugNode");

dojo.require("gdc._base.shape");
dojo.require("gdc.widget.AbstractNode");
dojo.require("gdc.widget.support.connectable");

dojo.declare(
"gdc.widget.samples.PlugNode",
gdc.widget.AbstractNode,
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	inputAnchorShape: null,
	outputAnchorShape: null,
	
	leftImage: null,
	mainImage: null,
	plugImage: null,
	labelText: null,
	
	leftImageSrc: "../gdc/widget/templates/PlugNode/outlet-empty-blue.png",
	leftImageAltSrc: "../gdc/widget/templates/PlugNode/outlet-empty-green.png",
	leftImageW: 31,
	leftImageH: 38,
	
	mainImageSrc: "../gdc/widget/templates/PlugNode/body-blue.png",
	mainImageAltSrc: "../gdc/widget/templates/PlugNode/body-green.png",
	mainImageW: 86,
	mainImageH: 38,
	
	plugImageSrc: "../gdc/widget/templates/PlugNode/plug.png",
	plugImageW: 27,
	plugImageH: 38,
	
	backgroundColor: "yellow",
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	width: 144,
	height: 38,
	
	selected: false,
	
	postCreate: function(){
		this.inherited("postCreate", arguments);
		
		dojo.subscribe("btp:SelectionChanged", this, "selectionChanged");
	},
	
	createTargetShape: function(parent){
		var shape = this.shape = parent.createGroup();
		
		var leftImage = this.leftImage = shape.createGdcImage({width:this.leftImageW, height:this.leftImageH, src:dojo.moduleUrl("gdc", this.leftImageAltSrc)+""});
		leftImage.setTransform({dx: 0});
		leftImage.rawNode.widget = this;
		
		var mainImage = this.mainImage = shape.createGdcImage({width:this.mainImageW, height:this.mainImageH, src:dojo.moduleUrl("gdc", this.mainImageAltSrc)+""});
		mainImage.setTransform({dx:31});
		
		var plugImage = this.plugImage = shape.createGdcImage({width:this.plugImageW, height:this.plugImageH, src:dojo.moduleUrl("gdc", this.plugImageSrc)+""});
		plugImage.setTransform({dx:117});
		plugImage.rawNode.widget = this;
		plugImage.rawNode.getDragTracker = function() {
			return "gdl.widget.tool.ConnectionDragCreationTool";
		}
		plugImage.rawNode.getConnectionClass = function(){
			return "gdc.widget.CurvylineConnection";
		};		
		
		var labelText = this.labelText = shape.createGdcText({width: 64, height:17, text: "Sample long text"});
		labelText.setFill(new dojo.Color("#FFFFFF"));
		labelText.setTransform({dx: 46, dy: 26});
		labelText.setFont({size: "10pt", family: "Arial,Verdana,san-serif"});
		
		this.createConnectionAnchors();
		return shape;
	},
	
	getShapeInfo: function(){
		return { type:'group',
			width:this.getWidth(), height:this.getHeight()
		};
	},
	
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},
	
	setBackgroundColor: function(color){
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
//		this.inputAnchorShape.setFill(new dojo.Color("red"));
//		this.outputAnchorShape.setFill(new dojo.Color("blue"));
	},
	
	setBorderColor: function(color){
		if(this._borderColorDone && this.borderColor == color){ return; }
		this._borderColorDone = true;
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
	},
	
	_onMouseOver: function(){
//		this.leftImage.
	},
	
	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchor
		var anchor1 = new anchorImpl(this);
		anchor1.endPointShape = this.mainImage;
		anchor1.centerOrigin = false;
		anchor1.offsetH = 86;
		anchor1.offsetV = 19;
		this.registerSourceConnectionAnchor(this.getShape().rawNode, "input", anchor1);

		// output anchor
		var anchor2 = new anchorImpl(this);
		anchor2.endPointShape = this.leftImage;
		anchor2.centerOrigin = false;
		anchor2.offsetH = 20;
		anchor2.offsetV = 21;
		this.registerTargetConnectionAnchor(this.getShape().rawNode, "output", anchor2);
	},
	
	selectionChanged: function(message){
		if(message.selection == this) {
			this.selected = true;
			this.leftImage.setShape({src: dojo.moduleUrl("gdc", this.leftImageAltSrc)+""});
			this.mainImage.setShape({src: dojo.moduleUrl("gdc", this.mainImageAltSrc)+""});
		} else {
			if(this.selected) {
				this.selected = false;
				this.leftImage.setShape({src: dojo.moduleUrl("gdc", this.leftImageSrc)+""});
				this.mainImage.setShape({src: dojo.moduleUrl("gdc", this.mainImageSrc)+""});
			}
		}
	},
	
	// Example:
	//  accept connections if the amount of input connections <= 2 (allows 3 connections) 
//	isTargetConnectionAcceptable: function(sourceNode, sourceAnchor) {
//		if(sourceNode.declaredClass=="gdc.widget.samples.PlugNode") return true;
//		else return false;
//	},
	
	onTargetConnectionAdded: function(connection) {
		console.debug(this.id +": connected from "+ connection.getSource().id);
	},
	
	onRemovingSourceConnection: function(connection) {
		console.debug(this.id+": disconnected from "+ connection.getTarget().id);
	},
	
	openContextMenu: function(){
		alert("dummy context menu: "+ this.id);	
	}
	
});
dojo.extend(gdc.widget.samples.PlugNode, gdc.widget.support.connectable);
