/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.samples.SampleNode");

// summary:
//		A sample gfx node widget that has an output anchor and an input anchor.

dojo.require("gdc.widget.AbstractNode");
dojo.require("gdc.widget.GfxContainer");
dojo.require("gdc.widget.support.connectable");

dojo.declare(
	"gdc.widget.samples.SampleNode",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	anchor: "gdc.widget.anchor.FixedAnchor",

	inputAnchorShape: null,
	outputAnchorShape: null,

	createTargetShape: function(parent){
		var shape = parent.createRect({width:this.getWidth(), height:this.getHeight()});

		// input anchor shape
		this.inputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.inputAnchorShape.setTransform({ dx:5, dy:18 });
		this.inputAnchorShape.rawNode.widget = this; // [custom property]

		// output anchor shape
		this.outputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.outputAnchorShape.setTransform({ dx:100, dy:18 });
		this.outputAnchorShape.rawNode.widget = this; // [custom property]
		this.outputAnchorShape.rawNode.getDragTracker = function(){
			return "gdl.widget.tool.ConnectionDragCreationTool";
		};
		this.outputAnchorShape.rawNode.getConnectionClass = function(){
			return "gdc.widget.CurvylineConnection";
		};

		this.createConnectionAnchors();
		return shape;
	},

	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchor
		var anchor1 = new anchorImpl(this);
		anchor1.endPointShape = this.inputAnchorShape;
		this.registerTargetConnectionAnchor(this.getShape().rawNode, "input", anchor1);

		// output anchor
		var anchor2 = new anchorImpl(this);
		anchor2.endPointShape = this.outputAnchorShape;
		this.registerSourceConnectionAnchor(this.getShape().rawNode, "output", anchor2);
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'rect',
			width:this.getWidth(), height:this.getHeight()
		};
	},

	// Override gdc.widget.AbstractNode
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},

// ======================= Setters/Getters =================================
	getBackgroundColor: function(){
		return this.backgroundColor;
	},
	
	setBackgroundColor: function(color){
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
		this.inputAnchorShape.setFill(new dojo.Color("blue"));
		this.outputAnchorShape.setFill(new dojo.Color("red"));
	},
	
	getBorderColor: function(){
		return this.borderColor;
	},
	
	setBorderColor: function(color){
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
	},
	
	isTargetConnectionAcceptable: function(sourceNode, sourceAnchor) {
		if(sourceNode.declaredClass=="gdc.widget.samples.PlugNode") return true;
		else return false;
	}
	
	// respond attach/detach example
//	onTargetConnectionAdded: function(connection) {
//		alert(this.id +": connected from "+ connection.getSource().id);
//	},
//	onRemovingSourceConnection: function(connection) {
//		alert(this.id+": disconnected from "+ connection.getTarget().id);
//	}
});
dojo.extend(gdc.widget.samples.SampleNode, gdc.widget.support.connectable);
