/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.support.connectable");

// summary:
//		Make a gfx node widget connectable.
//		This class is intended to be mixed into a gfx node widget such as Rect.
//		'this' points to a gfx node widget.

gdc.widget.support.connectable = {
	sourceConnectionAnchors: null, // hash
	targetConnectionAnchors: null, // hash
	
	maxSourceConnections: -1,
	maxTargetConnections: -1,

	registerSourceConnectionAnchor: function(/*Element*/elem, /*String*/anchorName, /*Anchor*/anchor){
		if(!this.sourceConnectionAnchors){ this.sourceConnectionAnchors = {}; }
		this.sourceConnectionAnchors[anchorName] = anchor;
		anchor.anchorName = anchorName;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(anchorName); // [custom property]
	},

	getSourceConnectionAnchor: function(/*Element*/clickedElement){
		var rootNode = this.getShape().rawNode;
		for(var elem = clickedElement; elem; elem = elem.parentNode){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(this.sourceConnectionAnchors[elem.anchorNames[i]]){
						return this.sourceConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == rootNode){ return null; }
		}
		return null;
	},

	getSourceConnectionAnchorByName: function(/*String*/anchorName){
		return this.sourceConnectionAnchors[anchorName];
	},

	getSourceConnectionAnchors: function(){
		return this.sourceConnectionAnchors;
	},


	registerTargetConnectionAnchor: function(/*Element*/elem, /*String*/anchorName, /*Anchor*/anchor){
		if(!this.targetConnectionAnchors){ this.targetConnectionAnchors = {}; }
		this.targetConnectionAnchors[anchorName] = anchor;
		anchor.anchorName = anchorName;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(anchorName); // [custom property]
	},

	getTargetConnectionAnchor: function(/*Element*/clickedElement){
		var rootNode = this.getShape().rawNode;
		for(var elem = clickedElement; elem; elem = elem.parentNode){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(this.targetConnectionAnchors[elem.anchorNames[i]]){
						return this.targetConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == rootNode){ return null; }
		}
		return null;
	},

	getTargetConnectionAnchorByName: function(/*String*/anchorName){
		return this.targetConnectionAnchors[anchorName];
	},

	getTargetConnectionAnchors: function(){
		return this.targetConnectionAnchors;
	},
	
	getSourceConnections: function() {
		var connections = [];
		for(var anchorName in this.sourceConnectionAnchors) {
			var anchor = this.sourceConnectionAnchors[anchorName];
			if(anchor.listeners) {
				for(var j=0; j<anchor.listeners.length; j++) {
					var con = anchor.listeners[j];
					if(!con.isGhost) { connections.push(con); }
				}
			}
		}
		return connections;
	},
	
	getTargetConnections: function() {
		var connections = [];
		for(var anchorName in this.targetConnectionAnchors) {
			var anchor = this.targetConnectionAnchors[anchorName];
			if(anchor.listeners) {
				for(var j=0; j<anchor.listeners.length; j++) {
					var con = anchor.listeners[j];
					if(!con.isGhost) connections.push(con);
				}
			}
		}
		return connections;
	}
};
