/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.support.connection");

// summary:
//		Make a gfx line widget connection.
//		This class is intended to be mixed into a gfx line widget such as Polyline.
//		'this' points to a gfx line widget.

gdc.widget.support.connection = {
	sourceAnchor: "", // "nodeId" or "nodeId:anchorName"
	targetAnchor: "", // "nodeId" or "nodeId:anchorName"
	router: "",

	sourceAnchorObj: null, // anchor object
	targetAnchorObj: null, // anchor object
	routerObj: null, // router object
	
	mouseOverColor: "",

	processParams: function(){
		if(this.sourceAnchor && !this.sourceAnchorObj){
			var anchor = this.sourceAnchor;
			this.sourceAnchor = null;
			this.setSourceAnchor(anchor);
		}
		if(this.targetAnchor && !this.targetAnchorObj){
			var anchor = this.targetAnchor;
			this.targetAnchor = null;
			this.setTargetAnchor(anchor);
		}
		this.registerCallback("removeShape", this.disconnect);
		this.registerCallback("restoreShape", this.reconnect);
		this.setRouter(this.router);
		this.anchorMoved();
	},

	cleanup: function(){
		this.disconnect();
	},

	disconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.removeAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.removeAnchorListener(this);
		}
		var editors = btp.getEditors();
		dojo.forEach(editors, function(editor){
			if(editor.popup){
				var popup = editor.popup;
				if(popup && popup.hide){
					popup.hide();
				}
			}
		});
		
	},

	reconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.addAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.addAnchorListener(this);
		}
	},


	_setAnchor: function(/*String|Anchor*/anchor, which){
		if(anchor && (this[which+"Anchor"] == anchor || this[which+"AnchorObj"] == anchor)){ return; }
		if(!anchor && (!this[which+"Anchor"] && !this[which+"AnchorObj"])){ return; }
		if(this[which+"AnchorObj"]){
			this[which+"AnchorObj"].removeAnchorListener(this); // remove the old anchor
		}
		if(!anchor){
			this[which+"Anchor"] = "";
			this[which+"AnchorObj"] = null;
			return;
		}
		var anchorObj;
		if(typeof(anchor) == "string"){
			var arr = anchor.split(":"); // "nodeId:anchorName"
			var id =  arr[0];
			var anchorName = arr[1] || "base"; // default anchor name is "base"
			var nodeWidget = dijit.byId(id)
			var cap = which.charAt(0).toUpperCase() + which.substring(1);
			anchorObj = nodeWidget && nodeWidget["get"+cap+"ConnectionAnchorByName"](anchorName);
		}else{
			this[which+"Anchor"] = anchor.owner && anchor.owner.id;
			anchorObj = anchor;
		}
		anchorObj.addAnchorListener(this); // add the new anchor
		this[which+"AnchorObj"] = anchorObj;
		this.anchorMoved();
	},

	setSourceAnchor: function(/*String|Anchor*/anchor){
		this._setAnchor(anchor, "source");
	},

	setTargetAnchor: function(/*String|Anchor*/anchor){
		this._setAnchor(anchor, "target");
	},
	
	getSource: function() {
		if(this.sourceAnchorObj) return this.sourceAnchorObj.owner;
		else return null;
	},
	
	getTarget: function() {
		if(this.targetAnchorObj) return this.targetAnchorObj.owner;
		else return null;
	},	

	setRouter: function(/*String|Router*/router){
		if(!router){
			if(this.router && this.router.destroy){
				this.router.destroy();
			}
			this.router = this.routerObj = null;
			return;
		}
		if(typeof(router) == "string"){
			try{
				dojo["require"](router);
				var routerImpl = dojo.getObject(router);
				this.routerObj = new routerImpl(this);
			}catch(e){
				console.debug("Specified router was not found: '"+router+"'");
			}
		}else if(typeof(router) == "object"){
			this.routerObj = router;
		}
		this.anchorMoved();
	},
	
	setMouseOverHandler: function(){
		dojo.connect(this.shape.rawNode, "onmouseover", this, "_onMouseOver");
	},
	
	_onMouseOver: function(e){
		var editArea = this.__editArea;
		var popup = editArea.popup;
		var oldConnection = popup.connection;
		if(oldConnection && oldConnection.origStroke){
				oldConnection.shape.setStroke(oldConnection.origStroke);
		}

		popup.showPopup(e, this);
	},

	anchorMoved: function(/*Widget*/widget){
		if(this.sourceAnchorObj){
			var ref;
			if(this.targetAnchorObj){
				ref = this.targetAnchorObj.getReferencePoint();
			}else{
				ref = this.getEnd();
			}
			var pt = this.sourceAnchorObj.getLocation(ref);
			var widget = this.sourceAnchorObj.owner.getParent();
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
/**** Consider this later *****/
			var dir = this.sourceAnchorObj.getDirection(pt);
			if(dir){
				pt.dir = dir;
			}
/*******************************/
			this.setStart(pt);
		}

		if(this.targetAnchorObj){
			var ref;
			if(this.sourceAnchorObj){
				ref = this.sourceAnchorObj.getReferencePoint();
			}else{
				ref = this.getStart();
			}
			var pt = this.targetAnchorObj.getLocation(ref);
			var widget = this.targetAnchorObj.owner.getParent();
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
/**** Consider this later *****/
			var dir = this.targetAnchorObj.getDirection(pt);
			if(dir){
				pt.dir = dir;
			}
/*******************************/
			this.setEnd(pt);
		}
		/* hide disconnect popup */
		var editors = btp.getEditors();
		dojo.forEach(editors, function(editor){
			if(editor.popup){
				var popup = editor.popup;
				if(popup && popup.hide){
					popup.hide();
				}
			}
		});		
		this.update();
	},

	_getOffsetPos: function(elem, parent){
		var top = 0, left = 0;
		for(; elem != null; elem = elem.offsetParent){
			if(elem == parent){ break; }
			top += elem.offsetTop;
			left += elem.offsetLeft;
		}
		return {top:top, left:left};
	}
};
