/*
	Copyright (c) 2007-2008, IBM Corp.
*/
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * 
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

dojo.provide("gdc.widget.util.common");
dojo.require("dojox.gfx");
//dojo.require("ibm_btp.widget.util.style");

gdc.widget.util.common = {
	// see dojo.overElement
	isOverElement: function(posX, posY, targetNode){
		var dh=dojo.html;
		var targetPos=dh.getAbsolutePosition(targetNode,true,dh.boxSizing.BORDER_BOX);
		var left=targetPos.x;
		var top=targetPos.y;
		var w=0;
		var h=0;
		var group=targetNode.getElementsByTagName('group')[0];
		if(group){
			w=parseInt(group.style.width);
			h=parseInt(group.style.height);
			if(isNaN(w)){ w=0; }
			if(isNaN(h)){ h=0; }
		} else {
			var targetBox=dh.getBorderBox(targetNode);
			w=targetBox.width;
			h=targetBox.height;
		}
		var delta = 5;
		return (left-delta<=posX && posX<=left+w+delta && top-delta<=posY && posY<=top+h+delta);	//	boolean
	},
	
	// instead of dojo.hasClass, make it work so that it works on both HTML node & gfx rawNode
	hasClass: function(node, className){
		var cls = (node.className) ? node.className : node.getAttribute('class');
		if(cls && typeof(cls) == "string"){
			var arr = cls.split(' ');
			for(var i=arr.length-1; i>=0; i--){
				if(arr[i] == className){
					return true;
				}
			}
		}
		return false;
	},

	getNodeCenterPos: function(node){
		var pos = dojo._abs(node, true);
		var box = dojo.contentBox(node);
		return { x:(pos.x+box.w/2), y:(pos.y+box.h/2) };
	},
	
	getBorderPos: function(posX, posY, node){
 		if(node){
			var widget = dijit.byNode(node);
			var pos = dojo._abs(node, true);
			var l = pos.x;
			var t = pos.y;
	 		var box = dojo.contentBox(node);
			var w = box.w;				
			var h = box.h;				
			var delta1=-h/w*(posX-l)+(posY-t);
			var delta2=h/w*(posX-l)+(posY-t-h);
			if(delta1>=0 && delta2>=0){
				return { x:posX, y:(t+h), dir:"bottom" };
			}else if(delta1>=0 && delta2<0){
				return { x:l, y:posY, dir:"left" };
			}else if(delta1<0 && delta2>=0){
				return { x:(l+w), y:posY, dir:"right" };
			}else if(delta1<0 && delta2<0){
				return { x:posX, y:t, dir:"top" };
			}
		}
 		return { x:posX, y:posY, dir:"left" };
	},

	getBorderMiddlePos: function(posX, posY, node){
 		if(node){
			var widget = dijit.byNode(node);
	 		var dh=dojo.html;
	 		var getPos = dh.getAbsolutePosition;
	 		var BB = dh.boxSizing.BORDER_BOX;
	 		var getBox = dh.getBorderBox;
 			var pos=getPos(node,true,BB);
 			var l=pos.x, t=pos.y;
			var w = null;
			var h = null;
			if(widget){
				w = ibm_btp.widget.getStyleValue(widget, "width");
				h = ibm_btp.widget.getStyleValue(widget, "height");
			}else{
		 		var box = dh.getBorderBox(node);
				w = box.width;				
				h = box.height;				
			}
			var delta1=-h/w*(posX-l)+(posY-t);
			var delta2=h/w*(posX-l)+(posY-t-h);
			if(delta1>=0 && delta2>=0){
				return { x:(l+w/2), y:(t+h), dir:"bottom" };
			}else if(delta1>=0 && delta2<0){
				return { x:l, y:(t+h/2), dir:"left" };
			}else if(delta1<0 && delta2>=0){
				return { x:(l+w), y:(t+h/2), dir:"right" };
			}else if(delta1<0 && delta2<0){
				return { x:(l+w/2), y:t, dir:"top" };
			}
		}
 		return { x:posX, y:posY, dir:"left" };
	},	
	
	getSurfaceRelativeBoundingBox: function(shape){
		var bb = shape.getBoundingBox();
		var left = bb.x;
		var top = bb.y;
		var matrix = null;
		while(!(shape instanceof dojox.gfx.Surface)){
			if(shape instanceof dojox.gfx.Group){
				matrix = shape.getTransform();
				if(matrix){
					left += matrix.dx;
					top += matrix.dy;
				}
			}
			shape = shape.getParent();
		}
		return { x:left, y:top, width:bb.width, height:bb.height };
	},

	// Return nearest parent widget that has both widget1 and widget2 as ancestors
	// There must be a better way...
	getOffsetParent: function(widget1, widget2){
		// first pass through the widget tree
		var temp1 = widget1;
		var temp2 = widget2;
		while(temp1){
			temp2 = widget2;
			while(temp2){
				if(temp1 == temp2){
					return temp1;
				}
				temp2 = temp2.parent;  //TODO: parent -> getParent()?
			}
			temp1 = temp1.parent;  //TODO: parent -> getParent()?
		}
		
		// second pass through the widget tree
		temp1 = widget1;
		temp2 = widget2;
		while(temp2){
			temp1 = widget1;
			while(temp1){
				if(temp1 == temp2){
					return temp1;
				}
				temp1 = temp1.parent;  //TODO: parent -> getParent()?
			}
			temp2 = temp2.parent;  //TODO: parent -> getParent()?
		}
		
		return null;
	},
	
	// Returns connect position within the target widget. Priority is the followings:
	//	1 : getConnectPosition is implementation in a widget
	//	2 : nearest border (default)
	getConnectPosition: function(e, widget){
		if(widget && !widget.isEditor){
			if(widget.getConnectPosition){ // if getConnectPosition is implemented in the widget
				return widget.getConnectPosition(e);
			}else{ // default
				return gdc.widget.util.common.getBorderPos(e.pageX, e.pageY, widget.domNode);
			}
		}else{
			return { x:e.pageX, y:e.pageY, dir:'left' };
		}
	},
	
	// returns parentNode of the surface
	// event handlers like onmousemove, onmouseop is attached to this node
	// since they cannot be attached to surface.rawNode
	getSurfaceParentNode: function(surface){
		return surface.rawNode.parentNode.parentNode;
	}
}
