/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdl.widget.ext.connectionEndpoints");

// summary:
//		Extension for creating draggable end points for a gfx line widget.
//		Note that this is not a widget.
//		this.owner points to the EditArea.
//		See also btp.widget.ext.selectionHandles

dojo.require("btp.dnd.DragMoveSource");
dojo.require("btp.dnd.DragManager");

dojo.declare(
	"gdl.widget.ext.connectionEndpoints",
	null,
{
	owner: null,
	endpointKnobType: "blackSquare",

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	// initializer
	constructor: function(owner){
		this.owner = owner;
		this._connectors = [];
		this._connectors.push(dojo.connect(owner, "setFocusFrame", this, "addConnectionEndpoints"));
		this._connectors.push(dojo.connect(owner, "removeFocusFrame", this, "removeConnectionEndpoints"));
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	/*    SOURCE           TARGET
	 *       0 ------------> 1
	 */
	addConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!btp.isGfxLineWidget(widget)) { return; }
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingConnectionEndpoints){
			; // already showing
		}else if(!widget.__connectionEndpoints){
			this.createConnectionEndpoints(widget);
		}else{
			for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
				this.owner.domNode.appendChild(widget.__connectionEndpoints[i]);
			}
		}
		widget.__showingConnectionEndpoints = true;

		// SOURCE
		knob = widget.__connectionEndpoints[0];
		if(widget.sourceDecorationObj && widget.sourceDecorationObj.getDimensions){
			var dim = widget.sourceDecorationObj.getDimensions();
			knob.style.width = dim.width + "px";
			knob.style.height = dim.height + "px";
			knob.style.top = dim.top + "px";
			knob.style.left = dim.left + "px";
		}else{
			var pt0 = widget.points[0];
			knob.style.top = pt0.y + "px";
			knob.style.left = pt0.x + "px";
		}

		// TARGET
		knob = widget.__connectionEndpoints[1];
		var target = widget.getTarget();
		if(target){
			knob.dropTarget = target.domNode.dropTarget;
			if(target.targetConnectionAnchors){
				knob.anchorNames = [];
				for(var n in target.targetConnectionAnchors){
					knob.anchorNames.push(n);
				}
			}
		}else{
			if(knob.dropTarget){
				knob.dropTarget = undefined;
				knob.anchorNames = undefined;
			}
		}
		if(widget.targetDecorationObj && widget.targetDecorationObj.getDimensions){
			var dim = widget.targetDecorationObj.getDimensions();
			knob.style.width = dim.width + "px";
			knob.style.height = dim.height + "px";
			knob.style.top = dim.top + "px";
			knob.style.left = dim.left + "px";
		}else{
			var pt1 = widget.points[widget.points.length - 1];
			knob.style.top = pt1.y + "px";
			knob.style.left = pt1.x + "px";
		}
	},

	removeConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget.__showingConnectionEndpoints){ return; }
		for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
			var e = widget.__connectionEndpoints[i];
			this.owner.domNode.removeChild(e);
			if(e.dropTarget){
				e.dropTarget = undefined;
				e.anchorNames = undefined;
			}
		}
		widget.__showingConnectionEndpoints = false;
	},

	createConnectionEndpoints: function(widget){
		var knobElem = this.owner.createKnob(this.endpointKnobType);

		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|");
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "SOURCE"){
					dirs |= this._SOURCE;
				}else if(arr[i] == "TARGET"){
					dirs |= this._TARGET;
				}else{
					alert("unrecognized direction: "+arr[i]);
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		widget.__connectionEndpoints = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 2; i++){
			knob = knobElem.cloneNode(true);
			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){
				_this.owner.setFocus(widget);
				return _this.owner.reconnectTrackerClass;
			};
//			if(!knob.style.cursor){
//				knob.style.cursor = "crosshair";
//			}
			widget.__connectionEndpoints.push(knob);
			this.owner.domNode.appendChild(knob);

			var ds = new btp.dnd.DragMoveSource(knob, "connection"); // @TODO should not hard-code "connection"
			ds.setDragTarget(this.owner.getMoveTarget());
			ds.onDragMove = function btp_dnd_DragMoveSource_onDragMove(e){
				var dx = e.pageX - ds.pageX;
				var dy = e.pageY - ds.pageY;
				ds.dragClone.style.top = ds.dragStartCssPosition.y + dy + ds.dragOffset.y + "px";
				ds.dragClone.style.left = ds.dragStartCssPosition.x + dx + ds.dragOffset.x + "px";
			};
//			ds.onDragMove = function btp_dnd_DragMoveSource_onDragMove(e){
//				var dx = e.pageX - ds.pageX;
//				var dy = e.pageY - ds.pageY;
//				ds.dragClone.style.top = ds.dragStartCssPosition.y + dy + ds.layerY - ds.domNode.offsetHeight + "px";
//				ds.dragClone.style.left = ds.dragStartCssPosition.x + dx + ds.layerX - ds.domNode.offsetWidth + "px";
//			};

			if(i == 0 && !(dirs & this._SOURCE)){
				knob.style.display = "none";
			}else if(i == 1 && !(dirs & this._TARGET)){
				knob.style.display = "none";
			}
		}
	}
});
