/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdl.widget.outline.CloneContentOutlinePage");
dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");

// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("gdl.widget.outline.CloneContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	canvas: null,
	rootGroup: null,
	_connectors: null,
	_topics: null,
	
	scale: 0.2,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
	},

	createOutlinePage: function(){
		if(!this.canvas){
			if(window.gdl){
				dojo["require"]("gdc.widget.Surface");
				var node = this.outlineView.domNode;
				this.canvas = new gdc.widget.Surface({width:1,height:1});
				node.style.backgroundColor = "InactiveBorder";
				node.style.overflow = "hidden";
				this.outlineView.addChild(this.canvas);
				this._resizeCanvas();
				
				var layoutParent = btp.getLayoutParent(this.outlineView.domNode);
				if(layoutParent) {
					dojo.connect(layoutParent, "layout", this, "_resizeCanvas");
				}
				
				var targetWidget = this.editArea;
				if(targetWidget.adjustCanvasSize){
					dojo.connect(targetWidget, "adjustCanvasWidth", this, "_resizeCanvas");
					dojo.connect(targetWidget, "adjustCanvasHeight", this, "_resizeCanvas");
				}				
				
			}
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.addChild(this.canvas);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.removeChild(this.canvas);
		}
	},
	
	_resizeCanvas: function() {
		var target = this.editArea.domNode;
		var targetWidth = target.scrollWidth;
		var targetHeight = target.scrollHeight;
		var outlineBox = dojo.contentBox(this.outlineView.domNode);
		var outlineWidth = outlineBox.w;
		var outlineHeight = outlineBox.h;
		
		var propX = outlineWidth / targetWidth;
		var propY = outlineHeight / targetHeight; 
		this.scale =  Math.min(propX, propY);
		
		var width = Math.floor(this.scale * targetWidth) + "px";
		var height = Math.floor(this.scale * targetHeight) + "px";
		this.canvas.domNode.style.width = width;
		this.canvas.domNode.style.height = height;
		this.canvas.surface.setDimensions(width, height);
		if(this.rootGroup) {
			this.rootGroup.setTransform(dojox.gfx.matrix.scale(this.scale, this.scale));
		}
	},

	refreshOutline: function(){
		this.canvas.surface.clear();
		var g = this.rootGroup = this.canvas.surface.createGroup();
		g.setTransform(dojox.gfx.matrix.scale(this.scale, this.scale));		

		var root = this.contentProvider.getElements(this.editArea.rootGroup);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshCloneView(null, root[i], true);
		}
	},

	_refreshCloneView: function(parentObj, obj, forceRefresh){
		var node;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.rootGroup; 
		
		var shape = obj.targetNode.shape;
		
		var fill = obj.targetNode.getFill();
		var stroke = obj.targetNode.getStroke();
		var transform = obj.targetNode.getTransform();

		if(!obj.outlineNode || forceRefresh) {
			if(shape) node = obj.outlineNode = parentNode.createShape(shape);
			else node = obj.outlineNode = parentNode.createGroup();	
		} else {
			if(shape) node = obj.outlineNode.setShape(shape);
			else node = obj.outlineNode;
		}
		node.outlineObj = obj;
		
		if(fill) node.setFill(fill);
		if(stroke) node.setStroke(stroke);
		if(transform) node.setTransform(transform);
		
		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshCloneView(obj, child, forceRefresh);
		}
	},

	updateOutline: function(message){
		if(message.propName == "@AddChild"
		|| message.propName == "@RemoveChild"
		|| message.propName == "@ForceRefresh"){
//			var root = this.contentProvider.getElements(this.editArea.rootGroup);
//			this._updateCloneView(root, this.rootGroup);
			this.refreshOutline();
		}else if(message.propName == "@SetStyles"){
//			var widget = message.hint.widget;
//			var gfxObject = widget.groupBox ? widget.groupBox : widget.shape;
//			var parentObj = this.contentProvider.getParent(gfxObject.outlineObj);
//			this._refreshCloneView(parentObj, gfxObject.outlineObj);
			var root = this.contentProvider.getElements(this.editArea.rootGroup);
			for(var i = 0, len = root.length; i < len; i++){
				this._refreshCloneView(null, root[i], false);
			}			
		}else{
			var widget = message.hint.widget;
			if(!widget || !widget.outlineObj) {
				this.refreshOutline();
				return;
			}
			var outlineNode = widget.outlineObj.outlineNode;
			var name = message.propName;
			var f = "set" + name.charAt(0).toUpperCase() + name.substring(1);
			if(outlineNode[f]){
				outlineNode[f](message.newValue);
			}else{
				outlineNode[name] = message.newValue;
			}
		}
	},

	_updateCloneView: function(objs, /*Widget*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.children ? parentNode.children.length : 0;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateCloneView(chlidObjs, parentNode.children[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.children[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshCloneView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.children[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.removeShape();
					delete node.outlineObj.targetNode.outlineNode;
					delete node.outlineObj;
					break;
				}
			}
		}
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
	},

	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
	},

	deleteOutlineTarget: function(){
	}
});
