/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdl.widget.tool.AbstractConnectionCreationTool");

dojo.require("btp.widget.tool.AbstractTool");
dojo.require("gdl.widget.command.ConnectionCommand");

dojo.declare(
	"gdl.widget.tool.AbstractConnectionCreationTool",
	btp.widget.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	sourceWidget: null,
	targetWidget: null,
	ghostLine: null,
	ghostLineCache: null,
	connectionWidgetClassName: null, // class name with suffix

	_operationStarted: false,

	// When the button is first pressed, the source node is determined and locked in.
	// After that time, the tool will be looking for the target node to complete the connection.
	handleMouseDown: function(e, editPartViewer){
		var widget = this.sourceWidget = btp.getWidgetByElement(e.target);
		if(!widget || !widget.getSourceConnectionAnchor){ return false; }
		if(!widget.canCreateSourceConnection()){
			this.sourceWidget = null;
			return false;
		}
		this.sourceConnectionAnchor = widget.getSourceConnectionAnchor(e.target);

		// get connection widget class name (ex. "gdc.widget.PolylineConnection:1")
		var elem = e.target;
		while(elem){
			this.connectionWidgetClassName = elem.getConnectionClass && elem.getConnectionClass(); // from the clicked DOM node
			if(this.connectionWidgetClassName){ break; }
			var widget = elem.widget || dijit.byNode(elem); // gfx raw node may have the widget property
			if(widget){
				this.connectionWidgetClassName = widget.getConnectionClass && widget.getConnectionClass(); // from the node widget
				break;
			}
			elem = elem.parentNode;
			if(!elem || elem.tagName == "BODY"){ break; }
		}
		if(!this.connectionWidgetClassName){
			this.connectionWidgetClassName = editPartViewer.selectedPaletteItem.name; // from the selected palette item
		}
		return true;
	},

	createGhostLine: function(e, editPartViewer){
		var className = this.connectionWidgetClassName.split(":")[0];
		if(!this.ghostLineCache){ this.ghostLineCache = {}; }
		if(this.ghostLineCache[className]){
			this.ghostLine = this.ghostLineCache[className];
		}else{
			this.ghostLine = this.ghostLineCache[className] = editPartViewer.createComponent(className, {lineColor:"#DCDCDC"});
			this.ghostLine.isGhost = true;
			if(this.ghostLine.sourceDecorationObj){
				this.ghostLine.sourceDecorationObj.lineColor = "#DCDCDC";
				this.ghostLine.sourceDecorationObj.fillColor = "#DCDCDC";
			}
			if(this.ghostLine.targetDecorationObj){
				this.ghostLine.targetDecorationObj.lineColor = "#DCDCDC";
				this.ghostLine.targetDecorationObj.fillColor = "#DCDCDC";
			}
		}
		editPartViewer.addChild(this.ghostLine);
		this.ghostLine.setSourceAnchor(this.sourceConnectionAnchor);

		// btp.widget._editArea.componentInfo#getAttributeInfo may be modified
		// so that it works even an argument is component name string (not an widget)
		var info = editPartViewer.getComponentInfo(this.connectionWidgetClassName);
		if(info){
			var attrs = info.attribute;
			for(var i = 0, len = attrs.length; i < len; i++){
				if(attrs[i].name == "router"){
					this.ghostLine.setRouter(attrs[i].value);
					break;
				}
			}
		}
	},

	removeGhostLine: function(e, editPartViewer){
		if(!this.ghostLine){ return; }
		editPartViewer.extensions["gdl.widget.ext.connectionEndpoints"].removeConnectionEndpoints(this.ghostLine);
		this.ghostLine.setEnd(this.ghostLine.getStart());
		this.ghostLine.setSourceAnchor(null); // remove the anchorListener
		this.ghostLine.setTargetAnchor(null); // remove the anchorListener if any
		editPartViewer.removeChild(this.ghostLine);
		this.ghostLine = null;
	},

	// Execute a command to create a connection
	handleCreateConnection: function(e, editPartViewer){
		var srcAnchor = this.sourceConnectionAnchor;
		var tgtAnchor = this.targetConnectionAnchor;
		if(srcAnchor && tgtAnchor){
			var props = {
				sourceAnchor: srcAnchor.owner.id+":"+srcAnchor.anchorName,
				targetAnchor: tgtAnchor.owner.id+":"+tgtAnchor.anchorName
			};
			var connection = editPartViewer.createComponent(this.connectionWidgetClassName, props);
			var offsetParent = gdl.getOffsetParent(this.sourceWidget, this.targetWidget);
			if(!offsetParent){ offsetParent = editPartViewer; }
			var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);
			this.getEditDomain().executeCommand(command);
			connection.setMouseOverHandler();
		}
		return true;
	},

	finishCreateConnection: function(e, editPartViewer){
		if(!this.sourceWidget){ return; } // already finished
		this.removeGhostLine(e, editPartViewer);
		if(editPartViewer.selectedPaletteItem){
			if(editPartViewer.selectedPaletteItem.deselect){
				editPartViewer.selectedPaletteItem.deselect();
			}
		}
		this._editDomain.loadDefaultTool();
		this.sourceWidget = this.targetWidget = null;
		this._operationStarted = false;
	}
});
