/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdl.widget.tool.ReconnectTracker");

dojo.require("btp.widget.tool.AbstractTool");
dojo.require("btp.dnd.DragMoveSource");
dojo.require("btp.dnd.DragManager");
dojo.require("gdl.widget.command.ConnectionCommand");
dojo.require("gdl._base.utils");

// summary:
//		A DragTracker for dragging a connection endpoint to reconnect.

dojo.declare(
	"gdl.widget.tool.ReconnectTracker",
	btp.widget.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	_operationStarted: false,

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	_lineWidget: null,
	_draggingKnob: null,
	_oppositeKnob: null,
	_moveTarget: null, // dummy drag target (DIV element) for the dragging knob
	_knobPosition: null, // 0: source, 1: target
	_idx0: -1, // index of the dragging end point
	_idx1: -1, // index of the opposite end point
	_draggingSideAnchor: null,
	_oppositeSideAnchor: null,
	_draggingSideWidget: null,
	_oppositeSideWidget: null,

	stopReconnecting: function(){
		if(!this._operationStarted){ return; } // already canceled
		this._operationStarted = false;
		this._editPartViewer.domNode.removeChild(this._editPartViewer.getMoveTarget());
		this._editPartViewer.domNode.style.cursor = "";
		this._oppositeSideWidget.__dt.disabled = false;
		this._draggingKnob.style.display = "";

		if(!this.sourceConnectionAnchor && !this.targetConnectionAnchor){
			this.onDropAborted();
		}else{
			this._lineWidget.anchorMoved();
		}
	},

	onDropAborted: function(){
		if(true){ // disconnect from the nodes
			// recover the dragging end point
			if(this._knobPosition == 0){	// dragging the source-side knob
				this._lineWidget.setSourceAnchor(this._draggingSideAnchor);
			}else{							// dragging the target-side knob
				this._lineWidget.setTargetAnchor(this._draggingSideAnchor);
			}
			// remove the connection
			this._editPartViewer.removeFocus();
			var command = new gdl.widget.command.ConnectionCommand.RemoveConnectionCommand(this._lineWidget);
			this.getEditDomain().executeCommand(command);
		}else{ // restore the connection
			// recover the dragging end point
			if(this._knobPosition == 0){	// dragging the source-side knob
				this._lineWidget.setSourceAnchor(this._draggingSideAnchor);
			}else{							// dragging the target-side knob
				this._lineWidget.setTargetAnchor(this._draggingSideAnchor);
			}
			this._editPartViewer.setFocusFrame(this._lineWidget); // to reset the endpoints position
		}
	},

	// ======== Event Handlers ====================================
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
	},

	handleDragStart: function(e, editPartViewer){
		this._operationStarted = true;
		this._lineWidget = this._editPartViewer.focusWidgets[0]; // gfx line widget
		if(!btp.isGfxLineWidget(this._lineWidget)){ alert("ReconnectTracker#handleDragStart: gfx line is expected, but found "+this._lineWidget.declaredClass); } // TODO remove this code later
		this._draggingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		this._oppositeKnob = (this._draggingKnob == this._lineWidget.__connectionEndpoints[0])
			? this._lineWidget.__connectionEndpoints[1] : this._lineWidget.__connectionEndpoints[0];
		this._moveTarget = this._editPartViewer.getMoveTarget();
		this._knobPosition = this._draggingKnob.knobPosition; // [custom property]
		this._idx0 = (this._knobPosition == 0) ? 0 : this._lineWidget.points.length - 1;
		this._idx1 = (this._knobPosition == 0) ? this._lineWidget.points.length - 1 : 0;
		this._draggingSideAnchor = (this._knobPosition == 0) ?
			 this._lineWidget.sourceAnchorObj : this._lineWidget.targetAnchorObj;
		this._oppositeSideAnchor = (this._knobPosition == 0) ?
			 this._lineWidget.targetAnchorObj : this._lineWidget.sourceAnchorObj;
		this._draggingSideWidget = this._draggingSideAnchor.owner;
		this._oppositeSideWidget = this._oppositeSideAnchor.owner;

		this.sourceConnectionAnchor = (this._knobPosition == 0) ? this._lineWidget.sourceAnchorObj : null;
		this.targetConnectionAnchor = (this._knobPosition == 0) ? null : this._lineWidget.targetAnchorObj;
		this._moveTarget.style.top = this._draggingKnob.offsetTop + "px";
		this._moveTarget.style.left = this._draggingKnob.offsetLeft + "px";
		this._editPartViewer.domNode.appendChild(this._moveTarget);
//		this._editPartViewer.domNode.style.cursor = "crosshair";
		this._oppositeSideWidget.__dt.disabled = true;
		this._draggingKnob.style.display = "none"; // hide the dragging knob because it may intercept mouse events for a target
	},

	 _findAnchor: function(targetWidget, clickedElement){
		if(!targetWidget){ return; }
		if(this._knobPosition == 0 && targetWidget.getSourceConnectionAnchor){
			return targetWidget.getSourceConnectionAnchor(clickedElement);
		}else if(this._knobPosition == 1 && targetWidget.getTargetConnectionAnchor){
			return targetWidget.getTargetConnectionAnchor(clickedElement);
		}
		return null;
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e, editPartViewer){
		if(!this._operationStarted){ return; } // drag is not started yet or perhaps was canceled with ESC
		if(e.target == this._draggingKnob && this._lineWidget.sourceAnchorObj && this._lineWidget.targetAnchorObj){
			// Dragging on the endpoint knob while both ends are anchored.
			this._skipping = true;
			return;
		}
		if(this._skipping){
			// Moved out of the endpoint knob, but skip the first event, because
			// mouseover event has not come yet and currentDropTarget is not available.
			this._skipping = false;
			return;
		}
		var dt = btp.dnd.dragManager.currentDropTarget;
		var targetWidget = this.targetWidget = dt && dt.data; // drop target widget
		var anchor = this._findAnchor(targetWidget, e.target);
		if(anchor && (this.sourceConnectionAnchor == anchor
					|| this.targetConnectionAnchor == anchor)){ return; } // already anchored
		if(this.sourceConnectionAnchor){ // release the old source anchor
			this.sourceConnectionAnchor = null;
			this._lineWidget.setSourceAnchor(null);
		}
		if(this.targetConnectionAnchor){ // release the old target anchor
			this.targetConnectionAnchor = null;
			this._lineWidget.setTargetAnchor(null);
		}
		if(anchor){
			if(this._knobPosition == 0){	// dragging the source-side knob
				this.sourceConnectionAnchor = anchor;
				this._lineWidget.setSourceAnchor(anchor);
			}else{							// dragging the target-side knob
				this.targetConnectionAnchor = anchor;
				this._lineWidget.setTargetAnchor(anchor);
			}
			var pt0 = this._lineWidget.points[this._idx0];
			this._draggingKnob.style.top = pt0.y + "px";
			this._draggingKnob.style.left = pt0.x + "px";
			var pt1 = this._lineWidget.points[this._idx1];
			this._oppositeKnob.style.top = pt1.y + "px";
			this._oppositeKnob.style.left = pt1.x + "px";
		}else{
			var moveTarget = this._moveTarget;
			var top = this._moveTarget.offsetTop;
			var left = this._moveTarget.offsetLeft;
			this._draggingKnob.style.top = top + "px";
			this._draggingKnob.style.left = left + "px";
			this._lineWidget.setPoint({x: left, y: top}, this._idx0);
			this._lineWidget.update();
		}
	},
	
	_recreateConnection: function(lineWidget, editPartViewer) {
		var sourceWidget = lineWidget.getSource();
		var sourceAnchor = lineWidget.sourceAnchorObj;
		var targetWidget = lineWidget.getTarget();
		var targetAnchor = lineWidget.targetAnchorObj;
		var connectionWidgetClassName = lineWidget.declaredClass;
		
		if(sourceAnchor && targetAnchor) {
			this.onDropAborted();	// disconnect first
			var props = {
				sourceAnchor: sourceAnchor.owner.id+":"+sourceAnchor.anchorName,
				targetAnchor: targetAnchor.owner.id+":"+targetAnchor.anchorName
			}
			var connection = editPartViewer.createComponent(connectionWidgetClassName, props);
			var offsetParent = gdl.getOffsetParent(this.sourceWidget, this.targetWidget);
			if(!offsetParent){ offsetParent = editPartViewer; }
			var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);
			this.getEditDomain().executeCommand(command);
			connection.setMouseOverHandler();
		}		
	},

	handleMouseUp: function(e, editPartViewer){
		if(!this._operationStarted){ return; } // drag is not started yet or perhaps was canceled with ESC
		if(this.sourceConnectionAnchor && this.sourceConnectionAnchor != this._draggingSideAnchor){
			this._recreateConnection(this._lineWidget, editPartViewer);
		}else if(this.targetConnectionAnchor && this.targetConnectionAnchor != this._draggingSideAnchor){
			this._recreateConnection(this._lineWidget, editPartViewer);
		}else{
		}
		this._lineWidget.getShape().moveToFront();
		this.stopReconnecting();
	},

	handleKeyDown: function(e, editPartViewer){
		if(e.keyCode == 27){ // ESC
			this.stopReconnecting();
		}
	}
});
