/*
	Copyright (c) 2007-2008, IBM Corp.
*/
// A simple cross-browser XML util library
// Tested on Firefox 2.0 and IE 6
// v0.6

flow = {};
flow.xml = {};

// ie missing Node defn...
if (window && (!window.Node || !window.Node.ELEMENT_NODE) )
{ 
   window.Node = { ELEMENT_NODE: 1, 
                ATTRIBUTE_NODE: 2,
			    TEXT_NODE: 3,
			    CDATA_SECTION_NODE: 4,
			    ENTITY_REFERENCE_NODE: 5,
			    ENTITY_NODE: 6,
			    PROCESSING_INSTRUCTION_NODE: 7,
			    COMMENT_NODE: 8,
			    DOCUMENT_NODE: 9,
			    DOCUMENT_TYPE_NODE: 10,
			    DOCUMENT_FRAGMENT_NODE: 11,
			    NOTATION_NODE: 12
			   };
}

/*
	MSXML versions:
	
	v2.x: Microsoft.XMLDOM, MSXML.DOMDocument
	v3.0: MSXML2.DOMDocument or Msxml2.DOMDocument.3.0
	v6.0: Msxml2.DOMDocument.6.0
 */
// ie specific
flow.xml._loadDOMImpl = function( ) {
	var progIDs = [ "Msxml2.DOMDocument.6.0", "Msxml2.DOMDocument.3.0", "MSXML2.DOMDocument", "MSXML.DOMDocument", "Microsoft.XMLDOM" ];
	var xmlDoc = null;
		
	for (var i=0; i<progIDs.length; i++) {
		try {
			xmlDoc = new ActiveXObject( progIDs[i] );
		}
		catch (e) {}
			
		if (xmlDoc) {
			//alert ( progIDs[i] + " used" );
			break;
		}
	}
	
	return xmlDoc;
};

flow.xml._loadXHRImpl = function( ) {
	var progIDs = [ "MSXML3.XMLHTTP", "MSXML2.XMLHTTP.3.0", "MSXML2.XMLHTTP", "Microsoft.XMLHTTP" ];
	var xhr = null;
		
	for (var i=0; i<progIDs.length; i++) {
		try {
			xhr = new ActiveXObject( progIDs[i] );
		}
		catch (e) {}
			
		if (xhr) {
			//alert ( progIDs[i] + " used" );
			break;
		}
	}
	
	return xhr;
};

flow.xml.createDocument = function( ) {
	var docImpl = document.implementation;
   	var xmlDoc = null;
	
	// W3C compliant
	if ( docImpl && docImpl.createDocument ) {
		xmlDoc = docImpl.createDocument("", "", null);	   
	}
	else if ( typeof ActiveXObject != "undefined" ) {
		xmlDoc = flow.xml._loadDOMImpl( );
	}
	
	if (!xmlDoc) {
		throw "Unable to create XML Document";
	}
	
	return xmlDoc;
};

// s is the xml string representation
// returns a Document object
flow.xml.loadDocument = function(s, mimeType) {
	var xmlDoc = null;
	
	if ( typeof DOMParser != "undefined" ) {
		xmlDoc = (new DOMParser()).parseFromString( s, (mimeType)?mimeType:"text/xml" );
	}
	else {
		xmlDoc = flow.xml.createDocument();
		xmlDoc.async = false;
		xmlDoc.loadXML( s );
	}
	
	return xmlDoc;
};

flow.xml.loadRemoteDocument = function(url) {
	var xmlDoc = null;
	var req = (window.XMLHttpRequest)?(new XMLHttpRequest()):(flow.xml._loadXHRImpl());
	req.open('GET', url, false); 
	req.send(null);
	
	if (req.status == 200) { 
		xmlDoc = flow.xml.loadDocument(req.responseText);
	}
	
	return xmlDoc;
};

flow.xml.innerXML = function(node) {
	
	if (!node) return "";
	
	if (typeof XMLSerializer != "undefined") {
		var r = (new XMLSerializer()).serializeToString( node );
		return r;
	}
	else if (node.xml) {
	   	return node.xml;
	}
	
	throw "Not supported";
};

flow.xml.XPathQuery = function(xpath, nsProps) {
	this.nsString = "";  // handle namespaces
	this.exprObj = null;
	this.xpath = xpath;
	this.nsProps = nsProps;
	
	if ( document.createExpression ) {
		this.exprObj = document.createExpression(xpath, 
												 function(nsPrefix) {
													 return nsProps[nsPrefix];
												 });
	}
	else {
		if (nsProps) {
			for (var nsPrefix in nsProps) {
				if (this.nsString) this.nsString += " ";
				
				this.nsString += "xmlns:" + nsPrefix + '="' + nsProps[nsPrefix] + '"';
			}
		}
	}
};

// ie specific
// setups the context node for selectNodes and selectSingleNode
flow.xml._setupContext = function( context, nsString ) {
	var xmlDoc = (context.ownerDocument == null)?(context):(context.ownerDocument);
	var ctx = (xmlDoc != context)?(context):(xmlDoc.documentElement);

	xmlDoc.setProperty("SelectionLanguage", "XPath");
	xmlDoc.setProperty("SelectionNamespaces", nsString);
	
	return ctx;
};

flow.xml.XPathQuery.prototype.selectNodes = function(context) {
	if (this.exprObj != null) {
		// we want the result in document order
		var r = this.exprObj.evaluate(context, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
		var nodeArr = new Array(r.snapshotLength);
		for (var i=0; i<r.snapshotLength; i++) {
			nodeArr[i] = r.snapshotItem(i);
		}
		return nodeArr;
	}
	else {
		var ctx = flow.xml._setupContext( context, this.nsString );
		return ctx.selectNodes( this.xpath );	
	}
};

flow.xml.XPathQuery.prototype.selectSingleNode = function(context) {
	if (this.exprObj != null) {
		var r = this.exprObj.evaluate(context, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
		return r.singleNodeValue;
	}
	else {
		var ctx = flow.xml._setupContext( context, this.nsString );
		return ctx.selectSingleNode( this.xpath );
	}
};

flow.xml.findNodes = function(context, xpath, namespaces) {
   return (new flow.xml.XPathQuery(xpath, namespaces)).selectNodes(context);
}; 

flow.xml.findNode = function(context, xpath, namespaces) {
   return (new flow.xml.XPathQuery(xpath, namespaces)).selectSingleNode(context);
}; 

// TODO:  IE
// name and class
// namespace
flow.xml.copyAttributes = function(destNode, srcNode) {
    //dojo.debug( flow.xml.innerXML( srcNode ) );
    
	if (srcNode.attributes && srcNode.attributes.length > 0) {
	   var nAttrs = srcNode.attributes.length;
	   for (var i=0; i<nAttrs; i++) {
	      var attr = srcNode.attributes[i];
	      var value = (attr.value==null)?"":attr.value;
		  if(attr.name =="xmlns" && value==srcNode.namespaceURI) continue;
	      if (attr.namespaceURI != null && attr.namespaceURI != "") {
	         flow.xml._setAttributeNS(destNode, attr.namespaceURI, attr.name, value);
	      }
	      else {
	         destNode.setAttribute(attr.name, value);
	      }
	      
	      //dojo.debug ("attr name=" + attr.name + ", attr value=" + attr.value + ", attr namespaceURI=" + attr.namespaceURI);
	   }
	}
};

// doc:  the doc that will be creating the element
flow.xml._createElementNS = function(doc, namespaceURI, name) {
   var node = null;
   
   if (document.createElementNS) {
      node = document.createElementNS(namespaceURI, name);
   }
   else {
      node = doc.createNode(1, // element node type
							name,
							namespaceURI );
   }
   
   return node;
};

// node:  the node that the attribute will be set
flow.xml._setAttributeNS = function(node, namespaceURI, name, value) {
   
   if (node.setAttributeNS) {
      node.setAttributeNS(namespaceURI, name, value);
   }
   else {
      var doc = (node.ownerDocument == null)?node:node.ownerDocument;
      var attrNode = doc.createNode(2, // attribute node type
							name,
							namespaceURI );
							
	  attrNode.nodeValue = value;
	  node.setAttributeNode(attrNode);
   }
};

flow.xml.importNodes = function(destNode, node, deep) {
    if (destNode == null) throw "Invalid argument";
    
	var _doc = (destNode.ownerDocument==null)?destNode:destNode.ownerDocument;  // for IE
	
	var n = flow.xml._importNode( _doc, node, deep );
	
	if (n != null) {
	   destNode.appendChild( n );
	}
	else {
	   throw "Unable to import";
	}
};

// TODO:  
// handle namespaces for non W3C
//
// Creates a copy of a node from an external doc that can be inserted into the current doc. 
// The returned node's parentNode is null.
//
// doc:   the dest node's document (for IE)
// node:  the external node that is to be imported
// deep:  all the node's children imported?
flow.xml._importNode = function(doc, node, deep) {
    if (doc == null || node == null) throw "Invalid argument";
    
    // W3C
	if (document.importNode) {
    return doc.importNode(node, deep);
	}
	else {
		var newNode = null;
		var _doc = doc;
		
		switch (node.nodeType) {
			case Node.ELEMENT_NODE:
			   // handle namespace - use node.namespaceURI and node.createElementNS, node.createAttributeNS, 
			   //                        doc.createNamespace, etc.
			   
			   if (node.namespaceURI != null && node.namespaceURI != "") {
			      newNode = flow.xml._createElementNS( _doc, node.namespaceURI, node.nodeName );
			   }
			   else {
			      newNode = _doc.createElement(node.nodeName);
			   }
			   
			   // handle attributes
			   flow.xml.copyAttributes(newNode, node);
			   
			   // all children?
			   if (deep && node.childNodes && node.childNodes.length > 0) {
			       var nNodes = node.childNodes.length;
			       for (var i=0; i<nNodes; i++) {
			          var iNode = flow.xml._importNode( _doc, node.childNodes[i], deep );
			          newNode.appendChild( iNode );
			       }
			   }
			   break;
			       	
			case Node.TEXT_NODE:
				newNode = _doc.createTextNode( node.nodeValue );
				break;
		    case Node.CDATA_SECTION_NODE:
				// TODO:  IE does not support createCDATASection, need to handle this differently...
				//newNode = _doc.createCDATASection( node.nodeValue );
				newNode = _doc.createTextNode( node.nodeValue );
				break;
		    case Node.COMMENT_NODE:
				newNode = _doc.createComment( node.nodeValue );
				break;
			case Node.ATTRIBUTE_NODE:
				// TODO: handle namespace
                newNode = _doc.createAttribute( node.name );
                newNode.setValue ( node.value );
                break;
//			case Node.PROCESSING_INSTRUCTION_NODE:
//				newNode = _doc.createProcessingInstruction( node.target, node.nodeValue );
//				break;
			case Node.DOCUMENT_FRAGMENT_NODE:
				newNode = _doc.createDocumentFragment();
				if (deep && node.childNodes && node.childNodes.length > 0) {
			       var nNodes = node.childNodes.length;
			       for (var i=0; i<nNodes; i++) {
			          newNode.appendChild( flow.xml._importNode( _doc, node.childNodes[i], deep ) );
			       }
			    }
			    break;
			// document node cannot be imported.	
			case Node.DOCUMENT_NODE:
			case Node.DOCUMENT_TYPE_NODE:
				newNode = null;
				break;
				
//			default:
//				throw "Node type not supported";	
		}
		
		return newNode;
	}
};

flow.xml.getLocalName = function(node) {
  return node.localName ? node.localName : node.baseName;
}

