/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.declare("Purexml", null, {

		HEADER_TABLE : "headerTable",
		HEADER_TREE : "headerTree",
		HEADER_CHILD : "headerChild",
		HEADER_FIELD : "headerField",

		TEXT_TABLE : "textTable",
		TEXT_TREE : "textTree",
		TEXT_CHILD : "textChild",
		TEXT_FIELD : "textField",

		TAB_CONTAINER : 'maintabs',
		TAB_WELCOME : 'tabWelcome',
		TAB_ROW : 'tabRow',
		TAB_CONTENT : 'tabContent',
		TAB_FORMAT : 'tabFormat',

		DISPLAY_HEADER : 'treeHeader',
		DISPLAY_TEXT : 'treeBlurb',
		DISPLAY_SECTION : 'tree',
		DISPLAY_DIALOG : 'treeDialog',

		NS_RAW : 'nsraw',
		MODE_PREVIEW : 'fly',
		MODE_CREATE : 'prep',

		VALIDATE : [
			{
				fields: [('col_' + this.uid)],
				tab: ('tabData_' + this.uid),
				message: "Please choose a DB2 column."
			}
		],

		constructor : function(ajaxUrl)
		{
			this.ajaxUrl = ajaxUrl
			this.treeUrl = ajaxUrl + '/method/getData/transport/xhr';
			// Received xml doc cache
			this.xmlDocSave = null;

			// Where to deposit XML XPath
			this.textbox = null;
		},

		setTableText : function(title, text)
		{
			this.tableTitle = title;
			this.tableText = text;
		},

		setContentText : function(title, text)
		{
			this.contentTitle = title;
			this.contentText = text;
		},

		setCriteriaText : function(title, text)
		{
			this.criteriaTitle = title;
			this.criteriaText = text;
		},

		setValueText : function(title, text)
		{
			this.valueTitle = title;
			this.valueText = text;
		},

		setUID : function(uid) {
			this.uid = uid;
		},

		init : function() {
			this.header_table = Purexml.prototype.HEADER_TABLE + "_" + this.uid;
			this.header_tree = Purexml.prototype.HEADER_TREE + "_" + this.uid;
			this.header_child =  Purexml.prototype.HEADER_CHILD + "_" + this.uid;
			this.header_field = Purexml.prototype.HEADER_FIELD + "_" + this.uid;

			this.text_table = Purexml.prototype.TEXT_TABLE + "_" + this.uid;
			this.text_tree = Purexml.prototype.TEXT_TREE + "_" + this.uid;
			this.text_child =  Purexml.prototype.TEXT_CHILD + "_" + this.uid;
			this.text_field = Purexml.prototype.TEXT_FIELD + "_" + this.uid;

			this.tab_container = Purexml.prototype.TAB_CONTAINER + "_" + this.uid;
			this.tab_welcome = Purexml.prototype.TAB_WELCOME + "_" + this.uid;
			this.tab_row =  Purexml.prototype.TAB_ROW + "_" + this.uid;
			this.tab_content = Purexml.prototype.TAB_CONTENT + "_" + this.uid;
			this.tab_format = Purexml.prototype.TAB_FORMAT + "_" + this.uid;

			this.display_header = Purexml.prototype.DISPLAY_HEADER + "_" + this.uid;
			this.display_text = Purexml.prototype.DISPLAY_TEXT + "_" + this.uid;
			this.display_section =  Purexml.prototype.DISPLAY_SECTION + "_" + this.uid;
			this.display_dialog = Purexml.prototype.DISPLAY_DIALOG + "_" + this.uid;

			this.ns_raw =  Purexml.prototype.NS_RAW + "_" + this.uid;
			this.mode_preview = Purexml.prototype.MODE_PREVIEW + "_" + this.uid;
			this.mode_create = Purexml.prototype.MODE_CREATE + "_" + this.uid;

			this.onClickExists();
		},

		getValue : function (value)
		{
			data = this.uncache();

			// if tree already rendered
			if (data) {
				// place click results in this form field
				this.textbox = value;

				this.setDialog(this.valueTitle, this.valueText, data);

				// show XML tree
				this.unhideDisplay();
			}
			else {
				window.alert('Please select a document element.');
				this.setTab(this.tab_row);
			}
		},

		subpath : function (id) {
			data = this.uncache();
			// if no params, assume function called back
			if (!id) var id = this.id;

			// if data not loaded
			if (!data){
				// store params
				this.id = id;

				// set callback
				this.callback = this.subpath;
				this.sendRequestTree();

				// wait for callback
				return;
			}

			// place click results in this form field
			this.textbox = id;
			this.setDialog(this.contentTitle,
						   this.contentText,
						   this.xmlDocSave);

			// show XML tree
			this.unhideDisplay();
		},

/**
	Get a representative row from server
*/
		sendRequestTree : function (url, row) {
			url = this.treeUrl;
			// request a representative row for this table, column
			var tbl = document.getElementById('tbl_'+this.uid).value;
			var col = document.getElementById('col_'+this.uid).value;

			if (!row) row = 'random';
			var _self = this;

			var args = {
				url:		url,			// url to query
				mimetype:	"text/html",	// mimetype of response
				method:		"get",			// submission method (get or post)
				content :   {table:tbl, column:col, row: row },
				load : function(type,data,evt) {
					_self.cache(type);
					if(_self.callback)
						_self.callback(type, data, evt);
				}
			};

			dojo.xhrGet(args);
		},

/* *
	Get a list of tables/columns from server
*/
		sendRequestSQL : function (url, event) {
			var args = {
				url:		url,			// url to query
				mimetype:	"text/html",	// mimetype of response
				method:		"get",			// submission method (get or post)
				preventCache:	true,
				content: {db: db },
				load:		onLoadSQL			// response handler
			};

			dojo.xhrGet(args);
		},

		setText : function (id, data) {
			var el = document.getElementById(id);
			el.innerHTML = data;
			return el;
		},

		setValue : function (id, data) {
			var el = document.getElementById(id);
			el.value = data;
			return el;
		},

		setXPath : function (data) {
		/*
		DB2 now aborts when fn:data is applied to a non-leaf element node
		Before, it would receive the contents of all the child nodes
		As a temporary fix, /text() is now appended to a non-leaf element node
		This returns an empty string
      	*/
			if (this.textbox
				&& (this.textbox != ('xpath_' + this.uid))
				&& data.indexOf('@') < 0) data += '/text()';

			this.setValue(this.textbox, data);
			this.hideDisplay();
		},

		setDialog : function (title, text, data) {
// TODO: set title		
//			this.setText(this.display_header, title);
//			dijit.byId(this.display_dialog).title = title;
			this.setText(this.display_text, text);
			this.setText(this.display_section, data);

			// Update namespace field
			var ns = document.getElementById(this.ns_raw);
			if (ns) {
				var spaces = ns.value;
				var nsbox = document.getElementById('namespace_'+this.uid);
				nsbox.value = spaces;
			}
		},

/**
	Display a list of tables/columns
*/
		displaySQL : function(data) {
			this.setDialog(this.tableTitle, this.tableText, data);
			// show it
			this.unhideDisplay();
		},

/**
	Display clickable tree rendering of XML
*/
		displayTree : function() {
			data = this.uncache();
			if (!data) {
				alert("Data is being loaded. Please try again");
			}
			// Place click results in this form field
			this.textbox = 'xpath_' + this.uid;

			this.setDialog(this.criteriaTitle, this.criteriaText, data);

			this.unhideDisplay();
		},

		onLoadTree : function(type, data, evt) {
			this.cache(data);
			if (this.callback) this.callback();
		},

		onLoadSQL : function(type, data, evt) {
//			this.displaySQL(data);
			this.displaySQL(type);
		},

		cache : function(data) {
			this.xmlDocSave = data;
		},

		uncache : function() {
			return this.xmlDocSave;
		},

		settable : function( table,col) {
			document.getElementById('tbl_'+this.uid).value=table;
			document.getElementById('col_'+this.uid).value=col;
			this.clearTree();
			this.sendRequestTree();
		},


/**
	Set the active dialog tab pane
*/
		setTab : function(tabId) {
			dojo.byId(this.tab_container).selectChild(tabId);
		},

		isFormValid : function() {
			for (var i = 0; i< Purexml.prototype.VALIDATE.length; i++) {
				if (!this.checkControls(Purexml.prototype.VALIDATE[i])) return false;
			}

			return true;
		},

		_validateRequired : function(field, text) {
          with (field) {
	        	if ((value == null) || (value == "")) {
	            	window.alert(text);
	            	return false;
	        	}
	        	value = value.replace(/^\s+|\s+$/, '');
	        	if(value.length < 1) {
	            	window.alert(text);
	            	return false;
	        	}
	        }
	        	return true;
        },
        // URL will be provided by caller, not hardcoded
       validateForm : function() {
            with (this.form) {
		        if (!this._validateRequired(table,
	                       'Please select a Table from available.')) {
		            return false;
		        }
		        if (!this._validateRequired(col,
	                    'Please fill out column from available.')) {
		            return false;
		        }
		        if (!this._validateRequired(xpath,
	                    'Please fill out the version field.')) {
		            return false;
		        }
            }
            return true;
        },


		checkControls : function(these) {
			var fields = these.fields;
			var tab = these.tab;
			var message = these.message;

			// for every field
			for (var i = 0; i < fields.length; i++) {
				var field = fields[i];
				var control = document.getElementById(field);

				// if blank
				if (control.value.length === 0) {
					// focus on it
					this.setTab(tab);
					control.focus();

					// complain
					window.alert(message);

					return false;
				}
			}

			return true;
		},

		submitForm : function(formId) {
			//TODO: Insert check for required elements
			var form = document.getElementById(formId);
			//window.alert('Submit');
			if(this.validateForm())
			//if(true)
			{
				form.submit();
			}
		},

		newElement : function(parent, type, id, cl, text) {
			var el = document.createElement(type);

			if (id) el.setAttribute('id', id);
			if (cl) el.setAttribute('class', cl);
			if (text) el.appendChild(document.createTextNode(text));

			if (parent) parent.appendChild(el);

			return el;
		},


		clearTree : function(doc) {
// clear title		
//			this.setText(this.display_header, '');
			this.setText(this.display_text, '');
			this.setText(this.display_section, '');

			// hide it
			this.hideDisplay();
		},

		hideDisplay : function(doc) {
			dijit.byId(this.display_dialog).hide();
		},

		unhideDisplay : function(doc) {
			var dlg = dijit.byId(this.display_dialog)
			dlg.resize();
			dlg.show();
		},

		show : function(id) {
			var el = document.getElementById(id);
			el.style.display = 'block';
		},

		hide : function(id) {
			var el = document.getElementById(id);
			el.style.display = 'none';
		},

		onClickExists : function() {
			this.hide('operatorBlock_' + this.uid);
			this.hide('valueBlock_' + this.uid);
		},

		onClickContains : function() {
			this.hide('operatorBlock_' + this.uid);
			this.show('valueBlock_' + this.uid);
		},

		onClickCompares : function() {
			this.show('operatorBlock_' + this.uid);
			this.show('valueBlock_' + this.uid);
		},
		
// invoke server - use it instead of InvokePlugin.invoke
    _handleResponse: function(response, args) {
        if(!response) {
            // no data from server
            throw new Error('Server response was empty.');
        }

        var data = dojo.fromJson(response);
        if(data['code'] == 0) {
            // unknown server error
//            this.showMessage(data['payload']);
            return;
        } else if (data['code'] > 0) {
            // known server error
            if (data['payload']['text']) {
//            	this.showMessage(data['payload']['text']);
            } else {
//            	this.showMessage(data['payload']);
            }
            
            return;
        }
        var method = data['method'];
        var params = data['payload'];

        try {
            // dispatch
            this[method](params);
        } catch (e) {
            // known client error
//            var msgs = dojo.i18n.getLocalization('hub', 'messages');
//            this.showMessage(msgs.CLIENT_ERROR + "\n" + e.message);
        }

    },

    invokeServer: function(method, data, transport) {
        var _self = this;
        
        if(typeof(data)=='string'){
        	data = dojo.by.id(data);
        }

        if(!transport) {
            transport = 'xhr';
        }

        if(transport == 'xhr') {
            var handleAs = "text";
        } else if (transport == 'iframe') {
            var handleAs = "text/html";
        } else {
            throw new Error("Unknown transport");
        }

        // clear any notification
//        var e = document.getElementById(this.getEditorId()+'_status');
//        if(e) {
//            e.innerHTML = '';
//		}
		
        var msgs = dojo.i18n.getLocalization('hub', 'messages');

		// build a request object and handler for dojo
		var kw = {
			url: _self.ajaxUrl+'/method/'+method+'/transport/'+transport,
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds

            error: function(response, args) {
                if(response.dojoType == "cancel"){
//                    _self.showMessage(msgs.CANCELLED);
                } else if (response.dojoType == "timeout"){
//                    _self.showMessage(msgs.TIMEOUT);
                } else {
//                    var err = TOM.i18n.man._('SERVER_ERROR') + " " + response.message;
//                    _self.showMessage(TOM.i18n.man._(err));
//					_self.showMessage(msgs.SERVER_ERROR+ " " + response.message);
                }
                return response;
            },

            load: function(response, args) {
                try {
                    data = response.body.childNodes[0].value;
                } catch(e) {
                    // not loaded via iframe
                }

				try {
                    if(hub.submit.processResponseCode(response)){
					    _self._handleResponse(response, args);
				    }
				} catch(e) {
	                  // error handling response
//	                  var err = msgs.CLIENT_ERROR + " " + e.message;
//	                  _self.showMessage(err);
				}

                return response;
            }
		};

        // decide what the data source is
        if(data.elements != undefined) {
            // data from a form
            kw['form'] = data;
        } else {
            // data from object name/value pairs
            kw['content'] = data;
        }
        
        // pick a transport and send
        if(transport == "iframe") {
            dojo.require("dojo.io.iframe");
			dojo.io.iframe.send(kw);
        } else {
            dojo.xhrPost(kw);
        }
        return false;
    }
    
//    showMessage: function(text) {
//        var e = dojo.byId(this.getEditorId()+'_status');
//        hub.notify(text, e);
//    }
		
});
