/*
	Copyright (c) 2007-2008, IBM Corp.
*/

dojo.provide("plugins.sql.BuilderPlugin");

dojo.require("hub.widgets");
dojo.require("dijit.Dialog");
dojo.require("dijit.Tooltip");
dojo.require("dijit.TitlePane");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dijit.form.ComboBox");

/**
 * Provides methods needed for the SQL builder UI. Defined outside the server
 * side plug-in class to make code more sane.
 */
dojo.declare('plugins.sql.BuilderPlugin', [hub.managers.InvokePlugin], {
	id: null,
	json: null,
	tables: null,
	filterGroups: null,
	tablesCheckBoxList: null,
	columnsCheckBoxList: null,
	advancedTitlePane: null,
	advancedPreview: null,

	popup_div: null,
	popup: null,


    constructor: function() {
	    this.labels = dojo.i18n.getLocalization('plugins.sql', 'labels');
	},

	init: function(tables, json) {
	    this.tables = tables;
	    this.json = dojo.fromJson(json);

	    this.filterGroups = [];
	    console.obj = this;
	},

	/**
	 * Called when the editor is loaded.
	 * Initializes variables, adds event listeners, and
	 * restores the previous state if not a new feed
	 */
	onLoadEditor: function() {

	    this.id = this.getEditorId();
	    this.tablesCheckBoxList = new hub.widgets.CheckBoxList({}, dojo.byId(this.id + '_tables'));
	    this.tablesCheckBoxList.appendGroup('tables', this.tables);
	    this.columnsCheckBoxList = new hub.widgets.CheckBoxList({}, dojo.byId(this.id + '_columns'));

	    var div = dojo.byId(this.id + '_advancedCheck');
	    var params = {'open':false, 'title': this.labels.SQL_BUILDER_ADVANCED};
	    this.advancedTitlePane = new dijit.TitlePane(params, div);

	    dojo.connect(dojo.byId(this.id + '_reset'), 'onclick', this, 'clickReset');
	    dojo.connect(dojo.byId(this.id + '_preview'), 'onclick', this, 'preview');

	    dojo.connect(dojo.byId(this.id + '_insert'), 'onclick', this, 'insertQuery');
	    dojo.connect(dojo.byId(this.id + '_previewAdvanced'), 'onclick', this, 'previewAdvanced');

	    dojo.connect(this.tablesCheckBoxList, 'onChange', this, 'tableOnChange');
	    dojo.connect(this.columnsCheckBoxList, 'onChange', this, 'columnOnChange');

        // create orderby element
        var data = {'identifier':'value', 'label':'label', 'items':[]};
        var columnStore = new dojo.data.ItemFileWriteStore({'data':data});
        var params = { 'autocomplete':'true', 'store':columnStore, 'searchAttr':'value' };
        var orderBy = new dijit.form.FilteringSelect(params, dojo.byId(this.id + '_orderby'));

	    // Restore previous query if it exists
	    if(this.json != null && this.json != '') {
		  	//var msgs = dojo.i18n.getLocalization('hub', 'panels');

        	this.popupDestroy();

	        // create 'Restoring query...' popup window
	        var popup_params = {'title': this.labels.SQL_RESTORING_TITLE, 'open':false, 'duration':200};
            this.popup = new dijit.Dialog(popup_params, dojo.byId(this.id + '_popup'));
	        this.popup.show();

	        var div = document.createElement('div');
	        var img = document.createElement('img');
	        img.src = hub.config.base_url + "/images/spinner.gif";
	        div.appendChild(img);
		    div.innerHTML += " " + this.labels.SQL_RESTORING_QUERY + "...";
	        this.popup.setContent(div);

			if(this.json.error) {
		        div.innerHTML += "<br><br>"+this.labels.SQL_RESTORING_QUERY_ERROR;

		        // add full error description to standard status pane
		        //hub.notify(this.json.error,dojo.byId(this.id +'_status_sql'));
		        this.showMessage(this.json.error, 'E');

		        // set failed query into advanced pane
	            if(!this.advancedTitlePane.open)
	                this.advancedTitlePane.toggle();
	            dojo.byId(this.id + '_textarea').value = this.json.query;
			}
			else {
		        // set tables checked (makes ajax calls)
		        for(var i=0; i<this.json.tables.length; i++) {
		            this.tablesCheckBoxList.setChecked('tables', this.json.tables[i], true);
			        // call directly
		            this.tableOnChange(this.json.tables[i],true);
		        }

		        var orderBy = dijit.byId(this.id + '_orderby');
	            orderBy.setValue(this.json.orderby);

		        // set sort (ASC or DESC)
		        var sort = dojo.byId(this.id + '_sort');
		        for(var child in sort.childNodes) {
		           if(sort.childNodes[child].value == this.json.sort) {
		                sort.childNodes[child].selected = true;
		                break;
		            }
		        }

		        // set LIMIT
		        dojo.byId(this.id + '_limit').value = this.json.limit;

		        // set advanced query
		        if(this.json.advanced != null) {
		            if(!this.advancedTitlePane.open)
		                this.advancedTitlePane.toggle();
		            dojo.byId(this.id + '_textarea').value = this.json.advanced;
		        }

				// finish restoring the previously saved state of the builder
		        this.restoreQuery(this.json);

			    // destroy loading message and show the page
			    this.popupDestroy();

			    // Builder state is restored. Clear it so it doesn't try to
			    // restore it again while user is editing
		    	this.json = null;
		    }
		}
	},

	/**
	 * Callback function when all ajax calls for table columns have completed.
	 * Finishes restoring the previously saved state of the builder.
	 * @param obj: holds json data needed to restore a previously saved state
	 */
	restoreQuery: function(obj) {
	    // check columns
	    for(var i=0; i<obj.columns.length; i++) {
	        var table = obj.columns[i].substring(0, obj.columns[i].indexOf('.'));
	        this.columnsCheckBoxList.setChecked(table, obj.columns[i], true);
	        // call directly
            this.columnOnChange(obj.columns[i],true);
	    }

	    // restore filter groups

	    for(var i in obj.filterGroups) {
	        if(i > 0 || this.filterGroups.length == 0)
	            this.addFilterGroup();
	        // restore filters
	        for(var j in obj.filterGroups[i].filters) {
	            if(j > 0 || this.filterGroups[i].filters.length == 0)
	                this.filterGroups[i].addFilter();
	            var filter = this.filterGroups[i].filters[j];
	            var saved = obj.filterGroups[i].filters[j];

	            // set value for column name
	            filter.columnSelect.setValue(saved.column);

	            // set value of operator (>, <, =, !=, etc...)
	            for(var child in filter.operator.childNodes) {
	                if(filter.operator.childNodes[child].value == saved.operator) {
	                    filter.operator.childNodes[child].selected = true;
	                    break;
	                }
	            }
	            // set value of column/param (combobox)
	            filter.comboBox.setValue(saved.comboBox);

	            // set parameter type
	            for(var child in filter.ptype.childNodes) {
	               if(filter.ptype.childNodes[child].value == saved.ptype) {
	                    filter.ptype.childNodes[child].selected = true;
	                    break;
	                }
	            }

	            // set parameter value
            	filter.pvalue.value=saved.pvalue;
				// adjust filter input fields attributes
				filter.changeParam();

	            // set condition for filter
	            for(var child in filter.condition.childNodes) {
	               if(filter.condition.childNodes[child].value == saved.condition) {
	                    filter.condition.childNodes[child].selected = true;
	                    break;
	                }
	            }
	        }

            // set compound condition for filter group
	        var done = this.filterGroups[i].done.childNodes;
	        for(var child in done) {
	           if(done[child].value == obj.filterGroups[i].condition) {
	                done[child].selected = true;
	                break;
	            }
	        }
	    }
	},

	/**
	 * Called when a table checkbox is clicked.
	 * If checked, an ajax call is made to the server requesting a
	 * list of the table's columns. Callback function is showColumns()
	 */
	tableOnChange: function(tableName, isset) {
	    if(isset) {
	        // only invoke if we've got a selected value
			hub.busyPage(this.tablesCheckBoxList.domNode,true,true);
    	    var req = new hub.submit.Request(hub.urls.getAjaxUrl(this.plugin_id,this.entry_id,
    	        'getColumns'), {'table' : tableName});
    	    req.submit();
		    this.showColumns(req.response);
	    }
	    else {
	        if(this.tablesCheckBoxList.getAllChecked().length == 0) {
	            this.reset();
	        }
	        else {
	            this.columnsCheckBoxList.removeGroup(tableName);
	            this.updateColumns();
	        }
	    }
	},

	/**
	 * Callback function after an ajax call is made requesting a table's columns.
	 * Updates the checkBoxList object as well as all the drop-down inputs on the page.
	 * @param params: return object from the server (contains the name
	 *   of the table and the names of the table's columns)
	 */
	showColumns: function(params) {
	    var columns = params['columns'];
	    var tableName = params['table'];
	    var star = [tableName + '.*'];
	    this.tablesCheckBoxList.setChecked('tables', tableName, true);
	    this.columnsCheckBoxList.appendGroup(tableName, star.concat(columns));
	    if(this.filterGroups.length == 0)
	        this.addFilterGroup();

	    this.updateColumns();

	    var filters = dojo.byId(this.id + '_filters');
	    var order = dojo.byId(this.id + '_order');

	    if(this.tablesCheckBoxList.getAllChecked().length > 0) {
	    	// comment out for release 1
	        //filters.style.display = 'block';
	        order.style.display = 'block';
	    }
	    else {
	    	// comment out for release 1
	        //filters.style.display = 'none';
	        order.style.display = 'none';
	        this.clearFilters();
	    }
		hub.busyPage(this.tablesCheckBoxList.domNode,false,false);
	},

	/**
	 * Called whenever a column checkbox is clicked. If the '*' box is checked,
	 * check all the column checkboxes for that table. Likewise, if it is
	 * unchecked, uncheck all the column checkboxes for that table
	 */
	columnOnChange: function(columnName, isset) {
	    var dot = columnName.lastIndexOf('.');
	    var table = columnName.substring(0, dot);
	    var column = columnName.substring(dot + 1);

	    if(column == "*") {
	        this.columnsCheckBoxList.setGroupChecked(table, isset);
	    }
	    // if uncheck any item in the group, we must unckeck '*' also
	    else if(isset==false && this.columnsCheckBoxList.isChecked(table+".*")) {
	    	this.columnsCheckBoxList.setChecked(table,table+".*",isset);
	    }
	},

	/**
	 * Adds a new group of filters to the ui and to the filterGroup object
	 */
	addFilterGroup: function() {
	    var filterDiv = dojo.byId(this.id + '_filters');
	    var newFilterGroup = new plugins.sql.FilterGroup();
	    var len = this.filterGroups.length;
	    newFilterGroup.updateColumns(this.columnsCheckBoxList.getAll());
	    filterDiv.appendChild(newFilterGroup.getHtmlElement());

	    dojo.connect(newFilterGroup.getDone(), 'onchange', this, 'updateFilterGroup');

        // new group has at least one filter
        var filterColumn=newFilterGroup.filters[0].columnSelect;
	    dojo.connect(filterColumn, 'onChange', newFilterGroup, 'updateFilterGroup');

	    this.filterGroups[len] = newFilterGroup;
	    newFilterGroup.updateFilterGroup();
	},

	/**
	 * Called when a filterGroup should be added or removed
	 */
	updateFilterGroup: function(event) {
	    var clear = false;
	    var filtersDiv = dojo.byId(this.id + '_filters');
	    var length = this.filterGroups.length;
	    var pos = length;
	    for(var i=0; i<length; i++) {
	        var done = this.filterGroups[i].getDone();
	        if(clear) {
	            filtersDiv.removeChild(this.filterGroups[i].getHtmlElement());
	        }
	        else if(done.value == "DONE") {
	            clear = true;
	            pos = i;
	        }
	        else if(i==length-1) {
	            this.addFilterGroup();
	            break;
	        }
	    }
	    if(clear) {
            this.filterGroups.splice(pos+1,length-pos-1);
	    }

		this.filterGroups[this.filterGroups.length-1].updateFilterGroup();
	},

	/**
	 * Updates all the dropdown forms on the page with the new columns.
	 */
	updateColumns: function() {
	    var columns = this.columnsCheckBoxList.getAll();
	    for(var i=0; i<this.filterGroups.length; i++){
	        this.filterGroups[i].updateColumns(columns);
	    }

	     var orderBy = dijit.byId(this.id + '_orderby');
	     orderBy.innerHTML = '';

         var data = {'identifier':'value', 'label':'label', 'items':[]};
         orderBy.store = new dojo.data.ItemFileWriteStore({'data':data});

	     // add default empty option
	     if(columns.length>0) {
           	orderBy.store.newItem({'value':'', 'label':''});
	     }

	     for(var i=0; i<columns.length; i++) {
	        if(columns[i].lastIndexOf('.*')==-1) {
           	   orderBy.store.newItem({'value':columns[i], 'label':columns[i]});
	        }
	     }
	},

	/**
	 * Resets the builder, clears out all values
	 */
	reset: function(event) {

	    // clear tables and columns
	    this.tablesCheckBoxList.setAllChecked(false);
	    this.columnsCheckBoxList.clear();

		// clear oreder by
        var orderBy = dijit.byId(this.id + '_orderby');
        orderBy.setValue('');

	    // clear limit
        dojo.byId(this.id + '_limit').value ='';

	    // clear filters
	    this.clearFilters();
	    dojo.byId(this.id + '_filters').style.display = 'none';
	    dojo.byId(this.id + '_order').style.display = 'none';

	    // clear advanced pane and close
        dojo.byId(this.id + '_textarea').value = '';
        if(this.advancedTitlePane.open)
 	       this.advancedTitlePane.toggle();

		this.clearStatusMessage();
	},

	/**
	 * Resets the builder, clears out all values by clicking Reset
	 */
	clickReset: function(event) {
	    if(confirm(this.labels.SQL_RESET_WARNING)) {
	    	this.reset();
		}
	},


	/**
	 * Removes all filters and filterGroups
	 */
	clearFilters: function() {
	    for(var i=0; i<this.filterGroups.length; i++) {
	        var elem = this.filterGroups[i].getHtmlElement();
	        elem.parentNode.removeChild(elem);
	    }
	    this.filterGroups = [];
	},

	/**
	 * Validate filters against correct parameters and replace params in SQL if necessary
	 * @param sql SQL statement generated from UI builder
	 * @param replace if necessary to replace parameters with values
	 * @return SQL statement, if validation success
	 * otherwize null
	 */
	validateSQLFilter: function(/*String*/sql,/*boolean*/replace) {

	   var statement=sql;
       //var msgs = dojo.i18n.getLocalization('hub', 'panels');

	   for(var i=0;i<this.filterGroups.length;i++) {
	   		var filterGroup=this.filterGroups[i];

		    for(var j=0;j<filterGroup.filters.length;j++) {
		       var filter=filterGroup.filters[j];
		       var columnValue=filter.columnSelect.getValue();
       		   var pname=filter.getParamName();
		       var pvalue=filter.getParamValue();
		       var ptype=filter.ptype.value;

console.log("sqlBuilder: validateSQLFilter(): ptype="+ptype+"; pname="+pname+"; pvalue="+pvalue);

		       // check for first column
		       if(columnValue!=null && columnValue!='') {

				   // check for select column
				   if(ptype==filter.column && (pname==null || pname==''))	{
			          //hub.notify(this.labels.SQL_MISSING_PARAMETER, dojo.byId(this.id +'_status_sql'));
			          this.showMessage(this.labels.SQL_MISSING_PARAMETER, 'E');
		       		  return null;
				   }
				   else {
					   // check for parameters only (not a constants)
				       if(ptype==filter.param_int ||
				          ptype==filter.param_string) {
				          if(pname==null || pname==':') {
					          //hub.notify(this.labels.SQL_MISSING_PARAMETER, dojo.byId(this.id +'_status_sql'));
					          this.showMessage(this.labels.SQL_MISSING_PARAMETER, 'E');
				       		  return null;
				          }
				       }
					   // check for int values only
				       if(ptype==filter.param_int ||
				          ptype==filter.const_int) {
						   if(pvalue=='') {
						   	  var msg=this.labels.SQL_MISSING_PARAMETER_VALUE;
						   	  msg=msg.replace('${0}',pname.substr(1));
				              //hub.notify(msg, dojo.byId(this.id +'_status_sql'));
				              this.showMessage(msg, 'E');
			       		      return null;
			       		   }
			       		   if(isNaN(parseInt(pvalue))) {
					          //hub.notify(this.labels.SQL_INVALID_PARAMETER_VALUE ,dojo.byId(this.id +'_status_sql'));
					          this.showMessage(this.labels.SQL_INVALID_PARAMETER_VALUE, 'E');
					          return null;
					       }
				       }
				   }
			   }
	           // replace param with value
	           if(replace==true && ptype!=filter.column) {
	               statement=statement.replace(pname,pvalue);
	           }
		   }
	   }
	   return statement;
	},

	/**
	 * Previews the SQL query created from the builder
	 */
	preview: function(event) {
	   this.clearStatusMessage();
	   //var msgs = dojo.i18n.getLocalization('hub', 'panels');

	   if(this.tablesCheckBoxList.getAllChecked().length == 0 ||
	      this.columnsCheckBoxList.getAllChecked().length == 0) {
          //hub.notify(this.labels.SQL_EMPTY,dojo.byId(this.id +'_status_sql'));
          this.showMessage(this.labels.SQL_EMPTY,'E');
	      return;
	   }

	   var limit=dojo.byId(this.id + '_limit').value;
	   if(limit!="" && (isNaN(parseInt(limit)) || limit < 0)) {
          //hub.notify(this.labels.SQL_INVALID_LIMIT,dojo.byId(this.id +'_status_sql'));
          this.showMessage(this.labels.SQL_INVALID_LIMIT,'E');
		  return;
	   }

	   var sql = this.getSQL();

	   var result=this.validateSQLFilter(sql,true);
	   if(result==null) return;

	   this._preview({'sql':result, 'limit':limit,'max':10 });
	},

	/**
	 * Previews the SQL query using the query from the 'advanced' pane
	 */
	previewAdvanced: function(event) {
		this.clearStatusMessage();

	    //var msgs = dojo.i18n.getLocalization('hub', 'panels');
	    var sql = dojo.byId(this.id + '_textarea').value;
        if(sql==null || sql=='') {
           //hub.notify(this.labels.SQL_EMPTY,dojo.byId(this.id +'_status_sql'));
           this.showMessage(this.labels.SQL_EMPTY, 'E');
	       return;
        }

	    // validate advanced SQL against parameters
	   	var pindex=sql.indexOf(':');
	   	if(pindex!=-1) {
	    	//var msgs = dojo.i18n.getLocalization('hub', 'panels');

        	this.popupDestroy();

	        // create 'Preview Advanced' popup window
	        var popup_params = {'title': this.labels.SQL_PREVIEW_ADVANCED, 'open':false, 'duration':200};
            this.popup = new dijit.Dialog(popup_params, dojo.byId(this.id + '_popup'));
	        this.popup.show();

		    var body = document.createElement('div');
		    body.style.fontSize = "0.8em";
		    body.style.textAlign = "center";

		    var p = document.createElement('p');
		    p.style.textAlign = "center";
		    p.innerHTML=this.labels.SQL_PARAMETERS_ADVANCED;
		    body.appendChild(p);

			this.advancedPreview=document.createElement('textarea');
			this.advancedPreview.style.width='45em';
			this.advancedPreview.style.height='90px';
			this.advancedPreview.value=sql;

		    p = document.createElement('p');
		    p.appendChild(this.advancedPreview);
			body.appendChild(p);

		    var br = document.createElement('br');
			body.appendChild(br);

		    var input = document.createElement('input');
		    input.type = "button";
		    input.value = this.labels.SQL_PREVIEW_RUN;
		    dojo.connect(input, 'onclick', this, '_runAdvancedPreview');
			body.appendChild(input);

		    this.popup.setContent(body);
		    return;
	   	}

	    this._preview({'sql':sql});
	},

	/**
	 * Called from popup Preview Advanced
	 */
	_runAdvancedPreview: function() {
		var sql_preview=this.advancedPreview.value;

		this.popupDestroy();
		this.advancedPreview=null;

	    this._preview( {'sql':sql_preview} );
	},

	/**
	 * Called by both preview functions to make the ajax call to the server
	 * with the SQL query. Renders a loading dialog box while waiting. The
	 * callback function is _renderPopup()
	 *
	 * @param params: object containing the sql query and the max number of results
	 */
	_preview: function(params) {

       	this.popupDestroy();

    	//var msgs = dojo.i18n.getLocalization('hub', 'panels');
	    var popup_params = {'title': this.labels.SQL_EXECUTING_TITLE, 'open':false, 'duration':200};
	    this.popup = new dijit.Dialog(popup_params, dojo.byId(this.id + '_popup'));
	    this.popup.show();

console.log("sql.BuilderPlugin: _preview: show popup");

	    this.popup_div = document.createElement('div');
	    var img = document.createElement('img');
	    img.src = hub.config.base_url + "/images/spinner.gif";
	    this.popup_div.appendChild(img);
	    this.popup_div.innerHTML += " " + this.labels.SQL_EXECUTING_QUERY + "...";
	    this.popup.setContent(this.popup_div);

    	var req = new hub.submit.Request(hub.urls.getAjaxUrl(this.plugin_id,this.entry_id,'testQuery'), params);
    	req.submit();

		this._renderPopup(req.response);
	},

	/**
	 * Callback function after an ajax call is made to the server to get
	 * a sql query result. Displays the results using a dojo Table inside
	 * the same dojo dialog box used for the 'loading' message.
	 *
	 * @param params: object containing the results, the number of results
	 *      to be displayed, and the total results of the query
	 */
	_renderPopup: function(params) {

console.log("sql.BuilderPlugin: _renderPopup start.");

	    if(!(params instanceof Object)) {
	        this.popupDestroy();
	        return;
	    }

    	//var msgs = dojo.i18n.getLocalization('hub', 'panels');

		if(params.error) {
	        this.popup_div.innerHTML += "<br><br> "+this.labels.SQL_EXECUTING_QUERY_ERROR;

	        // add full error description to standard status pane
	        //hub.notify(params.error,dojo.byId(this.id +'_status_sql'));
	        this.showMessage(params.error, 'E');
	    }
		else {
		    var width = window.innerWidth * 0.8;
		    var height = window.innerHeight * 0.9;

		    var body = document.createElement('div');
		    body.style.fontSize = "0.8em";
		    body.style.textAlign = "center";

		    var table = document.createElement('div');

		    var p = document.createElement('p');
		    p.style.textAlign = "center";
		    var summary = dojo.string.substitute(this.labels.SQL_EXECUTING_SUMMARY,
                [params['result'].length, params['total']]);
		    p.innerHTML = summary;

		    body.appendChild(table);
		    body.appendChild(p);

			var skipped=params.skipped;
			if(skipped>0) {
			    var p = document.createElement('p');
			    p.style.textAlign = "center";
			    var text = dojo.string.substitute(this.labels.SQL_EXECUTING_SKIPPED,[skipped]);
			    p.innerHTML = text;
			    body.appendChild(p);
			}

		    var input = document.createElement('input');
		    input.type = "button";
		    input.value = this.labels.SQL_PREVIEW_CLOSE;
		    dojo.connect(input, 'onclick', this, 'popupDestroy');
		    body.appendChild(input);

		    this.popup.setContent(body);

		    table2 = new hub.widgets.Table({},table);
		    table2.addRows(params['result']);

		    if(params['result'].length < 20) {
		        this.popup.resize({'w':width});
		    }
		    else {
		        this.popup.resize({'w':width, 'h':height});
		    }
		    this.popup.layout();
		    this.popup.domNode.style.backgroundColor = "white";
		}

console.log("sql.BuilderPlugin: _renderPopup end.");

	},

	/**
	 * Creates a SQL query from the builder UI and inserts it into
	 * the "advanced" text area.
	 *
	 * @param event:
	 */
	insertQuery: function(event) {
		if(dojo.byId(this.id + '_textarea').value=='' || confirm(this.labels.SQL_INSERT_WARNING)) {
	    	dojo.byId(this.id + '_textarea').value = this.getSQL();
	    }
	},

	/**
	 * Returns the SQL query generated from the builder UI
	 * @return: the SQL query generated from the builder UI as a string
	 */
	getSQL: function() {
		var sql = "";
	    var where = "";
	    var orderBy = "";

	    if(dojo.byId(this.id + '_filters').style.display == 'block') {
	        var where = "";
	        for(var i in this.filterGroups)
	            where += this.filterGroups[i].getSQL();
	        if(dojo.trim(where) != '')
	            where = " WHERE " + where;
	    }

	    if(dojo.byId(this.id + '_order').style.display == 'block') {
	        //var orderby = dojo.byId(this.id + '_orderby');
	        var orderby = dijit.byId(this.id + '_orderby');
	        var sort = dojo.byId(this.id + '_sort');
	        if(orderby.getValue() != null && orderby.getValue() != "") {
	            orderBy = " ORDER BY " + orderby.getValue() + " " + sort.value;
	        }

	        // limit is handled by JDBC API, so we don't need do anything here
	    }

		// check for selected tables and columns
		if(this.tablesCheckBoxList.getAllChecked().length==0 ||
			this.columnsCheckBoxList.getAllChecked().length==0) {
			return sql;
		}

		// exclude '.*' column from SELECT as we already have
		// columns selected to get correct SQL clause
		var checked=this.columnsCheckBoxList.getAllChecked();
        var columns = new Array();
        for(var i in checked) {
            if(checked[i].lastIndexOf('.*')==-1) {
	            columns.push(checked[i]);
            }
        }

		sql = "SELECT " + columns.join(", ") + " FROM "
	        + this.tablesCheckBoxList.getAllChecked().join(", ")
	        + where + orderBy;

	    return sql;
	},

	/**
	 * Returns an object containing the information needed to restore
	 * the state of the builder UI.
	 *
	 * @return: an object with all the values used to restore the UI
	 */
	getJSON: function() {
	    var obj = {};
	    obj.tables   = this.tablesCheckBoxList.getAllChecked();
	    obj.columns  = this.columnsCheckBoxList.getAllChecked();
	    obj.orderby  = dijit.byId(this.id + '_orderby').getValue();
	    obj.sort = dojo.byId(this.id + '_sort').value;
	    obj.limit = dojo.byId(this.id + '_limit').value;
	    obj.filterGroups = [];
	    for(var i in this.filterGroups) {
	        obj.filterGroups[i] = this.filterGroups[i].getJSON();
	    }
	    return obj;
	},

	/**
	 * Clears status message
	 */
	clearStatusMessage: function() {
          dojo.byId(this.id +'_status_sql').innerHTML='';
	},

	/**
	 * Generates the SQL query and submits it to the server.
	 * Also submits an object with data needed to restore the UI.
	 */
	submit: function() {

console.log("SQL Builder submit() start");

		this.clearStatusMessage();

	    var sql="";
	    var json = this.getJSON();
	    if(this.advancedTitlePane.open) {
	        sql = dojo.byId(this.id + '_textarea').value;
	        json.advanced = sql;
	    }
	    else {
	        sql = dojo.trim(this.getSQL());
	    }

	    //var msgs = dojo.i18n.getLocalization('hub', 'panels');

	    if(sql=="") {
	        //hub.notify(this.labels.SQL_EMPTY,dojo.byId(this.id +'_status_sql'));
	        this.showMessage(this.labels.SQL_EMPTY, 'E');
	        this.workflow.setBusy(false);
	    	return false;
	    }

	   // validate limit
	   var limit=dojo.byId(this.id + '_limit').value;
	   if(limit!="" && (isNaN(parseInt(limit)) || limit < 0)) {
          //hub.notify(this.labels.SQL_INVALID_LIMIT,dojo.byId(this.id +'_status_sql'));
          this.showMessage(this.labels.SQL_INVALID_LIMIT, 'E');
		  return false;
	   }

	   var result=this.validateSQLFilter(sql,false);

console.log("SQL filter validation result: "+result);

	   if(result==null) {
	      this.workflow.setBusy(false);
	   	  return false;
	   }

	   json = dojo.toJson(json);

	   this.invokeServer('saveSQL', {'sql':sql, 'limit':limit, 'json':json});

console.log("SQL Builder submit() end");

	},

	/**
	 *  Hides popup.
	 */
	popupDestroy: function() {

console.log("popupDestroy: start");

		if(this.popup==null) {
			return;
		}
		// We need to hide popup only (thus freeing any connections) MH-88, BZ #2774

		// !! do not call destroy() as we get script error:
		// FF:
		// this.domNode has no properties
		// https://localhost:8443/mashuphub/client/other/dojo-1.0.2/dojo/dojo.js Line 79
		// IE6:
		// Error: this.domNode.style is null or not a object
		// Code: 0
		// this error fail jscripts and avoid further 'preview'/'preview advanced'
		// IE hanging up in blocked state.

   		this.popup.hide();
   		this.popup=null;

console.log("popupDestroy: end");

	}

});

/**
 * Represents a group of filters. Creates the html elements
 * for the UI and stores the values.
 */
dojo.declare('plugins.sql.FilterGroup', null, {
    constructor: function() {
	    this.labels = dojo.i18n.getLocalization('plugins.sql', 'labels');

        this.outerDiv = document.createElement('div');

        this.div = document.createElement('div');
        this.div.style.className = "unsized";
        this.outerDiv.appendChild(this.div);

        var br = document.createElement('br');

		var label=document.createElement('div');
        label.style.margin = "20px 0px 5px 20px";
        label.style.width = "11em";
		label.innerHTML = this.labels.SQL_BUILDER_COMPOUND;

		var span=document.createElement('span');
		span.title=this.labels.SQL_BUILDER_FILTER_COMPOUND_TOOLTIP;
        var filter = new plugins.sql.Filter();
        this.done = filter.newConditionSelect();
		this.done.style.margin="0px 0px 20px 20px";
		span.appendChild(this.done);

        this.outerDiv.appendChild(br);
        this.outerDiv.appendChild(br);
        this.outerDiv.appendChild(label);
        this.outerDiv.appendChild(span);
        this.outerDiv.appendChild(br);
        this.outerDiv.appendChild(br);

        this.filters = [];
        this.columns = [];
        this.addFilter();
    },

    /**
     * Adds a new filter to the filter group
     */
    addFilter: function() {
        var filter = new plugins.sql.Filter();
        filter.updateColumns(this.columns);
        this.filters[this.filters.length] = filter;
        this.div.appendChild(filter.getHtmlElement());
        dojo.connect(filter.getDone(), 'onchange', this, 'updateFilters');

        var remove = filter.getRemove();
        dojo.connect(remove, 'onclick', this, 'removeFilter');

        if(this.filters.length==1) {
	        // exclude Remove button for first filter
            remove.parentNode.removeChild(remove);
        }
    },

    /**
     * Removes a filter from the filter group.
     * Called when a Remove button is clicked
     * @param event:
     */
    removeFilter: function(event) {
        var div = event.target.parentNode;
        div.removeAttribute('id');

        div.parentNode.removeChild(div);

        var length = this.filters.length;
        for(var i=1; i<length; i++) {
	        var fdiv=this.filters[i].getHtmlElement();
            if(fdiv.id=='') {
	            this.filters.splice(i,1);
				if(i==length-1) {
					this.filters[i-1].getDone().value="DONE";
				}
				break;
            }
        }
        length = this.filters.length;
    },

    /**
     * Called when a dropdown for AND/OR/AND NOT
     * is set. Adds or Removes filters accordingly.
     *
     * @param event:
     */
    updateFilters: function(event) {
        var clear = false;
        var length = this.filters.length;

        var pos = length;
        for(var i=0; i<length; i++) {
            if(clear) {
                var div = this.filters[i].getHtmlElement();
                div.parentNode.removeChild(div);
            }
            else if(this.filters[i].getDone().value == "DONE") {
                clear = true;
                pos = i;
            }
            else if(i==length-1) {
                this.addFilter();
            }
        }

        if(clear) {
            this.filters.splice(pos+1,length-pos-1);
        }
    },

    /**
     * Updates the filter group
     * @param columns: array of column names
     */
    updateFilterGroup: function() {

	    // adjust filterGroup Compound Condition
	    var lastFilter=this.filters[this.filters.length-1];
	    var lastFilterSelect=lastFilter.columnSelect.getValue();

	    if(this.filters.length==1 &&
	      (lastFilterSelect==null || lastFilterSelect=='')) {
		   this.done.disabled=true;
		}
		else {
		   this.done.disabled=null;
		}
    },

    /**
     * Updates the list of columns in every dropdown
     *
     * @param columns: array of column names
     */
    updateColumns: function(columns) {
        this.columns = columns;
        for(var i=0; i<this.filters.length; i++) {
            this.filters[i].updateColumns(columns);
        }
    },

    /**
     * Generates the portion of the SQL query represented by this filter group
     * @return: portion of the SQL string
     */
    getSQL: function() {
        var sql = "";
        for(var i=0; i<this.filters.length; i++)
            sql += this.filters[i].getSQL() + " ";
        if(dojo.trim(sql) == '')
            return '';
        sql = "(" + dojo.trim(sql) + ")";
        if(this.done.value != "DONE")
            sql += " " + this.done.value + " ";
        return sql;
    },

    /**
     * Creates an object with data needed to restore the UI
     * @return: json object
     */
    getJSON: function() {
        var filters = {'filters':[]};
        for(var i in this.filters) {
            var json = this.filters[i].getJSON();
            if(json != null && json != '') {
                filters.filters.push(json);
            }
        }
        filters.condition = this.done.value;
        return filters;
    },

    //
    // Doh, I didn't realize everything is public in js
    //
    getHtmlElement: function() {
        return this.outerDiv;
    },

    getDone: function() {
        return this.done;
    }
});

/**
 * Represents a single filter. Creates the html elements
 * for the UI and stores the values. One filter consists
 * of a column dropdown, an 'operator' dropdown (>, <, =, etc),
 * a comboBox for column names or user input strings or ints,
 * a 'parameter' dropdown stating the type of value for the
 * comboBox, and finally, a 'condition' dropdown (AND, OR, etc).
 * The last filter is denoted by leaving this last dropdown blank.
 */
dojo.declare('plugins.sql.Filter', null, {
    constructor: function() {

	    this.labels = dojo.i18n.getLocalization('plugins.sql', 'labels');

        this.div = document.createElement('div');
        // set attribute to check when delete
        this.div.setAttribute('id','filter');

		// first column select
		var span=document.createElement('span');
		span.title=this.labels.SQL_BUILDER_FILTER_COLUMN_TOOLTIP;
        var select1 = document.createElement('select');
		span.appendChild(select1);
        this.div.appendChild(span);

        var data = {'identifier':'value', 'label':'label', 'items':[]};
        var columnStore = new dojo.data.ItemFileWriteStore({'data':data});
        var params = { 'autocomplete':'true', 'store':columnStore, 'searchAttr':'value' };
        this.columnSelect = new dijit.form.FilteringSelect(params, select1);
        dojo.connect(this.columnSelect, 'onChange', this, 'changeColumnSelect');

        this.operator = this.newOperatorSelect();
        this.operator.style.width="5em";
        this.operator.title=this.labels.SQL_BUILDER_FILTER_OPERATOR_TOOLTIP;
        this.div.appendChild(this.operator);

		// second column select (dijit combobox)
		span=document.createElement('span');
		span.title=this.labels.SQL_BUILDER_FILTER_COLUMN_PARAM_TOOLTIP;
        var select2 = document.createElement('select');
		span.appendChild(select2);
        this.div.appendChild(span);

        var data = {'identifier':'name', 'label':'label', 'items':[]};
        var columnStore = new dojo.data.ItemFileWriteStore({'data':data});
        var params = { 'autocomplete':'true', 'store':columnStore, 'searchAttr':'name' };
        this.comboBox = new dijit.form.ComboBox(params, select2);

		// parameter type select
        this.ptype = this.newParamSelect();
        this.ptype.title=this.labels.SQL_BUILDER_FILTER_TYPE_TOOLTIP;
        this.div.appendChild(this.ptype);

        dojo.connect(this.ptype, 'onchange', this, 'changeParam');

		// parameter value input
        this.pvalue = document.createElement('input');
        this.pvalue.setAttribute('type', 'text');
        this.pvalue.style.width="9em";
        this.pvalue.title=this.labels.SQL_BUILDER_FILTER_PARAM_VALUE_TOOLTIP;
        this.pvalue.disabled=true;

		// adjust for look & feel
		var tmp=document.createElement('span');
        tmp.style.marginRight="0.1em";
        tmp.appendChild(this.pvalue);
        this.div.appendChild(tmp);

		// condition input
        this.condition = this.newConditionSelect();
        this.condition.title=this.labels.SQL_BUILDER_FILTER_CONDITION_TOOLTIP;
        this.div.appendChild(this.condition);

        this.remove = document.createElement('input');
        this.remove.setAttribute('type', 'image');
        this.remove.setAttribute('src', hub.urls.getImage('tabCloseHover.gif'));
        this.remove.setAttribute('width', '12');
        this.remove.setAttribute('height', '12');
        this.remove.style.marginLeft="0.3em";
        this.remove.style.verticalAlign="middle";
        this.remove.title=this.labels.SQL_BUILDER_FILTER_REMOVE_TOOLTIP;

        this.div.appendChild(this.remove);
        this.div.appendChild(document.createElement('br'));

        this.forms = [this.columnSelect, this.operator, this.comboBox, this.ptype, this.pvalue, this.condition];
        this.columns = [];
    },

    /**
     * Returns html element for the 'param' dropdown
     *
     * @return: html element for 'param' dropdown
     */
    newParamSelect: function() {
    	//var msgs = dojo.i18n.getLocalization('hub', 'panels');
        this.column       = this.labels.SQL_BUILDER_COLUMN;       // "Column";
        this.param_int    = this.labels.SQL_BUILDER_PARAM_INT;    // "Parameter (int)";
        this.const_int    = this.labels.SQL_BUILDER_CONST_INT;    // "Constant (int)";
        this.param_string = this.labels.SQL_BUILDER_PARAM_STRING; // "Parameter (string)";
        this.const_string = this.labels.SQL_BUILDER_CONST_STRING; //" Constant (string)";

        var values = [this.column,
                    this.param_string,
                    this.param_int,
                    this.const_string,
                    this.const_int];

        var select = this.newSelectElement(values, values);
        select.style.width = "10em";
        return select;
    },

    /**
     * Returns html element for the 'condition' dropdown
     * @return: html element for 'condition' dropdown
     */
    newConditionSelect: function() {
        var values = ['DONE', 'AND', 'OR', 'AND NOT'];
        var text = ['', 'AND', 'OR', 'AND NOT'];
        var condition = this.newSelectElement(values, text);
        condition.style.width = "7em";
        return condition;
    },

    /**
     * Returns html element for the 'operator' dropdown
     * @return: html element for 'operator' dropdown
     */
    newOperatorSelect: function() {
        var values = ['=', '!=', '>', '<', '>=', '<=', 'LIKE'];
        var operator = this.newSelectElement(values, values);
        operator.style.width = "4.5em";
        return operator;
    },

    /**
     * Creates an html element using the parameters as the
     * values and the displayed value for each <option> tag
     *
     * @param values: array of values for each <option> tag
     * @param text: array of values for innerHTML of each <option> tag
     * @return: html element for new dropdown
     */
    newSelectElement: function(values, text) {
        var select = document.createElement('select');

        // set style to align with combobox
        select.style.verticalAlign="middle";

        for(var i=0; i<values.length; i++) {
            var option = document.createElement('option');
            option.value = values[i];
            option.innerHTML = hub.htmlspecialchars(text[i]);
            select.appendChild(option);
        }
        return select;
    },

    /**
     * Updates the filter input fields attributes
     */
    changeColumnSelect: function() {
    	if(this.columnSelect.getValue()=='') {
    		this.operator.disabled=true;
    		this.comboBox.setDisabled(true);
    		this.ptype.disabled=true;
    		this.pvalue.disabled=true;
    		this.condition.disabled=true;
    	}
    	else {
    		this.operator.disabled=null;
    		this.comboBox.setDisabled(false);
    		this.ptype.disabled=null;
    		this.changeParam();
    		this.condition.disabled=null;
    	}
    },


    /**
     * Updates the filter input fields attributes
     */
    changeParam: function() {
    	var param=this.ptype.value;
	    this.comboBox.setDisabled(false);

    	if(param==this.column) {
	   		this.pvalue.disabled=true;
    	}
    	else {
    		this.pvalue.disabled=null;
    		if(param==this.const_int ||
    		   param==this.const_string ) {
    		   this.comboBox.setDisabled(true);
    		}
    	}
    },

    /**
     * Updates the columns in the 'columns' dropdown and the comboBox
     * @param columns: array of column names
     */
    updateColumns: function(columns) {
        this.columns = columns;
        this.columnSelect.innerHTML = '';
        this.comboBox.innerHTML = '';

        var data = {'identifier':'value', 'label':'label', 'items':[]};
        this.columnSelect.store = new dojo.data.ItemFileWriteStore({'data':data});

        var data = {'identifier':'name', 'label':'label', 'items':[]};
        this.comboBox.store = new dojo.data.ItemFileWriteStore({'data':data});

		// add default empty option
		if(columns.length>0) {
           	this.columnSelect.store.newItem({'value':'', 'label':''});
           	this.comboBox.store.newItem({'name':'', 'label':''});
		}

        for(var i=0; i<columns.length; i++) {
            if(columns[i].lastIndexOf('.*')==-1) {
           		this.columnSelect.store.newItem({'value':columns[i], 'label':columns[i]});
            	this.comboBox.store.newItem({'name':columns[i], 'label':columns[i]});
            }
        }

        this.changeColumnSelect();
    },

    /**
     * Returns the column/parameter name of the filter
     * otherwise null
     */
    getParamName: function() {
    	var name=this.comboBox.getValue();

  		if(this.ptype.value==this.const_int ||
   		   this.ptype.value==this.const_string ) {
   		   return null;
   		}
   		else if(this.ptype.value==this.column) {
   			return name;
   		}
   		return (':'+this.comboBox.getValue());
    },

    /**
     * Returns the parameter value of the filter
     * @return: parameter value, null for 'column' type
     */
    getParamValue: function() {
    	if(this.ptype.value == this.column) {
    		return null;
    	}
   		return this.pvalue.value;
    },

    /**
     * Returns the SQL string this filter represents
     * @return: portion of a SQL string
     */
    getSQL: function() {

		// check for first filter column
    	if(this.columnSelect.getValue()=='') return '';

        var value = this.comboBox.getValue();
        var defaultValue = this.pvalue.value;

		// check for second filter column


        if (this.ptype.value == this.param_string) {
            value = "':" + value + "'";
        }
        else if (this.ptype.value == this.param_int) {
            value = ":" + value;
        }
        else if (this.ptype.value == this.const_string) {
            value = "'" + defaultValue + "'";
        }
        else if (this.ptype.value == this.const_int) {
            value = defaultValue;
        }

        var sql = this.columnSelect.getValue() + " "
            + this.operator.value + " " + value;
        if(this.condition.value != "DONE")
            sql += " " + this.condition.value;

        return sql;
    },

    /**
     * Returns an object with the data needed to restore the UI for this filter
     * @return: object with data needed to restore this filter
     */
    getJSON: function() {

        var column=this.columnSelect.getValue();
        if(column==null || column=='') return '';

        var filter = {};
        filter.column   = column;
        filter.operator  = this.operator.value;
        filter.comboBox  = this.comboBox.getValue();
        filter.ptype     = this.ptype.value;
        filter.pvalue    = this.pvalue.value;
        filter.condition = this.condition.value;

        return filter;
    },

    getHtmlElement: function() {
        return this.div;
    },

    getRemove: function() {
        return this.remove;
    },

    getDone: function() {
        return this.condition;
    }
});