/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.data.atommodel");
dojo.require("dojox.data.XmlStore");
dojo.require('dijit._Widget');
dojo.declare('hub.data.atommodel.TableModel', hub.table.model.ItemizedModel, {

	pageSize: 20,
	totalRecords: null,
	initialSorting: "title",
	supportedSortings: ["title", "updated", "rating", "author", "id"],
	availablePages: [],
	currentPage: null,
	collection: "all",
	term: "*",
	searchType: "",
	
	constructor: function(args, node) {
		availablePages = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = "title";
		sorting.asc = true;
		this.setSortingToApply(sorting);
	},
	
	_getInitialUrl: function() {
		var url = hub.urls.getCatalog(this.collection, this.term, this.searchType, null, null);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        }
        var map = dojo.queryToObject(query);
        map["nologin"] = "true";
        if (this.pageSize != null) {
        	map["count"] = "" + this.pageSize;
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        return url;
	},

	getSupportedSortings: function() {
		var sortings = new Array();
		var i;
		for(i = 0; i < this.supportedSortings.length; i++) {
			var s = new hub.table.model.Sorting();
			s.by = this.supportedSortings[i];
			s.asc = true;
			sortings.push(s);
			s = new hub.table.model.Sorting();
			s.by = this.supportedSortings[i];
			s.asc = false;
			sortings.push(s);
		}
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return this.currentPage;
	},
	
	getCurrentPageSize: function() {
		if (this.getCurrentPage() == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},	

	getAvailablePages: function() {
		return this.availablePages;
	},
	
	_mixinSorting: function(url, sorting) {
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        }
        var map = dojo.queryToObject(query);
        if (sorting != null) {
        	map["sortBy"] = sorting.by;
        }
        if (sorting != null) {
        	map["sortOrder"] = sorting.asc ? "asc" : "desc";
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        console.debug("Complete URL with sorting = " + url);
        return url;
	},
	
	fetch: function() {
		var url = null;
		var sorting = this.getSortingToApply();
		var pageToLoad = this.getPageToLoad();
		if (pageToLoad == null) {
			url = this._getInitialUrl();
		}
		else {
			url = pageToLoad.url;
		}
		url = this._mixinSorting(url, sorting);
        var source = new dojox.data.XmlStore({'url': url});
        var __this = this;
		this.onBeforeFetched();
		var lock = this.onPageBegin();
        var handle = {
        	'startIndex': 0,
        	'itemsPerPage': 0,
        	'url_used': url,
        	'model': __this,
        	'source': source,
        	'sorting': sorting,
        	'rows': new Array(),
        	'prev_url': null,
        	'next_url': null,
        	'lock': lock,
        	
			__onBegin: function() {
			},
			
			__onComplete: function() {
				var i;
				for(i = 0; i < this.rows.length; i++) {
					this.model.onItem(this.lock, this.rows[i]);
				}
				if (this.model.onPageEnd(this.lock, false)) {
					this.model.currentSorting = this.sorting;
					var pages = new Array();
					var currentPosition = null;
					if (this.prev_link == null) {
						currentPosition = 0;
					}
					else if (this.next_link == null) {
						currentPosition = -1;
					}
					
					var page = new hub.table.model.Page();
					page.url = this.url_used;
					if (this.prev_link == null) {
						page.forwardAbsolute = 0;
					}
					if (this.next_link == null) {
						page.backwardAbsolute = 0;
					}
					page.relative = 0;
					page.fromRow = this.startIndex;
					page.toRow = this.startIndex + this.rows.length;
					page.size = this.rows.length;
					if (this.prev_url != null) {
						var prev_page = new hub.table.model.Page();
						prev_page.url = this.prev_url;
						if (page.backwardAbsolute != null) {
							prev_page.backwardAbsolute = page.backwardAbsolute + 1; 
						}
						if (page.forwardAbsolute != null) {
							prev_page.forwardAbsolute = page.forwardAbsolute - 1; 
						}
						prev_page.relative = -1;
						pages.push(prev_page);
					}
					pages.push(page);
					if (this.next_url != null) {
						var next_page = new hub.table.model.Page();
						next_page.url = this.next_url;
						if (page.backwardAbsolute != null) {
							next_page.backwardAbsolute = page.backwardAbsolute - 1; 
						}
						if (page.forwardAbsolute != null) {
							next_page.forwardAbsolute = page.forwardAbsolute + 1; 
						}
						next_page.relative = +1;
						pages.push(next_page);
					}
					this.model.availablePages = pages;
					this.model.currentPage = page;
					this.model.totalRecords = this.totalResults;
					this.model.pageToLoad = page;
					this.model.onFetched();
				}
			},
		
			__onItem: function(item, request) {
		        var dict = {};
		        var value;
		        var tag = this.source.getValue(item, 'tagName')
		        if(tag == 'os:startIndex') {
		            this.startIndex = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'os:itemsPerPage') {
		            this.itemsPerPage = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'os:totalResults') {
		            this.totalResults = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'link') {
		            var rel = this.source.getValue(item, '@rel');
		            var href = this.source.getValue(item, '@href');
		            if(rel == 'next') {
		                this.next_url = href;
		            } else if (rel == 'previous') {
		                this.prev_url = href;
		            }
		        } else if(tag == 'entry') {
		            var entryCategoryElement = this.source.getValue(item, "category");
		            var category = this.source.getValue(entryCategoryElement, "@term");
		            dict["entryCategory"] = category;
		            var content = this.source.getValue(item, "content");
		            
		            var cat = this.source.getValue(content, "childNodes");
		            
		            var atts = this.source.getValues(cat, "childNodes");
		            
		            for(var i=0; i < atts.length; i++) {
		                var node = atts[i];
		                if(this.source.isItem(node)) {
		                    if(this.source.hasAttribute(node, "catalog:tag")) {
		                        value = this.source.getValues(node, "catalog:tag");
		                        value = this._parseArray(value);
		                    } else if (this.source.hasAttribute(node, "catalog:category")) {
		                        value = this.source.getValues(node, "catalog:category");
		                        value = this._parseArray(value);
		                    }
		                    else {
		                        value = this.source.getValue(node, "text()");
		                    }
		                }
		                
		                var key = this.source.getValue(atts[i], 'tagName');
		                if (key.lastIndexOf(":") > 0)
		                	key = key.substring(key.lastIndexOf(":") + 1);
		                	
						dict[key] = value;                
		            }
		            
		            var idfield = this.source.getValue(this.source.getValue(item, "id"), "text()");
		            var unprefixedId = idfield;
		            if (idfield.indexOf("urn:uuid:") == 0) {
		            	unprefixedId = idfield.substring("urn:uuid:".length);
		            }
		            if (idfield.indexOf("urn:id:") == 0) {
		            	unprefixedId = idfield.substring("urn:id:".length);
					}
		            var pureId = unprefixedId;
		            if (unprefixedId.lastIndexOf('-') >=0) {
		            	pureId = unprefixedId.substring(unprefixedId.lastIndexOf('-') + 1);
		            }
		            dict['objectid'] = pureId;
		            dict['id'] = pureId;
		            this.rows.push(dict);
		        }
			},
			
			__onError: function(error) {
				if (this.lock == null) {
					this.model.onBeforeFetched();
					this.lock = this.model.onPageBegin();
				}
				if (this.model.onPageEnd(this.lock, true)) { // there is a error
					this.model.onFetchError(error);
				}
			},
			
		    /**
		     * Pulls an array of strings out of dojo data XML values.
		     *
		     * @param values Dojo data items having the values we want
		     */
		    _parseArray: function(values) {
		        var arr = new Array();
		        for(i in values) {
		            arr.push(this.source.getValue(values[i], "text()"));
		        }
		        return arr;
		    }			
        };
        source.fetch(
            {'onBegin' : dojo.hitch(handle, handle.__onBegin),
             'onItem' : dojo.hitch(handle, handle.__onItem),
             'onComplete' :dojo.hitch(handle, handle.__onComplete),
             'onError' : dojo.hitch(handle, handle.__onError)});
	},
	
	onFetched: function() {
	},

	onBeforeFetched: function() {
	},
	
	onFetchError: function(error) {
	}
});
