/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.data.jsonmodel");
dojo.require("dojox.data.XmlStore");
dojo.require('dijit._Widget');

dojo.declare("hub.table.jsonmodel.DataSelection", hub.table.model.DataSelection, {
	isDataSelected: function(row, area) {
		if ("isChecked" in row) {
			return (false != row.selected);
		}
		return false;
	}
});


dojo.declare('hub.data.jsonmodel.TableModel', hub.table.model.ItemizedModel, {

	pageSize: 20,
	totalRecords: null,
	initialSorting: "title",
	supportedSortings: ["title", "id", "rating"],
	availablePages: [],
	url: '',
	url_used: '',
	url_prev: '',
	url_next: '',
	currentPage: null,
	
	constructor: function(args, node) {
		availablePages = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = "title";
		sorting.asc = true;
		this.setSortingToApply(sorting);

		var page = new hub.table.model.Page();
		this.url = args.url;
    if (args.pageSize) {
      this.pageSize = args.pageSize;
    }
		page.url = this._getInitialUrl();
		page.relative = 0;
		page.fromRow = 0;
		page.size = this.pageSize;
		this.setPageToLoad(page);
		
	},
	
	_getInitialUrl: function() {
///->	
		var url = this.url;//hub.urls.getCatalog(this.collection, this.term, this.searchType, null, null);
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map["nologin"] = "true";
        
        if (this.pageSize != null) {
        	map["count"] = "" + this.pageSize;
        }
       	map["from"] = 0;

        if (this.term) {
          map["term"] = this.term;
        }

        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},

	getSupportedSortings: function() {
		var sortings = new Array();
		var i;
		for(i = 0; i < this.supportedSortings.lenght; i++) {
			var s = new hub.table.model.Soting();
			s.by = this.supportedSortings[i];
			s.asc = true;
			sortings.push(s);
			s = new hub.table.model.Soting();
			s.by = this.supportedSortings[i];
			s.asc = false;
			sortings.push(s);
		}
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return this.currentPage;
	},
	
	getCurrentPageSize: function() {
		if (this.getCurrentPage() == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},	

	getAvailablePages: function() {
		return this.availablePages;
	},
	
	_mixinSorting: function(url, sorting) {
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (sorting != null) {
        	map["sortBy"] = sorting.by;
        }
        if (sorting != null) {
        	map["sortOrder"] = sorting.asc ? "asc" : "desc";
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},
	
	_mixinOffsetCount: function(url, offset, count){
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (offset != null) {
        	map["from"] = offset;
        }
        if (count != null) {
        	map["count"] = count;
        }else{
        	map["count"] = this.pageSize;
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},
	
	_mixinPageNumber: function(url, pageNumber){
		if(pageNumber == null || pageNumber < 1){
			pageNumber = 1;
		}
		return this._mixinOffsetCount(url, (pageNumber-1)*this.pageSize, this.pageSize);
	},
	
	fetch: function() {
		var url = null;
		var sorting = this.getSortingToApply();
		var pageToLoad = this.getPageToLoad();
		if (pageToLoad == null) {
			url = this._getInitialUrl();
		}
		else {
			url = pageToLoad.url;
		}
		this.url = url
		url = this._mixinSorting(url, sorting);
//        console.debug("point A");
        
		this.onBeforeFetched();
		var lock = this.onPageBegin();

        var __this = this;

		var kw = {
		    url: url,
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
            model: this,
            sync: false,
            lock: lock,
            
            error: dojo.hitch(this, this.onPageError),
            
            load: function(response, args) {
				try {
                    if(!response) {
                        throw new Error('Form: Empty server response');
                    }
                    if(hub.submit.processResponseCode(response)){
///->
						var data = dojo.fromJson(response)
						var loaded_page = this.model.getPageToLoad();
            if (!loaded_page) {
              // ugly hack here
              loaded_page = new hub.table.model.Page();
              loaded_page.url = this.url;
            }
						var i;
						//	retrieving rows from result
						for(i = 0; i < data.rows.length; i++) {
//							console.debug("1111 = " + this.lock);
							this.model.onItem(this.lock, data.rows[i]);
						}
						if (this.model.onPageEnd(this.lock)) {
//							console.debug("2222 = " + this.lock);				
							this.sorting = this.model.getSortingToApply();

							// generating navigation pages
							var pages = new Array();
							// generating current (refresh) page
							var page = new hub.table.model.Page();
///->
							page.url = loaded_page.url;
							if (loaded_page.fromRow == 0) {
								page.forwardAbsolute = 0;
							}
							if (loaded_page.fromRow + this.model.pageSize >= data.total) {
								page.backwardAbsolute = 0;
							}
							page.relative = 0;
///->
							page.fromRow = loaded_page.fromRow;
							page.toRow = loaded_page.fromRow + data.rows.length;
							page.size = data.rows.length;
//							console.dir({loaded_page: loaded_page});
							if (loaded_page.fromRow > 0) {
							// generating previous page
								var prev_page = new hub.table.model.Page();
								prev_page.url = this.model._mixinOffsetCount(loaded_page.url, loaded_page.fromRow-this.model.pageSize);
								if (page.backwardAbsolute != null) {
									prev_page.backwardAbsolute = page.backwardAbsolute + 1; 
								}
								if (page.forwardAbsolute != null) {
									prev_page.forwardAbsolute = page.forwardAbsolute - 1; 
								}
								prev_page.relative = -1;
								prev_page.fromRow = loaded_page.fromRow - this.model.pageSize;
								pages.push(prev_page);
							}
							pages.push(page);
							if (loaded_page.fromRow + this.model.pageSize < data.total) {
							// generating next page
								var next_page = new hub.table.model.Page();
								next_page.url = this.model._mixinOffsetCount(loaded_page.url, loaded_page.fromRow+this.model.pageSize);
								if (page.backwardAbsolute != null) {
									next_page.backwardAbsolute = page.backwardAbsolute - 1; 
								}
								if (page.forwardAbsolute != null) {
									next_page.forwardAbsolute = page.forwardAbsolute + 1; 
								}
								next_page.relative = +1;
								next_page.fromRow = loaded_page.fromRow + this.model.pageSize;
								pages.push(next_page);
							}
							this.model.availablePages = pages;
							this.model.currentPage = page;
							this.model.totalRecords = data.total;
							this.model.pageToLoad = page;
							this.model.onFetched();
//							console.debug("onFetched called!!!");
						}
    
				    }
				} catch(e) {
                    var msgs = dojo.i18n.getLocalization('hub', 'messages');
                    var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
//                    hub.notify(err, this.getStatus());
				}
                return response;
            }
		};
	        // do the post
		dojo.xhrPost(kw);

//        console.debug("point C");		
	},
	
	onFetched: function() {
//		console.debug("onFetched");
	},

	onBeforeFetched: function() {
//		console.debug("onBeforeFetched");
	},
	
	onPageError: function() {
		// called never
	}
});
