/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.install");
dojo.provide("hub.install");
dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("hub", "messages");
dojo.require("dijit._Templated"); 
dojo.require("dijit._Widget"); 
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.form.Button");

/**
 * Initializes the Mashup Hub installer. Sets up the global configuration and
 * URL manager.
 */
dojo.declare("hub.install.Main",        
	[dijit._Widget, dijit._Templated ],{
	templatePath: dojo.moduleUrl("hub","widgets/templates/Wizard.html"),
	// Necessary to keep Dijit from using templateString
	templateString: "",
	// src: String
	// src url for header
	src: "",
	widgetsInTemplate: true,

    initStep: 'intro',
    
    localAuth: '',
    db2Driver: 'DB2',
    db2Port: 50000,
    mysqlDriver: 'MYSQL',
    mysqlPort: 3306,
    derbyDriver: 'DERBY',
    derbyPort: 1527,
    embeddedDerbyDriver: 'DERBY_EMBEDDED',
    idsDriver: 'IDS',
    idsPort: 9089,
   
    isInitialized: false,
    
    postMixInProperties: function() {
        // create base URL first
        var split = dojo.baseUrl.split('/');
        // based on dojo-1.0.1 folder location only
        split = split.slice(0, -4);

        // global configuration
        hub.config = {base_url: split.join('/'),
                      db2_port: this.db2Port,
                      db2_driver: this.db2Driver,
                      mysql_port: this.mysqlPort,
                      mysql_driver: this.mysqlDriver,
                      derby_port: this.derbyPort,
                      derby_driver: this.derbyDriver,
                      embedded_derby_driver: this.embeddedDerbyDriver,
                      ids_port: this.idsPort,
                      ids_driver: this.idsDriver,
                      max_label_length: 15};

        // create the URL manager
        dojo.require("hub.managers.Urls");
        hub.urls = new hub.managers.Urls(hub.config.base_url);

    },

    startup: function() {
        // initialize DB driver selection
        this._initialize();
        
        // show initial step
		var wiz = this.tabContainer;
        wiz.selectChild(this.initStep);
        this._updateControls(this.initStep);
    },

    /**
     * Flip the stack to the next page and highlight the associated step name.
     */
    _onNext: function() {
        // push stack container to next item
		// var wiz = this.tabContainer;
		var id = this.tabContainer.selectedChildWidget.id;
		switch(id){
			case 'db':
				this._checkDb();
				break;
			case 'site':
				this._checkSite();
	            this._onPreviewInstall();
				break;
			case 'install':
				this._doInstall();
				break;
			case 'report':
				window.location='/mashuphub/secured';
				break;
			default:	
		        this.tabContainer.forward();
		}
		id = this.tabContainer.selectedChildWidget.id;
        this._updateControls(id);
    },

    /**
     * Flip the stack to the prior  page and highlight the associated step name.
     */
    _onPrevious: function() {
        // push stack container to next item
		var wiz = this.tabContainer;
        wiz.back();
 		var id = wiz.selectedChildWidget.id;
        // get the list of steps
        this._updateControls(id);
    },

	_performStepAction: function(step){
	},

	/**
	 * Update Previous|Next Buttons
	 */
	_updateControls: function(step){
		var buttonDecline = this.buttonDecline;
		var buttonPrevious = this.buttonPrevious;
		var buttonNext = this.buttonNext;
		
		buttonDecline.style.visibility = "hidden";
		buttonPrevious.disabled = "";
		buttonNext.innerHTML = "Next";

		switch(step){
			case 'intro':
				buttonPrevious.disabled = "true";
				break;
			case 'license':
				buttonDecline.style.visibility = "visible";
				buttonNext.innerHTML = "I Accept";
				break;
			case 'db':
				break;
			case 'site':
				break;
			case 'install':
				buttonNext.innerHTML = "Install";			
				break;
			case 'report':
				buttonNext.innerHTML = "Finish";
				buttonPrevious.disabled = "true";
				break;
		}
        var steps = this.install_steps;
        dojo.query("li", steps).removeClass("selected");
        // highlight newly activated step
        dojo.query('li[id="' + step + '_step"]', steps).addClass("selected");
		
	},

    /**
     * Generate a report of all selected options before installing.
     */
    _onPreviewInstall: function() {
        var list = dojo.byId("install_review");
        var labels = dojo.query('label');
        // reset previous text
        list.innerHTML = "";

        for(var i=0; i < labels.length; i++) {
            var label = labels[i];
            if(label.className == 'section') {
                // create a heading
                var head = document.createElement("H3");
                head.innerHTML = label.innerHTML;
                // start a new list
                var ul = document.createElement("UL");
                list.appendChild(head);
                list.appendChild(ul);
            } 
            else {
                var li = document.createElement("LI");
                li.innerHTML = label.innerHTML;
                var controls = dojo.query("[name="+label.htmlFor+"]");
                // assume first match
                var c = controls[0];
                if(c.type != "password" && !c.disabled) {
                    li.innerHTML += " "+controls[0].value;
                    ul.appendChild(li);
                }
            }
        }
    },

    /**
     * Generate a report of all pre-install errors before installing.
     */
    _onPreInstallErrors: function(errors) {
		this.preInstallOk.style.display = "none";
		this.preInstallErrors.style.display = "block";
		dojo.forEach(errors, function(error){
            var li = document.createElement("li");
            li.innerHTML = error;
            dojo.byId("preInstallErrorsUL").appendChild(li);
		});
		this.buttonNext.disabled = "true";
		
    },

	_reqStatus: function(id, fields) {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    if(fields.length == 1) {
	        // just one missing field, state it
	        var ml = dojo.string.substitute(msgs.ERROR_INSTALL_MISSING_VALUE, 
	            [fields[0]]);
	    } else if(fields.length > 1) {
	        // multiple missing fields, format them
	        var ml = '';
	        for(var i=0; i<fields.length; i++) {
	            if (i < fields.length - 1) {
	                ml += fields[i] + ', ';
	            } else {
	                ml += fields[i];
	            }
	        }
	        ml = dojo.string.substitute(msgs.ERROR_INSTALL_MISSING_VALUES, [ml]);
	    }
	    this._helpStatus(id, ml);
	},
	
	/**
	 * Shows a help message after a failure to verify entered data.
	 *
	 * @param id: Where to show the message
	 * @param text: Message
	 */
	_helpStatus: function(id, text) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    var url = hub.urls.getImage('help-browser.png');
	    e.innerHTML = dojo.string.substitute('<img src="${0}" alt="Help icon" /> ${1}', [url, text]);
	},
	
	/**
	 * Clears a help message or busy status.
	 *
	 * @param id: Where to clear the message
	 */
	_clearStatus: function(id) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    e.innerHTML = '';
	},
	
	/**
	 * Shows a busy message while verifying data with a server.
	 *
	 * @param id: Where to show the message
	 * @param text: Message
	 */
	_busyStatus: function(id, text) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    var url = hub.urls.getImage('spinner.gif');
	    e.innerHTML = dojo.string.substitute('<img src="${0}" alt="Busy icon" /> ${1}', [url, text]);
	},
	
	/**
	 * Queries the server to verify entered data such as database server, 
	 * authentication server, etc. Publishes a request to continue to the next page
	 * on success.
	 *
	 * @param id: Where to post busy/help messages
	 * @param action: Action indicating what method should verify on the server
	 * @param form: Data to verify, possibly including more than what will be verified
	 *   (i.e., the entire form, not just the current page)
	 */
	_queryServer: function(id, action, form) {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    // go to server and check the connection
	    var kw = {
		url: hub.urls.postInstall(action),
		sync: true,
		form: form,
	        handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {           
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } 
	            else if(response.error) {
	                this._helpStatus(id, response.message);
	            } 
	            else {
	                this._clearStatus(id);
//	                this._next();
					this.tabContainer.forward();
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
//		dojo._blockAsync = false;
	},
	
	/**
	 * Shows an alert indicating the license agreement must be accepted before
	 * continuing.
	 */
	_declineLicense: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    alert(msgs.ERROR_INSTALL_LICENSE);
	},
	
	_setVars: function(vars){
		this.app_name.value = vars.app_name;
		this.app_path.value = vars.app_path;
		this.app_timezone.value = vars.app_timezone;
		this.db_driver.value = vars.db_driver;
		this.db_host.value = vars.db_host;
		this.db_name.value = vars.db_name;
		this.db_password.value = vars.db_password;
		this.db_port.value = vars.db_port;
		this.db_schema.value = vars.db_schema;	
		this.db_username.value = vars.db_username;
	},
	
	/**
	 * Updates default database field values based on the selected driver.
	 *
	 * @param widget: Driver selection box 
	 */
	_selectDbDriver: function() {
	    var driver = dojo.byId('db_driver');
	    var schema = dojo.byId('db_schema');
	    var port = dojo.byId('db_port');
	    
	    switch(driver.value){
	    	case hub.config.mysql_driver:
		        schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.mysql_port;
		        port.disabled = true;
		        port.className = 'disabled';
	    		break;
	    	case hub.config.derby_driver:
	    	case hub.config.embedded_derby_driver:
		        schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.derby_port;
		        port.disabled = true;
		        port.className = 'disabled';
	    		break;
	    	case hub.config.ids_driver:
	    		schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.ids_port;
		        port.disabled = false;
		        port.className = '';
	    		break;
	    	case hub.config.db2_driver:
	    	default:
		        schema.disabled = false;
		        schema.className = '';
		        port.value = hub.config.db2_port;
		        port.disabled = false;
		        port.className = '';
	    }
/*	    if(widget.value != hub.config.db2_driver) {
	        schema.disabled = true;
	        schema.className = 'disabled';
	        port.value = hub.config.mysql_port;
	        port.disabled = true;
	        port.className = 'disabled';
	    } else {
	        schema.disabled = false;
	        schema.className = '';
	        port.value = hub.config.db2_port;
	        port.disabled = false;
	        port.className = '';
	    }*/
	},
	
	
	/**
	 * Loads defaults from server
	 *
	 */
	_initialize: function(){
		if(this.isInitialized) return;
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
       	var id = 'init_status';           
	    this._busyStatus(id, "Initializing...");
	    var kw = {
			url: hub.urls.postInstall("init"),
		    handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } 
	            else if(response.error) {
	                this._clearStatus(id);
					this.preInstallCheck.style.display = "none";
					this.preInstallOk.style.display = "none";
					this.preInstallErrors.style.display = "block";
					if(response.messages) {
						dojo.forEach(response.messages, function(error){
				            var li = document.createElement("li");
				            li.innerHTML = error;
				            dojo.byId("preInstallErrorsUL").appendChild(li);
						});
					}
					else {
		                this._helpStatus(id, response.message);
					}
					this.buttonNext.disabled = "true";
	            } 
	            else {
	                this._clearStatus(id);
					this.preInstallCheck.style.display = "none";
					this.preInstallOk.style.display = "block";
					this.preInstallErrors.style.display = "none";
					this._setVars(response);
					this.isInitialized = true;
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrGet(kw);
	},

	/**
	 * Checks if all required database fields exist. Sends a request to the server
	 * to attempt a database connection, login, and reset.
	 *
	 * @todo: we must warn the user about clearing the DB before doing it
	 *   on the server; right now, the clear is silent
	 */
	_checkDb: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('db_status', msgs.STATUS_VERIFYING);
	
	    // check the values locally
	    var missing = new Array();
	    var form = dojo.byId('wizard_form_db');
	    if(!form.db_host.value)
	        missing.push(dojo.byId('db_host_label').innerHTML);
	    if(form.db_driver.value == hub.config.db2_driver && !form.db_port.value)
	        missing.push(dojo.byId('db_port_label').innerHTML);
	    if(!form.db_name.value)
	        missing.push(dojo.byId('db_name_label').innerHTML);
	    if(form.db_driver.value == hub.config.db2_driver && !form.db_schema.value)
	        missing.push(dojo.byId('db_schema_label').innerHTML);
	    if(!form.db_username.value)
	        missing.push(dojo.byId('db_username_label').innerHTML);
	    if(!form.db_password.value)
	        missing.push(dojo.byId('db_password_label').innerHTML);
	    if(missing.length) {
	        this._reqStatus('db_status', missing);
	        return;
	    }
	
	    // check server
	    this._queryServer('db_status', 'db', form);
	    return false;
	},
	
	/**
	 * Checks if all site config fields exist. Sends a request to the server to 
	 * test if the server code is where the path says it is.
	 */
	_checkSite: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('site_status', msgs.STATUS_VERIFYING);
	
	    var missing = new Array();
	    var form = dojo.byId('wizard_form_site');
	    if(!form.app_name.value)
	        missing.push(dojo.byId('app_name_label').innerHTML);
	    if(!form.app_path.value)
	        missing.push(dojo.byId('app_path').innerHTML);
	        
	    this._queryServer('site_status', 'site', form);
	    return false;
	},
	
	/**
	 * Executes the install.
	 */
	_doInstall: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('install_status', msgs.STATUS_INSTALLING);
       	var id = 'install_status';           
	    var kw = {
			url: hub.urls.postInstall("install"),
			sync: true,
		    handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        content: {
				app_name: dojo.byId('app_name').value,
				app_path: dojo.byId('app_path').value,
				app_timezone: dojo.byId('app_timezone').value,
				db_driver: dojo.byId('db_driver').value,
				db_host: dojo.byId('db_host').value,
				db_name: dojo.byId('db_name').value,
				db_password: dojo.byId('db_password').value,
				db_port: dojo.byId('db_port').value,
				db_schema: dojo.byId('db_schema').value,	
				db_username: dojo.byId('db_username').value
	        },
	
	        error: dojo.hitch(this, function(response, args) {
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } else if(response.error) {
	                this._helpStatus(id, response.message);
	            } else {
	                this._clearStatus(id);
//	                this._next();
					this.tabContainer.forward();
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
	}
    
});

/**
 * Override stack behavior so form element spanning multiple panels is ignored
 * and any panels it contains are treated as part of the wizard flow.
 *
dojo.declare('hub.install.Wizard', dijit.layout.StackContainer, {
    getChildren: function() {
        // get all descendant widgets
        var all = this.getDescendants();
        // get all children of the container and the form
        var children = this.inherited(arguments);
        var form = dojo.query("form > [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
        children = children.concat(form);
        // now filter out those not in the children; preserves order
        return dojo.filter(all, function(e) {
            var i = dojo.indexOf(children, e);
            return i != -1;
        });
    }
});

*/
