/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.main");
dojo.require('dijit._Widget');
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("hub", "messages");
dojo.requireLocalization("hub", "labels");

/**
 * Initializes the Mashup Hub UI and client-side configuration variables. 
 * It's a class so we can use the hub package elsewhere without all this
 *  initialization code running on page load.
 */
dojo.declare("hub.Main", dijit._Widget, {
    // holds the initial full name of the logged in user, or blank if anonymous
    username: '',
    userfullname: '',
    isAdmin: '',
    isBanned: '',
    isSecure: false,
    securePort: '',
    baseurl: '',
    pingSessionHandle: null,
    
    postMixInProperties: function() {
        // create base URL first
        var split = dojo.baseUrl.split('/');
        // based on dojo-1.0.1 folder location only
        split = split.slice(0, -4);
		this.baseurl = split.join('/');		
        var durl = new dojo._Url(window.location, this.baseurl);
        this.baseurl = durl.uri;
        this.isSecure = (durl.scheme == 'https');
        if (this.isSecure) {
        	console.debug("isSecure == " + this.isSecure);
        	if (durl.port) {
        		this.securePort = ":" + durl.port; 
        	} 
        }
        if(this.baseurl.charAt(this.baseurl.length-1)=="/"){
        	this.baseurl = this.baseurl.substring(0, this.baseurl.length-1);
        }
		 
        // get username  TODO !!!
		_this = this;
		
		dojo.xhrGet({
			url: this.baseurl + "/ui/currentuser",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.username = response.username;
				_this.userfullname = response.userfullname;
				_this.isAdmin = ("true" == response.isAdmin);
				_this.isBanned= ("true" == response.isBanned);
				var configured = ("true" == response.configured);
				if (!configured) {
					// come away from this layout.html to application root
					console.info("Application is not configured!");
					var p = window.location.pathname;
					var cut = p.indexOf("client/layout/layout.");
					if (cut >= 0) {
						p = p.substring(0, cut);
						console.info("Go to application root: " + p);
						if (dojo.isIE) {
							window.setTimeout('window.location.href = "' + p + '";', 50);
						}
						else {
							window.location.href = p;
						}
						return;
					}
				}
			}
		});
		
        // global configuration
        hub.config = {base_url: this.baseurl,
                      username: this.username,
                      userfullname: this.userfullname,
                      isAdmin: this.isAdmin,
                      isBanned: this.isBanned,
                      isSecure: this.isSecure,
                      securePort: this.securePort,
                      max_label_length: 15};
		
		// load paltette url
		dojo.xhrGet({
			url: this.baseurl + "/ui/paletteurl",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				hub.config.palette_url = response.palette_url;
			}
		});
                      
        hub.pingSession = function() {
        /*
			dojo.xhrGet({
				url: hub.config.base_url + "/ui/pingSession?dt=" + new Date().getTime(),
				handleAs: 'json',
				sync: false,
				load: function(response, args) {
					// we just ignore it
					console.debug("session pinged");
				}
			});
		*/	
        };

        // start listening for events that affect configuration
        dojo.subscribe('hub.login', this, '_onLogin');
        dojo.subscribe('hub.logout', this, '_onLogout');

        // initialize popup library
        dojo.require("hub.misc.popup");
        set_pl_IMAGE_PATH(hub.config.base_url+'/images/popup/');
        pl_preloadPopupLayerImages();

        // create important managers
        dojo.require("hub.managers.Urls");
        hub.urls = new hub.managers.Urls(hub.config.base_url);
    },

    startup: function() {
        // show the home page
        var labels = dojo.i18n.getLocalization('hub', 'labels');
        hub.showPage(labels.HOME_TAB, 'Home');
        
        this.pingSessionHandle = null;
        /*
        if (this.pingSessionHandle != null) {
            // startup() normally should never called twice,
            // but to have it 100% safe
        	window.clearInterval(pingSessionHandle);
        	this.pingSessionHandle = null;
        }
        this.pingSessionHandle = window.setInterval(hub.pingSession, 15 * 60 * 1000); // 15 minutes are adequate
        */
    },

    _onLogin: function(username) {
        hub.config.username = username;
    },

    _onLogout: function() {
        hub.config.username = '';
    }
});

/**
 * General exception class for hub. Contains a message.
 */
dojo.declare("hub.Exception", null, {
    message: '',
    constructor: function(msg) {
        this.message = msg ? msg : "";
    }
});


/**
 * Notifies the user of a message, either by writing it into the given element.
 * or by popping up a JS alert box.
 *
 * @param text: Text to display
 * @param element: Element in which to insert the text, or null to use a popup
 * @param cls: CSS classname to use
 */
hub.notify = function(text, element, class_name) {
    if(element) {
        if(!class_name) {
            class_name = 'TOM_editor_error';
        }
        element.className = class_name;
        element.innerHTML = text;
    } else {
        alert(text);
    }
}

/**
 * Ellipsizes a label to the given maximum number of characters, including the
 * ellipsis.
 *
 * @param label
 * @param max
 */
hub.shortenLabel = function(label) {
    // truncate name chars first, so comparison by label works
    if(label.length > hub.config.max_label_length) {
    	label = hub.srahclaicepslmth(label);
        return hub.htmlspecialchars(label.substr(0 , hub.config.max_label_length-3)) + '...';
    } else {
        return label;
    }
}

/**
 * Mimick's PHP's htmlspecialchars for &, <, and >. Use to encode text before
 * output from JS to avoid security issues.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.htmlspecialchars = function(text) {
    text = text.replace(/&/g,"&amp;");
    text = text.replace(/</g,"&lt;");
    text = text.replace(/>/g,"&gt;");
    text = text.replace(/\'/g,"&#39;");
    text = text.replace(/\"/g,'&quot;');
    return text;
}

hub.srahclaicepslmth = function(text) {
    text = text.replace(/&amp;/g,"&");
    text = text.replace(/&lt;/g,"<");
    text = text.replace(/&gt;/g,">");
    text = text.replace(/&#039;/g,"'");
    text = text.replace(/&quot;/g,'"');
    return text;
}

/**
 * Replace ' -> \' and " -> \". Use to encode text before
 * output as JS strings.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.escapeQuotes = function(text) {
    text = text.replace(/\'/g,"\\\'");
    text = text.replace(/\"/g,'\\\"');
    return text;
}
hub.escapeQuotes3 = function(text) {
    text = text.replace(/\'/g,"&#92;&#39;");
    text = text.replace(/\"/g,'&#92;&#34;');
    return text;
}
hub.escapeQuotes2 = function(text) {
    text = text.replace(/&#39;/g,"&#92;&#39;");
    text = text.replace(/&quot;/g,'&#92;&#34;');
    return text;
}

/**
 * Evaluates JavaScript and Dojo content in the given DOM node.
 * 
 * @param node The DOM node to evaluate
 * @param js_eval True to eval all script tags
 * @param dojo_prase True to parse with Dojo
 */
hub.evaluate = function(node, js_eval, dojo_parse) {
    var html = node.innerHTML;
    
    if(js_eval) {
        // execute any scripts
        var expr = new RegExp('(?:<!--(?:\n|\r|\s|.)*?-->)|(?:<SCRIPT.*?>)((\n|\r|\s|.)*?)(?:<\/SCRIPT>)', 'ig');
        for (var results = expr.exec(html); results != null; results = expr.exec(html)) {
            if (!results[1]) {
                continue;
            }
            // IE doesn't allow script elements to have childnodes,
            // so another way is to use the non-standard .text for
            // IE:  script.text = results[1]
            if (dojo.isIE > 0) {
                window.execScript(results[1]);
            } else {
                eval(results[1]);
            }
        }
    }
    
    if(dojo_parse) {
        // tell dojo to parse
        dojo.parser.parse(node); 
    }
}

/**
 * Renders a standard refreshing message either inline or as a block element.
 * 
 * @param node: Node to render into
 * @param block: True to render as a <p>, false as a <span>
 * @param className: CSS class name to attach to the <p> or <span>
 */
hub.busyMessage = function(node, block, class_name) {
    var display;
    // decide how to render
    if(block) {
        display = "block";
    } else {
        display = "inline";
    }
    if(!class_name) {
        // make sure we have an empty string
        class_name = '';
    }
    var msgs = dojo.i18n.getLocalization('hub', 'messages');
    var ml = "<span class='${0}' style='display: ${1}'><img src='${2}' /> ${3}</span>";
    var img = hub.urls.getImage('spinner.gif');
    var str = dojo.string.substitute(ml, [class_name, display, img, 
                                                     msgs.STATUS_LOADING]);
    if(node) {
        node.innerHTML = str;
    } else {
        return str;
    }
}

/**
 * Builds a link representation of a call to hub.showPage.
 *
 * First three params match those of hub.showPage.
 * 
 * @param shorten: Truncate the displayed text?
 * @param text: Text to display, defaults to title
 * @param popup: Text to popup, defaults to title
 */
hub.buildPageLink = function(title, template, args, shorten, text, popup) {
    if(!text) {
        // use title if no other text specified
        text = title;
    }
    if(shorten) {
        // truncate to configured length with ellipsis
        text = hub.shortenLabel(text);
    }
     if(popup == null) {
        // use the full title as the popup
        popup = title;
    } else if(shorten) {
        // use the full title plus the provided popup
        popup = title + " (" + popup + ")";
    }
    // always escape strings, even to prevent ' and " problems
//    title = hub.escapeQuotes2(title);
//    text = hub.escapeQuotes2(text);
//    popup = hub.escapeQuotes2(popup);
    result = "<a href='javascript:hub.showPage(\"";
    result += hub.escapeQuotes2(title);
    result += "\",\"";
    result += template;
    result += "\",";
    result += dojo.toJson(args);
    result += ")\' title='";
    result += hub.escapeQuotes2(popup);
    result += "'>";
    result += text;
    result += "</a>";
    return result;
//     var slots = "<a href='javascript:hub.showPage(\"${title}\",\"${template}\",${args})\' title='${popup}'>${text}</a>";
//    return dojo.string.substitute(slots, {title: title,
//                                          text: text,
//                                          popup: popup,
//                                          template: template,
//                                          args: dojo.toJson(args)});  
}

/**
 * Posts a request to show a client side templated page. The page will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param template: Name of the template to render, rooted at hub.pages
 * @param args: Object of args to pass to the listener
 */
hub.showPage = function(title, template, args) {
    dojo.publish('hub.show-template-page', [title, template, args]);
}

/**
 * Posts a request to show the result of visiting a URL. The URL will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param url: Location of the page to render
 * @param args: Object of options to pass to the listener
 */
hub.showUrl = function(title, url, args) {
    dojo.publish('hub.show-url-page', [title, url, args]);
}

/**
 * Posts a request to show the first screen of a plug-in editor UI. The plug-in
 * will render if an observer is available that can render it, else the request
 * is safely ignored.
 * 
 * @param title: Title of the page
 * @param plugin_id: Plug-in editor to show
 * @param object_id: Object to edit in the plug-in. If null, creates a new 
 * object.
 * @param args: Object of options to pass to the listener
 */
hub.showPlugin = function(title, plugin_id, object_id, args) {
    dojo.publish('hub.show-plugin-page', [title, plugin_id, object_id, args]);
}

/**
 * Posts a request to rename an existing page. The rename will happen if the 
 * given node exists in a page and an observer is available that can render it,
 * else the request is safely ignored.
 * 
 * @param node: Any node in the page
 * @param title: New title of the page
 * @param args: Object of options to pass to the listener
 */
hub.renamePage = function(node, title,  args) {
    dojo.publish('hub.rename-page', [node, title, args]);
}

/**
 * Posts a request to mark or unmark a page as busy and/or to lock it from all
 * interaction with the user. The page will indicate busy or lock if an 
 * observer is available that can handle the request, else the request is 
 * safely ignored.
 * 
 * @param node: Any node in the page
 * @param busy: True to mark busy, false to unmark
 * @param locked: True to lock, false to unlock
 */
hub.busyPage = function(node, busy, locked) {
    dojo.publish('hub.busy-page', [node, busy, locked]);
}

/**
 * Posts a request to close a page. The page will close if there an 
 * observer is available that can handle the request, else the request is 
 * safely ignored. The observer will indicate whether the close event was
 * successful or if something cancelled it.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if closed, false if aborted
 */
hub.closePage = function(node) {
    var d = new dojo.Deferred();
    if(node){
    	dojo.publish('hub.close-page', [node, d]);
    }
    // answer will come back later
    return d;
}

hub.closeAllPages = function() {
    var d = new dojo.Deferred();
    dojo.publish('hub.close-page', [null, d]);
    // answer will come back later
    return d;
}

hub.closeCurrentPage = function() {
    dojo.publish('hub.close-current-page', []);
}

hub.getCurrentTab = function(){
	var tab_container = dijit.byId('hub.tabs');
	return tab_container.selectedChildWidget;
}

/**
 * Posts a request to check if a page is active. The request will be answered
 * if an observer is available to handle it, else the request is 
 * safely ignored. The observer will indicate the result.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if active, false if not
 */
hub.isPageActive = function(node) {
    var d = new dojo.Deferred();
    dojo.publish('hub.is-page-active', [node, d]);
    // answer will come back later
    return d;
}

/**
 * Finds all child layout containers and resizes them (also resizes passed dijit).
 * @param d: dijit to find layout containers in and resize them
 */
hub.resizeLayoutContainers = function(d) {
	d.resize();
	var layoutContainers = dojo.query(".dijitLayoutContainer", d.containerNode || d.domNode).map(dijit.byNode);
	dojo.forEach(layoutContainers, function(child){hub.resizeLayoutContainers(child); });
}

hub.showHelp = function( title, url ) {
    dojo.require("hub.widgets.Dialog");
    
    if (url.indexOf("http") == 0) {
    } 
    else {
        url = hub.config.base_url + '/' + url;
    }
    
    //<%= me %>.workflow.test(); 
    //var node = dojo.doc.createElement('div');
    var node = dojo.body().appendChild(document.createElement("div"));
    var dialog = new hub.widgets.Dialog({title: title, showUnderlay: false, resizable: true, closeable: true}, node);
    
    //dialog.setContent("<img src='https://localhost:8443/mashuphub/client/plugins/com.ibm.mashuphub.plugin.feed/icons/btn32_atom1.gif' />");
    dialog.resize({w: 950, h: 450});
    dialog.show();
    
    // var test_url = 'https://localhost:8443/mashuphub/client/doc/userhelp/feedhelp/help_feeds_register.htm';
    
    dojo.style(dialog.containerNode, "overflow", "hidden");
    var iframe = document.createElement("iframe");
    iframe.style.width = "100%";
    iframe.style.height = "100%";
    iframe.style.borderWidth = "0";
    iframe.setAttribute("src", url);
    dialog.containerNode.appendChild(iframe);
}
