/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.managers.Plugin');
dojo.provide('hub.managers.InvokePlugin');
dojo.provide('hub.behaviors.Stack');
//dojo.require('hub.widgets.tabs.TabContainer');
dojo.require('hub.widgets.tabs');
dojo.requireLocalization("hub", "labels");

if (!hub.managers) {
    hub.managers = {};
}

// storage so plug-ins can easily emit code server side that references this
// client-side instance
hub.managers.plugins = {};



dojo.declare('hub.managers.Plugin', null, {
    // @todo: implement, port from tom/client/scripts/old/plugin.js
    // probably needs some updates to work with new 
    // hub.widgets.tabs.edit.WorkflowPane widget

    /**
     * Used by the server-side plug-in to build a new instance.
     * 
     * @param ajax_url URL to the server-side plug-in
     * @param editor_id Unique id of this client-side plug-in instance
     * @param unique Unique id of HTML elements in a single screen of
     *   this plug-in instance
     * @param entry_id Unique id of the object under editing
     * @param plugin_id Unique id of the plug-in
     */
    workflow: null,
    ajax_url: '',
    save_url: '',
    editor_id: null,
    entry_id: null,
    plugin_id: null,
    tab: null,
    upload_stack: null,
    filetype: null,
    msg_dialog: null,
    
    constructor: function(plugin_id, entry_id, workflow) {
        // build client ID from editor and unique ids
        this.entry_id = entry_id;
        this.plugin_id = plugin_id;
        this.editor_id = this.getEditorId();
/*        this.tab = TOM.tabs.man.getTabForObjectId(entry_id);
        if(this.tab == null) {
            // we might be in a tab that doesn't have an object id yet
            // so find our tab and give it the proper id
            this.tab = TOM.tabs.man.getMyTab(dojo.byId(entry_id));
            this.tab.setAttribute('entry_id', entry_id);
        }*/
        this.workflow = workflow;
        this.tab=this.workflow.getTab();

        // inform the workflow manager for this tab that we exist
//        var wm = this.tab.getAttribute('workflow');
//        wm.setPlugin(this);
        
        // a URL to use to show the next editor form when saving
        this.save_url = hub.config.base_url+"/ui/editor/name/save/entryid/"+this.entry_id;
		this.ajax_url = hub.config.base_url+'/plugin/ajax/pluginid/'+this.plugin_id+'/entryid/'+this.entry_id;
        // store this instance so the server side screens can access it
        // directly
        hub.managers.plugins[this.getEditorId()] = this;
    },
    
    getEditorId: function(){
    	return 'hub_plugins_'+this.plugin_id+'_'+this.entry_id;
    },

    /**
     * Returns an object that a TOM.editor.Workflow instance can use to 
     * compare whether the state of this screen has changed since the user
     * last visited it. If not implemented (i.e., raises a Error),
     * then the workflow instance uses its own logic to save state, namely
     * getting values from all named elements in the first form on the 
     * screen. Override this method only if form field checks will not work
     * for this plug-in.
     * 
     * @param screen The DOM node for the screen whose state is needed
     * @param index The index of the screen, include screens shown before
     *   the plug-in's first screen
     */
    getScreenState: function(screen, index) {
        // let the workflow use its default algorithm
        throw new Error();
    },

    /**
     * Proxy for TOM.editor.Workflow.previous for convenience of the 
     * plug-in.
     */
    previous: function() {
        //var wm = this.tab.getAttribute('workflow');
        //wm.previous();
        this.workflow.previous();
    },

    /**
     * Proxy for TOM.editor.Workflow.next for convenience of the 
     * plug-in.
     */
    next: function(func) {
        // get the workflow for this editor
//        var wm = this.tab.getAttribute('workflow');
//        wm.next(func);
		this.workflow.next(func);
    },

    /**
     * Gets the id of the object under editing.
     *
     * @return ID
     */
    getObjectId: function() {
        return this.entry_id;
    },

    /**
     * Gets the id of the plugin under editing.
     *
     * @return ID
     */
    getPluginId: function() {
        return this.plugin_id;
    },


    /**
     * Convenience method. Call to close the current tab, go to the 
     * details page, or show a message when the editor is complete. Fires
     * an 'edit-entry' event.
     *
     * @param params 
     *   'action' is 'save' to save 
     *   'text' is message text
     */
    done: function(params) {
//    	console.log('plugin.done');
        var action = params['action'];
        var text = params['text'];
        var saveCallbackObject = params['saveCallbackObject'];
        var saveCallbackFunction = params['saveCallbackFunction'];
		this.saveCallbackObject = saveCallbackObject;
		this.saveCallbackFunction = saveCallbackFunction;
        if (action == 'message' || action == 'save') {
            // go to the metadata / save page
            dojo.publish('workflow-done-' + this.tab.id, []);
            
        } else if (action == 'cancel') {
            // close tab, workflow and tab will make sure to ask for 
            // confirmation when needed
            hub.closePage(this.tab);
        }
    },
    
    /**
     * A workflow manager instance invokes this method when a tab closes
     * or an editor wizard is complete / cancelled.
     */
    dispose: function() {
        // throw away this plug-in object
        console.log('Plugin.dispose');
        for(member in this){
        	if(member != 'tab' && member != 'workflow'
        	&& this[member]!= null
        	&& dojo.isObject(this[member]) 
        	&& this[member].destroy!= null 
        	&& dojo.isFunction(this[member].destroy)){
        		console.log('destroyed');
        		this[member].destroy();
        	}
        }
        delete hub.managers.plugins[this.editor_id];
    },
    
    /**
     * Convenience method. Call to replace the entire contents of the 
     * editor area with new HTML.
     * 
     * @param html HTML to set as innerHTML
     */
    showEditor: function(html) {
//    alert('showEditor');
        // ask workflow manager for a new pane to render into
//        var wm = this.tab.getAttribute('workflow');
//        var screen = wm.appendScreen(html);
		var screen = this.workflow.appendScreenHTML(html);

        // let the plug-in do something now that the editor has loaded
        // the plug-in can change this method on each screen if it wants
        // since the script eval above runs first
/*        if(this.onLoadEditor) {
            this.onLoadEditor(screen);
        }*/
    },

    /**
     * Convenience method. Call to show a text message in a popup
     * 
     * @param msg Message content
     * @param typ Message type - 'I' for information, 'W' for warning, 'E' for error
     */
    showMessage: function(msg, typ) {
       //var e = dojo.byId(this.getEditorId()+'_status');
       //hub.notify(msg, e);
       this._showMsg(msg, typ, 500, 350);
    },
    
    // TODO:  destroy dialog
    _showMsg: function(msg, typ, width, height) {
       dojo.require("hub.widgets.Dialog");
       var labels = dojo.i18n.getLocalization('hub', 'labels');
       var title = "MashupHub ";
       var msgType = "";
       var icon = "";
       
       if (this.msg_dialog) {
          this.msg_dialog.hide();
          this.msg_dialog.destroyRecursive();
          this.msg_dialog = null;
       }
       
       if (!typ) {
          typ = 'E';
       }
       else {
          typ = typ.toUpperCase();
       }   
       
       if (typ == 'I') {
          msgType = labels.DIALOG_TITLE_INFO;
          icon = "<img border='0' src='../images/info_status.gif' />";
       }
       else if (typ == 'W') {
          msgType = labels.DIALOG_TITLE_WARNING;
          icon = "<img border='0' src='../images/warning_status.gif' />";
       }
       else if (typ == 'E') {
          msgType = labels.DIALOG_TITLE_ERROR;
          icon = "<img border='0' src='../images/error_status.gif' />";
       }
    
       title += msgType;
    
       var node = dojo.body().appendChild(document.createElement("div"));
       this.msg_dialog = new hub.widgets.Dialog({title: title, showUnderlay: false, resizable: true, closeable: true }, node);
    
       var msgContent = '<div><table width="100%"><tr>';
       msgContent += '<td valign="top" align="left" width="27">' + icon + '</td>';
       msgContent += '<td>' + msg + '</td>';
       msgContent += '</tr></table></div>';
    
       this.msg_dialog.setContent( msgContent );
       this.msg_dialog.resize({w: width, h: height});
       this.msg_dialog.show();
    },
    
    /**
     * Notifies the plugin that the entry has been renamed by the editor. 
     * This method makes sure that the surrounding UI is updated 
     * accordingly.
     * 1. Update tab title
     * 2. Update menu links
     */
    notifyRename: function( newName ){
        // @todo: remove after changing flow editor
        // no-op
    },
    
    /*
     * Used to make changes to the status string in the menu area
     * 
     * @param published - boolean true for PUBLISHED, false for UNPUBLISHED
     */
    notifyStatus: function( published ){
        // @todo: remove after changing flow editor
        // no-op
    },
    
    /**
     * The current implementation will refresh the content of the tab.
     *
     * @param pid - plugin id
     * @param oid - object id of clone
     * @param isStatic - signal to use a static tab or one that is refreshed 
     *    on changes
     */
    notifyClone: function( pid, oid, isStatic ){
        // @todo: convert or remove after changing flow editor
        //Since the clone is supposed to replace the content and is 
        // initiated by the user... there is no need to confirm
        this.confirmDispose = function(){ return true };
        var editorURL = hub.config.base_url + '/plugin/editor/entryid/'+oid+'/pluginid/'+pid;
        if( isStatic ) {
/*            TOM.tabs.man.showEditorTab(editorURL, this.tab.title, oid, 
                                       false, true);*/
            hub.showPlugin(	this.tab.title, 
            				oid, 
            				oid, 
            				{safe: true});
           
        } else {
/*            TOM.tabs.man.showTab(editorURL, this.tab.title, oid, 
                                 false, true);*/
			hub.showUrl(this.tab.title, editorURL, {safe: true});                             
        }
    }


});

dojo.declare('hub.managers.InvokePlugin', [hub.managers.Plugin], {
    // @todo: implement, port from tom/client/scripts/old/plugin.js
    
	dispose: function() {
console.log("hub.managers.InvokePlugin.dispose() start");
		this.inherited('dispose', arguments);
console.log("hub.managers.InvokePlugin.dispose() end");
	},

    /**
     * Handles a RPC from the server. Properly logs error codes.
     * 
     * @param response Response from the server, JSON encoded
     * @param args Args used to send the request
     */
    _handleResponse: function(response, args) {
        if(!response) {
            // no data from server
            throw new Error('Server response was empty.');
        }

        var data = dojo.fromJson(response);
        this.workflow.lastResponse = data;
//        alert(response);

        if(data['code'] == 0) {
            // unknown server error
            this.showMessage(data['payload']);
            return;
        } else if (data['code'] > 0) {
            // known server error
            if (data['payload']['text']) {
            	this.showMessage(data['payload']['text']);
            } else {
            	this.showMessage(data['payload']);
            }
            
            return;
        }
//        alert('_handleResponse');
        var method = data['method'];
        var params = data['payload'];
        // @todo: do we do anything with encoding? data is json encoded
        // already, so previous decode should unwrap, right?

        try {
            // dispatch
            this[method](params);
        } catch (e) {
            // known client error
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            this.showMessage(msgs.CLIENT_ERROR + "\n" + 
                             e.message);
        }

//        alert('_handleResponse workflow-next');
		if(method!='done'){
			dojo.publish('workflow-next-'+this.tab.id ,[]);
		}	        
    },

    /**
     * Makes a RPC from client to server.
     * 
     * @method Method to invoke server-side
     * @data Parameters for the server side method
     * @transport Method of transport, either 'xhr' or 'iframe'
     */
    invokeServer: function(method, data, transport) {
        var _self = this;
        _self.workflow.setBusy(true);
        
        if(typeof(data)=='string'){
        	data = dojo.by.id(data);
        }

        // set tab as busy
//		TOM.tabs.man.setTabBusy(true, this.tab);
//		hub.busyPage(this.tab, true, true);

        if(!transport) {
            transport = 'xhr';
        }

        if(transport == 'xhr') {
            var handleAs = "text";
        } else if (transport == 'iframe') {
            var handleAs = "text/html";
        } else {
            throw new Error("Unknown transport");
        }

        // clear any notification
        var e = document.getElementById(this.getEditorId()+'_status');
        if(e)
            e.innerHTML = '';

		var _self = this;
        var msgs = dojo.i18n.getLocalization('hub', 'messages');

		// build a request object and handler for dojo
		
		var kw = {
			url: this.ajax_url+'/method/'+method+'/transport/'+transport,
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
//            sync: true,

            error: function(response, args) {
                if(response.dojoType == "cancel"){
                    _self.showMessage(msgs.CANCELLED);
                } else if (response.dojoType == "timeout"){
                    _self.showMessage(msgs.TIMEOUT);
                } else {
//                    var err = TOM.i18n.man._('SERVER_ERROR') + " " + response.message;
//                    _self.showMessage(TOM.i18n.man._(err));
					_self.showMessage(msgs.SERVER_ERROR+ " " + response.message);
                }
                // always set tab to not busy when done
//                TOM.tabs.man.setTabBusy(false, _self.tab);
                _self.workflow.setBusy(false);
                
                return response;
            },

            load: function(response, args) {
                try {
                    // pull from text area
                    data = response.body.childNodes[0].value;
                } catch(e) {
                    // not loaded via iframe
                }

				try {
                    if(hub.submit.processResponseCode(response)){
					    _self._handleResponse(response, args);
				    }
				} catch(e) {
				                  // error handling response
				                  var err = msgs.CLIENT_ERROR + " " + e.message;
				                  _self.showMessage(err);
				}

                // stop spinner
//                TOM.tabs.man.setTabBusy(false, _self.tab);
//				hub.busyPage(_self.tab, false, false);
                _self.workflow.setBusy(false);
                return response;
            }
		};

        // decide what the data source is
        if(data.elements != undefined) {
            // data from a form
            kw['form'] = data;
        } else {
            // data from object name/value pairs
            kw['content'] = data;
        }
        
        // pick a transport and send
        if(transport == "iframe") {
            dojo.require("dojo.io.iframe");
			dojo.io.iframe.send(kw);
        } else {
            dojo.xhrPost(kw);
        }

        // convenience for use in onsubmit
        return false;
    }
});
