/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.managers.Urls');
dojo.require('hub.main');

/**
 * Manages the cache of all RESTful server URLs required by the core UI.
 * Currently, all paths are hardcoded off of a base URL calcuated when the
 * initial page renders. Later, this manager could grow to receive the entire
 * REST API in some format from the server (e.g., from the APP service
 * document).
 */
dojo.declare('hub.managers.Urls', null, {
    constructor: function(base_url) {
        this.base_url = base_url;
        this.server_url = base_url.substring(0, base_url.lastIndexOf('/client'));
    },

    /////////////////////////////////////
    // URLs for template page data
    ////////////////////////////////////

    /**
     * URL to object containing all data required for rendering the home page.
     */
    getHomePage: function() {
        return this.base_url + '/ui/home';
    },

    /**
     * URL to object containing search type info required for rendering the search box.
     */
    getSearchTypes: function() {
        return this.base_url + '/ui/searchtypes';
    },

    /**
     * URL to object containing all entry details.
     */
    getDetailsPage: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getDetailsPage requires object_id');
        }
        return this.base_url + '/ui/details/entryid/'+object_id;
    },

    /**
     * URL to object containing information about the logged in user.
     */
    getProfilePage: function() {
        return base_url + '/ui/profile';
    },

    /**
     * URL to object containing all details about feed parameters.
     */
    getParametersPage: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: getParametersPage requires object_id');
        }
        return this.base_url + '/plugin/params/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminTables: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminProxy: function() {
        return this.base_url + '/ui/update/name/proxy';
    },

    /**
     * URL to object containing user permissions.
     */
    getAdminPermissionsPage: function() {
        return this.base_url + '/ui/editor/name/adminpermissions';
    },

    /**
     * URL to object containing list of items that can be deleted.
     */
    getAdminDeletePage: function() {
        return this.base_url + '/ui/editor/name/adminobjects';
    },

    /**
     * URL to object containing banned users.
     */
    getAdminBanPage: function() {
        return this.base_url + '/ui/editor/name/adminusers';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminEmailPage: function() {
        return this.base_url + '/ui/editor/name/adminreports';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminCategoryPage: function() {
        return this.base_url + '/ui/editor/name/admincategory';
    },

    /**
     * URL to object containing tracking configuration.
     */
    getAdminEditTrackingPage: function() {
        return this.base_url + '/ui/editor/name/admintracking';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminTrackingData: function() {
        return this.base_url + '/ui/editor/name/admintrackeddata';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminProxyPage: function() {
        return this.base_url + '/ui/editor/name/adminproxy';
    },

    /**
     * URL to object containing admintdi.
     */
    getAdminTDI: function() {
        return this.base_url + '/ui/editor/name/admintdi';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getSettingsPage: function() {
        // @todo
    },

    /////////////////////////////////////
    // URLs for widget data / actions
    ////////////////////////////////////

    /**
     * URL to object or XML feed listing catalog entries for the given search
     * query and term.
     */
    getCatalog: function(collection, term, searchType, sortBy, sortOrder) {
		var url = "";
		var map = {
		};
		var c = collection;
		if (c.indexOf("ui.") != 0) {
			c = "ui." + c; 
		}
		map["collection"] = c;
		if (searchType != null) {
			map["searchType"] = searchType;
		} else if (term != null) {
			map["searchType"] = ""; // Just ensure it is set
		}
		if (term != null) {
			map["searchTerms"] = term;
		}
		else if (searchType != null){
			map["searchTerms"] = ""; // Just ensure it is set
		}
		if (sortBy != null) {
			map["sortBy"] = sortBy;
		}
		if (sortOrder != null) {
			map["sortOrder"] = sortOrder;
		}
		url = this.base_url + "/.." + "/atom"; // TODO: make it more robust
        url = url + '?' + dojo.objectToQuery(map);
        return url;
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategories: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categories';
        } else {
            return this.base_url + '/ui/categories/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategoriesList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categorieslist';
        } else {
            return this.base_url + '/ui/categorieslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all association names if object_id is null or all names
     * associations assigned to the object_id if not null.
     */
    getAssociationsList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/associationslist';
        } else {
            return this.base_url + '/ui/associationslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of ids of all association assigned to the object_id.
     */
    getAssociationsIdsList: function(object_id) {
        return this.base_url + '/ui/associationsidslist/entryid/'+object_id;
    },

    /**
     * URL to array of all comments for the object_id. Array items are objects
     * having 'commenttitle', 'commenttext', 'creatorid', and 'timestamp'.
     */
    getComments: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getComments requires object_id');
        }
        return this.base_url + '/ui/comments/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getRatings: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getRatings requires object_id');
        }
        return this.base_url + '/ui/ratings/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getTags: function(objectid) {
        if(objectid == null) {
            return this.base_url + '/ui/tags';
        } else {
            return this.base_url + '/ui/tags/entryid/'+objectid;
        }
    },

    /**
     * URL to array of objects with username as key and fullname as value.
     */
    getUserSearch: function(user) {
	    user = user.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchusers/user/' + user;
    },

    /**
     * URL to array of objects with groupname as both key and value.
     */
    getGroupSearch: function(group) {
	    group = group.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchgroups/group/' + group;
    },

    /////////////////////////////////////
    // URLs for submitting data back to server
    ////////////////////////////////////

    /**
     * URL to post a report about abuse on an object_id.
     */
    postAbuse: function(object_id) {
        // @todo
    },

    /**
     * URL to post a deletion request for an object_id.
     */
    postDelete: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postDelete requires object_id');
        }
        return this.base_url+'/plugin/delete/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post a cancel request for an object_id.
     */
    postCancel: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postCancel requires object_id');
        }
        return this.base_url+'/plugin/cancel/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post tags for an object_id with 'tags' as key and a comma
     * separated string of tag names as value.
     */
    postTags: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postTags requires object_id');
        }
        return this.base_url+'/ui/update/name/tags/entryid/'+object_id;
    },

    /**
     * URL to post a comment for an object_id.
     */
    postComment: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postComment requires object_id');
        }
        return this.base_url+'/ui/update/name/comment/entryid/'+object_id;
    },

    /**
     * URL to post a rating for an object_id.
     */
    postRating: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postRating requires object_id');
        }
        return this.base_url+'/ui/update/name/rating/entryid/'+object_id;
    },

    /**
     * URL to post a bunch of metadata for an object_id.
     */
    postMetadata: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postMetadata requires object_id');
        }
        // @todo: implement
    },

    /**
     * URL to post profile updates for an object_id.
     */
    postProfile: function() {
        return this.base_url + '/ui/update/name/profile';
    },

    /**
     * URL to post a reset of the entire Mashup Hub database.
     */
    postAdminInit: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminDelete: function() {
        return this.base_url + '/ui/update/name/adminobjects';
    },

    /**
     * URL to post changes to categories.
     */
    postAdminCategories: function() {
        return this.base_url + '/ui/update/name/admincategory';
    },

    /**
     * URL to post changes to user permissions.
     */
    postAdminPermissions: function() {
        return this.base_url + '/ui/update/name/adminpermissions';
    },

    /**
     * URL to post changes to the banned users.
     */
    postAdminBan: function() {
        return this.base_url + '/ui/update/name/adminusers';
    },

    /**
     * URL to post changes to the email notification config
     */
    postAdminEmail: function() {
        return this.base_url + '/ui/update/name/adminreports';
    },

    /**
     * URL to post changes to the email notification config
     */
    postReportEmail: function() {
        return this.base_url + '/ui/update/name/report';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingStatus: function() {
        return this.base_url + '/ui/update/name/admintrackingglobal';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingSettings: function() {
        return this.base_url + '/ui/update/name/trackingedittype';
    },
    
    /**
     * URL to clear tracking data.
     */
    postAdminTrackingClear: function() {
        return this.base_url + '/ui/update/name/trackingcleardata';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingPermissions: function() {
        return this.base_url + '/ui/update/name/admintrackingobject';
    },

    /**
     * URL to post tdi configuration.
     */
    postAdminTDI: function() {
        return this.base_url + '/ui/update/name/admintdi';
    },
    
    /**
     * URL to post connection info for database migration.
     */
    postAdminMigrateDB: function() {
        return this.base_url + '/ui/update/name/adminmigratedb';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminProxy: function() {
        return this.base_url + '/ui/update/name/adminproxy';
    },


    /**
     * URL to post a login request.
     */
    postLogin: function() {
        return this.base_url + '/ui/login';
    },

    /**
     * URL to post a logout request.
     */
    postLogout: function() {
        return this.base_url + '/ui/logout';
    },

    /**
     * URL to post an account registration request.

     */
    postRegister: function() {
        return this.base_url + '/ui/register';
    },

    /**
     * URL to post data for verification before continuing in the installer
     * wizard.
     *
     * @param action: Action name corresponding with the page to verify
     */
    postInstall: function(action) {
        return this.base_url + '/installer/' + action;
    },

    /////////////////////////////////////
    // URLs for static resources
    ////////////////////////////////////

    /**
     * URL to the base images directory.
     */
    getImage: function(filename) {
        return this.base_url + '/images/' + filename;
    },

    /**
     * URL to the icons directory under the images folder.
     *
     */
    getIcon: function(filename) {
        return this.base_url + '/images/icons/' + filename;
    },

    //////////////////////////////////////
    // FROM DETAILS PAGE
    //////////////////////////////////////

    postFavorites: function(objectid) {
        if(objectid == null) {
            throw new Error('Urls: postFavorites requires objectid');
        }
        return this.base_url + '/ui/update/name/stuff/entryid/'+objectid;
    },
    
    getPluginGenerate: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires pluginid');
        }       
        return this.base_url + '/plugin/generate/entryid/'+objectid+'/pluginid/'+pluginid;
    },
    
    getPluginParams: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires pluginid');
        }       
        return this.base_url + '/plugin/params/entryid/'+objectid+'/pluginid/'+pluginid;
    },

    getAjaxUrl: function(plugin_id,entry_id,method){
    	return this.base_url + '/plugin/ajax/entryid/'+entry_id+'/pluginid/'+plugin_id+ '/method/'+method+'/';
    },
    
    secureUrl: function(ourl) {
    	if (hub.config.isSecure) {
	        var purl = new dojo._Url(ourl);
	        if ((purl.scheme == 'http') && purl.authority) {
	        	// we need to securize it:
	        	var result = "https://" + purl.host + hub.config.securePort + 
	        	(purl.path ? purl.path : "")  + 
	        	(purl.query ? ("?" + purl.query) : "") +
	        	(purl.fragment ? ("#" + purl.fragment) : "");
	        	return result;   
	        }
	    }
    	return ourl;
    }
});
