/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.misc.popup');
//
// JavaScript popup layer support
// Version 1.11
//
// Author: Scott Morgan (morgans@us.ibm.com)
//
// Change history:
//    13 Oct 2006 - 1.00 Initial release.
//    25 Oct 2006 - 1.01 Add SurfAid support.
//    07 Nov 2006 - 1.02 Change visuals to support latest UI design.
//    11 Nov 2006 - 1.03 Modify zIndex on show and hide.
//    29 Nov 2006 - 1.04 Add support for lining up the layer to the left/right of the target.
//    17 Jan 2007 - 1.05 Add support for layers to be displayed in "bubbles" with new positioning styles.
//    22 Jan 2007 - 1.06 Put popup styles into a flag and add showPopupLayer2 function that uses them.
//    22 Jan 2007 - 1.07 Modal dialog layers now reposition like other layers on window resize, and
//                       moveable layers now reposition on window resize until the user has manually moved them.
//                       Previously neither repostioned on window resize.
//    29 Jan 2007 - 1.08 Allow the bubble width or height dimension to be smaller than the fin base if the fin
//                       is not to be displayed on that side.
//    08 Feb 2007 - 1.09 Allow forms displayed in bubble to retain field values on subsequent appearances.
//    09 Feb 2007 - 1.10 Add a flag that indicates the layer is a form, and stop using the modal dialog flag
//                       to indicate this as well, as not all forms need be modal.
//    18 Feb 2007 - 1.11 Prepare file for sharing with other development organizations.
//

//
// Global constants
//

//
// Path of the common image repository where the images will be found.
// The popup_layer.css file also references an image whose path will need to be changed to correspond
// to the path defined here.
//
pl_IMAGE_PATH = "images/popup/";

set_pl_IMAGE_PATH = function (path)
{
	pl_IMAGE_PATH = path;
	pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
	pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
	pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
	pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
	pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
	pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
	pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
	pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
	pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
	pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
	pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
	pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
	pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
	pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
	pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
	pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
	// Images for the banner - may also be referenced in popup_layers.css.
	pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
	pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
	pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
	pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
	pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	// HTML that is swapped to replace the fine being used with one pointing in a different direction.
	pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
}

//
// Externalized variables and constants
//
BANNER_ID_SUFFIX = "popupbanner";  // Banner id is the layer id + this suffix
login_iframe_active = false;


//
// Image constants
//
// Images for the bubble
pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
// Images for the banner - may also be referenced in popup_layers.css.
pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
pl_POPUPBANNERCLOSE_WIDTH = "6";
pl_POPUPBANNERCLOSE_HEIGHT = "7";
// 1 pixel transparent image
pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
// Bubble image dimensions
pl_BUBBLE_FIN_REACH = 42;//33;  // From the tip of the fin to the base.
pl_BUBBLE_FIN_BASE = 61;//56;   // Length of the base of the fin.
pl_BUBBLE_FIN_INSET = 12;//11;  // Amount the fin image overlays the bubble.
pl_BUBBLE_FIN_INSET_Y = 15; //@GINO: necessary for new GIF images
pl_BUBBLE_CORNER_WIDTH = 15;
pl_BUBBLE_CORNER_HEIGHT = 15;
pl_BUBBLE_SIDE_DEPTH = 15;
pl_BUBBLE_SIDE_LENGTH = 5;
// Banner image dimensions
pl_BANNER_BG_WIDTH = 84;
pl_BANNER_BG_HEIGHT = 28;
pl_BANNER_CLOSE_X_WIDTH = 6;
pl_BANNER_CLOSE_X_HEIGHT = 7;

// Bubble-specific table cell IDs
pl_BUBBLE_CONTENT_MINHEIGHT_ID = "popupLayerBubbleHeight";
pl_BUBBLE_CONTENT_MINWIDTH_ID = "popupLayerBubbleWidth";
// HTML that is swapped in and out of the table cell IDs that regulate a minimum dimension to ensure the fin fits on that side.
pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
// HTML that is swapped to replace the fine being used with one pointing in a different direction.
pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";

pl_LOGIN_IFRAME_ID_SUFFIX = "w3signonIFrame";  // IFrame id is the layer id + this suffix
pl_DRAG_START_DIFFERENTIAL = 0;  // Pixels the mouse must be moved more than while help down to start the drag operation
pl_MOUSEOVER_SHOW_DELAY = 250;  // Milliseconds
pl_MOUSEOUT_HIDE_DELAY = 250;  // Milliseconds
pl_DRAG_VISIBILITY_BUFFER = 20;  // Pixels of drag object to always appears in view when moved top and left

pl_BUBBLE_ID = "popupLayerBubble";  // ID of the bubble
pl_BUBBLE_CONTENT_ID = "popupLayerBubbleContent";  // ID of the bubble cell that contains the layer content
pl_BUBBLE_FIN_ID = "popupLayerBubbleFin";  // ID of the object that contains the bubble fin
pl_BUBBLECLONE_ID_ADDON = "_bubbleclone";  // ID of the layer within a bubble id layer id + this suffix
pl_CONTENT_MAIN_ID = "content-main";  // ID of section of web page where cloned layer within bubble is placed

// Sizing and positioning constants
pl_IDEAL_NORTH_SOUTH_POSITION = .4;
pl_IDEAL_EAST_WEST_FIN_POSITION = .25;
pl_BIG_OFFSCREEN = 10000;
pl_FINPOS_INDENT_DEFAULT = 8;

//
// External flag bits for the showPopupLayer call
//
PL_ADD_CLOSE_LINK =       0x0001;
PL_ADD_BANNER =           0x0002;
PL_DIALOG_BEHAVIOR =      0x0004;
PL_LOGIN_REQUIRED =       0x0008;
PL_MOUSE_OVER_SUPPORT =   0x0010;
PL_DISPLAY_IN_BUBBLE =    0x0020;
PL_SUGGESTED_POS_BELOW =  0x0040;
PL_SUGGESTED_POS_ABOVE =  0x0080;
PL_SUGGESTED_POS_LEFT =   0x0100;
PL_SUGGESTED_POS_RIGHT =  0x0200;
PL_FORCE_VISIBLE_TARGET = 0x0400;
PL_FORCE_POSITION =       0x0800;
PL_FORM =                 0x1000;


//
// Global variables
//
pl_active_layer_id = "";
pl_active_layer_obj = null;
pl_active_clonedlayer_obj = null;
pl_active_contentclone_obj = null;
pl_active_trigger_id = "";
pl_active_trigger_obj = null;
pl_active_target_obj = null;
pl_active_drag_obj = null;
pl_active_mybubblefin = null;
pl_active_layer_html = "";
pl_active_IFrame_id = "";
pl_active_surfaid_base_string = "";
pl_active_style_flags = 0x0;
pl_active_has_dragged = false;
pl_active_bubble_minwidth_cell = null;
pl_active_bubble_minheight_cell = null;
pl_clicked_layer = false;
pl_onclick_doc_prevEH =  "";
pl_onkeypress_doc_prevEH =  "";
pl_onresize_win_prevEH =  "";
pl_onunload_win_prevEH = "";
pl_onmousemove_body_prevEH = "";
pl_onmouseup_body_prevEH = "";
pl_onclick_trigger_prevEH =  "";
pl_onmouseover_trigger_prevEH = "";
pl_onfocus_trigger_prevEH = "";
pl_active_drag_obj_prevcursor = "";
pl_queued_show_layer_request = false;
pl_queued_hide_layer_request = false;
pl_queued_show_layer_timer_id = -1;
pl_queued_hide_layer_timer_id = -1;
pl_move_in_process = false;
pl_drag_mouse_down = false;
pl_move_init_x = 0;
pl_move_init_y = 0;
pl_move_min_x = 0;
pl_move_min_y = 0;
pl_move_offset_x = 0;
pl_move_offset_y = 0;
pl_bubble_images_preloaded = false;
pl_bubble_orig_height = 0;
pl_bubble_orig_width = 0;

//
// Preloaded images
//
pl_img_clearpix = null;
pl_img_banner_bg = null;
pl_img_close_x = null;
pl_img_bubble_fin_nl = null;
pl_img_bubble_fin_nr = null;
pl_img_bubble_fin_sl = null;
pl_img_bubble_fin_sr = null;
pl_img_bubble_fin_eu = null;
pl_img_bubble_fin_ed = null;
pl_img_bubble_fin_wu = null;
pl_img_bubble_fin_wd = null;
pl_img_bubble_corner_nw = null;
pl_img_bubble_corner_ne = null;
pl_img_bubble_corner_sw = null;
pl_img_bubble_corner_se = null;
pl_img_bubble_side_w = null;
pl_img_bubble_side_e = null;
pl_img_bubble_side_n = null;
pl_img_bubble_side_s = null;




//
// Call to preload the images when this script file loads, so they will already be loaded the first time requested.
//
//pl_preloadPopupLayerImages();



// -----------------------------------------------------------------------------
//
// ENTRY POINT functions called externally (no pl_ prefix)
//
// showPopupLayer2 - position and show the popup layer.
// hidePopupLayer - hide the popup layer if one is showing.
// getPopupLayerElementById - return the object in the popup layer with the specified ID.
//
// showPopupLayer - DEPRECATED, included for backward compatibility only.
//
// -----------------------------------------------------------------------------

//
// ENTRY POINT from web pages: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer2(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer2(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer2(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer2(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (style flags, bitwise OR'ed or added together, default = no styles set)
// Parm 5 (id of the object that will be used for dragging the layer, default = "")
// Parm 6 (SurfAid base string used for logging that layer was dismissed, default = "")
//
// Style flags passed in parm 4:
// - PL_ADD_CLOSE_LINK = add close link at lower left corner of popup.
// - PL_ADD_BANNER = add the onePlace banner with close icon to top of popup.
// - PL_FORM = the layer contains a form that may be submitted by the user.
// - PL_DIALOG_BEHAVIOR = popup layer acts like a modal dialog.
// - PL_LOGIN_REQUIRED = secure login is required.
// - PL_MOUSE_OVER_SUPPORT = mouse over and out support is desired to show and hide this object.
// - PL_DISPLAY_IN_BUBBLE = display the popup as a bubble enclosing the user-defined popup layer.
// - PL_SUGGESTED_POS_BELOW = Default if no position suggested or multiple positions suggested: position the popup below the target if it fits, else try above, left, then right.
// - PL_SUGGESTED_POS_ABOVE = position the popup above the target if it fits, else try below, left, then right.
// - PL_SUGGESTED_POS_LEFT = position the popup to the left of the target if it fits, else try right, below, then above.
// - PL_SUGGESTED_POS_RIGHT = position the popup to the right of the target if it fits, else try left, below, then above.
// - PL_FORCE_VISIBLE_TARGET = Always kep the target visible, even if this results in the popup layer not being fully visible.
// - PL_FORCE_POSITION = always use the suggested position and keep the target visible, even if this results in the popup layer not being fully visible.
//
showPopupLayer2 = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   // If acts like a dialog, clicking or mouse over outside the layer does not get processed,
   // and we do not display any other layers while this one is up, simulating the layer being modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return;
   }

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer2.arguments.length;
   if (numArgs < 6) {
      surfaid_base_string = "";
      if (numArgs < 5) {
         idDragger = "";
         if (numArgs < 4) {
            style_flags = 0x0;
            if (numArgs < 3) {
               return;
            }
         }
      }
   }
   // Now support passing in null for any optional parm to take its default value.
   if (style_flags == null) {
      style_flags = 0x0;
   }
   if (idDragger == null) {
      idDragger = "";
   }
   if (surfaid_base_string == null) {
      surfaid_base_string = "";
   }

   // If no suggested position was passed in, or more than one suggested position was passed in, set to default position of below pos.
   var sug_pos = style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT);
   if (sug_pos == 0x0) {
      style_flags |= PL_SUGGESTED_POS_BELOW;
   } else if ((sug_pos != PL_SUGGESTED_POS_BELOW) && (sug_pos != PL_SUGGESTED_POS_ABOVE) && (sug_pos != PL_SUGGESTED_POS_LEFT) && (sug_pos != PL_SUGGESTED_POS_RIGHT)) {
      if (style_flags & PL_SUGGESTED_POS_ABOVE) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_ABOVE);
      }
      if (style_flags & PL_SUGGESTED_POS_LEFT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_LEFT);
      }
      if (style_flags & PL_SUGGESTED_POS_RIGHT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_RIGHT);
      }
      if (!(style_flags & PL_SUGGESTED_POS_BELOW)) {
         style_flags |= PL_SUGGESTED_POS_BELOW;
      }
   }

   var trigger = document.getElementById(idTrigger);
   var layer = document.getElementById(idLayer);
   var target = document.getElementById(idTarget);
   // If any of the three objects is undefined, do not attempt to process.
   if ((layer == null) || (target == null) || (trigger == null)) {
      return;
   }

   var mybubble = null;
   var mybubblefin = null;

   // Set the mouse out and blur event handlers since they are cleared when called if mouse support requested.
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      trigger.onmouseout = pl_processTriggerMouseOutAndBlur;
      trigger.onblur = pl_processTriggerMouseOutAndBlur;
   }

   // If this is a mouse over action, and we do not have a previous request in the queue to show a layer.
   if ((style_flags & PL_MOUSE_OVER_SUPPORT) && (!pl_queued_show_layer_request)) {
      // If there is a queued hide request, kill it as any layer will be hidden when this queued show request runs.
      if (pl_queued_hide_layer_request) {
         pl_queued_hide_layer_request = false;
         pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 1");
      }
      pl_queued_show_layer_request = true;
      // Set a timer to show the layer.
      pl_queued_show_layer_timer_id = window.setTimeout("pl_showDelayedPopupLayer('" + idLayer + "', '" + idTarget + "', '" + idTrigger + "', " + style_flags + ", '" + idDragger + "', '" + surfaid_base_string + "')", pl_MOUSEOVER_SHOW_DELAY);
      // Exit and wait for the timer.
      return;
   }

   // Hide current layer if one is already showing.
   if ((pl_active_layer_obj != null)) {
      hidePopupLayer();
   }

   // Clear any possible queued requests
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 2");
   }
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 3");
   }

   // If we are displaying the layer in a bubble, create the bubble now.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {

      mybubblefin = document.getElementById(pl_BUBBLE_FIN_ID);
      if (mybubblefin == null) {
         // Create the new bubble fin
         mybubblefin = document.createElement("div");
         mybubblefin.id = pl_BUBBLE_FIN_ID;
         mybubblefin.style.position = "absolute";
         mybubblefin.style.top = "10px";
         mybubblefin.style.left = "10px";
         mybubblefin.style.zIndex = -1;
         mybubblefin.style.visibility = "hidden";
         mybubblefin.innerHTML = "";
         // Add it to the document
         document.body.appendChild(mybubblefin);
      }
      pl_active_mybubblefin = mybubblefin;

      // Get the bubble object, creating it the first time needed.
      mybubble = document.getElementById(pl_BUBBLE_ID);
      if (mybubble == null) {
         // Create the table layout for the bubble.
         var bubble_table = "";
         bubble_table += "<table class='w3pluto1' cellpadding='0' cellspacing='0' border='0' summary=''>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINWIDTH_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_N + "); background-repeat:repeat-x; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINHEIGHT_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_W + "); background-repeat:repeat-y; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td class='pl_bubble_content_area' id='" + pl_BUBBLE_CONTENT_ID + "'></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_E + "); background-repeat:repeat-y; background-position:top right; font-size: 1px;'></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_S + "); background-repeat:repeat-x; background-position:bottom left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "</table>";

         // Create the new bubble object and assign the bubble table to it.
         mybubble = document.createElement("div");
         mybubble.id = pl_BUBBLE_ID;
         mybubble.style.position = "absolute";
         mybubble.style.top = "10px";
         mybubble.style.left = "10px";
         mybubble.style.zIndex = -1;
         mybubble.style.visibility = "hidden";
         mybubble.innerHTML = bubble_table;
         // Make it a child of the w3 content area if one exists so it picks up styles like the rest of the content.
         var content_main = document.getElementById(pl_CONTENT_MAIN_ID);
         if (content_main != null) {
            content_main.appendChild(mybubble);
         } else {
            document.body.appendChild(mybubble);
         }
      }
      // Store the bubble cells that dictate minumum width and height depending in which side the fin will appear.
      if (pl_active_bubble_minwidth_cell == null) {
         pl_active_bubble_minwidth_cell = document.getElementById(pl_BUBBLE_CONTENT_MINWIDTH_ID);
      }
      if (pl_active_bubble_minheight_cell == null) {
         pl_active_bubble_minheight_cell = document.getElementById(pl_BUBBLE_CONTENT_MINHEIGHT_ID);
      }
      // Start with no min width or height until the positioning calc is run.
      pl_active_bubble_minwidth_cell.innerHTML = "";
      pl_active_bubble_minheight_cell.innerHTML = "";
      // Reset global vars that indicate the original layer width and height have not yet been saved for the positioning calc.
      pl_bubble_orig_width = 0;
      pl_bubble_orig_height = 0;

      // Put the layout table in the bubble.
      var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
      // Clear out any html from a previous bubble.
      mybubblecontent.innerHTML = "";
      // Clone the display layer so we get the entire DIV and all its styles, etc.
      var mycontentclone = layer.cloneNode(true);
      mycontentclone.style.position = "static";
      mycontentclone.style.visibility = "inherit";
      // Give the clone layer a unique ID so there is no confusion on the layer level caused by duplicate IDs.
      mycontentclone.id = mycontentclone.id + pl_BUBBLECLONE_ID_ADDON;
      // Add the cloned display layer into the bubble.
      mybubblecontent.appendChild(mycontentclone);

      // Now set the layer variable to point to the cloned layer for the following code, as we no longer need
      // to reference the original layer.
      pl_active_clonedlayer_obj = layer;
      pl_active_contentclone_obj = mycontentclone;
      layer = mycontentclone;
   }

   // If we are displaying the layer in a bubble, it is the bubble that will be the active layer.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      pl_active_layer_id = pl_BUBBLE_ID;
      pl_active_layer_obj = mybubble;
   } else {
      pl_active_layer_id = idLayer;
      pl_active_layer_obj = layer;
   }
   // Set globals for this showing of the layer
   pl_active_trigger_id = idTrigger;
   pl_active_trigger_obj = trigger;
   pl_active_target_obj = target;
   pl_active_style_flags = style_flags;
   pl_active_surfaid_base_string = surfaid_base_string;

   // Store the original html for the popup layer
   pl_active_layer_html = layer.innerHTML;
   // Add the Close link unless the optional parm said to not show it, and as long as it has not already been added.
   if (style_flags & PL_ADD_CLOSE_LINK) {
      if (layer.innerHTML.indexOf("pl_bottom_close_link") < 0) {
         layer.innerHTML = layer.innerHTML + "<table width='100%' cellpadding='0' cellspacing='0' border='0' summary=''><tr><td align='right' class='pl_bottom_close_link'><a class='pl_bottom_close_link_anchor' href='javascript:hidePopupLayer();'>Close</a></td></tr></table>"
      }
   }

   // Add the w3 banner to the beginning if optional parm said to do so.
   var banner = null;
   if (style_flags & PL_ADD_BANNER) {
      var banner_id = idLayer + BANNER_ID_SUFFIX;
     if (layer.innerHTML.indexOf(banner_id) < 0) {
        var banner_html = "<div id='" + banner_id + "' class='pl_banner'><a href='javascript:hidePopupLayer();'><img src='" + pl_POPUPBANNERCLOSE_FULLPATH + "' class='pl_banner_close' width='" + pl_POPUPBANNERCLOSE_WIDTH + "' height='" + pl_POPUPBANNERCLOSE_HEIGHT + "' alt='Close' /></a></div>"
        layer.innerHTML = banner_html + layer.innerHTML;
     }
     banner = document.getElementById(banner_id);
   }

   // Secure login support (supported for the Pluto-driven onePlace site only)
   var IFrame_id = "";
   if (style_flags & PL_LOGIN_REQUIRED) {
      // Set the flag that tells the login failure page code we are playing in an IFrame
      login_iframe_active = true;
      IFrame_id = idLayer + pl_LOGIN_IFRAME_ID_SUFFIX;
      pl_active_IFrame_id = IFrame_id;
      // Add the hidden layer that will contain the IFrame if not already added.
      if (layer.innerHTML.indexOf(IFrame_id) < 0) {
         layer.innerHTML = layer.innerHTML + "<div id='" + IFrame_id + "' name='" + IFrame_id + "' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></div>"
      }
      // Load the blank secure page into the IFrame
      document.getElementById(IFrame_id).innerHTML = "<iframe src='../spage/13' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></iframe>";
   }

   // Set the zIndex if not already set so the layer will appear above the w3 banner, which is zIndex=1
   // If display in bubble, set the bubble's z-order
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.zIndex = 2;
      mybubblefin.style.zIndex = 3;
   }
   if (layer.style.zIndex < 2) {
      layer.style.zIndex = 2;
   }

   // Get the object that will be used for dragging the layer, if any.
   if (idDragger != "") {
      // If display in bubble, we need to get the object from the cloned layer, not the display layer.
      if (style_flags & PL_DISPLAY_IN_BUBBLE) {
         // if the request was to make the entire layer draggable, and the layer is in a bubble, make the bubble draggable instead..
         if (idDragger == idLayer) {
            pl_active_drag_obj = document.getElementById(pl_BUBBLE_ID);
         } else {
            pl_active_drag_obj = getPopupLayerElementById(idDragger);
         }
      } else {
         pl_active_drag_obj = document.getElementById(idDragger);
      }
   }

   // Position the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();

   // Now make the popup layer visible
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.visibility = "visible";
      // Is fin showing or not based on position?
      if (show_fin) {
         mybubblefin.style.visibility = "visible";
      } else {
         mybubblefin.style.zIndex = -1;
      }
   } else {
      layer.style.visibility = "visible";
   }

   // And we reassign the layer var to be the bubble itself for event handling assignment below.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      layer = mybubble;
   }

   // Capture relevant events while the popup layer is up.
   // First we need to save any event handlers already assigned for system objects before we override them.
   // Generic handlers for all types of layers.
   pl_onkeypress_doc_prevEH = document.onkeypress;
   document.onkeypress = pl_processBodyKey;
   pl_onresize_win_prevEH = window.onresize;
   window.onresize = pl_processResize;
   pl_onunload_win_prevEH = window.onunload;
   window.onunload = pl_processWindowUnload;
   // Handlers for layers that support mouse over and out for display
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      pl_onmouseover_trigger_prevEH = trigger.onmouseover;
      trigger.onmouseover = pl_processTriggerMouseOverAndFocus;
      pl_onfocus_trigger_prevEH =  trigger.onfocus;
      trigger.onfocus = pl_processTriggerMouseOverAndFocus;
      layer.onmouseover = pl_processLayerMouseOver;
      layer.onmouseout = pl_processLayerMouseOut;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onmouseover = pl_processLayerMouseOver;
         mybubblefin.onmouseout = pl_processLayerMouseOut;
      }

   } else {
      // Handlers for layers that support clicking for display
      pl_onclick_doc_prevEH = document.onclick;
      document.onclick = pl_processBodyClick;
      pl_onclick_trigger_prevEH = trigger.onclick;
      trigger.onclick = pl_processTriggerClick;
      layer.onclick = pl_processLayerClick;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onclick = pl_processLayerClick;
      }
   }
   // Handlers for layers that support drag/drop.
   if (pl_active_drag_obj != null) {
      pl_onmousemove_body_prevEH = document.body.onmousemove;
      document.body.onmousemove = pl_processDragMouseMove;
      pl_onmouseup_body_prevEH = document.body.onmouseup;
      document.body.onmouseup = pl_processDragMouseUp;
      pl_active_drag_obj.onmousedown = pl_processDragMouseDown;
      pl_active_drag_obj.onlosecapture = pl_processDragLoseCapture;
      // Set cursor to the move pointer when the mouse is over the drag area.
      pl_active_drag_obj_prevcursor = pl_active_drag_obj.style.cursor;
      pl_active_drag_obj.style.cursor = "move";
   }
}


//
// DEPRECTATED - OLD ENTRY POINT from web pages, supported for backward compatibility: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (add the close link, default = true)
// Parm 5 (add the onePlace banner with close icon to top, default = false)
// Parm 6 (popup layer acts like a modal dialog, default = false)
// Parm 7 (secure login is required, default = false)
// Parm 8 (id of the object that will be used for dragging the layer, default = "")
// Parm 9 (mouse over and out support is desired to show and hide this object, default = false)
// Parm 10 (SurfAid base string used for logging that layer was dismissed, default = "")
// Parm 11 (align popup layer to the left or right of the target if possible, default = false)
//
showPopupLayer = function (idLayer, idTarget, idTrigger, add_close, add_w3_banner, dialog_behavior, login_required, idDragger, mouse_over_out_support, surfaid_base_string, align_left_right) {

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer.arguments.length;
   if (numArgs < 11) {
      align_left_right = false;
      if (numArgs < 10) {
         surfaid_base_string = "";
         if (numArgs < 9) {
            mouse_over_out_support = false;
            if (numArgs < 8) {
               idDragger = "";
               if (numArgs < 7) {
                  login_required = false;
                  if (numArgs < 6) {
                     dialog_behavior = false;
                     if (numArgs < 5) {
                        add_w3_banner = false;
                        if (numArgs < 4) {
                           add_close = true;
                           if (numArgs < 3) {
                              return;
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   // Now set the new style flag to pass to the new function.
   var style_flags = 0x0;
   if (add_close) {
      style_flags |= PL_ADD_CLOSE_LINK;
   }
   if (add_w3_banner) {
      style_flags |= PL_ADD_BANNER;
   }
   if (dialog_behavior) {
      style_flags |= PL_DIALOG_BEHAVIOR;
      style_flags |= PL_FORM;
   }
   if (login_required) {
      style_flags |= PL_LOGIN_REQUIRED;
   }
   if (mouse_over_out_support) {
      style_flags |= PL_MOUSE_OVER_SUPPORT;
   }
   if (align_left_right) {
      style_flags |= PL_SUGGESTED_POS_LEFT;
   }

   // Now pass the modified parms to the new function call.
   showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
}


//
// ENTRY POINT from web pages: hide the popup layer if one is showing.
//
// Optional parm:
// Parm 1 (do not notify SurfAid of this layer closing, default = false)
//
hidePopupLayer = function (ignore_surfaid) {
   if (pl_active_layer_obj != null) {
      var numArgs = hidePopupLayer.arguments.length;
      // Default SurfAid action is to log that the layer was canceled
      if (numArgs < 1) {
         ignore_surfaid = false;
      }
      // End any dragging that may be going on.
      pl_endLayerDrag();
      // Hid the bubble fin if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      // Hide the popup layer
      pl_active_layer_obj.style.visibility = "hidden";
      pl_active_layer_obj.style.left = 0;
      pl_active_layer_obj.style.top = 0;

      // 1.03 Set the zIndex so the layer is below the content.
      pl_active_layer_obj.style.zIndex = -1;

      // Restore the original html to the popup layer, unless this is a layer
      // which will be submitting a form and therefore can not have its html reset,
      // or the form submission stops.
      if (!(pl_active_style_flags & PL_FORM)) {
         if (pl_active_layer_html != "") {
            // No need to restore the HTML if in a bubble since we cloned the original layer and used the clone.
            if (!(pl_active_style_flags & PL_DISPLAY_IN_BUBBLE)) {
               pl_active_layer_obj.innerHTML = pl_active_layer_html;
            }
            pl_active_layer_html = "";
         }
      } else {
         // So this is a form.
         // Clear secure hidden IFrame if set. There was no need to clear them if we restored the inner html above,
         // which would have cleared the hidden IFrame as well.
         if (login_iframe_active) {
            if (pl_active_IFrame_id != "") {
               var secure_iframe_obj = document.getElementById(pl_active_IFrame_id);
               if (secure_iframe_obj != null) {
                  secure_iframe_obj.innerHTML = "";
               }
            }
         }
         // If the form is displayed in a bubble, we want to copy the cloned layer contents to the
         // original layer so that any changes to form controls will be present the next time the form is displayed.
         if (pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) {
            if ((pl_active_clonedlayer_obj != null) && (pl_active_contentclone_obj != null)) {
               pl_active_clonedlayer_obj.innerHTML = pl_active_contentclone_obj.innerHTML;
            }
         }
      }

      login_iframe_active = false;
      pl_active_layer_id = "";
      pl_active_trigger_id = "";
      pl_active_IFrame_id = "";
      pl_clicked_layer = false;
      pl_active_has_dragged = false;

      // Reset or clear event handlers.
      // Generic handlers for all types of layers.
      document.onkeypress = pl_onkeypress_doc_prevEH;
      pl_onkeypress_doc_prevEH = "";
      window.onresize = pl_onresize_win_prevEH;
      pl_onresize_win_prevEH = "";
      window.onunload = pl_onunload_win_prevEH;
      pl_onunload_win_prevEH = "";
      // Handlers for layers that support mouse over and out for display
      if (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT) {
         pl_active_trigger_obj.onmouseout = "";
         pl_active_trigger_obj.onblur = "";
         pl_active_trigger_obj.onmouseover = pl_onmouseover_trigger_prevEH;
         pl_onmouseover_trigger_prevEH = "";
         pl_active_trigger_obj.onfocus = pl_onfocus_trigger_prevEH;
         pl_onfocus_trigger_prevEH = "";
         pl_active_layer_obj.onmouseout = "";
         pl_active_layer_obj.onmouseover = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onmouseover = "";
            pl_active_mybubblefin.onmouseout = "";
         }
      } else {
         // Handlers for layers that support clicking for display
         document.onclick = pl_onclick_doc_prevEH;
         pl_onclick_doc_prevEH = "";
         pl_active_trigger_obj.onclick = pl_onclick_trigger_prevEH;
         pl_onclick_trigger_prevEH = "";
         pl_active_layer_obj.onclick = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onclick = "";
         }
      }
      // Handlers for layers that support drag/drop.
      if (pl_active_drag_obj != null) {
         document.body.onmousemove = pl_onmousemove_body_prevEH;
         pl_onmousemove_body_prevEH = "";
         document.body.onmouseup = pl_onmouseup_body_prevEH;
         pl_onmouseup_body_prevEH = "";
         pl_active_drag_obj.onmousedown = "";
         pl_active_drag_obj.onlosecapture = "";
         pl_active_drag_obj.style.cursor = pl_active_drag_obj_prevcursor;
         pl_active_drag_obj_prevcursor = "default";
         pl_active_drag_obj = null;
      }

// Do not reset queued items here as there may be valid queued requests when the current popup layer is hidden.
//      pl_queued_show_layer_request = false;
//      pl_queued_hide_layer_request = false;
//      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 8");
//      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 9");

      // Clear out the global objects.
      pl_active_target_obj = null;
      pl_active_trigger_obj = null;
      pl_active_layer_obj = null;
      pl_active_clonedlayer_obj = null;
      pl_active_contentclone_obj = null;
      pl_active_style_flags = 0x0;

      // Finally, notify SurfAid if necessary
      if ((pl_active_surfaid_base_string != "") && (!ignore_surfaid)) {
         sa_onclick(pl_active_surfaid_base_string);
      }
      pl_active_surfaid_base_string = "";
   }
}


//
// ENTRY POINT from web pages: return the object in the popup layer with the specified ID.
// Special function needed to return the field of the passed ID when included in a bubble layer.
// Also works when not in a bubble, but the field can be called directly using document.getElementById in that case.
//
getPopupLayerElementById = function (id) {
   var obj;
   var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
   obj = pl_findChildNode(mybubblecontent, id);
   if (obj == null) {
      obj = document.getElementById(id);
   }
   return obj;
}


// -----------------------------------------------------------------------------
//
// Local functions not called externally (pl_ prefix)
//
// -----------------------------------------------------------------------------

//
// Position the layer.
//
pl_positionPopupLayer = function () {

   var left_set = false;
   var top_set = false;
   var index = -1;
   var pos_left = false;
   var pos_right = false;
   var pos_below= false;
   var pos_above = false;
   var bubble_fin_space = 0;
   var left_layer_left;
   var left_offscreen;
   var right_layer_left;
   var right_offscreen;
   var below_layer_top;
   var below_offscreen;
   var above_layer_top;
   var above_offscreen;
   var offscreen = 0;
   var aOffScreen;
   var offset_for_bubble = 0;
   // Amount to adjust the fin from the corner of the target
   var finpos_indent = 0;

   // Return value of whether or not to show the fin
   var show_fin = false;

   // Global positioning attributes - assign to local variables here for convenience.
   var layer = pl_active_layer_obj;
   var target = pl_active_target_obj;
   var suggested_pos = (pl_active_style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT));
   var force_pos = ((pl_active_style_flags & PL_FORCE_POSITION) != 0x0);
   var force_visible_target = ((pl_active_style_flags & PL_FORCE_VISIBLE_TARGET) != 0x0);
   var display_in_bubble = ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) != 0x0);

   // Default position is below the target.
   if ((suggested_pos != PL_SUGGESTED_POS_LEFT) && (suggested_pos != PL_SUGGESTED_POS_RIGHT) && (suggested_pos != PL_SUGGESTED_POS_ABOVE) && (suggested_pos != PL_SUGGESTED_POS_BELOW)) {
      suggested_pos = PL_SUGGESTED_POS_BELOW;
   }

   var target_parent_offsetTop = 0;
   var target_parent_offsetLeft = 0;
   var target_parent = target.offsetParent;
   // Work up the object offset hierarchy, adding up the offsets up to the body tag
   while (target_parent != null) {
// 1.11 Keep going past the BODY tag in case the body itself has a margin, even though w3 and onePlace pages do not.
//      if (target_parent.tagName == "BODY") {
//         break;
//      }
      target_parent_offsetTop += target_parent.offsetTop;
      target_parent_offsetLeft += target_parent.offsetLeft;
      target_parent_offsetTop -= target_parent.scrollTop;   // MH-75
      target_parent_offsetLeft -= target_parent.scrollLeft; // MH-75
      target_parent = target_parent.offsetParent;
   }
   var client_width = document.documentElement.clientWidth;
   var client_height = document.documentElement.clientHeight;
   var client_v_scrolled = document.documentElement.scrollTop;
   var client_h_scrolled = document.documentElement.scrollLeft;
   var layer_left = -1;
   var layer_top = -1;
   var amt_hidden_below = 0;
   var amt_hidden_above = 0;
   var layer_height;
   var layer_width;
   // If we are in a bubble, then we need to run the positioning calcs based on the original size of the
   // bubble layer without considering any increases in width or height we may make later to accomodate the fin
   // if the original layer is not larget enough for the fin to fit on the desired side.  This comes into play
   // on resizes of the window which may change which side the fin appears on.
   if ((display_in_bubble) && (pl_bubble_orig_width > 0)) {
      layer_width = pl_bubble_orig_width;
   } else {
      layer_width = layer.offsetWidth;
      pl_bubble_orig_width = layer_width;
   }
   if ((display_in_bubble) && (pl_bubble_orig_height > 0)) {
      layer_height = pl_bubble_orig_height;
   } else {
      layer_height = layer.offsetHeight;
      pl_bubble_orig_height = layer_height;
   }

   // First we calculate all 4 positions (for left and right, if doesn't fit going down try again going up
   // and choose the best). We store how much it is off the screen if any for that axis.

   // Leave extra space for the fin if we are displaying the layer in a bubble.
   if (display_in_bubble) {
      bubble_fin_space = pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET;
   }

   // Left
   left_layer_left = target.offsetLeft + target_parent_offsetLeft - layer_width - bubble_fin_space;
   left_offscreen = Math.max(0, client_h_scrolled - left_layer_left);
   // Not allowed to be all the way off the left if we can't move it back over the target
   if (force_visible_target && (left_offscreen > client_h_scrolled)) {
      left_offscreen = pl_BIG_OFFSCREEN;
   }

   // Right
   right_layer_left = target.offsetLeft + target_parent_offsetLeft + target.offsetWidth + bubble_fin_space;
   right_offscreen = Math.max(0, (right_layer_left + layer_width) - (client_width + client_h_scrolled));

   // Below
   below_layer_top = target.offsetTop + target_parent_offsetTop + target.offsetHeight + bubble_fin_space;
   below_offscreen = Math.max(0, (below_layer_top + layer_height) - (client_height + client_v_scrolled));

   // Above
   above_layer_top = target.offsetTop + target_parent_offsetTop - layer_height - bubble_fin_space;
   above_offscreen = Math.max(0, client_v_scrolled - above_layer_top);
   // Not allowed to be all the way off the top if we can't move it back over the target
   if (force_visible_target && (above_offscreen > client_v_scrolled)) {
      above_offscreen = pl_BIG_OFFSCREEN;
   }

   // If we are forcing the suggested position to be used, use it even if if doesn't fit well.
   if (force_pos) {

      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         layer_left = left_layer_left;
         left_set = true;
         pos_left = true;
         break;
      case PL_SUGGESTED_POS_RIGHT:
         layer_left = right_layer_left;
         left_set = true;
         pos_right = true;
         break;
      case PL_SUGGESTED_POS_BELOW:
         layer_top = below_layer_top;
         top_set = true;
         pos_below = true;
         break;
      case PL_SUGGESTED_POS_ABOVE:
         layer_top = above_layer_top;
         top_set = true;
         pos_above = true;
         break;
      }
      // Since forced to be where it wants to be, always show the fin.
      if (display_in_bubble) {
         show_fin = true;
      }

   } else {
      // Else we are sugesting a desired placement, but choose the first that fits.  The order we check for a fit depends on
      // which was the suggested pos.
      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         aOffScreen = [left_offscreen, right_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 0:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 1:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_RIGHT:
         aOffScreen = [right_offscreen, left_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 1:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 0:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_BELOW:
         aOffScreen = [below_offscreen, above_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 0:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 1:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_ABOVE:
         aOffScreen = [above_offscreen, below_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 1:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 0:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;
      }
      // Now if we wanted to force that the target be visible, we are done now, even if the layer is off the screen a bit.
      // Otherwise we need to move the layer back onto the screen if it is off screen.
      if (force_visible_target) {
         if (display_in_bubble) {
            show_fin = true;
         }
      } else {
         // Are we off the screen? Then bring it back on.
         // If we wanted a bubble but have to adjust the layer because it was off the screen, and we therefore
         // are moving closer to the target than optimal, we don't get to display the bubble fin.
         if (offscreen > 0) {
            if (pos_left) {
               layer_left += offscreen;
            } else if (pos_right) {
               layer_left -= offscreen;
            } else if (pos_below) {
               layer_top -= offscreen;
            } else if (pos_above) {
               layer_top += offscreen;
            }
         } else {
            // If we were not off the screen, and we want a bubble, then we can display the fin.
            if (display_in_bubble) {
               show_fin = true;
            }
         }
      }
      // Now one last check to ensure that the layer is not off the screen in the opposite direction of what was
      // calculated when there is room to move it fully on. But such a move removes the space for the fin, if any.
      var offscreen_opposite = 0;
      if (pos_left) {
         offscreen_opposite = layer_left + layer_width - (client_width + client_h_scrolled);
         if ((layer_left > client_h_scrolled) && (offscreen_opposite > 0)) {
            layer_left -= Math.min(layer_left - client_h_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_right) {
         offscreen_opposite = client_h_scrolled - layer_left;
         if (offscreen_opposite > 0) {
            layer_left += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_below) {
         offscreen_opposite = client_v_scrolled - layer_top;
         if (offscreen_opposite > 0) {
            layer_top += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_above) {
         offscreen_opposite = layer_top + layer_height - (client_height + client_v_scrolled);
         if ((layer_top > client_v_scrolled) && (offscreen_opposite > 0)) {
            layer_top -= Math.min(layer_top - client_v_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      }
   }

   // Now we need to ensure the bubble is large enough for the fin to fit on the decided side by forcing the
   // necessary dimension to be able to accomodate the fin.  We then need to manually adjust the layer height
   // and width variables because IE does not reflect the changes immediately if we query them after we
   // make these changes.
   if (display_in_bubble) {
      if (pos_left || pos_right) {
         pl_active_bubble_minheight_cell.innerHTML = pl_BUBBLE_CONTENT_MINHEIGHT_HTML;
         pl_active_bubble_minwidth_cell.innerHTML = "";
         layer_height = Math.max(layer_height, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_HEIGHT));
      } else if (pos_below || pos_above) {
         pl_active_bubble_minwidth_cell.innerHTML = pl_BUBBLE_CONTENT_MINWIDTH_HTML;
         pl_active_bubble_minheight_cell.innerHTML = "";
         layer_width = Math.max(layer_width, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_WIDTH));
      }
   }

   // So we now have the key positioning, left/right/below/above, and we need to slide/position it along the
   // opposite axis now.

   // If we positioned it to the left or the right, we need to position it vertically to line up with
   // the target as best we can.  It may extend down from the top of the target, or extend up from the bottom
   // of the target.
   if (pos_left || pos_right) {
      if (display_in_bubble) {
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetHeight / 2) + .25));
         var layer_top2 = 0;
         var bubblefin_top = 0;
         offset_for_bubble = Math.min(layer_height * pl_IDEAL_EAST_WEST_FIN_POSITION, layer_height - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_HEIGHT)
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop - offset_for_bubble + finpos_indent, client_v_scrolled);

         // If layer off the top, move it on
         if (layer_top < client_v_scrolled) {
            layer_top = client_v_scrolled;
         // Else if layer off the bottom, move it on but keep it on the top as well
         } else if ((layer_top + layer_height) > (client_height + client_v_scrolled)) {
            layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
         }

         // Now handle the case where the layer is being pushed up by the edge of the window, and there is not enough height under the
         // target to fit the fin. In this case we want to move the layer above the target so the fin can point downward, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetTop + target_parent_offsetTop + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT > layer_top + layer_height) {
            layer_top2 = Math.min(layer_top, target.offsetTop + target_parent_offsetTop + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_top2 != layer_top) {
               if ((layer_top2 + layer_height) > (client_height + client_v_scrolled)) {
                  layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
               } else if (layer_top2 >= client_v_scrolled) {
                  layer_top = layer_top2;
               } else {
                  show_fin = false;
               }
            }
         }

         // Now where do we position the bubble fin, if any?
         if (show_fin) {
            // If the bubble is not aligned with the target, no fin to show.
            if ((layer_top + pl_BUBBLE_CORNER_HEIGHT > target.offsetTop + target_parent_offsetTop + target.offsetHeight - finpos_indent) || (layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT < target.offsetTop + target_parent_offsetTop + finpos_indent)) {
               show_fin = false;
            } else {
               if (pos_left) {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_ED;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_EU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left + layer_width - pl_BUBBLE_FIN_INSET + "px";
               } else {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WD;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left - (pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET) + "px";
               }
            }
         }
      // Else not displayed in bubble.
      } else {
         // Default top position is aligned with the top of the object but can't be off the top of the screen.
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop, client_v_scrolled);
         amt_hidden_below = Math.max(0, (layer_top + layer_height) - (client_height + client_v_scrolled));
         if (amt_hidden_below > 0) {
            layer_top = Math.min(target.offsetTop + target_parent_offsetTop + target.offsetHeight - layer_height, client_height + client_v_scrolled - layer_height);
            amt_hidden_above = Math.max(0, (client_v_scrolled - layer_top));
            if (amt_hidden_above > 0) {
               // If the layer is bigger than the client area, top-justify it.
               if (client_height <= layer_height) {
                  layer_top = client_v_scrolled;
               } else {
                  // Else popup layer will fit fully in client, so move it up until it all shows.
                  // Move it the smallest amount from the two default positions.
                  if (amt_hidden_below > amt_hidden_above) {
                     // If more is hidden below, the top-justify the layer
                     layer_top = client_v_scrolled;
                  } else {
                     // Else more is hidden above, so slide the layer up so its bottom is on the screen.
                     layer_top = target.offsetTop + target_parent_offsetTop - amt_hidden_below;
                  }
               }
            }
         }
      }

   // If we positioned it below or above the target, we need to position it horizontally. We try to line the left side
   // of the layer with the left side of the target as best we can.
   } else if (pos_below || pos_above) {
      var layer_left2 = 0;
      var bubblefin_left = 0;
      if (display_in_bubble) {
         offset_for_bubble = Math.min(layer_width * pl_IDEAL_NORTH_SOUTH_POSITION, layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetWidth / 2) + .25));
      }
      // Default left position is left-aligned with the object
      layer_left = target.offsetLeft + target_parent_offsetLeft - offset_for_bubble + finpos_indent;
      // But if it extends off the screen to the right, right-justify it
      if (layer_left + layer_width > client_width + client_h_scrolled) {
         layer_left = client_width + client_h_scrolled - layer_width ;
      }
      // But if it now extends off the screen to the left, left-justify it so you can read the beginning of the text
      layer_left = Math.max(client_h_scrolled, layer_left);

      // Now if it is in a bubble, we slide it to further to the left if we can, then attach the fin unless not needed.
      // If we are displaying in a bubble, reposition the layer if possible so it is offset so the fin (if needed) will
      // not be on the end of the layer unless that moves it off the glass.
      if (display_in_bubble) {
         // Now handle the case where the layer is being pushed to the left by the edge of the window, and there is not enough width under the
         // target to fit the fin. In this case we want to move the layer to the left of the target so the fin can point to the right, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetLeft + target_parent_offsetLeft + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH > layer_left + layer_width) {
            layer_left2 = Math.min(layer_left, target.offsetLeft + target_parent_offsetLeft + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_left2 != layer_left) {
               if ((layer_left2 + layer_width) > (client_width + client_h_scrolled)) {
                  layer_left = Math.max(client_h_scrolled, client_width + client_h_scrolled - layer_width);
               } else if (layer_left2 >= client_h_scrolled) {
                  layer_left = layer_left2;
               } else {
                  show_fin = false;
               }
            }
         }
         if (show_fin) {
            // Can only show fin if layer is under or over the target to a degree.
            if ((target.offsetLeft + target_parent_offsetLeft + target.offsetWidth - finpos_indent < layer_left + pl_BUBBLE_CORNER_WIDTH) || (target.offsetLeft + target_parent_offsetLeft + finpos_indent > layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH)) {
               show_fin = false;
            // Now see where the layer ended out in relation to the target.
            // Is layer to the left of target by at least the width of the fin plus the table corner, but not all
            // the way past the target?  Then we will use the arrow pointing to the right.
            } else if ((target.offsetLeft + target_parent_offsetLeft + finpos_indent >= layer_left + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH) && (layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH >= target.offsetLeft + target_parent_offsetLeft + finpos_indent)) {
               bubblefin_left = target.offsetLeft + target_parent_offsetLeft + finpos_indent - pl_BUBBLE_FIN_BASE;
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px"
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NR;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SR;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            // Else we use the arrow pointing to the left, as long as it fits and will still point to the target
            } else {
               bubblefin_left = Math.max(target.offsetLeft + target_parent_offsetLeft + finpos_indent, layer_left + pl_BUBBLE_CORNER_WIDTH);
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px";
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NL;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SL;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            }
         }
      }
   }

   // Set the position of the popup layer
   layer.style.top = layer_top + "px";
   layer.style.left = layer_left + "px";

   // return whether the fin should be displayed if appropriate.
   return show_fin;
}


//
// Called by the layer positioning routine, it is passed in an array of values indicatng how much the layer is off the screen
// in the default position in the four directions. It returns the index of the first that is not off the screen, and if all
// are off the screen it returns the index of which is off the screen the least.
//
pl_getFirstPosition = function (aOffScreen) {
   var index = -1;
   var minOffScreen = -1;

   if (aOffScreen[0] == 0) {
      index = 0;
   } else if (aOffScreen[1] == 0) {
      index = 1;
   } else if (aOffScreen[2] == 0) {
      index = 2;
   } else if (aOffScreen[3] == 0) {
      index = 3;
   } else {
      // Else all are off the screen.  Find which is off the screen the least.
      if (aOffScreen[0] <= aOffScreen[1]) {
         minOffScreen = aOffScreen[0];
         index = 0;
      } else {
         minOffScreen = aOffScreen[1];
         index = 1;
      }
      if (aOffScreen[2] < minOffScreen) {
         minOffScreen = aOffScreen[2];
         index = 2;
      }
      if (aOffScreen[3] < minOffScreen) {
         minOffScreen = aOffScreen[3];
         index = 3;
      }
   }
   return index;
}


//
// Recurse through children nodes, looking for node with the passed in id.
//
pl_findChildNode = function (root, idDragger) {
   var x;
   if (root != null) {
      if (root.id == idDragger) {
         return root;
      }
      if (root.childNodes.length > 0) {
         var childnode;
         for (x = 0; x < root.childNodes.length; x++) {
            childnode = pl_findChildNode(root.childNodes[x], idDragger);
            if (childnode != null) {
               return childnode;
            }
         }
      }
   }
   return null;
}


//
// Preloads the images used by the layer so they wil display quickly when first needed.
//
pl_preloadPopupLayerImages = function () {
   if (!pl_bubble_images_preloaded) {
      pl_img_clearpix = new Image(1, 1);
      pl_img_clearpix.src = pl_CLEARPIX_FULLPATH;
      pl_img_banner_bg = new Image(pl_BANNER_BG_WIDTH, pl_BANNER_BG_HEIGHT);
      pl_img_banner_bg.src = pl_POPUPBANNER_FULLPATH;
      pl_img_close_x = new Image(pl_BANNER_CLOSE_X_WIDTH, pl_BANNER_CLOSE_X_HEIGHT);
      pl_img_close_x.src = pl_POPUPBANNERCLOSE_FULLPATH;

      pl_img_bubble_corner_nw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_nw.src = pl_BUBBLE_CORNER_FULLPATH_NW;
      pl_img_bubble_corner_ne = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_ne.src = pl_BUBBLE_CORNER_FULLPATH_NE;
      pl_img_bubble_corner_sw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_sw.src = pl_BUBBLE_CORNER_FULLPATH_SW;
      pl_img_bubble_corner_se = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_se.src = pl_BUBBLE_CORNER_FULLPATH_SE;

      pl_img_bubble_side_w = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_w.src = pl_BUBBLE_SIDE_FULLPATH_W;
      pl_img_bubble_side_e = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_e.src = pl_BUBBLE_SIDE_FULLPATH_E;
      pl_img_bubble_side_n = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_n.src = pl_BUBBLE_SIDE_FULLPATH_N;
      pl_img_bubble_side_s = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_s.src = pl_BUBBLE_SIDE_FULLPATH_S;

      pl_img_bubble_fin_nl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nl.src = pl_BUBBLE_FIN_FULLPATH_NL;
      pl_img_bubble_fin_nr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nr.src = pl_BUBBLE_FIN_FULLPATH_NR;
      pl_img_bubble_fin_sl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sl.src = pl_BUBBLE_FIN_FULLPATH_SL;
      pl_img_bubble_fin_sr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sr.src = pl_BUBBLE_FIN_FULLPATH_SR;
      pl_img_bubble_fin_eu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_eu.src = pl_BUBBLE_FIN_FULLPATH_EU;
      pl_img_bubble_fin_ed = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_ed.src = pl_BUBBLE_FIN_FULLPATH_ED;
      pl_img_bubble_fin_wu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wu.src = pl_BUBBLE_FIN_FULLPATH_WU;
      pl_img_bubble_fin_wd = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wd.src = pl_BUBBLE_FIN_FULLPATH_WD;

      pl_bubble_images_preloaded = true;
   }
}


//
// If there is a queued request to show the layer, do it now.
//
pl_showDelayedPopupLayer = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   if (pl_queued_show_layer_request) {
      showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
   }
}


//
// If there is a queued request to hide the layer, do it now.
//
pl_hideDelayedPopupLayer = function () {
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      hidePopupLayer()
   }
}


//
// If a timer is set, clear it.
//
pl_cancelTimeout = function (timer_id, str_debug) {
   if (timer_id != -1) {
      clearTimeout(timer_id);
   }
   return -1;
}


//
// Event handler for cancelling a queued hide request, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOver = function (event) {
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");

   }
   return true;
}


//
// Event handler for cancelling a queued hide request, set for the trigger object if displayed on mouse over.
//
pl_processTriggerMouseOverAndFocus = function (event) {
   // Reset the mouse out and blur event handlers for the trigger now that we are over it again.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = pl_processTriggerMouseOutAndBlur;
      event.target.onmouseout = pl_processTriggerMouseOutAndBlur;
   } else if (event.srcElement) {
      event.srcElement.onblur = pl_processTriggerMouseOutAndBlur;
      event.srcElement.onmouseout = pl_processTriggerMouseOutAndBlur;
   }
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOut = function (event) {
   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 6");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the trigger object if displayed on mouse over or focus.
//
pl_processTriggerMouseOutAndBlur = function (event) {
   // When we get a blur or mouseout, we need to clear the objects event handler that caused it. We can't do
   // this on the close of the layer because blur and mouseover are set when a show is requested, and may
   // need to be cleared before the show takes place.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = "";
      event.target.onmouseout = "";
   } else if (event.srcElement) {
      event.srcElement.onblur = "";
      event.srcElement.onmouseout = "";
   }

   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 7");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for drag support, set for the drag object.
//
pl_processDragMouseDown = function (event) {
   if (event == null) {
      event = window.event
   }
   if (pl_isButton1(event)) {
      if ((event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
         // Store the initial pos of the mouse so we can know when it is moved enough to start the drag.
         pl_drag_mouse_down = true;
         pl_move_init_x = event.clientX;
         pl_move_init_y = event.clientY;
         // Return false if user clicks on the frag area so default drag action on an object in Firefox does not occur.
         return pl_setReturnValue(false, event);
      }
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseUp = function (event) {
   if (pl_drag_mouse_down) {
      pl_endLayerDrag();
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseMove = function (event) {
   if (event == null) {
      event = window.event
   }
   // If we are not dragging yet, and the mouse is down, have we moved enough to start the drag?
   if ((!pl_move_in_process) && (pl_drag_mouse_down) && (event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
      if ((Math.abs(pl_move_init_x - event.clientX) >= pl_DRAG_START_DIFFERENTIAL) || (Math.abs(pl_move_init_y - event.clientY) >= pl_DRAG_START_DIFFERENTIAL)) {
         // Record we are dragging now.
         pl_move_in_process = true;
         pl_active_has_dragged = true;
         // Store the initial offset of the mouse from the layer top-left, so we can move the layer with the mouse.
         pl_move_offset_x = pl_active_layer_obj.offsetLeft  - document.documentElement.scrollLeft - event.clientX;
         pl_move_offset_y = pl_active_layer_obj.offsetTop - document.documentElement.scrollTop - event.clientY;
         // Store the min position the layer can move up and left. so part of the drag area is always visible.
         var drag_x = pl_active_drag_obj.offsetLeft;
         var drag_y = pl_active_drag_obj.offsetTop;
         var drag_width = pl_active_drag_obj.clientWidth;
         var drag_height = pl_active_drag_obj.clientHeight;
         // Account for the drag area being the layer itself, or nested within the layer. We want the offset from the top left of the layer.
         if (pl_active_drag_obj.id == pl_active_layer_id) {
            drag_x = 0;
            drag_y = 0;
         } else {
            var parent = pl_active_drag_obj.offsetParent;
            while (parent != null) {
               if (parent.id == pl_active_layer_id) {
                  break;
               }
               drag_x += parent.offsetLeft;
               drag_y += parent.offsetTop;
               parent = parent.offsetParent;
            }
         }
         // Now we get the offset of the lower right corner of the drag area from the upper left of the layer, adjusted by how much we want to stay visible.
         var drag_x_right = Math.max(drag_x, drag_x + drag_width - pl_DRAG_VISIBILITY_BUFFER);
         var drag_y_bottom = Math.max(drag_y, drag_y + drag_height - pl_DRAG_VISIBILITY_BUFFER);
         // And now we calculate the min position of the layer so that the visibility buffer of the drag area is always visible upper left.
         pl_move_min_x = -drag_x_right;
         pl_move_min_y = -drag_y_bottom;
         // Capture the mouse to catch mouse ups when the mouse is outside the window.
         if (pl_active_drag_obj.setCapture) {
            pl_active_drag_obj.setCapture(true);
         }
      }
   }

   // If we are dragging, adjust the layer position
   if (pl_move_in_process && pl_isButton1(event) && (event != null) && (pl_active_layer_obj != null)) {
      pl_active_layer_obj.style.left = Math.max(pl_move_min_x, event.clientX + pl_move_offset_x + document.documentElement.scrollLeft) + "px";
      pl_active_layer_obj.style.top = Math.max(pl_move_min_y, event.clientY + pl_move_offset_y + document.documentElement.scrollTop) + "px";
      // Remove the bubble fin on a move if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      return pl_setReturnValue(false, event);
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the drag object, called when mouse capture is terminated.
//
pl_processDragLoseCapture = function (event) {
   pl_endLayerDrag();
   return pl_setReturnValue(true, event);
}


//
// Called when drag ends.
//
pl_endLayerDrag = function () {
   pl_move_in_process = false;
   pl_drag_mouse_down = false;
   pl_move_init_x = 0;
   pl_move_init_y = 0;
   pl_move_min_x = 0;
   pl_move_min_y = 0;
   pl_move_offset_x = 0;
   pl_move_offset_y = 0;
   if (document.releaseCapture) {
      document.releaseCapture();
   }
}


//
// Called to set return values during drag operations.
//
pl_setReturnValue = function (retval, event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      event.returnValue = retval;
//      if (event.cancelBubble) {
//         event.cancelBubble = retval;
//      }
//      if (!retval) {
//         if (event.stopPropagation) {
//            event.stopPropagation();
//         }
//         if (event.preventDefault) {
//            event.preventDefault();
//         }
//      }
   }
   return retval;
}


//
// Called to see if button 1 is down during drag operations.
//
pl_isButton1 = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      if (event.button) {
         if (event.button != 1) {
            return false;
         }
      } else {
         if (event.which) {
            if (event.which != 1) {
               return false;
            }
         }
      }
   }
   return true;
}


//
// Event handler: Set a flag that the popup layer was just clicked and send the click on to the general click handler
//
pl_processLayerClick = function () {
   pl_clicked_layer = true;
   return true;
}


//
// Event handler: Set a flag that the trigger was just clicked and send the click on to the general click handler
//
pl_processTriggerClick = function () {
   // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return false;
   }
   // Clicking on trigger when layer is showing gets rid if it, and does not pass click on to the body.
   if (pl_active_layer_obj != null) {
      // Unless the layer was shown using mouse over and out support, in which case click does not get rid of layer.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
      return false;
   }
   return true;
}


//
// Event handler: Cancel popup layer on a mouse click anywhere except on the popup layer itself
//
pl_processBodyClick = function (event) {
   if (!pl_isButton1(event)) {
      return true;
   }
   if (pl_active_layer_obj != null) {
      if (event == null) {
         event = window.event
      }
      // Do not dismiss the popup layer if it was the popup layer that was clicked
      if (pl_clicked_layer) {
         pl_clicked_layer = false;
         return true;
      }
      // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
      if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
         return false;
      }
      // If the mouse was released outside of the window, ignore this click event generated during a drag. Only IE will
      // handle this since Firefox does not capture mouse events outside the window.
      if ((event.clientX < 0) || (event.clientY < 0)) {
         return false;
      }
      if (document.documentElement) {
         if (document.documentElement.clientWidth && document.documentElement.clientHeight) {
            if ((event.clientX > document.documentElement.clientWidth) || (event.clientY > document.documentElement.clientHeight)) {
               return false;
            }
         }
      }
      // If the click came from the trigger, ignore it.
      if (event.target) {
         if (event.target.id == pl_active_trigger_id) {
            return false;
         }
      } else if (event.srcElement) {
         if (event.srcElement.id == pl_active_trigger_id) {
            return false;
         }
      }
      // Otherwise a click hides the layer unless the layer was shown using mouse over and out support.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
   }
   pl_clicked_layer = false;
   return true;
}


//
// Event handler: Cancel popup layer on ESC key
//
pl_processBodyKey = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event != null) {
      // Modal dialogs do not get dismissed with the ESC key.
      if ((event.keyCode == 27) && (!(pl_active_style_flags & PL_DIALOG_BEHAVIOR))) {
         hidePopupLayer();
      }
   }
   return true;
}


//
// Event handler: Reposition the popup layer on resize to stay aligned with target if it reflowed
//
pl_processResize = function () {
//   // Movable objects and modal objects do not get repositioned on window resize.
//   if ((pl_active_drag_obj != null) || (pl_active_style_flags & PL_DIALOG_BEHAVIOR)) {
   // Movable objects that have been moved by the user do not get repositioned on window resize.
   if ((pl_active_drag_obj != null) && pl_active_has_dragged) {
      return true;
   }
   // Reposition the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();
   if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
      if (show_fin) {
         pl_active_mybubblefin.style.visibility = "visible";
         pl_active_mybubblefin.style.zIndex = 3;
      } else {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
      }
   }
   return true;
}


//
// Event handler: Cancel popup layer if another page loads in the same window
//
pl_processWindowUnload = function () {
   hidePopupLayer();
   return true;
}
