/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.pages.edit");
dojo.require("dojox.dtl.widget");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit.TitlePane");
dojo.requireLocalization("hub", "popups");
dojo.requireLocalization("hub", "links");
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "messages");

/**
 * Save page shown at the end of the wizard. Fetches metadata about the entry
 * being saved from the server to pre-populate the form fields.
 */
dojo.declare('hub.pages.edit.Save', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Save.html"),
    entry_id: 0,

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	if(server_data.object.isSaved){
	    	hub.renamePage(this.tab, server_data.object.title, {});
    	}
        return {entry_id: this.entry_id,
        		legend: 'legend',
        		title: server_data.object.title,
        		description: server_data.object.description,
        		version: server_data.object.version,
        		tags: server_data.mytags, // where are tags?
        		doc: server_data.object.doc?server_data.object.doc:'',
        		useCache: server_data.object.useCache,
        		lifeTime: server_data.object.lifeTime,
		        panels: dojo.i18n.getLocalization('hub', 'panels'),
		        popups: dojo.i18n.getLocalization('hub', 'popups'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.entry_id);
    }
});

/**
 * Landing page shown after the save page with links to where the user can go
 * next. Fetches data from the server about the entry so it can build a
 * proper set of links based on the type (e.g., mashup versus widget).
 */
dojo.declare('hub.pages.edit.Landing', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Landing.html"),
   	entry_title:'',

    // Note:  objectType is from package.xml and it is not a translatable string
    //        feed, mashup, widget, page, etc.
     getObjectTypeForDisplay: function(objectType) {
       if (objectType == null) return this.labels.NEW_FEED_TAB;
       var result = objectType;

       if (objectType == 'mashup') {
          result = this.labels.NEW_MASHUP_TAB;
       } 
       else if (objectType == 'widget' || objectType == 'iwidget') {
          result = this.labels.NEW_WIDGET_TAB;
       }
       else if (objectType == 'page' || objectType == 'mashuppage') {
          result = this.labels.NEW_PAGE_TAB;
       }
       else {
          result = this.labels.NEW_FEED_TAB;
       }
       
       return result;
    },
    
    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.popups = dojo.i18n.getLocalization('hub', 'popups');
    	this.entry_title = server_data.object.title;
    	hub.renamePage(this.tab, this.entry_title, {});
    	var sourceString = (server_data.object.source!=null)?server_data.object.source.toLowerCase():"unknown";
    	var objectType = (server_data.object.objectType!=null)?server_data.object.objectType.toLowerCase():"object";
    	var otTitle = this.getObjectTypeForDisplay(objectType);
    	var otDisplayName = otTitle.toLowerCase();
		if (server_data.object.externalURL) {
			server_data.object.secureURL = hub.urls.secureUrl(server_data.object.externalURL);
		}
        return {entry_id: this.entry_id,
                plugin_id: this.plugin_id,
        		title: hub.escapeQuotes( server_data.object.title ),
        		landingTitle: dojo.string.substitute(this.panels.LANDING_TITLE, [otTitle]),
                subtitle: dojo.string.substitute(this.panels.LANDING_SUBTITLE, [otDisplayName, server_data.object.title]),
                taskGenerate: dojo.string.substitute(this.panels.LANDING_TASK_GENERATE, [otDisplayName]),
                taskDetails: dojo.string.substitute(this.panels.LANDING_TASK_DETAILS, [otDisplayName]),
                taskMetaData: dojo.string.substitute(this.panels.LANDING_TASK_EDIT_METADATA, [otDisplayName]),
                taskSource: dojo.string.substitute(this.panels.LANDING_TASK_EDIT_SOURCE, [otDisplayName]),
                taskCreateAnother: dojo.string.substitute(this.panels.LANDING_TASK_CREATE_ANOTHER, [otDisplayName]),
                taskUploadAnother: dojo.string.substitute(this.panels.LANDING_TASK_UPLOAD_ANOTHER, [otDisplayName]),
                taskGenerateTip: dojo.string.substitute(this.popups.LANDING_GENERATE, [otDisplayName]),
                taskDetailsTip: dojo.string.substitute(this.popups.LANDING_DETAILS, [otDisplayName]),
                taskMetaDataTip: dojo.string.substitute(this.popups.LANDING_EDIT_METADATA, [otDisplayName]),
                taskSourceTip: dojo.string.substitute(this.popups.LANDING_EDIT_SOURCE, [otDisplayName]),
                object: server_data.object,
                base_url: hub.config.base_url,
                editSourceUrl: hub.getCurrentTab().workflow==null || hub.getCurrentTab().workflow.pluginStartScreen==null?
                	'javascript:hub.showPlugin("'+hub.escapeQuotes( server_data.object.title )+'", '+this.plugin_id+', '+this.entry_id+', {editSource: true, current: true})':
                	'javascript:hub.getCurrentTab().workflow.showPlugin()',
                editInfoUrl:hub.getCurrentTab().workflow==null || hub.getCurrentTab().workflow.saveScreen==null?
                	'javascript:hub.showPlugin("'+hub.escapeQuotes( server_data.object.title )+'", '+this.plugin_id+', '+this.entry_id+', {current: true})':
                	'javascript:hub.getCurrentTab().workflow.showSave()',
                url:(server_data.object.hasPromptParams?
                	'javascript:hub.showUrl("Parameters","'+hub.urls.getParametersPage(this.entry_id,this.plugin_id)+'", {entryId: '+this.entry_id+'});':
                	server_data.object.secureURL),
                hasPromptParams: server_data.object.hasPromptParams,
                labels: this.labels,
                popups: this.popups,
                panels: this.panels};
    },

/*    postCreate: function(){
    	this.inherited('pstCreate', arguments);
//    	hub.renamePage(this.heading, this.entry_title, {});
    },*/

    /**
     * Gets an object indicating this page should no longer be permanent. The
     * save page has been submitted, so the user can now close the page without
     * giving confirmation.
     */
    getProperties: function() {
        return {permanent: false};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
//        return hub.urls.getLandingPage();
        return hub.urls.getDetailsPage(this.entry_id);
    }
});

/**
 * Feed chooser page shown when the user decides to create or register a new
 * feed. No server data needed.
 */
dojo.declare('hub.pages.edit.FeedChooser', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/FeedChooser.html"),

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function(response) {
        // @todo
        alert(dojo.toJson(response.payload.categories));
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                categories: response.payload.categories};

    },

    getUrl: function(){
    	return '../plugin/editor/source/Feeds';
    }
});

/**
 * Page collecting user name and password. Can be shown by plug-ins to get
 * credentials. Fetches any previously entered username and password from the
 * server.
 */
dojo.declare('hub.pages.edit.ProtectedResource', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/ProtectedResource.html"),

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getProtectedResourcePage();
    }
});

/**
 * Page collecting user name and password, plus database host, port, name, and
 * type. Can be shown by plug-ins. Fetches any previously defined and/or
 * selected databases from the server.
 */
dojo.declare('hub.pages.edit.DatabaseConnection', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/DatabaseConnection.html"),

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDatabaseConnectionPage();
    }
});
