/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.pages.main");
dojo.require("hub.pages");

/**
 * Login page. No server data to fetch.
 */
dojo.declare("hub.pages.Login", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Login.html"),

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        dojo.require('dijit.form.Button');
        dojo.require('hub.submit');
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Login page. Ajax request server for login url.
 */
dojo.declare("hub.pages.LoginPlease", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/LoginPlease.html"),

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {

		// get login URL
		var loginURL='#';

		dojo.xhrGet({
			url: hub.config.base_url + "/ui/loginURL",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				loginURL = response.loginURL;
			}
		});

        return {link: loginURL,
        		panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Account registration page. No server data to fetch.
 */
dojo.declare("hub.pages.Register", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Register.html"),

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        dojo.require('dijit.form.Button');
        dojo.require('hub.submit');
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                popups: dojo.i18n.getLocalization('hub', 'popups')};;
    }
});

/**
 * Tag and category clouds page. No need to fetch server data for the template
 * because the widgets will use the global tags and categories models.
 */
dojo.declare("hub.pages.Clouds", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Clouds.html"),

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        dojo.require('dijit.form.Slider');
        dojo.require('hub.widgets.social.view');
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                popups: dojo.i18n.getLocalization('hub', 'popups')};;
    }
});


/**
 * Catalog display page. Consists mostly of other widgets. No need to fetch
 * server data for the template because the widgets will fetch it themselves.
 */
dojo.declare("hub.pages.Catalog", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Catalog.html"),
    // default catalog query
    query: 'all',
    // default catalog query search term
    term: '*',
    collection: 'all',
    searchType: "",
    sortBy: "",
    sortOrder: "",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        // modules required to render all subwidgets
        dojo.require('hub.data.catalog');
        dojo.require('hub.data.social');
        dojo.require('hub.widgets.catalog');
        dojo.require('hub.widgets.social.view');

        // return object with data to interpolate into context
        // @note: uses the page's dojo assigned id to help uniquely name
        //   elements in the template
        return {id: this.id,
                collection: this.collection,
                term: this.term,
                searchType: this.searchType,
                sortBy: this.sortBy,
                sortOrder: this.sortOrder,
                panels: dojo.i18n.getLocalization('hub', 'panels'),
                links: dojo.i18n.getLocalization('hub', 'links'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Home page. Fetches some data from the server to complete links for launching
 * plug-in editors.
 */
dojo.declare("hub.pages.Home", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Home.html"),
    pluginsIds: {},

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the homepage.
     */
    getUrl: function() {
        return hub.urls.getHomePage();
    },

    /**
     * Gets an object indicating this page should be permanent.
     */
    getProperties: function() {
        return {permanent: true};
    },

    getPluginIds: function(plugins){
    	for(plugin in plugins){
//    		alert(plugins[plugin].name + ' = ' + plugins[plugin].id);
    		this.pluginsIds[plugins[plugin].name]=plugins[plugin].id;
    	}

    	return this.pluginsIds;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
//    	alert(dojo.toJson(server_data.plugins));
//    	alert(dojo.toJson(this.getPluginIds(server_data.plugins)));
        var obj = {logged_in: hub.config.username != '',
            labels: dojo.i18n.getLocalization('hub', 'labels'),
            panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            pluginIds: this.getPluginIds(server_data.plugins),
            img_catalog: hub.urls.getImage('catalogIcon2.gif'),
            img_creation: hub.urls.getImage('creationIcon2.gif'),
            img_start: hub.urls.getImage('gettingStartedIcon2.gif'),
            img_community: hub.urls.getImage('communityIcon2.gif')};

        // merge server data into context as-is (no name changes)
        // assumes JSONified object from server has properties matching those
        // needed by the template (else, we'd have to manually add them all to
        // obj under the proper names)
        return dojo.mixin(obj, server_data);
    },
    
    onRendered: function(){
    	this.inherited('onRendered', arguments);
    	this.tab.setStatic(true);
    }
});

/**
 * Details page. Fetches metadata about a catalog entry from the server given
 * an object ID to complete the rendering of the template.
 */
dojo.declare("hub.pages.Details", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Details.html"),
    // show details of this item
    object_id: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	this.tab.entryId = this.object_id;
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.edit');
        dojo.require('hub.submit.social');
        dojo.require('hub.widgets.social.edit');
        dojo.require('hub.widgets.social.view');
        dojo.require('hub.data.social');

		var gurl = "#";
		var orig_title = ""; 
		
		if (server_data.object) {
			// hub.escapeQuotes
			if (server_data.object.title){
				orig_title = server_data.object.title;
				server_data.object.title = hub.escapeQuotes(server_data.object.title);
			}
			if (server_data.object.externalURL) {
				server_data.object.secureURL = hub.urls.secureUrl(server_data.object.externalURL);
			}
			if (server_data.object.hasPromptParams) {
				gurl = 'javascript:hub.showUrl("Parameters","' + hub.urls.getParametersPage(server_data.object.id, server_data.object.pluginid) + '", {entryId: '+server_data.object.id+'});';
			} else {
//				gurl = hub.urls.getPluginGenerate(server_data.object.id, server_data.object.pluginid);
				gurl = server_data.object.externalURL ;
			}
			if ( gurl == "#" ) {
				gurl = server_data.object.url;
			}
		}
        var obj = {
        	objectid: this.object_id,
        	baseurl: hub.config.base_url,
        	generateURL: gurl,
        	secureURL: hub.urls.secureUrl(gurl),
        	trackedDataURL: hub.config.base_url+"/tracking/generate/entryid/"+this.object_id,
			addtomashupsURL: server_data.object?mm.createAddToMashupsLink(server_data):"", //Add To Mashups link
			title: orig_title,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon_feed: hub.urls.getImage('feedLarge.gif'),
            icon_widget: hub.urls.getImage('widgetLarge.gif'),
            icon_flow: hub.urls.getImage('mashupLarge.gif'),
            icon_page: hub.urls.getImage('pageLarge.gif')
            };

        return dojo.mixin(obj, server_data);
    },
    
    submitComment: function(){
		var form = dojo.byId("hub_details_user_comment_" + this.object_id);
		var msg = dojo.byId("saveCommentMessage_" + this.object_id);
		hub.submit.ajaxSubmit(new hub.submit.social.Comment(form, msg, this.object_id));
    },

    onRendered: function(){
    console.log('onRendered');
    	this.inherited('onRendered', arguments);
		dojo.connect(dijit.byId('postCommentButton_'+this.object_id), 'onClick', dojo.hitch(this, this.submitComment));
    	console.dir({button: dijit.byId('postCommentButton_'+this.object_id)});
    }
});

/**
 * Page with a form accepting parameters to generate a feed. Fetches parameter
 * names, types, default values, etc. from the server.
 */
dojo.declare("hub.pages.Parameters", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Parameters.html"),
    // parameters for this item
    object_id: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getParametersPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    }
});
dojo.declare('hub.pages.Save', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/Save.html"),

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
        return {id: 'object_id', legend: 'legend', name: 'empty', description: '', version: '0.5', tags: 'tags',
		        panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
//        return hub.urls.getSavePage();
		return null;
    }
});

/**
 * Settings page.
 */
dojo.declare("hub.pages.AdminTables", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminTables.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return null;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Page for reinitializing the database and sample data. No server data to 
 * fetch.
 */
dojo.declare('hub.pages.AdminMigrateDatabase', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminMigrateDatabase.html"),

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');
        
        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
            panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Page for deleting arbitrary items. Fetches name, id, etc. of all items.
 */
dojo.declare('hub.pages.AdminObjects', hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminObjects.html"),

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminDeletePage();
    }
});

/**
 * Settings page.
 */
dojo.declare("hub.pages.AdminProxy", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminProxy.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminProxyPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    }
});

/**
 * Permissions page.
 */
dojo.declare("hub.pages.AdminPermissions", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminPermissions.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminPermissionsPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Tracking page.
 */
dojo.declare("hub.pages.AdminTracking", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminTracking.html"),
    widgetsInTemplate: true,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEditTrackingPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Tracking data page.
 */
dojo.declare("hub.pages.AdminTrackingData", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminTrackingData.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminTrackingData();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
        	baseurl: hub.config.base_url,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon: hub.urls.getIcon('btn16_feed.gif'),
            base_img_url: hub.urls.getImage('')
            };

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Bans page.
 */
dojo.declare("hub.pages.AdminBannedUsers", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminBannedUsers.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminBanPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Categories page.
 */
dojo.declare("hub.pages.AdminCategories", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminCategories.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminCategoryPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

       var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    }
});

/**
 * Admin Report Emails page.
 */
dojo.declare("hub.pages.AdminReportEmails", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminReportEmails.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEmailPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
        	currentUser: hub.config.userfullname,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Report Abuse page.
 */
dojo.declare("hub.pages.ReportAbuse", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/ReportAbuse.html"),
    entryId: null,
    title: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return null;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	this.tab.entryId = this.entryId;
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
        	currentUser: hub.config.userfullname,
        	id: this.entryId,
        	title: this.title,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
	        links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * AdminTDI page.
 */
dojo.declare("hub.pages.AdminTDI", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.pages", "templates/AdminTDI.html"),

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminTDI();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        dojo.require('hub.managers.Urls');
        dojo.require('hub.submit');
        dojo.require('hub.submit.admin');

        var obj = {
        	baseurl: hub.config.base_url,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon: hub.urls.getIcon('btn16_feed.gif'),
			icon_feed: hub.urls.getImage('feedLarge.gif')
            };

        return dojo.mixin(obj, server_data);
    }
});

/**
 * SelectMashupsServer page.
 */
dojo.require("mm.pages.SelectMashupsServer");
dojo.declare("hub.pages.SelectMashupsServer", mm.pages.SelectMashupsServer, {
});

/**
 * AddFeedToMashups page. 
 */
dojo.require("mm.pages.AddFeedToMashups");
dojo.declare("hub.pages.AddFeedToMashups", mm.pages.AddFeedToMashups, {
});

/**
 * AddIWidgetToMashups page. 
 */
dojo.require("mm.pages.AddIWidgetToMashups");
dojo.declare("hub.pages.AddIWidgetToMashups", mm.pages.AddIWidgetToMashups, {
});

/**
 * AddMashupPageToMashups page.
 */
dojo.require("mm.pages.AddMashupPageToMashups");
dojo.declare("hub.pages.AddMashupPageToMashups", mm.pages.AddMashupPageToMashups, {
});

/**
 * FinishedAddToMashups page.
 */
dojo.require("mm.pages.FinishedAddToMashups");
dojo.declare("hub.pages.FinishedAddToMashups", mm.pages.FinishedAddToMashups,{
});

