/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.submit');
dojo.require('hub.main');
dojo.require("dojo.i18n");
dojo.requireLocalization("hub", "messages");
		dojo.require("hub.table.model");
		dojo.require("hub.table.table");
		dojo.require("hub.table.navigation");
		dojo.require("hub.data.jsonmodel");
dojo.require("hub.widgets.adminobjectscatalog");

dojo.declare('hub.submit.Request', null,{
	content: {},
    constants : {
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_LOGIN_REQUIRED : 1,
        ERROR_ADMIN_REQUIRED : 2,
        ERROR_BANNED: 16,
        ERROR_ACCESS_DENIED: 18,
        ERROR_DATA_COMP: 21
    },
    
    response: {},
    
    /**
     * @param form: Form or JS object to submit
     * @param status: Message area where error / server messages are posted
     * @param url: Post to this URL, or use it to form a full URL after calling
     *   getUrl()
     */
    constructor : function(url, content ,status) {
        // get reference to status element
        if(content){
        	this.content = content; 
        }
        if(status) {
            if(dojo.isString(status)) {
                this.status = dojo.byId(status);
            } else {
                this.status = status;
            }
        } else {
            this.status = null;
        }
        
        // store something as the URL
        if(url != null) {
            // use provided url
            this.url = url;
        }
    },

    getStatus: function() {
        return this.status;
    },

    getURL : function() {
        return this.url;
    },
    
    getContent : function() {
        return this.content;
    },
    
    /*
     * Submits the Form using dojo.xhrPost. 
     * 
     * @param validate: If false, does not perform validation.
     */
    submit: function(validate) {
        // indicate busy
///		hub.busyPage(this.status, true, false);
	      
		// build a request object and handler for dojo
		var kw = {
		    url: this.getURL(),
		    content: this.getContent(),
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
            sync: true,
            
            error: dojo.hitch(this, function(response, args) {
                // get error messages
                var msgs = dojo.i18n.getLocalization('hub', 'messages');
                
                if(response.dojoType == "cancel") {
                    hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.getStatus());
                } else if (response.dojoType == "timeout"){
                    hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.getStatus());
                } else {
                    var err = msgs.ERROR_REQUEST_SERVER+" "+response.message;
                    hub.notify(err, this.getStatus());
                }
                // indicate no longer busy
///		        hub.busyPage(this.status, false, false);
                return response;
            }),
            
            load: dojo.hitch(this, function(response, args) {
				try {
                    if(!response) {
                        throw new Error('Form: Empty server response');
                    }
                    if(hub.submit.processResponseCode(response)){
				    	this._onLoad(response);
				    }
				} catch(e) {
                    var msgs = dojo.i18n.getLocalization('hub', 'messages');
                    var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
                    hub.notify(err, this.getStatus());
				}
	                // indicate no longer busy
///		        hub.busyPage(this.status, false, false);
                return response;
            })
		};
	        // do the post
		dojo.xhrPost(kw);
		return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        this.response = data;
        if (!data) {
            throw new Error('Form: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
//            hub.notify(data.result, this.status, 'TOM_editor_msg');
        }/* else {
           	console.log('submit _onload 2');
            hub.notify(data.result, this.status);
        }*/
        return data.code;
    }
});
/**
 * Defines a base class for form validation, submission using XMLHTTPRequest,
 * notification when the request is complete, and basic handling of result
 * messages. This base class posts all form data and uses hub.notify to output
 * any message returned with a server result.
 */
dojo.declare('hub.submit.Form', null, {
    constants : {
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_DATA_COMP: 21
    },
    
    content: null,
    form: null,
    
    /**
     * @param form: Form or JS object to submit
     * @param status: Message area where error / server messages are posted
     * @param url: Post to this URL, or use it to form a full URL after calling
     *   getUrl()
     */
    constructor : function(form, status, url) {
        // get form or data object
        if(dojo.isString(form)){
            form = dojo.byId(form);
        }
        this.form = form;
        
        // get reference to status element
        if(status) {
            if(dojo.isString(status)) {
                this.status = dojo.byId(status);
            } else {
                this.status = status;
            }
        } else {
            this.status = null;
        }
        
        // store something as the URL
        if(url != null) {
            // use provided url
            this.url = url;
        } else if(this.form) {
            // if url is not defined, set from action attribute
            this.url = form.action;
        }
    },

    getStatus: function() {
        return this.status;
    },

    /**
     * Do a manual step of converting a form into a JS object. This way,
     * the Form base class can submit both form data and JS objects passed
     * manually.
     */
    getContent: function() {
    	if(this.content!=null){
    		return this.content;
    	}else{
	    	var widgets = dojo.query('div[id^="hub_widgets"]', this.form);
	    	var data = new Array();
	    	for(var i=0; i<widgets.length; i++){
	    		var didget = dijit.byId(widgets[i].id);
	    		if(didget!=null && didget.getFormData!=null && dojo.isFunction(didget.getFormData)){
			    	data = dojo.mixin(data, didget.getFormData());
		    	}
	    	}
    		
	        try {
		 	    return dojo.mixin(data, dojo.formToObject(this.form));
	        } catch(e) {
	            return dojo.mixin(data, this.form);
	        }
        }
        return null;
    },
        
    /*
     * Submits the Form using dojo.xhrPost. 
     * 
     * @param validate: If false, does not perform validation.
     */
    submit: function(validate) {
    console.log('Submit.submit()');
	// make sure the result is valid first
	if(validate && !this.validate()) {
	    hub.busyPage(this.form, false, false);
	    return false;
	}
        // indicate busy
	hub.busyPage(this.form, true, true);
      
	// build a request object and handler for dojo
	console.dir({data:this.getContent()});
	var kw = {
	    url: this.getURL(),
	    content: this.getContent(),
        handleAs: 'text', // let the callback decode
        timeout: 120000, // 2 minutes in milliseconds
        
        error: dojo.hitch(this, function(response, args) {
            // get error messages
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            
            if(response.dojoType == "cancel") {
                hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.getStatus());
            } else if (response.dojoType == "timeout"){
                hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.getStatus());
            } else {
                var err = msgs.ERROR_REQUEST_SERVER+" "+response.message;
                hub.notify(err, this.getStatus());
            }
            // indicate no longer busy
		    hub.busyPage(this.form, false, false);
            return response;
        }),
        
        load: dojo.hitch(this, function(response, args) {
			try {
	            if(!response) {
	                throw new Error('Form: Empty server response');
	            }
				if(hub.submit.processResponseCode(response)){
					this._onLoad(response);
				}
			} catch(e) {
	            var msgs = dojo.i18n.getLocalization('hub', 'messages');
	            var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
	            hub.notify(err, this.getStatus());
			}
 	        // indicate no longer busy
		    hub.busyPage(this.form, false, false);
            return response;
       })
	};
        // do the post
	dojo.xhrPost(kw);
	return true;
    },
          
    /**
     * Validates a required form field to ensure it is not blank, null, or
     * contains only whitespace. Gives the field focus if it is invalid.
     * 
     * @param field Form element to validate
     * @param text Notify the user with this text if invalid
     */
    _validateRequired : function(field, text) {
        with (field) {
	    if ((value == null) || (value == "")) {
	        hub.notify(text, this.status);
                field.focus();
	        return false;
	    }
	    value = value.replace(/^\s+|\s+$/, '');
	    if(value.length < 1) {
	        hub.notify(text, this.status);
	        return false;
	    }
	    return true;
        }
    },
    
    /**
     * Validates a field having a maximum and minimum length.
     * 
     * @param field Form element to validate
     * @param max Maximum required length, null if none
     * @param min Minimum required length, null if none
     * @param text Notify the user with this text if invalid
     */
    _validateLength : function(field, min, max, text) {
        with (field) {
	    value = value.replace(/^\s+|\s+$/, '');
	    if(max != null && value.length > max) {
                field.focus();
                hub.notify(text, this.status);
	        return false;
	    }
	    if(min != null && value.length < min) {
                field.focus();
                hub.notify(text, this.status);
	        return false;
	    }
        }
        return true;
    },
    
    getURL : function() {
        return this.url;
    },
    
    validate : function() {
        return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('Form: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits data required to login.
 */
dojo.declare('hub.submit.Login', hub.submit.Form, {
    constructor : function(form, message_id, url, from_page) {
        this.from_page = from_page;
    },
    getURL : function() {
        return hub.urls.postLogin();
    },
    
    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(login,
                msgs.ERROR_LOGIN_LOGIN, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(password,
                msgs.ERROR_LOGIN_PASSWORD, this.status)) {
	        return false;
	    }
        }
        return true;
    },

    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Login: empty response');
        } else if(data.code < 0) {
            // welcome the user
            var username = data.result;

            if(this.from_page) {
                // close the login page, if open
                hub.closePage(this.form);
            }

            // notify other components
            dojo.publish("hub.login", [username]);
            
            // clear any previous error
            hub.notify("", this.status);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits a new user registration.
 */
dojo.declare('hub.submit.Register', hub.submit.Form, {
    getURL : function() {
        return hub.urls.postRegister();
    },
    
    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(registerName,
                msgs.ERROR_REGISTER_NAME, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerEmail,
                msgs.ERROR_REGISTER_EMAIL, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerLogin,
                msgs.ERROR_REGISTER_LOGIN, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerPassword,
                msgs.ERROR_REGISTER_PASSWORD, this.status)) {
	        return false;
	    }
            if (registerPassword.value != registerPassword2.value) {
                hub.notify(msgs.ERROR_REGISTER_PASSWORD_MATCH, this.status);
                registerPassword.value = "";
                registerPassword2.value = "";
                registerPassword.focus();
                return false;
            }
        }
        return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Register: empty response');
        } else if(data.code < 0) {
            // welcome the user
            var username = data.result;
            // close the registration page
            hub.closePage(this.form);
            // notify other components
            dojo.publish('hub.login', [username]);
        } else {	
            hub.notify(data.result, this.status);
        }
        return data.code;
    }   
});


/**
 * Submits a request to add or remove an entry as a favorite
 */
dojo.declare('hub.submit.Favorites', hub.submit.Favorites, {
    // URL will be provided by caller, not hardcoded
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Favorites: empty response');
        } else if (data.code < 0) {
            // success, switch the label
            this.status.innerHTML = data.result;
            // notify other components
            dojo.publish('hub.update-favorites');
        } else {
            // IMPORTANT: purposely not giving the message field since
            // it's pointing at the link, not at an error area; always
            // use the popup alert
            hub.notify(data.result, null);
        }
        return data.code;
    }
});

/**
 * Submits a request to logout.
 */
dojo.declare('hub.submit.Logout', hub.submit.Form, {
    validate: function() {
        // try to close all pages first
        var d = hub.closeAllPages();
        var _self = this;
        // listen to deferred result to continue the submit
        d.addCallback(function(success) {
            if(success) {
                // resend this form without validating again
                hub.submit.ajaxSubmit(_self, false, true);
            } else {
                // @todo: seems like we should cleanup here, form or deferred?
            }
        });
        // never succeed right away, reinvoke submit later if all pages closed
        return false;
    },
    
    getURL : function() {
        return hub.urls.postLogout();
    },
    
    _onLoad : function(response, tab) {
        dojo.require("dojo.cookie");
        // alphaWorks support code
        // redirect to alphaWorks main site after logout
        if (dojo.cookie('IBM_WI_Username') != null) {
	    dojo.cookie('IBM_WI_Username', '', {expires: -1});
	    dojo.cookie('IBM_WI_Password', '', {expires: -1});
//	    window.location='http://aws01.alphaworks.ibm.com/mhrun/WSLogin.php';
	    window.location='/';
	}
        
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Logout: empty response');
        } else if(data.code < 0) {
            // notify components that logout completed
            dojo.publish('hub.logout');
        } else {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        }
        return data.code;
    }
});

hub.submit.processResponseCode = function(response){
//	console.log('hub.submit.processResponseCode: ');
    var constants = {
        ERROR_UNKNOWN: 0,
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_LOGIN_REQUIRED : 1,
        ERROR_ADMIN_REQUIRED : 2,
        ERROR_BANNED: 16,
        ERROR_ACCESS_DENIED: 18,
        ERROR_DATA_COMP: 21
    }
    var data = dojo.isString(response)?dojo.fromJson(response):response;
//	console.dir({response: data});
    var code = data.code;
    var result = true;
    if(code != null){
	   	if(dojo.isString(code)){
	   		code = parseInt(code); 
	   	}
	   	result = true;
	   	switch(code){
			case constants.ERROR_LOGIN_REQUIRED :
				hub.showPage('Login', 'LoginPlease', {current: true});
				result = false;
				break; 
			case constants.ERROR_ADMIN_REQUIRED :
			//	todo: processing
				console.warn('missing error processing')
				break; 
				result = false;
			case constants.ERROR_BANNED:
			//	todo: processing
				console.warn('missing error processing')
				result = false;
				break;
			case constants.ERROR_DATA_COMP:
				hub.notify('Data error on server side');
				result = false;
				break;
			case constants.ERROR_ACCESS_DENIED:
			    hub.notify('Access Denied');
			    result = false;
			    break;		
			case constants.ERROR_UNKNOWN :
				hub.notify('Unknown error on server side');
				result = true;
				break; 
			case constants.NO_CONTENT : 
			case constants.PAGE_CONTENT : 
			case constants.MESSAGE_CONTENT :
			default:
				break;
	   	}
   	}
   	return result;
};

/**
 * Simple helper function that invokes submit on the given form object and
 * either returns a value of false or undefined to prevent a refresh of the 
 * entire browser when used on an HTML form or HTML link respectively.
 *
 * @param hub.submit.Form: Form object to do validation and handle the response
 * @param no_result: If true, returns undefined instead of false; useful for 
 *   links
 * @param no_validate: If true, skips validation
 */
hub.submit.ajaxSubmit = function(form, no_result, no_validate) {
    try {
        form.submit(!no_validate);
    } catch (e) {
        hub.notify(e, form.getStatus());
    }
    return no_result ? undefined : false;
}
