/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.submit.admin');
dojo.require('hub.submit');
dojo.require('hub.widgets.tabs');
dojo.require('dojo.i18n');
dojo.require("hub.widgets.adminobjectscatalog");
dojo.requireLocalization("hub", "messages");

/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.TrackingSettingsForm', hub.submit.Form, {
/*
    getURL : function() {
        return hub.urls.postAdminTrackingSettings();
    },
*/
    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(trackingHost,
	            msgs.ERROR_TRACKING_HOST, this.status)) {
		    return false;
		}
		if (!this._validateRequired(trackingVender,
	            msgs.ERROR_TRACKING_VENDOR, this.status)) {
		    return false;
		}
		if (!this._validateRequired(trackingVpk,
	            msgs.ERROR_TRACKING_VENDOR_KEY, this.status)) {
		    return false;
		}
        }
        return true;
    }
});

/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.GlobalStatusForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('GlobalStatusForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.AdminTrackingPermissionsForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('AdminTrackingPermissionsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.ObjectsDeleteForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('ObjectsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	//var tab = hub.widgets.tabs.getTabForNode(this.form);
            dojo.publish('hub.delete-entry', [data.objects]);
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to reinitialize the database.
 */
dojo.declare('hub.submit.admin.InitForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('InitForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) { 
            // now notify about reset of the entire system
            dojo.publish('hub.reset');
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submit form for the category form in the admin settings.
 */
dojo.declare('hub.submit.admin.CategoryForm', hub.submit.Form, {
    validate : function() {
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new TOM.Exception();
        } else if (data.code == TOM.editor.Form.MESSAGE_CONTENT) {
            tab.refresh();
            // notify other components
            dojo.publish("hub.update-categories");
            TOM.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            TOM.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to reinitialize the database.
 */
dojo.declare('hub.submit.admin.ProxyForm', hub.submit.Form, {
    validate: function() {
        // try to close all pages first
        var d = hub.closeAllPages();
        var _self = this;
        // listen to deferred result to continue the submit
        d.addCallback(function(success) {
            if(success) {
                // resend this form without validating again
                hub.submit.ajaxSubmit(_self, false, true);
            } else {
                // @todo: seems like we should cleanup here, form or deferred?
            }
        });
        // never succeed right away, reinvoke submit later if all pages closed
        return false;
    },
          
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('ProxyForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) { 
            // now notify about reset of the entire system
            // dojo.publish('hub.reset');
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to set admins permissions.
 */
dojo.declare('hub.submit.admin.PermissionsForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('PermissionsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.BannedForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('BannedForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.CategoriesForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            dojo.publish("hub.update-categories");
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to send a email.
 */
dojo.declare('hub.submit.admin.ReportForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to change email list.
 */
dojo.declare('hub.submit.admin.EmailListForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits tdi configuration.
 */
dojo.declare('hub.submit.admin.TDIForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('TDIForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits connection info for database migration. 
 */
dojo.declare('hub.submit.admin.MigrateForm', hub.submit.Form, {
    _onLoad: function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('MigrateForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits tracking date to clear from.
 */
dojo.declare('hub.submit.admin.ClearTrackedDataForm', hub.submit.Form, {

    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(trackingDate,
	            msgs.ERROR_TRACKING_CLEAR_DATE, this.status)) {
		    return false;
		}
		with(trackingDate) {
			//do some regex validation
			if(!value.match("^([0-9]){2}(-){1}([0-9]){2}(-){1}([0-9]){4}$")) {
			    hub.notify(msgs.ERROR_TRACKING_CLEAR_FORMAT, this.status);
			    return false;
			}
		}
        }
        return true;
    }
});