/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.submit.edit');
dojo.require('hub.submit');
dojo.require('dojo.i18n');
dojo.requireLocalization("hub", "messages");
dojo.requireLocalization("hub", "labels");
    
/**
 * Submits metadata required to save an entry.
 */
dojo.declare('hub.submit.edit.Save', hub.submit.Form, {
    // URL will be provided by caller, not hardcoded
    validate : function() {
		console.log('hub.submit.edit.Save.validate');
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
		    if (!this._validateRequired(title,
	                msgs.ERROR_SAVE_TITLE, this.status)) {
		        return false;
		    }
		    if (!this._validateRequired(description,
	                msgs.ERROR_SAVE_DESCRIPTION, this.status)) {
		        return false;
		    }
		    if (!this._validateRequired(version, 
	                msgs.ERROR_SAVE_VERSION, this.status)) {
		        return false;
		    }
        }
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Save: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            // make sure the title of the tab is up to date
            // @todo: unsafe, user might change after submit
            tab = hub.tabs.moveTab(tab, this.form.title.value);
            // notify other components
            dojo.publish("hub.save-entry", [object_id]);
            
            // set a flag indicating we're at the end of the workflow;
            // don't destroy it though in case the user wants to go back
            // and continue editing
            tab.getAttribute('workflow').setComplete();
            
            // go to the landing page
            var object_id = tab.getAttribute('object_id');
            hub.showPage(this.form.title.value, 'Landing', 
                         {object_id: object_id, current: true, safe: true});
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits data to select a new entry type.
 */
dojo.declare('hub.submit.edit.New', hub.submit.Form, {

	workflow: null,
	plugin_id: 0,

    constructor : function(url, workflow) {
    	this.url = url;
    	this.workflow = workflow;
    	this.plugin_id = workflow.plugin_id;
    	this.content={pluginid: workflow.plugin_id};
    	this.form = workflow.plugin_dedicated?null:workflow.selectPluginForm.domNode;
    },
        
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('New: empty response');
        } else if(data.code < 0) {
            var labels = dojo.i18n.getLocalization('hub', 'labels');

            // generate a reasonable title
            var title = labels.UNTITLED_TAB + " " + data.result.object_id;
           	this.workflow.lastResponse = data;
           	this.workflow.entry_id = data.result.object_id;
           	hub.renamePage(this.workflow.tab, title);
           	this.workflow.plugin = new hub.managers.InvokePlugin(this.plugin_id, this.workflow.entry_id, this.workflow);
	        this.workflow.appendScreenHTML(this.workflow.lastResponse.result['html']);
           	dojo.publish('workflow-next-'+this.workflow.tab.id ,[]);
        } else {
            // error, show the message
            hub.notify(data.result, this.status);
        }
        
        return data.code;
    }
});

/**
 * Submits a request for deletion without confirming it.
 */
dojo.declare('hub.submit.edit.DeleteAlways', hub.submit.Form, {
    _onLoad : function(response, tab) {    
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('DeleteAlways: empty response');
        } else {
            if (data.code < 0) {
	            // success, close the tab
	            try {
	                hub.closePage(dojo.byId( this.status ));
	            } catch(e) {
	                // tab might already be closed
	            }
	        } else {
	            hub.notify(data.result, this.status, 'TOM_editor_msg');
	        }
	    }
        // fire an event so all tabs refresh
        dojo.publish('hub.delete-entry', [data.entryid]);
        return data.code;
	}
});

/**
 * Submits a request for deletion without confirming it.
 */
dojo.declare('hub.submit.edit.Cancel', hub.submit.Request, {
    _onLoad : function(response, tab) {    
        return -5;
	}
});

/**
 * Submits a deletion request after confirm it with the user.
 */
dojo.declare('hub.submit.edit.DeleteConfirm', hub.submit.edit.DeleteAlways, {
	getContent: function() {
		return "";
	},
    // URL will be provided by caller, not hardcoded
    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return confirm(msgs.PROMPT_DELETE);
    }
});

/**
 * Submits a request to toggle the favorite status.
 */
dojo.declare('hub.submit.edit.FavoritesForm', hub.submit.Form, {
	getContent: function() {
		return "";
	},
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('FavoritesForm: empty response');
	    }
// TODO just change text content this.status
        if (data.code < 0) {
        	var tab = hub.widgets.tabs.getTabForNode(this.status);
            tab.refresh();
	    } else {
//			hub.notify(data.result, this.status, 'TOM_editor_msg');
		}
        return data.code;
	}
});
