/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.submit.social');
dojo.require('hub.submit');
dojo.require('dojo.i18n');
dojo.requireLocalization("hub", "messages");

/**
 * Submits a rating for an entry.
 */
dojo.declare('hub.submit.social.Rating', hub.submit.Form, {
    getURL: function() {
        return hub.urls.postRating(this.url);
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Rating: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            // notify other components, url holds the object id
            dojo.publish("hub.rate-entry", [this.url]);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits tags for an entry.
 */
dojo.declare('hub.submit.social.Tags', hub.submit.Form, {
    getURL: function() {
        return hub.urls.postTags(this.url);
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Tags: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
            // notify other components, url holds the object id
            dojo.publish("hub.tag-entry", [this.url]);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a comment for an entry.
 */
dojo.declare('hub.submit.social.Comment', hub.submit.Form, {
    getURL : function() {
        return hub.urls.postComment(this.url);
    },

    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(commentTitle, 
                msgs.ERROR_COMMENT_TITLE, this.status)) {
	        return false;
	    }
	    if (!this._validateLength(commentText, 1, 500,
                msgs.ERROR_COMMENT_BODY, this.status)) {
	        return false;
	    }
        }
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Comment: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
            // notify other components
            dojo.publish("hub.comment-entry", [this.url]);
            this.form.reset();
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Handles validation of the user profile form.
 */
dojo.declare('hub.submit.social.Profile', hub.submit.Form, {
    methods : {
        getURL : function() {
            return hub.urls.postProfile();
        },

        validate : function() {
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            with (this.form) {
	        if (!this._validateRequired(profileName,
                    msgs.ERROR_REGISTER_NAME, this.status)) {
	            return false;
	        }
	        if (!this._validateRequired(profileEmail,
                    msgs.ERROR_REGISTER_EMAIL, this.status)) {
	            return false;
	        }
                // blank password ok, means no change
                if (profilePassword.value != profilePassword2.value) {
                    hub.notify(msgs.ERROR_REGISTER_PASSWORD_MATCH, this.status);
                    profilePassword.value = "";
                    profilePassword2.value = "";
                    profilePassword.focus();
                    return false;
                }
            }
            return true;
        },

        _onLoad : function(response, tab) {
            var code = this.inherited(arguments);
            if(code < 0) {
                // notify other components
                dojo.publish("hub.update-profile", [this.form.profileName.value]);
            }
            return code;
        }
    }
});

