/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.table.model");
dojo.require("dijit.dijit");

/**
 * Abstaract interface for table model.
 */
dojo.declare("hub.table.model.Model", null, {

	/**
	 * Planned page size, how many records it was planned for this table,
	 * this attribute value may be used as a hint, how many rows should be
	 * preallocated.
	 */
	getPlannedPageSize: function() {
		throw new Error("hub.table.model.Model.getPlannedPageSize() is abstract");
	},
	
	/**
	 * Size of current page, pricipally equivalent of getCurrentPage().size. But 
	 * may work quicker.
	 */
	getCurrentPageSize: function() {
		throw new Error("hub.table.model.Model.getCurrentPageSize() is abstract");
	},
	
	/**
	 * Returns row of the table, by row number.
	 */
	getRow: function(rowNum) {
		throw new Error("hub.table.model.Model.getRow(n) is abstract");
	},
	
	getSupportedSortings: function() {
		throw new Error("hub.table.model.Model.getSupportedSortings() is abstract");
	},
	
	getGuaranteedRowCount: function() {
		throw new Error("hub.table.model.Model.getGuaranteedRowCount() is abstract");
	},
	
	hasMoreRecords: function() {
		throw new Error("hub.table.model.Model.hasMoreRecords() is abstract");
	},
	
	getCurrentPage: function() {
		throw new Error("hub.table.model.Model.getCurrentPage() is abstract");
	},

	getAvailablePages: function() {
		throw new Error("hub.table.model.Model.getAvailablePages() is abstract");
	},
	
	setPageToLoad: function(page) {
		throw new Error("hub.table.model.Model.setPageToLoad(page) is abstract");
	},
	
	getPageToLoad: function() {
		throw new Error("hub.table.model.Model.getPageToLoad() is abstract");
	},
	
	setSortingToApply: function(sorting) {
		throw new Error("hub.table.model.Model.setSortingToApply(s) is abstract");
	},
	
	getSortingToApply: function() {
		throw new Error("hub.table.model.Model.getSortingToApply() is abstract");
	},
	
	getCurrentSorting: function() {
		throw new Error("hub.table.model.Model.getCurrentSorting() is abstract");
	},
	
	fetch: function() {
		throw new Error("hub.table.model.Model.fetch() is abstract");
	},
	
	isFetched: function() {
		throw new Error("hub.table.model.Model.isFetched() is abstract");
	},

	onBeforeFetched: function() {
		throw new Error("hub.table.model.Model.onBeforeFetched() is abstract");	
	},
	
	onFetched: function() {
		throw new Error("hub.table.model.Model.onFetched() is abstract");	
	},

	onFetchError: function(error) {
		throw new Error("hub.table.model.Model.onFetchError() is abstract");
	}
});

dojo.declare("hub.table.model.Page", null, {
	fromRow: null, // is not necessar
	toRow: null, // is not necessar
	size: 0, // will be toRow - fromRow in most case
	forwardAbsolute: null, // is not necessar
	backwardAbsolute: null, // is not necessar
	relative: null // is not necessar
});

dojo.declare("hub.table.model.Sorting", null, {
	by: null, // name of criterion to order by
	asc: true // sort ascenting
});

dojo.declare("hub.table.model.AbstractModel", [dijit._Widget, hub.table.model.Model], {
	pageToLoad: null,
	sortingToApply: null,
	currentSorting: null,
	constructor: function(args, node) {
		this.changesElements = {};
		this.isSelectedElements = {};
	},

	getPlannedPageSize: function() {
		return 0;
	},
	
	getCurrentPageSize: function() {
		return 0;
	},
	
	getRow: function(rowNum) {
		return null;
	},
	
	getSupportedSortings: function() {
		return new Array();
	},
	
	getGuaranteedRowCount: function() {
		return 0;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return null;
	},

	getAvailablePages: function() {
		return null;
	},
	
	setPageToLoad: function(page) {
		this.pageToLoad = page;
	},
	
	getPageToLoad: function() {
		return this.pageToLoad;
	},
	
	setSortingToApply: function(sorting) {
		this.sortingToApply = sorting;
	},
	
	getSortingToApply: function() {
		return this.sortingToApply;
	},
	
	getCurrentSorting: function() {
		return this.currentSorting;
	},
	
	onBeforeFetched: function() {
	
	},
	
	onFetched: function() {
	
	},

	onFetchError: function(error) {
		
	}
});

dojo.declare("hub.table.model.ItemizedModel", hub.table.model.AbstractModel, {
	rows: [],
	newRows: [],
	plannedSize: 0,
	lock: 0,
	fetched: false,
	constructor: function(args, node) {
		this.rows = [];
		this.plannedSize = 0;
	},
	
	onItem: function(l, item) {
		if (l == this.lock) {
			this.newRows.push(item);
			return true;
		} 
		else {
			return false;
		}
	},
	
	getRow: function(rowNum) {
		return this.rows[rowNum];
	},
	
	onPageBegin: function() {
		this.newRows = [];
		this.lock++;
		return this.lock;
	},
	
	onPageEnd: function(l, error) {
		if (l == this.lock) {
			if (error) {
				this.newRows = new Array();
				return true;
			}
			else {
				this.fetched = true;
				this.rows = this.newRows;
				this.plannedSize = Math.max(this.rows.length, this.plannedSize);
				return true;
			}
		}
		else {
			return false;
		}
	},
	
	isFetched: function() {
		return this.fetched;
	}
});

dojo.declare("hub.table.model.DataSelection", null, {
	getRowId: function(row) {
		if ("id" in row) {
			return row.id;
		}
		else if (row[0]) {
			return row[0];
		}
		else {
			throw new Error("row id is not defined");
		}
	},
	
	isDataSelected: function(row, area) {
		if ("selected" in row) {
			return (false != row.selected);
		}
		return false;
	}
});

dojo.declare("hub.table.model.NoSelection", hub.table.model.DataSelection, {
	getRowId: function(row) {
		return "";
	},
	
	isDataSelected: function(row, area) {
		return false;
	}
});

dojo.declare("hub.table.model.DebugModel", hub.table.model.ItemizedModel, {
	from: null,
	pageSize: 20,
	totalRecords: 111,
	fetched: false,

	constructor: function(args, node) {
	},

	getPlannedPageSize: function() {
		return this.pageSize;
	},
	
	getCurrentPageSize: function() {
		if (this.from == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},
	
	getSupportedSortings: function() {
		var sortings = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = 'value';
		sorting.asc = true;
		sortings.push(sorting);
		sorting = new hub.table.model.Sorting();
		sorting.by = 'value';
		sorting.asc = false;
		sortings.push(sorting);
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		if (this.from == null) {
			return null;
		}
		var page = new hub.table.model.Page();
		page.fromRow = this.from;
		page.toRow = Math.min(this.from + this.pageSize, this.totalRecords);
		page.size = page.toRow - page.fromRow;
		page.forwardAbsolute = this.from / this.pageSize; 
		// backwardAbsolute: null, // is not necessar
		page.relative = 0;
		return page;
	},

	getAvailablePages: function() {
		var pages = new Array();
		if (this.from != null) {
			var i = 0;
			for(i = 0; i < this.totalRecords; i = i + this.pageSize) {
				var page = new hub.table.model.Page();
				page.fromRow = i;
				page.toRow = Math.min(i + this.pageSize, this.totalRecords);
				page.size = page.toRow - page.fromRow;
				page.forwardAbsolute = i / this.pageSize; 
				// backwardAbsolute: null, // is not necessar
				page.relative = (i - this.from) / this.pageSize;
				pages.push(page);
			}
		}
		return pages;
	},
	
	fetch: function() {
		var from = 0;
		if (this.pageToLoad != null) {
			from = this.pageToLoad.fromRow;
		}
		var i = from;
		this.from = from;
		var sorting = this.sortingToApply;
		var asc = true;
		if (sorting != null) {
			asc = sorting.asc;
		}
		else {
			sorting = new hub.table.model.Sorting();
			sorting.asc = asc;
			sorting.by = 'value'; 
		}
		onBeforeFetched();
		var lock = this.onPageBegin();
		for(i = from; i < Math.max(from + this.pageSize, this.totalRecords); i++) {
			var row = {'value': i, 'id': i * i + 3};
			if (!asc) {
				row.value = this.totalRecords - i - 1;
			}
			row.id = row.value * row.value + 3
			this.onItem(lock, row);
		}
		if (this.onPageEnd(lock)) {
			this.currentSorting = sorting;
			this.pageToLoad = this.getCurrentPage();
			this.onFetched();
		}
	},
	
	onFetched: function() {
	},

	onBeforeFetched: function() {
	},
	
	onFetchError: function(error) {
		// called never
	}
});