/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.table.navigation");

/**
 * Represents navigation bar for table (normaly list of tabs, but may be other type of control.
 */
dojo.declare("hub.table.navigation.Navigation", [dijit._Widget, dijit._Templated], {
	/**
	 * Table model to listen to.
	 */
	model: null,
	
	/**
	 * Connection to model.
	 */
	_model_connects: [],
	
	constructor: function(args, node) {
		this._model_connects = new Array();
	},
	
    postMixInProperties: function() {
        if (dojo.isString(this.model)) {
        	// get model widget from id
        	this.model = dijit.byId(this.model);
        }
    },
    
	postCreate: function() {
        this.inherited(arguments);
        this.initialize();		
	},
	
	startup: function() {
		if (this.model != null) {
			this.fetched();
		}
	},
	
    /**
     * Sets new model.
     */
    setModel: function(newModel) {
    	this.model = newModel;
    	this.initialize();
		if (this.model != null) {
			this.fetched();
		}
    },
 
    /**
     * Initialize listeners.
     */
    initialize: function() {
   		dojo.forEach(this._model_connects, dojo.disconnect);
   		this._model_connects = new Array();
		if (this.model != null) {
			this._model_connects.push(dojo.connect(this.model, "onFetched", this, "fetched")); 
		}
    },
    
    destroy: function() {
    	dojo.forEach(this._model_connects, dojo.disconnect);
    	this._model_connects = new Array();
    },
    	
 	fetched: function() {
 		throw new Error("hub.table.navigation.Navigation is abstract");
 	}	
});

/**
 * Represents the simplest imaginable navigation with button to previous and button to the 
 * next pages.
 */
dojo.declare("hub.table.navigation.PrevNextNavigation", hub.table.navigation.Navigation, {
    templateString: '',
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.table", "templates/PrevNextNavigation.html"),
    widgetsInTemplate: true,

	/**
	 * Prev page ref. (if any)
	 */
	prevPage: null,
	
	/**
	 * Next page ref. (if any)
	 */
	nextPage: null,
	
	/**
	 * Current page ref. (if any)
	 */
	currentPage: null,
    
    /**
     * Adjust navigation controls to the state of model.
     */
	fetched: function() {
		if (this.model.isFetched()) {
			var pages = this.model.getAvailablePages();
			var i = 0;
			var currentPage = null;
			var prevPage = null;
			var nextPage = null;
			for(i = 0; i < pages.length; i++) {
				var page = pages[i];
				var relative = page.relative;
				if (relative == 0) {
					currentPage = page;
				}
				else if (relative == -1) {
					prevPage = page;
				}
				else if (relative == +1) {
					nextPage = page;
				}
			}
			this.currentPage = currentPage;
			this.prevPage = prevPage;
			this.nextPage = nextPage;
			this._updateContent();
		}
	},
	
	/**
	 * Updates content (buttons) of the navigation.
	 */
	_updateContent: function() {
		this.prevButton.setDisabled(this.prevPage == null);
		this.nextButton.setDisabled(this.nextPage == null);
		this.navigationStatus.innerHTML = this.getSummaryMarkup();
	},
	
    /**
     * Creates markup for summary text (selected record from XX to YY total ZZZ).
     */
    getSummaryMarkup: function() {
    	if (this.model == null) {
    		return "";
    	}
    	var currentPage = this.model.getCurrentPage();
    	if (currentPage != null) {
    		if (!this.msgs) {
    			this.msgs = dojo.i18n.getLocalization('hub', 'messages');
    		}
	        var start = currentPage.fromRow + 1;
	        var end = currentPage.toRow;
	        var total = this.model.getGuaranteedRowCount();
	        if (this.model.hasMoreRecords()) {
	        	total = "" + total + "+"; 
	        }
	        if (currentPage.size == 0) {
	        	return this.msgs.STATUS_CATALOG_NONE;
	        }
	        return dojo.string.substitute(this.msgs.STATUS_TABLE_SUMMARY, 
	                                          [start, end, total]);
	    }
	    return "";
    },	

	/**
	 * Called when prev button is activated.
	 */	
	_onPrev: function() {
		if ((this.prevPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.prevPage);
			this.model.fetch();
		} 
	},

    /**
     * Called when next button is activated.
     */
	_onNext: function() {
		if ((this.nextPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.nextPage);
			this.model.fetch();
		} 
	},
	
	/**
	 * Called when refresh button is activated.
	 */
	_onRefresh: function() {
		if ((this.currentPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.currentPage);
		}
		if (this.model != null) {
			this.model.fetch();
		}
	}
});
