/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.table.table');
dojo.require('dijit.dijit');
dojo.require('dijit.form._FormWidget');

/**
 * Generic table component with pagination, sorting and selection capabilities.
 */
dojo.declare('hub.table.table.Table', [dijit._Widget, dijit._Templated], {
	/**
	 * Table mode.
	 */ 
	model: null,
	
	/**
	 * Selection hook.
	 */
	selection: null,
	
	/**
	 * List of columns.
	 */
	columns: [],
	
	/**
	 * Rows
	 */
	rows: [],
	
	/**
	 * Should we reset to the first page when sorting is changed.
	 */
	resetOnSort: true,
	
	/**
	 * Selection area which will be used in setValue/getValue or null.
	 */
	valueSelectionArea: null,
	
	/**
	 * Selection area which will be "default". Clickable rows.
	 */
	defaultSelectionArea: "default",
	
	/**
	 * Default selection area mode.
	 */ 
	defaultSelectionAreaMode: 0,
	uniselectionReselectValue: true,
	selection: new hub.table.model.DataSelection(),
	changedElements: {},
	isSelectedElements: {},
	noDefaultSelectionTags: ['A'],
	templatePath: dojo.moduleUrl("hub.table", "templates/Table.html"),
	headersRendered: false,

    constructor: function(args, node) {
        // column metadata
        this.columns = new Array();
        // rows added to table so far for this query
        this.rows = new Array();
        this._model_connects = new Array();
        this._header_connects = new Array();
        this._cell_connects = new Array();
		this.changedElements = {};
		this.isSelectedElements = {};
		this.headersRendered = false;
    },

    postMixInProperties: function() {
        // urls for icons and such
        this.checkboxOnImageURL = hub.urls.getImage('checkboxOn.png');
        this.checkboxOffImageURL = hub.urls.getImage('checkboxOff.png');
        this.radioOnImageURL = hub.urls.getImage('radioOn.png');
        this.radioOffImageURL = hub.urls.getImage('radioOff.png');
    },
    
    destroy: function() {
    	this._model_connects = this._disconnect(this._model_connects);
    	this._header_connects = this._disconnect(this._header_connects);
    	this._cell_connects = this._disconnect(this._cell_connects);
    },
    
    /**
     * Sets model for the table.
     */
    setModel: function(newModel) {
    	this._model_connects = this._disconnect(this._model_connects);
    	this.model = null;
    	this.model = newModel;
    	this.initialize();
    },
    
    /**
     * Connects table to model hooks.
     */
    initialize: function() {
		if (this.model != null) {
			this._model_connects.push(dojo.connect(this.model, "onFetched", this, "fetched"));
			this._model_connects.push(dojo.connect(this.model, "onBeforeFetched", this, "beforeFetched"));
			this._model_connects.push(dojo.connect(this.model, "onFetchError", this, "fetchError")); 
		}
		if (this.model != null) {
			this.fetched();
		}
    },

	/**
	 * Called when model is fetched, or when it may be fetched (some initial state).
	 */
    fetched: function() {
        // clear out all table headers and body
        this.clearRows();

		if (!this.headersRendered) {
	        for(i in this.columns) {
	            var th = document.createElement('TH');
	            // title and base class
	            th.innerHTML = this.columns[i]['name'];
	            if (this.columns[i]['class']) {
		            dojo.addClass(th, this.columns[i]['class']);
		        }
		        if (this.columns[i]['sorting']) {
	                // make focusable
	                th.tabIndex = 0;
	                // mouse and key handler for sorting
	                this._header_connects.push(dojo.connect(th, 'onclick', this, '_sort'));
	                this._header_connects.push(dojo.connect(th, 'onkeypress', this, '_sort'));
	                // @todo: initialize first column sorting for now, need to be
	                //   smarter about what order the results are in
		        }
	            this.table_header.appendChild(th);
	        }
	        // footer status area, built here because we need to know how many 
	        // columns
	        if (this.table_footer) {
		        this.status_row = document.createElement('TD');
		        this.status_row.colSpan = this.columns.length;
		        this.status_row.className = 'note';
		        this.table_footer.appendChild(this.status_row);
		    }	        
	        this.headersRendered = true;
	    }

	    
        if ((this.model != null) && this.model.isFetched()) {
	        var currentSorting = this.model.getCurrentSorting();
	        if (currentSorting != null) {
	        	var j = 0;
		        for(var i in this.table_header.childNodes) {
		            var th = this.table_header.childNodes[i];
		            if ( (th.tagName) && ('TH' == th.tagName) ) {
		            	var sorting = this.columns[j]['sorting'];
						if (currentSorting.by == sorting) {
				           	if (currentSorting.asc) {
				               	dojo.addClass(th, 'selectedUp');
				               	dojo.removeClass(th, 'selectedDown');
				            }
				            else {
				               	dojo.addClass(th, 'selectedDown');
				               	dojo.removeClass(th, 'selectedUp');
			                }
			            }
			            else {
			                dojo.removeClass(th, 'selectedDown');
			                dojo.removeClass(th, 'selectedUp');
			            }
			            j = j + 1;
			        }
		        }
		    }
        	var i = 0;
        	var l = this.model.getCurrentPageSize();
        	for(i = 0; i < l; i++) {
        		var row = this.model.getRow(i);
        		this.addRow(row, i);
        	}
        	if (this.model.getCurrentPageSize() > 0) {
        		this._showNoStatus();
        	}
        	else {
        		this._showNoEntriesStatus();
        	}
			var area;
			for(area in this.changedElements) {
				var i;
				var count = this.model.getCurrentPageSize();
				for(i = 0; i < count; i++) {
					this._mergeSelected(i, area);
				}
			}
		}
    },
    
    /**
     * Called before request to model is done.
     */
    beforeFetched: function() {
    	this._showBusyStatus();
    },
    
    /**
     * Handles error situation in model.
     */
    fetchError: function(error) {
    	console.debug("fetchError: " + error);
    	this._showErrorStatus(error);
    },
	
	/**
	 * Adds convinient column.
	 */
	addColumn: function(id, name, sorting, buildFunction, clazz) {
        this.clearHeaders();
        this.clearRows();
        if (!clazz) {
        	clazz = id;
        }
		this.columns.push({
			'id': id,
			'name': name,
			'sorting': sorting,
			'buildFunction': buildFunction,
			'selectionArea': null,
			'selectionMode': null,
			'class': clazz
		});
		this.fetched();
	},
	
	/**
	 * Adds selection column.
	 */
	addSelectionColumn: function(id, name, selectionArea, mode, clazz) {
        this.clearHeaders();
        this.clearRows();
        if (!clazz) {
        	clazz = id;
        }
		this.columns.push({
			'id': id,
			'name': name,
			'sorting': null,
			'buildFunction': null,
			'selectionArea': selectionArea,
			'selectionMode': mode,
			'class': clazz
		});
		this.fetched();
	},
	
	/**
	 * Sets default selection area.
	 */
	setDefaultSelectionArea: function(area) {
		this.defaultSelectionArea = area;
	},
	
    /**
     * Clears all table headers.
     */
    clearHeaders: function() {
    	this._header_connects = this._disconnect(this._header_connects);
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_header.firstChild)  {
            this.table_header.removeChild(this.table_header.firstChild);
        }
        if (this.table_footer) {
	        while (this.table_footer.firstChild)  {
	            this.table_footer.removeChild(this.table_footer.firstChild);
	        }
	        this.status_row = null;
	    }
        this.headersRendered = false;
    },

    /**
     * Clears the body of the table.
     */
    clearRows: function() {
    	this._cell_connects = this._disconnect(this._cell_connects);
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_body.firstChild)  {
            this.table_body.removeChild(this.table_body.firstChild);
        };
        this.rows = [];
    },

    /**
     * Adds a single row to the table.
     * @param dict single row data keyed by column name:
     *  - json string in format {'col1': data1, 'col2': data2, ...}
     *  - json object
     */
    addRow: function(dict, rowNum) {
    	var row = dict;

    	// convert to json object data
		if(!(dict instanceof Object)) {
    		row = dojo.fromJson(dict);
    	}

        // create table row and columns
        var tr = document.createElement('TR');
        tr.setAttribute("rowNum", "" + rowNum);
        this._cell_connects.push(dojo.connect(tr, 'onclick', this, '_select'));
        this._cell_connects.push(dojo.connect(tr, 'onkeypress', this, '_select'));
        if ( (this.defaultSelectionArea != null) && this._isSelected(rowNum, this.defaultSelectionArea) ) {
        	dojo.addClass(tr, 'defaultSelected');
        }

        for(i in this.columns) {
        	var column = this.columns[i];
            var td = document.createElement('TD');
            if (column['class']) {
            	dojo.addClass(td, column['clazz']);
            }
            if (column['selectionArea'] == null) {
            	column['buildFunction'](td, row);
            }
            else {
            	td.tabIndex = 0;
            	td.setAttribute("selectionArea", column['selectionArea']);
            	var s = this._isSelected(rowNum, column['selectionArea']);
            	this._buildSelectionCell(td, column['selectionArea'], column['selectionMode'], s);
            }
            tr.appendChild(td);
        }

        // alternating row style
        if(this.rows.length % 2) {
            dojo.addClass(tr, 'alt');
        }

        this.table_body.appendChild(tr);
        this.rows.push(row);
    },

	/**
	 * Build selection cell.
	 */
	_buildSelectionCell: function(td, area, mode, s) {
		var imgOn = document.createElement('IMG');
		var imgOff = document.createElement('IMG');
		dojo.addClass(imgOn, "imgOn");
		dojo.addClass(imgOff, "imgOff");
		imgOn.src = this._is0__n(mode) ? this.checkboxOnImageURL : this.radioOnImageURL;
		imgOff.src = this._is0__n(mode) ? this.checkboxOffImageURL : this.radioOffImageURL;
		this._togleSelection(td, s);
		td.appendChild(imgOff);
		td.appendChild(imgOn);  
	},
	
	/**
	 * Toggles selection CSS classes for TD.
	 */
	_togleSelection: function(td, s) {
       	if (s) {
       		dojo.addClass(td, 'selected');
       		dojo.removeClass(td, 'unselected');
       	}
       	else {
       		dojo.addClass(td, 'unselected');
       		dojo.removeClass(td, 'selected');
       	}
	},
	
	/**
	 * Toggles default selection CSS classes for TR.
	 */
	_togleDefaultSelection: function(tr, s) {
       	if (s) {
       		dojo.addClass(tr, "defaultSelected");
       	}
       	else {
       		dojo.removeClass(tr, "defaultSelected");
       	}
	},
    
    /**
     * Called when sotring is activated.
     */
    _sort: function(event) {
        var src = event.target;

        // if key but not space, don't count as a sort
        if(event.charCode != 0 && event.charCode != null && event.charCode != dojo.keys.SPACE) {
            return;
        }
        
        var index = null;
        var asc = null;
        var th = null;
        // clear all styles
        for(var i in this.table_header.childNodes) {
            th = this.table_header.childNodes[i];
            if(src == th) {
                index = i;
                if(dojo.hasClass(th, 'selectedUp')) {
                    // switch from up to down
                    asc = false;
                } 
				else if(dojo.hasClass(th, 'selectedDown')) {
                    // switch from down to up
                    asc = true;
                }
                break;
            }
        }
        var selectedSorting = null;
        if (index != null) {
	        var sorting = this.columns[index]['sorting'];
	        var supportedSortings = this.model.getSupportedSortings();
	        var j = 0;
	        for(j = 0; j < supportedSortings.length; j++) {
	        	if (supportedSortings[j].by == sorting) {
	        		if ( (asc == null) || (asc == supportedSortings[j].asc) ) {
	        			selectedSorting = supportedSortings[j];
	        			break;
	        		}
	        	}
	        }
	        if (selectedSorting != null) {
		        for(var i in this.table_header.childNodes) {
		            th = this.table_header.childNodes[i];
		            if(src == th) {
		               	if (selectedSorting.asc) {
		                   	dojo.addClass(th, 'selectedUp');
		                   	dojo.removeClass(th, 'selectedDown');
		                }
		                else {
		                   	dojo.addClass(th, 'selectedDown');
		                   	dojo.removeClass(th, 'selectedUp');
		                }
		            }
		            else {
		            	dojo.removeClass(th, 'selectedDown');
		            	dojo.removeClass(th, 'selectedUp');
		            }
		        }
	        }
	        // fetch the newly sorted results from the server
	        // no start or max because a new sort means we're starting at
	        // the beginning again anyways
		}     

        if (selectedSorting != null) {
        	if (this.resetOnSort) {
        		// We start from the initial paging:
        		this.model.setPageToLoad(null);
        	}
        	this.model.setSortingToApply(selectedSorting);
        	this._showBusyStatus();
        	this.model.fetch();
        }	
    },
    
    /**
     * Shows empty status.
     */
    _showNoStatus: function() {
    	if (this.status_row) {
    		this.status_row.innerHTML = "";
    	}
    },
    
    /**
     * Shows busy status.
     */
    _showBusyStatus: function() {
    	if (this.status_row) {
    		hub.busyMessage(this.status_row, false, "note");
    	}
    },
    
    /**
     * Shows error status.
     */
    _showErrorStatus: function(error) {
    	if (this.status_row) {
    		hub.notify(error, this.status_row, 'TOM_editor_error');
    	}
    },
    
    /**
     * Show no entries status.
     */
    _showNoEntriesStatus: function() {
    	if (this.status_row) {
    		this._loadNLS();
            this.status_row.innerHTML = this.msgs.STATUS_TABLE_NONE;
    	}
    },
    
    /**
     * Loads NLS for messages.
     */
    _loadNLS: function() {
    	if (!this.msgs) {
    		this.msgs = dojo.i18n.getLocalization('hub', 'messages'); 
    	}
    },
    
    /**
     * Sets selection for the area.
     */
	_setSelected: function(rowNum, selected, area) {
		if (selected == "uni") {
			// Uni selection case:
			if ( (rowNum != null) && this._isDataSelected(rowNum, area) ) { // 
				this.changedElements[area] = this._getRowId(rowNum); // selection is repeated
				this.isSelectedElements[area] = false;
			}
			else if (rowNum != null) {
				this.changedElements[area] = this._getRowId(rowNum); // 1 row is selected 
				this.isSelectedElements[area] = true;
			}
			else {
				this.changedElements[area] = null; // selected nothing
				this.isSelectedElements[area] = true;
			}
			return;
		}
		// Usual selection algorithm:
		var id = this._getRowId(rowNum);
		var s = this._isDataSelected(rowNum, area);
		var changedElements = null;
		var isSelectedElements = null;
		if ( area in this.isSelectedElements ) {
			if ( this._isBoolean(this.isSelectedElements[area]) ) {
				this.isSelectedElements[area] = [];
				this.changedElements[area] = [];
			}
			changedElements = this.changedElements[area];
			isSelectedElements = this.isSelectedElements[area];
		}
		else {
			if ( selected == s ) {
				return; // no need to set the same info as in data source
			}
			this.changedElements[area] = [];
			this.isSelectedElements[area] = [];
			changedElements = this.changedElements[area];
			isSelectedElements = this.isSelectedElements[area];
		}
		
		var i;
		for(i = 0; i < changedElements.length; i++) {
			if (changedElements[i] == id) {
				break;
			}
		}
		changedElements[i] = id; // i == length, we create new, if not we just reassign
		isSelectedElements[i] = selected;
		if ( (s && isSelectedElements[i]) || (!s && !isSelectedElements[i]) ) {
			// information in arrays is extra, we a to delete it:
			delete changedElements[i];
			delete isSelectedElements[i];
		}
	},
	
	/**
	 * Merges current selection with selection that commes from this row from the data.
	 */
	_mergeSelected: function(rowNum, area) {
		var id = this._getRowId(rowNum);
		if (area in this.isSelectedElements) {
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			if (this._isBoolean(isSelectedElements)) {
				// Uni selection:
				if (id == changedElements) {
					this.isSelectedElements[area] = !this._isDataSelected(rowNum, area);
				}
				return;
			}
			var s = this._isDataSelected(rowNum, area);
			var i;
			for(i = 0; i < changedElements.length; i++) {
				if (changedElements[i] == id) {
					break;
				}
			}
			if (i == changedElements.length) {
				return;
			}
			if ( (s && isSelectedElements[i]) || (!s && !isSelectedElements[i]) ) {
				// information in arrays is extra, we a to delete it:
				delete changedElements[i];
				delete isSelectedElements[i];
			}
		}
	},

	/**
	 * Checks if row is selected in specified area.
	 */	
	_isSelected: function(rowNum, area) {
		var id = this._getRowId(rowNum);
		var s = this._isDataSelected(rowNum, area);
		if ( area in this.isSelectedElements ) {
			if (this._isBoolean(this.isSelectedElements[area])) {
				var soleId = this.changedElements[area];
				if (soleId == null) {
					return s;
				}
				else {
					return id == soleId;
				}
			}
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			var i;
			for(i = 0; i < changedElements.length; i++) {
				if (changedElements[i] == id) {
					s = isSelectedElements[i];
					break;
				}
			}
		}
		return s;
	},
	
	/**
	 * Gets status of selection for row (delegates call to selection module).
	 */
	_isDataSelected: function(rowNum, area) {
		return this.selection.isDataSelected(this.model.getRow(rowNum), area);
	},
	
	/**
	 * Gets row id (delegates call to selection module).
	 */
	_getRowId: function(rowNum) {
		return this.selection.getRowId(this.model.getRow(rowNum));
	},
	
	/**
	 * Check is value is changed from original data selection.
	 */
	_isChanged: function(rowNum, area) {
		var s1 = this._isDataSelected(rowNum, area);
		var s2 = this._isSelected(rowNum, area);
	},

	/**
	 * Changes selection.
	 * Listens for events on TR and TD. Find appropriate area and toggles selection.
	 */	
	_select: function(event) {
		var src = event.target;
		var k = 0;
		for(k = 0; k < this.noDefaultSelectionTags.length; k++) {
			if (src.tagName == this.noDefaultSelectionTags[k]) {
				return;
			}
		}
		var srcTD = null;
		var selectionDefault = true;
		var selectionArea = null;
		var selectionMode = this.defaultSelectionAreaMode;
		if (this.defaultSelectionArea) {
			selectionArea = "default";
			selectionMode = this.defaultSelectionAreaMode;
		}
		while (src.tagName != 'TR') {
			if (src.tagName == 'TD') {
				var area = src.getAttribute("selectionArea");
				if (area) {
					srcTD = src;
					var i;
					for(i in this.columns) {
						if (this.columns[i]['selectionArea'] == area) {
							selectionArea = area;
							selectionMode = this.columns[i]['selectionMode'];
							selectionDefault = false;
							break;
						}
					}
				}
			}
			src = src.parentNode;
		}
		var rowNum = src.getAttribute("rowNum");
		if ((rowNum != null) && (selectionArea != null)) {
			rowNum = parseInt(rowNum);
			if (this._is1__1(selectionMode)) {
				this._setSelected(rowNum, "uni", selectionArea);
					if (selectionDefault) {
						this._defaultSelectionRedraw();
					}
					else {
						this._selectionRedraw(selectionArea);
					}
			}
			else if (this._is0__1(selectionMode)) {
				if (this._isSelected(rowNum, selectionArea)) {
					this._setSelected(null, "uni", selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, false);
					}
					else {
						this._togleSelection(srcTD, false);
					}
				}
				else {
					this._setSelected(rowNum, "uni", selectionArea);
					if (selectionDefault) {
						this._defaultSelectionRedraw();
					}
					else {
						this._selectionRedraw(selectionArea);
					}
				}
			}
			else if (this._is0__n(selectionMode)) {
				if (this._isSelected(rowNum, selectionArea)) {
					this._setSelected(rowNum, false, selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, false);
					}
					else {
						this._togleSelection(srcTD, false);
					}
				}
				else {
					this._setSelected(rowNum, true, selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, true);
					}
					else {
						this._togleSelection(srcTD, true);
					}
				}
			}
		}
	},
	
	/**
	 * Redraw selection for default area. Adjusts styles accoriding to current selection.
	 */
	_defaultSelectionRedraw: function() {
		var _this = this; 
		dojo.query("tr[rowNum]", this.table_body).forEach(function(tr) {
			var rowNum = tr.getAttribute("rowNum");
			if (rowNum != null) {
				rowNum = parseInt(rowNum);
		        if ( (_this.defaultSelectionArea != null) && _this._isSelected(rowNum, _this.defaultSelectionArea) ) {
		        	_this._togleDefaultSelection(tr, true);
		        }
		        else {
		        	_this._togleDefaultSelection(tr, false);
		        }
			}			
		});
	},
	
	/**
	 * Redraw selection for area. Adjusts styles accoriding to current selection.
	 */
	_selectionRedraw: function(area) {
		var _this = this;
		dojo.query('td[selectionArea=\"' + area + '\"]', this.table_body).forEach(function(td) {
			var rowNum = td.parentNode.getAttribute("rowNum");
			if (rowNum != null) {
				rowNum = parseInt(rowNum);
	        	_this._togleSelection(td, _this._isSelected(rowNum, area));
			}			
		});
	},
	
	/**
	 * Check is selection mode code corresponds to 1..1
	 */
	_is1__1: function(mode) {
		return ( (1 == mode) || ("1..1" == mode) || ("1" == mode) );
	},
	
	/**
	 * Check is selection mode code corresponds to 0..1
	 */
	_is0__1: function(mode) {
		return ( (0 == mode) || ("0..1" == mode) || ("0" == mode) );
	},

	/**
	 * Check is selection mode code corresponds to 0..n
	 */ 
	_is0__n: function(mode) {
		return ( (mode > 1) || ("0..n" == mode) || (parseInt(mode) > 1) );
	},
	
	/**
	 * Check if value is exactly true of false.
	 */
	_isBoolean: function(v) {
		return (true == v) || (false == v);
	},
	
	/**
	 * Retursn value for individual ares.
	 * If selection is uni, selected element id is returned, or null if no selection, or [] if original selection was not changed.
	 * If selection is multi, array of changed elements is returned: [{id: ..., selected: true/false}, {id: ..., selected: true/false}, ...]
	 */
	getValueForArea: function(area) {
		if (area in this.isSelectedElements) {
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			if (this._isBoolean(isSelectedElements)) {
				if (isSelectedElements || uniselectionReselectValue) {
					return this.changedElements[area];
				}
				else {
					return [];
				}
			}
			var result = new Array();
			var i;
			for(i = 0; i < changedElements.length; i++) {
				result.push({'id':changedElements[i], 'selected':isSelectedElements[i]});
			}
			return result;
		}
		else {
			return [];
		}
	},
	
	/**
	 * Sets value for individual area.
	 * If value is singleton selection will be uni, with value as selected element.
	 * If value is null      selection will be uni(0..1), with no selection.
	 * If value is []        selection will be uni or multi, meaning no change from original value.
	 * If value is non empty array, selection will be multi, each record of form {id: ..., selected: true/false}.
	 */
	setValueForArea: function(area, value) {
		if (dojo.isArray(value)) {
			this.changedElements[area] = new Array();
			this.isSelectedElements[area] = new Array();
			var i = 0;
			for(i = 0; i < value.length; i++) {
				this.changedElements[area].push(value[i].id);
				this.isSelectedElements[area].push(value[i].selected);
			}
		}
		else { // uni selection
			this.changedElements[area] = new Array();
			this.isSelectedElements[area] = new Array();
			this.changedElements[area].push(value); // as id
			this.isSelectedElements[area].push(true);
		}
	},
	
	/**
	 * Sets value for this control,
	 * value may be only one area (valueSelectionArea), or all areas (whne valueSelectionArea = null).
	 */
	setValue: function(value) {
		var area;
		for(area in this.isSelectedElements) {
			if (this.valueSelectionArea == area) {
				this.setValueForArea(area, value);
				break;
			}
			if (this.formSelectionArea == null) {
				var v = [];
				if (value[area]) {
					v = value[area];
				}
				this.setValueForArea(area, v);
			}
		}
	},
	
	/**
	 * Returns value of this control,
	 * value may be only one area (valueSelectionArea), or all areas (whne valueSelectionArea = null).
	 */
	getValue: function() {
		var area;
		var result = {};
		for(area in this.isSelectedElements) {
			if (this.valueSelectionArea == area) {
				return this.getValueForArea(area);
				break;
			}
			if (this.formSelectionArea == null) {
				result[area] = this.getValueForArea(area);
			}
		}
		return result;
	},
	
	/**
	 * Disconnects array or individual listner.
	 * @return empty array or null
	 */
	_disconnect: function(a) {
		if (dojo.isArray(a)) {
			for(;;) {
				if (a.length == 0) {
					break;
				}
				var v = a.pop();
				if (v) {
					dojo.disconnect(v);
				}
				else {
					break;
				}
			}
		}
		else {
			if (a) {
				dojo.disconnect(a);
				a = null;
			}
		}
		return a;
	}
});