/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.widgets.Dialog");

dojo.require("dijit.Dialog");
dojo.require("dojo.dnd.move");
dojo.require("dojox.layout.ResizeHandle");

dojo.declare(
	"hub.widgets.Dialog",
	[dijit.Dialog, dijit._Container],
  {
    templateString: null,
    templatePath: dojo.moduleUrl("hub.widgets", "templates/Dialog.html"),
    actionNode: null,
    buttonNode: null,
    resizeHandleNode: null,
    operatorId: null,

    showUnderlay: false,
    hideTitleBar: false,
    hideActionBar: false,
    buttons: null,

    postCreate: function() {
      if (this.hideTitleBar) {
        this.titleBar.style.display = "none";
      }

      if (this.hideActionBar) {
        this.actionNode.style.display = "none";
      }

      //this.inherited("postCreate",arguments);
	  this.bgIframe = new dijit.BackgroundIframe(this.domNode);

      if (this.buttons) {
        for (var i = 0; i < this.buttons.length; i++) {
          var button = document.createElement("button")
          button.appendChild(document.createTextNode(this.buttons[i].label));

          if (this.buttons[i].onclick) {
            button.onclick = this.buttons[i].onclick;
          }

          this.buttons[i].node = this.buttonNode.appendChild(button);
        }
      } 
      else {
        // since IE doesn't do min-height
        this.actionNode.style.height = "13px";  // resizehandle image height size
      }
    },
    
    _setup: function() {
		if (this.titleBar) {
			//this._moveable = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {handle: this.titleBar, area: "margin", within: true});
			new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
            dojo.connect(this._moveable, "onFirstMove", function(){this.handle.focus()});
        }
        
        if (this.showUnderlay) {
           this._underlay = new dijit.DialogUnderlay();
        }
        
        //dojo.require("dojox.layout.ResizeHandle");

		var tmp = new dojox.layout.ResizeHandle({ 
				targetId: this.id,
				resizeAxis: "xy"}, this.resizeHandleNode);
    },
    
    layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
			    if (this.showUnderlay) {
				   this._underlay.layout();
				}
				this._position();
			}
	},
		
    show: function() {
      var doLayout = false;

      if (!this._alreadyInitialized) {
        this._setup();
        this._alreadyInitialized = true;
        doLayout = true;
      }

      this.domNode.style.display = "block";

      dojo.addClass(this.domNode, "hubDialog");

      if (this.showUnderlay) {
        this._underlay.show();
      } 
      else {
      }

      if (doLayout) {
        this.layout();
      }

      this.open = true;
      this._savedFocus = dijit.getFocus(this);
      dijit.focus(this.titleBar);

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    hide: function() {
      if (!this._alreadyInitialized) {
        return;
      }

      this.domNode.style.display = "none";

      if (this.showUnderlay) {
        this._underlay.hide();
      }

      dijit.focus(this._savedFocus);
      this.open = false;
    },
    
    resize: function(data) {
      var w = data.w;
      var h = data.h;

      dojo.style(this.domNode, "width", w+"px"); 
      dojo.style(this.domNode, "height", h+"px");

      // now resize the dialog contents
      var titlebarBox = dojo.marginBox(this.titleBar);

      // TODO: 4 is fixed padding (both sides, so 8), needs to be calculated
      // IE works differently, so no padding is needed (else we get a visual drawing bug)
      h -= titlebarBox.h + dojo.marginBox(this.actionNode).h + (dojo.isIE ? 1 : 9);
      
      dojo.style(this.containerNode, "height", h+"px");
   
      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    resizeChildren: function(data) {
      var children = this.getChildren();
      for (var i = 0; i < children.length; i++) {
        if (children[i].resize) {
          children[i].resize();
        }
      }
/*
      dojo.query(".hubResizable", this.domNode).forEach(function(node){
         dijit.byNode(node).resize();
      });
*/      
    }
    
    
  }
);
