/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.adminobjectscatalog');
dojo.require('hub.main');
dojo.require('hub.widgets');
dojo.require("dojo.i18n");
dojo.require("dojo.date.locale");
dojo.require("hub.table.model");
dojo.require("hub.table.table");
dojo.require("hub.table.navigation");
dojo.require("hub.data.atommodel");
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "links");
dojo.requireLocalization("hub", "popups");

/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.adminobjectscatalog.Catalog",  [dijit._Widget, dijit._Templated], {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/AdminObjectsCatalog.html"),
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,

    constructor: function(args, node) {
    },

    postMixInProperties: function() {
    	this.inherited(arguments);
    	this.model = new hub.data.atommodel.TableModel();
    	this.model.pageSize = 10;
    },

    postCreate: function() {
    	this.inherited(arguments);
    },

    startup: function() {
    	this.inherited(arguments);
    	
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
    	
    	this.table.addColumn("id", panels.ADMIN_CATALOG_TABLE_ID_LABEL, 'id', function(element, row) {
          element.innerHTML = row.objectid;
      });
      this.table.addColumn("name", panels.ADMIN_CATALOG_TABLE_NAME_LABEL, 'title', function(element, row) {
          element.innerHTML = hub.buildPageLink(hub.htmlspecialchars(row.name), 'Details', {object_id: row.objectid}, 
                                                false, null, '');
      });
      this.table.addColumn("desc", panels.ADMIN_CATALOG_TABLE_DESC_LABEL, null, function(element, row) {
          element.innerHTML = hub.htmlspecialchars(row.description);
      });
      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    },

    destroy: function() {
    	
    },
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
    		if (selection[i].selected) {
    			result["deletion" + label] = "";
    		}
    	}
    	return result;
    },

	onBeforeFetched: function() {
//	console.log('Catalog.onBeforeFetched()');
	},
     
	_onSearch: function() {
		var value = "";
		if (this.searchTerm) {
			value = this.searchTerm.getValue();
		}
		if (value == "") {
			value = "*";
		}
		this.model.term = value;
		this.model.setPageToLoad(null);
		this.model.fetch();
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.RelatedSelection", hub.table.model.DataSelection, {
	container: null,
	isDataSelected: function(row, area) {
//		console.log("RelatedSelection.isDataSelected");
		var result = false;
		var ids = this.container.selected_ids;
		for(var i = 0; i < ids.length; i++){
			if(ids[i] == row.id){
				result = true;
				break;
			}
		} 
//		console.dir({result: result, rowId: row.id, ids:this.container.selected_ids});
		return result; 
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.CategoriesSelection", hub.table.model.DataSelection, {
	container: null,
	isDataSelected: function(row, area) {
		if ("isChecked" in row) {
			return (false != row.isChecked);
		}
		return false;
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.Related",  [hub.widgets.adminobjectscatalog.Catalog], {
	selected_ids: new Array(),
    entryId: 0,
    startup: function() {
    },

	onBeforeFetched: function(){
//	console.log('Related.onBeforeFetched');
    	var url = hub.urls.getAssociationsIdsList(this.entryId);
    	var req = new hub.submit.Request(url);
    	req._onLoad = dojo.hitch(this, function(response){
    			response = dojo.fromJson(response);
//    			console.dir({response: response});
    			this.selected_ids = response.ids;
    		});
    	req.submit();
	},

    postCreate: function() {
    	this.inherited(arguments);
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
		this.table.defaultSelectionAreaMode=2;
		this.table.selection = new hub.widgets.adminobjectscatalog.RelatedSelection();
		this.table.selection.container = this;
		this.table.addColumn("name", panels.ADMIN_RELATED_TABLE_RELATED_LABEL, 'title', function(element, row) {
			element.innerHTML = hub.htmlspecialchars(row.name);			
		});
		this.table.setModel(this.model);
		dojo.connect(this.model, 'onBeforeFetched', this, 'onBeforeFetched');
		this.navigation.setModel(this.model);
		this.model.fetch();
    },
    
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
   			result["rel_" + label] = selection[i].selected?"on":"off";
    	}
    	return result;
    }
});


/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.adminobjectscatalog.Categories",  [dijit._Widget, dijit._Templated], {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/AdminCategories.html"),
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,
    
    entryId: 0,

    constructor: function(args, node) {
    },

    postMixInProperties: function() {
    	this.inherited(arguments);
    	this.model = new hub.data.jsonmodel.TableModel({url:hub.urls.getCategoriesList(this.entryId)});
    	this.model.pageSize = 10;
    },

    postCreate: function() {
    	this.inherited(arguments);
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
		this.table.defaultSelectionAreaMode=2;
		this.table.selection = new hub.widgets.adminobjectscatalog.CategoriesSelection();
/*		this.table.selection.isDataSelected =function(row, area) {
				if ("isChecked" in row) {
					return (false != row.isChecked);
				}
				return false;
			}*/
		this.table.addColumn("name", panels.ADMIN_CATEGORIES_TABLE_CATEGORIES_LABEL, 'title', function(element, row) {
			element.innerHTML = hub.htmlspecialchars(row.name);			
		});
		this.table.setModel(this.model);
		this.navigation.setModel(this.model);
		this.model.fetch();
    },

/*    startup: function() {
    console.log('Categories.startup()');
    	this.inherited(arguments);
    },*/

    destroy: function() {
    	
    },
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
   			result["cat_" + label] = selection[i].selected?"on":"off";
    	}
    	return result;
    }
     
});

/**
 * Subclass for object tracking
 */
dojo.declare("hub.widgets.adminobjectscatalog.ObjectTracking",  [hub.widgets.adminobjectscatalog.Catalog], {
    changedRows: null,

    postMixInProperties: function() {
    	this.inherited(arguments);

    	dojo.require("hub.data.jsonmodel")
    	this.model = new hub.data.jsonmodel.TableModel({url: hub.urls.getAdminEditTrackingPage(), pageSize: 10});
    },

    startup: function() {
      this.changedRows = [];

      // no selection
      this.table.setDefaultSelectionArea(null);

      var panels = dojo.i18n.getLocalization('hub', 'panels');

			this.table.addColumn("id", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL1_LABEL, 'id', function(element, row) {
          element.innerHTML = row.id;
      });
			this.table.addColumn("title", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL2_LABEL, 'title', function(element, row) {
          element.innerHTML = hub.buildPageLink(hub.htmlspecialchars(row.title), 'Details', {object_id: row.id}, 
                                                false, null, '');
      });

      var myThis = this;
      this.table.addColumn("isTrackable", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL3_LABEL, 'isTrackable', function(element, row) {
          var input = document.createElement("input");
          input.setAttribute("type", "checkbox");
          input.setAttribute("objecttype", "isTrackable");

          input.setAttribute("objectid", row.id);

          if (row.isTrackable) {
            input.setAttribute("checked", "true");
          }

          dojo.connect(input, "onchange", myThis, "onCheckboxClick");

          element.appendChild(input);
      });
      this.table.addColumn("isPerm", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL4_LABEL, 'isPerm', function(element, row) {
          var input = document.createElement("input");
          input.setAttribute("type", "checkbox");
          input.setAttribute("objecttype", "isPerm");

          input.setAttribute("objectid", row.id);

          if (row.isPerm) {
            input.setAttribute("checked", "true");
          }

          dojo.connect(input, "onchange", myThis, "onCheckboxClick");

          element.appendChild(input);
      });
      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    },

    onCheckboxClick: function(event) {
      var target = event.target;
      var id = target.getAttribute("objectid");
      var checked = target.checked;
      var type = target.getAttribute("objecttype");

      // update model
      var rows = this.model.rows;
      for (var i = 0; i < rows.length; i++) {
        var row = rows[i];
        if (row.id == id) {
          if (type == "isTrackable") {
            row.isTrackable = checked;
          } else {
            row.isPerm = checked;
          }

          this.changedRows[id] = {isTrackable: row.isTrackable, isPerm: row.isPerm}
          break;
        }
      }
    },

    getFormData: function() {
    	var result = {};

    	for (var id in this.changedRows) {
        var row = this.changedRows[id];
        result["track"+id] = row.isTrackable ? "on" : "off";
        result["perm"+id] = row.isPerm ? "on" : "off";
    	}

    	return result;
    }
});

/**
 * Subclass for object tracking data
 */
dojo.declare("hub.widgets.adminobjectscatalog.ObjectTrackingData",  [hub.widgets.adminobjectscatalog.Catalog], {
    postMixInProperties: function() {
    	this.inherited(arguments);

    	dojo.require("hub.data.jsonmodel")
    	this.model = new hub.data.jsonmodel.TableModel({url: hub.urls.getAdminTrackingData(), pageSize: 10});
    },

    startup: function() {
      // no selection
      this.table.setDefaultSelectionArea(null);

      var panels = dojo.i18n.getLocalization('hub', 'panels');

			this.table.addColumn("title", panels.ADMIN_TRACKED_TABLE_TITLE, 'title', function(element, row) {
          element.innerHTML = "<a href='"+hub.config.base_url+"/tracking/generate/entryid/"+row.id+"' target='_blank'><img src='"+hub.urls.getImage(row.img)+"' border='0' alt='"+ panels.ADMIN_TRACKED_DATA_ICON_ALT+"'></a> " + 
                              hub.buildPageLink(hub.htmlspecialchars(row.title), 'Details', {object_id: row.id}, 
                                                false, null, '');
      });

      this.table.addColumn("hits", panels.ADMIN_TRACKED_TABLE_HITS, 'hits', function(element, row) {
          element.innerHTML = row.hits;
      });

      this.table.addColumn("avg_load", panels.ADMIN_TRACKED_TABLE_AVGRESPONSE, 'avg_load', function(element, row) {
          element.innerHTML = row.avg_load;
      });

      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    }
});
