/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.catalog');
dojo.require('hub.main');
dojo.require('hub.widgets');
dojo.require("dojo.i18n");
dojo.require("dojo.date.locale");
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "links");
dojo.requireLocalization("hub", "popups");

/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.catalog.CatalogTable", hub.widgets.Table, {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/CatalogTable.html"),
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,

    constructor: function(args, node) {
        // has the model indicated it is stale?
        this.stale = false;
        // subscriptions to dojo.publish'ed events so we know when to refresh
        this.conns = new Array();
    },

    postMixInProperties: function() {
        // get model widget from id
        this.model = dijit.byId(this.model);

        // fetch all localized text
        this.links = dojo.i18n.getLocalization('hub', 'links');
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.msgs = dojo.i18n.getLocalization('hub', 'messages');
        this.pops = dojo.i18n.getLocalization('hub', 'popups');

        // urls for icons and such
        this.atom_img_url = hub.urls.getImage('xml.gif');
        this.feed_img_url = hub.urls.getImage('feedSmall.gif');
        this.widget_img_url = hub.urls.getImage('widgetSmall.gif');
        this.mashup_img_url = hub.urls.getImage('mashupSmall.gif');
        this.page_img_url = hub.urls.getImage('pageSmall.gif');
    },

    postCreate: function() {
        var labels = dojo.i18n.getLocalization('hub', 'labels');

        // register as an observer
        this.connect(this.model, 'onFetch', '_onBeforeFetch');
        this.connect(this.model, 'onBegin', '_onBeforeLoad');
        this.connect(this.model, 'onItem', '_onItemLoad');
        this.connect(this.model, 'onComplete', '_onAfterLoad');
        this.connect(this.model, 'onError', '_onLoadError');
        this.connect(this.model, 'onStale', '_onStale');
        // watch for bus events about page activation
        this.conns.push(dojo.subscribe('hub.activate-page', this, '_onShow'));


        // column metadata
        this.setColumns([{'name' : labels.CATALOG_COL_TITLE,
                          'class' : 'title',
                          'sortby' : 'title',
                          'markup' : this._getTitleMarkup},
                         {'name' : labels.CATALOG_COL_TAGS,
                          'class' : 'tags',
                          'sortby' : null,
                          'markup' : this._getTagsMarkup},
                         {'name' : labels.CATALOG_COL_UPDATED,
                          'class' : 'updated',
                          'sortby' : 'updated',
                          'markup' : this._getDateMarkup},
                         {'name' : labels.CATALOG_COL_RATING,
                          'class' : 'rating',
                          'sortby' : 'rating',
                          'markup' : this._getRatingMarkup},
                         {'name' : labels.CATALOG_COL_AUTHOR,
                          'class' : 'author',
                          'sortby' : 'author',
                          'markup' : this._getAuthorMarkup}
                        ]);

        // initialize navigation
        this._updateNavBar();

        // interpolate values
        this.inherited(arguments);
    },

    startup: function() {
        if(this.model && this.forceRefresh) {
            this.model.refresh();
        }
    },

    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },

    /**
     * Clears table rows and associated popups.
     */
    clearRows: function() {
        this.inherited(arguments);
        // clear out popups too
        this.popups.innerHTML = '';
    },
    
    /**
     * Render the table columns, navigation, and atom link.
     */
    initialize : function()  {
        // clear everything out
        this.clearHeaders();
        this.clearRows();
        
        for(var i in this.columns) {
            var th = document.createElement('TH');
            // title and base class
            th.innerHTML = this.columns[i]['name'];
            th.className = this.columns[i]['class'];
            if(this.columns[i]['sortby']) {
                // make focusable
                th.tabIndex = 0;
                // mouse and key handler for sorting
                this.connect(th, 'onclick', '_onSort');
                this.connect(th, 'onkeypress', '_onSort');
                // @todo: initialize first column sorting for now, need to be
                //   smarter about what order the results are in
                dojo.removeClass(th, 'selectedUp');
                dojo.removeClass(th, 'selectedDown');
                if(this.model.sortBy == this.columns[i]['sortby']) {
                	if (this.model.sortOrder == 'asc') {
                    	dojo.addClass(th, 'selectedUp');
                    }
                    if (this.model.sortOrder == 'desc') {
                    	dojo.addClass(th, 'selectedDown');
                    }
                }
            }
            this.table_header.appendChild(th);
        }

        // footer status area, built here because we need to know how many 
        // columns
        this.status_row = document.createElement('TD');
        this.status_row.colSpan = this.columns.length;
        this.status_row.className = 'note';
        this.table_footer.appendChild(this.status_row);
    },

    /**
     * Sorts the catalog ascending or descending by the clicked column 
     * header. Pressing space while the header has focus also sorts.
     */
    _onSort : function(event) {
        var src = event.target;

        // if key but not space, don't count as a sort
        if(event.charCode != 0 && event.charCode != null && event.charCode != dojo.keys.SPACE) {
            return;
        }
        
        var index = null;
        var asc = 1;
        // clear all styles
        for(var i in this.table_header.childNodes) {
            var th = this.table_header.childNodes[i];
            if(src == th) {
                index = i;
                if(dojo.hasClass(th, 'selectedUp')) {
                    // switch from up to down
                    dojo.removeClass(th, 'selectedUp');
                    dojo.addClass(th, 'selectedDown');
                    asc = 0;
                } else {
                    // switch from down to up
                    dojo.removeClass(th, 'selectedDown');
                    dojo.addClass(th, 'selectedUp');
                    asc = 1;
                }
            } else {
                // not soting by this column
                dojo.removeClass(th, 'selectedUp');
                dojo.removeClass(th, 'selectedDown');
            }
        }
        
        // fetch the newly sorted results from the server
        // no start or max because a new sort means we're starting at
        // the beginning again anyways
        this.model.fetch(null, null, this.columns[index]['sortby'], asc);
    },
    

    /**
     * Refresh if active tab. Else mark stale for later refresh.
     */
    _onStale: function() {
        var def = hub.isPageActive(this.domNode);
        def.addCallback(function(active) {
            if(active) {
                // refresh immediately
                this.stale = false;
                this.model.refresh();
            } else {
                // mark stale for later refresh
                this.stale = true;
            }
        });
    },

    /**
     * Refresh if stale and the page containing this widget is active.
     */
    _onShow: function(page) {
        if(this.stale && dojo.isDescendant(this.domNode, page.domNode)) {
            this.stale = false;
            this.model.refresh();
        }
    },
    
    /**
     * Show the refreshing message.
     */
    _onBeforeFetch: function() {
        this.clearRows();
        hub.busyMessage(this.status_row, false, "note");
    },
    
    _onBeforeLoad: function(total) {
        // reset row count
        this.row_count = 0;
    },
    
    /**
     * Show the error message.
     */
    _onLoadError: function(error) {
        hub.notify(error, this.status_row, 'TOM_editor_error');
    },

    /**
     * Render all columns of a single item row.
     */        
    _onItemLoad : function(dict) {
        // create popup and save its id
        var pid = this._createPopup(dict);
        // create table row and columns
        var tr = document.createElement('TR');
        
        for(i in this.columns) {
            var func = this.columns[i]['markup'];
            var td = document.createElement('TD');
            //td.style.overflow='hidden';
            var ml = func.call(this, dict);
            td.innerHTML = ml;
            dojo.addClass(td, this.columns[i]['class']);
            tr.appendChild(td);
        }
        
        // alternating row style
        if(this.row_count % 2) {
            dojo.addClass(tr, 'alt');
        }
        
        this.table_body.appendChild(tr);
        this.row_count += 1;
    },
    
    /**
     * Remove the refreshing message. Update the navigation and Atom feed
     * link.
     */        
    _onAfterLoad: function() {
        if(!this.row_count) {
            // no data, say so
            this.status_row.innerHTML = this.msgs.STATUS_CATALOG_NONE;
            this.status_nav.innerHTML = "";
        } else {
            // show a summary in the status row
            this.status_nav.innerHTML = this._getSummaryMarkup();
            this.status_row.innerHTML = "";
        }
        this._updateNavBar();
        this._updateFeedLink();
    },    
    
    /**
     * Builds a status line with start, end, and total information if
     * available.
     */
    _getSummaryMarkup: function() {
        var sum = this.model.getSummary();
        // check what information we have
        var start = sum[0];
        var max = sum[1];
        var total = sum[2];
        if(start != undefined && max != undefined && total != undefined) {
            var end = Math.min(start + max, total);
            return dojo.string.substitute(this.msgs.STATUS_CATALOG_SUMMARY, 
                                          [start+1, end, total]);
        } else {
            return '';
        }
    },
    
    /**
     * Title column content.
     */        
    _getTitleMarkup : function (dict) {
        // encode the title name
        var title = "";
        if (dict.name) {
        	title = hub.htmlspecialchars(dict.name);
        }
        
        // build an icon based on category
        var iconSrc;
        var iconTitle;
        var entryCategory = "";
        if (dict.entryCategory) {
        	entryCategory = dict.entryCategory;
        }
        switch(entryCategory) {
        case 'widget':
        case 'iwidget':
        case 'qedwidget':
            iconSrc = this.widget_img_url;
            iconTitle = this.pops.CATALOG_WIDGET_ICON;
            break;
        case 'mashuppage':
            iconSrc = this.page_img_url;
            iconTitle = this.pops.CATALOG_PAGE_ICON;
            break;    
        case 'mashup':
            iconSrc = this.mashup_img_url;
            iconTitle = this.pops.CATALOG_MASHUP_ICON;
            break;
        default:
            iconSrc = this.feed_img_url;
            iconTitle = this.pops.CATALOG_FEED_ICON;
            break;
        }
        
        // build ID names
        var spanId = this.id+"_entry_"+dict.objectid;
        var popupId = this.id+"_popup_"+dict.objectid+"_popupbanner";
        var popupId2 = this.id+"_popup_"+dict.objectid;
        
        var ml = "<div style=\"overflow:hidden;text-overflow:ellipsis;\">";
        // check if the icon should link to parameters form or 
        // directly to generate URL
        if(dict.parameters && parseInt(dict.parameters)) {
            // has parameters, link to parameters form
            // @todo: how do we show the parameters form?
            var genUrl = hub.config.base_url + '/plugin/params/entryid/'+dict.objectid+'/pluginid/'+dict.pluginid;

            var js = 'javascript:hub.tabs.showTab("'+genUrl+'","'+title+'","'+dict.objectid+'");';
            ml += "<a class='noLink' href='"+js+"' >";
        } else if (dict.downloadURL) { 
            // no parameters, use generate link
            var genUrl = dict.downloadURL;
            
            ml += "<a class='noLink' href='" + hub.urls.secureUrl(genUrl) + "' target='_blank'>";
        }               
        else {
        	ml += "<a class='noLink' href='javascript:void(0)'>";
        }
        ml += "<img width='19px' height='19px' src='"+iconSrc+"' alt='"+iconTitle+"' title='"+iconTitle+"'/></a>";
        
        ml += "&nbsp;<span id='"+spanId+"' ";
        ml += "onmouseover='showPopupLayer2(\""+popupId2+"\",\""+spanId+"\",\""+spanId+"\", PL_DISPLAY_IN_BUBBLE | PL_SUGGESTED_POS_RIGHT | PL_MOUSE_OVER_SUPPORT | PL_ADD_BANNER, \""+popupId+"\")'>";
        // @note: last arg is important; supresses creation of link title popup
        ml += hub.buildPageLink(title, 'Details', {object_id: dict.objectid}, 
                                false, null, '');
        ml += "</span></div>";

        return ml;
    },
    
    /**
     * Tags column content.
     */        
    _getTagsMarkup : function(dict) {
        // bail right away if tags is empty or non-existant
        if(!dict.tags) {
            // make sure we respect styles that require content
            return '&nbsp;';
        }
        var ml = "";
        for(var i=0; i < dict.tags.length; i++) {
            var tag = hub.htmlspecialchars(dict.tags[i]);
            ml += hub.buildPageLink(tag, 'Catalog', {collection: "all", searchType: 'tag', term: tag});
            if (i < dict.tags.length-1) {
                ml += ", ";
            }
        }
        return ml;
    },
    
    /**
     * Date column content.
     */        
    _getDateMarkup : function(dict) {
    	var locDate = "&nbsp;";
    	if (dict.dateModified) {
        	var date = new Date(parseInt(dict.dateModified)*1000);
        	locDate = dojo.date.locale.format(date);
        }
        // in a span with a title in case of overflow
        return '<span title="'+locDate+'">'+locDate+'</span>';
    },
    
    /**
     * Rating column content.
     */        
    _getRatingMarkup: function(dict) {
        var ml = '';
        var value = parseInt(dict.rating);
        dojo.require('hub.widgets.social.view');
        var w = new hub.widgets.social.view.RatingStarView();
        var urls = w.getStarUrls(value);
        var title = w.getRatingEstimate(value);
        ml += "<span title='"+title+"'>";
        for(var i=0; i < urls.length; i++) {
            ml += "<img src='"+urls[i]+"' />";
        }
        ml += '</span>';
        return ml;
    },
    
    /**
     * Author column content.
     */                
    _getAuthorMarkup : function(dict) {
    	var author = "";
    	if (dict.author) {
    		author = dict.author;
    	}
        var ml = hub.buildPageLink(author, 'Catalog', 
            {collection: "all", searchType: 'user', term: author});
        return ml;
    },
    
    /**
     * Creates all popup controls for titles.
     */                        
    _createPopup : function(dict) {
        var pid = this.id+"_popup_"+dict.objectid;
        var ml = "";
        
        var title = "";
        if (dict.name) {
        	title = hub.htmlspecialchars(dict.name);
        }
        
        var desc = "";
        if (dict.description) {
        	desc = hub.htmlspecialchars(dict.description);
        }
        
        var version = "";
        if (dict.version) {
        	version = hub.htmlspecialchars(dict.version);
        }
        
        ml += "<div id='"+pid+"' class='newPopup'>";
        ml += "<div style='padding:10px' >";
        ml += "<h3 class='clip'><span title='"+title+"'>"+title+"</span></h3>";
        ml += "<b>"+ this.panels.DETAILS_VERSION_TEXT +"&nbsp;</b>"+version;
        ml += "<p class='clip'><span title='"+desc+"'>"+desc+"</span></p>";
        ml += hub.buildPageLink(title, 'Details', 
                                {object_id: dict.objectid}, false, 
                                this.links.CATALOG_VIEW_DETAILS);
		
		//display addtomashups link if user is login
		if (hub.config.username && (dict.entryCategory == "feed" || dict.entryCategory == "mashup" || dict.entryCategory == "iwidget" || dict.entryCategory == "mashuppage")) {

			var serverUrl = hub.config.palette_url ? hub.config.palette_url : mm.getDefaultMMServerUrl();
			var templateName = "";
			
			ml += "&nbsp;|&nbsp;";
			
			// if is package page
			if (dict.entryCategory == "mashuppage" && dict.definitionURL) {
				// is mashuppage url
				ml += "<a target='_blank' href='" + dict.definitionURL + "'>" + this.links.ACTION_VIEW_PAGE_LINK + "</a>";
			} else {
				//the template to show
				if (!hub.config.palette_url) {
					templateName = "SelectMashupsServer";
				} else if (dict.entryCategory == 'feed' || dict.entryCategory == 'mashup') {
					templateName = "AddFeedToMashups";
				} else if (dict.entryCategory == 'iwidget') {
					templateName = "AddIWidgetToMashups";
				} else if (dict.entryCategory == 'mashuppage'){
					templateName = "AddMashupPageToMashups";
				};
				
				ml += hub.buildPageLink(title, templateName, {
											object_id: dict.objectid,
											object_title: dict.name,
											type: dict.entryCategory,
											server: serverUrl
										}, false, this.links.ACTION_ADD_TO_MASHUPS);
			}
		}
		
        ml += "</div></div>";
        this.popups.innerHTML += ml;
        return pid;
    },

    /**
     * Updates the link to the Atom feed content.
     */                
    _updateFeedLink : function() {
        this.feed_link.href = this.model.getUrl();
    },
    
    /**
     * Updates the navigation bar links, disabling next/prev as needed. Visibly
     * enables/disables as well as functionally.
     */                        
    _updateNavBar : function() {
    
        if(this.model.hasPrevious()) {
            this.previous_button.setDisabled(false);
        } else {
            this.previous_button.setDisabled(true);
        }        
       
        if(this.model.hasNext()) { 
            this.next_button.setDisabled(false);
        } else {
            this.next_button.setDisabled(true);
        }        
    },

    /**
     * Fetches the previous page of catalog items.
     */
    _onPrevious: function(event) {
        try {
            this.model.fetchPrevious();
        } catch(e) {}
    },

    /**
     * Fetches the next page of catalog items.
     */    
    _onNext: function(event) { 
        try {
            this.model.fetchNext();
        } catch(e) {}
    },

    /**
     * Refreshes the current page of catalog items.
     */    
    _onRefresh: function(event) {
        this.model.refresh();
    }
});
