/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.require("dojo.data.ItemFileReadStore");
dojo.provide('hub.widgets.chrome');
dojo.require('hub.main');
dojo.require('hub.behaviors');
dojo.require('hub.submit');
dojo.require('dijit.Tooltip');
dojo.require('dijit._Widget');
dojo.require('dijit._Templated');
dojo.require('dijit.layout.ContentPane');
dojo.require('dijit.layout.StackContainer');
dojo.require('dijit.form.TextBox'); // for search form
dojo.require('dijit.form.Button'); // for search form
dojo.require('dijit.form.CheckBox'); // radio button is in here
dojo.require("dojo.i18n");
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "links");

/**
 * Shows and hides the login box and welcome area panels. Observes the
 * date-profile, login, and logout signals.
 */
dojo.declare("hub.widgets.chrome.LoginBox", [dijit.layout.StackContainer, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/LoginBox.html"),
    // dojo.form.Button and dojo.layout.ContentPane in the template
    widgetsInTemplate: true,
    // resize login box to content, not the other way around
    doLayout: true,
    
    loginURL: "#", //"/mashuphub/secured",
    logoutURL: "#", //"/mashuphub/client/ui/logout",

    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.links = dojo.i18n.getLocalization('hub', 'links');

		// get login URL
		_this = this;
		dojo.xhrGet({
			//url: "https://localhost:8443/mashuphub/client/ui/searchtypes",
			//url: "../scripts/hub/widgets/json",
			url: hub.config.base_url + "/ui/loginURL",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.loginURL = response.loginURL;
				_this.logoutURL = response.logoutURL;
			},
			error: function(response, args) {
				console.log('An error occurred when retrieving login info from the server.');
				
				var sUrl = hub.config.base_url;
				var pos=sUrl.indexOf("/client");
				if (pos > 0) {
				   sUrl = sUrl.substring(0, pos);
				   _this.loginURL = sUrl + '/secured';
				   _this.logoutURL = sUrl + '/secured/logout';
				}
			}
		});

        // decide what to show first ... TODO remake 
        var logged_in = (hub.config.username && hub.config.username != '');
        //alert('logged in:' + logged_in);
        if(logged_in) {
	        if ( hub.config.isAdmin ) {
	            this.showSettings = 'true';
	            this.login_selected = 'false';
    	        this.welcome_selected = 'false';
	        } else {
	        	this.welcome_selected = 'true';
	            this.login_selected = 'false';
				this.showSettings = 'false';
	        }
        } else {
            this.login_selected = 'true';
            this.welcome_selected = 'false';
			this.showSettings = 'false';
        }

        // interpolate values
        this.inherited(arguments);
    },

    postCreate: function() {
        // show initial welcome
//        this._doWelcome(hub.config.username);

        // listen to bus events
        dojo.subscribe('hub.login', this, '_onLogin');
        dojo.subscribe('hub.logout', this, '_onLogoutDone');
        dojo.subscribe('hub.update-profile', this, '_onProfile');
        
        this.inherited(arguments);
    },
/*
    _doWelcome: function(name) {
        // set username
        var msg = dojo.string.substitute(this.panels.LOGIN_WELCOME, [name]);
        this.welcome_message.innerHTML = msg;
    },
*/    
    _onLogin: function(name) {
        // show logged in user
//        this._doWelcome(name);
        // show welcome pane
//        this.forward();
        // clear entered username and pass
//        this.login_form.reset();        
    },
    
    _onLogoutDone: function() {
        // show the login box again
        this.back();
    },
    
    _onProfile: function(signal, name) {
        this.welcome.innerHTML = name;
    },

    _onSubmit: function(event) {
    alert(this.loginURL);
    	window.location = this.loginURL; 
//        var form = new hub.submit.Login(this.login_form, this.login_message);
        // safe submit, catches all exceptions
//        hub.submit.ajaxSubmit(form);
    },

    _onShowRegister: function(event) {
        hub.showPage(this.labels.LOGIN_REGISTER_TAB, 'Register');
    },

    _onShowSettings: function(event) {
        hub.showPage(this.labels.LOGIN_SETTINGS_TAB, 'AdminMigrateDatabase');
    },

    _onShowHelp: function(event) {
		hub.showUrl(this.labels.LOGIN_HELP_TAB, 'doc/userhelp/hubhelp/help_hub_overview.htm');
    },

    _onLogout: function(event) {
        hub.submit.ajaxSubmit(new hub.submit.Logout());
    }
});

/**
 * Initiates a search of the catalog on user request. Just a controller, doesn't
 * actually have anything to do with submitting the search terms.
 */
dojo.declare("hub.widgets.chrome.SearchBox", [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/SearchBox.html"),
    // dojo.form.button, etc. in the template
    widgetsInTemplate: true,

    postMixInProperties: function() {
        // fetch all localized text
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.popups = dojo.i18n.getLocalization('hub', 'popups');
        
        // interpolate values
        this.inherited(arguments);
    },

    _onSubmit: function(event) {
        var term = this.form.searchTerm.value;
        if(term == null || term == "") {
            // default to all text
            term = "*";
        }
        var collection = dijit.byId("hubSelectSearchType").getValue();
        if(collection) {
        	if (collection == "favorites") {
            	hub.showPage(term, 'Catalog', {'collection': "ui.all", searchType: "favorites", term: term});
            }
            else {
            	hub.showPage(term, 'Catalog', {'collection': collection, searchType: "", term: term});
            }
        } else {
            hub.showPage(term, 'Catalog', {'collection': "ui.all", searchType: "", term: term});
        }
        return;
    }
});

dojo.declare("hub.widgets.chrome.DropDownSelect", dijit.form.DropDownButton, {
	showLabel: true,
	store: null,
	dropDown: null,
	_isPopulated: false,
	_lastOption: "",
	onChange: function(){},
	items: null,

	postMixInProperties: function(){
		this.inherited(arguments);

		// try
		_this = this;
		dojo.xhrGet({
			//url: "https://localhost:8443/mashuphub/client/ui/searchtypes",
			//url: "../scripts/hub/widgets/json",
			url: hub.config.base_url + "/ui/searchtypes",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.items = response.types;
			}
		});
		// 
		var span = dojo.doc.createElement("span"), selItem = null;
		if (!this.store)
		{
//			var items = dojo.query("> option", this.srcNodeRef).map(function(node){
//					node.style.display="none";
//					return { value: node.getAttribute("value"), name: String(node.innerHTML) };
//			});
//			var items = this.types;
			this.store = new dojo.data.ItemFileReadStore({data: {identifier:"value", items:this.items}});
		}
		var selItem;
		if(this.items && this.items.length && !this.value)
		{
			if ( this.srcNodeRef.selectedIndex && this.srcNodeRef.selectedIndex != -1 ) {
				selItem = this.items[this.srcNodeRef.selectedIndex];
			} else {
				var _selIndex = 0;
				for (var i = 0; i < this.items.length; i++) {
					if (this.items[i].value && this.items[i].value.indexOf("all") != -1) {
						_selIndex = i;
					}
				}
				selItem = this.items[_selIndex];
			}
		}
		this._initSelect(selItem, span);
		if (!this.dropDown)
		{
			this.dropDown = new dijit.Menu();
		}
		dojo.place(span, this.srcNodeRef, "first");
	},

	_initSelect: function(item, span){
		if (item)
		{
			this.value = item.value;
			span.innerHTML = item.name;
		}
		else if (this.store)
		{
			var _this = this;
			this.store.fetch({onComplete: function(i, r){ 
				if (i.length && i[0]) _this._initSelect(i[0], span);
			}});
		}
	},
	_menuItemClick: function(item){
//		alert(item.name);
		var val = item.value;
		if (this._lastOption != val)
		{
			this.setValue(val);
			this.setLabel(item.name); //-- to avoid jumping html elements, 
//			but can uncommented, if the widget will be enclosed into container with fixed
//			size		
			this._lastOption = val;
			this.onChange(val);
			dojo.forEach(this.dropDown.getChildren(), function(ch){
				if (ch.label == item.name){
					dojo.addClass(ch.iconNode, "selectedIcon");
				} else {
					dojo.removeClass(ch.iconNode, "selectedIcon");
				}
			});
		}
	},
	_loadCallback: function(items, request){
		var dropDown = this.dropDown, _this = this;
		if (!dropDown) { return; }
		dojo.forEach(_this.items, function(item){
			if ( item.value == _this.value ) {
			dropDown.addChild(new dijit.MenuItem({
					label: item.name, iconClass: "selectedIcon",
					onClick: function(){ _this._menuItemClick(item);}}));
			} else {
			dropDown.addChild(new dijit.MenuItem({
					label: item.name,
					onClick: function(){ _this._menuItemClick(item);}}));
			}
		});
		_this._isPopulated = true;
		hub.widgets.chrome.DropDownSelect.superclass._toggleDropDown.call(this);
	},
	_toggleDropDown: function(){
		var dropDown = this.dropDown;
		if (dropDown && !dropDown.isShowingNow && !this._isPopulated)
		{
			this.store.fetch({onComplete:dojo.hitch(this, "_loadCallback")});
			return;			
		}
		this.inherited(arguments);
	}
});