/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.edit');
dojo.require('hub.main');
dojo.require('dijit._Widget');
dojo.require('dijit._Templated');
dojo.require("dojo.i18n");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "messages");
dojo.requireLocalization("hub", "panels");

/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.Connection', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/Connection.html"),    
    conns: '',

    /**
     * Called before page loads. Fetch all localized text.
     */ 
    postMixInProperties: function() {
        //this.panels = dojo.i18n.getLocalization('hub', 'panels');
        //this.links = dojo.i18n.getLocalization('hub', 'links');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
    },
    
    /**
     * Called after page loads.
     */
    postCreate: function() {
        this.setConnections(dojo.fromJson(this.conns));
    },
    
    /**
     * Add connections to the connection list
     *
     * @param conns: array of connection objects
     */
    setConnections: function(conns) {
        this.node_source.innerHTML = "<option value='-1'>New...</option>"
        for(var i in conns) {
            var option = document.createElement('option');
            option.value = conns[i].id;
            option.innerHTML = conns[i].name;
            this.node_source.appendChild(option);
        }        
        this.conns = conns;
    },

    /**
     * Called when a connection is selected. Populates the
     * other fields on the page based on the connection selected
     */
    onSelectConnection: function(event) {
        if(!this.conns) return;
        
        var selectedIndex = event.target.selectedIndex;
        var connId = event.target.options[selectedIndex].value;
        
        if(connId == "-1") {
            this.reset();
            return;
        }
    
        for(var i=0; i<this.conns.length; i++) {
            if(this.conns[i].id != connId)
                continue;
            this.node_username.value = this.conns[i].username;
            this.node_password.value = this.conns[i].password;
        }
    },
    
    /**
     * Reset the connection fields in the form
     */
    reset: function() {
        this.node_source.selectedIndex = 0;
        this.node_username.value = "";
        this.node_password.value = "";
    }
    
});

/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.DatabaseConnection', hub.widgets.edit.Connection, {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/DatabaseConnection.html"),

    /**
     * Called when a connection is selected. Populates
     * the form fields based on the selected connection.
     */
    onSelectConnection: function(event) {
        if(!this.conns) return;
        
        var selectedIndex = event.target.selectedIndex;
        var connId = event.target.options[selectedIndex].value;
        
        if(connId == "-1") {
            this.reset();
            return;
        }
    
        for(var i=0; i<this.conns.length; i++) {
            if(this.conns[i].id != connId)
                continue;

            var connection = this.conns[i];            
            this.node_dbname.value = this.conns[i].dbname;
            this.node_dbhost.value = this.conns[i].dbhost;
            this.node_dbusr.value = this.conns[i].dbusr;
            this.node_dbpwd.value = this.conns[i].dbpwd;  
                      
            var select = this.node_dbtype;            
            for(var j=0; j<select.options.length; j++) {
                if(select.options[j].value == connection.dbtype) {
                    select.selectedIndex = j;
                    break;
                }
            }
        }
    },
    
    /**
     * Resets all connection fields in the form
     */
    reset: function() {
        this.node_dbtype.selectedIndex = 0;
        this.node_dbhost.value = "";
        this.node_dbname.value = "";
        this.node_dbusr.value = "";
        this.node_dbpwd.value = "";
    }
    
});


dojo.require('dijit.form.Button');
dojo.require('dijit.form.CheckBox');
dojo.require('hub.submit');

/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.AclAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/AclAssign.html"),    
    registered: "REGISTERED",
    anonymous: "ANONYMOUS",
    creator: "HubAdmin",
    entryid: 0,
    users: '',
    groups: '',
    lastSearch: '',
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        //users, groups, registered, anonymous, creator
        this.userDelta = {};
        this.groupDelta = {};
        this.defaultPermission = {'edit':0, 'read':1};
        //this.status = dojo.byId('saveEntryMessage_' + this.id);
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the appropriate
     * permission level of the feed (public, private, or custom)
     */
//    init: function() {
    postCreate: function() {
    	var req = new hub.submit.Request(hub.config.base_url + '/ui/acl/entryid/' + this.entryid);
    	req.submit();
        var users = [];
        var groups = [];
        var groupRegistered = [];
        var groupAnonymous = [];
        var creator = '';
        this.creator = req.response.creator;
        for(i = 0; i < req.response.user_acls.length; i++){
        	var acl = req.response.user_acls[i]
        	users[i]=[acl.username, parseInt(acl.r), parseInt(acl.e)];
        	if(acl.username == this.creator){
        		creator = users[i];
        	}
        }
        for(i = 0; i < req.response.group_acls.length; i++){
        	var acl = req.response.group_acls[i]
        	groups[i]=[acl.groupname, parseInt(acl.r), parseInt(acl.e)];
        	if(acl.groupname == this.registered){
        		groupRegistered = groups[i];
        	}else if(acl.groupname == this.anonymous){
        		groupAnonymous = groups[i];
        	}
        }
        
        //console.debug('ACL users:')
        //console.dir(users);
        //console.debug('ACL groups:')
        //console.dir(groups);
        
/*        var users = dojo.fromJson(this.users);
        var groups = dojo.fromJson(this.groups);
        delete this.users;
        delete this.groups;*/
        this.initUsers(users);      
        this.initGroups(groups);
        
        //console.debug('ACL: users & groups intialized.');
        
        // REGISTERED must be the first group returned by UserManager
        // and ANONYMOUS the second group returned
        var isPublic = (groups.length == 2
            && this.sameArray(groupRegistered,[this.registered,1,0])
            && this.sameArray(groupAnonymous,[this.anonymous,1,0])
            && users.length == 1
            && this.sameArray(creator,[this.creator,1,1]));

        var isPrivate = (groups.length == 2
            && this.sameArray(groupRegistered,[this.registered,0,0])
            && this.sameArray(groupAnonymous,[this.anonymous,0,0])
            && users.length == 1
            && this.sameArray(creator,[this.creator,1,1]));


    
        if(isPublic) {
            this.radio_public.checked = true;
        } else if(isPrivate) {
            this.radio_private.checked = true; 
        } else {
            this.radio_custom.checked = true;
            this.node_custom.style.display = 'block';
        }
    },
        
    /**
     * Render the initial users who have acl's assigned to them
     *
     * @param users: array of user/permission arrays
     */
    initUsers: function(users) {
    	//console.debug('ACL: initUsers');
        var user, arr, obj = null;
        for(var i=0; i<users.length; i++) {
            // arr = [username, read, edit, admin]
            // ex: ['jchristi', 1, 0, 0]
            arr = users[i];
            user  = arr[0];
            obj = {'read':arr[1], 'edit':arr[2]};
            // skip creator or existing user
            if(user != this.creator && !this.userDelta[user]) {
                this.addUser(user);
                this.setPermissionCheckboxes(user, obj);
            }
            this.userDelta[user] = obj;
        }
        this.udelta.value = dojo.toJson(this.userDelta);   
        //console.debug('ACL: End of initUsers');     
    },
        
    /**
     * Render the initial groups who have acl's assigned to them
     *
     * @param users: array of group/permission arrays
     */
    initGroups: function(groups) {
    	//console.debug('ACL: initGroups'); 
        var group, arr, obj = null;
        for(var j=0; j<groups.length; j++) {
            // arr = [group, read, edit]
            // ex: ['REGISTERED', 1, 0]
            arr = groups[j];
            group = arr[0];
            obj = {'read':arr[1], 'edit':arr[2]}; 
            // skip duplicate groups
            if(!this.groupDelta[group]) {
            	this.addGroup(group);
                this.setPermissionCheckboxes(group, obj);
            }
            
            this.groupDelta[group] = obj;
        }
        
        this.gdelta.value = dojo.toJson(this.groupDelta);
        //console.debug('ACL: End of initGroups'); 
    },
    
    /**
     * Called when a permission level is checked (public, private, custom)
     */
    radioButtonClick: function(event) {
        this.toggleCustomSettings(event.target);
        this.setPermissions(event.target.value);
    },
        
    /**
     * Show or hide the custom permissions menu
     */
    toggleCustomSettings: function(input) {
        if(input.value == 'custom' && input.checked)
            this.node_custom.style.display = 'block';
        else
            this.node_custom.style.display = 'none';
    },
        
    /**
     * Set initial permissions for given permission level
     *
     * @param permissionLevel: permission level "public", "private", or "custom"
     */
    setPermissions: function(permissionLevel) {                    
        var regObj;
        var anonObj;            
        if(permissionLevel == "public") {
            regObj  = {'edit':1, 'read':1};
            anonObj = {'edit':0, 'read':1};
        } else if(permissionLevel == "private") {
            regObj  = {'edit':0, 'read':0};
            anonObj = {'edit':0, 'read':0};
        } else { // permissionLevel == "custom"
            regObj  = {'edit':0, 'read':1};
            anonObj = {'edit':0, 'read':1};
        }
        
        this.userDelta = {};
        this.userDelta[this.creator] = {'edit':1, 'read':1};
        this.groupDelta = {}; 
        this.groupDelta[this.registered] = regObj;
        this.groupDelta[this.anonymous] = anonObj;
        this.udelta.value = dojo.toJson(this.userDelta);
        this.gdelta.value = dojo.toJson(this.groupDelta);
    },
        
    /**
     * Make a call to the server to search for a user or users
     * Callback function is handleSearch()
     */
    searchUsers: function() {
//        this.connect(this.node_add, 'onclick', 'addUsers'); 
		this.lastSearch = 'users';
        var user = this.node_search.value;
        
        if(user == "") {
          //alert(TOM.i18n.man._('EMPTY_SEARCH'));
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }  
        else if(user.indexOf('*')!=-1 || user.indexOf('?')!=-1) {
          hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
          return;
        }
        
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(user);
/*        var form = new hub.submit.Form({'user':user}, 'saveEntry_' + this.id, url);
        console.dir(form);
        hub.submit.ajaxSubmit(form);*/
        var req = new hub.submit.Request(url, {user: user});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Make a call to the server to search for a group or groups
     * Callback function is handleSearch()
     */
    searchGroups: function() {
//        this.connect(this.node_add, 'onclick', 'addGroups');
		this.lastSearch = 'groups';
        var group = this.node_search.value;
        
        if(group == "") {
          // use saveEntryMessage instead of popup
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }        
        else if(group.indexOf('*')!=-1 || group.indexOf('?')!=-1) {
           hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
           return;
        }
        
        this.spinner.style.visibility = 'visible';
        var url = hub.urls.getGroupSearch(group);
/*        var form = new hub.submit.Form({'group':group}, 'saveEntry_' + this.id, url);
        this.connect(form, '_onLoad', 'handleSearch');
        hub.submit.ajaxSubmit(form);*/
        var req = new hub.submit.Request(url, {group: group});
        this.connect(req, '_onLoad', 'handleGroupsSearch');
        req.submit();
        
    },
    
    /**
     * Callback function after a user or group search. List the 
     * results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function (response){
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for(var key=0; key < users.length; key++) {
        	if(users[key].username != this.creator){
				var option = dojo.doc.createElement('option');
				option.setAttribute('value', users[key].username);
				option.innerHTML = users[key].fullname;
				selected.appendChild(option);
            }    
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Callback function after a user or group search. List the 
     * results in an multi-select form
     *
     * @param response: array of users
     */
    handleGroupsSearch: function (response){
    	var groups = dojo.fromJson(response).groups;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for(var key=0; key < groups.length; key++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', groups[key]);
			option.innerHTML = groups[key];
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    addItems: function(){
    	switch(this.lastSearch){
    		case 'users':
    			this.addUsers();
    			break;
    		case 'groups':
    			this.addGroups();
    			break;
    	}
    },
        
    /**
     * Add selected users to the acl list
     */
    addUsers: function() {
        var selected = this.node_selected;
        for(var i=0; i<selected.options.length; i++) {
            if(selected.options[i].selected) {
                var username = selected.options[i].value;
                var fullname = selected.options[i].innerHTML;
                this.addUser(username, fullname);
            }
        }
        this.udelta.value = dojo.toJson(this.userDelta);
    },
        
    /**
     * Add selected groups to the acl list
     */
    addGroups: function() {
        var selected = this.node_selected;
        for(var i=0; i<selected.options.length; i++) {
            if(selected.options[i].selected) {
                var group = selected.options[i].value;
                var fullname = selected.options[i].innerHTML;
                this.addGroup(group);
            }
        }
        this.gdelta.value = dojo.toJson(this.groupDelta);
    },
        
    /**
     * Add a user to the acl list
     */
    addUser: function(user, fullname) {
        if(user == this.creator || this.userDelta[user]) 
            return;
        var id = user + '_' + this.entryid;
        var users = this.node_users;
        users.appendChild(this.addHTML(id, user, 'user', fullname));
        this.setPermissionCheckboxes(user, this.defaultPermission);
        this.userDelta[user] = this.defaultPermission;
        // call is made at end of addUsers for better performance
        // this.udelta.value = dojo.toJson(this.userDelta);
    },
        
    /**
     * Add a group to the acl list
     */
    addGroup: function(group) {
    	//console.debug('ACL: addGroup');
        if(this.groupDelta[group])
            return;
        var id = group + '_' + this.entryid;
        var groups = this.node_groups;
        groups.appendChild(this.addHTML(id, group, 'group'));
        this.setPermissionCheckboxes(group, this.defaultPermission);
        this.groupDelta[group] = this.defaultPermission;
        // call is made at end of addGroups for better performance
        // this.gdelta.value = dojo.toJson(this.groupDelta);
        //console.debug('ACL: End of addGroup');
    },
    
    /**
     * Generate the html elements for adding a user or group
     * 
     * @param id: id of the group or user
     * @param name: name of the group or user
     * @param type: "group" or "user"
     * @return: html element of the user or group
     */
    addHTML: function(id, name, type, hint) {
        var tr = document.createElement('tr');
        var td, input = null;
        var display = 'block';
        var setPermissionMethod = 'setUserPermission';
        var removeMethod = 'removeUser'; 

        if(name == this.registered  || name == this.anonymous || name == this.creator) {
            display = 'none';
        }

        if(type == 'group') {
            setPermissionMethod = 'setGroupPermission';
            removeMethod = 'removeGroup';
        }

        for(var i=0; i<4; i++) {
            td = document.createElement('td');
            td.style.paddingRight = "13px";
            tr.appendChild(td);
            input = document.createElement('input');
            
            switch(i) {
                case 0:
                    td.innerHTML = hint ? 
                    	name + ' <span style="white-space: nowrap; font: 10px; color: gray">('+hint+')</span>':
                    	name;
                    
                    break;
                case 1:
                	input.type = "checkbox";
                    input.value = 'e';
                    input.id = id + '_e';
                    td.appendChild(input);
                    // IE:  set it after this node is attached to the doc
                    //input.checked = (this.defaultPermission.edit==1)?true:false;
                    this.connect(input, 'onclick', setPermissionMethod);
                    break;
                case 2:
                	input.type = "checkbox";
                    input.value = 'r';
                    input.id = id + '_r';
                    td.appendChild(input);
                    // IE:  set it after this node is attached to the doc
                    //input.checked = (this.defaultPermission.read==1)?true:false;
                    this.connect(input, 'onclick', setPermissionMethod);
                    break;
                case 3:
                    input.type = "button";
                    input.id = id + '_remove';
                    input.value = this.labels.ACL_REMOVE;
                    input.style.display = display;
                    input.name = name;
                    td.appendChild(input);
                    this.connect(input, 'onclick', removeMethod);
                    break;
            }
        }
        
        return tr;
    },
        
    /**
     * Called when a permission check box is checked. Evaluates and sets
     * appropriate permissions and checks or unchecks other checkboxes if
     * necessary.
     */
    setUserPermission: function(event) {
        var id = event.target.id;
        var permission = event.target.value;
        var user = id.substring(0, id.indexOf('_' + this.entryid));
        var obj = this.evalPermission(permission, event.target.checked);
        this.userDelta[user] = obj;                     
        this.udelta.value = dojo.toJson(this.userDelta);            
        this.setPermissionCheckboxes(user, obj);
    },        
        
    /**
     * Called when a permission check box is checked. Evaluates and sets
     * appropriate permissions and checks or unchecks other checkboxes if
     * necessary.
     */
    setGroupPermission: function(event) {
        var id = event.target.id;
        var permission = event.target.value;
        var group = id.substring(0, id.indexOf('_' + this.entryid));
        var obj = this.evalPermission(permission, event.target.checked);
        this.groupDelta[group] = obj;
        this.gdelta.value = dojo.toJson(this.groupDelta);            
        this.setPermissionCheckboxes(group, obj);
    },
        
    /**
     * Returns a permission object based on the given permission
     *
     * @param permission: 'a', 'e', or 'r', standing for 'admin', 'edit', or 'read'
     * @param value: true or false (if the permission is checked or unchecked)
     * @return: an object representing the permission
     */
    evalPermission: function(permission, value) {
        var obj = {};         
/*        if(permission == 'a') {
            obj.admin = (value ? 1 : 0);
            obj.edit = 1;
            obj.read = 1;
        } else*/ if(permission == 'e') {
//            obj.admin = 0;
            obj.edit = (value ? 1 : 0);
            obj.read = 1;
        } else if(permission == 'r') {
//            obj.admin = 0;
            obj.edit = 0;
            obj.read = (value ? 1 : 0);
        } else { // permission == 'n'
//            obj.admin = 0;
            obj.edit = 0;
            obj.read = 0;
        }            
        return obj;
    },
        
    /**
     * Set appropriate checkboxes for a specific group or user,
     * given a permissions object
     *
     * @param user: name of the group or user
     * @param obj: permissions object for the group or user
     */
    setPermissionCheckboxes: function(user, obj) {          
//        var a = dojo.byId(user + '_' + this.entryid + '_a');
        var e = dojo.byId(user + '_' + this.entryid + '_e');
        var r = dojo.byId(user + '_' + this.entryid + '_r');         
//        a.checked = (obj.admin == 1);
        e.checked = (obj.edit == 1);
        r.checked = (obj.read == 1);
    },
        
    /**
     * Remove a user from the acl list
     */
    removeUser: function(event) {
        delete this.userDelta[event.target.name];
        this.udelta.value = dojo.toJson(this.userDelta);   
        this.remove(event);
    },
        
    /**
     * Remove a user from the acl list
     */
    removeGroup: function(event) {
        delete this.groupDelta[event.target.name];
        this.gdelta.value = dojo.toJson(this.groupDelta);   
        this.remove(event);
    },
        
    /**
     * Remove a group or user from the acl list
     */
    remove: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    },
    
    /**
     * Compares two arrays to see if they have the same values
     * Not the same as arr1 == arr2, which checks to see if it
     * is the same object.
     *
     * @arr1: first array
     * @arr2: second array to compare with the first
     * @return: true if the two arrays contain the same elements
     */
    sameArray: function(arr1, arr2) {
//    	console.log('acl.sameArray');
//    	console.dir({a1: arr1, a2: arr2});
        if(arr1.length != arr2.length)
            return false;
           
        for(var i=0; i<arr1.length; i++) {
            if(dojo.isArray(arr1[i]) && dojo.isArray(arr2[i])) {
                if(!this.sameArray(arr1[i], arr2[i]))
                    return false;
            } else {
                if(arr1[i] != arr2[i])
                    return false;
            }
        }
        return true;        
    }
});

/**
 * Builds and handles the UI for the AdminPermissions.html editor
 */
dojo.declare('hub.widgets.edit.AdminAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl('hub.widgets', 'templates/AdminAssign.html'),
    currentUser: '',
    adminList: [],
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the initial list of administrators
     */
    postCreate: function() {
    	this.adminList = [];
	   	var req = new hub.submit.Request(hub.config.base_url + '/ui/adminlist');
    	req.submit();        
        this.currentUser = req.response.currentUser;
        for (i=0; i<req.response.admins.length; i++) {
        	var id = req.response.admins[i].id;
        	this.addAdmin(id);
        	this.adminList[i] = id;
        }
    },
        
    /**
     * Make a call to the server to search for users
     * Callback function is handleSearch()
     */
    searchUsers: function() {    
        var searchSelection = this.node_search.value;
        if (searchSelection == "") {
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }  
        else if(searchSelection.indexOf('*')!=-1 || searchSelection.indexOf('?')!=-1) {
          hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
          return;
        }
        
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(searchSelection);
        var req = new hub.submit.Request(url, {user: searchSelection});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Callback function after a user search. List the results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function(response) {
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for (var i=0; i<users.length; i++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', users[i].username);
			option.innerHTML = users[i].username;
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Add selected users to the admin list
     */
    addItems: function() {
        var selected = this.node_selected;
        for (var i=0; i<selected.options.length; i++) {
            if (selected.options[i].selected) {
                var username = selected.options[i].value;
                this.addAdmin(username);
            }
        }
    },
    
    /**
     * Add a user to the admin list
     */
    addAdmin: function(id) {
    
    	// Check for duplicates
    	for (var i=0; i<this.adminList.length; i++) {
    		if (this.adminList[i] == id) {
    			return;
    		}
    	}
        var admins = this.node_admins;
        admins.appendChild(this.addHTML(id));
    },
    
    /**
     * Generate the html elements for adding a user to the admin list
     * 
     * @param id: id of the user
     * @return: html element
     */
    addHTML: function(id) {
		var tr = document.createElement('tr');
		var td, input = null;
		
		// Add name
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		td.innerHTML = id + "<input type='hidden' name='admin" + id + "' value='" + id + "'/>";
		tr.appendChild(td);
		
		// Add remove button
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		if (id != this.currentUser) {
			input = document.createElement('input');
			input.type = 'button';
			input.id = id + '_remove';
			input.value = this.labels.ACL_REMOVE;
			input.style.display = 'block';
			input.name = id;
			td.appendChild(input);
			this.connect(input, 'onclick', 'removeAdmin');
		} else {
			td.innerHTML = this.labels.ACL_CURRENTUSER;
		}
		tr.appendChild(td);		
		
		return tr;
    },
        
    /**
     * Remove a user from the admin list
     */
    removeAdmin: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    }
});

/**
 * Builds and handles the UI for the AdminBannedUsers.html editor
 */
dojo.declare('hub.widgets.edit.BannedAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/BannedAssign.html"),
    currentUser: '',
    banList: [],
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the initial list of banned users
     */
    postCreate: function() {
    	this.banList = [];
	   	var req = new hub.submit.Request(hub.config.base_url + '/ui/bannedlist');
    	req.submit();
        this.currentUser = req.response.currentUser;
        for (i=0; i<req.response.banned.length; i++) {
        	var id = req.response.banned[i].id;
        	this.addBan(id);
        	this.banList[i] = id;
        }
    },
        
    /**
     * Make a call to the server to search for users
     * Callback function is handleSearch()
     */
    searchUsers: function() {    
        var searchSelection = this.node_search.value;
        if (searchSelection == "") {
          hub.notify('Search query is empty');
          return;
        }
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(searchSelection);
        var req = new hub.submit.Request(url, {user: searchSelection});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Callback function after a user search. List the results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function(response) {
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for (var i=0; i<users.length; i++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', users[i].username);
			option.innerHTML = users[i].username;
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Add selected users to the ban list
     */
    addItems: function() {
        var selected = this.node_selected;
        for (var i=0; i<selected.options.length; i++) {
            if (selected.options[i].selected) {
                var username = selected.options[i].value;
                this.addBan(username);
            }
        }
    },
    
    /**
     * Add a user to the ban list
     */
    addBan: function(id) {
    
    	// Check for duplicates
    	for (var i=0; i<this.banList.length; i++) {
    		if (this.banList[i] == id) {
    			return;
    		}
    	}
        var bans = this.node_bans;
        bans.appendChild(this.addHTML(id));
    },
    
    /**
     * Generate the html elements for adding a user to the ban list
     * 
     * @param id: id of the user
     * @return: html element
     */
    addHTML: function(id) {
		var tr = document.createElement('tr');
		var td, input = null;
		
		// Add name
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		td.innerHTML = id + "<input type='hidden' name='ban" + id + "' value='" + id + "'/>";
		tr.appendChild(td);
		
		// Add remove button
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		input = document.createElement('input');
		input.type = 'button';
		input.id = id + '_remove';
		input.value = this.labels.ACL_REMOVE;
		input.style.display = 'block';
		input.name = id;
		td.appendChild(input);
		this.connect(input, 'onclick', 'removeBan');
		tr.appendChild(td);		
		
		return tr;
    },
        
    /**
     * Remove a user from the ban list
     */
    removeBan: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    }
});

dojo.declare('hub.widgets.edit.CategoriesCBL', [hub.widgets.CheckBoxList],{
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],
	entryid: 0,
	constructor: function(args, node){
		this.entryid = args.entryid; 
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
    	var req = new hub.submit.Request(hub.urls.getCategoriesList(this.entryid));
    	if(req.submit()){
//	    	console.dir(req.response);
    	}
//		this.init_labels = req.response.result;
		var items = req.response.rows;
		for(i in items){
			this.init_names[i] = 'cat_'+items[i].id;
			this.init_labels[i] = items[i].name;
            if( items[i].isChecked){
            	this.checked['cat_'+items[i].id]=true;
            }
		}    	
	}
});

dojo.declare('hub.widgets.edit.ObjectsCBL', [hub.widgets.CheckBoxList],{
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],
	entryid: 0,
	constructor: function(args, node){
		this.entryid = args.entryid; 
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
    	var req = new hub.submit.Request(hub.urls.getAssociationsList(this.entryid));
    	if(req.submit()){
//	    	console.dir(req.response);
    	}
//		this.init_labels = req.response.result;
		var items = req.response.rows;
		for(i in items){
			this.init_names[i] = 'rel_'+items[i].id;
			this.init_labels[i] = items[i].name;
            if( items[i].isChecked){
            	this.checked['rel_'+items[i].id]=true;
            }
		}    	
	}

});