/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.tabs.Workflow');
dojo.require('hub.main');
dojo.require('hub.submit.edit');
dojo.require('hub.widgets');
dojo.require('hub.widgets.edit');
dojo.require('hub.pages.edit');
dojo.require('hub.widgets.social.edit');
dojo.require('dijit.layout.StackContainer');
dojo.require('hub.widgets.forms.SelectPlugin');
dojo.require('hub.managers.Plugin');
dojo.require('hub.managers.InvokePlugin');
/**
 * Manages a set of child content panes. UI can drive tabs to flip foward and 
 * backward through panes. Supports caching of previous panes so they can be 
 * restored without going to the server. Also supports rendering of both 
 * plug-ins and standard template forms as separate pages in the stack.
 */ 
dojo.declare('hub.widgets.tabs.Workflow', [dijit._Widget, dijit._Templated ], {
    // @todo: implement
    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("hub","widgets/templates/Workflow.html"),
    plugin: null,
    entryId: null,
    pluginId: null,
    pluginDedicated: false,
    pluginStartScreen: null,
    saveScreen: null,
    reedit: false,
    selectPluginForm: null,
    complete: false,
    totalScreens: 0,
    currentScreen: 0,
//    processors: null,
    states: null,
    lastResponse: null,
    type: null,
    tab: null,
    saved: false,
    locked: false,
    fullscreen: false,
    editAsNew: false,
    labels: null,
    
	startup: function(){
//        console.log('>>Workflow.startup');
		this.setBusy(true);
//        console.log('<<Workflow.startup');
		this.inherited('startup', arguments);
        this.reedit = (!this.editAsNew && this.entryId != null && this.entryId > 0);
        
//        dojo.connect(this.workflowContainer, 'selectChild', this, 'setNotBusy');
        
//        console.log('this.reedit = ' + this.reedit + ' this.entryId = ' + this.entryId);
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;
/*        if(this.entryId){
        	this.reedit = true;
        }*/
		if(this.entryId && !this.editSource){
        	this._loadSaveForm({entry_id: this.entryId});
        	this.currentScreen++;
		}else if(this.pluginId 
        && (!this.entryId || this.entryId && this.editSource)){
        	this.pluginDedicated = true;
        	this._loadPluginForm();
//        	this.currentScreen++;
        }else{
        	this._loadPluginChooser();
			this.selectPluginForm.setType(this.type);
        }
        this.states = new Array();
        
        if(!this.fullscreen){
        	this.workflowNavigation.style.display = "";
        	//this.workflowContainer.style.height = "100%";
        }
        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.entryId && !this.editAsNew) {
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);
        }
        this._nextHandle = dojo.subscribe('workflow-next-'+this.tab.id, null, dojo.hitch(this, this._onNextStepComplete));
        this._doneHandle = dojo.subscribe('workflow-done-'+this.tab.id, null, dojo.hitch(this, this._onDone));
        if(this.domNode!=null){
	        this.updateControls();
	        this.workflowLayout.layout();
        }
	},
	
	postMixInProperties: function() {
	    this.labels = dojo.i18n.getLocalization('hub', 'labels');
	},

    /**
     * Watches for tab closing and cleans up any junk we may have created.
     * Watches for tab content replacement, and updates the stack pointer
     * appropriately.
     *
     * @param signal Always 'invalidate-tab-' plus the entryId
     * @param payload Object having an action that is either 
     *   'replace' or 'close'
     */
    _onInvalidate: function(signal, payload) {
        if(payload.action == 'close') {
            // only dispose if closing, not replacing with new content
            this.cancel(payload);
        }
    },


    /**
     * Performs a diff detection for changes based on all values in a form.
     * Checks to see if the plug-in has methods
     *
     */
    _cmpStates: function(s1, s2) {
        if(s1 == null || s2 == null) {
            // not objects, just compare if both are null
            return s1 == s2;
        }
        // check that all values in s1 are equal to those in s2
        for(var key in s1) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        // check that all values in s2 are equal to those in s1
        // NOTE: have to do this step incase size(s2) > size(s1) but we have
        //   no length property on objects
        for(var key in s2) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        return true;
    },

    /**
     * Gets the state of a single screen in workflow for later diff 
     * detection when we must decide if we should go to the server for the
     * the next page or use a cached copy.
     *
     * @return State object that will be compared later
     */
    _getScreenState: function() {
//    console.log('Workflow._getScreenState()');
        var screen = this.workflowContainer.selectedChildWidget.domNode;

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        var state = null;
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
//            console.dir({form: form});
            state = dojo.formToObject(form);
            var files = dojo.query("input[type=file]", form);
            for(var i = 0 ; i < files.length; i++){
            	state[files[i].name] = files[i].value;
            }
            
            
        }
        return state;
    },

	setTab: function(tab){
		this.tab = tab;
	},
	
	getTab: function(){
		return this.tab;
	},

    _onScreenAdded: function() {
        // parse tab for dojo content
//        console.log('workflow._onScreenAdded');
        var children = this.workflowContainer.getChildren();
//        dojo.parser.parse(children[children.length-1].domNode);
		if(dojo.isIE > 0){
	        hub.evaluate(children[children.length-1].domNode, true, true);
        }else{
	        dojo.parser.parse(children[children.length-1].domNode);
        }
        // set tab unbusy
//        console.log('>>workflow._onScreenAdded');
        this.setBusy(false);        
//        console.log('<<workflow._onScreenAdded');
    },
    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param node DomNode to initialize on the screen
     * @param options abortRender|showControls|pluginStart
     * @return New screen DOM node
     */
    appendScreenDOM: function(node, options) {
//		console.log('Workflow.appendScreenDOM');
		if(options == null){
			options = {};
		}

        var newTab =  new dojox.layout.ContentPane({
		                        title:'newChild',
		                        refreshOnShow:true
			                }
				        );
       
        // find my tabContainer and add our new ContentPane
        newTab.domNode.appendChild(node);
        newTab.resize = true;
        this.workflowContainer.addChild(newTab);
        newTab.domNode.style.width = "100%";
        newTab.domNode.style.height = "100%";
        newTab.domNode.style.marginTop = "0px";
        newTab.domNode.style.marginRigth = "0px";
        newTab.domNode.style.marginBottom = "0px";
        newTab.domNode.style.marginLeft = "0px";
        newTab.domNode.style.paddingTop = "0px";
        newTab.domNode.style.paddingRigth = "0px";
        newTab.domNode.style.paddingBottom = "0px";
        newTab.domNode.style.paddingLeft = "0px";
        // evaluate Dojo only, JS seems to be evaluated for us by the tab
//        hub.evaluate(newTab, true, true);
        this.totalScreens++;
        if(options.pluginStart==null && options.pluginStart){
        	this.pluginStartScreen = this.currentScreen;
        }
        if(options.showControls==null){
        	newTab.showControls = this.fullscreen?false:true;
        }else{
        	newTab.showControls = options.showControls;
        }
        if(options.abortRender==null){
        	this._onScreenAdded();
        }
        return newTab;
    },

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param html Content to initialize on the screen
     * @return New screen DOM node
     */
    appendScreenHTML: function(html, options) {
// 		alert('appendScreenHTML');
        // create, populate, append element
        var div = document.createElement('DIV');
        // to make sure we don't mess up large editors
        if(this.fullscreen) {       
        	div.style.height = "100%";
       	} else {
			div.style.height = "1%"; 
       	}
        div.style.width = "100%";
        
        div.innerHTML = html;
        var result = this.appendScreenDOM(div, options);
        if (dojo.isSafari) {
        	var lowercase = html.toLowerCase();
        	for(;;) {
        		if (lowercase.indexOf("<script") < 0) {
        			break;
        		}
       			var sstart = lowercase.indexOf("<script");
       			var start = lowercase.indexOf(">", sstart);
       			var end = lowercase.indexOf("</script", sstart);
       			if ((end < 0) || (start < 0)) {
       				console.warn("Incomplete Script: " + html.substring(sstart));
       				break;
       			}
       			start = start + 1;
       			var js = html.substring(start, end);
       			html = html.substring(end + 1);
       			lowercase = html.toLowerCase();
       			dojo.eval(js);
        	}
        }
        return result;
    },

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param page_cls Templated class to initialize on the screen
     * @return New screen DOM node
     */
 	appendScreenDijit: function(/*page_cls*/ dijit_cls, args){
// 		alert('appendScreenDijit');
        var div = document.createElement('DIV');

        var widget = new dijit_cls(args, div);

        return this.appendScreenDOM(div, args);
	},

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param page_cls Templated class to initialize on the screen
     * @return New screen DOM node
     */
 	appendScreenDjango: function(/*page_cls*/ page_cls, args){
// 		alert('appendScreenDjango');
        var attach_div = document.createElement('DIV');
        var page_div = document.createElement('DIV');
        attach_div.appendChild(page_div);
//        console.log('>>workflow.appendScreenDjango');
        this.setBusy(true)
//        console.log('<<workflow.appendScreenDjango');
        args.abortRender = true;
        
        var newTab = this.appendScreenDOM(attach_div, args);
        // create attach point widget
        var ap = new dojox.dtl.AttachPoint(null, attach_div);
        // create page widget
        args.tab = this.tab;
        var p = new page_cls(args, page_div);
        // listen for render complete
        p.isRendered().addCallback(dojo.hitch(this, this._onScreenAdded));
        
        return newTab;
	},

	onCancel: function(){
//		console.log('workflow.onCancel')
		this._callSaveCallback({cancel: true, action: 'cancel'});
		if(this.reedit){
			this.tab.setPage(hub.pages.Details, {object_id: this.entryId});
			this.dispose;
		}else{
			hub.closePage(this.tab);
		}
	},
	
    /**
     * Contacts the server to delete the current entry if it's new.
     *
     * @param payload Contains information about a close signal
     */
    cancel: function(payload) {
//    	console.log('Workflow.cancel');
    	if(this.locked){
    		return false;
    	}
        // only need to delete if defining a new entry and we have a 
        // plug-in reference
//		console.log('this.reedit = ' + this.reedit);

		if (this.reedit) {
			// make cancel call (restore an entry data from temporary table, if the entry has been opened for editing source)
			// server-side code will check if really nessesary to restore data 
	        try {
	            var cancel_url = hub.config.base_url + "/plugin/cancel/entryid/" + this.entryId + "/pluginid/" + this.pluginId;
	            var form = new hub.submit.edit.Cancel(cancel_url, null, null);
	            form.submit();
	        } catch (e) {}
		}
		
        if(this.plugin && !this.reedit) {
            // delete existing entry, we already asked for confirmation
            var delete_url = hub.config.base_url + "/plugin/delete/entryid/" +
                this.entryId + "/pluginid/" + this.pluginId;
            var form = new hub.submit.edit.DeleteAlways(null, null, delete_url);
            try {
                form.submit();
            } catch (e) {
//                this.showMessage(e);
                hub.notify(e);
            }
            // continue with final dispose operations
            this.dispose();
        } else if(payload.external) {
            // if triggered by an external source, go back to details page,
            // user is not trying to explicitly close the tab

            // continue with final dispose operations
            // important that his comes before reuse of tab
            this.dispose();

            // cancel any close operation
            this.domNode.setAttribute('abort_close', true);

            // go to the details page
            var entryId = this.domNode.getAttribute('object_id');
            var title = this.domNode.getAttribute('title');
/*            TOM.tabs.man.showTab(hub.config.base_url+'/ui/details/entryid/'+entryId, // url
                                 title, // title 
                                 entryId, // object
                                 true, // current tab
                                 true); // doesn't matter if safe or not*/
			hub.showUrl(title, hub.config.base_url+'/ui/details/entryid/'+entryId, {safe: true});                             
                    
        } else {
            // user is explicitly trying to close the tab
            // continue with final dispose operations
            this.dispose();
        }
    },

    /**
     * Completes editing by disposing of this object and any plug-in.
     */
    dispose: function() {
        // remove as listener else we're still hanging around
//        console.log('Workflow.dispose')
/*        TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                   'invalidate-tab-'+this.entryId);*/
                                   
        dojo.unsubscribe(this._observerHandle);
        dojo.unsubscribe(this._nextHandle);
        dojo.unsubscribe(this._doneHandle);

        // inform a plug-in that it is also complete
        if(this.plugin) {
            this.plugin.dispose();
            this.plugin = null;
        }

        // for good measure, dump our history
        this.history = new Array();
        this.screens = new Array();
//        this.complete = true;
        this.setComplete();

        // remove us from the tab, avoid cyclic invocation of this method
//        this.domNode.delAttribute('workflow', true);
    },
    
    setBusy: function(/*boolean*/ busy){
//    	console.log('>>workflow.setBusy('+busy+')');
    	this.locked = busy;
		this.tab.setBusy(busy);
//    	console.log('<<workflow.setBusy('+busy+')');
		
		return busy;
    },
    
    setNotBusy: function(){
//     	console.log('>>workflow.setNotBusy');
    	this.setBusy(false);
//     	console.log('<<workflow.setNotBusy');
    },
    
    /**
     * Go to the next screen in the workflow. This might mean getting it
     * from the server or using a hidden panel if the current screen has
     * not changed in any way.
     *
     * @param func Function to invoke if we have to go to server
     */
    next: function() {
		console.log('>>workflow.next');
    	if(this.locked){
    		return false;
    	}
    	this.setBusy(true);
		console.log('<<workflow.next');
    	console.log('totalScreens='+this.totalScreens+' > currentScreen='+this.currentScreen);
   		var currentState = this._getScreenState();
   		var savedState = this.states[this.currentScreen];
    	if(this.currentScreen < this.totalScreens){
    	console.log('it is not the last screen');
    	//  it is not the last screen
    		console.dir({currentState:currentState,savedState:savedState});
	   		if(this._cmpStates(currentState,savedState)){
	    	console.log('>>the state hasn`t changed');
	   		//  the state hasn`t changed
   				this.workflowContainer.forward();
   				this.currentScreen++;
		    	this.updateControls();
		    	this.setBusy(false);
//	    	console.log('<<the state hasn`t changed');
   				return true;
   			}else{
	    	console.log('the state has changed');
	   		//  the state has changed
	   			this.saved = false;
   				var nextTab=null;
//   				while(this.totalScreens>this.currentScreen+1){
   				while(this.totalScreens>this.currentScreen){
		    	console.log('deleting next screens');
   				//  deleting next screens
   					nextTab = this.workflowContainer.getChildren()[this.totalScreens-1];
   					nextTab.destroyDescendants();
   					this.workflowContainer.removeChild(nextTab);
   					this.totalScreens--;
   					if(this.currentScreen < this.saveScreen+1){
   						this.complete = false;
   					}
//   					console.log('next false-ing complete');
   				}
   			}
    	}
    	var processor = null;//this.processors[this.currentScreen];
//		console.log('complete = ' + this.complete);
    	if(!this.isComplete()){
    	console.log('workflow is not completed');
			if(!this.pluginDedicated && this.currentScreen==1){    	
//    		console.log('it is time to load plugin form');
		        processor = dojo.hitch(this, this._loadPluginForm);
		    }else{
    		console.log('submiting current step');
		        processor = dojo.hitch(this, this._submitCurrentForm);
		    }
    	}else{
	   	console.log('workflow completed saving current entry');
	        processor = dojo.hitch(this, this._saveCurrentEntry);
    	}
		var result = processor();
   		return true;
    },


    /**
     * Go to the previous screen in the workflow. Doesn't contact the server
     * in any way. Just shows a previously hidden panel.
     */
    previous: function() {
    	if(this.locked){
    		return false;
    	}
    	var fromSave = ((this.saveScreen != null) && (this.saveScreen + 1 == this.currentScreen));
    	if (fromSave) {
    		this._callSaveCallback({cancel: false, action: "prev"});
    	}	
		this.currentScreen--;
    	this.workflowContainer.back();
//        console.log('Workflow.previous -> updateControls');
    	this.updateControls();
    },
    
	_onDone: function(){
   		var currentState = this._getScreenState();
   		this.states[this.currentScreen] = currentState;
		//	the entry was inserted, now it should be saved via saveForm
		this._loadSaveForm({entry_id: this.entryId});
		//	show the next screen
		this.workflowContainer.forward();
		this.currentScreen++;
//        console.log('>>Workflow.onDone');
		this.updateControls();
		this.setBusy(false);
//        console.log('<<Workflow.onDone');
		return true;
	},
    
    _onNextStepComplete: function(){
//    console.log('Workflow._onNextStepComplete');
   		var currentState = this._getScreenState();
   		this.states[this.currentScreen] = currentState;
   		if(this.lastResponse.method){
   			if(this.lastResponse.method =='done'){
   			//	the entry was inserted, now it should be saved via saveForm
				this._loadSaveForm({entry_id: this.entryId});
	//			this.processors[this.currentScreen] = dojo.hitch(this, this._saveCurrentEntry);
			}else{
//				this.plugin[this.lastResponse.method](this.lastResponse.payload);
			} 
   		}else{
   			if(this.isComplete()){
	   		//	the entry was saved via saveForm
	   			if(this.reedit){
				//  show details page (it will be rendered, as tabs.refreshAllTabs() will be called as a result of hub.save-entry event)	   			
			        var page_cls = hub.pages.Details;
			        
			        this.tab.page_args = {};
			        this.tab.page_args.object_id = this.entryId;
					this.tab.page_args.tab = this.tab;
					this.tab.page_cls = page_cls;
					this.dispose();				
		   			return true;
	   			}else{
			        var page_cls = hub.pages.edit.Landing;
			        
			        // ask page *class* for properties (static method call)
			        var props = page_cls.prototype.getProperties();
			        // mix given args on top of class-specified args
			        //dojo.mixin(props, args);
			        props.entry_id = this.entryId;
			        props.plugin_id = this.pluginId;
			        props.entry_title = 'New entry'; // @TODO Put title from Save here 
					props.tab = this.tab;
			        // get/create tab
			        // instruct the tab to render the page when appropriate
			        props.showControls = false;
			        this.appendScreenDjango(page_cls, props);
	   			}
   			}
   		}
		//	show the next screen
		this.workflowContainer.forward();
		this.currentScreen++;
//        console.log('>>Workflow._onNextStepComplete()');
		this.updateControls();
		this.setBusy(false);
//        console.log('<<Workflow._onNextStepComplete()');
		return true;
	},

	_loadPluginChooser: function(){
        var div = document.createElement('div');
        var newTab = this.appendScreenDOM(div);

        var pluginChooser = new hub.widgets.forms.SelectPlugin({type:this.type}, div);
        
        this.selectPluginForm = pluginChooser;
        dojo.connect(this.selectPluginForm.inputSelect, 'onchange', this, 'releaseNextButton');
        this.buttonNext.setDisabled(true);
		this.workflowContainer.selectChild(newTab);
//		console.log('>>workflow._loadPluginChooser');
    	this.setBusy(false);
//		console.log('<<workflow._loadPluginChooser');
    	this.currentScreen++;
        return newTab; 
	},

	/**
	 * load first plugin page
	 */
    _loadPluginForm: function(){
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    if(!this.pluginDedicated){
		    this.pluginId = this._getScreenState().pluginid;
	    }
	    var url = hub.config.base_url + '/plugin/editor/json/1';
	    if(this.entryId > 0){
	    	url = url +'/entryid/'+this.entryId;
	    }
//	    var form = new hub.submit.Form(	this.pluginDedicated?null:this.selectPluginForm.domNode,
	    var form = new hub.submit.Form(	null, null,	hub.config.base_url + '/plugin/editor/json/1');
	    if(this.pluginDedicated){
	    	form.content = {pluginid:this.pluginId, entryid: this.entryId, cloneentry: this.editAsNew};
	    }else{
	    	form.content = dojo.formToObject(this.selectPluginForm.domNode);
	    }
	    form._onLoad = dojo.hitch(this, function(response){
            var labels = dojo.i18n.getLocalization('hub', 'labels');
	        var response = dojo.fromJson(response);
	       	this.lastResponse = response;
	       	this.entryId = response.result.object_id;
		    this.tab.entryId = this.entryId;
		    if(this.entryId && !this._observerHandle) {
               this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);
            }
	       	if(!this.reedit && !this.editAsNew){
	       		var label = '';
	       		switch(this.type){
	       			case 'Mashups': 
	       				label = labels.NEW_MASHUP_TAB;
	       				break;
	       			case 'Widgets': 
	       				label = labels.NEW_WIDGET_TAB;
	       				break;
	       			case 'pages': 
	       				label = labels.NEW_PAGE_TAB;
	       				break;
	       			case 'Feeds':
	       			default: 
	       				label = labels.NEW_FEED_TAB;
	       				break;
	       		}
		       	hub.renamePage(this.tab, label+' '+this.entryId);
	       	}
	       	this.plugin = new hub.managers.InvokePlugin(this.pluginId, this.entryId, this);
			this.appendScreenHTML(this.lastResponse.result['html'], {pluginStart:true});
			this.pluginStartScreen = this.currentScreen;
//		console.log('>>workflow._loadPluginForm');
//        	this.setBusy(false);
//		console.log('<<workflow._loadPluginForm');
           	dojo.publish('workflow-next-'+this.tab.id ,[]);
	    });

	    hub.submit.ajaxSubmit(form);
	    return true;
    },

    _submitCurrentForm: function() {
//        var screen = this.workflowContainer.getChildren()[this.currentScreen].domNode;
//        var screen = this.workflowContainer.getChildren()[this.currentScreen-1].domNode;
//		console.log('_submitCurrentForm');
        var screen = this.workflowContainer.selectedChildWidget.domNode;
        // first see if the plug-in wants to give us a custom state

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
//        var state = new Object();
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
            //form.submit();
            var func = form.onsubmit;
            func();
//            state = dojo.formToObject(form);
        }
        return null;//state;
    },

    _loadSaveForm: function(args) {
        dojo.require('hub.pages.edit');
        args.showControls = true;
        this.appendScreenDjango(hub.pages.edit.Save, args);
        this.saveScreen = this.currentScreen;
        this.setComplete();
//		console.log('load save complete = ' + this.complete);
		
		
    },

	_saveCurrentEntry: function(){
//        var screen = this.workflowContainer.getChildren()[this.currentScreen].domNode;
//        var screen = this.workflowContainer.getChildren()[this.currentScreen-1].domNode;
//		console.log('_saveCurrentEntry');
        var screen = this.workflowContainer.selectedChildWidget.domNode;
        // first see if the plug-in wants to give us a custom state

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
		    var saveForm = new hub.submit.edit.Save(form,
		    	 							null, 
		    	 							hub.config.base_url + '/plugin/update/entryid/' + this.entryId + '/pluginid/' + this.pluginId);
		    saveForm._onLoad = dojo.hitch(this, function(response){
				this.lastResponse = response;
				this.saved = true;
				this._callSaveCallback({cancel: false, action: "save"});
				if (form && form.title) {
				   // _renamePage calls htmlspecialchars on title...
				   hub.renamePage(this.tab, form.title.value);
				}
				dojo.publish('workflow-next-'+this.tab.id,[]);
//				console.log('workflow._saveCurrentEntry -> publishing events');
		        dojo.publish('hub.save-entry', [this.entryId]);
//		        dojo.publish('hub.edit-entry', [this.entryId]);
		    });
	
		    hub.submit.ajaxSubmit(saveForm);
//		console.log('>>workflow._saveCurrentEntry');
//		    this.setBusy(false);
//		console.log('<<workflow._saveCurrentEntry');
	    }
	    
	    return true; //result;
	},

	updateControls: function(){
//    	this.buttonPrevious.disabled = this.currentScreen==1?"true":"";
    	this.buttonPrevious.setDisabled(this.currentScreen==1? true: false);
    	var finished = this.isComplete() && this.currentScreen == this.saveScreen+1;
//    	console.log('Workflow.updateControls: isComplete()=' + this.isComplete() 
//    										+ ' currentScreen='+ this.currentScreen + '==' 
//    										+ ' totalScreens=' + this.totalScreens);
/*    	if(finished){
    		this.buttonNext.innerHTML = "Finish";
    	}else{
    		this.buttonNext.innerHTML = "Next";
    	}*/
    	this.buttonNext.setLabel(finished?this.labels.BUTTON_FINISH:this.labels.BUTTON_NEXT);
    	
//    	if(this.fullscreen && !finished){
		if(this.workflowContainer.selectedChildWidget!=null){
	    	if(!this.workflowContainer.selectedChildWidget.showControls){
	    	   	this.workflowNavigation.style.display = "none";
	    	}else{
	    	   	this.workflowNavigation.style.display = "";
	    	}
    	}
//    	this.workflowLayout.resize();
        hub.resizeLayoutContainers(this.workflowLayout); // MH-87
	},
	
	releaseNextButton: function(event){
        this.buttonNext.setDisabled(false);
	},

    /**
     * Go to the first screen in the plug-in editor if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showPlugin: function() {
    	var pluginStartTab = this.workflowContainer.getChildren()[this.pluginStartScreen]; 
    	this.workflowContainer.selectChild(pluginStartTab);
    	this.currentScreen = this.pluginStartScreen+1;
//    	this.complete = false;
    	this.updateControls();
    },

    /**
     * Go to the save form if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showSave: function() {
    	var saveTab = this.workflowContainer.getChildren()[this.saveScreen]; 
    	this.workflowContainer.selectChild(saveTab);
    	this.currentScreen = this.saveScreen+1;
//    	this.complete = true;
    	this.updateControls();
    },

    /** 
     * Stores a reference to the plug-in that is part of the workflow.
     * 
     * @param plugin The plugin
     */
    setPlugin: function(plugin) {
        // make sure we throw away any old observer for an old object id
        if(this.entryId) {
            // remove as listener else we're still hanging around
            /*TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                       'invalidate-tab-'+this.entryId);*/
            dojo.unsubscribe(this._observerHandle);
        }
        this.plugin = plugin;
        // for convenience, pull object id and plug-in id into this object
        this.entryId = this.plugin.getObjectId();
        this.pluginId = this.plugin.getPluginId();
        // track at which part in the history the plug-in was shown
        this.plugin_ptr = this.history_ptr+1;

        // listen for close events on the tab because we need to clean-up
/*        TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                'invalidate-tab-'+this.entryId);*/
		this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);                
    },

    /** 
     * Gets a reference to the plug-in that is part of the workflow.
     */
    getPlugin: function() {
        return this.plugin;
    },

    /**
     * Gets the number of screens in the history.
     *
     * @return integer count
     */
    getHistorySize: function() {
        return this.history.length;
    },

    /**
     * Sets a flag indicating this workflow has been run to 
     * completion. The flag is unset if the workflow moves to any screen
     * after the flag is set.
     */
    setComplete: function() {
        this.complete = true;
    },

    /**
     * Gets the value of the completion flag.
     */
    isComplete: function() {
        return this.complete;
    }, 
    
    isModified: function(){
    	return (this.plugin!=null || this.saveScreen!=null) && !this.saved;
    },
    
    isReedit: function() {
        return this.reedit;
    },
    
    isSaved: function() {
        return this.saved;
    },
    
    _callSaveCallback: function(m) {
		if (this.plugin) {
			if (this.plugin.saveCallbackObject && this.plugin.saveCallbackFunction) {
				var p = {};
				p.saved = this.isSaved();
				p.reedit = this.isReedit();
				dojo.mixin(p, m);
				this.plugin.saveCallbackObject[this.plugin.saveCallbackFunction](p);
			}
		}    
	}
});
