/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.tabs.edit');
dojo.require('hub.main');
dojo.require('dijit.layout.StackContainer');
dojo.require('hub.widgets.forms.SelectPlugin');
/**
 * Manages a set of child content panes. UI can drive tabs to flip foward and 
 * backward through panes. Supports caching of previous panes so they can be 
 * restored without going to the server. Also supports rendering of both 
 * plug-ins and standard template forms as separate pages in the stack.
 */ 
dojo.declare('hub.widgets.tabs.edit.WorkflowPane', [dijit._Widget, dijit._Templated ], {
    // @todo: implement
    widgetsInTemplate: true,
    templatePath: dojo.moduleUrl("hub","widgets/templates/Workflow.html"),
    tab: null,
    plugin: null,
    object_id: null,
    plugin_id: null,
    moved_next: false,
    history_ptr: -1,
    screen_ptr: -1,
    plugin_ptr: null,
    screens: null,
    history: null,
    reedit: false,
    complete: false,
    _hitchedInvalidate: null,
    _observerHandle: null,
    
    /**
     * @param tab Reference to the tab containing this workflow
     * @param object_id Define if object under editing is known up front,
     *   otherwise it will be set when setPlugin is called later
     */
/*    constructor: function(tab, object_id) {
    	this.inherited({}, tab);
        // keep track of our tab
        this.tab = tab;
        this.plugin = null;
        this.object_id = object_id;
        this.plugin_id = null;
        this.moved_next = false;
        this.history_ptr = -1;
        this.screen_ptr = -1;
        this.plugin_ptr = null;
        this.screens = this.tab.childNodes;
        this.history = new Array();
        this.reedit = (object_id != null);
        this.complete = false;
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;

        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.object_id) {
/*            TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                    'invalidate-tab-'+this.object_id);*
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);
        }

        // clear out any old content when we're getting started, but don't
        // use setContent since we overrode that to append screens in our
        // tab
        this.tab.innerHTML = '';
    },*/

	startup: function(){
//        this.tab = tab;
//        this.screens = this.workflowContainer.childNodes;
        this.history = new Array();
        this.reedit = (this.object_id != null);
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;

        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.object_id) {
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);
        }

        // clear out any old content when we're getting started, but don't
        // use setContent since we overrode that to append screens in our
        // tab
//        this.tab.innerHTML = '';
//        this.workflowContainer.innerHTML = '';
		dojo.byId('intro').style.display = "block";
		dojo.byId('intro').style.visibility = "visible";
		
	},

    /**
     * Watches for tab closing and cleans up any junk we may have created.
     * Watches for tab content replacement, and updates the stack pointer
     * appropriately.
     *
     * @param signal Always 'invalidate-tab-' plus the object_id
     * @param payload Object having an action that is either 
     *   'replace' or 'close'
     */
    _onInvalidate: function(signal, payload) {
        if(payload.action == 'close') {
            // only dispose if closing, not replacing with new content
            this.cancel(payload);
        }
    },

    /**
     * Advanced to the next item in history and its corresponding screen.
     */
    _ptrNext: function() {
        this.history_ptr += 1;
        for(var i = this.screen_ptr+1; i < this.screens.length; i++) {
            if(this.screens[i].nodeName == 'DIV') {
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Retreat to the previous item in history and its corresponding 
     * screen.
     */
    _ptrPrevious: function() {
        this.history_ptr -= 1;
        for(var i = this.screen_ptr-1; i >= 0; i--) {
            if(this.screens[i].nodeName == 'DIV') {
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Jumps to a specific item in history and its corresponding screen.
     */
    _ptrJump: function(history_ptr) {
        // compute difference in history counts
        var diff = Math.abs(history_ptr - this.history_ptr);
        // determine direction of jump
        if(history_ptr > this.history_ptr) {
            var i = this.screen_ptr + 1;
        } else {
            var i = this.screen_ptr - 1;
        }
        // store new history pointer
        this.history_ptr = history_ptr;

        var count = 0;
        for(; i >= 0; i--) {
            if(this.screens[i].nodeName == 'DIV') {
                // count number of screens we move
                count += 1;
            }
            if(count == diff) {
                // stop when we've moved the proper number of screens
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Updates the held plug-in with the unique client ID for the shown
     * screen.
     */
    _updatePlugin: function(node) {
        // do nothing if no plug-in set
        if(!this.plugin)
            return;

        var divs = dojo.query('div', node);
        for(var i = 0; i < divs.length; i++) {
            if(divs[i].id) {
                // find first div with an id, and assume that's the unique
                // id for this screen of the editor
                this.plugin.setClientId(divs[i].id);
                break;
            }
        }
    },

    /**
     * Performs a diff detection for changes based on all values in a form.
     * Checks to see if the plug-in has methods
     *
     */
    _diffStates: function(s1, s2) {
        if(s1 == null || s2 == null) {
            // not objects, just compare if both are null
            return s1 == s2;
        }
        // check that all values in s1 are equal to those in s2
        for(var key in s1) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        // check that all values in s2 are equal to those in s1
        // NOTE: have to do this step incase size(s2) > size(s1) but we have
        //   no length property on objects
        for(var key in s2) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        return true;
    },

    /**
     * Gets the state of a single screen in workflow for later diff 
     * detection when we must decide if we should go to the server for the
     * the next page or use a cached copy.
     *
     * @return State object that will be compared later
     */
    _getScreenState: function() {
        var screen = this.screens[this.screen_ptr];

        // first see if the plug-in wants to give us a custom state
        if(this.plugin) {
            try {
                // if plug-in does not implement, we get a TOM exception
                // give the order index for this screen so the plug-in can
                // implement save logic for multiple screens
                return this.plugin.getScreenState(screen, this.history_ptr);
            } catch(e) {
                try {
                    if(e.getClass().getName() != 'TOM.Exception') {
                        // wasn't the TOM exception we were expecting
                        throw e;                            
                    }
                } catch (tmp) {
                    // wasn't the TOM exception we were expecting
                    throw e;
                }
            }
        }

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        var state = new Object();
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
            for(var i = 0; i < form.elements.length; i++) {
                // get the current element
                var e = form.elements[i];
                var name = e.name;
                if(name) {
                    // if it has a name, get its value
                    var val = e.value || e.checked;
                    // store value keyed by name
                    state[name] = val;
                }
            }
        }
        return state;
    },


    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param html Content to initialize on the screen
     * @return New screen DOM node
     */
    appendScreen: function(html) {
        // hide current screen if it exists
        try {
            this.screens[this.screen_ptr].style.display = 'none';
        } catch(e) {
        }
        // create, populate, append element
        var div = document.createElement('DIV');
        // to make sure we don't mess up large editors
        div.style.height = "100%";
        div.innerHTML = html;
        this.workflowContainer.appendChild(div);
        // update the plug-in client id based on the content
        this._updatePlugin(div);
        // evaluate Dojo only, JS seems to be evaluated for us by the tab
        hub.evaluate(div, false, true);
        if(this.history_ptr < 0) {
            // try to locate a previous button on the first screen; we have
            // to jump through hoops here because we don't know the unique
            // id for the button since it may be from a plugin or not
            var nodes = dojo.query('button');
            if(nodes.length) {
                // only consider the third match from the end
                // super ugly!
                var button = dijit.byNode(nodes[nodes.length-3].parentNode.parentNode);
                if(button)
                    button.setDisabled(true);
            }
        }
        // update our pointers
        this._ptrNext();
        return div;
    },

    /**
     * Contacts the server to delete the current entry if it's new.
     *
     * @param payload Contains information about a close signal
     */
    cancel: function(payload) {
        // only need to delete if defining a new entry and we have a 
        // plug-in reference
        if(!this.reedit && this.plugin) {
            // delete existing entry, we already asked for confirmation
            var delete_url = hub.config.base_url + "/plugin/delete/entryid/" +
                this.object_id + "/pluginid/" + this.plugin_id
            var form = new TOM.editor.DeleteAlways(delete_url);
            try {
                form.submit();
            } catch (e) {
                this.showMessage(e);
            }
            // continue with final dispose operations
            this.dispose();
        } else if(payload.external) {
            // if triggered by an external source, go back to details page,
            // user is not trying to explicitly close the tab

            // continue with final dispose operations
            // important that his comes before reuse of tab
            this.dispose();

            // cancel any close operation
            this.domNode.setAttribute('abort_close', true);

            // go to the details page
            var object_id = this.domNode.getAttribute('object_id');
            var title = this.domNode.getAttribute('title');
/*            TOM.tabs.man.showTab(hub.config.base_url+'/ui/details/entryid/'+object_id, // url
                                 title, // title 
                                 object_id, // object
                                 true, // current tab
                                 true); // doesn't matter if safe or not*/
			hub.showUrl(title, hub.config.base_url+'/ui/details/entryid/'+object_id, {safe: true})                             
                    
        } else {
            // user is explicitly trying to close the tab
            // continue with final dispose operations
            this.dispose();
        }
    },

    /**
     * Completes editing by disposing of this object and any plug-in.
     */
    dispose: function() {
        // remove as listener else we're still hanging around
/*        TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                   'invalidate-tab-'+this.object_id);*/
        dojo.unsubscribe(this._observerHandle);

        // inform a plug-in that it is also complete
        if(this.plugin) {
            this.plugin.dispose();
            this.plugin = null;
        }

        // for good measure, dump our history
        this.history = new Array();
        this.screens = new Array();
        this.complete = true;

        // remove us from the tab, avoid cyclic invocation of this method
        this.domNode.delAttribute('workflow', true);
    },
    
    myNext: function(){
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    var kw = {
		url: hub.config.base_url + '/plugin/editor/json/1',
		sync: true,
		form: "formSelectPlugin",
	        handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {           
	            if(response.dojoType == "cancel") {
	                alert(msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                alert(msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                alert(response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                alert( msgs.ERROR_REQUEST_SERVER);
	            } else if(response.error) {
	                alert(response.message);
	            } else {
//					this.workflowContainer.forward();
					var childDiv = dojo.doc.createElement('div');
					childDiv.innerHTML=response['result']['html'];
			        var newChild =  new dojox.layout.ContentPane({
							                        title:'newChild',
							                        refreshOnShow:true
							                },
							                childDiv
							        );
			       
			        // find my tabContainer and add our new ContentPane
			        this.workflowContainer.addChild(newChild);
			        this.workflowContainer.selectChild(newChild);
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
	    
    	
    },

    /**
     * Go to the next screen in the workflow. This might mean getting it
     * from the server or using a hidden panel if the current screen has
     * not changed in any way.
     *
     * @param func Function to invoke if we have to go to server
     */
    next: function(func) {
        this.complete = false;
        // check if DOM in history equals DOM in screen
        var old_state = this.history[this.history_ptr];
        var new_state = this._getScreenState(); //this.screens[this.screen_ptr].innerHTML;
        var same = this._diffStates(old_state, new_state);
        // check if we have history from the next screen
        var next_state = this.history[this.history_ptr+1];
        // save the current state as history
        this.history[this.history_ptr] = new_state;

        if(same && next_state != null) { 
            // hide this element
            this.screens[this.screen_ptr].style.display = 'none';
            // move to next available screen
            this._ptrNext();
            // show the next element, it's up to date
            this.screens[this.screen_ptr].style.display = 'block';
        } else {
            // pop all screens past the current since they are now all 
            // stale; keep current
            while(this.screens.length > this.screen_ptr+1) {
                this.workflowContainer.removeChild(this.workflowContainer.lastChild);
            }
            this.moved_next = true;
            // invoke the next function
            func();
        }
    },

    /**
     * Go to the previous screen in the workflow. Doesn't contact the server
     * in any way. Just shows a previously hidden panel.
     */
    previous: function() {
        if(this.history_ptr > 0) {
            this.complete = false;
            // we use this as an indicator that we've visted this screen
            this.history[this.history_ptr] = this._getScreenState(); 
            // hide the current item, no signals
            this.screens[this.screen_ptr].style.display = 'none';
            // update the pointers
            this._ptrPrevious();
            // show the previous item, no signals
            this.screens[this.screen_ptr].style.display = 'block';
            // didn't move next
            this.moved_next = false;
            // update the plug-in client id based on the content
            this._updatePlugin(this.screens[this.screen_ptr]);
        }
        // do nothing if there is no previous step
    },

    /**
     * Go to the first screen in the plug-in editor if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showPlugin: function() {
        if(this.plugin_ptr != null) {
            this.complete = false;
            // hide the current screen
            this.screens[this.screen_ptr].style.display = 'none';
            // update the pointers
            this._ptrJump(this.plugin_ptr);
            // show the first screen of the plug-in
            this.screens[this.screen_ptr].style.display = 'block';
            // didn't move next
            this.moved_next = false;
            // change the re-editing flag to true
            this.reedit = true;
            // update the plug-in client id based on the content
            this._updatePlugin(this.screens[this.screen_ptr]);
        }
    },

    /** 
     * Stores a reference to the plug-in that is part of the workflow.
     * 
     * @param plugin The plugin
     */
    setPlugin: function(plugin) {
        // make sure we throw away any old observer for an old object id
        if(this.object_id) {
            // remove as listener else we're still hanging around
            /*TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                       'invalidate-tab-'+this.object_id);*/
            dojo.unsubscribe(this._observerHandle);
        }
        this.plugin = plugin;
        // for convenience, pull object id and plug-in id into this object
        this.object_id = this.plugin.getObjectId();
        this.plugin_id = this.plugin.getPluginId();
        // track at which part in the history the plug-in was shown
        this.plugin_ptr = this.history_ptr+1;

        // listen for close events on the tab because we need to clean-up
/*        TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                'invalidate-tab-'+this.object_id);*/
		this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);                
    },

    /** 
     * Gets a reference to the plug-in that is part of the workflow.
     */
    getPlugin: function() {
        return this.plugin;
    },

    /**
     * Gets the number of screens in the history.
     *
     * @return integer count
     */
    getHistorySize: function() {
        return this.history.length;
    },

    /**
     * Sets a flag indicating this workflow has been run to 
     * completion. The flag is unset if the workflow moves to any screen
     * after the flag is set.
     */
    setComplete: function() {
        this.complete = true;
    },

    /**
     * Gets the value of the completion flag.
     */
    isComplete: function() {
        return this.complete;
    }
});
