/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.HttpUrl");
/**
 * Url utilites
 */
dojo.declare("mm.HttpUrl", null, {
    constructor: function( /*String*/urlString){
        this.scheme = this._extractScheme(urlString);
        this.server = this._extractServer(urlString);
        this.port = this._extractPort(urlString);
        this.path = this._extractPath(urlString);
        this.query = this._extractQuery(urlString);
        this.anchor = this._extractAnchor(urlString);
    },
    
    addParameter: function( /*String*/name, /*String*/ value){
        this.query += ((this.query != null && this.query != "") ? "&" : "") + name + "=" + value;
    },
	
	getHttpUrl: function(){
		//using the covention to convert url from https to http
		var result =  new mm.HttpUrl(this.toString());
		if(result.scheme == "https:"){
			result.scheme = "http:";
			var port = parseInt(this.port) + 80 - 443;
			result.port = port + ""
		};
		return result;
	},
	
	getHttpsUrl: function(){
		//using the covention to convert url from http to https
		var result = new mm.HttpUrl(this.toString());
		if(result.scheme == "http:"){
			result.scheme = "https:";
			var port = parseInt(this.port) - 80 + 443;
			result.port = port + ""
		};
		return result;
	},
	
    toString: function(){
        var str = "";
        if (this.server != "") {
            str += this.scheme + "//" + this.server;
        }
        if (this.port != "") {
            str += ":" + this.port;
        }
        
        str += "/" + this.path;
        
        if (this.query != "") {
            str += "?" + this.query;
        }
        if (this.anchor != "") {
            str += "#" + this.anchor;
        }
        
        return str;
    },
    _extractScheme: function(/*String*/urlString){
        var indexOfScheme = urlString.indexOf("://");
        
        if (indexOfScheme == -1) 
            return window.location.protocol; // location.protocol returns the protocol like http: or https:
        return urlString.substring(0, indexOfScheme + 1);
    },
    _extractServer: function(/*String*/urlString){
        var indexOfScheme = urlString.indexOf(this.scheme);
        var retVal = "";
        if (indexOfScheme == 0) {
            var indexOfSlash = urlString.indexOf("/", indexOfScheme + this.scheme.length + 2);
            var serverAndPort = urlString.substring(indexOfScheme + this.scheme.length + 2, indexOfSlash);
            retVal = serverAndPort.split(":")[0];
        }
        else {
            retVal = window.location.hostname;
        }
        
        return retVal;
    },
    _extractPort: function(/*String*/urlString){
        var indexOfServer = urlString.indexOf(this.server);
        var retVal = "";
        if (indexOfServer >= 0) {
            var indexOfSlash = urlString.indexOf("/", indexOfServer);
            var serverAndPort = urlString.substring(indexOfServer, indexOfSlash);
            var serverAndPortParts = serverAndPort.split(":");
            if (serverAndPortParts.length > 1) {
                retVal = serverAndPortParts[1];
            }
        }
        
        if (retVal == "") {
            if (urlString.indexOf("/") == 0) 
                retVal = window.location.port;
            else 
                retVal = (this.scheme == "https:" ? "443" : "80");
        }
        
        return retVal;
    },
    _extractPath: function( /*String*/urlString){
        var indexOfScheme = urlString.indexOf(this.scheme);
        var startIndex = 0;
        
        if (indexOfScheme == 0) 
            startIndex = this.scheme.length + 2;
        
        var retVal = "";
        
        var indexOfSlash = urlString.indexOf("/", startIndex);
        var indexOfQuery = urlString.indexOf("?");
        var indexOfHash = urlString.lastIndexOf("#");
        if (indexOfQuery >= 0) {
            retVal = urlString.substring(indexOfSlash + 1, indexOfQuery);
        }
        else {
            if (indexOfHash >= 0) {
                retVal = urlString.substring(indexOfSlash + 1, indexOfHash);
            }
            else {
                retVal = urlString.substring(indexOfSlash + 1);
            }
        }
        
        return retVal;
    },
    _extractQuery: function( /*String*/urlString){
        var retVal = "";
        var urlParts = urlString.split("?");
        
        if (urlParts.length > 1) {
            retVal = urlParts[1].split("#")[0];
        }
        
        return retVal;
    },
    _extractAnchor: function( /*String*/urlString){
        var retVal = "";
        var urlParts = urlString.split("#");
        
        if (urlParts.length > 1) {
            retVal = urlParts[urlParts.length - 1];
        }
        
        return retVal;
    }
});