/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.MashupMaker");
dojo.require("mm.HttpUrl");
dojo.require("mm.xhr.CrossDomainXmlHttpRequest");

/**
 * The Mashup Maker Server from Hub
 **/
dojo.declare("mm.MashupMaker", null, {
	//the url to Mashup Maker
	_serverUrl: "http://localhost:9080/mum",
	
	//the MashupMaker related URLs relative to MashupMaker context root
	contextRootServer: "/",
	contextRootBuilder: "/",
	contextRootEnabler: "/",
	
	contentHandler: "/mycontenthandler",
	serviceProxy: "/crossdomain/ServiceProxy.jsp",
	iconConfig: "/conf/icons.conf",
	feedViewerConfig: "/conf/feedViewers.conf",
	
	_username: "",
	
	_password: "",
	
	//if the Mashup Maker in same server as Hub
	_isLocal: false,
	
	constructor: function(/*string*/server, /*string?*/ username, /*string?*/ password){
        //remove last "/' in server
        if(server.lastIndexOf("/") == server.length -1){
            server = server.substring(0, server.length -1);
        }
		this._username = username ? username : "";
		this._password = password ? password : "";
		
		var originalUrl = new mm.HttpUrl(server);
		var mumUrl = originalUrl;
		var hubUrl = new mm.HttpUrl(window.location.href);
		
		//use same url scheme for Mashup Maker and Mashup Hub
		if (hubUrl.scheme == "http:") {
			mumUrl = mumUrl.getHttpUrl();
		}
		if (hubUrl.scheme == "https:") {
			mumUrl = mumUrl.getHttpsUrl();
		}
		
		//determine if Mashup Maker in same server as Hub
		if ((mumUrl.scheme == hubUrl.scheme) && (mumUrl.server == hubUrl.server) && (mumUrl.port == hubUrl.port)) {
			//Local Mashup Maker Server
			this._isLocal = true;
		} else {
			//Remote Mashup Maker Server
			this._isLocal = false;
		};
		
		//trim url, and extract contextRootXXX from url
		this._processMashupMakerUrl(originalUrl);
		this._serverUrl = mumUrl.toString();
		this.serverUrl = originalUrl.toString();
	},
	
	GET: function(/*string*/url, /*object*/ headers, /*function*/ callback){
		this._send({
			"method": "GET",
			"url": url,
			"headers": headers,
			"data": null,
			"callback": callback
		});
	},
	
	POST: function(/*string*/url, /*object*/ headers, /*string*/ data, /*function*/ callback){
		this._send({
			"method": "POST",
			"url": url,
			"headers": headers,
			"data": data,
			"callback": callback
		});
	},
	
	_send: function(kwArgs){
		/*
		 * kwArgs:
		 * 	method: string, http method
		 *  url: string, url
		 *  headers: object, http headers
		 *  data: string, http request body
		 *  callback: function, the callback function
		 */
		var xhr;
		if (this._isLocal && !this._username) {
			xhr = this._getXhr();
			//TODO: convert username and password to LTPA token?
			xhr.open(kwArgs.method, this._serverUrl + kwArgs.url, true, this._username, this._password);
			
		}else{
			xhr = this._getNativeXhr();
			var url = new mm.HttpUrl(this.serverUrl + kwArgs.url);
			var urlStr = [
				hub.config.base_url, 
				"mm", 
				url.scheme, 
				encodeURIComponent((this._username ? (encodeURIComponent(this._username) + ":" + encodeURIComponent(this._password) + "@") : "") + url.server + ":" + url.port),
				url.path + (url.query ? "?" : "") + url.query
			].join("/");
			xhr.open(kwArgs.method, urlStr);
		}
		if (kwArgs.headers) {
			for (var key in kwArgs.headers) {
				xhr.setRequestHeader(key, kwArgs.headers[key]);
			};
		};
		xhr.onreadystatechange = function(){
			if (xhr.readyState == 4) {
				kwArgs.callback(xhr);
			}
		};
		xhr.send(kwArgs.data);
	},
	
	_getXhr: function(){
		if (this._isLocal) {
			return this._getNativeXhr();
		} else {
			return new mm.xhr.CrossDomainXmlHttpRequest(this._serverUrl + this.serviceProxy);
		}
	},
	
	_getNativeXhr: function(){
		if (window.XMLHttpRequest) {
			return new XMLHttpRequest();
		} else if (window.ActiveXObject) {//IE
			try {
				return new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					return new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) {
				}
			};
					};
		throw new Error("No XmlHttpRequest support.");
	},
	
	_processMashupMakerUrl: function(/*mm.HttpUrl*/url){
		/*
		 * 1, detect Application Server type of Mashup Maker
		 *    Tomcat, WAS or LWI
		 * 2, trim URL path to content root, for example
		 *    http://localhost:8080/mashupmaker/enabler#
		 *    -->
		 *    http://locahost:8080/mashupmaker
		 * 3, change values of contextRootServer, contextRootBuilder
		 *    and contextRootEnabler based on type of Application Server
		 *    because in differnt build they have different URL
		 */
		url.query = "";//empty query (?xxx=xxx)
		url.anchor = "";//empty anchor (#xxx)
		var contextRoots = {
			"mashupmaker": "tomcat",
			//"LWI": "??" //TODO:LWI contextRoot
			"mum": "WAS"
		};
        
        var ctxRoot = url.path.substring(url.path.lastIndexOf("/"));
		
		var serverType = contextRoots[ctxRoot] ? contextRoots[ctxRoot] : "WAS";

		if (serverType == "WAS") {
			this.contextRootServer = "";
			this.contextRootBuilder = "";
			this.contextRootEnabler = "";
			
			this.contentHandler = "/mycontenthandler";
			//TODO: bug 581
			this.serviceProxy = "/crossdomain/ServiceProxy.jsp2";
			this.iconConfig = "/conf/icons.conf";
			this.feedViewerConfig = "/conf/feedViewers.conf";
			
		} else if (serverType == "tomcat") {
			this.contextRootServer = "/../mm.webapp";
			this.contextRootBuilder = "/../mm";
			this.contextRootEnabler = "/";
			
			this.contentHandler = "/../mm.webapp/mycontenthandler";
			this.serviceProxy = "/../mm/crossdomain/ServiceProxy.jsp";
			this.iconConfig = "/../mm/conf/icons.conf";
			this.feedViewerConfig = "/../mm/conf/feedViewers.conf";
		} else if (serverType == "LWI") {
			//TODO: path conf for LWI
		}
	}
});
