/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.NavigationModel");

dojo.require("mm.model.NavigationNode");

dojo.declare("mm.NavigationModel", null, {
	XML_NS: {
		atom: "http://www.w3.org/2005/Atom",
		nm: "http://www.ibm.com/xmlns/prod/lotus/remix/v1.0/navigation-model",
		xhtml: "http://www.w3.org/1999/xhtml",
		thr: "http://purl.org/syndication/thread/1.0",
		app: "http://www.w3.org/2007/app",
		model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
		base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"
	},
	
	constructor: function(/*mm.Server*/server){
		this.server = server;
	},
	
	getNavigationNodes: function(/*string*/nodeId, /*integer*/levels, /*function*/callback) {
		nodeId = (nodeId == null || nodeId == "") ? "default" : nodeId;
		levels = levels < 1 ? 1 : levels;
		
		var url = this.server.contentHandler + "?uri=nm:id:" + nodeId + "&levels=" + levels;
		var _this = this;
		this.server.GET(url, {}, function(xhr){
			if (xhr.status == 200) {
				var doc = mm.utils.createDocument(xhr.responseText);
				var rootIdNode = mm.xpath.evaluateXPath("atom:id", doc.documentElement, _this.XML_NS)[0];
				var rootId = rootIdNode ? rootIdNode.textContent || rootIdNode.text : nodeId;
				rootId = _this.parseId(rootId);
				
				var nodes = [];
				var rootNode = null;
				var childNodes = [];
				var entries = mm.xpath.evaluateXPath("atom:entry", doc.documentElement, _this.XML_NS);
				dojo.forEach(entries, function(entry){
					var nodeTypeNode = mm.xpath.evaluateXPath("atom:content/model:navigation-node/@type", entry, _this.XML_NS)[0];
					if (!nodeTypeNode || (nodeTypeNode && nodeTypeNode.value != "shared-root")) {
						var idNode = mm.xpath.evaluateXPath("atom:id", entry, _this.XML_NS)[0];
						var id = idNode ? idNode.textContent || idNode.text : "";
						id = _this.parseId(id);
						if (rootId == id) {
							// root node
							rootNode = new mm.model.NavigationNode(entry);
						} else {
							childNodes.push(new mm.model.NavigationNode(entry));
						}
					}
				});
				if (rootNode) {
					dojo.forEach(childNodes, function(node){
						rootNode.addChild(node);
					});
				}
				nodes.push(rootNode);
				callback(nodes);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			}
		});
	},
	
	parseId: function(/*string*/id) {
		// get rid of "nm:oid:"
		if (dojo.isString(id) && id.indexOf("nm:oid:") == 0) {
			return id.substring(7);
		}
        // get rid of "nm:id:"
        if (dojo.isString(id) && id.indexOf("nm:id:") == 0) {
			return id.substring(6);
		}
		return id;
	}
});