/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.pages.AddFeedToMashups");

dojo.require("hub.main");
dojo.require("hub.pages");

/**
 * AddFeedToMashups page.
 */
dojo.declare("mm.pages.AddFeedToMashups", hub.pages._Page, {
	// template to replace node with
	templatePath: dojo.moduleUrl("mm.pages", "templates/AddFeedToMashups.html"),
	
	type: "feed", //constant
	
	//attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	
	// parameters form caller
	object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	categories: null,
	feedViewers: null,
	icons: null,
	
	constructor: function(){
	},
	
	/**
	 * Gets the RESTful URL to query for information required to complete the
	 * rendering of the page.
	 */
	getUrl: function(){
		return hub.urls.getDetailsPage(this.object_id);
	},
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		var viewerTitle = " ";
		
		//find default feed viewer for feed type
		for (var i = 0; i < this.feedViewers.length; i++) {
			if (dojo.indexOf(this.feedViewers[i].types, server_data.object.source) != -1) {
				viewerTitle = this.feedViewers[i].displayName;
				this.feedViewers[i].checked = true;
				break;
			}
		}
		
		// If no default viewer, check the first feed viewer
		if (viewerTitle === " " && this.feedViewers.length > 0) {
			viewerTitle = this.feedViewers[0].displayName;
			this.feedViewers[0].checked = true;
		}
		
		if (server_data && server_data.object.parameters) {
			dojo.forEach(server_data.object.parameters, function(param) {
				param.optional = param.optional === "Y" ? true : false;
			});
		}
		
		var obj = {
			server: this.server,
			mm_categories: this.categories,
			feedViewers: this.feedViewers,
			icons: this.icons,
			categoryTitle: this.categories[0] ? this.categories[0].title : "", //select the first category
			viewerTitle: viewerTitle,
			palette_url: hub.config.palette_url,
			//messages
			links: dojo.i18n.getLocalization('hub', 'links'),
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_feed: hub.urls.getImage('feedLarge.gif'),
			icon_mashup: hub.urls.getImage('mashupLarge.gif')
		};
		return dojo.mixin(obj, server_data);
	},
	
	// @Override
	// Initial attach points and connect events
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("add_feed_to_mashups_form_" + this.object_id);
			this.message = dojo.byId("add_feed_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("add_feed_to_mashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("add_feed_to_mashups_serverbutton_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		if (!form.feedurl) {
			this.message.innerHTML = messages.ERROR_MASHUPS_URL;
			this.button.disabled = false;
			return;
		}
		
		if (!form.mm_category) {
			this.message.innerHTML = messages.ERROR_MASHUPS_CATEGORY;
			this.button.disabled = false;
			return;
		}
		
		// Validate and generate url with parameters
		var feedParams = {};
		for(var key in form){
			if(key.indexOf("param_") == 0){
				var paramName = key.substring(6);
				var paramRequired = key["option_param_" + paramName];
				if(paramRequired && (form["param_" + paramName] == "")){
					this.message.innerHTML = messages.ERROR_MASHUPS_PARAMETER + paramName;
					return;
				}else{
					feedParams[paramName] = form["param_" + paramName];
				}
			}
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogModel = new mm.CatalogModel(mashupMaker);
		
		var feedEntry = new mm.model.CatalogFeedEntry();
		feedEntry.feedUrl = form.feedurl;
		feedEntry.feedParams = feedParams;
		feedEntry.category = form.mm_category;
		feedEntry.title = form.title;
		feedEntry.description = form.description;
		feedEntry.icon = form.icon;
		for(var i=0; i<this.feedViewers.length;i++){
			if(this.feedViewers[i].displayName == form.feedViewer){
				feedEntry.feedViewer = this.feedViewers[i].definition;
				for(var key in this.feedViewers[i].metadata){
					feedEntry.metadata[key] = this.feedViewers[i].metadata[key];
				};
				feedEntry.feedUrlItemName = this.feedViewers[i].feedUrlItemName;
			}
		}
		var _this = this;
		catalogModel.addToCatalog(feedEntry, function(data, error){
			if (!error) {
				hub.showPage(_this.object_title, "FinishedAddToMashups", {
					object_id: _this.object_id,
					object_title: _this.object_title,
					type: _this.type
				});
			} else {
				_this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
				_this.button.disabled = false;
			}
		});
		this.button.disabled = true;		
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	},
	
	/**
	 * @Override
	 * Callback function, get category, feed viewer, icon set from lotus mashups
	 * @param {Object} response
	 * @param {Object} args
	 */
	_onPageDataLoad: function(response, args){
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogMode = new mm.CatalogModel(mashupMaker);
		var feedViewerConfig = new mm.FeedViewerConfig(mashupMaker);
		var iconConfig = new mm.IconConfig(mashupMaker);
		
		var inError = false;
		
		function callback(type, data, error){
			if (!error) {
				this[type] = data;
				if (this.categories && this.feedViewers && this.icons) {
					hub.pages._Page.prototype._onPageDataLoad.apply(this, [response, args]);
				};
			} else if(!inError){
				inError = true;
				hub.showPage(this.object_title, "SelectMashupsServer", {
					object_id: this.object_id,
					object_title: this.object_title,
					server: this.server,
					username: this.username,
					password: this.password,
					type: this.type,
					error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_CATEGORIES
				});
			}
		};
		
		catalogMode.getCategories(dojo.hitch(this, callback, "categories"));
		feedViewerConfig.load(dojo.hitch(this, callback, "feedViewers"));
		iconConfig.load(dojo.hitch(this, callback, "icons"));
	}
});
