/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.pages.AddIWidgetToMashups");

dojo.require("hub.main");
dojo.require("hub.pages");
dojo.require("mm.IWidgetPackageModel");
dojo.require("mm.utils");

/**
 * AddIWidgetToMashups page.
 */
dojo.declare("mm.pages.AddIWidgetToMashups", hub.pages._Page, {
	// template to replace node with
	templatePath: dojo.moduleUrl("mm.pages", "templates/AddIWidgetToMashups.html"),
	
	type: "iwidget", //constant
	
	isPackage: false,
	
	definition: "",
	
	//attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	
	// parameters form caller
	object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	categories: null,
	icons: null,
	//parameters from Hub
	details: null,
	
	constructor: function(){
	},
	
	/**
	 * Gets the RESTful URL to query for information required to complete the
	 * rendering of the page.
	 */
	getUrl: function(){
		return hub.urls.getDetailsPage(this.object_id);
	},
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		this.details = server_data;
		if (!server_data.pluginData.definitionUrl) {
			this.isPackage = true;
		}else {
			this.definition = server_data.pluginData.definitionUrl;
		}
		var obj = {
			server: this.server,
			isPackage: this.isPackage,
			mm_categories: this.categories,
			icons: this.icons,
			categoryTitle: this.categories[0] ? this.categories[0].title : "", //select the first category
			palette_url: hub.config.palette_url,
			//messages
			links: dojo.i18n.getLocalization('hub', 'links'),
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_widget: hub.urls.getImage('widgetLarge.gif')
		};
		return dojo.mixin(obj, server_data);
	},
	
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		var _this = this;
		window.setTimeout(dojo.hitch(this, function(){
			if (this.isPackage) {
				var widgetTable = dojo.byId("iwidget_package_table_" + this.object_id);
				var packageModel = new mm.IWidgetPackageModel();
				packageModel.packageUrl = this.details.object.url;
				packageModel.getWidgetList(dojo.hitch(this, function(data, error){
					if (!error) {
						widgetTable.style.display = "none";
						var tBody = document.createElement("tbody");
						widgetTable.appendChild(tBody);
						for (var i = 0; i < data.length; i++) {
							var tr = tBody.insertRow(i);
							var tdTitle = tr.insertCell(0);
							tdTitle.innerHTML = mm.utils.summary(data[i].title, 20);
							var tdDesc = tr.insertCell(1);
							tdDesc.innerHTML = mm.utils.summary(data[i].description, 50);
						}
						widgetTable.style.display = "";
					} else {
						_this.message.innerHTML = messages.ERROR_MASHUPS_GET_WIDGETPACKAGELIST + " " + error.status + "," + error.statusText;
						_this.button.disabled = false;
					}
				}));
			}
			this.form = dojo.byId("add_iwidget_to_mashups_form_" + this.object_id);
			this.message = dojo.byId("add_iwidget_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("add_iwidget_to_mashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("add_iwidget_to_mashups_serverbutton_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		
		if (!form.mm_category) {
			this.message.innerHTML = messages.ERROR_MASHUPS_CATEGORY;
			this.button.disabled = false;
			return;
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogModel = new mm.CatalogModel(mashupMaker);
		
		var entry = new mm.model.CatalogEntry();
		var _this = this;
		if(this.isPackage){
			var deployer = new mm.IWidgetDeployer(mashupMaker);
			deployer.packageUrl = this.details.object.url;
			deployer.category = form.mm_category;
			deployer.title = form.title;
			deployer.description = form.description; 
			
			this.button.disabled = true;
			
			deployer.deploy(dojo.hitch(this, function(data, error){
				if (!error) {
					hub.showPage(_this.object_title, "FinishedAddToMashups", {
						object_id: _this.object_id,
						object_title: _this.object_title,
						type: _this.type
					});
				} else {
					this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
					this.button.disabled = false;
				}
			}));
			
		}else{
			//add to catalog model
			entry.category = form.mm_category;
			entry.title = form.title;
			entry.description = form.description;
			entry.icon = form.icon;
			entry.definition = this.definition;
			
			this.button.disabled = true;

			catalogModel.addToCatalog(entry, dojo.hitch(this, function(data, error){
				if (!error) {
					hub.showPage(_this.object_title, "FinishedAddToMashups", {
						object_id: _this.object_id,
						object_title: _this.object_title,
						type: _this.type
					});
				} else {
					this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
					this.button.disabled = false;
				}
			}));
		}	
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	},
	
	/**
	 * @Override
	 * Callback function, get category, feed viewer, icon set from lotus mashups
	 * @param {Object} response
	 * @param {Object} args
	 */
	_onPageDataLoad: function(response, args){
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogMode = new mm.CatalogModel(mashupMaker);
		var iconConfig = new mm.IconConfig(mashupMaker);
		
		var inError = false;
		
		function callback(type, data, error){
			if (!error) {
				this[type] = data;
				if (this.categories && this.icons) {
					hub.pages._Page.prototype._onPageDataLoad.apply(this, [response, args]);
				};
			} else  if(!inError){
				inError = true;
				hub.showPage(this.object_title, "SelectMashupsServer", {
					object_id: this.object_id,
					object_title: this.object_title,
					server: this.server,
					username: this.username,
					password: this.password,
					type: this.type,
					error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_CATEGORIES
				});
			}
		};
		
		catalogMode.getCategories(dojo.hitch(this, callback, "categories"));		
		iconConfig.load(dojo.hitch(this, callback, "icons"));
	}
});
