/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.pages.AddMashupPageToMashups");

dojo.require("hub.main");
dojo.require("hub.pages");
dojo.require("dojo.string");

/**
 * Add Mashup Page to Mashups
 */
dojo.declare("mm.pages.AddMashupPageToMashups", hub.pages._Page, {
    // template to replace node with
    templatePath: dojo.moduleUrl("mm.pages", "templates/AddMashupPageToMashups.html"),
	
	type: 'mashuppage',	// Constant
	
	isPackage: false,
	
	// attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	titlePane: null,
	tree: null,
	
	// parameters form caller
    object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	navigationRoot : null,
	navigationStore : null,
	navigationTree: null,
	icons: "",
	selectedIdentity: "",
	
	details: null,
	navigationPath: [],
	
	constructor:function(){
	},
	
    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
		dojo.require("dijit.TitlePane");
		dojo.require("dijit.Tree");
		dojo.require("dijit.Menu");
		dojo.require("mm.data.NavigationModelStore");
		
		this.details = server_data;
		
		if (!server_data.pluginData.definitionUrl) {
			this.isPackage = true;
		}
		
        var obj = {
			// mm parameters
			server: this.server,
			username: this.username,
			password: this.password,
			//navigationRoot: this.navigationRoot[0],
			icons: this.icons,
			isPackage: this.isPackage,	// Tell if the mashup page is a definitionURL or a Package
			palette_url: hub.config.palette_url,
			//messages
			labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_page: hub.urls.getImage('pageLarge.gif')
        };
        return dojo.mixin(obj, server_data);
    },
	
	/**
	 * @Override
     * Uses the rendered deferred to indicate page rendering is complete.
     * Can be used as an extension point for dojo.connect too, but only if it is
     * known ahead of time that rendering is not already complete at time of
     * connection.
     */
    onRendered: function() {
		this.inherited(arguments);
		/*
    	var navigationStore = new dojo.data.ItemFileReadStore(data);
		this.navigationTree = new dijit.Tree({store: navigationStore, onClick: this.onTitleClick}, treeDiv);
		*/
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("addToMashupsForm_" + this.object_id);
			this.message = dojo.byId("add_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("addtomashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("changeServerButton_" + this.object_id);
			this.titlePane = dijit.byId("navigation_node_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			var self = this;
			var callback = function(root, error) {
				if (!error) {
					self.selectedIdentity = dojo.isArray(root.id) ? root.id[0] : root.id;
					var title = dojo.isArray(root.title) ? root.title[0] : root.title;
					self.titlePane.setTitle("<img src='../images/nav_home.png'/>");
					var rootNode = self.tree._navToRootOrFirstNode();
					rootNode.labelNode.innerHTML = "<img src='../images/nav_home.png'/>";
					if (self.tree.isExpandable) {
						self.tree._expandNode(rootNode);
					}
				} else {
					hub.showPage(self.object_title, "SelectMashupsServer", {
						object_id: self.object_id,
						object_title: self.object_title,
						server: self.server,
						username: self.username,
						password: self.password,
						type: self.type,
						error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_NAVIGATIONNODES
					});
				}
			}
			var _navigationStore = new mm.data.NavigationModelStore({url:this.server, username:this.username, password: this.password, callback: callback});
			
			this.tree = new dijit.Tree({
				id: "navigation_tree_" + this.object_id,
				store : _navigationStore,
				labelAttr : "title",
                typeAttr : "type",
				persist : false
			}, dojo.byId("navigation_treeDiv_" + this.object_id));
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(this.tree, "onClick", this, "onTitleClick");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
    },
	
	onTitleClick: function(node) {
		// summary:
		// 		set titlpane's title on tree node click
		var store = this.tree.store;
		var path = [];
		path.push(node.title[0]);
		var parentId = dojo.isArray(node.parentId) ? node.parentId[0] : node.parentId;
		this.selectedIdentity = dojo.isArray(node.id) ? node.id[0] : node.id;
		while (parentId) {
			var item = store._getItemByIdentity(parentId);
			path.push(item.title[0]);
			parentId = dojo.isArray(item.parentId) ? item.parentId[0] : item.parentId;
		}
		if (path.length > 0) {
			this.navigationPath = path.reverse();
			if (this.navigationPath.length > 0 && dojo.string.trim(this.navigationPath[0]) == "") {
				this.titlePane.setTitle("<img src='../images/nav_home.png'/>" + this.navigationPath.join(" > "));
			} else {
				this.titlePane.setTitle(this.navigationPath.join(" > "));
			}
		}
	},
	
	onSubmit: function() {
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		this.message.innerHTML = "";
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		if (!this.selectedIdentity) {
			this.message.innerHTML = messages.ERROR_MASHUPS_NAVIGATION;
			this.button.disabled = false;
			return;
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		
		if (this.isPackage) {
			var deployer = new mm.MashupPageDeployer(mashupMaker);
			deployer.packageUrl = this.details.object.url;
			deployer.target = this.selectedIdentity;
			deployer.title = form.title;
			deployer.position = form.position;
			deployer.description = form.description; 
		
			this.button.disabled = true;
			
			var _this = this;
			deployer.deploy(dojo.hitch(this, function(data, error){
					if (!error) {
						_this.navigationPath.push(_this.object_title);
						hub.showPage(_this.object_title, "FinishedAddToMashups", {
							object_id: _this.object_id,
							object_title: _this.object_title,
							type: _this.type,
							navigationPath: _this.navigationPath
						});
					} else {
						_this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
						_this.button.disabled = false;
					}
			}));
		}
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	}
});
