/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("mm.pages.SelectMashupsServer");

dojo.require("hub.main");
dojo.require("hub.pages");

dojo.declare("mm.pages.SelectMashupsServer", hub.pages._Page, {
	// template to replace node with
	templatePath: dojo.moduleUrl("mm.pages", "templates/SelectMashupsServer.html"),
	
	//attach points
	form: null,
	button: null,
	message: null,
	
	object_id: "",
	object_title: "",
	type: "", //one of the value "feed", "iwidget", "mashuppage"
	
	server: "",
	username: "",
	password: "",
	error: "",
	
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		var obj = {
			object_id: this.object_id,
			object_title: this.object_title,
			// mm parameters
			server: this.server ? this.server : mm.getDefaultMMServerUrl(),
			username: this.username ? this.username : "",
			password: this.password ? this.password : "",
			error: this.error,
			//page
			isFeed: this.type == "feed",
			isWidget: this.type == "iwidget",
			isPage: this.type == "mashuppage",
			isMashup: this.type == "mashup",
			// messages
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_feed: hub.urls.getImage('feedLarge.gif'),
			icon_widget: hub.urls.getImage('widgetLarge.gif'),
			icon_page: hub.urls.getImage('pageLarge.gif'),
			icon_mashup: hub.urls.getImage('mashupLarge.gif')
		};
		return obj;
	},
	
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("select_server_form_" + this.object_id);
			this.message = dojo.byId("select_server_message_" + this.object_id);
			this.button = dojo.byId("select_server_button_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_NEXT;
			dojo.byId("previous_button_" + this.object_id).innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("cancel_button_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	/**
	 * click handler: continue to addtomashup page
	 */
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');

		//validate inputs
		if (form.server == "") {
			this.message.innerHTML = messages.ERROR_MASHUPS_SERVER;
			return false;
		}
		//show add to mashups page
		var templateName;
		if(this.type == "feed" || this.type == "mashup"){
			templateName = "AddFeedToMashups";
		}else if(this.type == "iwidget"){
			templateName = "AddIWidgetToMashups";
		}else if(this.type == "mashuppage"){
			templateName = "AddMashupPageToMashups";
		}
		hub.showPage(form.object_title, templateName, form);
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	}
});
