/*
	Copyright (c) 2007-2008, IBM Corp.
*/
/*
 * summary:
 *   An Object that implement Cross Domain XmlHttpRequest 
 *   this object implements XmlHttpRequest API defined by W3C 
 *   please reference http://www.w3.org/TR/XMLHttpRequest/
 *   var xhr = new mm.xhr.CrossDomainXmlHttpRequest(serviceProxyUrl);
 * serviceProxyUrl:
 *   URL to Service Proxy of target domain, e.g. "http://target_domain/crossdomain/ServiceProxy.jsp"
 *   There must be a service proxy support at target domain, so that we can invoke serivce on target domain
 * example:
 *	var xhr = new mm.xhr.CrossDomainXmlHttpRequest("http://target_domain/crossdomain/ServiceProxy.jsp");
 *	xhr.onreadystatechange = function(){
 *		if(xhr.readyState == 4){
 *			alert(xhr.responseText);
 *		}
 *	}
 *	xhr.open("POST", "http://target_domain/crossdomain/sample/service.jsp");
 *	xhr.send("zhang ke");
 */
dojo.provide("mm.xhr.CrossDomainXmlHttpRequest");

mm.xhr.CALLBACK_PROXY_URL = [window.location.protocol,"//",window.location.host, window.location.pathname.substring(0, window.location.pathname.lastIndexOf("/")), "/", dojo.moduleUrl("mm.xhr", "CallbackProxy.jsp").uri].join("");
mm.xhr.REQUEST_PAGE_URL = [window.location.protocol,"//",window.location.host, window.location.pathname.substring(0, window.location.pathname.lastIndexOf("/")), "/", dojo.moduleUrl("mm.xhr", "Request.html").uri].join("");
mm.xhr.DEFAULT_TIMEOUT = 30000;

mm.xhr.CrossDomainXmlHttpRequest = function(/*String*/serviceProxyUrl){
	
	if(!serviceProxyUrl) throw new Error("serviceProxyUrl not set!");
	
	//construct the request object
	var request = {
		"callbackProxyUrl" : mm.xhr.CALLBACK_PROXY_URL,
		"serviceProxyUrl" : serviceProxyUrl,
		
		"method": "",
		"url": "",
		"async": true,
		"username": "",
		"password": "",
		
		"headers" : "",
		"body" : ""
	};
	
	//event handler
	this.onreadystatechange = function(){};
	
	//state constant
	this.UNSENT = 0;
  	this.OPENED = 1;
  	this.HEADERS_RECEIVED = 2;
  	this.LOADING = 3;
  	this.DONE = 4;
  	
	this.readyState = this.UNSENT;
	
	//request API
	this.open = function(/*in DOMString*/method, /*in DOMString*/url, /*in boolean?*/ async, /*in DOMString?*/username, /*in DOMString?*/ password){
		if((typeof method == "undefined") || (typeof url == "undefined")){
			throw new Error("method or url not set");
		}
		request.method = method;
		request.url = url;
		request.async = typeof async == "undefined" ? true : async;
		request.username = typeof username == "undefined" ? "" : username; 
		request.password = typeof password == "undefined" ? "" : password;
		//change to OPENED state
		this.readyState = this.OPENED;
		this.onreadystatechange();
	}
	
	this.setRequestHeader = function(/*in DOMString*/header, /*in DOMString*/value){
		//encode request header into string
		//please reference http://www.w3.org/TR/XMLHttpRequest/#getallresponseheaders
		request.headers = request.headers + header + ":" + value + "\r\n";
	}
	
  	this.send = function(/*in DOMString or Document?*/ data){
		//TODO: support Document
		request.body = data?data:"";
		//create an iframe to send request to ServiceProxy.jsp in target domain
		//1, the iframe load Request.html in this domain
		//  Request.html retrive request object from its frameElement._request,
		//  encode request object into a form, 
		//  submit the form to ServiceProxy.jsp in target domain
		//2, iframe load ServiceProxy.jsp in target domain
		//  extract request object,
		//  use XmlHttpRequest to invoke service
		//  encode response object into a form
		//  submit the form to CallbackProxy.jsp in this domain
		//3, iframe load CallbackProxy.jsp in this domain
		//  extract response object
		//  notify parent by invoke iframe._complete()
		var iframe = document.createElement("iframe");
		iframe.setAttribute("src", mm.xhr.REQUEST_PAGE_URL);
		iframe.setAttribute("style", "width:0px;height:0px;border:0px;");
		iframe._request = request;
		var _this = this;
		//call back function, will be invoked by CallBackProxy.jsp
		iframe._complete = function(){
			iframe._stopTimeout();
			//set response
			_this.responseText = iframe._response.responseText;
			_this.status = iframe._response.status;
			_this.statusText = iframe._response.statusText;
			responseHeaders = iframe._response.responseHeaders;
			//change to DONE state
			_this.readyState = _this.DONE;
			_this.onreadystatechange();
			//delete the iframe
			window.setTimeout(function(){document.documentElement.removeChild(iframe)},0);
		};
		
		//FIXME: temp solution for time out
		var timeoutId = null;
		iframe._startTimeout = function(){
			var timeout = mm.xhr.DEFAULT_TIMEOUT;
			timeoutId = window.setTimeout(function(){
				_this.responseText = "";
				_this.status = 408;
				_this.statusText = "Time out";
				_this.readyState = _this.DONE;
				_this.onreadystatechange();
				window.setTimeout(function(){document.documentElement.removeChild(iframe)},0);
			}, timeout);
		}
		iframe._stopTimeout = function(){
			window.clearTimeout(timeoutId);
		}
		
		//FIXME: temp solution for test service proxy
		/*var link = document.createElement("link");
		link.setAttribute("href", serviceProxyUrl);
		link.setAttribute("type", "text/css");
		link.setAttribute("rel", "stylesheet");
		console.debug(link.sheet);
		document.documentElement.appendChild(link);
		link.on
		var interval = window.setInterval(function(){
			alert(link.sheet);
			if(link.sheet)window.clearInterval(interval);
			//link.parentNode.removeChild(link);
		},1);*/
		
		
		
		document.documentElement.appendChild(iframe);
		//FIXME: how to get notified, if service proxy url is incoreect or unreachable
	};
	
	this.abort = function(){
		throw new Error("abourt() not supported!");
	}
	
	// response API
	var responseHeaders = "";
	
	this.responseText;
	this.responseXML = null;
	this.status;
	this.statusText;
	
	this.getAllResponseHeaders = function(){
		//return a string contains all response headers
		//please reference http://www.w3.org/TR/XMLHttpRequest/#getallresponseheaders
		return responseHeaders;
	};
	
	this.getResponseHeader = function(/*in DOMString*/header){
		//return value of specific header
		var headers = responseHeaders.split("\r\n");
		for(var i=0;i<headers.length;i++){
			if(headers[i].indexOf(header + ":") == 0){
				return headers[i].substring(header.length + 1);
			}
		}
	};
};
