/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide( "mm.xpath" );

// generic implementation
mm.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
	namespaces = namespaces ? namespaces : {};
    if (navigator.userAgent.toLowerCase().indexOf("safari") != -1) {
        return mm.xpath.safari.evaluateXPath(xpathExpr,doc,namespaces);
    }
    else if (typeof ActiveXObject != "undefined") {
        return mm.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
    }
    else {
        return mm.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
    }
}

// Safari implementation
dojo.provide("mm.xpath.safari");
mm.xpath.safari.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {

    var result;
    if (typeof XPathResult != "undefined") {
        //use build in xpath support for Safari 3.0
        var results = document.evaluate(xpathExpr,doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        var thisResult;
        result = new Array();
        var len = 0;
        do {
            thisResult = results.iterateNext();
            if (thisResult) {
                result[len] = thisResult;
                len++;
            }
        } while ( thisResult );
    }
    else {
        //use javeline Safari2 xpath support
        if (doc.selectNodes) {
            result = doc.selectNodes(xpathExpr);
        }
    }
    return result;
}



// IE Implementation

dojo.provide( "mm.xpath.ie" );

mm.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        if (namespaces) {        
            var ns = "";
            for (var prop in namespaces) {
                ns += "xmlns:"+prop+"='"+namespaces[prop]+"' ";
            }
            if (doc.ownerDocument) {
                doc.ownerDocument.setProperty("SelectionNamespaces", ns); 
                doc.ownerDocument.setProperty("SelectionLanguage", "XPath"); 
            } else {
                doc.setProperty("SelectionNamespaces", ns);
                doc.setProperty("SelectionLanguage", "XPath"); 
            }
        }
        var result = doc.selectNodes(xpathExpr);

        var thisResult;
        var resultSet = new Array();
        var len = 0;
        for (var i=0; i<result.length; i++) {
            thisResult = result[i];
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        }
        return resultSet;
}


// GECKO Implementation

dojo.provide( "mm.xpath.gecko" );

mm.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
		var xmlDoc = doc.ownerDocument ? doc.ownerDocument : doc;
		var result;
        try {
            result = xmlDoc.evaluate( xpathExpr, doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        } 
        catch (exc)
        {
            //Callers should catch this and can substitute their own error message
            throw new Error("Error with xpath expression" + exc);
        }
    
        var thisResult;
        var resultSet = new Array();
        var len = 0;
        do {
            thisResult = result.iterateNext();
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        } while ( thisResult );
        return resultSet;
}

