main.logmsg("** ConnectorFeedLib script library v.1.0 loading...");

// Returns only xx characters of the string passed in
function abbv(str, len) {
	if (str == null || len <= 3)
		return ""
	else if (str.length() <= len)
		return str
	else
		return str.substring(0,len-3) + "...";
}

// Returns CDATA encoded value if it contains invalid XML chars
function xmlVal(val) {
	if (val == null)
		val = "";
	// make sure input has a 'length' function.
	if ( typeof(val) == 'string' ){
		val = abbv(val,255);
	}
	if (!val.equals(removeFunkyChars(val)))
		return "<![CDATA[" + val + "]]>"
	else
		return val;
}

function fixElementName(str) {
	if (str == null)
		return "";

	str = system.remove("$@{}[];:/\\", removeFunkyChars(str));

	if (str.startsWith("["))
		str = String(str).substring(1);
	if (str.endsWith("]"))
		str = str.substring(0, str.length()-1);

	return str;
}

// This function takes the Attributes in "entry" and
// adds them to a new Entry object using extended RSS att names,
// e.g. tdi:attname, which it then returns.
function atomContentXML( entry, entIndex ) {
	entCount = entIndex + 1;
	var attnames = entry.getAttributeNames();
	var xml = "<entry xmlns='http://www.w3.org/2005/Atom'>\n";
	xml += "   <title type='text'>Item " + entCount + "</title>\n"
		+ "   <subtitle type='text'>" + abbv(entry.toString(), 30) + "</subtitle>\n"
		+ "   <summary type='text'>" + xmlVal(abbv(entry.toString(), 2048)) + "</summary>\n"
		+ "   <id>urn:uuid:" + entCount + "</id>\n"
		+ "   " + makeDateElement("atom");

	xml += "   <content type='application/xml'>\n";
	xml += "    <record xmlns='urn:www-ibm-com:tdi:mashup:1.0'>\n";

	for (var attn in attnames) {
		var att = entry.getAttribute(attn);
		xml += "      <" + fixElementName(attn) + ">";

		if (att.size() <= 1)
			xml += xmlVal(att.getValue()) + "</" + fixElementName(attn) + ">\n"
		else {
			xml += "\n";
			for (var i = 0; i < att.size(); i++)
				xml += "         <value>" + xmlVal(att.getValue(i)) + "</value>\n";
			xml += "      </" + fixElementName(attn) + ">\n";
		}
	}
	xml += "    </record>\n";
	xml += "   </content>\n";

	return xml + "</entry>\n";
}

function getFirstEntryAsSchema(CI) {
	var schema = null;
	logmsg("- Using first entry read with CI as schema");
	var firstEntry = CI.getNextEntry();
	if (firstEntry != null) {
		schema = new java.util.Vector();
		attnames = firstEntry.getAttributeNames();
		for (var att in attnames) {
			sEntry = system.newEntry();
			sEntry.setAttribute("name", att);
			val = firstEntry.getAttribute(att).getValue(0);
			if (val != null)
				syntax = val.getClass().getName()
			else
				syntax = "*unknown*";
			sEntry.setAttribute("syntax", syntax);
		}
	} else {
		logmsg("No entries found");
	}
	return schema;
}

function querySchema(CI) {
	var schema = null;
	if (CI == null)
		return null;

	var ciType = CI.getClass().getName();
	logmsg("Querying schema for CI");

	if (ciType == "com.ibm.di.connector.JDBCConnector") {
		if (CI.getParam("jdbcSelect") == null || CI.getParam("jdbcSelect") == "") {
			logmsg("JDBC metadata read for table " + CI.getParam("jdbcTable"));
			schema = CI.querySchema(CI.getParam("jdbcTable"))
		}
		if (schema == null)
			schema = getFirstEntryAsSchema(CI);
	} else if (ciType == "com.ibm.di.connector.LDAPConnector") {
		if (CI.getNextEntry() != null)
			schema = CI.querySchema(null);
	} else {
		schema = getFirstEntryAsSchema(CI)
	}
	logmsg("Got " + schema.size() + " attributes in schema");

	return schema;
}

