/*
	Copyright (c) 2007-2008, IBM Corp.
*/
main.logmsg("** FeedServerLib script library v.1.0 loading...");

// ----------------------------------------------------------
// Define some function needed in the AL
// ----------------------------------------------------------

// Parse out nouns from the URL e.g. http://localhost/ATOM/myFeed
// Parameters:
//   url: the base portion of a url (e.g. /ATOM/myFeed)
//   partnum: which part to get (e.g. 1 gives ATOM, 2 gives myFeed)
function getUrlPart(url, partnum) {
	if (url == null)
		return "";

	var urlBase = getUrlBase();
	if (!system.startsWithIC(url, urlBase))
		return "";
	url = url.slice(urlBase.length - 1); // save the trailing slash
	var parts = url.split("/");
	if (parts.length <= partnum)
		return "";
	return parts[partnum];
}

// Parse out the Feed name from the URL
function getFeedName(url) {
	var feedName = getUrlPart(url, 2);
	if (!feedName)
		feedName = "Feeds";
	return feedName;
}

// Parse out the protocol requested for the returned data e.g. RSS, ATOM, JSON, ...
// There is no default.
// If the feedProtocol (positional parameter 1) is missing, so is everything else
// That will trigger the "help" function
function getFeedProtocol(url) {
	if (url == null)
		url = "";

	// silently ignore noise requests
	if (url.toLowerCase() == "/favicons.ico" ||
	    url.toLowerCase() == "/robots.txt")
		return "404";

	var urlBase = getUrlBase();

//	task.logmsg("--- getFeedProtocol(url) - url: '" + url + "'");
//	task.logmsg("                       urlBase: '" + urlBase + "'");

	if (url + "/" == urlBase)
		// Welcome to the front door. Need some help?
		// this is needed to prevent the 404 if they omit trailing slash
		return "html";
		//TODO: Might be better to do a redirect instead

	if (!system.startsWithIC(url, urlBase)) {
		errmsg("** Request does not start with urlBase of "+urlBase);
		errmsg("   Ignoring request with URL of "+url);
		return "404";
	}
	var feedProtocol = getUrlPart(url, 1);
	if (feedProtocol == "")
		return "html";
//	else if (feedProtocol == "null" || feedProtocol == null)
//	     	return "atom";

	return feedProtocol.toLowerCase();
}

// Parse out Feed operation from the url e.g. http://localhost/ATOM/myFeed/myOperation
// Default: feed
function getFeedOperation(url) {
	var feedOperation = getUrlPart(url, 3);
	return feedOperation == "" ? "feed" : feedOperation;
}

// The type of Feed (Connector or AL) is stored in the Feeds Entry object
// as a property named after that feed.
function getFeedType(feedName) {
	return Feeds.getProperty(feedName + ".type");
}

// Calls the 99_ErrorFeed AL to return error info
function callErrorFeed(msg, accum) {
	var tcb = system.newTCB();
	var errEntry = system.newEntry();

	errEntry.setAttribute("msg", msg);
	errEntry.setAttribute("feedName", feedName);
	errEntry.setAttribute("http.host", work.getString("http.host"));
	errEntry.setAttribute("http.base", work.getString("http.base"));
	tcb.setInitialWorkEntry(errEntry);
	tcb.setAccumulator(accum);

	logmsg("--> Starting 99_ErrorFeed");

	var al = main.startAL("AssemblyLines/99_ErrorFeed", tcb);
	al.join();
}

function shortName( name ) {
	var p = name.indexOf(".", 5);
	var tName = name.substring(p+1);
	return tName;
}

// Returns the RSS or ATOM element XML snippet corresponding
// to the "name" parameter (e.g. "atom.feed.title"). This is
// built based on Attributes found in the "entry" parameter
// (which must be of type Entry).
function getItemValue( entry, type, name ) {
	var itemValue = "";
	var tName;

	if (name != null && name.trim() != "") {

		var att = entry.getAttribute( name );
		var tName = shortName(name);

		if (att == null || att.size() == 0)
			itemValue += "<" + tName + " />\n"
		else {
			for (var i = 0; i < att.size(); i++) {
				var val = att.getValue(i);
				if (val == null)
					val = ""
				else
					val = val.toString().trim();

				if (val != "") {
					if (!val.equals(removeFunkyChars(val)) ||
						(tName.toLowerCase().indexOf("encoded") >= 0))
						val = "<![CDATA[" + val + "]]>";

					if ((type == "entry" || type == "feed") &&
						   (tName.toLowerCase() == "link"))
						itemValue += "<link href=\"" + val + "\" />"
					else
						itemValue += "<" + tName + ">"
							     + val + "</" + tName + ">\n";
				}
			}
		}
	}

//	if (deBug)
//		logmsg("@@ getItemValue() - " + name + " = " + itemValue.substring(0,itemValue.length()-1));

	return itemValue;
}

// Number of results to display
function getNumResults() {

	// Read it from url params
	res = work.getString("http.qs.numResults");

	// Otherwise read it from properties
	if (res == null)
		res = system.getTDIProperty("MIS-Properties", "ItemsPerRequest");

	// Otherwise default to 20
	if (res == null || !system.isValidInt(res))
		return 20

	return system.toInt(res);
}

// Return a correctly formatted RSS or ATOM date
// element as an XML snippet with the current date/time
function makeDateElement( feedType ) {
	if (feedType.toLowerCase() == "rss")
		return "<pubDate>"
				+ system.formatDate(new java.util.Date(),
								 	"EEE, d MMM yyyy HH:mm:ss '+0000'")
				+ "</pubDate>\n";
	else {
		var dtstr = system.formatDate(new java.util.Date(),
			 			 				"yyyy-MM-dd'T'HH:mm:ss'Z'");

		return "<updated>"
				+ dtstr
				+ "</updated>\n";
	}
}

// Return the XML snippet for the specified "type"
// (e.g. "channel" or "item") by using all attributes
// with this "type" value in the middle of their name
// (e.g. rss.channel.title -> "channel")
function rssXML( entry, type ) {
	if (entry == null) {
		errmsg("** rssXML() Error - entry passed was null");
		return "";
	}

	type = type.toLowerCase();

	var item = "<" + type + ">\n";
	var attnames = entry.getAttributeNames();
	var foundDate = false;

	for (var i = 0; i < attnames.length; i++)
		if (attnames[i].toLowerCase().startsWith("rss." + type + ".")) {
			item += getItemValue(entry, type, attnames[i]);

			if (shortName(attnames[i]).toLowerCase().indexOf("pubdate") >= 0)
				foundDate = true;
		}

	if (!foundDate)
		item += makeDateElement( "rss" );

	if (type == "item")
		item += "</" + type + ">\n";

	return item;
}

// Return the XML snippet for the specified "type"
// (e.g. "feed" or "entry") by using all attributes
// with this "type" value in the middle of their name
// (e.g. atom.feed.title -> "feed")
function atomXML( entry, type ) {
	if (entry == null) {
		errmsg("** atomXML() Error - entry passed was null");
		return "";
	}

if (deBug)
	logmsg("returning " + entry);

	type = type.toLowerCase();

	var item = (type == "entry") ? "<entry>\n" : "";
	var attnames = entry.getAttributeNames();
	var foundDate = false;

	for (var i = 0; i < attnames.length; i++)
		if (attnames[i].toLowerCase().startsWith("atom." + type + ".")) {
			item += getItemValue(entry, type, attnames[i]);

			if (shortName(attnames[i]).toLowerCase().indexOf("updated") >= 0)
				foundDate = true;
		}

	if (!foundDate)
		item += makeDateElement( "atom" );

	if (type == "entry")
		item += "</" + type + ">\n";

	return item;
}

// Return string with XML snippet for the start of either
// an RSS or ATOM return message
function startTags( feedType ) {
	if (feedType.toLowerCase() == "rss")
		return 	"<?xml version='1.0' encoding='UTF-8'?>\n"
					+ "<!-- generator='TDI' -->\n"
					+ "<rss version='2.0' "
						+ "xmlns:content='http://purl.org/rss/1.0/modules/content/' "
						+ "xmlns:wfw='http://wellformedweb.org/CommentAPI/' "
						+ "xmlns:MIS='http://www.tdi-users.org'>\n"
						+ "xmlns:dc='http://purl.org/dc/elements/1.1/'>\n"
	else
		return "<?xml version='1.0' encoding='UTF-8'?>\n"
					+ "<!-- generator='TDI' -->\n"
					+ "<feed xmlns='http://www.w3.org/2005/Atom'>\n";
}

// Return string with XML snippet for the end of either
// an RSS or ATOM return message
function endTags( feedType ) {
	if (feedType.toLowerCase() == "rss")
		return "</channel>\n"
				+ "</rss>\n"
	else
		return "</feed>\n";
}

// These next two methods are used to convert RSS to ATOM
// if RSS was returned by the Feed but ATOM is the protocol
// requested.
//
// Converts a single element
function convertRSSfield2Atom( entry, rssfield, atomfield ) {
	if (entry == null)
		return;

	var a = entry.getAttribute(rssfield);
	if (a != null)
		if (shortName(rssfield) == "pubDate") {
			var dt = system.parseDate(a.getValue(), "EEE, d MMM yyyy HH:mm:ss '+0000'");
			entry.setAttribute(atomfield, system.formatDate(dt, "yyyy-MM-dd'T'HH:mm:ss'Z'"));
		} else
			entry.setAttribute(atomfield, a);
}

// Converts the whole Entry
function convertRSS2Atom( entry ) {
	convertRSSfield2Atom( entry, "rss.channel.description", "atom.feed.subtitle");
	convertRSSfield2Atom( entry, "rss.channel.generator",   "atom.feed.generator");
	convertRSSfield2Atom( entry, "rss.channel.link",		"atom.feed.link");
	convertRSSfield2Atom( entry, "rss.channel.title",		"atom.feed.title");
	convertRSSfield2Atom( entry, "rss.channel.pubDate",		"atom.feed.modified");
	convertRSSfield2Atom( entry, "rss.item.content:encoded","atom.feed.content");
	convertRSSfield2Atom( entry, "rss.item.description",	"atom.feed.summary");
	convertRSSfield2Atom( entry, "rss.item.guid",			"atom.entry.id");
	convertRSSfield2Atom( entry, "rss.item.link",			"atom.entry.link");
	convertRSSfield2Atom( entry, "rss.item.title",			"atom.entry.title");
	convertRSSfield2Atom( entry, "rss.item.pubDate",		"atom.entry.modified");

	var dt = entry.getString("rss.channel.pubDate");
	if (dt != null) {
		dt = system.parseDate(dt, "EEE, d MMM yyyy HH:mm:ss '+0000'");
		entry.setAttribute("atom.feed.updated", system.formatDate(dt, "yyyy-MM-dd'T'HH:mm:ss'Z'"));
	}
	var dt = entry.getString("rss.item.pubDate");
	if (dt != null) {
		dt = system.parseDate(dt, "EEE, d MMM yyyy HH:mm:ss '+0000'");
		entry.setAttribute("atom.entry.updated", system.formatDate(dt, "yyyy-MM-dd'T'HH:mm:ss'Z'"));
	}
}

// Return the Config object for the named AL.
function getALConfig(alname) {
	var mmcfg = main.getMetamergeConfig();
	var alcfg = mmcfg.getAssemblyLine(alname);
	if (alcfg == null) {
		errmsg("ERROR", "** Feed Server - ERROR ** No feed AL found with name \""
						+ work.getString("feedName") + "\"" );
		system.exitFlow();
	}
	return alcfg;
}

// Return the version of the running TDI Server
function getTDIVersion() {
	return "6.1.1";
}

// Return true if the operation <feedOp> is defined for an AL called <feedName>
function operationDefinedForFeed(feedName, feedOp) { // returns true/false
	var alcfg = getALConfig(alname);

	logmsg("--? Is " + feedOp + " defined for " + feedName);

	if (alcfg == null)
		logmsg("--! feed AL not defined")
	else
	if (alcfg.getOperation(feedOp) == null)
		logmsg("--! operation not defined for AL")
	else
		logmsg("--> op set!");

	if (alcfg == null)
		return false
	else
		return alcfg.getOperation(feedOp) != null;
}

// Call the feedAL with the specified feedOp (if supported by the AL).
function callFeedAL(feedName, feedOp, maxCycles) { // returns the accumulator
	var tcb = system.newTCB();
	var accum = new java.util.Vector(); // accumulator for the called AL
	var passInOpEntry = system.newEntry();

	var feedAL = Feeds.getProperty(feedName + ".exec");
	var feedConnector = feedAL; // In case it's a "connector" feed type
	var feedType = Feeds.getProperty(feedName + ".type");
	var i;

	if (feedType == "connector") {
		feedAL = g_ConnectorFeedAL;
		logmsg("--> Preparing to call " + feedAL + " for Connector: " + feedConnector);
	} else {
		if (!feedAL)
			feedAL = feedName;
		logmsg("--> Preparing to call feed AL: " + feedAL + " (operation: " + feedOp + ")")
	}

	var feedXML = null;
	var lf = "\n";

	// The Feed AL is called with an Accumulator which will collect
	// all Work Entries processed. This will be used in the
	// "Build Feed XML" Script component to construct the return XML payload.
	tcb.setAccumulator(accum);

	if (feedOp != null && feedOp != "")
		tcb.setALOperation(feedOp);

	// Since there is no way to access the TCB from the
	// called AL, we have to pass it in as an Attribute
	passInOpEntry.setAttribute("tcb", tcb);

	// Pass in all http.* attributes
	attnames = work.getAttributeNames();
	for (i = 0; i < attnames.length; i++)
		if (attnames[i].startsWith("http.")) {
			passInOpEntry.setAttribute(attnames[i], work.getString(attnames[i]));
		}

	// And now to pass in default parameter values and remove
	// any parameters with "publish='false'" in Feeds.xml
	var feedParams = Feeds.getObject(feedName);
	if (feedParams != null) {
		for (i = 0; i < feedParams.getLength(); i++) {
			var param = feedParams.item(i);
			var patts = param.getAttributes();
			var pid = patts.getNamedItem("id").getNodeValue();
//	logmsg("---------- looking at param " + pid + "  : " + patts.getNamedItem("id"));
			// remove if publish='false'
			var pub = patts.getNamedItem("publish");
			if (pub)
				pub = pub.getNodeValue();
			if (pub && pub == "false")
				passInOpEntry.removeAttribute("http.qs." + pid);
//	logmsg("-----------     pub: " + pub);
if (deBug && pub && pub == "false")
	logmsg("** callFeedAL(" + feedName + ") - removing private parameter http.qs." + pid);

			// set defaults
			var defval = system.selectSingleNode(param, "default_value");
			if (defval && defval.getChildNodes().getLength() > 0)
				defval = defval.getChildNodes().item(0).getNodeValue()
			else
				defval = null;

			if (defval) {
				if (passInOpEntry.getString("http.qs." + pid) == null)
					passInOpEntry.setAttribute("http.qs." + pid, defval);
if (deBug && defval)
   logmsg("** callFeedAL(" + feedName + ") - setting default value for parameter http.qs." + pid + " = " + defval);
			}
//logmsg("-----------     def: " + defval);
		}
	}

	passInOpEntry.setAttribute("feedProtocol", work.getString("feedProtocol"));
	passInOpEntry.setAttribute("feedOperation", feedOp);
	passInOpEntry.setAttribute("feedName", feedName);
	passInOpEntry.setAttribute("feedConnector", feedConnector);
	passInOpEntry.setAttribute("Feeds", Feeds);
	passInOpEntry.setAttribute("Accumulator", accum);
	passInOpEntry.setAttribute("numResults", numResults);
	passInOpEntry.setAttribute("MIS_version", mis_version);
	passInOpEntry.setAttribute("TDI_version", getTDIVersion());
	passInOpEntry.setAttribute("deBug", deBug);

// Debug display
if (deBug) {
	logmsg("@@ callFeedAL() --- OPENTRY passed to Feed AL follows -------------");
	task.dumpEntry(passInOpEntry);
}

	tcb.setOperationInitParams(passInOpEntry);

	// Call the Feed AL with Accumulator to catch iterated Entries
	try {
		var al = main.startAL("AssemblyLines/" + feedAL, tcb);
		al.join();
	} catch (ex) {
		errmsg("Feed Server - ERROR ** " + ex );
	}
	return accum;
}

// Return either the Output Map for this AL, or if not
// defined then the Output Schema.
function getALSchema(alname) {
	var e;
	var i;
	var alcfg = getALConfig(alname);
	var vect = new java.util.Vector();

	// First check to see if there is a published schema (i.e. an Output Map
	// for the "Default" Operation
	var defop = alcfg.getOperation("Default");

	if (defop != null) {

		// First we check the Output Map of the Default operation of the AL
		var outmap = defop.getAttributeMap(false); // false = "Output Map"
		if (outmap != null) {

			var outatts = outmap.getAttributeNames();
			for (i = 0; i < outatts.size(); i++)
				if (outmap.getAttributeMapItem(outatts[i]).getEnabled()) {
					var omi = outmap.getAttributeMapItem(outatts[i]);
					e = system.newEntry();
					e.setAttribute("rss.item.title", outatts[i]);
					e.setAttribute("rss.item.guid", outatts[i]);
					e.setAttribute("rss.item.description", "N/A");
					vect.add(e);
				}

		}

		// If none found, then check the Output Schema
		if (vect.size() == 0) { // If none found, try the Output Schema
			var outsch = defop.getSchema(false);
			if (outsch != null) {
				var schatts = outsch.getItemNames();
				for (i = 0; i < schatts.size(); i++) {
					e = system.newEntry();
					e.setAttribute("rss.item.title", schatts[i]);
					e.setAttribute("rss.item.guid", schatts[i]);
					e.setAttribute("rss.item.description", outsch.getItem(schatts[i]).getExternalSyntax());
					e.setAttribute("rss.item.comments", outsch.getItem(schatts[i]).getJavaClass());
					vect.add(e);
				}
			}
		}
	}
	return vect;
}

// Get the schema for a Feed (either Connector or AL)
function getFeedSchema(feedName) {
	if (getFeedType(feedName) == "assemblyline")
		return getALSchema(feedName)
	else
		return getConnectorSchema(feedName);
}

// Get the schema for a Connector feed
function getConnectorSchema(feedName) {
	return callFeedAL(g_ConnectorFeedAL, "Schema", 1);
}

// Get the URL Base from the configuration file
function getUrlBase() {
	var urlBase = system.getTDIProperty("MIS-Properties", "urlBase");
	if (urlBase == null || urlBase == "" || urlBase == "/") return "/";
	if (urlBase.charAt(0) != "/") urlBase = "/" + urlBase;
	if (urlBase.charAt(urlBase.length-1) != "/") urlBase += "/";
	return urlBase;
}

// Returns xml string for a nodelist, parsing deep
function nodexml(nodelist, namespace) {
	var s = "";
	if (nodelist == null)
		return "";
//	logmsg(">>-- nodexml(" + nodelist.getClass() + ") length: " + nodelist.getLength());
	for (var i = 0; i < nodelist.getLength(); i++) {
		var node = nodelist.item(i);

		if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) { // Element
			s += "<" + ((namespace) ? namespace + ":" : "") +
						node.getNodeName() + nodexml(node.getAttributes(), namespace) + ">" +
						nodexml(node.getChildNodes(), namespace) +
					 "</" + ((namespace) ? namespace + ":" : "") +
					 	node.getNodeName() + ">\n";
		} else if (node.getNodeType() == org.w3c.dom.Node.ATTRIBUTE_NODE) { // Attribute
			s += " " + node.getNodeName() + "=\"" + node.getNodeValue() + "\"";
		} else if (node.getNodeType() == org.w3c.dom.Node.TEXT_NODE) { // Text
			val = node.getNodeValue();
			if (val != null && val.trim() != "") {
				if (val.indexOf("<") >= 0 || val.indexOf(">") >= 0)
					s += system.remove("<>$",val)
				else
					s += val;
			}
		} else if (node.getNodeType() == org.w3c.dom.Node.CDATA_SECTION_NODE) { // Text
			val = node.getNodeValue();
			if (val != null && val.trim() != "")
				s += "<![CDATA[" + val + "]]>";
		}
	}

	return s;
}

function parameterXML(Feeds, feedName, host, urlBase) {
	var paramsxml = "<?xml version='1.0' encoding='UTF-8'?>\n" +
					"<feed xmlns='http://www.w3.org/2005/Atom'>\n" +
					"   <id>" + host + urlBase + "/Parameters</id>\n" +
					"   <title>Mashup Integration Server</title>\n" +
					"   <subtitle>Parameters for Feed: " + feedName + "</subtitle>\n" +
					"   <link rel='self' href='" + host + urlBase + feedName + "/Parameters' />\n" +
					"   <generator>TDI</generator>\n" +
					"   " + makeDateElement("atom");

	param = Feeds.getObject(feedName);
	for (i = 0; i < param.getLength(); i++) {
		atts = param.item(i).getAttributes();
		attname = atts.getNamedItem("id").getNodeValue();
		publish = atts.getNamedItem("publish").getNodeValue();

		if (!publish || publish.toLowerCase() == "true")
			paramsxml += "      <entry>\n" +
					 "         <id>urn:uuid:" + i + "</id>\n" +
					 "         " + makeDateElement("atom") +
					 "         <title>" + attname + "</title>\n" +
					 "         <content type='application/xml' xmlns:MIS='urn:www-ibm-com:tdi:mashup:1.0'>\n" +
					 "         <MIS:parameter " + nodexml(atts) + ">\n" +
								nodexml(param.item(i).getChildNodes(), "MIS") +
					 "         </MIS:parameter>\n" +
					 "         </content>\n" +
					 "      </entry>\n";

	}

	return paramsxml + "</feed>";
}

// Remove illegal characters
// Ampersands are removed if not a part of the "amp, lt, gt, apos, quot" patterns (we inject only these into strings)
function removeFunkyChars(s) {
    var input = String(s);
    var eyecatcher = ";aMppErSaNd;";

    // Search for ampersand
    if (input.indexOf("&") > -1) {

	   	// Replace out ampersand in valid patterns with eye-catcher string
    	input = input.replace(/&amp;/g, eyecatcher+"amp;");
    	input = input.replace(/&lt;/g, eyecatcher+"lt;");
    	input = input.replace(/&gt;/g, eyecatcher+"gt;");
    	input = input.replace(/&apos;/g, eyecatcher+"apos;");
    	input = input.replace(/&quot;/g, eyecatcher+"quot;");

    	// Remove ampersands
    	input = input.replace(/&/g, "");

    	// Undo eye-catcher string
    	input = input.replace(/;aMppErSaNd;/g, "&");
	}

	// TODO figure out what chars actually need to be removed
    return String( system.remove( "<>", system.removeInvalidXMLChars( input ) ) );
}

// Return ReadFeeds connector, or null if cannot load
function getReadFeedsConnector() {
	try {
		feedConn = system.loadConnector("Connectors/ReadFeeds");
		feedConn.initialize(null);
		feedConn.selectEntries();
//		logmsg("@@Loaded " + system.getTDIProperty("MIS-Properties", "FeedDefinitionFile"));
		return feedConn;
	} catch (ex) {
//		logmsg("@@Could not load " + system.getTDIProperty("MIS-Properties", "FeedDefinitionFile"));
		return null;
	}
}
