/* Copyright IBM Corp. 2008  All Rights Reserved.                    */
if(!dojo._hasResource["tagservices.skins"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["tagservices.skins"] = true;
dojo.provide("tagservices.skins");

dojo.require("dojo.string");
dojo.require("dojo.dnd.move");
dojo.require("dojox.uuid.generateRandomUuid");
dojo.require("com.ibm.mm.builder.widget.mumResizeHandler");
dojo.require("com.ibm.mm.builder.widget.mumHeightResizer");
dojo.require("com.ibm.mm.builder.dnd.WidgetDropTarget");
dojo.require("com.ibm.mm.builder.iwidgetbean.pageMode");

/*
Each skin should have a skin.html and skin.js file.
skin.html file serves as a template with desirable variables. As a skin developer, you may
define any number of variables for you to use as long as you setup the variable correctly
in the process function which is defined in skin.js

skin.js file is a json string file, it defines all the variables in the skin.html and how they should be created.
In this file, function process can be defined as follows:

process:function(skinId,tagId) {
}

this function will be called to form all the variables which are used in the skin.html.
you may have variables in skin.html file like this
	${var01}
	${var02}
When you have these, you will have to define these two variables with correct values. Of course,
in process function, you have chance to set these variables to other values. Since function process
will be given two objects, one is the skinId and the other is the tagId (widget id normally), so 
a lot of action can be taken to set up all necessary variables.

There are two variable that you can use in the skin.html file, but you must not provide the value for,
these two variables are
	${widgetBody}
	${skinId}
Even if you provide values for these two variables, they will be overwritten;
*/
dojo.declare("tagservices.skins", null, 
{
	trace:false,
	skinIdPrefix:"mm_ibm_skin_",
	allSkins:[],
	allVars:[],
	allSkinURL:[],
	tempSkinHtmlObj:null,
	tempSkinJsObj:null,
	
	isProcessing:0,
	waitingList:[],
	subscribeHandler:null,
	skinAttachEvent:"/skinservice/attachEnd",
	
	processTag: function(tag){
		this.tempSkinHtmlObj = new Object();
		this.tempSkinJsObj = new Object();
		if (tag == null) {
			return;
		}
		if (this.trace) {
			com.ibm.enabler.debug.log("aTag " + tag.id + " " + tag.getAttribute("skin"));
			com.ibm.enabler.debug.log(this._getNodeMarkup(tag));
		}
		var skinId = tag.getAttribute("skin");	
		this._getSkinTemplate(skinId,tag);	
	},
	
	_getSkinTemplate:function(skinId,tag) {
		 var aSkin = this.allSkins[skinId];
		 if (aSkin == null) {
		 	 var skinroot = this._getSkinRoot(skinId);
			 if (skinroot == ""){
				com.ibm.enabler.debug.error(com.ibm.mm.builder.coreWidgetsMessages.E_SKIN_NOT_EXIST_0);
				return;		 	
			 }
			 var thisObj = this;
		 	 var skinMarkupURL = skinroot + "skin.html";
			 dojo.xhrGet({
				 url: skinMarkupURL,
				 handleAs: "text",
				 sync: true,
				 load:function(htmlData) {	
				 	 var skinVarURL = skinroot + "skin.js";
					 dojo.xhrGet({
						 url: skinVarURL,
						 handleAs: "json",
						 sync: true,
						 load:function(jsData) {
						 	// remove the copyright in the skin.html		
							htmlData = htmlData.replace(/<!--.*-->\s*/,"");
							thisObj.allSkins[skinId] = htmlData;
							thisObj.allVars[skinId] = jsData;
							thisObj.startAttatchSkin(skinId,tag);
						 },
						 error:function(data) {
							thisObj.allSkins[skinId] = htmlData;
							thisObj.startAttatchSkin(skinId,tag);						 	
						 }
					 });				 	
				 },
				 error:function(data) {
					com.ibm.enabler.debug.error(dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_SKIN_LOAD_TEMPLATE_FAIL_1,[data]));
				 }
			 });
		 }else{
		 	this.startAttatchSkin(skinId,tag);
		 }
	},
	
	startAttatchSkin:function(skinId,tag){
		if (this.isProcessing == 0){
			this.executeReplaceSkin(skinId,tag);
		}else{
			this.addToWaitingList(skinId,tag);
		}
	},
	
	chooseNext:function(){
		if (this.waitingList.length!=0){
			var theObject = this.waitingList.pop();
			this.executeReplaceSkin(theObject.skin,theObject.tag);
		}
	},
	
	addToWaitingList:function(skinId,tag){
		if (this.subscribeHandler==null){
			this.subscribeHandler = dojo.subscribe(this.skinAttachEvent, this, "chooseNext");
		}
		var waitObj = new Object();
		waitObj.skin = skinId;
		waitObj.tag = tag;
		this.waitingList.push(waitObj);
	},	
	
	executeReplaceSkin:function(skinId,tag){
		this.isProcessing = 1;
		var temp = this.allSkins[skinId];
		var container;
		if (temp != null) {
			var nodeToReplace = this._getNodeToReplace(tag);  
			var parentNode = nodeToReplace.parentNode;	 
			var anId = dojox.uuid.generateRandomUuid();
			var args = this._getArgs(skinId, tag, anId);
			var widgetMD = mainPage.getWidgetMetadataByInstanceId(tag.id);
			if(widgetMD){
					args.title = widgetMD.widgetname;
			}
			if (args == null) {
				com.ibm.enabler.debug.error(com.ibm.mm.builder.coreWidgetsMessages.E_SKIN_NOT_EXIST_0);
				return;
			}
			
			mainPage.loadCss(args.skinRoot + "skin.css");
			var newMarkup = dojo.string.substitute(temp, args);
			//TODO
			newMarkup = newMarkup.replace(/\/>/g,"><\/div>");	
			
			var newNode = this._getNodeFromMarkup(newMarkup);
			//TODO
			dojo.addClass(newNode,"mumWidgetTitleBar");
			
			parentNode.replaceChild(newNode, nodeToReplace); 
			var tNode = dojo.byId(anId);
			tNode.parentNode.replaceChild(tag, tNode);
			
			// new code to insert drag handler and resize handler
			var tid = tag.id;
			var skinDiv = dojo.byId(this.skinIdPrefix + tid);
			
			container = skinDiv.parentNode;
			
			if(container.className.indexOf("nva")<0) {
				//add editBar and related event
				var editBarNode = this._getEditBarNode(args);			
				newNode.onmouseout = function(){com.ibm.mm.builder.skinHelper.hideEditBar(args.widgetId+"_dragHandler_editbar");};
				var widgetBody = dojo.query(".widgetBody",newNode);
				if(widgetBody.length)
					widgetBody[0].onmouseover = function(){com.ibm.mm.builder.skinHelper.displayEditBar(args.widgetId+"_dragHandler_editbar");};
				newNode.insertBefore(editBarNode,newNode.firstChild);
			}
								
			if (container.className.indexOf("freednd")!=-1){
				// 1. init resizer handler
				var resizeBar = dojo.byId(tid+com.ibm.mm.builder.constant.resizeHandler);

				// prepare the parameter for the resizer
				var resizerdiv = document.createElement("div");
				var mHeight=0,mWidth=0,marginHeight=0,marginWidth=0,resizerHeight=16;
				if (typeof args.resizerHeight != "undefined"){
					resizerHeight = args.resizerHeight;
				}
				if (typeof args.minHeight != "undefined"){
					mHeight = args.minHeight;
				}
				if (typeof args.minWidth != "undefined"){
					mWidth = args.minWidth;
				}				
				if (typeof args.marginHeight != "undefined"){
					marginHeight = args.marginHeight;
				}	
				if (typeof args.marginWidth != "undefined"){
					marginWidth = args.marginWidth;
				}								
				var resizeHandlerProps = new Object();
				resizeHandlerProps.targetElmId = this.skinIdPrefix + tid;
				resizeHandlerProps.contentElmId = tid+"_content";
				resizeHandlerProps.id = tid + com.ibm.mm.builder.constant.resizeHandlerExt;
				if (mHeight>0){
					resizeHandlerProps.minHeight = mHeight;
				}
				if (mWidth>0){
					resizeHandlerProps.minWidth = mWidth;
				}
				// end the parameter
				var handl = null;
				if (dijit.byId(resizeHandlerProps.id)){
					handl = dijit.byId(resizeHandlerProps.id);
				}else{
					handl = new com.ibm.mm.builder.widget.mumResizeHandler(resizeHandlerProps,resizerdiv);
				}
				resizeBar.appendChild(handl.domNode);
				
				// 2. init drag handler
				var dragHandler = dojo.byId(tid+com.ibm.mm.builder.constant.dragHandler);
				//dojo.addClass(dragHandler,"mumWidgetTitleBar");
				
				// 3. init the height and width of div
				var locationStyle = tag.getAttribute("locationstyle");
				var height = this._getValueFromStyle("height",locationStyle);
				var width = this._getValueFromStyle("width",locationStyle);
				//TODO
				if (height!=null){
					var heightNumber = parseInt(height);
					var widthNumber = parseInt(width);
					widthNumber = widthNumber - marginWidth;
					var contentHeight = heightNumber - resizerHeight;//resizer height
					contentHeight = contentHeight - marginHeight;
					var contentDiv = dojo.byId(tid+com.ibm.mm.builder.constant.widgetContent);
					dojo._setMarginBox(contentDiv,0,0,widthNumber,contentHeight,contentDiv.style);	
				}
				
				// 4. init the drop target to enabler nested drop
				//new com.ibm.mm.builder.dnd.WidgetDropTarget(dojo.byId(this.skinIdPrefix + tid),{type:"freeform"});
				new com.ibm.mm.builder.dnd.WidgetDropTarget(dojo.byId(tid + com.ibm.mm.builder.constant.dragHandler+"_editbar"),{type:"freeform"});
				// 5. enable it to drag
				dojo.addClass(dojo.byId(com.ibm.mm.builder.constant.skinIdPrefix + tid),"dojoDndItem")
				
				if (com.ibm.mm.builder.pageMode.getPenStatus() == false){
					var editbar = dojo.byId(tid + com.ibm.mm.builder.constant.dragHandler+"_editbar");
					dojo.byId(tid + com.ibm.mm.builder.constant.resizeHandlerExt).style.display = "none";
					editbar.style.display = "none";
					dojo.byId(tid+"_widgetBody").onmouseover = function(){};
				}
				// end				
			}else if (container.className.indexOf("nva")!=-1){
				var iconDiv = dojo.byId(tid + "_icon");
				var iconSrc = mainPage.getWidgetMetadataByInstanceId(tid).widgetIconSrc
				iconDiv.innerHTML = "<img src='" + iconSrc + "'/>";
				// concat the display title if it is too long
				var title = mainPage.getWidgetMetadataByInstanceId(tid).widgetname;
				if(title.length>10) {
					dojo.byId(tid+ "_name_middle").innerHTML = title.substring(0,8) + "...";
					dojo.byId(tid+ "_name_middle").title = title;
				} else {
					dojo.byId(tid + "_name_middle").innerHTML = title;
				}
				dojo.addClass(dojo.byId(com.ibm.mm.builder.constant.skinIdPrefix + tid),"dojoDndItem");
				dojo.byId(tid + "_wireIcon").title = com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_WIRE_ICON_TITLE;
				dojo.byId(tid + "_editIcon").title = com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_EDIT_ICON_TITLE;
				new com.ibm.mm.builder.dnd.WidgetDropTarget(dojo.byId(tid+com.ibm.mm.builder.constant.nva_skin_upper),{type:"nva"});
			}else{
				if (container.className.indexOf("fixeddnd")!=-1){
					// 1. init vertical resizer handler
					var resizeBar = dojo.byId(tid+com.ibm.mm.builder.constant.resizeHandler);
					var resizerdiv = document.createElement("div");
					// prepare the parameter for height resizer
					var resizeHandlerProps = new Object();
					resizeHandlerProps.id = tid + com.ibm.mm.builder.constant.resizeHeightExt;
					resizeHandlerProps.targetElmId = this.skinIdPrefix + tid;
					resizeHandlerProps.contentElmId = tid + com.ibm.mm.builder.constant.widgetContent;
					var mHeight=0,resizerHeight=16;
					if (typeof args.resizerHeight != "undefined"){
						resizerHeight = args.resizerHeight;
					}
					if (typeof args.minHeight != "undefined"){
						mHeight = args.minHeight;
					}
					resizeHandlerProps.minHeight = parseInt(mHeight);
					// end the parameter
					var handl = null;
					if (dijit.byId(resizeHandlerProps.id)){
						handl = dijit.byId(resizeHandlerProps.id);
					}else{
						handl = new com.ibm.mm.builder.widget.mumHeightResizer(resizeHandlerProps,resizerdiv);
					}
					resizeBar.appendChild(handl.domNode);

					// 2. init the height and width of div
					var locationStyle = tag.getAttribute("locationstyle");
					var height = this._getValueFromStyle("height",locationStyle);
					if (height!=null){
						var heightNumber = parseInt(height.substr(0,height.length - 2));
						var contentHeight = heightNumber - resizerHeight;
						var contentDiv = dojo.byId(tid+com.ibm.mm.builder.constant.widgetContent);
						dojo._setMarginBox(contentDiv,0,0,-1,contentHeight,contentDiv.style);					
					}
							
					// 3. add dojoDndItem to make it can dnd from column to column
					dojo.addClass(dojo.byId(this.skinIdPrefix + tid),"dojoDndItem");
					
					// 4. init the drop target to enabler nested drop
					new com.ibm.mm.builder.dnd.WidgetDropTarget(dojo.byId(tid + com.ibm.mm.builder.constant.dragHandler+"_editbar"),{type:"column"});
									
					if (com.ibm.mm.builder.pageMode.getPenStatus() == false){
						var editbar = dojo.byId(tid + com.ibm.mm.builder.constant.dragHandler+"_editbar");						
						editbar.style.display = "none";
						dojo.byId(tid + com.ibm.mm.builder.constant.resizeHeightExt).style.display = "none";
						dojo.removeClass(dojo.byId(this.skinIdPrefix + tid), "dojoDndItem");
						dojo.byId(tid+"_widgetBody").onmouseover = function(){};
					}
				}
			}
		}// end of else
		if(container.className.indexOf("nva")<0) {
			dojo.subscribe(com.ibm.mm.builder.Topics.WidgetMenu_Hide+tag.id,this,"_widgetOnMouseOut");
		}
		this.isProcessing = 0;
		dojo.publish(this.skinAttachEvent, [null]);
	},

	_widgetOnMouseOut:function(iwidgetId){
		dojo.byId(this.skinIdPrefix+iwidgetId).onmouseout=function(){com.ibm.mm.builder.skinHelper.hideEditBar(iwidgetId+"_dragHandler_editbar");};
		com.ibm.mm.builder.skinHelper.hideEditBar(iwidgetId+"_dragHandler_editbar");
	},
	
	_getEditBarNode:function(args){
		/*<div id="${widgetId}_dragHandler_editbar" style="display:none" class="editBar" onmouseover="javascript:com.ibm.mm.builder.skinHelper.setEditBarOpaque('${widgetId}_dragHandler_editbar')" >
			<div id="${widgetId}_dragHandler" class="widgetDragHandler" >
				<div class="widgetDragHandlerMiddle">
					<img src="${iconUrl}"  class="widgetIconStyle">
					<span class="widgetTitleShadow" >${title}</span>
					<span class="widgetTitle" >${title}</span>
					<img src="/mum/images/interactions_icon.png" class="widgetInteraction" title="Click to wire widgets" onclick="javascript:com.ibm.mm.builder.skinHelper.lauchWiring('${widgetId}')" border=0>
					<img id="${menuId}" src="/mum/images/context_menu.png" class="widgetContext" title="Click to open menu of options" onclick="javascript:com.ibm.mm.builder.skinHelper.createMenu(event,'${menuId}','${widgetId}')" border=0>	
				</div>	
				<div class="widgetDragHandlerLeft"></div>
				<div class="widgetDragHandlerRight"></div>
			</div>
			<div class="widgetDragHandlerBG"></div>
			<div class="widgetDragHandlerCorner"></div>
			<div class="editBarRightTopCornerShadow"></div>	
			<div class="editBarCornerShadow"></div>
		</div>*/
		var editBar = document.createElement("div");
		dojo.addClass(editBar,"editBar");
		editBar.id = args.widgetId + '_dragHandler_editbar';
		editBar.style.display = "none";
		editBar.onmouseover = function(){com.ibm.mm.builder.skinHelper.setEditBarOpaque(args.widgetId + '_dragHandler_editbar');};
		
		var widgetDragHandler = document.createElement("div");
		widgetDragHandler.id = args.widgetId + '_dragHandler';
		dojo.addClass(widgetDragHandler,"widgetDragHandler");
		
		var widgetDragHandlerMiddle = document.createElement("div");
		dojo.addClass(widgetDragHandlerMiddle,"widgetDragHandlerMiddle");
		
		var widgetIcon = document.createElement("img");
		widgetIcon.src= args.iconUrl;
		dojo.addClass(widgetIcon,"widgetIconStyle");
		
		var widgetTitleShadow = document.createElement("span");
		widgetTitleShadow.innerHTML= args.title;
		dojo.addClass(widgetTitleShadow,"widgetTitleShadow");
		
		var widgetTitle = document.createElement("span");
		widgetTitle.innerHTML= args.title;
		dojo.addClass(widgetTitle,"widgetTitle");
		
		if(dojo.isSafari || dojo.isIE){
			widgetTitleShadow.style.width = "75%";
			widgetTitle.style.width = "75%";
		}
		
		var wiringIcon = document.createElement("img");
		wiringIcon.src= builderRootPath + "/images/d_wiring_icon.png";
		wiringIcon.alt= com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_WIRE_ICON_TITLE;
		if(dojo.isFF)  // FF does not display alt as tip for image
			wiringIcon.title= com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_WIRE_ICON_TITLE;
		dojo.addClass(wiringIcon,"widgetInteraction");
		wiringIcon.onclick = function(){com.ibm.mm.builder.skinHelper.lauchWiring(args.widgetId)};
		// fix dnd issue when click the icon
		wiringIcon.onmousedown = function(evt) {dojo.stopEvent(evt)}
		
		var menuIcon = document.createElement("img");
		menuIcon.id= args.widgetId + "_menu";
		menuIcon.src= builderRootPath + "/images/context_menu.png";
		menuIcon.alt= com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_EDIT_ICON_TITLE;
		if(dojo.isFF)  // FF does not display alt as tip for image
			menuIcon.title= com.ibm.mm.builder.coreWidgetsStrings.I_SKIN_EDIT_ICON_TITLE;
		dojo.addClass(menuIcon,"widgetContext");
		menuIcon.onclick = function(event){com.ibm.mm.builder.skinHelper.createMenu(event,args.widgetId+"_menu",args.widgetId)};
		// fix dnd issue when click the icon
		menuIcon.onmousedown = function(evt) {dojo.stopEvent(evt)}
		
		widgetDragHandlerMiddle.appendChild(widgetIcon);
		widgetDragHandlerMiddle.appendChild(widgetTitleShadow);
		widgetDragHandlerMiddle.appendChild(widgetTitle);
		widgetDragHandlerMiddle.appendChild(wiringIcon);
		widgetDragHandlerMiddle.appendChild(menuIcon);
		
		var widgetDragHandlerLeft = document.createElement("div");
		dojo.addClass(widgetDragHandlerLeft,"widgetDragHandlerLeft");
		
		var widgetDragHandlerRight = document.createElement("div");
		dojo.addClass(widgetDragHandlerRight,"widgetDragHandlerRight");
		
		widgetDragHandler.appendChild(widgetDragHandlerLeft);
		widgetDragHandler.appendChild(widgetDragHandlerMiddle);
		widgetDragHandler.appendChild(widgetDragHandlerRight);
		
		var widgetDragHandlerBG = document.createElement("div");
		dojo.addClass(widgetDragHandlerBG,"widgetDragHandlerBG");
		
		var widgetDragHandlerCorner = document.createElement("div");
		dojo.addClass(widgetDragHandlerCorner,"widgetDragHandlerCorner");
		
		var editBarRightTopCornerShadow = document.createElement("div");
		dojo.addClass(editBarRightTopCornerShadow,"editBarRightTopCornerShadow");
		
		var editBarCornerShadow = document.createElement("div");
		dojo.addClass(editBarCornerShadow,"editBarCornerShadow");
		
		editBar.appendChild(widgetDragHandler);
		editBar.appendChild(widgetDragHandlerBG);
		editBar.appendChild(widgetDragHandlerCorner);
		editBar.appendChild(editBarRightTopCornerShadow);
		editBar.appendChild(editBarCornerShadow);
	
		return editBar;
	},
		
	//_getValueFromStyle will parse the width,height.. from the locationStyle string
	//locationStyle should be "position:absolute;width:300px;height:400px;left:20px;top:400px;z-Index:40;"
	_getValueFromStyle:function(key,locationStyle){
		if (locationStyle.indexOf(key)==-1){return null;}
		var pos = locationStyle.indexOf(key);
		var substr1 = locationStyle.substr(pos,locationStyle.length);
		var startPos = substr1.indexOf(":");
		var endPos = substr1.indexOf(";");
		var value = substr1.substr(startPos + 1,endPos - startPos);
		return value;
	},

	_getSkinRoot:function(skinId) {
		try{
			if (this.allSkinURL[skinId]!=null){
				return this.allSkinURL[skinId];	
			}
			var curThemeId = mainPage.themeId;
			var themeModel = com.ibm.mashups.enabler.model.Factory.getThemeModel();
			themeModel.setStrategy(new com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy(5));
			themeModel.setCursorPosition(0);
			var theme = null;
			while (themeModel.hasNext()) {
				theme = themeModel.next();
				if (theme.getID().indexOf(curThemeId) != -1){
					break;
				}
			}
			var baseURL = "";
			if (theme!=null){
				var skinModel = themeModel.getSkinModel(theme);
				skinModel.setStrategy(new com.ibm.mashups.enabler.strategy.ListLoadAheadStrategy(10));
				skinModel.setCursorPosition(0);
				while (skinModel.hasNext()) {
					var skin = skinModel.next();
					if (skin.getName() == skinId){
						baseURL = skin.getBaseUrl();
						break;
					}
				}			
			}else{
				com.ibm.enabler.debug.error(com.ibm.mm.builder.coreWidgetsMessages.E_SKIN_THEME_NOT_EXIST_0);
			}
			this.allSkinURL[skinId] = baseURL;
			return baseURL;			
		}catch (ex){
			com.ibm.enabler.debug.error(dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_SKIN_LOAD_ROOT_FAIL_1,[ex.message]));
		}
	},

	_getArgs:function(skinId, tag, anId) {
		args = {};
		//fixme. use the skin substitution to do this.
		//here we will allow users to do things.
		var skinVar = this.allVars[skinId];
		if (skinVar != null) {
			dojo.mixin(args, skinVar);
		}		 
		//in case, skin defines the widgetBody and skinId, we need to overwrite.
		args.widgetBody = "<div id='" +anId + "'/>";
		args.skinId = this.skinIdPrefix+tag.id;
		var skinRoot = this._getSkinRoot(skinId);
		if (skinRoot == ""){
			return null;
		}
		args.skinRoot = skinRoot;
		if (args.process == "undefined" || args.process == null) {
			 args.process = function(skinId, tagId){
			 }
		}
		args.process(skinId, tag.id);
		return args;
	},

	_getNodeMarkup:function(tag) {
		 var newNode = document.createElement("div");
		 newNode.appendChild(tag.cloneNode(true));
		 var markup = newNode.innerHTML;
		 newNode.removeChild(newNode.firstChild);
		 return markup;
	},

	_getNodeFromMarkup:function(markup) {
		 var newNode = document.createElement("div");
		 newNode.innerHTML = markup;
		 return newNode.removeChild(newNode.firstChild);		  	 
	},

	_getNodeToReplace:function(tag) {
		 var node = dojo.byId(this.skinIdPrefix+tag.id);
		 if (node == null) {
			 node = tag;
		 }
		 return node;
	},

	_showParent:function(node) {
		 var parent = node.parentNode;
		 while (parent!=null) {
			 parent = parent.parentNode;
		 }
	},

	_dummy:function() {
	}
});

}
