/* Copyright IBM Corp. 2008  All Rights Reserved.                    */
dojo.require("dojox.uuid.generateRandomUuid");

dojo.declare("tagservices.state", null,
{
	id:null,
	loaded:false,
	constructor:function(tagId, tagLoaded)
	{
		this.id = tagId;
		this.loaded = tagLoaded;
	}
});

var TagService = {	

	trace:false,
	_tagTypes:null,
	_tagStates:[],
	tagStatusChange:"/tagservices/tagstatuschanged",
	pageStatusChange:"/tagservices/pagestatuschanged",

	init: function() { 	 
		this._loadTags(); 	
		dojo.subscribe(this.tagStatusChange, this, "_handleTagChange");
		dojo.subscribe(this.pageStatusChange, this, "_handlePageChange");
		dojo.publish(this.pageStatusChange, [null]);
		//dojo.addOnLoad(dojo.hitch(this, "processChildTags"));
	},

	_handleTagChange:function(arg) {
		 var aNode = dojo.byId(arg);
		 if (aNode != null) {
			 this.processTag(arg);
		 }
	},

	_handlePageChange:function(arg) {
		 this.resetAllTags();
		 this.processChildTags(null);
	},


	processTag:function(node) {
		var tag = null, tags= null;		 
		if (node == null) {
			return;
		}
		var currentNode = dojo.byId(node);
		var aNode = currentNode.cloneNode(false);
		var theParentNode = document.createElement('div');
		theParentNode.appendChild(aNode);
		this.resetTag(currentNode);
		for (var index=0; index<this._tagTypes.length;index++) {
			tag = this._tagTypes[index];
			if (tag.loaded == null){
				this._loadTagHandler(tag);
			}
			tags = dojo.query(tag.match, theParentNode);
			if (tags.length == 1) {
				try{
					tag.tagHandler.processTag(currentNode);
				}catch(error){
					com.ibm.enabler.debug.error(dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_TAG_SERVICE_ATTACH_FAIL_1,[error.message]));
				}				 
			}
		}
	},

	processChildTags:function(node) {
		var tag = null, tags= null;
		node = (node == null)?document:node;
		for (var index=0; index<this._tagTypes.length;index++) {
			tag = this._tagTypes[index];
			tags = dojo.query(tag.match, node);
			if (tags.length > 0) {
				this._processTypeTags(tags, tag);
			}
		}
	},

	_loadTags:function() {	
		if (this._tagTypes == null) {
			var thisObj = this;
			dojo.xhrGet({
				url: ibmConfig["contextRootEnabler"]+"/js/tagservices/tagservice.entries.js",
				handleAs: "text",
				sync: true,
				load:function(result) {
					thisObj._tagTypes = dojo.fromJson(result);
				},
				error:function(data) {
				 	com.ibm.enabler.debug.error(dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_TAG_SERVICE_LOAD_JS_FAIL_1,[data]));
				}
			});
		}
	},

	_processTypeTags:function(tags, tag) {
		 if (tag.loaded == null) {
			 this._loadTagHandler(tag);
		 }
		 if (tag.tagHandler != null) {
			 var aState = null;
			 var thisObj = this;
			 var tagList=[];
			 var anId=null;

			 //here we setup all the semantic tag list
			 tags.forEach(function(item){ 
				 if (item.id == null || item.id == "undefined" || item.id == "") {
					 item.id = thisObj._getNewId();					 
				 }
				 
				 anId = item.id + "_" + tag.match;
				 aState = thisObj._tagStates[anId];
				 if (aState == null) {
					 aState = new tagservices.state(anId, false);
					 thisObj._tagStates[aState.id] = aState;
				 }
				 tagList[tagList.length] = item.id;
			 });

			 //we start process each tag.
			 var itemId = null;
			 for (var index=0;index<tagList.length;index++) {
				 itemId = tagList[index];
				 anId = itemId +"_"+tag.match;
				 aState = this._tagStates[anId];
				 if (aState.loaded == false) {
					 if (this.trace) { com.ibm.enabler.debug.log("processing " + itemId);}					 
					 tag.tagHandler.processTag(dojo.byId(itemId));
					 aState.loaded = true;
				 }
			 }
		 }
	},

	_loadTagHandler:function(tag) {
		 dojo.registerModulePath(tag.module, ibmConfig["contextRootEnabler"] + tag.path);
		 dojo.require(tag.baseClass);
		 var jsonStr = "{create:function(){return new " + tag.baseClass + "()}}";
		 var classCreator = dojo.fromJson(jsonStr);
		 tag.tagHandler = classCreator.create();
		 tag.loaded = true;
	},

	resetTag:function(tagId) {
		 var aState = this._tagStates[tagId];
		 if (aState != null) {
			 aState.loaded = false;
		 }
	},
	
	_getNewId:function() {
		 return dojox.uuid.generateRandomUuid();
	},

	resetTag:function(tagId) {
		 var aState, anId;
		 var node = dojo.byId(tagId);
		 if (node == null) {
			 return;
		 }
		 for (var index=0; index<this._tagTypes.length;index++) {
			 tag = this._tagTypes[index];
			 anId = node.id + "_" + tag.match;
			 aState = this._tagStates[anId];
			 if (aState != null) {
				 aState.loaded = false;
			 }
		 }
	},
	
	resetAllTags:function() {
		 this._tagStates = [];
	}
}

//TagService.init();
dojo.addOnLoad(dojo.hitch(TagService, "init"));
