/* (C) Copyright IBM Corp. 2007  All Rights Reserved.                */
/**
 * This js file is the semantic tagging service for click-to-action.  It is
 * loaded when either a "c2a:source" or a "c2a:target" node is detected on a
 * page.  
 *
 * During initialization, it does the following:
 * 
 *   - sets up a callback function to handle dom changes
 *   - sets up a callback function for adding menu items to other live objects
 *   - processes c2a:targets and builds an array of c2a target types
 *   - processes c2a:source nodes and checks for matching targets 
 *   - if a match is found, it generates UI to place a menu icon next 
 *     to the source value on the page
 *
 * When the user clicks on a menu icon, it:
 *
 *    - 
 *
 * Bring this back when you add jsp stuff below
<%@ page language="java" contentType="text/html;charset=UTF-8" %>

 */

var SemTagC2A = {

        // -------------------------------------------------------------
        // Constant Definitions
        // -------------------------------------------------------------
        serviceId:  "c2a",
        c2aIdBaseStr: "IBMC2ASRVC_",
        c2aSourceIdBase: "",
        c2aTargetIdBase: "",

        // c2a Source tag names
        sourceTagName:      "c2a:source",
        typeTagName:        "c2a:type",
        valueTagName:       "c2a:value",
        namespaceTagName:   "c2a:namespace",
        defaultNameSpaceStr:"IBM_C2A_DEFAULT_NAMESPACE",
        headerTagName:      "c2a:header",
        broadcastTagName:   "c2a:broadcast",

        // c2a Target tag names.
        // (Note: Target tag also contains the c2a:type, c2a:value, and 
        //        c2a:namespace tags defined above.)
        // 
        targetTagName:    "c2a:target",
        actionTagName:    "c2a:action",
        menulabelTagName: "c2a:menulabel",
        paramTagName:     "c2a:paramname",

        // -------------------------------------------------------------
        // Target Definition Arrays
        // -------------------------------------------------------------

        // Optimization: 2D array (indexed by c2a:namespace and c2a:type values)
        // to quickly determine if a c2a target of given namespace and type exists
        // on page.  Prevents us from having to traverse Targets array to
        // NOTE: Index order matters, namespace is first index, type is second.
        //       eg --> a[c2a:namespaceValue,c2a:typeValue]
        targetsAvail:	null,    

        // targetNodeIDs is an array of target DOM nodes IDs which is used at
        // menu generation time
        targetNodeIDs:		new Array(),    

        // -------------------------------------------------------------
        // Source Definition Arrays
        // -------------------------------------------------------------

        // sourceNodeIDs is an array of source DOM nodes IDs which is 
        // use at client-side aggregation 
        sourceNodeIDs:		new Array(),    

        // -------------------------------------------------------------
        // Current Source Values
        // -------------------------------------------------------------
        currentValue:		"",
        currentValNode:      null,
        
        // -------------------------------------------------------------
        // counter used to generate unique IDs for source/target nodes
        // -------------------------------------------------------------
        IDctr:	0,

        // -------------------------------------------------------------
        // Bootstrap Function
        // -------------------------------------------------------------
        init: function () {
        
           // initialize counter used to generate unique IDs for source/target nodes
           IDctr = 0,

           SemTagC2A.c2aSourceIdBase = SemTagC2A.c2aIdBaseStr + "Source_";
           SemTagC2A.c2aTargetIdBase = SemTagC2A.c2aIdBaseStr + "Target_";
         
           // create regular expressions for tag comparisons
           SemTagC2A.defineRegExpressions();
           
           //
           //  set up a call back to rescan the page when the DOM changes
           //
           SemTagSvc.setCallback(SemTagC2A.serviceId,SemTagC2A.parseDOM);

           //  set up callback for add menu items to non-c2a:source live object
           SemTagSvc.setSpecialMenuProvider(SemTagC2A.getSpecialC2AMenuItems);
           
           // parse the DOM for sources and targets
           SemTagC2A.parseDOM();	
        },

        // -------------------------------------------------------------
        // Callback Functions -- functions that the framework or browser
        // will call to handle events
        // -------------------------------------------------------------

        getSpecialC2AMenuItems:  function(elem) {

            // This method checks for c2a targets for objects that are marked
            // using a microformat rather than a c2a:source.  In this case,
            // the semantic tags themselves provide the source namespace and type.

            // Note:  this code has to return quickly, or it will 
            // make the menu system very slow

            var menuItems = null;
            var namespaceNode = null;
            var namespaceName = null;

            // Check all target namespaces to find out whether the element
            // we have been passed is a sub-element for that namespace.  
            // 
            // Microformats may be nested.  We look for the closest parent
            // whose semantic tagging matches a target namespace.
            //
            // For example, if a portlet contains the following markup and the
            // user clicks on the LOF menu attached to the street-address node.
            // we want to give them actions in the adr namespace, NOT vcard
            //
            //   <div class="vcard">
            //      <a class="email fn" href="mailto:stephan.hesmer@de.ibm.com">Stephan Hesmer</a>
            //      <li class="org">IBM Deutschland Entwicklung GmbH</li>
            //      <div class="adr">
            //      <span class="street-address">Schoenaicherstr. 220</span>
            //      <li><span class="locality">Boeblingen</span>, <span class="region">BW</span><span class="postal-code">71032</span></li>
            //      <li class="country-name">Germany</li>
            //      <div class="tel">
            //         <li><span class="value">+49-7031-16-3275</span> (<abbr class="type" title="WORK">w</abbr>, <abbr class="type" title="VOICE">v</abbr></li>
            //      ....
            for (name in SemTagC2A.targetsAvail) {
                 var node = SemTagSvc.getParentByClassName(name,elem);
                 if (node == null) continue;

                 // First matching parent we find is stored.  Subsquent matching parent nodes are
                 // compared with the previous match to see which is nested inside the other.  We only
                 // keep the innermost node.
                 if (namespaceNode == null) {
                     namespaceNode = node;
                     namespaceName = name;
                 } else {
                     var tempNode = SemTagSvc.getParentByClassName(namespaceName , node);
                     if (tempNode != null) {
                         namespaceNode = node;
                         namespaceName = name;
                     }                       
                 }
            
            }
            if (namespaceNode == null) return null;

            // 
            try {
               // Targets for microformats may use any element(s) of the microformat
               // definition, so we get all menu items for this namespace
               // and set the currentValue to the whole DOM node.  The particular value
               // for a target will be extracted at execution time

               SemTagC2A.currentValNode = namespaceNode;
               menuItems = SemTagC2A.getTargetMenuItems({"namespace":namespaceName,
                                                         "type":"*",
                                                         "value": SemTagC2A.currentValNode}
                                                       );
            } catch (e) {
                // TBD:  Log error, don't alert.  This is for debugging
                alert("error getting special c2a menu items, exception is:  " + e);
            }
            return menuItems;
        },

        showHover:  function(event) {
		// TBD: Third argument specifies the menu hover text. It will need to be localized.
              SemTagSvc.showHover(event, SemTagC2A.handleClick,"Click for Actions");
        },
        
        handleClick: function (event) {
           // Clear values that were set previously
           SemTagC2A.currentValue = "";
           SemTagC2A.curValueNode = null;

           // This function is called when the user clicks on the menu icon inserted next
           // to the c2a:value tag on the page.  Fetch node from event.
           var liveNode = SemTagSvc.getLiveElementFromEvent(event);
           
           // node is always child c2a:value node.  Get it's parent (ie. c2a:source node)...
           var sourceNode = SemTagSvc.getParentByClassName(SemTagC2A.sourceTagName,liveNode);

           // ... and then go build up menu with the proper menu items
           SemTagC2A.generateMenuContents(event, sourceNode);
        },

        // -------------------------------------------------------------
        // DOM Parsing functions
        // -------------------------------------------------------------
        parseDOM: function() {
            //  Split up sources and targets into separate arrays
            SemTagC2A.processDomNodes(SemTagSvc.getNodes(SemTagC2A.serviceId));

            //  check all c2a sources to determine if they have matching c2a targets.  
            //  If so, add the UI to support the action menu on c2a sources
            SemTagC2A.processSources(SemTagC2A.sourceNodeIDs);
        },
        
        processDomNodes: function(newNodeList) {
           // first scan class member variable source and target arrays to see if any
           // nodes we are holding onto have been deleted from DOM.  If so, remove them from 
           // arrays to prevent ghost source and targets (i.e. ensure class data mirrors DOM).
           var nodeID;
           // run through array of Source node IDs...
           var i = 0;
           while ( i < SemTagC2A.sourceNodeIDs.length ) {
              nodeID = SemTagC2A.sourceNodeIDs[i];
              if (!document.getElementById(nodeID)) {
                 // this source node is no longer present in DOM so remove it from array
                 SemTagC2A.sourceNodeIDs.splice (i,1);
              }else{
                 i++;
              }
           }
           // ...now check Target array
           i = 0; 
           while ( i < SemTagC2A.targetNodeIDs.length ) {
              nodeID = SemTagC2A.targetNodeIDs[i];
              if (!document.getElementById(nodeID)) {
                 // a target node is no longer present in DOM so remove it from array
                 SemTagC2A.targetNodeIDs.splice (i,1);
              }else{
                 i++;
              }
           }
 
           // Finally, scan new list of nodes, copying sources and targets to 
           // corresponding internal source/target arrays.  NOTE: first check new nodes
           // to ensure we don't already have them in these arrays (i.e. prevent duplicates).
           
           while (newNodeList.length > 0) {
              var node = newNodeList.pop();
              // first check to see if node has an ID that we previously assigned.
              var nodeIDstr = node.id;
              if (nodeIDstr != null && nodeIDstr.search(SemTagC2A.c2aIdBaseStr) == 0) {
                 // found base c2a ID string used to generate unique IDs - this node is a 
                 // duplicate (ie. it is already in our list) so drop it on floor.
                 continue;
              }
              // now get node type and add it to appropriate member variable source/target array
              if (SemTagC2A.isSource(node)) {
                 // give it a unique ID we can use when DOM is updated to indicate we 
                 // know about this node, thus preventing duplicates in member source array
                 node.id = SemTagC2A.c2aSourceIdBase + SemTagC2A.IDctr;
                 SemTagC2A.IDctr++;
                 // a new source node is present in DOM so add it's ID to Sources array
                 SemTagC2A.sourceNodeIDs.push(node.id);
              }
              else if (SemTagC2A.isTarget(node)) {
                 // give it a unique ID we can use when DOM is updated to indicate we 
                 // know about this node, thus preventing duplicates in target array
                 node.id = SemTagC2A.c2aTargetIdBase + SemTagC2A.IDctr;
                 SemTagC2A.IDctr++;
                 // and add the node's ID to Targets array
                 SemTagC2A.targetNodeIDs.push(node.id);
              } 
              else {
                 alert("C2A: Passed unknown node type: " + node);
              }
           }
           // ok, we have an updated Targets array.  Now refresh the availability array we use
           // to track known targets.  Note: We don't have an avaiability array for Sources. 
           SemTagC2A.UpdateAvailTargets();           
        },

        processSources: function(sourceIDs) {
           // iterate over c2a source nodes to add menu if necessary
           for (var i = 0; i < sourceIDs.length; i++) {
              // fetch live DOM c2a source node
              var sourceNode = document.getElementById(sourceIDs[i]);
              
              // get the type and header fields
              var info = SemTagC2A.getSourceInfo(sourceNode);
              
              // see if there is even target avail for this source
              var hasTarget = SemTagC2A.isTargetDefined(info);
              
              // fetch sub-element (c2a:value tag) on which hover will be placed
              var valueNode = null;
              var nodes = sourceNode.childNodes;
              for (var j=0; j<nodes.length; j++) {
                var cname = SemTagUtil.getNodeClassValue(nodes[j]);
                if (cname.match(SemTagC2A.valueRegExp)) {
                    valueNode = nodes[j];
                    break;
                }
              }
              
              if (valueNode == null) {
                 // If we get here it means the c2a:source tag has not been specified properly in
                 // the markup.  Specifically, it is missing its child c2a:value tag.
                 alert("Click-to-Action error: Missing c2a:value tag");
                 break;
              }

              // get hover state since we could have previously added hover              
              var hasHover = valueNode.getAttribute("hasHover");
              
              // If matching target was found for this source OR source has a menu header defined in
              // it ("c2a:header" tag), then source should have a menu hover.  Add hover to source's child 
              // c2a:value node (if it doesn't have one) and update hover state.
              if (hasTarget || info.header != "" ) {
                 // have we already put a hover on this node?                
                 if (hasHover == null || hasHover == "false") {  
                    // create the menu icon and insert it into the DOM after the source's c2a:value node
                    SemTagSvc.addHover(valueNode, SemTagC2A.showHover, SemTagC2A.handleClick);           
                    // no support in Semantic Tag framework to track hover status so set our own an attribute.
                    valueNode.setAttribute("hasHover","true");
                 }
              }
              else  // otherwise, remove hover from c2a:value node if it exist and update hover state
              {
                 if (hasHover == "true") {
                    SemTagSvc.removeHover(valueNode,SemTagC2A.showHover,SemTagC2A.handleClick);
                 }
                 // now update c2a:value node's hover state
                 valueNode.setAttribute("hasHover","false");
              }
           }
        },

        // -------------------------------------------------------------
        // Utility Functions
        // -------------------------------------------------------------
        defineRegExpressions:  function() {

            // Source Tag regular expressions for comparison
            SemTagC2A.sourceRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.sourceTagName + "(\\s|$)");
            SemTagC2A.valueRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.valueTagName + "(\\s|$)");
            SemTagC2A.namespaceRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.namespaceTagName + "(\\s|$)");
            SemTagC2A.typeRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.typeTagName + "(\\s|$)");
            SemTagC2A.headerRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.headerTagName + "(\\s|$)");
            SemTagC2A.broadcastRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.broadcastTagName + "(\\s|$)"),
           
            // Target Tag regular expressions for comparison
            SemTagC2A.targetRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.targetTagName + "(\\s|$)"),
            SemTagC2A.actionRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.actionTagName + "(\\s|$)");
            SemTagC2A.menulabelRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.menulabelTagName + "(\\s|$)");
            SemTagC2A.paramRegExp = new RegExp("(^|\\s)" + 
                SemTagC2A.paramTagName + "(\\s|$)");
        },

        isTarget:  function(node) {
            return (SemTagUtil.getNodeClassValue(node).match(SemTagC2A.targetRegExp));
        },

        isSource:  function(node) {
            return (SemTagUtil.getNodeClassValue(node).match(SemTagC2A.sourceRegExp));
        },
        
        UpdateAvailTargets: function() {
           // reset target availability array
           SemTagC2A.targetsAvail = null;
           SemTagC2A.targetsAvail = new Array();
           
           for (var j=0; j<SemTagC2A.targetNodeIDs.length; j++) {
              // fetch live DOM c2a target node 
              var parentNode = document.getElementById(SemTagC2A.targetNodeIDs[j]);
              // Find it's c2a "type" and "namespace" child element values.  These will
              // be used as indexes.  NOTE: code assumes that namespace is optional.
              var theChildNodes = parentNode.childNodes;
              var namespace = null;
              var type = null;
              for (var i=0; i < theChildNodes.length; i++) {
                 var node = theChildNodes[i];
                 var cname = SemTagUtil.getNodeClassValue(node);
                 if (cname == "") continue;
                 if (cname.match(SemTagC2A.typeRegExp)) {
                    type = SemTagC2A.getElementValue(node);
                 } else if (cname.match(SemTagC2A.namespaceRegExp)) {
                    namespace = SemTagC2A.getElementValue(node);
                 }
              }
              // now, use target's namespace and type values as indexes into an array
              // to set availability.
              if (type != null) {
                 if (namespace == null) {
                    // namespace is optional - it wasn't there so set default one
                    namespace = SemTagC2A.defaultNameSpaceStr;
                 }
                 if (typeof(SemTagC2A.targetsAvail[namespace]) == "undefined") {
                    SemTagC2A.targetsAvail[namespace] = new Array();
                 }
                 // set availability to true indicating that a target with these 
                 // namespace and type values exists
                 SemTagC2A.targetsAvail[namespace][type] = true;
              }
           }
        },

        isTargetDefined:  function(info) {
            var retVal = false;
            if (info.namespace == "") { 
               info.namespace = SemTagC2A.defaultNameSpaceStr;
            }
            if (typeof(SemTagC2A.targetsAvail[info.namespace]) != "undefined") {
            	var exists = SemTagC2A.targetsAvail[info.namespace][info.type];
                retVal = (typeof(exists) != "undefined" && exists ? true:false);
            }
            return retVal;
        },

        isTargetSourceMatch:  function(source, target) {
            var retVal = (source.namespace == target.namespace && 
                (source.type == "*" || source.type == target.type));
            return retVal;
        },
	
        getElementValue: function (element) {
            var text;
            if  (SemTagUtil.getNodeClassValue(element).match(SemTagC2A.headerRegExp)) {
                text = element.innerHTML;
            } else {
                text = element.getAttribute("title") || element.firstChild.nodeValue;
            }
            // trim off any whitespace
            return (text == null?  "" : text.replace(/^\s*(\S*(\s+\S+)*)\s*$/, "$1"));
        },

        getSourceInfo:  function(sourceNode) {
            var info = null;
            var nodes = sourceNode.childNodes;
            var namespace="";
            var type="";
            var value="";
            var header="";
            var broadcast=false;

            for (var i=0; i<nodes.length; i++) {
                var node = nodes[i];
                var cname = SemTagUtil.getNodeClassValue(node);
                if (cname == null) continue;
                if (cname.match(SemTagC2A.namespaceRegExp)) {
                    namespace = SemTagC2A.getElementValue(node);
                }else if (cname.match(SemTagC2A.typeRegExp)) {
                    type = SemTagC2A.getElementValue(node);
                }else if (cname.match(SemTagC2A.valueRegExp)) {
                    value = SemTagC2A.getElementValue(node);
                }else if (cname.match(SemTagC2A.headerRegExp)) {
                    header = SemTagC2A.getElementValue(node);
                }else if (cname.match(SemTagC2A.broadcastRegExp)) {
                    broadcast = true;
                }
            }
            info = {"namespace":namespace,"type":type,"value":value,
                "header":header, "broadcast":broadcast};

            return info;
        },

        // -------------------------------------------------------------
        // Menu Item Generation functions
        // -------------------------------------------------------------
        // Get the menu header and target actions to generate the menu markup
        // for this source
        generateMenuContents: function (event, sourceNode) {

           var menuItems = null;
           var menuHdrJson = null;
                
           // retrieve child data from live DOM c2a:source node (e.g. type,value,header,etc)
           var info = SemTagC2A.getSourceInfo(sourceNode);

           // collect the menu actions
           if (SemTagC2A.isTargetDefined(info) == true) {
              menuItems = SemTagC2A.getTargetMenuItems(info);
              
              // Now create a C2A "Broadcast" menu item if functionality was 
              // requested in C2A source tag definition
              if (info.broadcast && menuItems.length > 1) {
                 menuItems.push(SemTagSvc.getMenuItemJson("Invoke all Actions",
                                                          "javascript:SemTagC2A.executeMenuAction("+
                                                              menuItems.length+");" ));
              }
           }

           if (info.header != "" ) {
              menuHdrJson = SemTagSvc.getMenuHeaderJson(info.header, "text/html", -100);  
           }
           
           if (menuItems != null || info.header != "")
           {
              SemTagSvc.setMenuData(event, menuItems, "", menuHdrJson, "");
           }
        },
        
        getTargetMenuItems:  function(sourceInfo) {
            var menuItems = new Array();
            for (var i = 0; i < SemTagC2A.targetNodeIDs.length; i++) {
                // fetch live DOM c2a:target node
                var targetNode = document.getElementById(SemTagC2A.targetNodeIDs[i]);
                // create JSON to mirror c2a:target node data and initialize
                // "namespace" member since it is optional in tag definition
                var targetInfo = {};
                targetInfo.namespace=SemTagC2A.defaultNameSpaceStr;
                // retrieve c2a:target tag's data from child nodes
                if (SemTagC2A.isTarget(targetNode)) {
                    var nodes = targetNode.childNodes;
                    for (var j=0; j < nodes.length; j++) {
                      var cname = SemTagUtil.getNodeClassValue(nodes[j]);
                      if (cname == "") continue;
                      if (cname.match(SemTagC2A.namespaceRegExp))
                          targetInfo.namespace = SemTagC2A.getElementValue(nodes[j]);
                      else if (cname.match(SemTagC2A.menulabelRegExp))
                          targetInfo.menulabel = SemTagC2A.getElementValue(nodes[j]);
                      else if (cname.match(SemTagC2A.typeRegExp))
                          targetInfo.type = SemTagC2A.getElementValue(nodes[j]);
                      else if (cname.match(SemTagC2A.actionRegExp))
                          targetInfo.action = SemTagC2A.getElementValue(nodes[j]);
                    }
                }
                    
                // Check for a match.  If it matches, add menu items
                if (SemTagC2A.isTargetSourceMatch(sourceInfo, targetInfo) == true) {
                    menuItems.push(SemTagSvc.getMenuItemJson(targetInfo.menulabel,
                        "javascript:SemTagC2A.executeMenuAction("+i + ");" ));
                }
            }

            // If there are menu items, save away the source value 
            if (menuItems.length > 0) {
                SemTagC2A.currentValue = sourceInfo.value;
            }

            return menuItems;
        },

        // -------------------------------------------------------------
        // Action execution functions
        // -------------------------------------------------------------
        executeMenuAction: function (index) {
        
           if (index<0 || index>SemTagC2A.targetNodeIDs.length) {
              alert ("ERROR: Index out of bounds for C2A targetNode array: index = " + index);
              return;
           } else if (index == SemTagC2A.targetNodeIDs.length) {
              // user selected C2A "broadcast" or "invoke all" option - execute all matching server-side and
              // client-side targets that matched source
              // TBD: Implementation notes:
              //    1) We should keep menuitems array around so we have don't have to reparse targets...
              //    2) Here is where we would call a server-side entry point to execute multiple actions
              //    3) Also need to understand how to call multiple client-side (ie. JavaScript) functions at once
              //      
              alert ("executing C2A Broadcast (ie. Invoke all actions) feature");
           } else {
              var theTarget = {};
              // fetch live DOM c2a:target node
              var targetNode = document.getElementById(SemTagC2A.targetNodeIDs[index]);
              // re
                 var nodes = targetNode.childNodes;
                 for (var j=0; j < nodes.length; j++) {
                    var cname = SemTagUtil.getNodeClassValue(nodes[j]);
                    if (cname.match(SemTagC2A.paramRegExp)) {
                       theTarget.paramname = SemTagC2A.getElementValue(nodes[j]);
                    }else if (cname.match(SemTagC2A.typeRegExp)) {
                       theTarget.type = SemTagC2A.getElementValue(nodes[j]);
                    }else if (cname.match(SemTagC2A.actionRegExp)) {
                       theTarget.action = SemTagC2A.getElementValue(nodes[j]);
                    }
                 }

                 // check for microformat value
                 if (SemTagC2A.currentValNode != null) {
                    var valarray = SemTagSvc.getElementsByClassName(theTarget.type,SemTagC2A.currentValNode, 1);
                    if (valarray.length != 0) 
                        SemTagC2A.currentValue = SemTagSvc.getTextValue(valarray[0]);
                    // clear currentValNode
                    SemTagC2A.currentValNode = null;
                 }

                 if (theTarget.action.indexOf("javascript:") == 0) {	
                    // For javascript actions, substitute arg val and use 
                    // eval function to invoke js
                    var action = theTarget.action.replace( /@@@ARGS@@@/g, "SemTagC2A.currentValue" );
                    eval(action);
                 }else {
                    // server-side action, use the C2A form to execute
                    // the action with the parameter value
                    var myform = SemTagC2A.getC2AForm();
                    myform.action=theTarget.action;
                    // set the parameter name and value
                    myform.elements[0].name=theTarget.paramname;
                    myform.elements[0].value = SemTagC2A.currentValue;
	                     	
                    // submit the form
                    myform.submit();
                 }
           }
        },

        createC2AForm: function () {
        
            // build up a buffer representing a FORM.  This FORM will get submitted during C2A server-side
            //   C2A action execution

            var buf = '<form method="POST" id="C2A_ServerSideActionForm"';             
            buf += ' action=""><input type="hidden" id="c2aValue" value=""/>';
            buf += '</form>';

            var tag = document.createElement("div");
            tag.setAttribute("id", "C2A_FORM_ID");
            tag.style.position = "absolute";
            tag.style.display = "none";
            tag.style.zIndex = "99999";

            tag.innerHTML = buf;
            document.body.insertBefore(tag, document.body.firstChild);

            //CSA support
            if ( this._isCSA() ) {

               // We rely on special Portal form processing when in client side aggregation (CSA) mode
               var form = document.forms["C2A_ServerSideActionForm"];

               form.submit = function ()
                             {
                                com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER.handle( this.id, this.action ); 
                             }

            }

            return (document.forms["C2A_ServerSideActionForm"]);
        },

        // utility function to determine if we are in Portal's Client Side Aggregation mode
        //
        _isCSA: function () {

            var retVal = false;

            try
            {
                retVal = (typeof(eval("com.ibm.portal.aggregation.forms.PORTLET_FORM_HANDLER") )!= "undefined" );
            }
            catch (e)
            {
               //do nothing here... something was not defined so we aren't in CSA
            }
            
            return retVal;
        },

        getC2AForm:  function() {
            var c2aForm = document.forms["C2A_ServerSideActionForm"];
            if ( c2aForm == null) {
                c2a_Form = SemTagC2A.createC2AForm();
            }
            return c2a_Form;
        }

}

SemTagC2A.init();
