<%-- Copyright IBM Corp. 2008  All Rights Reserved.                    --%>
// Method that is called by Person service during dynamic popup rendering.
// Fill the "out parameters", as appropriate, which are then passed to SemTagSvc.setMenuData().
//
// person: (in) Person JSON that holds the information about the person
// bidi: (in) Bidi setting (string)
// menuItems: (out) Array to be filled with elements of type menu item JSON
// selector: (out) CSS selector to be used for the menu items
// header: (out) Buffer to be filled with the popup header HTML; use .write() to append
// footer: (out) Buffer to be filled with the popup footer HTML; use .write() to append
//
getMarkup: function(person, bidi, menuItems, selector, header, footer) {
	if (!person._inprofile) {
		var viewMethod = (person._profileinplace)? "javascript:doShowProfile(\"" + person.uid + "\")": "javascript:SemTagPerson.openPersonRecord()";
		menuItems.push(SemTagSvc.getMenuItemJson(SemTagPerson.nls.view_person, viewMethod, -100));
	}
	var email = (typeof(person.email)!='undefined' && person.email.internet)? person.email.internet: null;
	if (email) menuItems.push(SemTagSvc.getMenuItemJson(SemTagPerson.nls.send_email, "mailto:" + email, -90));
	if (typeof(SemTagAwrns)!='undefined' && typeof(person.X.ststatus)!='undefined' && person.X.ststatus=="online") {
		var chatMethod = "javascript:SemTagAwrns.openChat(\"" + person.X.imn + "\")";
		menuItems.push(SemTagSvc.getMenuItemJson(SemTagPerson.nls.start_chat, chatMethod, -80));
	}

	// process the extra menu items available
	var items = SemTagSvc.getActions("person");
	if (items) {
		for (i=0; i<items.length; i++) {
			var showif = items[i].showif;
			var show = true;
			if (showif) {
				try {
					var showifMethod = eval(showif);
					show = showifMethod.call(null,SemTagPerson.currentPerson);
				}
				catch (e) {
					try {
						var showifMethod = eval(showif);
						show = showifMethod.call(null,SemTagPerson.currentPerson);
					}
					catch (e2) {
						show = false;
					}
				}
			}
			if (show) {
				var label = items[i].label;
				if (label.match(/nls\.[a-z]/)) label = eval(label);
				var href = items[i].url.replace( /@@@ARGS@@@/g, "SemTagPerson.currentPerson" );
				menuItems.push(SemTagSvc.getMenuItemJson(label, href, items[i].order));
			}
		}
	}

	if (!person._inline) {
		<% pageContext.include("/ui/templates/businesscard.jsp"); %>
	}
}, 

// Method that is called by Person service during inline rendering.
// Fill the "buffer" with the HTML that should be inserted after the hCard in the page.
//
// person: (in) Person JSON that holds the information about the person
// bidi: (in) Bidi setting (string)
// buffer: (out) Buffer to be filled with the popup header HTML; use .write() to append
//
getInlineMarkup: function (person, bidi, buffer) {
	var email = person.email.internet;
	buffer.write("<div class='personMenu'>");
	buffer.write("<div class='photoCard'><img src='" + person.photo + "' alt=''/></div>");
	buffer.write("<div class='businessCard' dir='" + bidi + "'>");
	var classAttr = (person._headeronly? "class='vcard X-person-inside-inline'": "");
	buffer.write("<ul " + classAttr + " dir='" + bidi + "'>");
	buffer.write("<li class='cardName'><span class='fn'>" + person.fn + "</span></li>");
	buffer.write("<li class='email'><span class'type' style='display:none;'>internet</span><span class='value'>" + email + "</span></li>");
	if (person.title) buffer.write("<li>" + person.title + "</li>");
	if (person.tel && person.tel.voice) buffer.write("<li>" + person.tel.voice + "</li>");
	buffer.write("</ul>");
	buffer.write("</div>");
	if (person._headeronly) {
		buffer.write("<div style='clear:both;'>");
	}
	else {
		buffer.write("<div class='personMenuActions' dir='" + bidi + "' style='clear:both;'>");
		buffer.write("<ul>");
		buffer.write("<li><a href='NEEDSWORK" + "'>" + SemTagPerson.nls.view_person + "</a></li>");
		buffer.write("<li><a href='mailto:" + email + "'>" + SemTagPerson.nls.send_email + "</a></li>");
		buffer.write("<li><a href='NEEDSWORK" + "'>" + SemTagPerson.nls.add_to_contact + "</a></li>");
		buffer.write("</ul>");
	}
	buffer.write("</div>");
	buffer.write("</div>");
},

// support method -- WebSphere Portal specific
openPersonRecord: function() {
	var person = SemTagPerson.currentPerson;
	var userId = (typeof(person.email)!='undefined' && typeof(person.email.internet)!='undefined' && 0<person.email.internet.length)? person.email.internet: person.uid;
	var userId2 = encodeURIComponent( userId );
	var userId3 = encodeURIComponent( userId2 );
	window.open("<%= com.ibm.wps.engine.tags.ConstantsTag.getBaseURL() %>/poc?uri=personrecord:" + userId3, null, "height=640,width=512,resizable=yes,scrollbars=yes,status=yes,toolbar=no,menubar=no,location=no");
},

<%--
// support method -- WebSphere Portal specific
addToSametimeList: function() {
	var email = SemTagPerson.currentPerson.email.internet;
	var fn_ = encodeURIComponent( SemTagPerson.currentPerson.fn );
	var fn = encodeURIComponent( fn_ );
	window.open("/wps/poc?uri=addtostlist:" + email + "/" + fn, null, "height=384,width=768,resizable=yes,status=yes,toolbar=no,menubar=no,location=no");
},
--%>
